const projectSchema = {
  "type": "object",
  "properties": {
    "projects": {
      "type": "array",
      "minItems": 40,
      "maxItems": 100,
      "items": {
        "type": "object",
        "properties": {
          "projectId": {
            "$ref": "#/definitions/positiveInt"
          },
          "projectName": {
            "type": "string",
            "faker": "name.firstName"
          },
          "preparedBy": {
            "type": "string",
            "faker": "name.findName"
          },
          "createdOn": {
            "type": "string",
            "faker": "date.future"
          },
          "updatedOn": {
            "type": "string",
            "faker": "date.future"
          }
        },
        "required": ["projectId", "projectName", "preparedBy", "createdOn", "updatedOn"]
      }
    }
  },
  "required": ["projects"],
  "definitions": {
    "positiveInt": {
      "type": "integer",
      "minimum": 1,
      "minimumExclusive": true,
      "maximum": 20,
      "unique": true
    }
  }
};

const subProjectSchema = {
  "type": "object",
  "properties": {
    "subprojects": {
      "type": "array",
      "minItems": 20,
      "maxItems": 50,
      "items": {
        "type": "object",
        "properties": {
          "projectId": {
            "$ref": "#/definitions/positiveInt"
          },
          "subprojectId": {
            "$ref": "#/definitions/positiveInt"
          },
          "subprojectName": {
            "type": "string",
            "faker": "name.firstName"
          },
          "theory": {
            "type": "string",
            "pattern": "elastic|platic"
          }
        },
        "required": ["projectId", "subprojectId", "subprojectName", "theory"]
      }
    }
  },
  "required": ["subprojects"],
  "definitions": {
    "positiveInt": {
      "type": "integer",
      "minimum": 1,
      "minimumExclusive": true,
      "maximum": 20,
      "unique": true
    }
  }
};

module.exports = {
  projectSchema: projectSchema,
  subProjectSchema: subProjectSchema
};
