import { Component, OnInit, Input } from '@angular/core';
import { MainPageService } from '../../main-page.service';
import { PipeFunctionModel } from '../../../models/dictionaries/pipe-function.model';
import { FormGroup, FormControl, Validators } from '@angular/forms';
declare var $: any;

@Component({
  selector: 'app-pipe-functions-grid',
  templateUrl: './pipe-functions-grid.component.html',
  styleUrls: ['./pipe-functions-grid.component.scss']
})
export class PipeFunctionsGridComponent implements OnInit {
  isPipeFunctionsHidden = true;
  pipeFunctionsGridData: PipeFunctionModel[];
  editedPipeFunction = new PipeFunctionModel();
  rowNumber: number;
  public formGroup: FormGroup;
  headers: string[] = ['Name', 'Code', ''];

  constructor(private mainPageService: MainPageService) {}

  public ngOnInit(): void {
    this.loadGridDataFromDb();
    this.mainPageService.deletePipeFunctionSubject.subscribe( (value) => {
      this.pipeFunctionsGridData = this.pipeFunctionsGridData.filter(x => x._id !== value);
    });
    this.createForm();
  }

  loadGridDataFromDb() {
    this.mainPageService.loadAllPipeFunctions().subscribe((response) => {
      this.pipeFunctionsGridData = response.sort(function (a, b) {
        return (a.name < b.name) ? -1 : 1;
      });
    });
  }

  toggle() {
    const actionButtons = $('.mat-icon-button');
    $(actionButtons[1]).css('visibility', 'hidden');
    for (let i = 2; i < actionButtons.length; i += 2) {
        $(actionButtons[i]).css('visibility', 'hidden');
    }
    this.isPipeFunctionsHidden = !this.isPipeFunctionsHidden;
  }

  saveNewPipeFunction() {
    this.mainPageService.addNewPipeFunctionIntoDb(JSON.stringify(this.editedPipeFunction)).subscribe((response) => {
      this.editedPipeFunction._id = response._id;
      this.editedPipeFunction.id = response.id;
      this.pipeFunctionsGridData.push(this.editedPipeFunction);
      this.editedPipeFunction = new PipeFunctionModel();
    });
  }

  setChoosenElement(element, row) {
    this.editedPipeFunction = {
      ...element,
    };
    this.rowNumber = row;
    this.createForm();
  }

  editPipeFunction() {
    console.log(this.pipeFunctionsGridData[this.rowNumber]);
    Object.assign(this.pipeFunctionsGridData[this.rowNumber], this.editedPipeFunction);
    this.mainPageService.updatePipeFunctionInDb(JSON.stringify(this.editedPipeFunction)).subscribe((response) => {});
  }

  deletePipeFunction(element) {
    this.mainPageService.removePipeFunction(element._id).subscribe((res) => {}, (error: any) => {});
    this.mainPageService.deletePipeFunctionSubject.next(element._id);
  }

  public prepareCreate() {
    this.editedPipeFunction = new PipeFunctionModel();
    this.createForm();
  }

  private createForm() {
    this.formGroup = new FormGroup({
      name: new FormControl(this.editedPipeFunction.name, Validators.required),
      code: new FormControl(this.editedPipeFunction.code)
    });
  }
}
