import { Component, OnInit } from '@angular/core';
import { MainPageService } from '../../main-page.service';
import { FormGroup, FormControl, Validators } from '@angular/forms';
import { BasicDictionaryModel } from '../../../models/dictionaries/basic-dictionary.model';
declare var $: any;

@Component({
  selector: 'app-pipe-material-types-grid',
  templateUrl: './pipe-material-types-grid.component.html',
  styleUrls: ['./pipe-material-types-grid.component.scss']
})
export class PipeMaterialTypesGridComponent implements OnInit {
  isPipeMaterialTypesHidden = true;
  pipeMaterialTypesGridData: BasicDictionaryModel[];
  editedPipeMaterialType = new BasicDictionaryModel();
  rowNumber: number;
  public formGroup: FormGroup;
  headers: string[] = ['Name', 'Code', ''];

  constructor(private mainPageService: MainPageService) {}

  public ngOnInit(): void {
    this.loadGridDataFromDb();
    this.mainPageService.deletePipeFunctionSubject.subscribe( (value) => {
      this.pipeMaterialTypesGridData = this.pipeMaterialTypesGridData.filter(x => x._id !== value);
    });
    this.createForm();
  }

  loadGridDataFromDb() {
    this.mainPageService.loadAllPipeMaterialTypes().subscribe((response) => {
      this.pipeMaterialTypesGridData = response.sort(function (a, b) {
        return (a.name < b.name) ? -1 : 1;
      });
    });
  }

  toggle() {
    const actionButtons = $('.mat-icon-button');
    $(actionButtons[1]).css('visibility', 'hidden');
    for (let i = 2; i < actionButtons.length; i += 2) {
        $(actionButtons[i]).css('visibility', 'hidden');
    }
    this.isPipeMaterialTypesHidden = !this.isPipeMaterialTypesHidden;
  }

  saveNewPipeMaterialType() {
    this.mainPageService.addNewPipeMaterialTypeIntoDb(JSON.stringify(this.editedPipeMaterialType)).subscribe((response) => {
      this.editedPipeMaterialType._id = response._id;
      this.editedPipeMaterialType.id = response.id;
      this.pipeMaterialTypesGridData.push(this.editedPipeMaterialType);
      this.editedPipeMaterialType = new BasicDictionaryModel();
    });
  }

  setChoosenElement(element, row) {
    this.editedPipeMaterialType = {
      ...element,
    };
    this.rowNumber = row;
    this.createForm();
  }

  editPipeMaterialType() {
    console.log(this.pipeMaterialTypesGridData[this.rowNumber]);
    Object.assign(this.pipeMaterialTypesGridData[this.rowNumber], this.editedPipeMaterialType);
    this.mainPageService.updatePipeMaterialTypeInDb(JSON.stringify(this.editedPipeMaterialType)).subscribe(() => { });
  }

  deletePipeMaterialType(element) {
    this.mainPageService.removePipeMaterialType(element._id).subscribe(() => { }, () => { });
    this.mainPageService.deletePipeMaterialTypeSubject.next(element._id);
  }

  public prepareCreate() {
    this.editedPipeMaterialType = new BasicDictionaryModel();
    this.createForm();
  }

  private createForm() {
    this.formGroup = new FormGroup({
      name: new FormControl(this.editedPipeMaterialType.name, Validators.required),
      code: new FormControl(this.editedPipeMaterialType.code)
    });
  }
}
