import { Component, OnInit } from '@angular/core';
import { MainPageService } from '../../main-page.service';
import { SystemModel } from '../../../models/dictionaries/system.model';

declare var $: any;


@Component({
  selector: 'app-systems-grid',
  templateUrl: './systems-grid.component.html',
  styleUrls: ['./systems-grid.component.scss']
})
export class SystemsGridComponent implements OnInit {
  isSystemsHidden = true;
  systemsGridData: SystemModel[];
  templateSystem = new SystemModel();
  editedSystem = new SystemModel();
  rowNumber: number;
  headers: string[] = ['Name', 'Description', ''];

  constructor(private mainPageService: MainPageService) {}

  public ngOnInit(): void {
    this.loadGridDataFromDb();
    this.mainPageService.deleteSystemSubject.subscribe( (value) => {
      this.systemsGridData = this.systemsGridData.filter(x => x._id !== value);
    });
  }

  loadGridDataFromDb() {
    this.mainPageService.loadAllSystems().subscribe((response) => {
      this.systemsGridData = response.sort(function (a, b) {
        return (a.name < b.name) ? -1 : 1;
      });
    });
  }

  toggle() {
    const actionButtons = $('.mat-icon-button');
    $(actionButtons[1]).css('visibility', 'hidden');
    for (let i = 2; i < actionButtons.length; i += 2) {
        $(actionButtons[i]).css('visibility', 'hidden');
    }
    this.isSystemsHidden = !this.isSystemsHidden;
  }

  saveNewSystem() {
    this.mainPageService.addNewSystemIntoDb(JSON.stringify(this.templateSystem)).subscribe((response) => {
      this.templateSystem._id = response;
      this.systemsGridData.push(this.templateSystem);
      this.templateSystem = new SystemModel();
    });
  }

  setChoosenElement(element, row) {
    this.editedSystem = {
      ...element,
    };
    this.rowNumber = row;
  }

  editSystem() {
    console.log(this.systemsGridData[this.rowNumber]);
    Object.assign(this.systemsGridData[this.rowNumber], this.editedSystem);
    this.mainPageService.updateSystemInDb(JSON.stringify(this.editedSystem)).subscribe(() => { });
  }

  deleteSystem(element) {
    this.mainPageService.removeSystem(element._id).subscribe(() => { }, () => { });
    this.mainPageService.deleteSystemSubject.next(element._id);
  }
}
