import { Component, OnInit, Input, Injector } from '@angular/core';
import { NavService, INavItem } from '../../service/common/nav.service';
import { Router } from '@angular/router';
import { TranslateService } from '@ngx-translate/core';

@Component({
  selector: 'app-navigation',
  templateUrl: './navigation.component.html',
  styleUrls: ['./navigation.component.scss']
})
export class NavigationComponent implements OnInit {
  public showNavBar = false;
  private navItems: Array<INavItem> = [];
  constructor(public translate: TranslateService, private navService: NavService, private router: Router, private inj:Injector) {


    this.navService.$navItemsChanged.subscribe(navItems => {
      if (!navItems) {
        this.showNavBar = false;
      } else {
        this.showNavBar = true;
        this.navItems = navItems;
      }
    });
  }


  navigate(navItem: INavItem) {
    if (navItem.routeUrl) {
      this.router.navigateByUrl(navItem.routeUrl);
    }
  }

  getNavButtonClass(totalItems, itemIndex) {
    if (itemIndex === 0) {
      return 'navigation-buttonhome';
    } else if (itemIndex < (totalItems - 1)) {
      return 'navigation-buttonhomeattached';
    } else {
      return 'navigation-buttonalone';
    }
  }

  getNavItemId(totalItems, itemIndex) {
    if (itemIndex === 0) {
      return 'backToProjects';
    } else if (itemIndex < (totalItems - 1)) {
      return 'backToProject';
    }
  }

  getNavItemClass(totalItems, itemIndex) {
    if (itemIndex === 0) {
      return 'navigation-home';
    } else if (itemIndex < (totalItems - 1)) {
      return 'navigation-homeitem';
    } else {
      return 'navigation-loneitem';
    }
  }

  ngOnInit() {
  }

}
