﻿define(['js/view3d/VIEW3D.Loads', 'js/view3d/controls/VIEW3D.controls.Cursor3D'], function() {

    CORE.calculations = {
        calculationCanceled: false,
        checkProgressInterval: null,
        cancelCalculations: function() {
            clearInterval(CORE.calculations.checkProgressInterval);
            CORE.calculations.calculationCanceled = true;
        },
        actualProgress: ko.observable(0),
        actualStatus: ko.observable(null),

        calculate: function(model) {
            console.log("CORE.calculations.calculate: BEGIN");

            CORE.calculations.calculationCanceled = false;
            CORE.calculations.setCalculationCanceled();
            VIEWUI.viewModel.calculationError(false);

            VIEWUI.statusBar.clear();
            CORE.webApiManager.startCalculation(model, callbackCalculation, errorCallback);

            // === start calculations ===
            function callbackCalculation(results) {
                try {
                    if (!CORE.calculations.calculationCanceled) {
                        if (results.isOK) {
                            CORE.calculations.loadResults(results.constructionResults);
                            console.log("CORE.calculations.importResults: results model successfully imported");
                            CORE.calculations.setCalculationSuccess("Calculation succeeded", results.constructionResults.exceptions);
                            if (typeof CORE.options.callbackForCalculationStateChanging === 'function') {
                                CORE.options.callbackForCalculationStateChanging(false, true);
                            }
                        } else {
                            CORE.calculations.setCalculationNotCalc(results.constructionResults.exceptions);
                            if (typeof CORE.options.callbackForCalculationStateChanging === 'function') {
                                CORE.options.callbackForCalculationStateChanging(false, false);
                            }
                        };
                    }
                } catch (ex) {
                    VIEWUI.statusBar.setError("Calculation succeeded, but failed to load results model");
                    console.error("Failed to load results model, reason: " + ex);
                    console.error(ex);
                }

                VIEWUI.popups.calculationsProgress.hide();
            }

            function errorCallback() {
                console.log("CORE.calculations: calculation request failed or canceld");
                if (typeof CORE.options.callbackForCalculationStateChanging === 'function') {
                    CORE.options.callbackForCalculationStateChanging(false, false);
                }

                VIEWUI.popups.calculationsProgress.hide();
                VIEWUI.viewModel.calculationError(true);
            }
        },

        // exceptions are in CheckProgess and Load requests (not in load results)
        loadResults: function(proto) {
            var bars = CORE.model.bars();
            var nodes = CORE.model.nodes();
            var barsResults = [];
            var nodesResults = [];
            var offset = 0.0;
            var i, j;
            var maxGlobalUtilization = proto.utilization * 100;
            for (i = 0; i < proto.resultBars.length; i++) {
                // We are assuming that results are grouped by barId and sorted dist2
                var barId = proto.resultBars[i].id;
                
                var stabilityBar = Math.max(proto.resultBars[i].installationResult.bendingBuckling.utilization,
                                            proto.resultBars[i].installationResult.compressionInteract.utilization,
                                            proto.resultBars[i].installationResult.compresionBuckling.utilization);
                var values = {
                    stressNormal: (proto.resultBars[i].installationResult.stressNormal.utilization) * 100,
                    stressShear: (proto.resultBars[i].installationResult.stressShear.utilization) * 100,
                    stressInteraction: (proto.resultBars[i].installationResult.stressInteraction.utilization) * 100,
                    displacement: (proto.resultBars[i].installationResult.displacement.utilization) * 100,
                    stability: (stabilityBar) * 100,
                    utilization: (proto.resultBars[i].installationResult.globalUtilization) * 100
                }

                if (values.utilization > maxGlobalUtilization) maxGlobalUtilization = values.utilization;

                var currentBarResults = barsResults[barsResults.length - 1];
                var k;
                if (currentBarResults === undefined || barId !== currentBarResults.bar.id) {
                    currentBarResults = null;
                    for (k = 0; k < barsResults.length; k++) {
                        if (barsResults[k].bar.id === barId) {
                            currentBarResults = barsResults[k];
                            offset = currentBarResults.data.splines[currentBarResults.data.splines.length - 1].end;
                            break;
                        }
                    }
                    if (currentBarResults == null) {
                        for (j = 0; j < bars.length; j++) {
                            if (bars[j].id === barId) {
                                currentBarResults = new MODEL.BarResults(bars[j], {
                                    splines: [],
                                    values: values
                                });
                                barsResults.push(currentBarResults);
                                offset = 0.0;
                                break;
                            }
                        }
                    }
                }
                if (currentBarResults != null) {
                    var barResult = proto.resultBars[i];
                    for (k = 0; k < barResult.polynominals.length; k++) {
                        currentBarResults.data.splines.push({
                            FX: [barResult.polynominals[k].fx.a, barResult.polynominals[k].fx.b, barResult.polynominals[k].fx.c, barResult.polynominals[k].fx.d],
                            FYMZ: [barResult.polynominals[k].fymz.a, barResult.polynominals[k].fymz.b, barResult.polynominals[k].fymz.c, barResult.polynominals[k].fymz.d],
                            FZMY: [barResult.polynominals[k].fzmy.a, barResult.polynominals[k].fzmy.b, barResult.polynominals[k].fzmy.c, barResult.polynominals[k].fzmy.d],
                            MX: [barResult.polynominals[k].mx.a, barResult.polynominals[k].mx.b, barResult.polynominals[k].mx.c, barResult.polynominals[k].mx.d],
                            RX: [barResult.polynominals[k].rx.a, barResult.polynominals[k].rx.b, barResult.polynominals[k].rx.c, barResult.polynominals[k].rx.d],
                            UX: [barResult.polynominals[k].ux.a, barResult.polynominals[k].ux.b, barResult.polynominals[k].ux.c, barResult.polynominals[k].ux.d],
                            UYRZ: [barResult.polynominals[k].uyrz.a, barResult.polynominals[k].uyrz.b, barResult.polynominals[k].uyrz.c, barResult.polynominals[k].uyrz.d],
                            UZRY: [barResult.polynominals[k].uzry.a, barResult.polynominals[k].uzry.b, barResult.polynominals[k].uzry.c, barResult.polynominals[k].uzry.d],
                            end: barResult.polynominals[k].dist2 + offset
                        });
                        currentBarResults.data.extrema = barResult.extrema[0];
                    }
                } else {
                    console.log("WARNING: found results for not existing bar");
                }
            }
            for (i = 0; i < proto.resultNodes.length; i++) {
                var nodeId = proto.resultNodes[i].id;
                var pressure = proto.resultNodes[i].installationResult.pressure;
                var reactions = {
                    Fx: !isEmpty(proto.resultNodes[i].force) ? proto.resultNodes[i].force.x : 0,
                    Fy: !isEmpty(proto.resultNodes[i].force) ? proto.resultNodes[i].force.y : 0,
                    Fz: !isEmpty(proto.resultNodes[i].force) ? proto.resultNodes[i].force.z : 0,
                    Mx: !isEmpty(proto.resultNodes[i].moment) ? proto.resultNodes[i].moment.x : 0,
                    My: !isEmpty(proto.resultNodes[i].moment) ? proto.resultNodes[i].moment.y : 0,
                    Mz: !isEmpty(proto.resultNodes[i].moment) ? proto.resultNodes[i].moment.z : 0
                }
                var utilization = {
                    local: (proto.resultNodes[i].installationResult.globalUtilization) * 100,
                    global: (proto.resultNodes[i].installationResult.globalUtilization) * 100
                }

                if (utilization.global > maxGlobalUtilization) maxGlobalUtilization = utilization.global;

                for (j = 0; j < nodes.length; j++) {
                    if (nodes[j].id === nodeId) {
                        var currentNodeResults = new MODEL.NodeResults(nodes[j], {
                            reactions: reactions,
                            pressure: pressure,
                            utilization: utilization
                        });
                        nodesResults.push(currentNodeResults);
                        break;
                    }
                }
            }
            // resulta are calculated in local space this transformations converts it to global space (origin is Node1)
            for (var br = 0; br < barsResults.length; br++) {
                for (var el = 0; el < proto.resultBars.length; el++) {
                    if (proto.resultBars[el].id === barsResults[br].bar.id) {
                        if (proto.resultBars[el].baseX === null) {
                            throw new Error("proto.resultBars[" + el + "].baseX === null");
                        }
                        barsResults[br].data.baseX = proto.resultBars[el].baseX;
                        barsResults[br].data.baseY = proto.resultBars[el].baseY;
                        barsResults[br].data.baseZ = proto.resultBars[el].baseZ;
                    }
                }
            }
            CORE.model.barsResults(barsResults);
            CORE.model.nodesResults(nodesResults);

            VIEWUI.viewModel.maxGlobalUtilization(maxGlobalUtilization);
            render();
        },
        setCalculationError: function(exceptions) {
            VIEWUI.statusBar.setError("Calculation failed, errors occured!");
            VIEWUI.exceptionWindow.setCalculationExceptions(exceptions);
        },
        setCalculationSuccess: function(message, exceptions) { // might be some info messages in the future
            VIEWUI.statusBar.setSuccess(message);
            VIEWUI.exceptionWindow.setCalculationExceptions(exceptions);
        },
        setCalculationNotCalc: function(exceptions) { // might be some info messages in the future
            VIEWUI.statusBar.setSuccess("Calculation break, errors occured!");
            VIEWUI.exceptionWindow.setCalculationExceptions(exceptions);
        },
        setCalculationCanceled: function() { // might be some info messages in the future
            VIEWUI.statusBar.setSuccess("Calculation has been canceled!");
            VIEWUI.exceptionWindow.removeAllExceptions();
        }
    }
});