var scene, scene2, camera, camera2, renderer, controls;
var geometry, material, mesh, gridLine;
var webGlContainer, canvas, canvas2D, canvasesContainer, height, width;
var geometry, cube, rectangle, rectMesh, planeMesh;
var beginHeight, beginWidth, cursorState, overlayNeedsUpdate;
var squareGeometry, raycasterPointCloudTreshold;
var v, scale_factor, isMouseDown, mouseMoved;
var raycaster, mouse;
var epsilonGrid = 1000; // 1mm
var translation;
var CORE = CORE || {};

define(['js/model/MODEL.ConstructionViewModel', 'js/view3d/VIEW3D.Construction', 'js/view3d/VIEW3D.ActiveWorkplane', 'js/view3d/VIEW3D.ResultsWindowWorkplane',
    'js/view3d/VIEW3D.BitmapFont', 'js/view3d/VIEW3D.ResultsWindow.Results', 'js/view3d/VIEW3D.AxisHelper', 'js/view3d/VIEW3D.Measurements',
    'js/view3d/controls/VIEW3D.controls.ControlsManager', 'js/view3d/utils/VIEW3D.utils.Labeling',
    'js/core/CORE.units', 'js/core/CORE.xconverter', 'js/helpers/HELPERS.inputState', 'js/viewui/VIEWUI', 'js/view3d/VIEW3D', 'js/model/MODEL.Workplane', 'js/model/MODEL.LoadCase',
    'js/threejs_functions'
], function() {

    MODE = {
        SELECT: "SELECT",
        MOVE: "MOVE",
        EDIT: "EDIT",
        DRAW_NODE: "DRAW_NODE",
        DRAW_WORKPLANE: "DRAW_WORKPLANE",
        DRAW_BAR: "DRAW_BAR",
        EDIT_SUPPORT: "EDIT_SUPPORT",
        EDIT_NODE_FORCE: "EDIT_NODE_FORCE",
        EDIT_NODE_MOMENT: "EDIT_NODE_MOMENT",
        EDIT_BAR_FORCE: "EDIT_BAR_FORCE",
        EDIT_BAR_UNIFORM: "EDIT_BAR_UNIFORM",
        EDIT_BAR_TRAPEZOIDAL: "EDIT_BAR_TRAPEZOIDAL",
        ZOOM_AREA: "ZOOM_AREA",
        SELECT_AREA: "SELECT_AREA",
        CUT: "MODE.CUT",
        SNAP: "MODE.SNAP",
        EDIT_BOX: "EDIT_BOX",
        MOVE_CONSTRUCTION: "MOVE_CONSTRUCTION",
    };

    SAVE_MODE = { FullConstraints: 0, NoConstraints: 2 }; // order = priority during save, from highest to lowest

    SUPPORT_TYPE = [];
    MODULE_PROFILE_TYPE = [];

    elementsPropertiesCache = {
        barForceModel: 0,
        barUniformModel: 0,
        barTrapezoidalModel: 0
    };



    CORE.options = {
        // default options (customizable on init via jQuery parameters)
        showObjectList: true,
        showToolbar: true, // UNUSED
        showPopups: true, // UNUSED
        debugMode: false,
        viewOnlyMode: false, // UNUSED
        useOrthographicProjection: true,
        mode2D: false, // mode2D: false, 
        nodeSize: 6.0, // nodeSize: 10,  
        runLocally: false,
        width: '100%',
        height: '100%',
        colors: {
            selection: new THREE.Color(1, 0, 0),
            node: new THREE.Color(0, 0, 0),
            bar: new THREE.Color(0, 0, 1),
            loads: {
                nodeForce: new THREE.Color(0, 0.4, 0)
            }
        },
        language: "en",
        isCrm: false, // CRM does not allow /lang/Controller/Method requests, using /Controller/Method instead if run in CRM
        demoMode: false,
        disableLoadAndSave: false,
        hideIcon: false,
        callback: null,
        callbackForCalculationStateChanging: null
    };

    CORE.topBorder = 250;

    CORE.heightOfLoadsTableViewer = 0;

    CORE.whichViewerTab = "geometry";

    CORE.mode = ko.observable(MODE.SELECT);

    CORE.rootPath = "";

    CORE.currentJobVersionId = ko.observable(0); // 0 = latest

    CORE.jobVersionLoaded = false;

    CORE.autoSaveDisabled = CORE.options.disableLoadAndSave;

    CORE.disableConstraints = false;

    CORE.viewerFullyLoaded = false;
    CORE.viewerInProgress = false;
    CORE.viewerReloading = false; // used in crm loading. If true, ignore all exceptions during reloading


    CORE.adjustTopBorder = function() {
        CORE.topBorder = 250 + CORE.heightOfLoadsTableViewer;
    }

    CORE.initMeasurements = function() {
        // DEBUG MODE
        //var tmp = scene.add;
        //scene.add = function (object) {
        //    tmp.call(this, object);
        //}
        CORE.measurements = new MODEL.Measurements(CORE.model.nodes);
        VIEW3D.measurements = new VIEW3D.Measurements(CORE.measurements.getAllMeasurements.bind(CORE.measurements), CORE.measurements.modelChange, CORE.aspectRatio, camera, undefined, CORE.mode);
        if (VIEWUI.viewOptions.measurementsVisible()) {
            VIEW3D.measurements.show(true);
            scene.add(VIEW3D.measurements);
        } else {
            VIEW3D.measurements.show(false);
        }

        CORE.totalMeasurements = new MODEL.TotalMeasurements(CORE.model.nodes);
        VIEW3D.totalMeasurements = new VIEW3D.Measurements(
            CORE.totalMeasurements.getAllMeasurements.bind(CORE.totalMeasurements),
            CORE.measurements.modelChange,
            CORE.aspectRatio,
            camera, {
                measurementOffset0: 0.35,
                measurementOffset1: 0.42,
                measurementOffset2: 0.45,
                textAnchor: 0.42,
                textOffset: -18,
                color: VIEW3D.measurementColor
            },
            CORE.mode);
        if (VIEWUI.viewOptions.measurementsVisible()) {
            VIEW3D.totalMeasurements.show(true);
            scene.add(VIEW3D.totalMeasurements);
        } else {
            VIEW3D.totalMeasurements.show(false);
        }
        HELPERS.subscribePropertyChange(this, VIEWUI.viewOptions.measurementsVisible, "measurementsVisible", function(measurementsVisible) {
            if (measurementsVisible && scene.children.indexOf(this.supportsObject3D) === -1) {
                VIEW3D.measurements.show(true);
                scene.add(VIEW3D.measurements);
                VIEW3D.totalMeasurements.show(true);
                scene.add(VIEW3D.totalMeasurements);
                render();
            } else {
                VIEW3D.measurements.show(false);
                scene.remove(VIEW3D.measurements);
                VIEW3D.totalMeasurements.show(false);
                scene.remove(VIEW3D.totalMeasurements);
                render();
            }
        });
    };

    CORE.getSupportImageById = function(id) {
        for (var i = 0; i < supportImageCache.length; i++) {
            if (supportImageCache[i].id === id) {
                return supportImageCache[i].data;
            }
        }
        return null;
    }

    CORE.renderLoopActive = true;

    CORE.startRenderLoop = function() {
        CORE.renderLoopActive = true;
    }

    CORE.stopRenderLoop = function() {
        CORE.renderLoopActive = false;
    }

    CORE.initView = function() {
        try {
            //init view 
            //var light = new THREE.AmbientLight(0x404040); // soft white light
            //scene.add(light);
            CORE.loadFontScale = ko.observable(1);
            CORE.loadModelScale = ko.observable(1);
            CORE.constructionView3D = new VIEW3D.Construction(CORE.model, camera);
            scene.add(CORE.constructionView3D);

            CORE.initMeasurements();

            CORE.view3D = new VIEW3D.controls.ControlsManager(scene, camera, CORE.axisHelperNamespace, renderer.domElement, CORE.model, CORE.constructionView3D, CORE.mode, VIEW3D.measurements, CORE.aspectRatio);
            VIEWUI.statusBar.initCursorPos(CORE.view3D.getControls().mouse3d.control, CORE.view3D.getControls().cursor3D.control)

            if (CORE.options.mode2D) {
                CORE.view3D.cameraControl.resetView(0, 0, -1); //Top view
                CORE.view3D.cameraControl.rotationEnabled = false;
            }
            CORE.view3D.cameraControl.zoomAll();
            HELPERS.subscribePropertyChange(this, MODEL.repository.objectsToLoad, "MODEL.repository.objectsToLoad", function() {
                if (MODEL.repository.objectsToLoad() === 0 && !CORE.viewerFullyLoaded) {
                    $('#loadingLabel').remove();
                    CORE.refreshAspectRatio();
                    CORE.view3D.cameraControl.zoomAll();
                    CORE.viewerFullyLoaded = true;
                    CORE.viewerReloading = false;
                    var viewerLoadingEnd = performance.now();
                    console.info("Viewer loading time: " + ((viewerLoadingEnd - viewerLoadingStart) / 1000).toFixed(2) + " s");
                    CORE.options.callback(false);
                    // Extension.loadPipes();
                }
            });
            MODEL.repository.objectsToLoad.notifySubscribers();

            CORE.constructionView3D.activeWorkplane.init();
            CORE.constructionView3D.activeWorkplane.draw();

            CORE.startRenderLoop();

            requestAnimationFrame(renderNow);
            render();

            render(MODE_TYPE.ROTATE); // for axishelper
            renderNow();
            if (CORE.options.hideIcon) {
                $('#engineerIcon').hide();
            }
            //VIEWUI.viewOptions.showRealElements.subscribe(function (newVal) { debugger; })

        } catch (ex) {
            if (!CORE.viewerFullyLoaded && !CORE.viewerReloading)
                VIEWERLOADER.setError("CORE.initView failed", ex);
            else
                VIEWUI.statusBar.setError("Internal error occured");
        }
    };

    CORE.init = function(_viewerContainer, _options) {
        console.log("CORE.init() begin");
        CORE.options.language = _options.language;
        CORE.viewerFullyLoaded = false;
        if (!_viewerContainer.length) {
            throw new Error("CORE.init: Provided container element does not exist!");
        }
        viewerContainer = _viewerContainer;
        if (typeof _options !== 'undefined') {
            $.extend(CORE.options, _options); //overwrite (merge) default options with user-defined ones
        }
        CORE.rootPath = rootPath; //rename?
        var _this = this;
        if (CORE.options.useOrthographicProjection) {
            camera = new THREE.OrthographicCamera(-width / 10, width / 10, height / 10, -height / 10, -1000, 1000);
            camera.zoom = 4; // later changed automatically by ZoomAll
        } else {
            camera = new THREE.PerspectiveCamera(50, width / height, 0.1, 1000);
        }
        camera.changeEvent = ko.observable();

        CORE.model = new MODEL.ConstructionViewModel();
        CORE.model.init();
        try {
            CORE.webApiManager.importUnitsData(function(data) {
                CORE.units = new MODEL.Units(data);
            });
        } catch (error) {
            console.error("CORE.init: An error occured while loading unit info" + error);
            throw error;
        }
        console.log("CORE.init: Unit info loaded successfully");
        var callback = function(data) {
            translation = data;
            CORE.constructionUndoRedo = new MODEL.utils.UndoRedo(CORE.model);
            CORE.nodeAttachController = new MODEL.NodeAttachController(CORE.model.nodes, CORE.model.bars);
            CORE.nodeAttachController.setDetachMode(true); // this mode is much safer
            CORE.BarGammaReset = new MODEL.BarGammaReset(CORE.model.bars);
            CORE.NodeMerger = new MODEL.NodeMerger(CORE.model);
            CORE.model.nodeMerge = CORE.NodeMerger.merge.bind(CORE.NodeMerger);
            CORE.barForcesController = new MODEL.BarAttachmentsController(CORE.model.loads, [MODEL.OBJECT_TYPE.BarForce, MODEL.OBJECT_TYPE.BarTrapezoidal]); // for barForce-bar "compatiblity"
            CORE.nodeBarsRelation = new MODEL.NodeBarsRelation(CORE.model.bars);
            VIEWUI.init(_viewerContainer, _this.initModules); // VIEW3D can only be initialized after VIEWUI (it contains renderer's divs)
            console.log("CORE.init: system data loaded");
            console.log("CORE.init() end");
        }
        console.log("CORE.init: getting translation data");
        MODEL.repository.getTranslation(callback);
        $(window).on('beforeunload', function() {
            if (CORE.viewerInProgress && !CORE.options.demoMode) {
                return translation.ON_UNLOAD_MESSAGE;
            }
        });
    };
    CORE.initModules = function() {
            console.log("CORE.initModules() begin");
            if (typeof CORE.options.width !== 'undefined') {
                viewerContainer.css("width", CORE.options.width); //default width = 100%
            }
            if (typeof CORE.options.height !== 'undefined') {
                viewerContainer.css("height", CORE.options.height); //default height = 100%
            }
            webGlContainer = document.getElementById("webGlContainer");
            if (webGlContainer == null || webGlContainer === 'undefined') {
                console.error("webGlContainer is null or undefined!");
                return;
            }
            // ================================ BEGIN INDEX ====================================
            scene = new THREE.Scene();
            var width = viewerContainer.innerWidth();
            var height = viewerContainer.innerHeight();
            try {
                renderer = new THREE.WebGLRenderer({
                    preserveDrawingBuffer: true,
                    antialias: true
                });
            } catch (err) {
                console.warn("Starting viewer without antialias");
                renderer = new THREE.WebGLRenderer({
                    preserveDrawingBuffer: true,
                    antialias: false
                });
            }
            //height = parseInt(window.getComputedStyle(webGlContainer, null).getPropertyValue("height"));
            //width = parseInt(window.getComputedStyle(webGlContainer, null).getPropertyValue("width"));
            //beginHeight = height;
            //beginWidth = width;
            //renderer.setSize(width, height);

            initContextLostHandler();

            renderer.setClearColor(0xffffff, 1);
            renderer.clear();
            webGlContainer.appendChild(renderer.domElement);
            canvas = renderer.context.canvas;
            renderer.autoClear = false;
            CORE.aspectRatio = ko.observable(1);
            CORE.screenSize = ko.observable({ width: width, height: height });
            CORE.refreshAspectRatio = function() { // global function, can be called from outside (change namespace?)

                if (isEmpty(webGlContainer)) {
                    console.error("webGlContainer is null or undefined!");
                    return;
                }

                width = parseInt(window.getComputedStyle(webGlContainer, null).getPropertyValue("width"));
                height = parseInt(window.getComputedStyle(webGlContainer, null).getPropertyValue("height"));

                width += 1; // to fix 1-pixel margin on the right of the container
                height += 1;

                if (CORE.options.useOrthographicProjection) {
                    camera.left = -width / 50;
                    camera.right = width / 50;
                    camera.top = height / 50;
                    camera.bottom = -height / 50;
                } else {
                    camera.aspect = width / height;
                }
                camera.updateMatrixWorld();
                camera.updateProjectionMatrix();
                renderer.setSize(width, height);
                CORE.aspectRatio(width / height);
                CORE.screenSize({ width: width, height: height });
                render();
                resizeCanvas();
            };
            CORE.refreshAspectRatio();
            if (!CORE.options.useOrthographicProjection) {
                camera.position.set(0, 0, 40);
            }
            if (CORE.options.debugMode) {
                SetStats();
            }
            //animate();
            render();
            window.addEventListener('resize',
                function() {
                    CORE.refreshAspectRatio();
                    camera.changeEvent.notifySubscribers();
                });
            resizeCanvas();
            mouseMoved = false;
            selectMultiple = false;
            raycaster = new THREE.Raycaster();
            mouse = new THREE.Vector2();
            raycasterPointCloudTreshold = 0.5;
            // ========================== END INDEX =====================
            camera.position.set(10, -10, 10);
            camera.up.set(0, 0, 1);
            camera.lookAt(new THREE.Vector3(0, 0, 0));
            camera.updateMatrix();
            CORE.initResultsRenderer();
            CORE.bitmapFont = new VIEW3D.BitmapFont();
            VIEW3D.AxisHelper.init();
            CORE.axisHelperNamespace = VIEW3D.AxisHelper;
            // var workplane1 = CORE.model.addWorkplane(new MODEL.Workplane(new MODEL.types.ThreeVectors(
            //     new MODEL.types.Vector(0, 0, 0),
            //     new MODEL.types.Vector(1, 0, 0),
            //     new MODEL.types.Vector(0, 1, 0)
            // )));
            // workplane1.name("Plane XY");
            // var workplane2 = CORE.model.addWorkplane(new MODEL.Workplane(new MODEL.types.ThreeVectors(
            //     new MODEL.types.Vector(0, 0, 0),
            //     new MODEL.types.Vector(1, 0, 0),
            //     new MODEL.types.Vector(0, 0, 1)
            // )));
            // workplane2.name("Plane XZ");

            // var workplane3 = CORE.model.addWorkplane(new MODEL.Workplane(new MODEL.types.ThreeVectors(
            //     new MODEL.types.Vector(0, 0, 0),
            //     new MODEL.types.Vector(0, 1, 0),
            //     new MODEL.types.Vector(0, 0, 1)
            // )));
            // workplane3.name("Plane YZ");

            //CORE.model.activeWorkplane(workplane1);

            if (CORE.showLoads) CORE.showLoads(false);
            if (CORE.showExceptions) {
                CORE.showExceptions(false);
            }

            if (CORE.options.disableLoadAndSave) {
                CORE.model.autoSave();
                render();
                console.log("CORE.initModules() end");
                return;
            };

            // LOAD HARDCODED LISTS FROM SERVER
            MODEL.repository.getSystemTypes(function(systemTypes) {

                for (var i = 0; i < systemTypes.loadCaseTypes.length; i++) {
                    CORE.model.addLoadCase(new MODEL.LoadCase(systemTypes.loadCaseTypes[i]));
                }

                setSupportTypes(systemTypes.supportTypes);

                //for deleting in future
                function setSupportTypes( /*supportTypes*/ ) {
                    SUPPORT_TYPE.push('Default');
                    CORE.model.addSupport(new MODEL.NodeSupport('Default',
                        1,
                        'Default',
                        1));
                };

                // setMaterialsTypes(systemTypes.materials);
                //
                //
                // function setSupportTypes(supportTypes) {
                //
                //     SUPPORT_TYPE.push('NONE');
                //     for (var i = 0; i < supportTypes.length; i++) {
                //         SUPPORT_TYPE.push(supportTypes[i].SS_NAME);
                //         var newSupport = {
                //             id: supportTypes[i].SS_ID,
                //             data: "data:image/png;base64," + supportTypes[i].SS_IMAGE
                //         };
                //         supportImageCache.push(newSupport);
                //         CORE.model.addSupport(new MODEL.NodeSupport(supportTypes[i].SS_NAME,
                //             supportTypes[i].SS_3DM_ID,
                //             supportTypes[i].SS_TYPE,
                //             supportTypes[i].SS_ID));
                //         //MODEL.repository.get3DModel(supportTypes[i].SS_ID);
                //     }
                // }
                // function setMaterialsTypes(materialsData) {
                //
                //     var loader = new THREE.MTLLoader();
                //     var materialCreator;
                //     for (var i = 0; i < materialsData.length; i++) {
                //         materialCreator = loader.parse(materialsData[i]);
                //         for (var materialName in materialCreator.materialsInfo) {
                //             if (materialCreator.materialsInfo.hasOwnProperty(materialName)) {
                //                 materials[materialName] = materialCreator.materialsInfo[materialName];
                //             }
                //         }
                //     }
                // }

                CORE.model.prepareProtoBuilder(function() {
                    CORE.model.autoSave(); // init save/load after protobuf file has been loaded
                });

                render();
                console.log("CORE.initModules() end");

            });


        } // CORE.initModules

    CORE.initResultsRenderer = function() {
        console.log("CORE.initResultsRenderer() begin");
        resultsWebGlContainer = $("#resultsWebGlContainer")[0];
        if (isEmpty(resultsWebGlContainer)) {
            var div = $('<div id="resultsWebGlContainer" class="resultsWebGlContainer"></div>');
            $("#hiddenContainers").append(div);
            resultsWebGlContainer = div[0];
            resultsWebGlContainer.appendChild(resultsWindowRenderer.domElement);
            CORE.resultsView3D.init();
            return;
        }

        //$(resultsWebGlContainer).empty();
        //    CORE.resultsView3D.dispose();

        resultsWindowScene = new THREE.Scene();
        resultsWindowCamera = new THREE.OrthographicCamera(-2.3, 2.3, 2.3, -2.3, -1000, 1000);
        try {
            resultsWindowRenderer = new THREE.WebGLRenderer({ antialias: true, alpha: true });
        } catch (err) {
            resultsWindowRenderer = new THREE.WebGLRenderer({ antialias: false, alpha: true });
        }
        var height = parseInt(window.getComputedStyle(resultsWebGlContainer, null).getPropertyValue("height"));
        var width = parseInt(window.getComputedStyle(resultsWebGlContainer, null).getPropertyValue("width"));
        beginHeight = height;
        beginWidth = width;
        resultsWindowRenderer.setSize(width, height);
        resultsWindowRenderer.setClearColor(0xFFFFFF, 1); // set to 0 for transparent background
        resultsWebGlContainer.appendChild(resultsWindowRenderer.domElement);
        resultsWindowRenderer.autoClear = false;
        raycasterPointCloudTreshold = 0.5;
        resultsWindowCamera.lookAt(new THREE.Vector3(0, 0, 0));
        resultsWindowCamera.updateProjectionMatrix();
        resultsWindowCamera.updateMatrix();
        // === grid ===
        resultsWindowGridCamera = new THREE.OrthographicCamera(-1.95, 1.95, 2.3, -2.3, -1000, 1000);
        resultsWindowGridCamera.updateProjectionMatrix();
        resultsWindowGridCamera.updateMatrix();
        resultsWindowGridScene = new THREE.Scene();
        resultsWindowGridScene.add(new VIEW3D.ResultsWindowWorkplane(null, 50, 1, '#BAC3CA'));
        CORE.resultsWindowModel = new MODEL.ResultsWindowViewModel();
        CORE.resultsWindowModel.init();
        CORE.resultsView3D = new VIEW3D.ResultsWindow.Results(CORE.resultsWindowModel);
        CORE.resultsView3D.init();
        resultsWindowScene.add(CORE.resultsView3D);
        //animate();
        render();
        console.log("CORE.initResultsRenderer() end");
    };

    //CORE.destroy = function () {
    //    CORE.view3D.disposeSubscriptions();
    //    CORE = undefined;
    //    VIEWUI = undefined;
    //    VIEW3D = undefined;
    //    $(window).off('scroll');
    //    $(window).off('resize');
    //    $('body').off('click');
    //}

    CORE.hideDefinitionLoadMode = function() {
        VIEWUI.toolbar.switchMode(MODE.SELECT);
        VIEWUI.viewModel.popupMode(null);
    }

    CORE.reset = function() {
        CORE.view3D.disposeSubscriptions();
        $(window).off('scroll');
        $(window).off('resize');
        $('body').off('click');
        VIEWUI.viewModel.popupMode(null);
    }
    CORE.showLoads = ko.observable(false);
    CORE.showExceptions = ko.observable(false);

    CORE.isResultsScreen = false;

    CORE.setPropertiesContainerIsEnabled = function() {
        $('#VIEWUI_propertiesContainer').find("input").not('.neverReadonly').prop('readonly', CORE.isResultsScreen);
    }

    CORE.hideCalculations = function() {
        VIEWUI.viewOptions.showCalcDisplacement(false);
        VIEWUI.viewOptions.showCalcForceFx(false);
        VIEWUI.viewOptions.showCalcForceFy(false);
        VIEWUI.viewOptions.showCalcForceFz(false);
        VIEWUI.viewOptions.showCalcForceMx(false);
        VIEWUI.viewOptions.showCalcForceMy(false);
        VIEWUI.viewOptions.showCalcForceMz(false);
        //CORE.model.barsResults(null);
        VIEWUI.exceptionWindow.removeInvalidExceptions([]);
        //VIEWUI.viewModel.calculationsFinished(false);
    }

    CORE.openGeometryScreen = function() {
        CORE.isResultsScreen = false;
        CORE.hideCalculations();
        CORE.setPropertiesContainerIsEnabled();
        CORE.hideDefinitionLoadMode();
        CORE.showLoads(false);
        CORE.showExceptions(false);
        $('.View3dInput').removeClass('VIEWUI_pointerDisabled');
        CORE.model.deselectAll();
    }

    CORE.openLoadScreen = function() {
        CORE.model.deselectAll();
        CORE.isResultsScreen = false;
        CORE.hideCalculations();
        CORE.setPropertiesContainerIsEnabled();
        CORE.showLoads(true);
        CORE.showExceptions(false);
        // CORE.model.loads()[0].selected(true);
        $('.View3dInput').addClass('VIEWUI_pointerDisabled');
    }

    CORE.openCalculationScreen = function(calculationModes) {
        CORE.isResultsScreen = true;
        CORE.setPropertiesContainerIsEnabled();
        CORE.hideDefinitionLoadMode();
        //disabel ocntainer of prop
        CORE.showLoads(false);
        CORE.showExceptions(true);
        $('.View3dInput').addClass('VIEWUI_pointerDisabled');
        CORE.model.deselectAll();
        if (calculationModes[0]) {
            VIEWUI.viewOptions.showCalcDisplacement(true);
        }
        if (calculationModes[1]) {
            VIEWUI.viewOptions.showCalcForceFx(true);
        }
        if (calculationModes[2]) {
            VIEWUI.viewOptions.showCalcForceFy(true);
        }
        if (calculationModes[3]) {
            VIEWUI.viewOptions.showCalcForceFz(true);
        }
        if (calculationModes[4]) {
            VIEWUI.viewOptions.showCalcForceMx(true);
        }
        if (calculationModes[5]) {
            VIEWUI.viewOptions.showCalcForceMy(true);
        }
        if (calculationModes[6]) {
            VIEWUI.viewOptions.showCalcForceMz(true);
        }
        if (!VIEWUI.viewModel.calculationsFinished()) {
            CORE.calculate();
        }
    }

    CORE.calculate = function() {
        var proto = CORE.model.save();
        VIEWUI.popups.calculationsProgress.show();
        CORE.calculations.calculate(proto);
        if (typeof CORE.options.callbackForCalculationStateChanging === 'function') {
            CORE.options.callbackForCalculationStateChanging(true);
        }
    }

    CORE.getSaveData = function() {
        var saveData = {
            model: model = CORE.model.save(),
            subProjectId: window.subProjectId
        };

        saveData.model.typicalId = window.constructionData.typicalId;
        saveData.model.system = window.constructionData.system;
        saveData.model.corrosion = window.constructionData.corrosion;
        saveData.model.seismic = window.constructionData.seismic;
        saveData.model.id = window.constructionData.id;
        saveData.model.jobId = window.constructionData.jobId;

        return saveData;
    }

    CORE.getLocalModel = function() {
        return {
            constructionData: CORE.model.save(),
            constraints: CORE.model.constraints.data()
        };
    }

    CORE.clearResults = true;

    CORE.setLocalModel = function(model) {
        //CORE.model.loadModel(model.constructionData);
        CORE.clearResults = false;
        model.constraints.constructionData = model.constructionData;
        CORE.model.constraints.data(model.constraints);
        if (model.constructionResults != undefined) {
            CORE.calculations.loadResults(model.constructionResults)
        }
    }

    CORE.saveInNewSubproject = function(subId) {
        window.subProjectId = subId;
        var model = CORE.model.save();
        return new Promise((resolve, reject) => {
            CORE.model.sendProto(model, undefined, () => {
                resolve();
            });
        })
    }

    CORE.generateSS = function() {
        let img = VIEWUI.toolbar.generateReport();
        return img;
    }

    CORE.returnHeightOfLoadsTableViewer = function() {
        return CORE.heightOfLoadsTableViewer;
    }

    CORE.showRealisticView = function() {
        VIEWUI.viewOptions.showRealElements(true);
    }

    CORE.showEngineringView = function() {
        VIEWUI.viewOptions.showRealElements(false);
    }

    CORE.loadFromDb = function() {
        return new Promise((resolve, reject) => {
            CORE.model.load(CORE.options.appId, CORE.options.projId, CORE.options.jobId, 0, resolve, reject);
        });
    }

    CORE.visibleCalcForce = function(forceName) {
        VIEWUI.viewOptions.visibleCalcForce(forceName);
    }

    CORE.showCalcDisplacement = function(shouldShow) {
        VIEWUI.viewOptions.showCalcDisplacement(shouldShow);
    }

    CORE.showCalcForces = function(forceName, shouldShow) {
        if (forceName == "Fx") {
            VIEWUI.viewOptions.showCalcForceFx(shouldShow);
        } else if (forceName == "Fy") {
            VIEWUI.viewOptions.showCalcForceFy(shouldShow);
        } else if (forceName == "Fz") {
            VIEWUI.viewOptions.showCalcForceFz(shouldShow);
        } else if (forceName == "Mx") {
            VIEWUI.viewOptions.showCalcForceMx(shouldShow);
        } else if (forceName == "My") {
            VIEWUI.viewOptions.showCalcForceMy(shouldShow);
        } else if (forceName == "Mz") {
            VIEWUI.viewOptions.showCalcForceMz(shouldShow);
        }
    }

    CORE.setShowLoads = function(value) {
        VIEWUI.viewOptions.loadsVisible(value);
    }

    CORE.setShowDimensionLines = function(value) {
        VIEWUI.viewOptions.measurementsVisible(value);
    }

    CORE.setShowMeshes = function(value, type) {
        var collection = Extension[type];
        collection.forEach(eachObj => {
            eachObj.material.transparent = !value;
        });
        render();
    }

    CORE.getFontSize = function() {
        if (CORE.constructionView3D.barsResults.displacement) {
            return CORE.constructionView3D.barsResults.displacement.children[0].material.uniforms.scale.value;
        }
        return 1.0; //default value
    }
    CORE.changeFontSize = function(value) {
        if (CORE.constructionView3D.barsResults.displacement) {
            CORE.constructionView3D.barsResults.displacement.children[0].material.uniforms.scale.value = value;
        }
        if (CORE.constructionView3D.barsResults.forces) {
            CORE.constructionView3D.barsResults.forces.fx.children[2].material.uniforms.scale.value = value;
            CORE.constructionView3D.barsResults.forces.fy.children[2].material.uniforms.scale.value = value;
            CORE.constructionView3D.barsResults.forces.fz.children[2].material.uniforms.scale.value = value;
            CORE.constructionView3D.barsResults.forces.mx.children[2].material.uniforms.scale.value = value;
            CORE.constructionView3D.barsResults.forces.my.children[2].material.uniforms.scale.value = value;
            CORE.constructionView3D.barsResults.forces.mz.children[2].material.uniforms.scale.value = value;
        }
        render();
    }
    CORE.constructionTransparencyChange = function(event) {
        const value = 1.0 - event;
        Extension.UNIFORMS.setOpacity(value);
        render();

    }

    CORE.changeConstructionTransparency = function(value) {

        render();
    }

    CORE.changeLoadFontSize = function(value) {
        CORE.loadFontScale(value);
    }

    CORE.changeLoadModelScale = function(value) {
        CORE.loadModelScale(value);
    }

    CORE.stylePropertiesContainer = function(maxHeightDiff, top) {
        if (document.getElementById("VIEWUI_propertiesContainer") !== null) {
            document.getElementById("VIEWUI_propertiesContainer").style.maxHeight = "calc(100% - " + maxHeightDiff + "px)";
            document.getElementById("VIEWUI_propertiesContainer").style.top = top + "px";
        }
    }

    CORE.zoomAll = function() {
        CORE.view3D.cameraControl.zoomAll();
    }

    CORE.getViewUi = function() {
        return VIEWUI;
    }

    CORE.getTranslations = function () {
        return translation;
    }
});