﻿CORE.modelConverter = {

    ignoredProperties: ["selectedObjects"/*, "selected"*/, "modelArrays", "loads", "supports", "workplanes"],
   
    doNotObserve: ["type", "id"],
    //deepCopyProperties: ["loads"],

    // Rules of convertion:
    // - all top level non-array properties are ignored
    // - properties and arrays in ignoredProperties are ignored
    // - all related objects which contain an 'id' property (e.g. nodes in a bar) are first converted to SimpleRelatedObject
    // - SimpleRelatedObject contains only type and id
    convertModelToJson: function (model) {

        // strip observables
        model = ko.toJS(model);

        // convert related objects to simple related objects (containing only type and id)
        for (arrayName in model) {                           // for each array


            var array = model[arrayName];
            var ignoreThis = this.ignoredProperties.indexOf(arrayName) !== -1;

            if (!array || array.constructor !== Array || ignoreThis) { delete model[arrayName]; continue; } // ignore

            for (var i = 0, ii = array.length; i < ii; i++)     // for each object
            {
                var object = array[i];
                for (var propName in object) {                      // iterate over properties

                    //if (arrayName === 'bars') debugger;

                    var ignoreThis = this.ignoredProperties.indexOf(propName) !== -1;
                    if (ignoreThis) { delete model[arrayName][i][propName]; continue; }

                    var prop = object[propName];
                    if (typeof prop === 'undefined')
                        object[propName] = null;

                    //var deepCopy = isElementInSet(this.deepCopyProperties, propName);

                    if (prop && prop.constructor === Array) {
                        for (var j = 0, jj = prop.length; j < jj; j++) {
                            object[propName][j] = this.getSimpleRelatedObject(prop[j]);   // simpleRelatedObject contains only type and id
                        }
                    }

                    if (prop && typeof prop.id !== 'undefined') {
                        object[propName] = this.getSimpleRelatedObject(prop);
                    }
                }
            }

        }

        json = JSON.stringify(model);

        return json;

    },

    getSimpleRelatedObject: function (object) {                  // converts property to simpleRelatedObject (contains only type and id)
        var simpleRelatedObject = {};
        if (isEmpty(object.id))
            throw 'CORE.storageManager.getSimpleRelatedObject: Missing id in related object';
        if (isEmpty(object.type))
            throw 'CORE.storageManager.getSimpleRelatedObject: Missing type in related object';
        if (typeof object.id !== 'number')
            throw 'CORE.storageManager.getSimpleRelatedObject: object.id is not a number';
        simpleRelatedObject.id = object.id;
        simpleRelatedObject.relatedType = object.type;
        if(!isEmpty(object.parentType))
            simpleRelatedObject.relatedParentType = object.parentType;

        return simpleRelatedObject;
    },

    convertJsonToModel: function (json) {

        //var model = 

        //var model = JSON.parse(json);

        //var vm = ko.mapping.fromJS(model);
        var orginalModel = JSON.parse(json);

        var model = {};
        model.nodes = [];
        var orginalNodes = orginalModel.Header.Nodes.Item;
        for (var i = 0 ; i < orginalNodes.length; i++) {
            var orginalNode = orginalNodes[i];
            var node = { selected: false, type: "Node", connectorType: null, connector: { type: "NodeConnector", connectionsCount: function () { } }, createsBars: function () { } };
            node.id = Number(orginalNode["@ID"]);
            node.name = orginalNode["@Name"];
            node.position = { x: Number(orginalNode["@X"]) / 100, y: Number(orginalNode["@Y"]) / 100, z: Number(orginalNode["@Z"]) / 100, }
            model.nodes.push(node);
        }

        model.bars = [];
        var orginalBars = orginalModel.Header.Rods.Item;
        for (var i = 0 ; i < orginalBars.length; i++) {
            var orginalBar = orginalBars[i];
            var bar = { selected: false, type: "Bar", profile : null, material : null, rotation:{x:0,y:0,z:0} };
            bar.name = orginalBar["@Name"];
            bar.id = Number(orginalBar["@ID"]);
            bar.node1 = { id: Number(orginalBar["@N1"]), type: "Node" }
            bar.node2 = { id: Number(orginalBar["@N2"]), type: "Node" }
            model.bars.push(bar);
        }

        return model;//ko.mapping.fromJSON(json);

    },


}