﻿CORE.storageManager.load = {
    loadUsingProtoBuf: function (appId, projId, jobId) {

        ProtoBuf.loadProtoFile(CORE.rootPath + "js/core/ConstructionModel.proto", function (err, builder) {
            if (!isEmpty(err)) {
                console.error("CORE.storageManager.loadUsingProtoBuf:"); console.error(err);
                //throw err;
                return;
            }
            console.log("CORE.storageManager.loadUsingProtoBuf: proto file loaded successfully");

            var protoModel;
            try {
                CORE.webApiManager.loadFromDb(/*80500, 23492, 25464 */CORE.options.appId, CORE.options.projId, CORE.options.jobId, callback);

                function callback(data) {

                    var ConstructionModel = builder.build("ConstructionModel"); // ConstructionModel this is a constructor

                    var b64string = data;//constructionModel.toBase64(); //for base64 encoding

                    protoModel = ConstructionModel.decode64(b64string);

                    console.log("CORE.storageManager.loadUsingProtoBuf: proto message parsed successfully");

                    CORE.storageManager.load.load(protoModel);
                }

            }
            catch(error) {
                console.error("CORE.storageManager.loadUsingProtoBuf: proto message parsing failed, reason: " + error);
                return;
            }

            
 
        });
    },


    load: function (protoModel) {

        console.log("CORE.storageManager.load: deleting all objects...");
        CORE.model.deselectAll();
        CORE.storageManager.removeAll();
        console.log("CORE.storageManager.load: all objects deleted");

        console.log("CORE.storageManager.load.load: begin proto to model conversion");

        var arraysToConvert = ["loads", "supports", "nodes", "bars"];

        for(var i=0, ii=arraysToConvert.length; i<ii; i++)
        {
            var arrayName = arraysToConvert[i];
            convertAndSaveArray(arrayName, protoModel, CORE.model);
        }

        function convertAndSaveArray(arrayName, protoModel, model) {

            CORE.storageManager.load.currentArrayName = arrayName;

            if(isEmpty(protoModel[arrayName])) 
            {
                console.warn("CORE.storageManager.load.load.convertArray: protoModel does not contain array " + arrayName);
                return;
            }

            var modelArray = [];    // output array
            var convertedItemCount = 0;

            var loadsArray = CORE.model.modelArrays[MODEL.OBJECT_TYPE.Load]();      // existing model arrays (for assigning i.e. loads to nodes (node.loads = ...))
            var supportsArray = CORE.model.modelArrays[MODEL.OBJECT_TYPE.Support]();
            var nodesArray = CORE.model.modelArrays[MODEL.OBJECT_TYPE.Node]();

            for(var i=0, ii=protoModel[arrayName].length; i< ii; i++)
            {
                CORE.storageManager.load.currentObjectIndex = i;

                var objectType = protoModel[arrayName][i].type;
                if(isEmpty(objectType)) 
                {
                    console.error("CORE.storageManager.load.load: cannot get type information from " + i + ". object in array: " + arrayName);
                    return;
                }

                var protoToModelFunction = CORE.storageManager.load.protoToModel[objectType];
                if(isEmpty(protoToModelFunction)) 
                {
                    console.error("CORE.storageManager.load.load: missing protoToModel function for type " + objectType);
                    return;
                }

                var modelObject = protoToModelFunction(protoModel[arrayName][i], loadsArray, supportsArray, nodesArray); // convert proto message object using appropriate conversion function

                var parentObjectType = objectType;                                // TODO: change to uniform type specification (object.type = Node,Bar,Load...)
                if (!isEmpty(modelObject.parentType))
                {
                    parentObjectType = modelObject.parentType;
                }

                var addFunction = model.addFunctions[parentObjectType];
                if (isEmpty(addFunction)) {
                    console.error("CORE.storageManager.load.load: missing add function for type " + parentObjectType);
                    return;
                }

                model.addFunctions[parentObjectType](modelObject, model);                      // add model object to the model array using appropriate add function

                convertedItemCount++;
            }

            console.info("CORE.storageManager.load.load: loaded " + convertedItemCount + " " + arrayName);
        }

        console.log("CORE.storageManager.load.load: load completed");

    },
}