var Extension = Extension || {};

Extension.arms = [];
Extension.pointMesh = '';
Extension.possibleArmDirections = [
    { id : 'minX', text : 'dir -X', dirVec : new THREE.Vector3(-1,0,0) },
    { id : 'minY', text : 'dir -Y', dirVec : new THREE.Vector3(0,-1,0) },
    { id : 'minZ', text : 'dir -Z', dirVec : new THREE.Vector3(0,0,-1) },
    { id : 'maxX', text : 'dir +X', dirVec : new THREE.Vector3(1,0,0) },
    { id : 'maxY', text : 'dir +Y', dirVec : new THREE.Vector3(0,1,0) },
    { id : 'maxZ', text : 'dir +Z', dirVec : new THREE.Vector3(0,0,1) }
];
Extension.possibleReleaseCodes = [
    { id : 1, text : 'YZ hinged Node in local' },
    { id : 2, text : 'X hinged Node in Global' },
    { id : 3, text : 'Y hinged Node in Global' },
    { id : 4, text : 'Z hinged Node in Global' },
    { id : 5, text : 'XYZ hinged Node in Global' },
    { id : 6, text : 'pinned Node, free in X' },
    { id : 7, text : 'X and Z hinged Node' },
    { id : 8, text : 'Seismic X is fix' },
    { id : 9, text : 'Seismic X is fix' },
    { id : 11, text : 'X hinged Node in local' },
    { id : 12, text : 'Y hinged Node in local' },
    { id : 13, text : 'Z hinged Node in local' },
    { id : 14, text : 'pinned Node with real release' },
    { id : 15, text : 'X and Z real hinged Node' },
    { id : 16, text : 'Y and Z real hinged Node with x translation' },
    { id : 17, text : 'Y and Z real hinged Node' },
    { id : 18, text : 'X hinged Node in local' },
    { id : 19, text : 'Y hinged Node in local' }
];
Extension.armsIntersection = new THREE.Vector3();
Extension.armsTranslationVector = new THREE.Vector3(0, 0, 0);
Extension.newArmLength = 0;
Extension.modelConfigBoundingBoxMin = null;
Extension.modelConfigBoundingBoxMax = null;
Extension.isDouble = false;

Extension.prepareFullConnectorConfiguration = function() {
    var connector = { 
        _id : Extension.stpId, 
        center : Extension.armsIntersection,
        translationVector: Extension.armsTranslationVector, 
        origin: new THREE.Vector3().addVectors(Extension.armsIntersection, Extension.armsTranslationVector),
        rotation : {x : 0 , y : 0, z : 0},
        arms : [],
        isDouble : Extension.isDouble,
        config : {
            boxMin : Extension.modelConfigBoundingBoxMin,
            boxMax : Extension.modelConfigBoundingBoxMax,
            normal : new THREE.Vector3(0,0,1)
        }
    };
    for (var j = 0; j < Extension.arms.length; j++) {
        var armDef = Extension.arms[j];
        armDef.config.length = armDef.armLength;
        var arm = {
            id: armDef.id,
            direction: armDef.dir !== null ? armDef.dir : {x : 0 , y : 0, z : 0},
            baseX: armDef.baseX,
            baseY: armDef.baseY,
            baseZ: armDef.baseZ,
            config: armDef.config,
            pushButtonConfig: "{}",
            armLength: armDef.armLength
        };
        connector.arms.push(arm);
    }

    return connector;
}

Extension.getArmDirection = function(dirVector) {
    if(dirVector.x < -0.5) return Extension.possibleArmDirections[0];
    if(dirVector.y < -0.5) return Extension.possibleArmDirections[1];
    if(dirVector.z < -0.5) return Extension.possibleArmDirections[2];
    if(dirVector.x > 0.5) return Extension.possibleArmDirections[3];
    if(dirVector.y > 0.5) return Extension.possibleArmDirections[4];
    if(dirVector.z > 0.5) return Extension.possibleArmDirections[5];
}

Extension.createArmDefiniton = function(arm, i) {
    var armVector = Extension.calculateArmDir(arm.mesh, Extension.armsIntersection);
    var armDef = { id : i +1 , dir : armVector.clone().normalize() };
    armDef.name = "Arm " + armDef.id;
    armDef.dirObj = [Extension.getArmDirection(armDef.dir)];
    armDef.lineGroupMesh = arm.lineGroupMesh;
    armDef.armLength = armVector.clone().length();
    armDef.config = {};
    var calculations = Extension.calculateArmSize(armDef, Extension.connectorMeshes);
    armDef.config.distances = calculations.distances;
    armDef.baseX = calculations.versors.baseX;
    armDef.baseY = calculations.versors.baseY;
    armDef.baseZ = calculations.versors.baseZ;

    return armDef;
}

Extension.removeArmDefintion = function(arm) {
    var newArms = [];
    for (var j = 0; j < Extension.arms.length; j++) {
        var itArm = Extension.arms[j];
        if(arm.id !== itArm.id) {
            itArm.id = newArms.length + 1;
            itArm.name = "Arm " + itArm.id;
            var newMesh = Extension.createArmLineMeshFromPoints(Extension.armsIntersection, itArm.lineGroupMesh.endPos, itArm.id);
            Extension.resetLineGroupMesh(itArm, newMesh);

            newArms.push(itArm);
        } else {
            scene.remove(itArm.lineGroupMesh);
        }
    }
    Extension.arms = newArms;
}

Extension.createEmptyArm = function() {
    var id = Extension.arms.length + 1;
    var armDef = { id : id, name : "Arm " + id, dirObj : null, dir : null, armLength : 0, config : {} };
    Extension.arms.push(armDef);
    return id;
}

Extension.getDirectionById = function(id) {
    for (var j = 0; j < Extension.possibleArmDirections.length; j++) {
        var dir = Extension.possibleArmDirections[j];
        if(id === dir.id) return dir;
    }
}

Extension.getRelaseById = function(id) {
    for (var j = 0; j < Extension.possibleReleaseCodes.length; j++) {
        var rc = Extension.possibleReleaseCodes[j];
        if(id === rc.id) return [rc];
    }
    return [];
}

// newDir is obj from Extension.possibleArmDirections
Extension.changeArmDirection = function(armDef, dirId) {
    var newDir = Extension.getDirectionById(dirId);
    armDef.dir = newDir.dirVec;
    armDef.dirObj = [newDir];
    armDef.armLength = Extension.newArmLength;
    var originalArmsIntersection = Extension.armsIntersection.clone();
    originalArmsIntersection.x += Extension.armsTranslationVector.x;
    originalArmsIntersection.y += Extension.armsTranslationVector.y;
    originalArmsIntersection.z += Extension.armsTranslationVector.z;
    var newEnd = originalArmsIntersection.clone().add(newDir.dirVec.clone().multiplyScalar(Extension.newArmLength));
    var newMesh = Extension.createArmLineMeshFromPoints(originalArmsIntersection, newEnd, armDef.id);

    Extension.resetLineGroupMesh(armDef, newMesh);
}

Extension.resetLineGroupMesh = function(armDef, newMesh) {
    scene.remove(armDef.lineGroupMesh);
    armDef.lineGroupMesh = newMesh;
    scene.add(armDef.lineGroupMesh);
}

Extension.loadConfiguration = function(config, meshes) {
    Extension.armsIntersection = new THREE.Vector3(config.center.x, config.center.y, config.center.z);
    if(config.config != null && config.config != undefined) {
        Extension.modelConfigBoundingBoxMin = config.config.boxMin;
        Extension.modelConfigBoundingBoxMax = config.config.boxMax;
    }
    Extension.isDouble = config.isDouble;
    if(config.translationVector!==null && config.translationVector!==undefined){
        Extension.armsTranslationVector = new THREE.Vector3(config.translationVector.x,config.translationVector.y,config.translationVector.z);
    }
    Extension.pointMesh = Extension.createPointMesh(Extension.armsIntersection);
    scene.add(Extension.pointMesh);
    
    for(var i = 0 ; i < config.arms.length ; i++) {
        var arm = config.arms[i];
        Extension.newArmLength = arm.armLength;
        var armDef = Extension.loadArmDefiniton(arm, i);
        scene.add(armDef.lineGroupMesh);
        Extension.arms.push(armDef);
        if(armDef.config.distances === undefined) {
            armDef.config.distances = Extension.calculateArmSize(armDef, meshes).distances;
        }
        if(armDef.baseX === undefined) {
            var versors = Extension.calculateArmSize(armDef, meshes).versors;
            armDef.baseX = versors.baseX;
            armDef.baseY = versors.baseY;
            armDef.baseZ = versors.baseZ;
        }
    }
}

Extension.loadArmDefiniton = function(arm, i) {
    var d = arm.direction;
    var armDef = { id : i +1 , dir : new THREE.Vector3(d.x, d.y, d.z) };
    
    armDef.name = "Arm " + armDef.id;
    armDef.armLength = arm.armLength;
    armDef.dirObj = [Extension.getArmDirection(armDef.dir)];
    var end = Extension.calculateArmEnd(armDef, 1);
    armDef.lineGroupMesh = Extension.createArmLineMeshFromPoints(Extension.armsIntersection, end, armDef.id);
    armDef.config = arm.config;

    return armDef;
}

Extension.calculateArmEnd = function (armDef, scale) {
    return armDef.dir.clone().multiplyScalar(armDef.armLength*scale).add(Extension.armsIntersection);
}

Extension.rewriteArms = function(){
    for(let i=0; i<Extension.arms.length; i++){
        Extension.changeArmDirection(Extension.arms[i],Extension.arms[i].dirObj[0].id);
    }
    scene.remove(Extension.pointMesh);
    var pointMeshCoordinates = Extension.armsIntersection.clone();
    pointMeshCoordinates.x += Extension.armsTranslationVector.x;
    pointMeshCoordinates.y += Extension.armsTranslationVector.y;
    pointMeshCoordinates.z += Extension.armsTranslationVector.z;
    Extension.pointMesh = Extension.createPointMesh(pointMeshCoordinates);
    scene.add(Extension.pointMesh);
    
    render();
}
