var Extension = Extension || {};
Extension.armSizeIterationCount = 20;
Extension.initialMinDistance = 99999999;
Extension.calculateArmSize = function(armDef, meshes) {
    var armDir = armDef.dir;
    if(armDef.dirObj !== undefined && armDef.dirObj[0] !== undefined && armDef.dirObj[0].dirVec !== undefined) {
        armDir = armDef.dirObj[0].dirVec;
    }
    var versors = Extension.calculateVersorsFromDirection(armDir);
    var distances = [];
    var versYres = Extension.processArmSizeVersor(armDef, versors.baseY, meshes);
    console.log('For arm ' + armDef.id + ' for versor base y distance is ' + versYres.mindist);
    if(versYres.closest !== null) distances.push(new THREE.Vector3(0,1,0).multiplyScalar(versYres.mindist));

    var versMinYres = Extension.processArmSizeVersor(armDef, versors.baseY.clone().multiplyScalar(-1), meshes);
    console.log('For arm ' + armDef.id + ' for versor base - y distance is ' + versMinYres.mindist);
    if(versMinYres.closest !== null) distances.push(new THREE.Vector3(0,-1,0).multiplyScalar(versMinYres.mindist));

    var versZres = Extension.processArmSizeVersor(armDef, versors.baseZ, meshes);
    console.log('For arm ' + armDef.id + ' for versor base z distance is ' + versZres.mindist);
    if(versZres.closest !== null) distances.push(new THREE.Vector3(0,0,1).multiplyScalar(versZres.mindist));

    var versMinZres = Extension.processArmSizeVersor(armDef, versors.baseZ.clone().multiplyScalar(-1), meshes);
    console.log('For arm ' + armDef.id + ' for versor base - z distance is ' + versMinZres.mindist);
    if(versMinZres.closest !== null) distances.push(new THREE.Vector3(0,0,-1).multiplyScalar(versMinZres.mindist));

    return {distances: distances, versors: versors};
}

Extension.processArmSizeVersor = function(armDef, versor, meshes) {
    var mindist = Extension.initialMinDistance; var closest = null;
    for (let j = 1; j < Extension.armSizeIterationCount - 3; j++) {
        var current = Extension.calculateArmEnd(armDef , j/Extension.armSizeIterationCount);
        var res = Extension.processArmSizeRay(current, versor, meshes);
        if(res.mindist < mindist) {
            mindist = res.mindist;
            closest = res.closest;
        }
    }
    return { mindist : mindist, closest : closest };
}

Extension.processArmSizeRay = function(point, direction, meshes) {
    var mindist = Extension.initialMinDistance; var closest = null;
    var ray = new THREE.Ray(point, direction);
    for (var j = 0; j < meshes.length; j++) {
        var mesh = meshes[j];
        var res = Extension.intersectRayWithMesh(ray, mesh);
        if(res.mindist < mindist) {
            mindist = res.mindist;
            closest = res.closest;
        }
    }
    return { mindist : mindist, closest : closest };
}

Extension.intersectRayWithMesh = function(ray, mesh) {
    var mindist = Extension.initialMinDistance; var closest = null;
    var verticies = mesh.geometry.attributes.position.array; var indicies = mesh.geometry.index.array;
    for (var k = 0; k < indicies.length; k+=3) {
        var v1 = Extension.getPoint(indicies[k], verticies);
        var v2 = Extension.getPoint(indicies[k+1], verticies);
        var v3 = Extension.getPoint(indicies[k+2], verticies);
        var inter = ray.intersectTriangle(v1,v2,v3, false);
        if(inter !== null) {
            var dist = ray.origin.distanceTo(inter);
            if(dist < mindist) {
                mindist = dist;
                closest = inter;
            }
        }
    }
    return { mindist : mindist, closest : closest };
}