var Extension = Extension || {};

Extension.BaseArmValidator = function(mesh, plane) {
    this.mesh = mesh;
    this.plane = plane;
    this.planeNormal = plane.normal;

    this.isValidArm = function() {
        return true;
    }

    this.isCorrectDirection = function() {
        var bbox = this.mesh.boundingBox;
        var xDiff = bbox.max.x - bbox.min.x; var yDiff = bbox.max.y - bbox.min.y; var zDiff = bbox.max.z - bbox.min.z;
        if(Math.abs(this.planeNormal.x) > 0.5) return xDiff > yDiff && xDiff > zDiff;
        if(Math.abs(this.planeNormal.y) > 0.5) return yDiff > xDiff && yDiff > zDiff;
        if(Math.abs(this.planeNormal.z) > 0.5) return zDiff > yDiff && zDiff > xDiff;
        return false;
    }

    this.isSteelSheet = function() {
        var bbox = this.mesh.boundingBox;
        minDim = 0;
        if(Math.abs(this.planeNormal.x) > 0.5) minDim =  bbox.max.x - bbox.min.x;
        if(Math.abs(this.planeNormal.y) > 0.5) minDim =  bbox.max.y - bbox.min.y;
        if(Math.abs(this.planeNormal.z) > 0.5) minDim =  bbox.max.z - bbox.min.z;
    
        // var minDim = Math.min(bbox.max.x - bbox.min.x, bbox.max.y - bbox.min.y);
        // minDim = Math.min(minDim, bbox.max.z - bbox.min.z);
    
        return minDim < 0.007;//less them 0.7 cm
    }

    this.calculateInsideBox = function(bbox) {
        return new THREE.Box3().setFromCenterAndSize(bbox.getCenter(), bbox.getSize().divideScalar(3));
    }

    this.isInsideEmpty = function() {
        var insideBox = this.calculateInsideBox(this.mesh.boundingBox);
    
        for (var j = 0; j < this.mesh.geometry.vertices.length; j++) {
            var v = this.mesh.geometry.vertices[j];
            if(insideBox.containsPoint(v)) return false;
        }
    
        return true;
    }

    this.areArmDimensionsFitForCrossSection = function() {
        var bbox = this.mesh.boundingBox;
        var xDiff = bbox.max.x - bbox.min.x; var yDiff = bbox.max.y - bbox.min.y; var zDiff = bbox.max.z - bbox.min.z;
        if(Math.abs(this.planeNormal.x) > 0.5) return yDiff/zDiff > 0.25 && yDiff/zDiff < 4;
        if(Math.abs(this.planeNormal.y) > 0.5) return xDiff/zDiff > 0.25 && xDiff/zDiff < 4;
        if(Math.abs(this.planeNormal.z) > 0.5) return yDiff/xDiff > 0.25 && yDiff/xDiff < 4;
        return false;
    }
    
    this.isSpaceForCrossSection = function(mesh){
        var insideBox = this.calculateInsideBox(this.mesh.boundingBox);
        var faces = this.mesh.geometry.faces
        for (var k = 0; k < faces.length; k++) {
            var face = faces[k];
            var v1 = this.mesh.geometry.vertices[face.a];
            var v2 = this.mesh.geometry.vertices[face.b];
            var v3 = this.mesh.geometry.vertices[face.c];
            if(Extension.boxIntersectsTriangle(insideBox, new THREE.Triangle(v1,v2,v3))) return false;
        }
        return true;
    }
}

Extension.ConnectorArmValidator = function(mesh, plane) {
    Extension.BaseArmValidator.call(this, mesh, plane);

    this.isValidArm = function() {
        if(!this.isCorrectDirection(this.mesh, this.plane.normal)) return false;
        if(this.isSteelSheet(this.mesh, this.plane.normal)) return false;
        //if(!Extension.isInsideEmpty(mesh)) return false;//possibly it is no longer needed - isCorrectDirection resolves cases that this test had before
        return true;
    }
}

Extension.ConnectorBaseArmValidator = function(mesh, plane) {
    Extension.BaseArmValidator.call(this, mesh, plane);

    this.isValidArm = function() {
        if(!this.isCorrectDirection(this.mesh, this.plane.normal)) return false;
        if(this.isSteelSheet(this.mesh, this.plane.normal)) return false;
        if(!this.isSpaceForCrossSection(this.mesh)) return false;
        if(!this.areArmDimensionsFitForCrossSection(this.mesh, this.plane.normal)) return false;
        return true;
    }
}