var Extension = Extension || {};

Extension.processBaseArms = function(arms, mesh) {
    var validArms = [];
    for(var i = 0 ; i < arms.length ; i++) {
        var arm = arms[i];
        if(arm.isValid) validArms.push(arm);
    }

    if(validArms.length === 1) {
        var arm = validArms[0];
        Extension.armsIntersection = Extension.calculateBaseArmPosition(arm, mesh);
        Extension.pointMesh = Extension.createPointMesh(Extension.armsIntersection);
        scene.add(Extension.pointMesh);

        Extension.newArmLength = Extension.calculateArmDir(arm.mesh, Extension.armsIntersection).length();
        arm.lineGroupMesh = Extension.createArmLineMesh(arm.mesh, Extension.armsIntersection, 1);
        scene.add(arm.lineGroupMesh);
        Extension.arms.push(Extension.createArmDefiniton(arm, 0));
    }
}

Extension.calculateBaseArmPosition = function(arm, mesh) {
    var point = arm.line.start;
    if(arm.plane.normal.x > 0.5 || arm.plane.normal.y > 0.5 || arm.plane.normal.z > 0.5) point = arm.line.end;
    var dir = arm.plane.normal.clone().multiplyScalar(-1).normalize();
    var res = Extension.processArmSizeRay(point, dir, Extension.connectorMeshes);
    if(res.closest === null) {
        var ray = new THREE.Ray(point, dir);
        res.closest = ray.intersectPlane(Extension.getBaseMovePlane(dir, mesh));
        var mover = new MovePointOutsideOpeningExecutor(res.closest, arm.plane.normal.clone().normalize());
        res.closest = mover.movePointOutsideOpening();
    } else {
        res.closest.add(arm.plane.normal.clone().normalize().multiplyScalar(0.0001));
    }
    return res.closest;//can be null then bounding box must be used
}

function MovePointOutsideOpeningExecutor(end, normal) {
    this.end = end;
    this.normal = normal;
    this.movePointOutsideOpening = function() {
        var start = this.end.clone().add(this.normal.clone().divideScalar(100));
        var step = (this.end.clone().sub(start)).divideScalar(Extension.armSizeIterationCount);
        var versors = Extension.calculateVersorsFromDirection(step);
        var p = start.clone();
        var lastRes = null;
        for (var j = 0; j < Extension.armSizeIterationCount; j++) {
            p.add(step);
            var res = this.movePointOutsideOpeningSingleIteration(p, versors);
            if(this.compareMovePointResults(lastRes, res)) {
                lastRes = res;
            } else {
                return p.sub(step);
            }
        }
        return this.end;
    }
    this.compareMovePointResults = function(lastRes, res) {
        if(lastRes === null) return true;
        for (var j = 0; j < lastRes.length; j++) {
            if(res[j].mindist > lastRes[j].mindist*0.9) return true;
        }
        return false;
    }
    this.movePointOutsideOpeningSingleIteration = function(p, versors){
        var res = [];
        res.push(Extension.processArmSizeRay(p, versors.baseY, Extension.connectorMeshes));
        res.push(Extension.processArmSizeRay(p, versors.baseZ, Extension.connectorMeshes));
        res.push(Extension.processArmSizeRay(p, versors.baseY.clone().multiplyScalar(-1), Extension.connectorMeshes));
        res.push(Extension.processArmSizeRay(p, versors.baseZ.clone().multiplyScalar(-1), Extension.connectorMeshes));
        return res;
    }
}

Extension.createBaseArm = function(mesh, position, index, plane, color) {
    var armTriangles = Extension.processPlaneIntersection(mesh, position, index, plane, color);
    var armMesh = Extension.createTrianglesMesh(armTriangles, color, plane.normal);
    var validator = new Extension.ConnectorBaseArmValidator(armMesh, plane);
    return { mesh : armMesh, isValid : validator.isValidArm(), line : Extension.calculateArmLine(mesh, armMesh, plane, true), plane : plane };
}

Extension.recogniseBaseArms = function(mesh, position, index) {
    var bbox = new THREE.Box3().setFromObject(mesh);
    Extension.modelConfigBoundingBoxMin = bbox.min;
    Extension.modelConfigBoundingBoxMax = bbox.max;
    var arms = [];

    var planeX = new THREE.Plane(new THREE.Vector3(-1,0,0), bbox.min.x);
    arms.push(Extension.createBaseArm(mesh, position, index, planeX, new THREE.Vector3(1,0,0)));
    var planeZ = new THREE.Plane(new THREE.Vector3(0,0,-1), bbox.min.z);
    arms.push(Extension.createBaseArm(mesh, position, index, planeZ, new THREE.Vector3(0,0,1)));
    var planeY = new THREE.Plane(new THREE.Vector3(0,-1,0), bbox.min.y);
    arms.push(Extension.createBaseArm(mesh, position, index, planeY, new THREE.Vector3(0,1,0)));
    var planeX2 = new THREE.Plane(new THREE.Vector3(1,0,0), -bbox.max.x);
    arms.push(Extension.createBaseArm(mesh, position, index, planeX2, new THREE.Vector3(1,1,0)));
    var planeZ2 = new THREE.Plane(new THREE.Vector3(0,0,1), -bbox.max.z);
    arms.push(Extension.createBaseArm(mesh, position, index, planeZ2, new THREE.Vector3(0,1,1)));
    var planeY2 = new THREE.Plane(new THREE.Vector3(0,1,0), -bbox.max.y);
    arms.push(Extension.createBaseArm(mesh, position, index, planeY2, new THREE.Vector3(1,0,1)));

    Extension.processBaseArms(arms, mesh);
}

Extension.getBaseMovePlane = function(dir, mesh) {
    var bbox = new THREE.Box3().setFromObject(mesh);
    if(dir.x < -0.5) return new THREE.Plane(new THREE.Vector3(-1,0,0), bbox.min.x);
    if(dir.z < -0.5) return new THREE.Plane(new THREE.Vector3(0,0,-1), bbox.min.z);
    if(dir.y < -0.5) return new THREE.Plane(new THREE.Vector3(0,-1,0), bbox.min.y);
    if(dir.x > 0.5) return new THREE.Plane(new THREE.Vector3(1,0,0), -bbox.max.x);
    if(dir.z > 0.5) return new THREE.Plane(new THREE.Vector3(0,0,1), -bbox.max.z);
    if(dir.y > 0.5) return new THREE.Plane(new THREE.Vector3(0,1,0), -bbox.max.y);
}