var Extension = Extension || {}

Extension.clippingLow = new THREE.Vector3(-100,-100,-100);
Extension.clippingHigh = new THREE.Vector3(100,100,100);

Extension.setClippingUniforms = function() {
    Extension.UNIFORMS.setClipping(Extension.clippingLow, Extension.clippingHigh);
}

Extension.resetClipping = function() {
    Extension.clippingLow = new THREE.Vector3(Extension.geoDefinition.From.X, Extension.geoDefinition.From.Y, Extension.geoDefinition.From.Z);
    Extension.clippingHigh = new THREE.Vector3(Extension.geoDefinition.To.X, Extension.geoDefinition.To.Y, Extension.geoDefinition.To.Z);
}


Extension.fillings = [];

Extension.calculateFillings = function(obj, bb , mathPlane) {
    // if(obj.revitModel.ElementType === "Other") {
    //     return;
    // }

    var pointsOfIntersection = new THREE.Geometry();

    var a = new THREE.Vector3(), b = new THREE.Vector3(), c = new THREE.Vector3();
    var planePointA = new THREE.Vector3(), planePointB = new THREE.Vector3(), planePointC = new THREE.Vector3();
    var lineAB = new THREE.Line3(), lineBC = new THREE.Line3(), lineCA = new THREE.Line3();

    var pointOfIntersection = new THREE.Vector3();

    function setPointOfIntersection(line, plane) {
        pointOfIntersection = plane.intersectLine(line);
        
        if (pointOfIntersection && (bb.containsPoint(pointOfIntersection) 
            || obj.revitModel.IsRectangularSurface || obj.revitModel.ElementType === "Wall" )) {// && bb.containsPoint(pointOfIntersection)
          pointsOfIntersection.vertices.push(pointOfIntersection.clone());
        };
    }

    function getPosition(positions, id) {
        return new THREE.Vector3(positions[3*id], positions[3*id + 1], positions[3*id + 2]);
    }

    var positions = obj.geometry.getAttribute('position').array;
    var indices = obj.geometry.getIndex().array;
    for (var faceIt = 0; faceIt < indices.length; faceIt = faceIt + 3) {
        var v1 = indices[faceIt];
        var v2 = indices[faceIt + 1];
        var v3 = indices[faceIt + 2];

        var p1 = getPosition(positions, v1);
        var p2 = getPosition(positions, v2);
        var p3 = getPosition(positions, v3);

        obj.localToWorld(a.copy(p1));
        obj.localToWorld(b.copy(p2));
        obj.localToWorld(c.copy(p3));

        lineAB = new THREE.Line3(a, b);
        lineBC = new THREE.Line3(b, c);
        lineCA = new THREE.Line3(c, a);
        setPointOfIntersection(lineAB, mathPlane);
        setPointOfIntersection(lineBC, mathPlane);
        setPointOfIntersection(lineCA, mathPlane);
    }

    function calculateArea(a, b, c) {
        var al = a.distanceTo(b);
        var bl = b.distanceTo(c);
        var cl = c.distanceTo(a);

        var s = (al + bl + cl) / 2;
        var area = Math.sqrt(s*(s - al)*(s - bl)*(s - cl));

        return area;
    }

    for (let i = 1; i < pointsOfIntersection.vertices.length - 1; i++) {
        var face = new THREE.Face3(0,i,i+1);
        // var area = calculateArea(pointsOfIntersection.vertices[0], pointsOfIntersection.vertices[i], pointsOfIntersection.vertices[i+1]);
        // if(area < 1) {
        //     pointsOfIntersection.faces.push(face);
        // }
        pointsOfIntersection.faces.push(face);
    }
    if(pointsOfIntersection.vertices.length > 0) {
        var material = Extension.MATERIAL.createSheet(0xff0000, 0.5, true, true);
        //material.fragmentShader = Extension.SHADER.fragmentClippingDiffuse;
        //var material = new THREE.MeshStandardMaterial({ color: 0xff0000, side: THREE.DoubleSide });
        pointsOfIntersection.computeFaceNormals();
        var mesh = new THREE.Mesh(pointsOfIntersection, material);
        Extension.fillings.push(mesh);
    }
}

Extension.refreshClipping = function() {
    Extension.setClippingUniforms();

    for (var i = 0; i < Extension.fillings.length; i++) {
        var filling = Extension.fillings[i];
        scene.remove(filling);
    }

    Extension.fillings = [];
    var bb = new THREE.Box3(Extension.clippingLow.clone().sub(new THREE.Vector3(3.5,3.5,3.5)), Extension.clippingHigh.clone().add(new THREE.Vector3(3.5,3.5,3.5)));
    for (var i = 0; i < Extension.meshes.length; i++) {
        var mesh = Extension.meshes[i];
        if(mesh.boundingBox !== undefined && mesh.boundingBox.intersectsBox(bb)) {

            Extension.calculateFillings(mesh, bb, new THREE.Plane(new THREE.Vector3(0, 1, 0), -Extension.clippingLow.y));
            Extension.calculateFillings(mesh, bb, new THREE.Plane(new THREE.Vector3(0, -1, 0), Extension.clippingHigh.y));
            Extension.calculateFillings(mesh, bb, new THREE.Plane(new THREE.Vector3(1, 0, 0), -Extension.clippingLow.x));
            Extension.calculateFillings(mesh, bb, new THREE.Plane(new THREE.Vector3(-1, 0, 0), Extension.clippingHigh.x));
            Extension.calculateFillings(mesh, bb, new THREE.Plane(new THREE.Vector3(0, 0, 1), -Extension.clippingLow.z));
            Extension.calculateFillings(mesh, bb, new THREE.Plane(new THREE.Vector3(0, 0, -1), Extension.clippingHigh.z));
        }
    }

    for (var i = 0; i < Extension.fillings.length; i++) {
        var filling = Extension.fillings[i];
        scene.add(filling);
    }
}
Extension.clipPlanes = function(geoDefinition) {
    Extension.geoDefinition = geoDefinition;
    Extension.resetClipping();
    Extension.refreshClipping();
    // var planes = [];
    // planes.push(new THREE.Plane(new THREE.Vector3(1, 0, 0), -geoDefinition.From.X));
    // planes.push(new THREE.Plane(new THREE.Vector3(-1, 0, 0), geoDefinition.To.X));

    // planes.push(new THREE.Plane(new THREE.Vector3(0, 1, 0), -geoDefinition.From.Y));
    // planes.push(new THREE.Plane(new THREE.Vector3(0, -1, 0), geoDefinition.To.Y));

    // planes.push(new THREE.Plane(new THREE.Vector3(0, 0, 1), -geoDefinition.From.Z));
    // planes.push(new THREE.Plane(new THREE.Vector3(0, 0, -1), geoDefinition.To.Z));

    // renderer.clippingPlanes = planes;

    var bb = new THREE.Box3(new THREE.Vector3(geoDefinition.From.X, geoDefinition.From.Y, geoDefinition.From.Z), new THREE.Vector3(geoDefinition.To.X, geoDefinition.To.Y, geoDefinition.To.Z));
    Extension.externalBoundingBox.push(bb);
}

Extension.setMinXPlane = function(minX) {
    Extension.clippingLow.x = minX;
    Extension.refreshClipping();
    render();
}

Extension.setMaxXPlane = function(maxX) {
    Extension.clippingHigh.x = maxX;
    Extension.refreshClipping();
    render();
}

Extension.setMinYPlane = function(minY) {
    Extension.clippingLow.y = minY;
    Extension.refreshClipping();
    render();
}

Extension.setMaxYPlane = function(maxY) {
    Extension.clippingHigh.y = maxY;
    Extension.refreshClipping();
    render();
}

Extension.setMinZPlane = function(minZ) {
    Extension.clippingLow.z = minZ;
    Extension.refreshClipping();
    render();
}

Extension.setMaxZPlane = function(maxZ) {
    Extension.clippingHigh.z = maxZ;
    Extension.refreshClipping();
    render();
}

Extension.removeAdditionalClipPlanes = function() {
    Extension.resetClipping();
    Extension.refreshClipping();
    render();
}