var Extension = Extension || {};

Extension.currentConnectorCrossSectionsConfig = null;

Extension.loadConnectorVariantConfig = function (config, groupId) {
    Extension.currentConnectorCrossSectionsConfig = config;
    if (groupId === null) {
        for (var i = 0; i < config.armGroups.length; i++) {
            for (var j = 0; j < config.armGroups[i].arms.length; j++) {
                var arm = config.armGroups[i].arms[j];
                for (var k = 0; k < arm.crossSections.length; k++) {
                    var cs = arm.crossSections[k];
                    cs.activeCrossSection = [{
                        id: cs.crossSectionId,
                        text: cs.crossSectionName
                    }];
                }
            }
        }
    } else {
        for (var j = 0; j < config.armGroups[groupId - 1].arms.length; j++) {
            var arm = config.armGroups[groupId - 1].arms[j];
            for (var k = 0; k < arm.crossSections.length; k++) {
                var cs = arm.crossSections[k];
                cs.activeCrossSection = [{
                    id: cs.crossSectionId,
                    text: cs.crossSectionName
                }];
            }
        }
    }

}

Extension.getConnectorCrossSectionsConfig = function () {
    return Extension.createEmptyConfiguration();
}

Extension.createEmptyConfiguration = function () {
    var config = {
        _id: Extension.stpVariantId,
        armGroups: [],
        armData: [],
        connectorId: Extension.stpId
    };

    config.armGroups.push({
        _id: 1,
        armGroupName: 'Default',
        arms: []
    })

    for (var j = 0; j < Extension.arms.length; j++) {
        var armDef = Extension.arms[j];
        config.armGroups[0].arms.push({
            armId: armDef.id,
            name: "Arm " + armDef.id,
            crossSections: []
        });
        config.armData.push({
            armId: armDef.id,
            borderBeam: false,
            releaseId: '',
        });
    }

    return config;
}

Extension.createEmptyArmGroup = function (groupName) {
    var armGroup = {
        _id: Extension.currentConnectorCrossSectionsConfig.armGroups.length + 1,
        armGroupName: groupName,
        arms: []
    }

    for (var j = 0; j < Extension.arms.length; j++) {
        var armDef = Extension.arms[j];
        armGroup.arms.push({
            armId: armDef.id,
            name: "Arm " + armDef.id,
            crossSections: []
        });
    }

    Extension.currentConnectorCrossSectionsConfig.armGroups.push(armGroup)

}

Extension.removeArmGroup = function (armGroup) {
    if (armGroup._id === 1) {
        return;
    }
    var newArmGroups = [];
    for (var j = 0; j < Extension.currentConnectorCrossSectionsConfig.armGroups.length; j++) {
        var curArmGroup = Extension.currentConnectorCrossSectionsConfig.armGroups[j];
        if (curArmGroup._id !== armGroup._id) {
            var newArmGroup = {
                _id: newArmGroups.length + 1,
                armGroupName: curArmGroup.armGroupName,
                arms: curArmGroup.arms,
            }
            newArmGroups.push(newArmGroup);
        }
    }
    Extension.currentConnectorCrossSectionsConfig.armGroups = newArmGroups;
}

Extension.findArmByNum = function (num, groupId) {
    for (var j = 0; j < Extension.currentConnectorCrossSectionsConfig.armGroups[groupId - 1].arms.length; j++) {
        var arm = Extension.currentConnectorCrossSectionsConfig.armGroups[groupId - 1].arms[j];
        if (arm.armId === num) {
            return arm;
        }
    }

    return null;
}

Extension.createEmptyConnectorCrossSection = function (arm, groupId) {
    var armDef = Extension.findArmByNum(arm.armId, groupId);
    armDef.crossSections.push({
        crossSectionId: null,
        allow0rotation: true,
        allow90rotation: false,
        allow180rotation: false,
        allow270rotation: false,
        rotationAngle: 0,
        isShown: false
    });
}

Extension.changeCrossSectionIsShown = function (crossSection, arm) {
    crossSection.isShown = true;
    for (var j = 0; j < arm.crossSections.length; j++) {
        var cs = arm.crossSections[j];
        if (cs !== crossSection) cs.isShown = false;
    }
}

Extension.createEmptyArmData = function (id) {
    var newArmData = {
        armId: id,
        borderBeam: false,
        releaseId: ''
    };
    this.currentConnectorCrossSectionsConfig.armData.push(newArmData);
}

Extension.removeArmData = function (id) {
    var newArmData = [];
    for (var j = 0; j < Extension.currentConnectorCrossSectionsConfig.armData.length; j++) {
        var itArmData = Extension.currentConnectorCrossSectionsConfig.armData[j];
        if (id !== itArmData.armId) {
            var newData = {
                armId: newArmData.length + 1,
                borderBeam: itArmData.borderBeam,
                releaseId: itArmData.releaseId,
            }
            newArmData.push(newData);
        }
    }
    Extension.currentConnectorCrossSectionsConfig.armData = newArmData;
}
