var Extension = Extension || {};
Extension.modelCache = [];

Extension.clearConnectorCrossSectionVisualisation = function() {
    for (var prop in Extension.modelCache) {
        if (Extension.modelCache.hasOwnProperty(prop)) {
            if(Extension.modelCache[prop] !== undefined) scene.remove(Extension.modelCache[prop]);
            Extension.modelCache[prop] = undefined;
        }
    }
}

Extension.showConnectorCrossSectionVisualisation = function(groupId) {
    for (var j = 0; j < Extension.currentConnectorCrossSectionsConfig.armGroups[groupId - 1].arms.length; j++) {
        var arm = Extension.currentConnectorCrossSectionsConfig.armGroups[groupId - 1].arms[j];
        for (var k = 0; k < arm.crossSections.length; k++) {
            var cs = arm.crossSections[k];
            cs.activeCrossSection = [{ id : cs.crossSectionId, text : cs.crossSectionName }];
            // if(cs.isShown) {
            //     Extension.showCrossSectionForArm(cs, arm);
            // }
        }
    }

    Extension.executeLightConstraints(groupId);
}

Extension.removeCrossSectionForArm = function(arm, crossSection, groupId) {
    //here should be logic change: if there are any cross section for this arm are left probably one of them should be shown
    if(Extension.modelCache[arm.armId] !== undefined) scene.remove(Extension.modelCache[arm.armId]);
    Extension.modelCache[arm.armId] = undefined;
    render();
    if(arm.crossSections.length > 0) {
        ExtensionInterface.changeShownCrossSection(arm, arm.crossSections[0], groupId);
    }

}

Extension.executeLightConstraints = function(groupId) {
    new Extension.LightConstraintsExecutor().execute(
        Extension.prepareFullConnectorConfiguration(),
        Extension.currentConnectorCrossSectionsConfig.armGroups[groupId - 1],
        r => Extension.showConstraintsVisualisation(r, groupId)
    );
}

Extension.showConstraintsVisualisation = function(constraints, groupId) {
    for (var j = 0; j < Extension.currentConnectorCrossSectionsConfig.armGroups[groupId - 1].arms.length; j++) {
        var arm = Extension.currentConnectorCrossSectionsConfig.armGroups[groupId - 1].arms[j];
        for (var k = 0; k < arm.crossSections.length; k++) {
            var cs = arm.crossSections[k];
            if(cs.isShown) {
                var constraint = constraints.MemberConstraints[j+1];
                if(constraint === null && constraint === undefined) {
                    if(Extension.modelCache[arm.armId] !== undefined) scene.remove(Extension.modelCache[arm.armId]);
                    Extension.modelCache[arm.armId] = undefined;
                } else {
                    Extension.showCrossSectionForArm(cs, arm, constraint);
                }
            }
        }
    }


    if(constraints.NodeConstraints[1].Orientation !== null){
        var transform = VIEW3D.utils.getMatrix4FromBigBase(constraints.NodeConstraints[1].Orientation);
        //Extension.applyConnectorMeshMatrix(transform);
        Extension.connector3dModel.setTransform(transform);
    }

    if(constraints.NodeConstraints[1].Orientation_d !== null && Extension.connectorDouble3dModel !== null){
        var transform = VIEW3D.utils.getMatrix4FromBigBase(constraints.NodeConstraints[1].Orientation_d);
        //Extension.applyConnectorMeshMatrix(transform);
        Extension.connectorDouble3dModel.setTransform(transform);
    }
}

Extension.prepareCrossSectionInstancedProfileMesh = function(mesh, transform, length) {
    mesh.boundingBox = new THREE.Box3().setFromObject(mesh);
    var ca = mesh.geometry.attributes.color.array;
    var color = new THREE.Vector3(ca[0],ca[1],ca[2]);

    var instProfile = new VIEW3D.InstancedProfile(mesh);
    instProfile.addBar(transform, length, color, {});

    var instProfile2 = new VIEW3D.InstancedProfile(mesh, true);
    instProfile2.addBar(transform, length, color, {});

    var profile = new THREE.Group();
    profile.add( instProfile );
    profile.add( instProfile2 );

    return profile;
}

Extension.showCrossSectionForArm = function(crossSection, arm, constraint) {
    Extension.getProtoMesh(crossSection.crossSectionId).then(mesh => {
        var transform = VIEW3D.utils.getMatrix4FromBigBase(constraint.Orientation);
        var profile = Extension.prepareCrossSectionInstancedProfileMesh(mesh, transform, constraint.Length);

        if(Extension.modelCache[arm.armId] !== undefined) scene.remove(Extension.modelCache[arm.armId]);
        scene.add(profile);
        Extension.modelCache[arm.armId] = profile;
        render();
    });
}

// Extension.showCrossSectionForArm = function(crossSection, arm) {
//     var armDef = Extension.arms[arm.armId - 1];
//     Extension.getProtoMesh(crossSection.crossSectionId).then(mesh => {
//         mesh.boundingBox = new THREE.Box3().setFromObject(mesh);
        
//         var transform = Extension.positionCrossSection(mesh, armDef, crossSection);
//         var ca = mesh.geometry.attributes.color.array;
//         var color = new THREE.Vector3(ca[0],ca[1],ca[2]);

//         var instProfile = new VIEW3D.InstancedProfile(mesh);
//         instProfile.addBar(transform, 0.2, color, {});

//         var instProfile2 = new VIEW3D.InstancedProfile(mesh, true);
//         instProfile2.addBar(transform, 0.2, color, {});

//         var profile = new THREE.Group();
//         profile.add( instProfile );
//         profile.add( instProfile2 );

//         if(Extension.modelCache[arm.armId] !== undefined) scene.remove(Extension.modelCache[arm.armId]);
//         scene.add(profile);
//         Extension.modelCache[arm.armId] = profile;
//         render();
//     });
// }

Extension.positionCrossSection = function(mesh, armDef, crossSection) {
    var m = new THREE.Matrix4();

    //also translate cross section to intersection point
    var center = Extension.armsIntersection;
    if(armDef.border) {
        center = Extension.getBorderPosition(armDef,center);
    }
    m.multiply(new THREE.Matrix4().makeTranslation(center.x, center.y, center.z));

    if(armDef.dir.x > 0.5) {
        //no rotation needed
    } else if(armDef.dir.x < -0.5) {
        m.multiply(new THREE.Matrix4().makeRotationZ(3.14));//rotate 180 degrees around z axis
    } else if(armDef.dir.y > 0.5) {
        m.multiply(new THREE.Matrix4().makeRotationZ(3.14/2));//rotate 90 degrees around z axis
    } else if(armDef.dir.y < -0.5) {
        m.multiply(new THREE.Matrix4().makeRotationZ(-3.14/2));//rotate -90 degrees around z axis
    } else if(armDef.dir.z > 0.5) {
        m.multiply(new THREE.Matrix4().makeRotationY(-3.14/2));//rotate -90 degrees around y axis
    } else if(armDef.dir.z < -0.5) {
        m.multiply(new THREE.Matrix4().makeRotationY(3.14/2));//rotate 90 degrees around y axis
    }

    if(crossSection.allow90rotation) m.multiply(new THREE.Matrix4().makeRotationX(3.14/2)); //rotate 90 degrees around x axis
    if(crossSection.allow180rotation) m.multiply(new THREE.Matrix4().makeRotationX(3.14)); //rotate 180 degrees around x axis
    if(crossSection.allow270rotation) m.multiply(new THREE.Matrix4().makeRotationX(3.14*1.5)); //rotate 270 degrees around x axis

    //mesh.geometry.applyMatrix(m);
    return m;
}

// Extension.positionCrossSection = function(mesh, armDef, crossSection) {
//     debugger;
//     var m = new THREE.Matrix4();
//     m.makeRotationX(3.14);
//     m.makeRotationZ(3.14);
//     m.makeTranslation(0,1,0);

//     if(crossSection.allow90rotation) mesh.geometry.rotateX(3.14/2); //rotate 90 degrees around x axis
//     if(crossSection.allow180rotation) mesh.geometry.rotateX(3.14); //rotate 180 degrees around x axis
//     if(crossSection.allow270rotation) mesh.geometry.rotateX(3.14*1.5); //rotate 270 degrees around x axis

//     if(armDef.dir.x > 0.5) {
//         //no rotation needed
//     } else if(armDef.dir.x < -0.5) {
//         mesh.geometry.rotateZ(3.14);//rotate 180 degrees around z axis
//     } else if(armDef.dir.y > 0.5) {
//         mesh.geometry.rotateZ(3.14/2);//rotate 90 degrees around z axis
//     } else if(armDef.dir.y < -0.5) {
//         mesh.geometry.rotateZ(-3.14/2);//rotate -90 degrees around z axis
//     } else if(armDef.dir.z > 0.5) {
//         mesh.geometry.rotateY(-3.14/2);//rotate -90 degrees around y axis
//     } else if(armDef.dir.z < -0.5) {
//         mesh.geometry.rotateY(3.14/2);//rotate 90 degrees around y axis
//     }

//     //also translate cross section to intersection point
//     var center = Extension.armsIntersection;
//     if(armDef.border) {
//         center = Extension.getBorderPosition(armDef,center);
//     }
    
//     mesh.geometry.translate(center.x, center.y, center.z);
// }

Extension.getBorderPosition = function(armDef, center) {
    var mesh = CORE.constructionView3D;
    var bb = new THREE.Box3().setFromObject(mesh);

    if(armDef.dir.x > 0.5) {
        return new THREE.Vector3(bb.min.x, center.y, center.z);
    } else if(armDef.dir.x < -0.5) {
        return new THREE.Vector3(bb.max.x, center.y, center.z);
    } else if(armDef.dir.y > 0.5) {
        return new THREE.Vector3(center.x, bb.min.y, center.z);
    } else if(armDef.dir.y < -0.5) {
        return new THREE.Vector3(center.x, bb.max.y, center.z);
    } else if(armDef.dir.z > 0.5) {
        return new THREE.Vector3(center.x, center.y, bb.min.z);
    } else if(armDef.dir.z < -0.5) {
        return new THREE.Vector3(center.x, center.y, bb.max.z);
    }
}