var Extension = Extension || {};

Extension.rotateOpenPartToTop = function(mesh, openPartResponse) {
    if(openPartResponse.isTopOpen) return;
    else if(openPartResponse.isBottomOpen) mesh.geometry.rotateX(Math.PI);
    else if(openPartResponse.isLeftOpen) mesh.geometry.rotateX(Math.PI/2);
    else if(openPartResponse.isRightOpen) mesh.geometry.rotateX(-Math.PI/2);
}

Extension.findOpenPart = function(mesh, bbox) {
    var cent = bbox.getCenter(); var size = bbox.getSize();
    var verticies = mesh.geometry.attributes.position.array; var indicies = mesh.geometry.index.array;
    var bbTop = new THREE.Box3(new THREE.Vector3(cent.x - size.x, cent.y - size.y/5 , cent.z + size.z*0.45), new THREE.Vector3(cent.x + size.x, cent.y + size.y/5, cent.z + size.z));
    var bbBottom = new THREE.Box3(new THREE.Vector3(cent.x - size.x, cent.y - size.y/5 , cent.z - size.z), new THREE.Vector3(cent.x + size.x, cent.y + size.y/5, cent.z - size.z*0.45));
    var bbLeft = new THREE.Box3(new THREE.Vector3(cent.x - size.x, cent.y + size.y*0.45 , cent.z - size.z/5), new THREE.Vector3(cent.x + size.x, cent.y + size.y, cent.z + size.z/5));
    var bbRight = new THREE.Box3(new THREE.Vector3(cent.x - size.x, cent.y - size.y , cent.z - size.z/5), new THREE.Vector3(cent.x + size.x, cent.y - size.y*0.45, cent.z + size.z/5));
    var response = {};
    response.isTopOpen = !Extension.areVerticiesInBox(verticies, indicies, bbTop);
    response.isBottomOpen = !Extension.areVerticiesInBox(verticies, indicies, bbBottom);
    response.isLeftOpen = !Extension.areVerticiesInBox(verticies, indicies, bbLeft);
    response.isRightOpen = !Extension.areVerticiesInBox(verticies, indicies, bbRight);
    response.isOpen = response.isTopOpen || response.isBottomOpen || response.isLeftOpen || response.isRightOpen;
    
    return response;
}

Extension.areVerticiesInBox = function(verticies, indicies, bbox) {
    for (var k = 0; k < indicies.length; k+=3) {
        var v1 = Extension.getPoint(indicies[k], verticies);
        var v2 = Extension.getPoint(indicies[k+1], verticies);
        var v3 = Extension.getPoint(indicies[k+2], verticies);
        var t = new THREE.Triangle(v1,v2,v3);
        if(Extension.boxIntersectsTriangle(bbox, t)) return true;//bbox.intersectsTriangle(t) <- not in this THREE.js version
    }
    // for (var j = 0; j < verticies.length; j+=3) {
    //     var v = new THREE.Vector3(verticies[j], verticies[j + 1], verticies[j + 2]);
    //     if(bbox.containsPoint(v)) return true;
    // }
    return false;
}

Extension.prepareFullCrossSectionConfiguration = function(boundingBox, isOpenResponse, isCricle) {
    var crossSectionConfig = { 
        _id : Extension.stpId, 
        config : {
            isCircle : isCricle,
            isOpen : isOpenResponse.isOpen,
            width : Extension.calculateWidth(boundingBox),
            height : Extension.calculateHeight(boundingBox),
            length : Extension.calculateLength(boundingBox)
        }
    };

    return crossSectionConfig;
}

Extension.calculateLength = function(boundingBox) {
    return boundingBox.getSize().x;
}

Extension.calculateHeight = function(boundingBox) {
    return boundingBox.getSize().z;
}

Extension.calculateWidth = function(boundingBox) {
    return boundingBox.getSize().y;
}

Extension.allignToXAxis = function(mesh, direction) {
    if(direction.y > 0.99) mesh.geometry.rotateZ(-1.57);
    else if(direction.z > 0.99) mesh.geometry.rotateY(1.57);
}

Extension.moveToCenter = function(bb, mesh, direction) {
    var center = null;
    if(direction.x > 0.99) center = new THREE.Vector3(bb.min.x, (bb.min.y + bb.max.y)/2,(bb.min.z + bb.max.z)/2);
    else if(direction.y > 0.99) center = new THREE.Vector3((bb.min.x + bb.max.x)/2, bb.min.y, (bb.min.z + bb.max.z)/2);
    else center = new THREE.Vector3((bb.min.x + bb.max.x)/2, (bb.min.y + bb.max.y)/2, bb.min.z);
    mesh.geometry.translate(-center.x, -center.y, -center.z);
    //mesh.position.set(-center.x, -center.y, -center.z);
}

Extension.recogniseDirection = function(boundingBox) {
    var size = boundingBox.getSize();
    if(size.x >= size.y && size.x >= size.z) return new THREE.Vector3(1,0,0);
    if(size.y >= size.x && size.y >= size.z) return new THREE.Vector3(0,1,0);
    return new THREE.Vector3(0,0,1);
}

Extension.recogniseCrossSection = function(mesh, position) {
    var bbox = new THREE.Box3().setFromObject(mesh);
    var direction = Extension.recogniseDirection(bbox);
    Extension.moveToCenter(bbox, mesh, direction);
    Extension.allignToXAxis(mesh, direction);
    bbox = bbox.setFromObject(mesh);

    var isOpenResponse = Extension.findOpenPart(mesh, bbox);
    Extension.rotateOpenPartToTop(mesh, isOpenResponse);
    bbox = bbox.setFromObject(mesh);
    var isCircle = Extension.checkIfIsCircular(mesh, bbox);
    debugger;
    var config = Extension.prepareFullCrossSectionConfiguration(bbox, isOpenResponse, isCircle);
    CORE.webApiManager.postCrossSectionConfig(config, function() {});
}

Extension.checkIfIsCircular = function(mesh, bbox) {
    var radius = Math.min(bbox.getSize().z, bbox.getSize().y) / 2;
    var plane = new THREE.Plane(new THREE.Vector3(1,0,0), 0);
    for (var i = 0; i < mesh.geometry.attributes.position.array.length; i+=3) {
        var ver =Extension.getPoint(i/3, mesh.geometry.attributes.position.array);
        var projVer = plane.projectPoint(ver);
        var len = projVer.length();
        if(radius + 0.0001 < len) return false;
    }
    return true;
}