var Extension = Extension || {};

Extension.compareBoundingBoxes = function(item, group)
{
    for (var k = 0; k < group.length; k++) {
        var groupItem = group[k];
        if(item.boundingBox.intersectsBox(groupItem.boundingBox)) return true;
    }
    return false;
}

Extension.findGroups = function(items) {
    var groups = [];
    for (var j = 0; j < items.length; j++) {
        var item = items[j];
        var isInGroup = false;
        for (var k = 0; k < groups.length; k++) {
            var group = groups[k];
            if(Extension.compareBoundingBoxes(item, group)) {
                group.push(item);
                isInGroup = true;
                //perhaps break here is in order
            }
        }
        if(isInGroup === false) {
            groups.push([item]);
        }
    }
    return groups;
}

Extension.mergeGroups = function(groups) {
    var results = [];
    for (var k = 0; k < groups.length; k++) {
        var group = groups[k];
        var part = []; var triangles = [];
        for (var j = 0; j < group.length; j++) {
            var element = group[j];
            part = part.concat(element.part);
            triangles = triangles.concat(element.triangles);
        }
        var partMash = Extension.createTrianglesMesh(triangles, new THREE.Vector3(1,1,1));//color is not important here
        var bb = partMash.boundingBox;
        results.push({ part : part, triangles : triangles, boundingBox : bb});
    }
    return results;
}

Extension.mergeInterconnectedParts = function(items) {
    var groups = Extension.findGroups(items);
    var results = Extension.mergeGroups(groups);

    return results;
}

Extension.findTopPart = function(item1, item2) {
    return item1.boundingBox.min.z > item2.boundingBox.min.z ? item1 : item2;
}

Extension.findBottomPart = function(item1, item2) {
    return item1.boundingBox.min.z > item2.boundingBox.min.z ? item2 : item1;
}

Extension.calculateHorizontalArm = function(bbox) {
    var size = bbox.getSize();
    var alongX = size.x >= size.y;
    var direction = new THREE.Vector3(alongX ? 1 : 0, alongX ? 0 : 1, 0);
    var length = alongX ? size.x : size.y;
    return { direction : direction, length : length };
}

Extension.createDoubleConnectorArms = function(mergedItems) {
    var topPart = Extension.findTopPart(mergedItems[0], mergedItems[1]);
    var bottomPart = Extension.findBottomPart(mergedItems[0], mergedItems[1]);
    
    
    var center = topPart.boundingBox.getCenter();
    Extension.modelConfigBoundingBoxMin = topPart.boundingBox.min;
    Extension.modelConfigBoundingBoxMax = topPart.boundingBox.max;
    Extension.isDouble = true;
    var zAxis = (topPart.boundingBox.min.z + bottomPart.boundingBox.max.z) / 2;
    Extension.armsIntersection = new THREE.Vector3(center.x, center.y, zAxis);
    Extension.pointMesh =Extension.createPointMesh(Extension.armsIntersection);
    scene.add(Extension.pointMesh);
    var horArm = Extension.calculateHorizontalArm(topPart.boundingBox);
    var arm1 = { direction : horArm.direction, config : {}, armLength : horArm.length };
    var arm2 = { direction : new THREE.Vector3(0,0,1), config : {}, armLength : topPart.boundingBox.max.z - bottomPart.boundingBox.min.z};
    Extension.newArmLength = topPart.boundingBox.max.x - topPart.boundingBox.min.x;
    var armDef1 = Extension.loadArmDefiniton(arm1, 0);
    var armDef2 = Extension.loadArmDefiniton(arm2, 1);
    ExtensionInterface.setDistances(armDef1, Extension.calculateArmSize(armDef1, Extension.connectorMeshes));
    ExtensionInterface.setDistances(armDef2, Extension.calculateArmSize(armDef2, Extension.connectorMeshes));
    //armDef1.config.distances = Extension.calculateArmSize(armDef1, Extension.connectorMeshes);
    //armDef2.config.distances = Extension.calculateArmSize(armDef2, Extension.connectorMeshes);
    
    scene.add(armDef1.lineGroupMesh);
    Extension.arms.push(armDef1);

    scene.add(armDef2.lineGroupMesh);
    Extension.arms.push(armDef2);

    return [topPart, bottomPart];
}
Extension.testColors = [
    new THREE.Vector3(1,0,0),
    new THREE.Vector3(0,1,0),
    new THREE.Vector3(0,0,1),
    new THREE.Vector3(0,1,1),
    new THREE.Vector3(1,1,0),
    new THREE.Vector3(1,0,1),
];
Extension.recogniseDoubleConnector = function(mesh, position, index) {
    var items = [];
    var parts = Extension.findSeperateParts(position, index);
    for (var j = 0; j < parts.length; j++) {
        var part = parts[j];
        var triangles = Extension.findTrianglesOnPlane(Extension.preparePartLookUp(position, part), index, position);
        var partMash = Extension.createTrianglesMesh(triangles, Extension.testColors[j % Extension.testColors.length]);//color is not important here
        //CORE.constructionView3D.children.push(partMash);
        var bb = partMash.boundingBox;
        items.push({ part : part, triangles : triangles, boundingBox : bb});
    }
    var mergedItems = Extension.mergeInterconnectedParts(items);
    if(mergedItems.length !== 2) {
        return { isDoubleConnector : false };
    } else {
        var topBottomParts = Extension.createDoubleConnectorArms(mergedItems);
        return { isDoubleConnector : true, parts : topBottomParts };
    }
}

Extension.preparePartLookUp = function(position, part){
    var visitedIndicies = Extension.createVisitedArray(position.length/3);
    for (var j = 0; j < part.length; j++) {
        var i = part[j];
        visitedIndicies[i] = true;
    }
    return visitedIndicies;
}

Extension.divideDoubleConnectorModel = function(parts, color) {
    var results = [];
    for (var i = 0; i < parts.length; i++) {
        results.push(Extension.createGeometryFromModelPart(parts[i], color));
    }
    return results;
}

Extension.createGeometryFromModelPart = function(part, color) {
    function addVertexToPart(vertex, geo) { 
        geo.position.push(vertex.x); geo.position.push(vertex.y); geo.position.push(vertex.z); 
        geo.index.push(geo.index.length); 
    }
    
    var geo = { position : [], index : [], color : color };
    for (var i = 0; i < part.triangles.length; i++) {
        var triangle = part.triangles[i];
        addVertexToPart(triangle.a, geo);
        addVertexToPart(triangle.b, geo);
        addVertexToPart(triangle.c, geo);
    }

    return geo;
}

