var Extension = Extension || {};
Extension.planeBoundingBox = null;
Extension.plane = null;
Extension.getPoint = function(vIndex, position) {
    return new THREE.Vector3(position[3*vIndex], position[3*vIndex + 1], position[3*vIndex + 2]);
}

Extension.isPointInTriangles = function(triangles, point) {
    if(Extension.planeBoundingBox !== null)
    {
        var prPoint = Extension.plane.projectPoint(point);
        var dist = Extension.planeBoundingBox.distanceToPoint(prPoint);
        return dist < 0.00001;
    }

    for (var i = 0; i < triangles.length; i++) {
        var triangle = triangles[i];
        if(triangle.containsPoint(point)) return true;
    }
    return false;
}

Extension.findTrianglesOnPlane = function(isIndexOnPlane, index, position) {
    var triangles= [];
    for (var i = 0; i < index.length; i+=3) {
        if(isIndexOnPlane[index[i]] && isIndexOnPlane[index[i + 1]] && isIndexOnPlane[index[i + 2]]) {
            triangles.push(new THREE.Triangle(
                Extension.getPoint(index[i], position),
                Extension.getPoint(index[i + 1], position),
                Extension.getPoint(index[i + 2], position)
            ));
        }
    }

    return triangles;
}

Extension.checkVertex = function(ind, currentInd, planeTriangles, position, visitedVerts) {
    return ind !== currentInd &&
        visitedVerts[ind] !== true &&//not yet visited
        Extension.isPointInTriangles(planeTriangles, Extension.getPoint(ind, position));
}

Extension.visitVertex = function(vIndex, visitedVerts, planeTriangles, position, index) {
    for (var i = 0; i < index.length; i+=3) {
        if(Extension.checkIfSamePoint(index[i], vIndex, position) || 
            Extension.checkIfSamePoint(index[i + 1], vIndex, position) || 
            Extension.checkIfSamePoint(index[i + 2], vIndex, position)) {
            if(Extension.checkVertex(index[i], vIndex, planeTriangles, position, visitedVerts)) {
                visitedVerts[index[i]] = true;
                Extension.visitVertex(index[i], visitedVerts, planeTriangles, position, index);
            }
            if(Extension.checkVertex(index[i+1], vIndex, planeTriangles, position, visitedVerts)) {
                visitedVerts[index[i + 1]] = true;
                Extension.visitVertex(index[i+1], visitedVerts, planeTriangles, position, index);
            }
            if(Extension.checkVertex(index[i+2], vIndex, planeTriangles, position, visitedVerts)) {
                visitedVerts[index[i + 2]] = true;
                Extension.visitVertex(index[i+2], visitedVerts, planeTriangles, position, index);
            }
        }
    }
}

Extension.checkIfSamePoint = function(vi1, vi2, position) {
    var v1 = Extension.getPoint(vi1, position);
    var v2 = Extension.getPoint(vi2, position);
    return v1.distanceTo( v2 ) < 0.00001;
}

Extension.processPlaneIntersection = function(mesh, position, index, plane, color) {
    var ind = 0;
    var indexesOnPlane = [];
    var isIndexOnPlane = [];
    for (var i = 0; i < position.length; i+=3) {
        var pos = new THREE.Vector3(position[i], position[i+1], position[i+2]);
        var  dist = plane.distanceToPoint(pos);
        var isOnPlane = Math.abs(dist) <0.00001;
        if(isOnPlane) {
            indexesOnPlane.push(ind);
        }
        isIndexOnPlane.push(isOnPlane);
        ind+=1;
    }

    var endConstant = Extension.calculateEndConstantForPlaneIntersection(mesh, plane);
    var triangles = Extension.findTrianglesOfActualArm(mesh, position, index, plane, plane.constant, endConstant);
    if(triangles.length === 0) triangles = Extension.findTrianglesOnPlane(isIndexOnPlane, index, position);
    //var triangles = Extension.findTrianglesOnPlane(isIndexOnPlane, index, position);

    var visited = isIndexOnPlane.slice(0);
    for (var i = 0; i < indexesOnPlane.length; i++) {
        var vInd = indexesOnPlane[i];
        Extension.visitVertex(vInd, visited, triangles, position, index);
    }

    return Extension.findTrianglesOnPlane(visited, index, position);
}

Extension.findSeperateParts = function(position, index) {
    var parts = [];
    var ind = 0;
    var visitedIndicies = Extension.createVisitedArray(position.length/3);
    for (var i = 0; i < position.length; i+=3) {
        var result = [];
        Extension.findRecursiveVertexPart(ind, position, index, visitedIndicies, result);
        if(result.length > 0) {
            parts.push(result);
        }
        ind+=1;
    }
    return parts;
}

Extension.findRecursiveVertexPart = function(vInd, position, index, visited, result) {
    if(visited[vInd] === true) return;
    result.push(vInd);
    visited[vInd] = true;
    for (var i = 0; i < index.length; i+=3) {
        if(Extension.checkIfSamePoint(index[i], vInd, position) || 
            Extension.checkIfSamePoint(index[i + 1], vInd, position) || 
            Extension.checkIfSamePoint(index[i + 2], vInd, position)) {
            if(visited[index[i]] === false) {
                Extension.findRecursiveVertexPart(index[i], position, index, visited, result);
            }
            if(visited[index[i + 1]] === false) {
                Extension.findRecursiveVertexPart(index[i + 1], position, index, visited, result);
            }
            if(visited[index[i + 2]] === false) {
                Extension.findRecursiveVertexPart(index[i + 2], position, index, visited, result);
            }
        }
    }
}

Extension.createVisitedArray = function(len) {
    var visited = [];
    for (var i = 0; i < len; i++) {
        visited.push(false);
    }
    return visited;
}

Extension.calculateEndConstantForPlaneIntersection = function(mesh, plane) {
    var planeNormal = plane.normal;
    if(planeNormal.x > 0.5) {
        return -mesh.boundingBox.min.x;
    } else if (planeNormal.x < -0.5) {
        return mesh.boundingBox.max.x;
    } else if(planeNormal.y > 0.5) {
        return -mesh.boundingBox.min.y;
    } else if (planeNormal.y < -0.5) {
        return mesh.boundingBox.max.y;
    } else if(planeNormal.z > 0.5) {
        return -mesh.boundingBox.min.z;
    } else if (planeNormal.z < -0.5) {
        return mesh.boundingBox.max.z;
    }
}

Extension.manageMaxSize = function(size, maxSize) {
    if(size.x > maxSize.x) maxSize.x = size.x;
    if(size.y > maxSize.y) maxSize.y = size.y;
    if(size.z > maxSize.z) maxSize.z = size.z;
}

Extension.checkSizeInRegardToMaxSize = function(size, maxSize) {
    function isDimensionBigEnough(val, maxVal) {
        return 2*val + 0.0001 >= maxVal;
    }
    return isDimensionBigEnough(size.x, maxSize.x) &&
            isDimensionBigEnough(size.y, maxSize.y) &&
            isDimensionBigEnough(size.z, maxSize.z);
}

Extension.findTrianglesOfActualArm = function(mesh, position, index, originalPlane, startConstant, endConstant) {
    var lastSize = new THREE.Vector3(); var lastTriangles = []; var lastCenter = new THREE.Vector3(); var maxSize = new THREE.Vector3();
    var iteratorVal = (endConstant - startConstant)/20;
    for (let constant = startConstant; constant < endConstant; constant+=iteratorVal) {
        var plane = new THREE.Plane(originalPlane.normal, constant);
        var intersections = Extension.cutMeshWithPlane(mesh, position, index, plane);
        var triangles = Extension.createTrianglesForIntersections(intersections, originalPlane);
        var bbox = new THREE.Box3().setFromPoints(intersections);
        var size = bbox.getSize(); var center = bbox.getCenter();
        //var area = Extension.calculateTrianglesArea(triangles);
        if(size.distanceTo(lastSize) > 0.001 || center.distanceTo(lastCenter) > 0.001 || !Extension.checkSizeInRegardToMaxSize(size, maxSize)) {
            Extension.manageMaxSize(size, maxSize);
            lastSize = size;
            lastCenter = center;
            lastTriangles = triangles;
        } else {
            Extension.planeBoundingBox = bbox; Extension.plane = originalPlane;
            return lastTriangles;
        }
    }
    Extension.planeBoundingBox = null; Extension.plane = null
    return [];
}

Extension.createTrianglesForIntersections = function(intersections, plane) {
    var triangles = [];
    for (var j = 0; j < intersections.length; j++) {
        var vertex = intersections[j];
        var newVertex = plane.projectPoint(vertex);
        intersections[j] = newVertex;
    }

    for (let j = 1; j < intersections.length - 1; j++) {
        triangles.push(new THREE.Triangle(intersections[0], intersections[j], intersections[j+1]));
    }

    return triangles;
}

Extension.calculateTrianglesArea = function(triangles) {
    var area = 0;
    for (var j = 0; j < triangles.length; j++) {
        area += triangles[j].area();
    }
    return area;
}

Extension.isVertexAlreadyAdded = function(vertices, vertex) {
    for (var j = 0; j < vertices.length; j++) {
        var v = vertices[j];
        if(vertex.distanceTo(v) < 0.01) return true;
    }
    return false;
}

Extension.cutMeshWithPlane = function(mesh, position, index, mathPlane) {
    var pointsOfIntersection = new THREE.Geometry();

    var a = new THREE.Vector3(), b = new THREE.Vector3(), c = new THREE.Vector3();
    var lineAB = new THREE.Line3(), lineBC = new THREE.Line3(), lineCA = new THREE.Line3();

    var pointOfIntersection = new THREE.Vector3();

    function setPointOfIntersection(line, plane) {
        pointOfIntersection = plane.intersectLine(line);
        
        if (pointOfIntersection && !Extension.isVertexAlreadyAdded(pointsOfIntersection.vertices, pointOfIntersection)) {
          pointsOfIntersection.vertices.push(pointOfIntersection.clone());
        };
    }

    function getPosition(positions, id) {
        return new THREE.Vector3(positions[3*id], positions[3*id + 1], positions[3*id + 2]);
    }

    for (var faceIt = 0; faceIt < index.length; faceIt = faceIt + 3) {
        var v1 = index[faceIt];
        var v2 = index[faceIt + 1];
        var v3 = index[faceIt + 2];

        var p1 = getPosition(position, v1);
        var p2 = getPosition(position, v2);
        var p3 = getPosition(position, v3);

        mesh.localToWorld(a.copy(p1));
        mesh.localToWorld(b.copy(p2));
        mesh.localToWorld(c.copy(p3));

        lineAB = new THREE.Line3(a, b);
        lineBC = new THREE.Line3(b, c);
        lineCA = new THREE.Line3(c, a);
        setPointOfIntersection(lineAB, mathPlane);
        setPointOfIntersection(lineBC, mathPlane);
        setPointOfIntersection(lineCA, mathPlane);
    }

    return pointsOfIntersection.vertices;
}

Extension.calculateArmStartPoint = function(armMesh, planeNormal) {
    var bb = armMesh.boundingBox;
    var min = bb.min;
    var max = bb.max;
    if(planeNormal.x > 0.5) {
        return new THREE.Vector3(max.x , (max.y + min.y)/2, (max.z + min.z)/2);
    } else if (planeNormal.x < -0.5) {
        return new THREE.Vector3(min.x , (max.y + min.y)/2, (max.z + min.z)/2);
    } else if(planeNormal.y > 0.5) {
        return new THREE.Vector3((max.x + min.x)/2 , max.y, (max.z + min.z)/2);
    } else if (planeNormal.y < -0.5) {
        return new THREE.Vector3((max.x + min.x)/2 , min.y, (max.z + min.z)/2);
    } else if(planeNormal.z > 0.5) {
        return new THREE.Vector3((max.x + min.x)/2 , (max.y + min.y)/2, max.z);
    } else if (planeNormal.z < -0.5) {
        return new THREE.Vector3((max.x + min.x)/2 , (max.y + min.y)/2, min.z);
    }
}