var Extension = Extension || {};

//http://geomalgorithms.com/a07-_distance.html
//https://gist.github.com/cuberoot/b5047c83cf277fee1b82
Extension.calculateDistanceBetweenLines = function (startPt1, endPt1, startPt2, endPt2) {
    var u, v, w, a, b, c, d, e, D, sc, sN, sD, tc, tN, tD;
    EPSILON = 0.000001;
    u = new THREE.Vector3();
    u.subVectors(endPt1, startPt1);
    v = new THREE.Vector3();
    v.subVectors(endPt2, startPt2);
    w = new THREE.Vector3();
    w.subVectors(startPt1, startPt2);
    
    a = u.dot(u);           // always >= 0
    b = u.dot(v);
    c = v.dot(v);           // always >= 0
    d = u.dot(w);
    e = v.dot(w);
    
    D = (a * c) - (b * b);  // always >= 0
    sD = D;                 // default sD = D >= 0
    tD = D;                 // default tD = D >= 0
    
    // compute the line parameters of the two closest points
    if (D < EPSILON) {  
        // the lines are almost parallel
        sN = 0.0;         // force using point P0 on segment S1
        sD = 1.0;         // to prevent possible division by 0.0 later
        tN = e;
        tD = c;
    }
    else 
    {
        // get the closest points on the infinite lines
        sN = (b*e - c*d);
        tN = (a*e - b*d);
        if (sN < 0.0) {     
            // sc < 0 => the s=0 edge is visible
            sN = 0.0;
            tN = e;
            tD = c;
        }
        else if (sN > sD) 
        {  
            // sc > 1  => the s=1 edge is visible
            sN = sD;
            tN = e + b;
            tD = c;
        }
    }
    
    if (tN < 0.0) {            
        // tc < 0 => the t=0 edge is visible
        tN = 0.0;
        // recompute sc for this edge
        if (-d < 0.0) 
        {
            sN = 0.0;
        }
        else if (-d > a) 
        {
            sN = sD;
        }
        else 
        {
            sN = -d;
            sD = a;
        }
    }
    else if (tN > tD) 
    {      
        // tc > 1  => the t=1 edge is visible
        tN = tD;
        // recompute sc for this edge
        if ((-d + b) < 0.0) 
        {
            sN = 0;
        }
        else if ((-d + b) > a) 
        {
            sN = sD;
        }
        else 
        {
            sN = (-d +  b);
            sD = a;
        }
    }
    
    // finally do the division to get sc and tc
    sc = (Math.abs(sN) < EPSILON ? 0.0 : sN / sD);
    tc = (Math.abs(tN) < EPSILON ? 0.0 : tN / tD);

    // get the difference of the two closest points
    var sc_mult_u = new THREE.Vector3();
    sc_mult_u.copy(u);
    sc_mult_u.multiplyScalar(sc);
    var tc_mult_v = new THREE.Vector3();
    tc_mult_v.copy(v);
    tc_mult_v.multiplyScalar(tc);
    var dP = new THREE.Vector3();
    dP.copy(w);
    dP.add(sc_mult_u);
    dP.sub(tc_mult_v);

    return dP.length();   // return the closest distance
}

Extension.intersectionPoint = function(P0, P1, Q0, Q1) {
    var DP = new THREE.Vector3().subVectors(P1, P0);
    var DQ = new THREE.Vector3().subVectors(Q1, Q0);
    var PQ = new THREE.Vector3().subVectors(P0, Q0);

    var a = DP.dot(DP);           // always >= 0
    var b = DP.dot(DQ);
    var c = DQ.dot(DQ);           // always >= 0
    var d = DP.dot(PQ);
    var e = DQ.dot(PQ);

    var DD = a * c- b * b

    if(DD < 0.00001) return { dist: 0 , point: P0.clone().add(P1).divideScalar(2)};

    var tt = (b * e - c * d) / DD
    var uu = (a * e - b * d) / DD

    var Ptt = DP.clone().multiplyScalar(tt).add(P0);
    var Quu = DQ.clone().multiplyScalar(uu).add(Q0);
    var Dist = Quu.clone().sub(Ptt).length();

    return { dist : Dist, point: Ptt};
}

Extension.boxIntersectsTriangle = function (box, triangle) {
    var v0 = new THREE.Vector3();
    var v1 = new THREE.Vector3();
    var v2 = new THREE.Vector3();

    var f0 = new THREE.Vector3();
    var f1 = new THREE.Vector3();
    var f2 = new THREE.Vector3();

    var testAxis = new THREE.Vector3();

    var center = new THREE.Vector3();
    var extents = new THREE.Vector3();

    var triangleNormal = new THREE.Vector3();

    function satForAxes( axes ) {

        var i, j;

        for ( i = 0, j = axes.length - 3; i <= j; i += 3 ) {

            testAxis.fromArray( axes, i );
            var r = extents.x * Math.abs( testAxis.x ) + extents.y * Math.abs( testAxis.y ) + extents.z * Math.abs( testAxis.z );
            var p0 = v0.dot( testAxis );
            var p1 = v1.dot( testAxis );
            var p2 = v2.dot( testAxis );
            if ( Math.max( - Math.max( p0, p1, p2 ), Math.min( p0, p1, p2 ) ) > r ) {
                return false;
            }
        }
        return true;
    }
    if ( box.isEmpty() ) {
        return false;
    }

    box.getCenter( center );
    extents.subVectors( box.max, center );

    v0.subVectors( triangle.a, center );
    v1.subVectors( triangle.b, center );
    v2.subVectors( triangle.c, center );

    // compute edge vectors for triangle
    f0.subVectors( v1, v0 );
    f1.subVectors( v2, v1 );
    f2.subVectors( v0, v2 );

    // test against axes that are given by cross product combinations of the edges of the triangle and the edges of the aabb
    // make an axis testing of each of the 3 sides of the aabb against each of the 3 sides of the triangle = 9 axis of separation
    // axis_ij = u_i x f_j (u0, u1, u2 = face normals of aabb = x,y,z axes vectors since aabb is axis aligned)
    var axes = [
        0, - f0.z, f0.y, 0, - f1.z, f1.y, 0, - f2.z, f2.y,
        f0.z, 0, - f0.x, f1.z, 0, - f1.x, f2.z, 0, - f2.x,
        - f0.y, f0.x, 0, - f1.y, f1.x, 0, - f2.y, f2.x, 0
    ];
    if ( ! satForAxes( axes ) ) {

        return false;

    }

    // test 3 face normals from the aabb
    axes = [ 1, 0, 0, 0, 1, 0, 0, 0, 1 ];
    if ( ! satForAxes( axes ) ) {

        return false;

    }

    // finally testing the face normal of the triangle
    // use already existing triangle edge vectors here
    triangleNormal.crossVectors( f0, f1 );
    axes = [ triangleNormal.x, triangleNormal.y, triangleNormal.z ];
    return satForAxes( axes );

};

Extension.calculateVersorsFromDirection = function(dir) {
    return Extension.calculateVersorsFromPoints(new THREE.Vector3(0,0,0) , dir);
}

Extension.calculateVersorsFromPoints = function(pt1, pt2) {
    var vectX = pt2.clone().sub(pt1);

    var len = vectX.length();
    var lxy = Math.sqrt(vectX.x * vectX.x + vectX.y * vectX.y);
    var ca = 1;
    var sa = 0;
    var cb = lxy / len;
    var sb = -vectX.z / len;
    var cg = 1;
    var sg = 0;

    if (Math.abs(lxy) > 0.00001)
    {
        ca = vectX.x / lxy;
        sa = vectX.y / lxy;
    }

    vectX.normalize();

    var res = {
        origin : pt1.clone(),
        baseX : vectX,
        baseY : new THREE.Vector3(ca * sb * sg - sa * cg, sa * sb * sg + ca * cg, cb * sg),
        baseZ : new THREE.Vector3(ca * sb * cg + sa * sg, sa * sb * cg - ca * sg, cb * cg)
    };

    return res;
}