Extension.UNIFORMS = {
    uniforms: [],
    setOpacity: function(opacity) {
        for (var i = 0; i < Extension.UNIFORMS.uniforms.length; i++) {
            var uniform = Extension.UNIFORMS.uniforms[i];
            uniform.opacity.value = opacity;
        }
    },
    setClipping: function(cl, ch) {
        var uniforms = Extension.UNIFORMS.clipping;
        uniforms.clippingLow.value.copy(new THREE.Vector3(cl.x - 0.01, cl.y - 0.01, cl.z - 0.01));
        uniforms.clippingHigh.value.copy(new THREE.Vector3(ch.x + 0.01, ch.y + 0.01, ch.z + 0.01));

        for (var i = 0; i < Extension.UNIFORMS.uniforms.length; i++) {
            var uniform = Extension.UNIFORMS.uniforms[i];
            uniform.clippingLow.value.copy(uniforms.clippingLow.value);
            uniform.clippingHigh.value.copy(uniforms.clippingHigh.value);
        }
    },

    getClipping: function(colorHex, opacity) {
        var uniforms = Extension.UNIFORMS.clipping;
        var item = {
            color: { type: "c", value: new THREE.Color(colorHex) },
            clippingLow: { type: "v3", value: new THREE.Vector3(-100, -100, -100) },
            clippingHigh: { type: "v3", value: new THREE.Vector3(100, 100, 100) },
            opacity: { type: "f", value: opacity },
            orignalOpacity: opacity
        };


        item.clippingLow.value.copy(uniforms.clippingLow.value);
        item.clippingHigh.value.copy(uniforms.clippingHigh.value);

        Extension.UNIFORMS.uniforms.push(item);

        return item;
    },
    clipping: {
        color: { type: "c", value: new THREE.Color(0xff0000) },
        clippingLow: { type: "v3", value: new THREE.Vector3(-100, -100, -100) },
        clippingHigh: { type: "v3", value: new THREE.Vector3(100, 100, 100) },
        opacity: { type: "f", value: 1.0 }
    }
};

Extension.SHADER = {

    vertex: '\
		uniform vec3 color;\
		varying vec3 pixelNormal;\
		\
		void main() {\
			\
			pixelNormal = normal;\
			gl_Position = projectionMatrix * modelViewMatrix * vec4( position, 1.0 );\
			\
		}',

    vertexClipping: '\
		uniform vec3 color;\
		uniform vec3 clippingLow;\
		uniform vec3 clippingHigh;\
		\
		varying vec3 pixelNormal;\
		varying vec4 worldPosition;\
		varying vec3 camPosition;\
		\
		void main() {\
			\
			pixelNormal = normal;\
			worldPosition = modelMatrix * vec4( position, 1.0 );\
			camPosition = cameraPosition;\
			\
			gl_Position = projectionMatrix * modelViewMatrix * vec4( position, 1.0 );\
			\
		}',

    fragment: '\
		uniform vec3 color;\
		uniform float  opacity;\
		varying vec3 pixelNormal;\
		\
		void main( void ) {\
			\
			float shade = (\
				  3.0 * pow ( abs ( pixelNormal.y ), 2.0 )\
				+ 2.0 * pow ( abs ( pixelNormal.z ), 2.0 )\
				+ 1.0 * pow ( abs ( pixelNormal.x ), 2.0 )\
			) / 3.0;\
			\
			gl_FragColor = vec4( color * shade, opacity );\
			\
		}',

    fragmentClipping: '\
		uniform vec3 color;\
		uniform vec3 clippingLow;\
		uniform vec3 clippingHigh;\
		uniform float  opacity;\
		\
		varying vec3 pixelNormal;\
		varying vec4 worldPosition;\
		\
		void main( void ) {\
			\
			float shade = (\
				  3.0 * pow ( abs ( pixelNormal.y ), 2.0 )\
				+ 2.0 * pow ( abs ( pixelNormal.z ), 2.0 )\
				+ 1.0 * pow ( abs ( pixelNormal.x ), 2.0 )\
			) / 3.0;\
			\
			if (\
				   worldPosition.x < clippingLow.x\
				|| worldPosition.x > clippingHigh.x\
				|| worldPosition.y < clippingLow.y\
				|| worldPosition.y > clippingHigh.y\
				|| worldPosition.z < clippingLow.z\
				|| worldPosition.z > clippingHigh.z\
			) {\
				\
				discard;\
				\
			} else {\
				\
				gl_FragColor = vec4( color * shade, opacity );\
				\
			}\
			\
		}',
    fragmentClippingDiffuse: '\
		uniform vec3 color;\
		uniform vec3 clippingLow;\
		uniform vec3 clippingHigh;\
		uniform float  opacity;\
		\
		varying vec3 pixelNormal;\
		varying vec4 worldPosition;\
		\
		void main( void ) {\
			if (\
					worldPosition.x < clippingLow.x\
				|| worldPosition.x > clippingHigh.x\
				|| worldPosition.y < clippingLow.y\
				|| worldPosition.y > clippingHigh.y\
				|| worldPosition.z < clippingLow.z\
				|| worldPosition.z > clippingHigh.z\
			) {\
				\
				discard;\
				\
			} else {\
				\
				gl_FragColor = vec4( color , opacity );\
				\
			}\
			\
		}'
};

Extension.MATERIAL = {
    createSheet: function(colorHex, opacity, transparent, diffuse = false) {
        var uniforms = Extension.UNIFORMS.getClipping(colorHex, opacity);
        var material = new THREE.ShaderMaterial({
            uniforms: uniforms,
            vertexShader: Extension.SHADER.vertexClipping,
            fragmentShader: diffuse ? Extension.SHADER.fragmentClippingDiffuse : Extension.SHADER.fragmentClipping,
            side: THREE.DoubleSide,
            opacity: opacity,
            transparent: transparent
        });

        if (diffuse) {
            material.depthFunc = THREE.LessDepth;
            //material.blending = THREE.NoBlending;
        }
        return material;
    },
    // sheet: new THREE.ShaderMaterial( {
    // 	uniforms:       Extension.UNIFORMS.clipping,
    // 	vertexShader:   Extension.SHADER.vertexClipping,
    // 	fragmentShader: Extension.SHADER.fragmentClipping,
    // 	side : THREE.DoubleSide
    // } )
};