var Extension = Extension || {};

Extension.createTrianglesMesh = function(triangles, color, planeNormal) {
    var geom = new THREE.Geometry();
    
    for (var i = 0; i < triangles.length; i++) {
        var triangle = triangles[i];
        geom.vertices.push(triangle.a);
        geom.vertices.push(triangle.b);
        geom.vertices.push(triangle.c);
    
        geom.faces.push( new THREE.Face3( i*3, i*3 + 1, i*3 + 2 ) );
    }

    geom.computeFaceNormals();

    var material = new VIEW3D.materials.Metal({
        diffuseColor: color
    });
    material.side = THREE.DoubleSide;
    
    var mesh = new THREE.Mesh( geom, material );
    mesh.boundingBox = new THREE.Box3().setFromObject(mesh);
    
    if(planeNormal !== undefined) {
        mesh.armStart = Extension.calculateArmStartPoint(mesh, planeNormal);
    } else {
        mesh.armStart = mesh.boundingBox.getCenter();
    }
    
    return mesh;
}

Extension.createPointMesh = function(point) {
    var dotGeometry = new THREE.Geometry();
    dotGeometry.vertices.push(point);
    var dotMaterial = new THREE.PointsMaterial( { size: 10, sizeAttenuation: false, color: 0xff0000 } );
    var dot = new THREE.Points( dotGeometry, dotMaterial );
    return dot;
}

Extension.createArmLineMesh = function(armMesh, point, id) {
    var dir = Extension.calculateArmDir(armMesh, point);
    var end = point.clone().add(dir);
    return Extension.createArmLineMeshFromPoints(point, end, id);
}

Extension.createArmLineMeshFromPoints = function(start, end, id) {
    var geometry = new THREE.Geometry();
    geometry.vertices.push(end);
    geometry.vertices.push(start);
    var material = new THREE.LineBasicMaterial( { color: 0x0000ff } );
    var line = new THREE.Line( geometry, material );
    var lbl = VIEW3D.utils.Labeling.createSimpleLabelMesh("Arm " + id, end);

    var group = new THREE.Group();
    group.add( line ); group.line = line;
    group.add( lbl ); group.label = lbl;
    group.startPos = start;
    group.endPos = end;

    return group;
}