var Extension = Extension || {}
Extension.Pipe3DModelId = 100000;

Extension.actionIterateObjects = function(objs, action) {
    for (var i = 0; i < objs.length; i++){
        var obj = objs[i];
        action(obj);
    }
}

Extension.getBars = function(condition) {
    var resultBars = [];
    var bars = CORE.model.bars();
    Extension.actionIterateObjects(bars, bar => {
        if(condition(bar)) {
            resultBars.push(bar);
        }
    });
    return resultBars;
}

Extension.getHorizontalBars = function() {
    return Extension.getBars(bar => {
        var node1 = bar.node1().position();
        var node2 = bar.node2().position();
        return Math.abs(node1.z - node2.z) < 0.00001;
    });
}

Extension.getVerticalBars = function() {
    return Extension.getBars(bar => {
        var node1 = bar.node1().position();
        var node2 = bar.node2().position();
        return Math.abs(node1.z - node2.z) >= 0.00001;
    });
}

Extension.getHorizontalNodes = function() {
    var minZ = 1;
    var maxZ = 0;
    var nodes = CORE.model.nodes();
    Extension.actionIterateObjects(nodes, node => {
        if(node.position().z < minZ) {
            minZ = node.position().z;
        }
        if(node.position().z > maxZ) {
            maxZ = node.position().z;
        }
    });
    var resultNodes = [];
    var lastNodes = [];
    var topNodes = [];
    Extension.actionIterateObjects(nodes, node => {
        if(Math.abs(node.position().z - minZ) < 0.000001) {
            lastNodes.push(node);
        }
        else if(Math.abs(node.position().z - maxZ) < 0.000001) {
            topNodes.push(node);
        }
        else {
            resultNodes.push(node);
        }
    });

    return { otherNodes : resultNodes , lastNodes : lastNodes, minZ : minZ, topNodes : topNodes };
}

Extension.getVerticalNodes = function() {
    var maxX = -1;
    var nodes = CORE.model.nodes();
    Extension.actionIterateObjects(nodes, node => {
        if(node.position().x > maxX) {
            maxX = node.position().x;
        }
    });
    var resultNodes = [];
    Extension.actionIterateObjects(nodes, node => {
        if(Math.abs(node.position().x - maxX) < 0.000001) {
            resultNodes.push(node);
        }
    });

    return resultNodes;
}

Extension.fitTypicalIntoPipes = function(pipes) {
    var lastPipe = pipes[pipes.length - 1];
    var nodes = Extension.getVerticalNodes();
    var horNodes = Extension.getHorizontalNodes();

    Extension.actionIterateObjects(horNodes.otherNodes, node => {
        var oldPosition = node.position();
        if(pipes.minZ - pipes.maxDiamater > oldPosition.z) {
            node.position(new MODEL.types.Vector(oldPosition.x, oldPosition.y, (pipes.minZ - pipes.maxDiamater)*(oldPosition.z/horNodes.minZ)));
        }
    });

    Extension.actionIterateObjects(nodes, node => {
        var oldPosition = node.position();
        node.position(new MODEL.types.Vector(lastPipe.offset + pipes.maxDiamater*2, oldPosition.y, oldPosition.z));
    });

    
    Extension.actionIterateObjects(horNodes.lastNodes, node => {
        var oldPosition = node.position();
        node.position(new MODEL.types.Vector(oldPosition.x, oldPosition.y, pipes.minZ - pipes.maxDiamater));
    });

    Extension.actionIterateObjects(horNodes.topNodes, node => {
        var oldPosition = node.position();
        node.position(new MODEL.types.Vector(oldPosition.x, oldPosition.y, Extension.revitModelsMaxZ));
    });
    

    // var verticalBars = Extension.getVerticalBars();
    // Extension.actionIterateBars(verticalBars, bar => {
    //     if(bar.node1().position().x > 0) {
    //         var oldPosition = bar.node1().position();
    //         bar.node1().position(new MODEL.types.Vector(lastPipe.offset + pipes.maxDiamater*2, oldPosition.y, oldPosition.z));
    //         oldPosition = bar.node2().position();
    //         bar.node2().position(new MODEL.types.Vector(lastPipe.offset + pipes.maxDiamater*2, oldPosition.y, oldPosition.z));
    //     }
    // });


}

Extension.getPipes = function() {
    var pipes = [];
    var pipesCount = externalPipes.getPipeCount();
    var maxDiamater = 0;

    for (var i = 0; i < pipesCount; i++) {
        var pipeModel = externalPipes.getPipe(i);
        pipes.push(pipeModel);   
        if(pipeModel.diameter > maxDiamater){
            maxDiamater = pipeModel.diameter;
        }
    }
    pipes.maxDiamater = maxDiamater;
    return pipes;
}

Extension.getTypicalData = function () {
    var horNodes = Extension.getHorizontalNodes();
    var result = [];
    Extension.actionIterateObjects(horNodes.lastNodes, node => {
        var position = node.position();
        result.push(position);
    });
    return JSON.stringify(result);
}

Extension.loadPipes = function() {
    if(typeof externalPipes === 'undefined' || CORE.model.nodes().length < 3) {
        return;
    }
    //externalPipes.getTypicalData = Extension.getTypicalData;
    var attributes = {
        position: {
            type: Float32Array,
            size: 3
        },
        color: {
            type: Float32Array,
            size: 3
        },
        normal: {
            type: Float32Array,
            size: 3
        }
    };
    var material = new VIEW3D.materials.Metal();
    var pipesMesh = new VIEW3D.MultiMesh(attributes, material);
    scene.add(pipesMesh);
    var bs = CORE.view3D.cameraControl.getConstructionBS();
    var center = bs.center;

    var pipes = Extension.getPipes();
        pipes.minZ = 1;
        for (var i = 0; i < pipes.length; i++) {
            var pipeModel = pipes[i];
            //var templateObject = MODEL.utils.Proto.getObject3dFromBase64(model.data, Extension.Pipe3DModelId); //loader.parse(model.data);
            //var template = new VIEW3D.PipeTemplate(templateObject, model.config);
            var zPos = 0 + (pipes.maxDiamater/2) - (pipeModel.diameter/2) + pipeModel.height;
            if(pipes.minZ > zPos) {
                pipes.minZ = zPos;
            }
            // pipesMesh.addMeshFromTemplate(template, 
            // {
            //     transform: new THREE.Matrix4().makeTranslation( pipeModel.offset, 0, zPos),
            //     color: new THREE.Vector3(0.5, 0.5, 0.5),
            //     size: pipeModel.diameter
            // });    
        }
        Extension.fitTypicalIntoPipes(pipes);
        CORE.view3D.cameraControl.resetView(-1, 1, -1, true);

    
}