var Extension = Extension || {}
Extension.activePlane = undefined;
Extension.intersectionPoint = undefined;

Extension.isPipePlaneToolActive = ko.observable(false);

Extension.activatePipePlaneTool = function(selPipe) {
    Extension.clearActivePlane();
    // var selPipe = Extension.getFirstSelectedPipe();
    // if(selPipe === undefined) {
    //     alert('No pipe selected');
    //     return
    // }
    Extension.addPipePlaneClipping(selPipe);
    
    // Extension.addCrossSectionPipePlaneClipping(selPipe);

    var plane = Extension.buildPipePlane(selPipe);
    Extension.activePlane = plane;
    scene.add(plane);
    render();
}

Extension.addPipePlaneClipping = function(pipe) {
    var dir = pipe.direction;
    var inter = pipe.lastMouseOverPosition;
    var plane = undefined;
    if(Math.abs(dir.X) > 0.9) {
        Extension.setMaxXPlane(inter.x);
        //plane = new THREE.Plane(new THREE.Vector3( -1, 0, 0 ), inter.x*1.01);
    }
    if(Math.abs(dir.Y) > 0.9) {
        Extension.setMinYPlane(inter.y);
        //plane = new THREE.Plane(new THREE.Vector3( 0, 1, 0 ), -inter.y*1.02);
    }
    if(Math.abs(dir.Z) > 0.9) {
        Extension.setMaxZPlane(inter.z);
        //lane = new THREE.Plane(new THREE.Vector3( 0, 0, -1 ), inter.z*0.98);
    }
    //Extension.addNewClipPlane(plane);
}

Extension.addCrossSectionPipePlaneClipping = function(pipe) {
    var dir = pipe.direction;
    var inter = pipe.lastMouseOverPosition;
    var plane = undefined;
    
    if(Math.abs(dir.X) > 0.9) {
        Extension.setMinXPlane(inter.x - 0.05);
        //plane = new THREE.Plane(new THREE.Vector3( 1, 0, 0 ), -(inter.x - 0.5)*1.01);
    }
    if(Math.abs(dir.Y) > 0.9) {
        Extension.setMaxYPlane(inter.y + 0.05);
        //plane = new THREE.Plane(new THREE.Vector3( 0, -1, 0 ), (inter.y + 0.5)*1.02);
    }
    if(Math.abs(dir.Z) > 0.9) {
        Extension.setMinZPlane(inter.z - 0.05);
        //plane = new THREE.Plane(new THREE.Vector3( 0, 0, 1 ), -(inter.z - 0.5)*0.98);
    }
    //Extension.addNewClipPlane(plane);
}

Extension.calculatePlaneWidth = function(dir){
    var from = Extension.geoDefinition.From;
    var to = Extension.geoDefinition.To;
    if(Math.abs(dir.X) > 0.9) {
        return to.Z - from.Z;
    }
    if(Math.abs(dir.Y) > 0.9) {
        return to.X - from.X;
    }
    if(Math.abs(dir.Z) > 0.9) {
        return to.Y - from.Y;
    }
}

Extension.calculatePlaneHeight = function(dir){
    var from = Extension.geoDefinition.From;
    var to = Extension.geoDefinition.To;
    if(Math.abs(dir.X) > 0.9) {
        return to.Y - from.Y;
    }
    if(Math.abs(dir.Y) > 0.9) {
        return to.Z - from.Z;
    }
    if(Math.abs(dir.Z) > 0.9) {
        return to.X - from.X;
    }
}

Extension.buildPipePlane = function(pipe) {
    var dir = pipe.direction;
    var bb = pipe.boundingBox;
    var inter = pipe.lastMouseOverPosition;
    Extension.intersectionPoint = inter;
    var geometry = new THREE.PlaneGeometry( Extension.calculatePlaneWidth(dir)*0.99, Extension.calculatePlaneHeight(dir)*0.99);
    
    //for some reason lookAt method did not result in desired results - so rotations are calculated by hand
    //1.57 = Pi/2 = 90 degrees
    geometry.rotateX(dir.Y*1.57);
    geometry.rotateY(dir.X*1.57);
    geometry.rotateZ(dir.Z*1.57);

    geometry.translate(dir.X*inter.x, dir.Y*inter.y, dir.Z*inter.z);
    var material = Extension.MATERIAL.createSheet(0x98fb98, 0.2, true, true);
    //var material = new THREE.MeshStandardMaterial ( {color: 0x98fb98, side: THREE.DoubleSide, opacity : 0.3, transparent : true } );
    var plane = new THREE.Mesh( geometry, material );
    var helper = new THREE.EdgesHelper(plane, 0xff0000);
    helper.material.linewidth = 5;
    helper.material.depthTest = false;

    var group = new THREE.Group();
    group.add( plane );
    group.add( helper );

    return group;
}

Extension.clearActivePlane = function() {
    if(Extension.activePlane !== undefined) {
        Extension.removeAdditionalClipPlanes();
        scene.remove(Extension.activePlane);
        Extension.activePlane = undefined;
    }
}

Extension.moveCameraToActivePlane = function() {
    scene.remove(Extension.activePlane);

    var camera = CORE.view3D.cameraControl.camera;
    var pipe = Extension.getFirstSelectedPipe();
    Extension.addCrossSectionPipePlaneClipping(pipe);
    var dir = pipe.direction;
    var bb = pipe.boundingBox;
    var inter = Extension.intersectionPoint;
    var lookAt = new THREE.Vector3( -dir.X, dir.Y, -dir.Z ).add(inter);
    camera.position.set(inter.x + 1*dir.X, inter.y - 1*dir.Y, inter.z + 1*dir.Z);
    camera.up.set(0, 0, 1);
    camera.lookAt(lookAt);
    camera.updateMatrix();
}
