var Extension = Extension || {};

Extension.protoManagerMeshCache = [];


Extension.getProtoMesh = function(id) {
    return new Promise((resolve, reject) => {
        if(Extension.protoManagerMeshCache[id] !== undefined) {
            resolve(Extension.cloneMesh(Extension.protoManagerMeshCache[id]));
        } else {
            CORE.webApiManager.getStpFile(id, function(stp) {
                var result = Extension.getProtoResultFromBase64(stp);
                var resMesh = undefined;
                if(result.parts.length === 1) {
                    var part = result.parts[0].geometry;
                    var mesh = Extension.createMesh(part.position, part.color, part.index);
                    resMesh = mesh;
                } else {
                    var res = THREE.Group();
                    for (var j = 0; j < result.parts.length; j++) {
                        var part = result.parts[j].geometry;
                        var mesh = Extension.createMesh(part.position, part.color, part.index);
                        res.add(mesh);
                    }
                    resMesh = mesh;
                }
                Extension.protoManagerMeshCache[id] = resMesh;
                resolve(Extension.cloneMesh(resMesh));
            });
        }
      });
}

Extension.cloneMesh = function(mesh) {
    return new THREE.Mesh(mesh.geometry.clone(), mesh.material.clone());
}

Extension.getProtoResultFromBase64 = function(stp){
    var b64Length = ProtoBuf.util.base64.length(stp);
    var outputBuffer = new Uint8Array(b64Length);
    ProtoBuf.util.base64.decode(stp, outputBuffer, 0);
    var result = Extension.stpProtoBuilder.ItemModel.decode(outputBuffer);
    return result;
}

Extension.storeProto = function(id, result) {
    var buffer = Extension.stpProtoBuilder.ItemModel.encode(result).finish();
    var base64Val = ProtoBuf.util.base64.encode(buffer, 0, buffer.length);
    CORE.webApiManager.updateViewerModel(id, Extension.stpItemType, base64Val, null, function() {});
}

Extension.storeDoubleProto = function(id, result1, result2) {
    var buffer1 = Extension.stpProtoBuilder.ItemModel.encode(result1).finish();
    var base64Val = ProtoBuf.util.base64.encode(buffer1, 0, buffer1.length);
    var buffer2 = Extension.stpProtoBuilder.ItemModel.encode(result2).finish();
    var base64ValDouble = ProtoBuf.util.base64.encode(buffer2, 0, buffer2.length);

    CORE.webApiManager.updateViewerModel(id, Extension.stpItemType, base64Val, base64ValDouble, function() {});
}