var Extension = Extension || {}
Extension.revitModelsMaxZ = 5;
Extension.externalBoundingBox = Extension.externalBoundingBox || [];
Extension.geoDefinition = undefined;

Extension.isTransparent = function(elementType) {
    return elementType !== "Pipe" && elementType !== "Duct";
}

Extension.buildEdge = function(mesh) {
    //var helper = new THREE.EdgesHelper(mesh, 0x27408B);
    //helper.material.linewidth = 2;
    var edges = new THREE.EdgesGeometry(mesh.geometry);
    var helper = new THREE.LineSegments(edges, Extension.MATERIAL.createSheet(0x0000ff, 1, Extension.isTransparent(mesh.revitModel.ElementType)));

    return helper;
}

Extension.buildWall = function(bb) {
    var geometry = new THREE.BufferGeometry();
    var verts = [
        bb.min.x, bb.min.y, bb.min.z,
        bb.max.x, bb.min.y, bb.min.z,
        bb.max.x, bb.max.y, bb.min.z,
        bb.min.x, bb.max.y, bb.min.z,
        bb.min.x, bb.min.y, bb.max.z,
        bb.max.x, bb.min.y, bb.max.z,
        bb.max.x, bb.max.y, bb.max.z,
        bb.min.x, bb.max.y, bb.max.z
    ];
    var index = [
        0, 1, 2, //bottom
        0, 2, 3, //bottom
        4, 5, 6, //top
        4, 6, 7, //top
        0, 1, 5, //front
        0, 4, 5, //front
        1, 2, 6, //right
        1, 5, 6, //right
        2, 3, 7, //back
        2, 6, 7, //back
        3, 0, 4, //left
        3, 7, 4 //left
    ];

    var geometry = new THREE.BufferGeometry();
    geometry.addAttribute('position', new THREE.Float32BufferAttribute(verts, 3));
    geometry.setIndex(index);

    return geometry;
}

Extension.buildMesh = function(meshModel, model) {
    var geometry = new THREE.BufferGeometry();
    geometry.addAttribute('position', new THREE.Float32BufferAttribute(meshModel.FaceVerticies, 3));
    geometry.addAttribute('normal', new THREE.Float32BufferAttribute(meshModel.FaceNormals, 3));
    geometry.setIndex(meshModel.FaceIndices);

    geometry.translate(meshModel.Translation.X, meshModel.Translation.Y, meshModel.Translation.Z);
    //var material = Extension.MATERIAL.sheet;
    var material = Extension.MATERIAL.createSheet(model.ColorHex, model.Opacity, Extension.isTransparent(model.ElementType));
    //var material = new THREE.MeshStandardMaterial({ color: model.ColorHex, transparent: true, opacity: model.Opacity, side: THREE.DoubleSide });
    var mesh = new THREE.Mesh(geometry, material);

    var bbox = new THREE.Box3().setFromObject(mesh);
    if (model.IsRectangularSurface && bbox.min.z > 0 && bbox.min.z < Extension.revitModelsMaxZ) {
        Extension.revitModelsMaxZ = bbox.min.z;
    }

    if (model.ElementType === "Wall") {
        //var wallGeo = Extension.buildWall(bbox);
        //mesh = new THREE.Mesh(wallGeo, material);
    }

    mesh.boundingBox = bbox;
    mesh.revitModel = model;
    return mesh;
}

Extension.meshes = [];
Extension.Pipe = [];
Extension.Duct = [];

Extension.buildMeshes = function(model) {
    var meshes = [];
    var actualMeshes = [];
    for (var j = 0; j < model.Meshes.length; j++) {
        var meshModel = model.Meshes[j];
        var mesh = Extension.buildMesh(meshModel, model);
        var edge = Extension.buildEdge(mesh);
        Extension.meshes.push(mesh);
        Extension.meshes.push(edge);
        actualMeshes.push(mesh);
        if (!Extension.isTransparent(model.ElementType)) {
            Extension[model.ElementType].push(mesh);
            Extension[model.ElementType].push(edge);
        }
    }
    model.calculatedMeshes = Extension.meshes;
    model.boundingBox = Extension.consolidateBoundingBox(actualMeshes);
    model.boundingBoxObject = new THREE.Box3Helper(model.boundingBox, 0x0000ff);
    return Extension.meshes;
}

Extension.consolidateBoundingBox = function(meshes) {
    var minX = 100,
        minY = 100,
        minZ = 100,
        maxX = -100,
        maxY = -100,
        maxZ = -100;
    for (var j = 0; j < meshes.length; j++) {
        var mesh = meshes[j];
        var bb = mesh.boundingBox;
        if (bb.max.x > maxX) {
            maxX = bb.max.x;
        }
        if (bb.min.x < minX) {
            minX = bb.min.x;
        }
        if (bb.max.y > maxY) {
            maxY = bb.max.y;
        }
        if (bb.min.y < minY) {
            minY = bb.min.y;
        }
        if (bb.max.z > maxZ) {
            maxZ = bb.max.z;
        }
        if (bb.min.z < minZ) {
            minZ = bb.min.z;
        }
    }
    return new THREE.Box3(new THREE.Vector3(minX, minY, minZ), new THREE.Vector3(maxX, maxY, maxZ));
}

Extension.getParamValue = function(model, paramName) {
    for (var i = 0; i < model.Parameters.length; i++) {
        var param = model.Parameters[i];
        if (param.Name === paramName) {
            return param.Value;
        }
    }
}

Extension.pipes = [];
Extension.getFirstSelectedPipe = function() {
    for (var i = 0; i < Extension.pipes.length; i++) {
        var pipe = Extension.pipes[i];
        if (pipe.selected() === true) {
            return pipe;
        }
    }
}

Extension.processPipe = function(model) {
    model.isSelected = false;

    model.selected = function(val) {
        if (val === undefined) return model.isSelected;
        if (val === model.isSelected) return;
        model.isSelected = val;
        if (val) {
            //scene.add(model.boundingBoxObject);
            if (Extension.isPipePlaneToolActive()) {
                Extension.activatePipePlaneTool(model);
            }
        } else {
            //scene.remove(model.boundingBoxObject);
            Extension.clearActivePlane();
        }
        render();
    }

    model.direction = Extension.getParamValue(model, "Direction");
    model.direction.X = Math.abs(model.direction.X);
    model.direction.Y = Math.abs(model.direction.Y);
    model.direction.Z = Math.abs(model.direction.Z);
    model.origin = Extension.getParamValue(model, "Origin");
}

Extension.processModel = function(model) {
    if (model.ElementType === "Pipe") {
        Extension.processPipe(model);
        Extension.pipes.push(model);
    }
}

Extension.loadRevitModels = function() {
    if (typeof revitDataProxy === 'undefined') {
        return;
        $.getJSON("assets/revitResponse.json", function(json) {
            revitDataProxy = {
                getGeometryDefinitonJson: function() {
                    return JSON.stringify(json);
                }
            };
            Extension.loadRevitModels();
        });
        return;
    }

    var geoDefinition = JSON.parse(revitDataProxy.getGeometryDefinitonJson());
    geoDefinition.From.Z = geoDefinition.From.Z - 0.1;
    geoDefinition.To.Z = geoDefinition.To.Z + 0.1;

    Extension.geoDefinition = geoDefinition;

    for (var i = 0; i < geoDefinition.Models.length; i++) {
        var model = geoDefinition.Models[i];
        Extension.processModel(model);
        var meshes = Extension.buildMeshes(model);
        CORE.constructionView3D.externalModels.addModel(model, meshes, model.ElementType === "Pipe");
    }
    Extension.clipPlanes(geoDefinition);
    var light = new THREE.HemisphereLight(0xffffbb, 0x080820, 1);
    //var light = new THREE.AmbientLight( 0xbbbbbb );
    //light.position.set( 0, -50, 50 );
    //scene.add(light);
}