var Extension = Extension || {};
Extension.stpId = null;
Extension.stpVariantId = null;
Extension.stpItemType = null;
Extension.stpProtoBuilder = null;
Extension.connectorMeshes = [];

Extension.isStpUnique = function(result, part, i) {
    for (let j = i + 1; j < result.parts.length; j++) {
        var otherPart = result.parts[j].geometry;
        if(part.position.length === otherPart.position.length &&
            part.index.length === otherPart.index.length
        ) {
            return false;
        }
    }
    return true;
}

Extension.createMesh = function(position, color, index) {
    var vertices = new Float32Array(position);
    var colors = new Float32Array(color);
    var indices = new Uint16Array(index);
    var geometry = new THREE.BufferGeometry();
    geometry.addAttribute('position', new THREE.BufferAttribute(vertices, 3));
    geometry.addAttribute( 'color', new THREE.BufferAttribute( colors, 3 ) );
    geometry.setIndex(new THREE.BufferAttribute(indices, 1));
    geometry.computeVertexNormals();

    var material = new THREE.MeshStandardMaterial({
        //vertexColors: THREE.VertexColors,
        normalScale : new THREE.Vector2(0.3, 0.3),
        metalness : 0.4,
        color: new THREE.Vector3(color[0], color[1], color[2]) //color
    });
    material.side = THREE.DoubleSide;
    var mesh = new THREE.Mesh(geometry, material);
    mesh.boundingBox = new THREE.Box3().setFromObject(mesh);

    return mesh;
}

Extension.initScene = function() {
    //var light = new THREE.DirectionalLight( 0xffffff );
    //light.position.set( 0, 1, 1 ).normalize();
    var light = new THREE.AmbientLight( 0xaaaaaa, 0.5 );
    //var light = new THREE.HemisphereLight( 0xffffff, 0xaaaaaa, 0.45 );
    scene.add( light );

    render(2);
    CORE.view3D.cameraControl.zoomAll();
}

Extension.processCrossSectionStp = function(result, config) {
    for (var i = 0; i < result.parts.length; i++) {//result.parts.length
        var part = result.parts[i].geometry;
        var mesh = Extension.createMesh(part.position, part.color, part.index);

        if(config.isEmpty === true) {
            Extension.recogniseCrossSection(mesh, part.position);
            part.position = mesh.geometry.attributes.position.array;
        }
        mesh.boundingBox = new THREE.Box3().setFromObject(mesh);
        var profile = Extension.prepareCrossSectionInstancedProfileMesh(mesh, new THREE.Matrix4(), mesh.boundingBox.getSize().x);
        CORE.constructionView3D.children.push(profile);
    }

    if(config.isEmpty === true) {
        Extension.storeProto(Extension.stpId, result);
    }
}

Extension.applyConnectorMeshMatrix = function(matrix) {
    for (var i = 0; i < Extension.connectorMeshes.length; i++) {
        var mesh = Extension.connectorMeshes[i];
        if(mesh.originalGeometry === undefined) mesh.originalGeometry = mesh.geometry.clone();
        mesh.geometry = mesh.originalGeometry.clone().applyMatrix(matrix);
    }
}

Extension.processConnectorStp = function(res, doubleResult, config) {
    Extension.connectorMeshes = [];
    Extension.connector3dModel = new VIEW3D.DmtConnector(res.parts[0].geometry);
    if(doubleResult !== null && doubleResult.parts.length > 0) Extension.connectorDouble3dModel = new VIEW3D.DmtConnector(doubleResult.parts[0].geometry);
    function processPart(result) {
        for (var i = 0; i < result.parts.length; i++) {//result.parts.length
            var part = result.parts[i].geometry;
            var mesh = Extension.createMesh(part.position, part.color, part.index);
            Extension.connectorMeshes.push(mesh);
            //CORE.constructionView3D.children.push(mesh);
            
            if(config.isEmpty === true) {
                var res = Extension.recogniseDoubleConnector(mesh, part.position, part.index);
                if(res.isDoubleConnector) {
                    var dividedModels = Extension.divideDoubleConnectorModel(res.parts, part.color);
                    Extension.storeDoubleProto(Extension.stpId, 
                        { parts : [ { geometry : dividedModels[0]} ] },
                        { parts : [ { geometry : dividedModels[1]} ] } );
                } else {
                    Extension.recogniseArms(mesh, part.position, part.index);
                }
            }
        }
    }
    processPart(res);
    if(doubleResult !== null && doubleResult.parts.length > 0) {
        processPart(doubleResult);
        Extension.connectorDouble3dModel.setTransform(new THREE.Matrix4());
        CORE.constructionView3D.children.push(Extension.connectorDouble3dModel);
    }

    Extension.connector3dModel.setTransform(new THREE.Matrix4());
    CORE.constructionView3D.children.push(Extension.connector3dModel);

    if(config.isEmpty !== true) {
        Extension.loadConfiguration(config, Extension.connectorMeshes);
    } else {
        var config = Extension.prepareFullConnectorConfiguration();
        CORE.webApiManager.postConnectorConfig(config, function() {});
    }
}

Extension.processPipeFasteningStp = function(res, doubleResult, config) {
    Extension.connectorMeshes = [];
    Extension.connector3dModel = new VIEW3D.DmtConnector(res.parts[0].geometry);
    if(doubleResult !== null && doubleResult.parts.length > 0) Extension.connectorDouble3dModel = new VIEW3D.DmtConnector(doubleResult.parts[0].geometry);
    function processPart(result) {
        for (var i = 0; i < result.parts.length; i++) {//result.parts.length
            var part = result.parts[i].geometry;
            var mesh = Extension.createMesh(part.position, part.color, part.index);
            Extension.connectorMeshes.push(mesh);
            //CORE.constructionView3D.children.push(mesh);
            
            if(config.isEmpty === true) {
                var res = Extension.recogniseDoubleConnector(mesh, part.position, part.index);
                if(res.isDoubleConnector) {
                    var dividedModels = Extension.divideDoubleConnectorModel(res.parts, part.color);
                    Extension.storeDoubleProto(Extension.stpId, 
                        { parts : [ { geometry : dividedModels[0]} ] },
                        { parts : [ { geometry : dividedModels[1]} ] } );
                } else {
                    Extension.recogniseArms(mesh, part.position, part.index);
                }
            }
        }
    }
    processPart(res);
    if(doubleResult !== null && doubleResult.parts.length > 0) {
        processPart(doubleResult);
        Extension.connectorDouble3dModel.setTransform(new THREE.Matrix4());
        CORE.constructionView3D.children.push(Extension.connectorDouble3dModel);
    }

    Extension.connector3dModel.setTransform(new THREE.Matrix4());
    CORE.constructionView3D.children.push(Extension.connector3dModel);

    if(config.isEmpty !== true) {
        Extension.loadConfiguration(config, Extension.connectorMeshes);
    } else {
        var config = Extension.prepareFullConnectorConfiguration();
        CORE.webApiManager.postConnectorConfig(config, function() {});
    }
}

Extension.connector3dModel = null;
Extension.connectorDouble3dModel = null;

Extension.processBaseStp = function(result, config) {
    Extension.connectorMeshes = [];
    Extension.connector3dModel = new VIEW3D.DmtConnector(result.parts[0].geometry);

    for (var i = 0; i < result.parts.length; i++) {//result.parts.length
        var part = result.parts[i].geometry;
        
        var mesh = Extension.createMesh(part.position, part.color, part.index);
        Extension.connectorMeshes.push(mesh);
        //CORE.constructionView3D.children.push(mesh);
        if(config.isEmpty === true) {
            Extension.recogniseBaseArms(mesh, part.position, part.index);
        }
    }

    Extension.connector3dModel.setTransform(new THREE.Matrix4());
    CORE.constructionView3D.children.push(Extension.connector3dModel);
    
    if(config.isEmpty !== true) {
        Extension.loadConfiguration(config, Extension.connectorMeshes);
    } else {
        var config = Extension.prepareFullConnectorConfiguration();
        CORE.webApiManager.postConnectorConfig(config, function() {});
    }
}

Extension.processPipeRingStp = function(result, config) {
    Extension.connector3dModel = new VIEW3D.DmtConnector(result.parts[0].geometry);

    Extension.connector3dModel.setTransform(new THREE.Matrix4());
    CORE.constructionView3D.children.push(Extension.connector3dModel);
    
    // CORE.refreshAspectRatio();
}

Extension.loadConfigAndProtoModel = function(stpId, itemType, callback) {
    CORE.webApiManager.getConfig(stpId, itemType, function(config) {
        CORE.webApiManager.getStpFile(stpId, function(stp) {
            CORE.webApiManager.getDoubleStpFile(stpId, function(doubleStp) {
                ProtoBuf.load("/assets/ItemModel.proto",
                    null,
                    function (error, root) {
                        Extension.stpProtoBuilder = {};
                        Extension.stpProtoBuilder.ItemModel = root.lookup("ItemModel");
                        var result = Extension.getProtoResultFromBase64(stp);
                        var doubleResult = doubleStp !== null ? Extension.getProtoResultFromBase64(doubleStp) : null;
                        if(itemType === "BASE") Extension.processBaseStp(result, config);
                        if(itemType === "CONNECTOR") Extension.processConnectorStp(result,doubleResult, config);
                        if(itemType === "PIPE_FASTENING") Extension.processPipeFasteningStp(result,doubleResult, config);
                        if(itemType === "CROSS_SECTION") Extension.processCrossSectionStp(result, config);
                        if(itemType === "PIPE_RING") Extension.processPipeRingStp(result, config);
                        ExtensionInterface.armsTranslationVector( Extension.armsTranslationVector);
                        ExtensionInterface.rewriteArms();
                        Extension.initScene();
                        ExtensionInterface.loadingCompleated(true);
                        if(callback !== undefined) callback();
                    });
            });
        });
    });
}
Extension.loadStp = function(stpId, itemType, variantId) {
    Extension.stpId = stpId;
    Extension.stpVariantId = variantId !== "null" ? variantId : null;
    Extension.stpItemType = itemType;

    if(Extension.stpVariantId !== null && Extension.stpItemType !== "PIPE_RING") {
        CORE.webApiManager.getVariantConfig(Extension.stpVariantId, itemType, function(variantConfig) {
            if(variantConfig.isEmpty !== true) {
                Extension.loadConfigAndProtoModel(stpId, itemType, () => Extension.loadConnectorVariantConfig(variantConfig, null));
            } else {
                Extension.loadConfigAndProtoModel(stpId, itemType);
            }
        });
    } else {
        Extension.loadConfigAndProtoModel(stpId, itemType);
    }

    
}