﻿var KO_STATUS = {
    ADDED: "added",
    DELETED: "deleted"
}
function isEmpty(value) {
    return (typeof value === "undefined" || value === null);
}

function isEmptyDictionary(obj) {
    for (var key in obj) {
        return false;
    }
    return true;
}

Function.prototype.inheritsFrom = function(parent) {
    this.prototype = Object.create(parent.prototype);
    this.prototype.constructor = this;
}
function subtractArray(fromArray, subtractArray) {
    var result = fromArray.filter(function(item) {
        return subtractArray.indexOf(item) === -1;
    });
    return result;
}
var addElementToSet = function(set, element) {
    if (element == null) {
        throw new Error("HELPERS.addElementToSet: Cannot add null");
    }
    if (isEmpty(set)) {
        throw new Error("HELPERS.addElementToSet: Passed set is empty");
    }
    if (set.indexOf(element) !== -1) {
        throw new Error("HELPERS.addElementToSet: Cannot add the same element twice");
    }
    set.push(element);
    return element;
}
var removeElementFromSet = function(set, element) {
    var index = set.indexOf(element);
    if (index === -1) {
        throw new Error("HELPERS.removeElementFromSet: Element not found");
    }
    set.splice(index, 1);
}
var isElementInSet = function(set, element) {
    return set.indexOf(ko.unwrap(element)) !== -1;
}

HELPERS = {
    getMaximumAbs: function(valuesArr) {
        return Math.max.apply(null, valuesArr.map(Math.abs));
    },
    getSign: function(number) {
        return number && number / Math.abs(number);
    },
    toNumber: function(object) {
        return typeof object !== 'boolean' ? Number(object) : object;
    },
    toVector3: function (vector) {   // convert any vector to THREE.Vector3
        vector = ko.unwrap(vector);
        if (typeof vector.vector === 'undefined') {
            return new THREE.Vector3(vector.x, vector.y, vector.z);
        } else { // if (vector instanceof MODEL.types.ExtendedVector)
            return new THREE.Vector3(vector.vector.x, vector.vector.y, vector.vector.z);
        }
    },
    toVector: function (vector) {   // convert any vector to MODEL.types.Vector
        vector = ko.unwrap(vector);
        if (isEmpty(vector.x) || isEmpty(vector.y) || isEmpty(vector.z)) throw new Error("HELPERS.toVector: passed vector is invalid");
        return new MODEL.types.Vector(vector.x, vector.y, vector.z);
    },
    subscribeArrayChange: function(observer, observable, observableName, onItemAdded, onItemRemoved, additionalParam) {
        //console.log("HELPERS.subscribeArrayChange: subscribing for array " + observableName);
        if (typeof observer === 'undefined') {
            throw new Error('Observer is undefined');
        }
        if (typeof observable === 'undefined') {
            throw new Error('Observable is undefined');
        }
        if (!ko.isObservable(observable) && observable().constructor === Array) {
            console.error(observable);
            throw new Error('Parameter "observable" is not an observable array');
        }
        if (typeof onItemAdded === 'undefined') {
            console.warn('HELPERS.subscribeArrayChange: onItemAdded is undefined. If intentional, please pass null instead');
        }
        if (typeof onItemRemoved === 'undefined') {
            //console.warn('HELPERS.subscribeArrayChange: onItemRemoved is undefined. If intentional, please pass null instead');
        }
        return observable.subscribe(function(changes) {
            for (var i = 0; i < changes.length; i++) {
                if (changes[i].status === KO_STATUS.ADDED && !isEmpty(onItemAdded)) {
                    //console.log("HELPERS.subscribeArrayChange: " + observableName + " added: ");
                    //console.log(changes[i].value);
                    onItemAdded.call(observer, changes[i].value, changes[i].index, additionalParam);
                }
                if (changes[i].status === KO_STATUS.DELETED && !isEmpty(onItemRemoved)) {
                    //console.log("HELPERS.subscribeArrayChange: " + observableName + " deleted: ");
                    //console.log(changes[i].value);
                    onItemRemoved.call(observer, changes[i].value, changes[i].index, additionalParam);
                }
            }
        }, null, "arrayChange");
    },
    subscribePropertyChange: function(observer, observable, observableName, onPropertyChanged, customNewValue, customThis) { //customNewValue, customEvent - optional
        //console.log("HELPERS.subscribePropertyChange: subscribing for property " + observableName);
        if (typeof observer === 'undefined') {
            throw new Error('Observer is undefined');
        }
        if (typeof observable === 'undefined') {
            throw new Error('Observable is undefined');
        }
        if (typeof observable.subscribe === 'undefined') {
            throw new Error('Observable is not a knockout object');
        }
        if (typeof onPropertyChanged === 'undefined') {
            throw new Error('onPropertyChanged is undefined');
        }
        return observable.subscribeChanged(function(newValue, oldValue) {
            //console.log("HELPERS.subscribeArrayChange: " + observableName + " changed, new value:");
            //console.log(newValue);
            if (isEmpty(customNewValue)) {
                onPropertyChanged.call(observer, newValue, oldValue, customThis);
            } else {
                onPropertyChanged.call(observer, customNewValue, oldValue, customThis);
            }
        });
    },
    /*subscribeRecurrent: function (observer, observable, observableName, onPropertyChanged, customNewValue) {    // for observing Vector type     
        observable = ko.toJS(observable);

        for (var propertyName in observable) {
            if (observable.hasOwnProperty(propertyName)) {
                var property = observable[propertyName];
                if (typeof property === 'object') {
                    this.subscribeRecurrent(observer, property, observableName + '.' + propertyName, onPropertyChanged, customNewValue);
                } else {
                    this.subscribePropertyChange(observer, property, observableName + '.' + propertyName, onPropertyChanged, customNewValue);
                }
            }
        }
    },*/
    subscribeVectorPropertyChange: function(observer, observableVector, observableName, onPropertyChanged, customNewValue, customThis) { // when xyz separately is in need to be observed
        //this.subscribePropertyChange(observer, observableVector, observableName, onPropertyChanged);
        if (typeof customNewValue === 'undefined') {
            customNewValue = observableVector;
        }
        if (typeof ko.unwrap(observableVector).x === 'undefined') {
            throw new Error("observableVector.x is undefined");
        }
        this.subscribePropertyChange(observer, ko.unwrap(observableVector).x, observableName, onPropertyChanged, customNewValue, customThis);
        if (typeof ko.unwrap(observableVector).y === 'undefined') {
            throw new Error("observableVector.y is undefined");
        }
        this.subscribePropertyChange(observer, ko.unwrap(observableVector).y, observableName, onPropertyChanged, customNewValue, customThis);
        if (typeof ko.unwrap(observableVector).z === 'undefined') {
            throw new Error("observableVector.z is undefined");
        }
        this.subscribePropertyChange(observer, ko.unwrap(observableVector).z, observableName, onPropertyChanged, customNewValue, customThis);
    },
    /*subscribeVectorPropertyChange: function (observer, observableVector, observableName, onPropertyChanged, customNewValue) {   // when xyz separately is in need to be observed
        //this.subscribePropertyChange(observer, observableVector, observableName, onPropertyChanged);
        if (typeof customNewValue === 'undefined') {
            customNewValue = observableVector;
        }
        var vector = ko.unwrap(observableVector);
        if (vector instanceof MODEL.types.Vector) {
            this.subscribePropertyChange(observer, observableVector.x, observableName, onPropertyChanged, customNewValue);
            this.subscribePropertyChange(observer, observableVector.y, observableName, onPropertyChanged, customNewValue);
            this.subscribePropertyChange(observer, observableVector.z, observableName, onPropertyChanged, customNewValue);
        }
        else
            if (vector instanceof MODEL.types.ExtendedVector) {
                this.subscribeVectorPropertyChange(observer, vector.vector, observableName, onPropertyChanged, customNewValue);
                this.subscribePropertyChange(observer, vector.x, observableName, onPropertyChanged, customNewValue);
            }
            else {
                throw 'HELPERS.subscribeVectorPropertyChange: passed unsupported vector object, use subscribePropertyChange instead';
            }
    },*/
    addToScene: function (element, scene) {
        if (scene.children.indexOf(element) === -1) {
            scene.add(element);
        }
    },
    removeFromScene: function (element, scene) {
        if (scene.children.indexOf(element) !== -1) {
            scene.remove(element);
        }
    },
	isInScene: function(element, scene) {
        if (scene.children.indexOf(element) !== -1) {
            return true;
        }
        return false;
    },
    toRelative: function (bar, xAbsolute) {
        bar = ko.unwrap(bar);
        xAbsoluteValue = ko.unwrap(xAbsolute);
        var node1Position = HELPERS.toVector3(bar.node1().position());
        var node2Position = HELPERS.toVector3(bar.node2().position());
        var barLength = new THREE.Vector3().subVectors(node2Position, node1Position).length();

        var relativeValue = xAbsoluteValue / barLength * 100;
        if (ko.isObservable(xAbsolute)) { xAbsolute(relativeValue); return xAbsolute; };
        return relativeValue;
    },
    toAbsolute: function (bar, xRelative) {
        bar = ko.unwrap(bar);
        xAbsoluteValue = ko.unwrap(xRelative);
        var node1Position = HELPERS.toVector3(bar.node1().position());
        var node2Position = HELPERS.toVector3(bar.node2().position());
        var barLength = new THREE.Vector3().subVectors(node2Position, node1Position).length();

        var relativeValue =  barLength * (xAbsoluteValue / 100);
        if (ko.isObservable(xRelative)) { xRelative(relativeValue); return xRelative; };
        return relativeValue;
    },
    getBarLength: function (bar) {
        if (isEmpty(bar))
            throw new Error("HELPERS.getBarLength: bar is empty");
        bar = ko.unwrap(bar);
        var node1Position = HELPERS.toVector3(bar.node1().position());
        var node2Position = HELPERS.toVector3(bar.node2().position());
        var barLength = new THREE.Vector3().subVectors(node2Position, node1Position).length();

        return barLength;
    },
    formatNumber: function (value) {   // 123456789.12345 => 12.34e+7 (max 8 digits) OR => 12e+7 (max 5 digits)

        if (isEmpty(value)) return value;
        return value.toFixed(2);       // THIS FUNCTION IS TODO

        var output;
        var valueStr = value.toString();
        var maxDigitCount = 5;

        var charsLeft = maxDigitCount;

        var pointPosition = valueStr.indexOf(".");
        var decimalPartLength;
        var fractionalPartLength;

        if (pointPosition !== -1) {
            decimalPartLength = pointPosition;
            fractionalPartLength = valueStr.length - decimalPartLength - 1;
        }
        else {
            decimalPartLength = valueStr.length;
            fractionalPartLength = 0;
        }

        if (pointPosition !== -1 || valueStr.length < maxDigitCount)
            charsLeft--;    // "." is in output

        charsLeft -= decimalPartLength;

        if (charsLeft === 0) {
            output = valueStr;
        }
        else if (charsLeft > 0) {
            output = value.toFixed(charsLeft);
        }
        else {  // charsLeft < 0
            return null;        // TODO this part
        }


        if (output.length !== maxDigitCount) console.info("Not equal!");
        return output;

        var valueToFixed = value.toFixed();
        var valueToFixedStr = valueToFixed.toString();

        if (valueToFixedStr.length > maxDigitCount) {   // need to truncate the number

            var charsLeft = maxDigitCount;  // start at provided max count

            charsLeft -= 2; // reserved for "e[sign]"

        }
        else
            return valueToFixed;

    },

    disposeAll: function (observable) {
        var beforeDisposeLength;
        while (observable._subscriptions.beforeChange.length > 0) {
            beforeDisposeLength = observable._subscriptions.beforeChange.length;
            if (!observable._subscriptions.beforeChange[0].isDisposed)
                observable._subscriptions.beforeChange[0].dispose();
            if (beforeDisposeLength === observable._subscriptions.beforeChange.length) { console.info("HELPERS.disposeAll: beforeDisposeLength hasn't diminished, aborting..."); break; }
        }
        while (observable._subscriptions.change.length > 0) {
            beforeDisposeLength = observable._subscriptions.change.length;
            if (!observable._subscriptions.change[0].isDisposed)
                observable._subscriptions.change[0].dispose();
            if (beforeDisposeLength === observable._subscriptions.change.length) { console.info("HELPERS.disposeAll: beforeDisposeLength hasn't diminished, aborting..."); break; }
        }
    },
    safeDispose: function(disposable) {
        if(!isEmpty(disposable)) {
            disposable.dispose();
        }
        return null;
    },

    createFormFromDiv: function ($formDiv) {

        var $form = $("<form id='pdfGeneratorForm'></form>");

        var attributes = $formDiv.prop("attributes");
        var children = $formDiv.children().clone();
        // loop through <div> attributes and apply them on <form>
        $.each(attributes, function () {
            $form.attr(this.name, this.value);
        });
        // copy all children
        $.each(children, function () {
            $form.append(this);
        });

        return $form[0];
    }
}
ko.subscribable.fn.subscribeChanged = function(callback) {
    var oldValue;
    var beforSubscription = this.subscribe(function(_oldValue) {
        oldValue = _oldValue;
    }, this, 'beforeChange');
    var afterSubscription = this.subscribe(function(newValue) {
        callback(newValue, oldValue);
    });
    return {
        dispose: function() {
            beforSubscription.dispose();
            afterSubscription.dispose();
        }
    };
};
var subscriptionsField;
switch (ko.DEBUG || ko.version) {
case true:
    subscriptionsField = '_subscriptions';
    break;
case "3.0.0":
    subscriptionsField = 'F';
    break;
case "3.1.0":
    subscriptionsField = 'H';
    break;
case "3.2.0":
    subscriptionsField = 'M';
    break;
case "3.3.0":
    subscriptionsField = 'G';
    break;
case "3.4.0":
    subscriptionsField = 'K';
    break;
default:
    throw new Error("Unsupported Knockout version. Only v3.0.0 to v3.4.0 are supported when minified. Current version is " + ko.version);
}
ko.watch = function (subscribable, withChilren, callback, unwatch, userData) {
    if (typeof subscribable === 'undefined') throw new Error("Requested to watch non-existing property!");
    if (typeof subscribable['notifySubscribers'] == 'function') {
        // observable or observableArray
        var isArray = typeof subscribable.pop === 'function';
        var i;
        if (isArray) {
            //observableArray
            if (withChilren) {
                for (i = 0; i < subscribable().length; i++) {
                    ko.watch(subscribable()[i], true, callback, unwatch, userData);
                }
            }
            if (!unwatch) {
                subscribable.subscribe(function(changes) {
                    if (withChilren) {
                        ko.utils.arrayForEach(changes, function(item) {
                            if (item.status === 'added') {
                                ko.watch(item.value, true, callback, false, userData);
                            }
                            if (item.status === 'deleted') {
                                ko.watch(item.value, true, callback, true, userData);
                            }
                        });
                    }
                    callback(userData);
                }, undefined, 'arrayChange')._koWatch = true;
            }
        } else {
            //observable
            subscribable.subscribe(function() {
                callback(userData);
            })._koWatch = true;
        }
        if (unwatch) {
            var subsc = subscribable[subscriptionsField];
            if (subsc) {
                if (subsc.change) {
                    for (i = subsc.change.length - 1; i >= 0; i--) {
                        if (subsc.change[i]._koWatch === true) {
                            subsc.change[i].dispose();
                        }
                    }
                }
            }
        }
    }
    if (withChilren) {
        var unwrappedSubscribable = ko.unwrap(subscribable);
        if (typeof unwrappedSubscribable['watchModel'] == 'function') {
            //MODEL object (like Node or Bar)
            unwrappedSubscribable.watchModel(callback, unwatch, userData);
        }
    }
}