﻿var MODEL = MODEL || {};
var barsCount = 0;

MODEL.Bar = function(node1, node2, gamma, profileId) {
    this.type = MODEL.OBJECT_TYPE.Bar;
    this.id = 0;
    this.name = ko.observable(translation.BAR + ' ' + (barsCount + 1));
    this.node1 = ko.observable(node1);
    this.node2 = ko.observable(node2);
    this.gamma = ko.observable(0);
    this.profileId = ko.observable(profileId);     
    this.selected = ko.observable(false);
    this.isProfileCircle = ko.observable(false);
    this.isGammaAuto = ko.observable(false);
    this.offset = ko.observable(false);
    //details
    this.attachedNodes = ko.observableArray();  
    this.alternativeProfiles = ko.observableArray();
    this.alternativeRotations = ko.observableArray();
    this.modifyableLength = ko.observable(false);
    this.modifyableRotation = ko.observable(false);
};
MODEL.Bar.prototype = {
    constructor: MODEL.Bar,
    watchModel: function(callback, unwatch) {
        ko.watch(this.node1, false, callback, unwatch);
        ko.watch(this.node2, false, callback, unwatch);
        ko.watch(this.gamma, false, callback, unwatch);
        ko.watch(this.profileId, false, callback, unwatch);
        ko.watch(this.isGammaAuto, false, callback, unwatch);
        ko.watch(this.offset, false, callback, unwatch);
    },
    save: function() {

        var alternativeProfiles = [];
        for (var i = 0; i < this.alternativeProfiles().length; i++) {
            alternativeProfiles[i] = this.alternativeProfiles()[i];
        }

        var alternativeRotations = [];
        for (var i = 0; i < this.alternativeRotations().length; i++) {
            alternativeRotations[i] = this.alternativeRotations()[i];
        }

        var attachedNodesId = [];
        for (var i = 0; i < this.attachedNodes().length; i++) {
            attachedNodesId[i] = this.attachedNodes()[i].id;
        }

        var details = {
            modifyableLength: this.modifyableLength(),
            modifyableRotation: this.modifyableRotation(),
            alternativeProfiles,
            alternativeRotations,
            attachedNodesId
        };

        return {
            id: this.id,
            name: this.name(),
            node1Id: this.node1().id,
            node2Id: this.node2().id,
            gamma: this.gamma(),
            profileId: this.profileId(),
            isGammaAuto: false,
            details: details
        }
    },
    load: function(object, construction) {
        this.id = object.id;
        this.name(object.name);
        this.node1(construction.getNodeById(object.node1Id));
        this.node2(construction.getNodeById(object.node2Id));
        this.gamma(object.gamma);
        this.profileId(object.profileId);
        this.isGammaAuto(false);

        // details
        this.modifyableLength(object.details.modifyableLength);
        this.modifyableRotation(object.details.modifyableRotation);
        for (var i = 0; i < object.details.alternativeProfiles.length; i++) {
            this.alternativeProfiles.push(object.details.alternativeProfiles[i]);
        }
        for (var i = 0; i < object.details.alternativeRotations.length; i++) {
            this.alternativeRotations.push(object.details.alternativeRotations[i]);
        }
        for (var i = 0; i < object.details.attachedNodesId.length; i++) {
            var nodeId = object.details.attachedNodesId[i];
            var node = construction.getNodeById(nodeId);
            if (!isEmpty(node)) {
                this.attachedNodes.push(node);
            } else {
                console.error("Attached node [" + object.attachedNodesId[i] + "] does not exists.");
            }

        }
        // this.offset(object.constrainsInfo.isOffset);
    },
    vectorProjection: function(vec) {
        var p = this.node1().position();
        var pD = this.node2().position().sub(p);
        var nD = vec.sub(p);
        var a = pD.dot(pD);
        var b = nD.dot(pD);
        var s = b / a;
        if (s < 0 || s > 1) {
            if (s < 0) {
                return this.node1().position();
            } else { //s > 1
                return this.node2().position();
            }
        }
        return p.add(pD.multiplyScalar(s));
    },
    barIntersection: function(bar) {
        var p0 = this.node1().position();
        var p0D = this.node2().position().sub(p0);
        var p1 = bar.node1().position();
        var p1D = bar.node2().position().sub(p1);
        var w0 = p0.sub(p1);
        var a = p0D.dot(p0D);
        var b = p0D.dot(p1D);
        var c = p1D.dot(p1D);
        var d = p0D.dot(w0);
        var e = p1D.dot(w0);
        var sc = (b * e - c * d) / (a * c - b * b);
        var tc = (a * e - b * d) / (a * c - b * b);
        p0 = p0.add(p0D.multiplyScalar(sc));
        p1 = p1.add(p1D.multiplyScalar(tc));
        if (p0.squaredDistanceTo(p1) < 0.01 * 0.01) {
            return p0.add(p1).multiplyScalar(0.5);
        }
        return null; //bars are too far
    },
    getLength: function() {
        var startPosition = this.node1().position();
        var endPosition = this.node2().position();
        return startPosition.distanceTo(endPosition);
    }
};