﻿var MODEL = MODEL || {};
// objects - array with objects with barAttachments, objectTypes - optional (process with given types only)
MODEL.BarAttachmentsController = function (objects, objectTypes) {

    var onBarAttachmentRemoved = function (barAttachment, index, object) {
        if (!barAttachment.bar) {
            console.error("MODEL.BarAttachmentsController.onBarAttachmentRemoved: barAttachment.bar is empty");
            return;
        }
        object.subs[index][0].dispose();
        object.subs[index][1].dispose();
        object.subs[index][2].dispose();
        object.subs[index][3].dispose();
    }

    // limit x value from top (barLength) and bottom (0)
    var validateBarAttachment = function (barAttachment, object) {
        if (isEmpty(barAttachment.bar)) {
            console.error("MODEL.BarAttachmentsController.validateBarAttachment: barAttachment.bar is empty");
        }
        var barLength = HELPERS.getBarLength(barAttachment.bar);

        var point1 = 0;
        var point2 = 0;
        if (object.points) {
            if (object.points()[0])
                point1 = object.points()[0]().distance;
            if (object.points()[1])
                point2 = object.points()[1]().distance;


        }

        if (barAttachment.x() > barLength || point1 > barLength || point2 > barLength) {   // mild
            barAttachment.x(barLength);
        }
        if (barAttachment.x() < 0) {
            barAttachment.x(0);
        }
    };


    // delete attachment when x > barLength
    var validateAggressiveBarAttachment = function (barAttachment, object) {
        var barLength = HELPERS.getBarLength(barAttachment.bar);

        var point1 = 0;
        var point2 = 0;
        if (object.points) {        // if trapezoidal

            if (object.points()[0])
                point1 = object.points()[0]().distance;
            if (object.points()[1])
                point2 = object.points()[1]().distance;

            if (point1 > barLength && point2 > barLength) {      // aggresive, delete attachment when x > barLength
                aggresiveRemove(barAttachment, object);
                return;
            }

            if (point1 > barLength) {                            // mild
                object.points()[0]().distance = barLength;
                object.points()[0].valueHasMutated();
            }

            if (point2 > barLength) {                            // mild
                object.points()[1]().distance = barLength;
                object.points()[1].valueHasMutated();
            }

            return;
        }

        if (barAttachment.x() > barLength) {
            aggresiveRemove(barAttachment, object);
        }
        if (barAttachment.x() < 0) {
            barAttachment.x(0);
        }
    };

    var aggresiveRemove = function (barAttachment, object) {
        object.removeAssignment(barAttachment.bar, barAttachment.x());
        object.remove();
    }


    var onBarPositionChanged = function (barAttachment, oldValue, object) {
        validateAggressiveBarAttachment(barAttachment, object);
    }

    var onBarAttachmentXChanged = function (barAttachment, oldValue, object) {
        validateBarAttachment(barAttachment, object);
    }

    var onBarAttachmentAdded = function (barAttachment, index, object) {
        if (!barAttachment.bar) {
            console.error("MODEL.BarAttachmentsController.onBarAttachmentAdded: barAttachment.bar is empty");
            return;
        }
        object.subs[index] = [];
        object.subs[index][0] = HELPERS.subscribePropertyChange(this, barAttachment.bar.node1().position, "barAttachment.bar.node1.position", onBarPositionChanged, barAttachment, object);
        object.subs[index][1] = HELPERS.subscribePropertyChange(this, barAttachment.bar.node2().position, "barAttachment.bar.node2.position", onBarPositionChanged, barAttachment, object);
        object.subs[index][2] = HELPERS.subscribePropertyChange(this, barAttachment.x, "barAttachment.x", onBarAttachmentXChanged, barAttachment, object);
        object.subs[index][3] = HELPERS.subscribePropertyChange(this, barAttachment.x, "barAttachment.x", onBarAttachmentXChanged, barAttachment, object);
    }

    var i;
    var onObjectAdded = function (object) {
        if (!isEmpty(objectTypes) && objectTypes.indexOf(object.type) === -1) {
            return;
        }
        object.subs = [];
        object.subs[-1] = HELPERS.subscribeArrayChange(this, object.barAttachments, "objects.barAttachments", onBarAttachmentAdded, onBarAttachmentRemoved, object);

        var barAttachments = object.barAttachments();
        for (var j = 0, jj = barAttachments.length; j < jj; j++) {
            if (!isEmpty(barAttachments[j].bar)) {
                validateBarAttachment(barAttachments[j], j, object);
                onBarAttachmentAdded(barAttachments[j], j, object);
            } else {
                console.error("MODEL.BarAttachmentsController.onObjectAdded: " + object.type + "[" + i + "].barAttachment[" + j + "].bar is empty");
            }
        }
    }

    var onObjectRemoved = function (object) {
        if (!isEmpty(objectTypes) && objectTypes.indexOf(object.type) === -1) {
            return;
        }
        object.subs[-1].dispose();
        for (i = 0; i < object.subs.length; i++) {
            for (var j = 0; j < object.subs[i].length; j++) {
                if (typeof object.subs[i][j].dispose !== 'function') console.error("MODEL.BarAttachmentsController: subs[object.id][i][j].dispose doesn't exist!");
                object.subs[i][j].dispose();
            }
        }
        object.subs = [];
    }
    // init
    HELPERS.subscribeArrayChange(this, objects, "loads", onObjectAdded, onObjectRemoved);

    for (i = 0; i < objects().length; i++) {
        onObjectAdded(objects()[i]);
    }
}

// synchronize xAbsolute and xRelative
MODEL.BarAttachmentsController.SyncX = function (xAbsolute, xRelative, bar, isAbsoluteNow) {
    this.barAttachmentSubs = [];
    var subs = [];
    var disable = false;
    subs.push(HELPERS.subscribePropertyChange(this, xRelative, "xRelative", function () {
        if (!isEmpty(bar)) {
            if (isEmpty(xRelative)) {
                console.error("MODEL.BarAttachmentsController.SyncX: passed xRelative is empty");
            }
            var xAbsoluteNew = convertX(xRelative, null, true, HELPERS.getBarLength(bar));
            if (!disable) {
                disable = true;
                xAbsolute(xAbsoluteNew);
                disable = false;
            }
        }
    }));
    subs.push(HELPERS.subscribePropertyChange(this, xAbsolute, "xAbsolute", function () {
        if (!isEmpty(bar)) {
            if (isEmpty(xAbsolute)) {
                console.error("MODEL.BarAttachmentsController.SyncX: passed xAbsolute is empty");
            }
            var xRelativeNew = convertX(xAbsolute, null, false, HELPERS.getBarLength(bar));

            if (!disable) {
                disable = true;
                xRelative(xRelativeNew);
                disable = false;
            }
        }
    }));
    // convert also when subscribing
    if (!isEmpty(bar)) {
        if (isAbsoluteNow) {
            var xRelativeNew = convertX(xAbsolute, null, false, HELPERS.getBarLength(bar));
            if (!disable) {
                disable = true;
                xRelative(xRelativeNew);
                disable = false;
            }
        }
        else {
            var xAbsoluteNew = convertX(xRelative, null, true, HELPERS.getBarLength(bar));
            if (!disable) {
                disable = true;
                xAbsolute(xAbsoluteNew);
                disable = false;
            }
        }
    }
    return subs;
}