﻿var MODEL = MODEL || {};

MODEL.Constraints = function(construction) {
    this.data = ko.observable(null);
    this.constraintsApplied = ko.observable();
    this.construction = construction;
    this.clearConstraints = function() {
            // used in undo/redo
            var data = this.data();
            for (var property in data) {
                // clear all arrays
                if (data.hasOwnProperty(property)) {
                    if (Array.isArray(data[property]))
                        data[property] = [];
                    else if (typeof data[property] === 'object')
                        data[property] = {};
                }
            }
        }
        // BAR: Main
    this.getMemberConstraints = function(barId) {
            if (isEmpty(this.data())) {
                return null;
            }
            return this.data().MemberConstraints[barId];
        }
        // BAR: Profile
    this.getProfilesForBar = function(barId) {
        var result = [];
        if (this.data() != null) {
            var MemberConstraints = this.data().MemberConstraints[barId];
            // not yet loaded
            if (MemberConstraints) {
                for (var i = 0; i < MemberConstraints.profiles.length; i++) {
                    var profile = MemberConstraints.profiles[i];
                    if (!isEmpty(profile)) {
                        result.push(profile);
                    } else {
                        console.error("Profile from constraints does not exist in system. Profile id: " +
                        MemberConstraints.profiles[i]);
                    }
                }
            }
        }
        return result;
    }
    this.getBarProfileName = function(barId, profileId) {
        if (isEmpty(this.data().MemberConstraints[barId])) {
            return -1;
        }
        var list = this.data().MemberConstraints[barId].profiles;
        for (var i = 0; i < list.length; i++) {
            if (list[i].elementId === profileId) {
                return list[i].elementName;
            }
        }
        return "error";
    }
    this.getBarProfileModelId = function(barId, profileId) {
        if (isEmpty(this.data()) || isEmpty(this.data().MemberConstraints[barId])) {
            return -1;
        }
        var list = this.data().MemberConstraints[barId].profiles;
        for (var i = 0; i < list.length; i++) {
            if (list[i].elementId === profileId) {
                return list[i].model3DId;
            }
        }
        return -1;
    }
    this.getBarProfile = function(barId, profileId) {
            if (isEmpty(this.data().MemberConstraints[barId])) {
                return -1;
            }
            var list = this.data().MemberConstraints[barId].profiles;
            for (var i = 0; i < list.length; i++) {
                if (list[i].elementId === profileId) {
                    return list[i];
                }
            }
            return -1;
        }
        // NODE: Main
    this.getNodeConstraints = function(nodeId) {
            if (isEmpty(this.data()) || nodeId === 0) {
                return null;
            }
            return this.data().nodeConstraints[nodeId];
        }
        // NODE: Connector
    this.getConnectorsForNode = function(nodeId) {
        var result = [];
        if (this.data() != null) {
            var nodeConstraints = this.data().nodeConstraints[nodeId];
            // not yet loaded
            if (nodeConstraints) {
                for (var i = 0; i < nodeConstraints.connectors.length; i++) {
                    var connector = nodeConstraints.connectors[i];
                    if (!isEmpty(connector)) {
                        result.push(connector);
                    } else {
                        console.error("Connector from constraints does not exist in system. Connector id: " +
                            nodeConstraints.connectors[i]);
                    }
                }
            }
        }
        return result;
    }
    this.getNodeConnectorName = function(nodeId, connectorId) {
        if (isEmpty(this.data().nodeConstraints[nodeId])) {
            return -1;
        }
        var list = this.data().nodeConstraints[nodeId].connectors;
        for (var i = 0; i < list.length; i++) {
            if (list[i].elementId === connectorId) {
                return list[i].elementName;
            }
        }
        return "error";
    }
    this.getConnectorModelIds = function(nodeId, connectorId) {
        var connector = this.getConnector(nodeId, connectorId, "Node");
        if (isEmpty(connector)) {
            return [];
        }
        if (this.data().nodeConstraints[nodeId].isDouble) {
            return [connector.model3DId, connector.model3DId_d];
        } else {
            return [connector.model3DId];
        }
    }
    this.checkIfConnecotrsAreFast = function(nodeId) {
            if (this.data() != null) {
                var nodeConstraints = this.data().nodeConstraints[nodeId];
                // not yet loaded
                if (nodeConstraints) {
                    return nodeConstraints.isNormalAndFast;
                }
            }
            return null;
        }
        // CONNECTOR: Node
    this.getConnector = function(elementId, connectorId, elementType) {
            var i, list;
            if (isEmpty(this.data().nodeConstraints[elementId])) {
                return null;
            }
            list = this.data().nodeConstraints[elementId].connectors;
            if(list === null) return null;
            for (i = 0; i < list.length; i++) {
                if (list[i].elementId === connectorId) {
                    return list[i];
                }
            }
            return null;
        }
        // SUPPORT: MAIN
    this.getNodeSupportModelId = function(node) {
        var nodeConstraints = this.data().nodeConstraints[node.id];
        if (isEmpty(this.data().nodeConstraints[node.id])) {
            return -1;
        }
        var supportId = node.supportId();
        for (var i = 0; i < nodeConstraints.supports.length; i++) {
            var supportConstraints = nodeConstraints.supports[i];
            if (supportConstraints.elementId === supportId) {
                return supportConstraints.model3DId;
            }
        }
        return -1;
    }

    // OTHER
    var containsElement = function(container, elementId) {
        for (var i = 0; i < container.length; i++) {
            if (container[i].elementId === elementId) {
                return true;
            }
        }
        return false;
    }
    var getElementFromList = function(list, wrappedId) {
        var id = ko.unwrap(wrappedId);
        for (var i = 0, ii = list.length; i < ii; i++) {
            if (list[i].elementId === id)
                return list[i];
        }
        return null;
    }
    this.applyConstraintsToModel = function(constrains) {
        CORE.autoSaveDisabled = true;
        if (!isEmpty(constrains)) {
            var i, j, defaultModelId, defaultProfilesId, defaultConnectorsId;
            if (!CORE.disableConstraints) {
                var isFast;
                var supportOk = false;
                for (i = 0; i < this.construction.nodes().length; i++) {
                    var node = this.construction.nodes()[i];
                    var nodeConstraints = constrains.nodeConstraints[node.id];
                    // data for this node exists
                    if (!isEmpty(nodeConstraints) && nodeConstraints.orientation !== null) {
                        node.orientation.baseX = new MODEL.types
                            .Vector(nodeConstraints.orientation.baseX.x,
                                nodeConstraints.orientation.baseX.y,
                                nodeConstraints.orientation.baseX.z);
                        node.orientation.baseY = new MODEL.types
                            .Vector(nodeConstraints.orientation.baseY.x,
                                nodeConstraints.orientation.baseY.y,
                                nodeConstraints.orientation.baseY.z);
                        node.orientation.baseZ = new MODEL.types
                            .Vector(nodeConstraints.orientation.baseZ.x,
                                nodeConstraints.orientation.baseZ.y,
                                nodeConstraints.orientation.baseZ.z);
                        // connectors exist
                        if (nodeConstraints.connectors.length > 0) {
                            // node.showFastOptions(nodeConstraints.isNormalAndFast);

                            if ((isEmpty(node.connectorId()) ||
                                    !containsElement(nodeConstraints.connectors, node.connectorId()))) {
                                var defaultConnectorId = nodeConstraints.connectors[0].elementId;
                                // isFast = nodeConstraints.connectors[0].IsFast;
                                if (!isEmpty(defaultConnectorId)) {
                                    //assign default connector if nothimg was selected
                                    //or previously selected connector do not fit any more
                                    node.connectorId(defaultConnectorId);
                                    // node.isFast(isFast);
                                } else {
                                    node.connectorId(0);
                                }
                            }
                        } else {
                            //no connector ware found
                            node.connectorId(0);
                        }
                    }
                }
                for (i = 0; i < this.construction.bars().length; i++) {
                    var bar = this.construction.bars()[i];
                    var MemberConstraints = constrains.MemberConstraints[bar.id];
                    if (!isEmpty(MemberConstraints)) {
                        if (bar.isGammaAuto()) {
                            bar.gamma(MemberConstraints.gamma);
                        }

                        if (constrains.constructionData) {
                            for (z = 0; z < constrains.constructionData.bars.length; z++) {
                                var barModel = constrains.constructionData.bars[z];
                                if(barModel.id === bar.id) {
                                    bar.profileId(barModel.profileId);
                                }
                            }
                        }

                        if (isEmpty(bar.profileId()) || !containsElement(MemberConstraints.profiles, bar.profileId())) {
                            defaultProfilesId = MemberConstraints.profiles.length > 0 ? MemberConstraints.profiles[0] : null;
                            if (!isEmpty(defaultProfilesId)) {
                                //assign default connector if nothimg was selected 
                                //or previously selected connector do not fit any more
                                bar.profileId(defaultProfilesId.elementId);
                                bar.isProfileCircle(defaultProfilesId.IsCircle);
                            } else {
                                bar.profileId(0);
                                bar.isProfileCircle(false);
                            }
                        }
                    }
                }

                if (constrains.constructionData) {
                    CORE.model.connectors([])
                    for (i = 0; i < constrains.constructionData.connectors.length; i++) {
                        var newConnector = new MODEL.Connector();
                        newConnector.load(constrains.constructionData.connectors[i]);
                        CORE.model.connectors.push(newConnector);
                    }
                }
            }
            CORE.model.constraints.constraintsApplied.valueHasMutated();
        }
        setTimeout(function() {
                CORE.autoSaveDisabled = false;
                CORE.constructionUndoRedo.addToHistory();
            },
            1);
        CORE.disableConstraints = false;
    }
    HELPERS.subscribePropertyChange(this, this.data, "MODEL.Constraints.data", this.applyConstraintsToModel);
}