﻿var MODEL = MODEL || {};

define(['js/core/CORE.webApiManager', 'js/model/MODEL.Units'],
    function() {
        MODEL.ConstructionViewModel = function() {
            return {
                activeWorkplane: ko.observable(null),
                selectedObjects: ko.observableArray(),
                definingObject: ko.observable(null), //object that is currently beeing defined
                modelChange: ko.observable(), // event
                systemPreferences: ko.observable(new MODEL.SystemPreferences()),
                validatorSubs: [],
                // MODELS ARRAYS
                loadCases: ko.observableArray(),
                workplanes: ko.observableArray(),
                supports: ko.observableArray(),
                nodes: ko.observableArray(),
                bars: ko.observableArray(),
                connectors: ko.observableArray(),
                loads: ko.observableArray(),
                fills: ko.observableArray(),
                boxes: ko.observableArray(),
                barsResults: ko.observable(null),
                nodesResults: ko.observable(null),
                missingElements: ko.observableArray(),
                forceLoads: ko.observableArray(),
                trapezoidalUniformLoads: ko.observableArray(),
                externalModels : ko.observableArray(),
                // FUNCTIONS
                waitingForResponse: ko.observable(false),
                pendingSave: null,
                saveMode: null,
                autoSave: function() {
                    var setAutoSave = function(error, isNewJob) {
                        if (error) {
                            console.error("LOAD error, autosave disabled.");
                        }
                        // setTimout prevents saving on the beginning
                        setTimeout(function() {

                            CORE.initView();
                            // save on change
                            CORE.model.modelChange.subscribe(function(mode) {

                                try {
                                    if (mode === SAVE_MODE.NoConstraints && CORE.options.demoMode) return;

                                    if (!CORE.autoSaveDisabled) {
                                        var proto = CORE.model.save();
                                        CORE.model.sendProto(proto, mode);
                                    }
                                    if (CORE.jobVersionLoaded) {
                                        CORE.jobVersionLoaded = false;
                                    } else {
                                        CORE.currentJobVersionId(0);
                                    }

                                } catch (exception) {
                                    if (!CORE.viewerReloading)
                                        throw exception;
                                }

                            });

                            if (isNewJob) {
                                CORE.model.modelChange.notifySubscribers();
                            }
                        }, 0);

                    };
                    if (!CORE.options.runLocally && !CORE.options.disableLoadAndSave) {
                        CORE.model.load(CORE.options.appId,
                            CORE.options.projId,
                            CORE.options.jobId,
                            null,
                            setAutoSave,
                            setAutoSave);
                    } else {
                        CORE.initView();
                    }
                },
                resetCounters: function() {
                    nodesCount = 0;
                    barsCount = 0;
                    nodeSupportsCount = 0;
                    MODEL.load.node.nodeForceCount = 0;
                    MODEL.load.node.nodeMomentCount = 0;
                    MODEL.load.bar.barForceCount = 0;
                    MODEL.load.bar.barUniformCount = 0;
                    MODEL.load.bar.barTrapezoidalCount = 0;
                },
                removeAll: function() {
                    var model = CORE.model;

                    this.forceLoads([]);
                    this.trapezoidalUniformLoads([]);
                    this.connectors([]);
                    // order is important
                    var arrayNames = [
                        "loads", "bars", "nodes", "fills", "boxes"
                    ];
                    for (var i = 0; i < arrayNames.length; i++) {
                        var arrayName = arrayNames[i];
                        var unwrappedArray = ko.unwrap(model[arrayName]);
                        while (unwrappedArray.length > 0) {
                            var object = unwrappedArray[0];
                            object.remove();
                        }
                    }
                },
                isActiveWorkplane: function(workplane) {
                    return workplane === this.activeWorkplane();
                },
                getResultsForBar: function(id) {
                    for (var i = 0; i < this.barsResults().length; i++) {
                        if (this.barsResults()[i].bar.id === ko.unwrap(id)) {
                            return this.barsResults()[i].data.values;
                        }
                    }
                    return null;
                },
                getResultsForNode: function(id) {
                    for (var i = 0; i < this.nodesResults().length; i++) {
                        if (this.nodesResults()[i].node.id === ko.unwrap(id)) {
                            return this.nodesResults()[i].data.values;
                        }
                    }
                    return null;
                },

                getAllObjects: function() {
                    var selectedObjects = [];
                    var i;
                    for (i = 0; i < this.nodes().length; i++) {
                        selectedObjects.push(this.nodes()[i]);
                    }
                    for (i = 0; i < this.bars().length; i++) {
                        selectedObjects.push(this.bars()[i]);
                    }
                    for (i = 0; i < this.loads().length; i++) {
                        selectedObjects.push(this.loads()[i]);
                    }
                    for (i = 0; i < this.supports().length; i++) {
                        selectedObjects.push(this.supports()[i]);
                    }
                    for (i = 0; i < this.fills().length; i++) {
                        selectedObjects.push(this.fills()[i]);
                    }
                    for (i = 0; i < this.boxes().length; i++) {
                        selectedObjects.push(this.boxes()[i]);
                    }
                    for (i = 0; i < this.externalModels().length; i++) {
                        selectedObjects.push(this.externalModels()[i]);
                    }
                    return selectedObjects;
                },
                getSelectedObjectsFromArray: function(modelArray) {
                    modelArray = ko.unwrap(modelArray);
                    var selectedObjects = [];
                    for (var i = 0, ii = modelArray.length; i < ii; i++) {
                        if (modelArray[i].selected()) {
                            selectedObjects.push(modelArray[i]);
                        }
                    }
                    return selectedObjects;
                },
                removeObjects: function(wrappedObjects) {
                    var objects = ko.unwrap(wrappedObjects);
                    var supportsCount = 0;
                    while (objects.length > supportsCount) {
                        var objectType = objects[supportsCount].type;
                        if (objectType !== MODEL.OBJECT_TYPE.NodeSupport && objectType !== MODEL.OBJECT_TYPE.Bar && objectType !== MODEL.OBJECT_TYPE.Node) {
                            // common interface
                            objects[supportsCount].remove();
                        } else {
                            supportsCount++;
                        }
                    }
                    if (this.nodes().length === 0) {
                        CORE.view3D.cameraControl.zoomAll();
                    }
                },
                removeSelectedObjects: function() {
                    this.removeObjects(this.selectedObjects);
                },
                onObjectSelectedChanged: function(object, oldValue, customThis) {
                    var self = this;
                    if (!isEmpty(customThis)) {
                        self = customThis;
                    }
                    if (object.selected()) {
                        addElementToSet(self.selectedObjects, object);
                    } else {
                        removeElementFromSet(self.selectedObjects, object);
                    }
                },
                selectAll: function() {
                    var allObjects = this.getAllObjects();
                    for (var i = 0; i < allObjects.length; i++) {
                        if (typeof allObjects[i].selected === 'function') {
                            allObjects[i].selected(true);
                        }
                    }
                },
                deselectAll: function() {
                    var allObjects = this.getAllObjects();
                    for (var i = 0; i < allObjects.length; i++) {
                        if (typeof allObjects[i].selected === 'function') {
                            allObjects[i].selected(false);
                        }
                    }
                },
                // WORKPLANES
                addWorkplane: function(workplane, modelInstance) {
                    if (workplane.type !== MODEL.OBJECT_TYPE.Workplane) {
                        throw new Error("MODEL.addWorkplane: tried to add a non-workplane object to the workplanes array");
                    }
                    var self = this;
                    if (!isEmpty(modelInstance)) {
                        self = modelInstance;
                    }
                    var newWorkplane = addElementToSet(self.workplanes, workplane);
                    workplanesCount++;
                    newWorkplane.remove = function() {
                        self.removeWorkplane(newWorkplane);
                    };
                    return newWorkplane;
                },
                removeWorkplane: function(workplane) {
                    if (this.activeWorkplane() === workplane) {
                        this.activeWorkplane(null);
                    }
                    removeElementFromSet(this.workplanes, workplane);
                },
                // LoadCases
                addLoadCase: function(loadCase, modelInstance) {
                    if (loadCase.type !== MODEL.OBJECT_TYPE.LoadCase) {
                        throw new Error("MODEL.addLoadCase: tried to add a non-load case object to the loadcases array");
                    }
                    var self = this;
                    if (!isEmpty(modelInstance)) {
                        self = modelInstance;
                    }
                    var newLoadCase = addElementToSet(self.loadCases, loadCase);
                    loadCasesCount++;
                    newLoadCase.remove = function() {
                        self.removeLoadCase(newLoadCase);
                    };
                    return newLoadCase;
                },
                removeLoadCase: function(loadCase) {
                    // if (this.activeWorkplane() === loadCase) {
                    //     this.activeWorkplane(null);
                    // }
                    removeElementFromSet(this.loadCases, loadCase);
                },

                // NODES
                nodeMerge: null,
                addNode: function(node) {
                    if (node.type !== MODEL.OBJECT_TYPE.Node) {
                        throw new Error("MODEL.addNode: tried to add a non-node object to the nodes array");
                    }
                    if (node.id === 0) {
                        nodesCount++;
                        node.id = nodesCount;
                        node.name("Node " + nodesCount);
                    } else {
                        nodesCount = Math.max(nodesCount, node.id);
                    }
                    if (node.id > nodesCount) {
                        nodesCount = node.id;
                    }
                    if (node.name() === "" || node.name() === "Node 0") {
                        node.name("Node " + nodesCount);
                    }
                    if (!isEmpty(this.nodeMerge)) {
                        var existing = this.nodeMerge(node);
                        if (!isEmpty(existing)) {
                            console.info("Node " + node.id + " merged with node " + existing.id);
                            return existing;
                        }
                    }
                    addElementToSet(this.nodes, node);
                    var self = this;
                    node.remove = function() { self.removeNode(node); };
                    HELPERS.subscribePropertyChange(this, node.selected, "Node", this.onObjectSelectedChanged, node);
                    return node;
                },
                removeNode: function(node) {
                    if (!isElementInSet(this.nodes, node)) {
                        throw new Error("MODEL.removeNode: tried to remove node which is not in the nodes array");
                    }
                    var createsBars = node.createsBars();
                    var i;
                    while (createsBars.length > 0) {
                        // bar will remove itself from createsBars list
                        this.removeBar(createsBars[0]);
                    }
                    if (typeof node.supportId === 'function' && !isEmpty(node.supportId())) {
                        for (i = 0; i < CORE.model.supports().length; i++) {
                            if (CORE.model.supports()[i].id === node.supportId()) {
                                CORE.model.supports()[i].removeAssignment(node);
                            }
                        }
                    }
                    if (node.selected() && isElementInSet(this.selectedObjects, node)) {
                        removeElementFromSet(this.selectedObjects, node);
                    }
                    if (isElementInSet(this.nodes, node)) {
                        removeElementFromSet(this.nodes, node);
                    }
                },
                mergeNodes: function(targetNode, nodeToRemove) {
                    if (targetNode.id !== nodeToRemove.id) {
                        var createsBars = nodeToRemove.createsBars();
                        var i;
                        for (i = 0; i < createsBars.length; i++) {
                            var bar = createsBars[i];
                            if (bar.node1() === nodeToRemove) {
                                bar.node1(targetNode);
                            } else if (bar.node2() === nodeToRemove) {
                                bar.node2(targetNode);
                            } else {
                                throw new Error("Bar is on node createsBars list but its not set as node1 or node2");
                            }
                        }
                        if (isElementInSet(this.nodes, nodeToRemove)) {
                            this.removeNode(nodeToRemove);
                        }
                    }
                },
                getNodeById: function(id) {
                    for (var i = 0; i < this.nodes().length; i++) {
                        if (this.nodes()[i].id === id) {
                            return this.nodes()[i];
                        }
                    }
                    return null;
                },
                // BARS
                addBar: function(bar, modelInstance) {
                    if (bar.type !== MODEL.OBJECT_TYPE.Bar) {
                        throw new Error("MODEL.addBar: tried to add a non-bar object to the bars array");
                    }
                    var self = this;
                    if (!isEmpty(modelInstance)) {
                        self = modelInstance;
                    }
                    if (bar.id === 0) {
                        barsCount++;
                        bar.id = barsCount;
                        bar.name("Bar " + barsCount);
                    } else {
                        barsCount = Math.max(barsCount, bar.id);
                    }
                    if (bar.id > barsCount) {
                        barsCount = bar.id;
                    }
                    if (bar.name() === "" || bar.name() === "Bar 0") {
                        bar.name("Bar " + barsCount);
                    }
                    //if (isEmpty(bar.node1())) throw new Error("Bar.node1 is empty!");
                    //if (isEmpty(bar.node2())) throw new Error("Bar.node2 is empty!");
                    if (isEmpty(bar.node1()) || isEmpty(bar.node2())) {
                        throw new Error("Bar " + bar.id + " wll not be added because one of the nodes is missing.");
                    };
                    if (self.nodes.indexOf(bar.node1()) === -1 ||
                        self.nodes.indexOf(bar.node2()) === -1) {
                        throw new Error("Cannot add bar which uses node that is not in nodes set");
                    }
                    bar = addElementToSet(self.bars, bar);
                    bar.remove = function() { self.removeBar(bar); };
                    HELPERS.subscribePropertyChange(self, bar.selected, "Bar", self.onObjectSelectedChanged, bar);
                    return bar;
                },
                removeBar: function(bar) {
                    if (!isElementInSet(this.bars, bar)) {
                        throw new Error("MODEL.removeBar: tried to remove bar which is not in the bars array");
                    }
                    if (bar.selected() && isElementInSet(this.selectedObjects, bar)) {
                        removeElementFromSet(this.selectedObjects, bar);
                    }
                    var i, j;
                    for (i = 0; i < bar.attachedNodes().length; i++) {
                        var node = this.getNodeById(bar.attachedNodes()[i].id);
                        if (node.supportId() !== 0) {
                            var supp = this.getSupportById(node.supportId());
                            supp.removeAssignment(node);
                            node.supportId(0);
                        }
                    }
                    var loads = CORE.model.loads();
                    var loadsToRemove = [];
                    for (i = 0; i < loads.length; i++) {
                        var barAttachments = loads[i].barAttachments();
                        for (j = 0; j < barAttachments.length; j++) {
                            if (loads[i].barAttachments()[j].bar.id === bar.id) {
                                loadsToRemove.push(loads[i]);
                            }
                        }
                    }
                    for (i = 0; i < loadsToRemove.length; i++) {
                        this.removeLoad(loadsToRemove[i]);
                    }

                    if (isElementInSet(this.bars, bar)) {
                        removeElementFromSet(this.bars, bar);
                    }
                    var node1ToChange = true;
                    var node2ToChange = true;
                    if (bar
                        .node1()
                        .createsBars()
                        .length ===
                        0 &&
                        isElementInSet(this.nodes, bar.node1()) &&
                        node1ToChange) {
                        HELPERS.disposeAll(bar.node1);
                        this.removeNode(bar.node1());
                    }
                    if (bar
                        .node2()
                        .createsBars()
                        .length ===
                        0 &&
                        isElementInSet(this.nodes, bar.node2()) &&
                        node2ToChange) {
                        HELPERS.disposeAll(bar.node2);
                        this.removeNode(bar.node2());
                    }
                },
                getBarById: function(id) {
                    for (var i = 0; i < this.bars().length; i++) {
                        if (this.bars()[i].id === id) {
                            return this.bars()[i];
                        }
                    }
                    return null;
                },
                isLoadNameUnique: function(name) {
                    for (var i = 0; i < this.loads().length; i++) {
                        if (this.loads()[i].name() === name) {
                            return false;
                        }
                    }
                    return true;
                },
                generateUniqueLoadName: function(name) {
                    if (this.isLoadNameUnique(name)) {
                        return name;
                    } else {
                        let i = 1;
                        while (true) {
                            let newName = name + " (" + i + ")";
                            if (this.isLoadNameUnique(newName)) {
                                return newName;
                            }
                            i++;
                        }
                    }
                },
                // LOADS
                addLoad: function(load, modelInstance) {
                    if (load.type !== MODEL.OBJECT_TYPE.BarForce &&
                        load.type !== MODEL.OBJECT_TYPE.BarTrapezoidal) {
                        throw new Error("MODEL.addLoad: not supported type of load");
                    }
                    var self = this;
                    if (!isEmpty(modelInstance)) {
                        self = modelInstance;
                    }
                    load.name(self.generateUniqueLoadName(load.name()));
                    switch (load.type) {
                        case MODEL.OBJECT_TYPE.BarForce:
                            for (i = 1; i <= self.forceLoads().length; i++) {
                                //self.forceLoads()[i - 1].name("Point " + i);
                            }
                            //load.name("Point " + (self.forceLoads().length + 1));
                            if (self.forceLoads().length > 0) {
                                if (self.trapezoidalUniformLoads().length > 0) {
                                    if (self.forceLoads()[self.forceLoads().length - 1].id > self.trapezoidalUniformLoads()[self.trapezoidalUniformLoads().length - 1].id) {
                                        load.id = self.forceLoads()[self.forceLoads().length - 1].id + 1;
                                    } else {
                                        load.id = self.trapezoidalUniformLoads()[self.trapezoidalUniformLoads().length - 1].id + 1;
                                    }
                                } else {
                                    load.id = self.forceLoads()[self.forceLoads().length - 1].id + 1;
                                }
                            } else {
                                if (self.trapezoidalUniformLoads().length > 0) {
                                    load.id = self.trapezoidalUniformLoads()[self.trapezoidalUniformLoads().length - 1].id + 1;
                                } else {
                                    load.id = self.forceLoads().length + self.trapezoidalUniformLoads().length + 1;
                                }
                            }
                            addElementToSet(self.forceLoads, load);
                            // setting viewer tables positions after adding new load
                            // if (document.getElementById("VIEWUI_propertiesContainer") !== null) {
                            //     var minimumStyleTopValue = 30;
                            //     var minimumHeightOfLoadsTable = 84;
                            //     var heigthOfOneLoad = 19;
                            //     var maxTopPosition = minimumStyleTopValue + 140;
                            //     var currentPosition = minimumStyleTopValue + minimumHeightOfLoadsTable + (self.trapezoidalUniformLoads().length + self.forceLoads().length) * heigthOfOneLoad;

                            //     if (currentPosition > maxTopPosition) {
                            //         CORE.heightOfLoadsTableViewer = maxTopPosition
                            //     } else {
                            //         CORE.heightOfLoadsTableViewer = currentPosition;
                            //     }
                            //     if (CORE.showLoads()) {
                            //         const minimumStyleTopValue = 10;
                            //         document.getElementById("VIEWUI_propertiesContainer").style.top = minimumStyleTopValue + CORE.heightOfLoadsTableViewer + "px";
                            //         let variable = 160 + CORE.heightOfLoadsTableViewer;
                            //         let maxHeight = 'calc(100% - ' + variable + "px)";
                            //         document.getElementById("VIEWUI_propertiesContainer").style.maxHeight = maxHeight;
                            //     }
                            // }

                            break;
                        case MODEL.OBJECT_TYPE.BarTrapezoidal:
                            for (i = 1; i <= self.trapezoidalUniformLoads().length; i++) {
                                //self.trapezoidalUniformLoads()[i - 1].name("Distributed " + i);
                            }
                            //load.name("Distributed " + (self.trapezoidalUniformLoads().length + 1));
                            if (self.trapezoidalUniformLoads().length > 0) {
                                if (self.forceLoads().length > 0) {
                                    if (self.forceLoads()[self.forceLoads().length - 1].id > self.trapezoidalUniformLoads()[self.trapezoidalUniformLoads().length - 1].id) {
                                        load.id = self.forceLoads()[self.forceLoads().length - 1].id + 1;
                                    } else {
                                        load.id = self.trapezoidalUniformLoads()[self.trapezoidalUniformLoads().length - 1].id + 1;
                                    }
                                } else {
                                    load.id = self.trapezoidalUniformLoads()[self.trapezoidalUniformLoads().length - 1].id + 1;
                                }
                            } else {
                                if (self.forceLoads().length > 0) {
                                    load.id = self.forceLoads()[self.forceLoads().length - 1].id + 1;
                                } else {
                                    load.id = self.forceLoads().length + self.trapezoidalUniformLoads().length + 1;
                                }
                            }
                            addElementToSet(self.trapezoidalUniformLoads, load);
                            // setting viewer tables positions after adding new load
                            
                            /*if (document.getElementById("VIEWUI_propertiesContainer") !== null) {
                                var minimumStyleTopValue = 30;
                                var minimumHeightOfLoadsTable = 84;
                                var heigthOfOneLoad = 19;
                                var maxTopPosition = minimumStyleTopValue + 140;
                                var currentPosition = minimumStyleTopValue + minimumHeightOfLoadsTable + (self.trapezoidalUniformLoads().length + self.forceLoads().length) * heigthOfOneLoad;

                                if (currentPosition > maxTopPosition) {
                                    CORE.heightOfLoadsTableViewer = maxTopPosition
                                } else {
                                    CORE.heightOfLoadsTableViewer = currentPosition;
                                }
                                if (CORE.showLoads()) {
                                    const minimumStyleTopValue = 10;
                                    document.getElementById("VIEWUI_propertiesContainer").style.top = minimumStyleTopValue + CORE.heightOfLoadsTableViewer + "px";
                                    let variable = 160 + CORE.heightOfLoadsTableViewer;
                                    let maxHeight = 'calc(100% - ' + variable + "px)";
                                    document.getElementById("VIEWUI_propertiesContainer").style.maxHeight = maxHeight;
                                }
                            }*/

                            break;
                    }

                    VIEWUI.adjustPropertiesHeight();

                    var newLoad = addElementToSet(self.loads, load);
                    newLoad.remove = function() {
                        self.removeLoad(newLoad);
                    };
                    HELPERS.subscribePropertyChange(self,
                        newLoad.selected,
                        "Load",
                        self.onObjectSelectedChanged,
                        newLoad);
                    return newLoad;
                },
                removeLoad: function(load) {
                    if (load.selected() && isElementInSet(CORE.model.selectedObjects, load)) {
                        removeElementFromSet(CORE.model.selectedObjects, load);
                    }
                    load.removeAllAssignments();

                    if (isElementInSet(CORE.model.loads, load)) {
                        removeElementFromSet(CORE.model.loads, load);
                    }

                    if (isElementInSet(CORE.model.forceLoads, load)) {
                        removeElementFromSet(CORE.model.forceLoads, load);
                    }

                    if (isElementInSet(CORE.model.trapezoidalUniformLoads, load)) {
                        removeElementFromSet(CORE.model.trapezoidalUniformLoads, load);
                    }
                    VIEWUI.adjustPropertiesHeight();
                },
                getMaxAbsLoadValue: function() {
                    var loads = this.loads();
                    var maxVal = 0;
                    for (var i = 0, ii = loads.length; i < ii; i++) {
                        if (typeof loads[i].getMaxAbsValue !== 'function') continue;

                        var maxLoadVal = loads[i].getMaxAbsValue();
                        if (maxLoadVal > maxVal) maxVal = maxLoadVal;
                    }
                    return maxVal;
                },
                // SUPPORTS
                addSupport: function(support, modelInstance) {
                    if (support.type !== MODEL.OBJECT_TYPE.NodeSupport) {
                        throw new Error("MODEL.addSupport: tried to add a non-support object to the supports array");
                    }
                    var self = this;
                    if (!isEmpty(modelInstance)) {
                        self = modelInstance;
                    }
                    if (support.id === 0) {
                        nodeSupportsCount++;
                        support.id = nodeSupportsCount;
                    } else {
                        nodeSupportsCount = Math.max(nodeSupportsCount, support.id);
                    }
                    if (support.id > nodeSupportsCount) {
                        nodeSupportsCount = support.id;
                    }
                    var newSupport = addElementToSet(self.supports, support);
                    newSupport.remove = function() {
                        self.removeSupport(newSupport);
                    };
                    HELPERS.subscribePropertyChange(self,
                        newSupport.selected,
                        "Support",
                        self.onObjectSelectedChanged,
                        newSupport);
                    return newSupport;
                },
                removeSupport: function(support) {
                    if (support.selected()) {
                        removeElementFromSet(this.selectedObjects, support);
                    }
                    support.removeAllAssignments();
                    removeElementFromSet(this.supports, support);
                },
                getSupportById: function(id) {
                    if (id === 0) {
                        return {
                            name: ko.observable(SUPPORT_TYPE[0])
                        }
                    }
                    for (var i = 0; i < this.supports().length; i++) {
                        if (this.supports()[i].id === id) {
                            return this.supports()[i];
                        }
                    }
                    return null;
                },
                // FILLS
                addFill: function(fill) {
                    if (fill.type !== MODEL.OBJECT_TYPE.Fill) {
                        throw new Error("MODEL.addFill: tried to add a non-fill object to the fills array");
                    }
                    if (fill.id === 0) {
                        fillCount++;
                        fill.id = fillCount;
                        fill.name("Fill " + fillCount);
                    } else {
                        fillCount = Math.max(fillCount, fill.id);
                    }
                    if (fill.id > fillCount) {
                        fillCount = fill.id;
                    }
                    if (fill.name() === "") {
                        fill.name("Fill " + fillCount);
                    }
                    var self = this;
                    var newFill = addElementToSet(self.fills, fill);
                    newFill.remove = function() {
                        self.removeFill(newFill);
                    };
                    HELPERS.subscribePropertyChange(self,
                        newFill.selected,
                        "Fill",
                        self.onObjectSelectedChanged,
                        newFill);
                    return newFill;
                },
                removeFill: function(fill) {
                    if (fill.selected()) {
                        removeElementFromSet(this.selectedObjects, fill);
                    }
                    removeElementFromSet(this.fills, fill);
                },
                // BOX
                addBox: function(box) {
                    if (box.type !== MODEL.OBJECT_TYPE.Box) {
                        throw new Error("MODEL.addBox: tried to add a non-box object to the boxes array");
                    }
                    boxesCount++;
                    if (box.id === 0) {
                        box.id = boxesCount;
                        box.name(translation.BOX + boxesCount);
                    } else {
                        boxesCount = Math.max(boxesCount, box.id);
                    }
                    if (box.id > boxesCount) {
                        boxesCount = box.id;
                    }
                    if (box.name() === "") {
                        box.name(translation.BOX + boxesCount);
                    }
                    var self = this;
                    var newBox = addElementToSet(self.boxes, box);
                    newBox.remove = function() {
                        self.removeBox(newBox);
                    };
                    HELPERS.subscribePropertyChange(this, newBox.selected, "box", this.onObjectSelectedChanged, newBox);
                    return newBox;
                },
                removeBox: function(box) {
                    if (box.selected()) {
                        removeElementFromSet(this.selectedObjects, box);
                    }
                    removeElementFromSet(this.boxes, box);
                },
                addExternalModel: function(externalModel) {
                    addElementToSet(this.externalModels, externalModel);
                    if(externalModel.selected !== undefined) HELPERS.subscribePropertyChange(this, externalModel.selected, "ExternalModel", this.onObjectSelectedChanged, externalModel);
                },
                removeExternalModel: function(externalModel) {
                    if (externalModel.selected !== undefined && externalModel.selected() && externalModel.isSelectable) {
                        removeElementFromSet(this.selectedObjects, externalModel);
                    }
                    removeElementFromSet(this.externalModels, externalModel);
                },

                // INIT
                init: function() {
                    this.constraints = new MODEL.Constraints(this);
                    // type <--> object array binding
                    this.modelArrays = {};
                    this.modelArrays[MODEL.OBJECT_TYPE.Node] = this.nodes;
                    this.modelArrays[MODEL.OBJECT_TYPE.Bar] = this.bars;
                    this.modelArrays[MODEL.OBJECT_TYPE.Load] = this.loads;
                    this.modelArrays[MODEL.OBJECT_TYPE.Support] = this.supports;
                    this.modelArrays[MODEL.OBJECT_TYPE.Workplane] = this.workplanes;
                    this.modelArrays[MODEL.OBJECT_TYPE.Fill] = this.fills;
                    this.modelArrays[MODEL.OBJECT_TYPE.Box] = this.boxes;
                    this.addFunctions = {};
                    this.addFunctions[MODEL.OBJECT_TYPE.Node] = this.addNode;
                    this.addFunctions[MODEL.OBJECT_TYPE.Bar] = this.addBar;
                    this.addFunctions[MODEL.OBJECT_TYPE.Load] = this.addLoad;
                    this.addFunctions[MODEL.OBJECT_TYPE.Support] = this.addSupport;
                    this.addFunctions[MODEL.OBJECT_TYPE.Workplane] = this.addWorkplane;
                    this.addFunctions[MODEL.OBJECT_TYPE.Fill] = this.addFill;
                    this.addFunctions[MODEL.OBJECT_TYPE.Box] = this.addBox;
                    this.addFunctions[MODEL.OBJECT_TYPE.LoadCase] = this.addLoadCase;
                    var willBeTriggered = false;
                    this.saveMode = { mode: SAVE_MODE.NoConstraints };
                    var self = this;
                    this.watchModel(
                        function(newMode) {
                            if (!newMode) {
                                self.saveMode = { mode: SAVE_MODE.FullConstraints };
                            }
                            if (newMode && newMode.mode < self.saveMode.mode) {
                                // overwrite current mode only if higher priority (FullConstraints = highest priority)
                                self.saveMode.mode = newMode.mode;
                            }
                            if (self.saveMode.mode === SAVE_MODE.NoConstraints ||
                                self.saveMode.mode === SAVE_MODE.FullConstraints) {}
                            if (self.saveMode.mode === SAVE_MODE.FullConstraints) {
                                CORE.disableConstraints = false;
                            } else if (self.saveMode.mode === SAVE_MODE.NoConstraints) {
                                CORE.disableConstraints = true;
                            }
                            if (!willBeTriggered) {
                                willBeTriggered = true;
                                setTimeout(function() {
                                        if (typeof CORE.options.callbackForCalculationStateChanging === 'function') {
                                            CORE.options.callbackForCalculationStateChanging(false, false);
                                        }
                                        self.modelChange.notifySubscribers(self.saveMode);
                                        //self.saveMode.mode = SAVE_MODE.NoConstraints; // moved to be executed later
                                        willBeTriggered = false;
                                    },
                                    1);
                            }
                        },
                        false);
                    this.modelChange.subscribe(function() {
                        if (CORE.clearResults) {
                            // reset results if any
                            self.barsResults(null);
                            self.nodesResults(null);
                            VIEWUI.exceptionWindow.removeInvalidExceptions([]);
                            //if (VIEWUI.statusBar.status === STATUS.SUCCESS) {
                            //VIEWUI.statusBar.clear();
                            //}
                        } else {
                            CORE.clearResults = true;
                        }
                    });
                },
                watchModel: function(callback, unwatch) {
                    ko.watch(this.nodes, true, callback, unwatch);
                    ko.watch(this.bars, true, callback, unwatch);
                    ko.watch(this.loads, true, callback, unwatch, { mode: SAVE_MODE.NoConstraints });
                    ko.watch(this.workplanes, true, callback, unwatch);
                    ko.watch(this.supports, true, callback, unwatch);
                    ko.watch(this.fills, true, callback, unwatch);
                    ko.watch(this.boxes, true, callback, unwatch, { mode: SAVE_MODE.NoConstraints });
                },
                protoBuilder: {},
                prepareProtoBuilder: function(callback) {
                    var self = this;
                    ProtoBuf.load(CORE.rootPath + "js/core/ConstructionModel.proto",
                        null,
                        function(error, root) {
                            if (error) {
                                throw new Error("MODEL.ConstructionViewModel: proto file cannot be loaded, reason: " + error);
                            }
                            self.protoBuilder.ConstructionModel = root.lookup("ConstructionModel");
                            self.protoBuilder.ResultsModel = root.lookup("ResultsModel");
                            self.protoBuilder.Model3d = root.lookup("Model3d");
                            callback();
                        });
                },
                save: function() {
                    var ConstructionModel = this.protoBuilder.ConstructionModel;
                    var protoBufModel = {};
                    var i;
                    var nodesToSave = [];
                    for (i = 0; i < this.nodes().length; i++) {
                        nodesToSave[i] = this.nodes()[i].save();
                    }
                    var barsToSave = [];
                    for (i = 0; i < this.bars().length; i++) {
                        barsToSave[i] = this.bars()[i].save();
                    }
                    var connectorsToSave = [];
                    for (i = 0; i < this.connectors().length; i++) {
                        connectorsToSave[i] = this.connectors()[i].save();
                    }
                    var supportsToSave = [];
                    for (i = 0; i < this.supports().length; i++) {
                        supportsToSave[i] = this.supports()[i].save();
                    }
                    var loadsToSave = [];
                    for (i = 0; i < this.loads().length; i++) {
                        loadsToSave[i] = this.loads()[i].save();
                    }
                    var fillsToSave = [];
                    for (i = 0; i < this.fills().length; i++) {
                        fillsToSave[i] = this.fills()[i].save();
                    }
                    var boxesToSave = [];
                    for (i = 0; i < this.boxes().length; i++) {
                        boxesToSave[i] = this.boxes()[i].save();
                    }
                    protoBufModel.appId = CORE.options.appId;
                    protoBufModel.projId = CORE.options.projId;
                    protoBufModel.jobId = CORE.options.jobId;
                    protoBufModel.nodes = nodesToSave;
                    protoBufModel.bars = barsToSave;
                    protoBufModel.connectors = connectorsToSave;
                    protoBufModel.supports = supportsToSave;
                    protoBufModel.loads = loadsToSave;
                    protoBufModel.systemPreferences = this.systemPreferences().save();
                    protoBufModel.fills = fillsToSave;
                    protoBufModel.boxes = boxesToSave;
                    return protoBufModel;
                },
                sendProto: function(protoBufModel, mode, callback, errorCallback, versionName) {
                    if (mode) {
                        this.saveMode = mode;
                    }

                    var preferences = {
                        unitDistance: VIEWUI.viewOptions.unitDistance(),
                        unitDistancePrecision: VIEWUI.viewOptions.unitDistancePrecision(),
                        unitForce: VIEWUI.viewOptions.unitForce(),
                        unitForcePrecision: VIEWUI.viewOptions.unitForcePrecisionResults(),
                        unitStress: VIEWUI.viewOptions.unitStress(),
                        unitStressPrecision: VIEWUI.viewOptions.unitStressPrecision(),
                        unitMomentForce: VIEWUI.viewOptions.unitMomentForce(),
                        unitMomentDistance: VIEWUI.viewOptions.unitMomentDistance(),
                        unitMomentPrecision: VIEWUI.viewOptions.unitMomentPrecision(),
                        unitPressureForce: VIEWUI.viewOptions.unitPressureForce(),
                        unitPressureSurface: VIEWUI.viewOptions.unitPressureSurface(),
                        unitPressurePrecision: VIEWUI.viewOptions.unitPressurePrecision(),
                        unitAngle: VIEWUI.viewOptions.unitAngle(),
                        unitAnglePrecision: VIEWUI.viewOptions.unitAnglePrecision(),
                        unitUtilization: VIEWUI.viewOptions.unitUtilization(),
                        unitUtilizationPrecision: VIEWUI.viewOptions.unitUtilizationPrecision(),
                        unitDisplacement: VIEWUI.viewOptions.unitDisplacement(),
                        unitDisplacementPrecision: VIEWUI.viewOptions.unitDisplacementPrecision(),
                        showRealElements: VIEWUI.viewOptions.showRealElements(),
                        supportsVisible: VIEWUI.viewOptions.supportsVisible(),
                        loadsVisible: VIEWUI.viewOptions.loadsVisible(),
                        measurementsVisible: VIEWUI.viewOptions.measurementsVisible(),
                        pushButtonsVisible: VIEWUI.viewOptions.pushButtonsVisible(),
                        boxesVisible: VIEWUI.viewOptions.boxesVisible()
                    };
                    CORE.webApiManager.saveToDbUsingProtoBuf(CORE.options.systemId,
                        protoBufModel,
                        JSON.stringify(preferences),
                        versionName,
                        this.saveMode,
                        function(response) {
                            try {
                                if (!isEmpty(response.constraints)) {
                                    if (!CORE.disableConstraints) {
                                        CORE.model.constraints.data(response.constraints);
                                    }
                                } else {
                                    CORE.constructionUndoRedo.addToHistory();
                                }
                                CORE.disableConstraints = false;
                                if (!isEmpty(callback)) {
                                    callback(response);
                                }
                                var saveLogMessage = "MODEL.ConstructionViewModel.save: SAVE COMPLETED, save mode: " + CORE.model.saveMode.mode;

                                console.log(saveLogMessage);

                                CORE.model.saveMode.mode = SAVE_MODE.NoConstraints;
                            } catch (ex) {
                                console.error("MODEL.ConstructionViewModel.save: constraints processing failed: ", ex);
                                VIEWUI.statusBar.setError("Constraints processing failed");
                                if (errorCallback) errorCallback(ex);
                            }
                        },
                        function(ex) {
                            console.error("MODEL.ConstructionViewModel.save: SAVE FAILED", ex);
                            VIEWUI.statusBar.setError("Save failed");
                            if (errorCallback) errorCallback();
                        });
                },
                load: function(appId, projId, jobId, versionId, customCallback, errorCustomCallback) {
                    CORE.webApiManager.loadFromDb(CORE.options.appId,
                        CORE.options.projId,
                        CORE.options.jobId,
                        CORE.options.systemId,
                        versionId,
                        callback,
                        errorCallback);
                    var self = this;

                    function callback(loadResult) {
                        if (loadResult != null) {
                            var call = function(isManual) {
                                VIEWUI.statusBar.clear();
                                try {
                                    // 1 - load construction model
                                    loadConstructionModel();
                                    // Enable autosave if no elements remains to change
                                    HELPERS.subscribeArrayChange(this,
                                        CORE.model.missingElements,
                                        "CORE.model.missingElements",
                                        null,
                                        function() {
                                            if (CORE.model.missingElements().length === 0) {
                                                for (var i = 0; i < CORE.model.validatorSubs.length; i++) {
                                                    CORE.model.validatorSubs[i].dispose();
                                                }
                                                CORE.autoSaveDisabled = false;
                                                CORE.disableConstraints = false;
                                                var proto = CORE.model.save();
                                                CORE.model.sendProto(proto);
                                            } else {
                                                CORE.autoSaveDisabled = true;
                                                CORE.disableConstraints = true;
                                            }
                                        });
                                    // 2 - load contraints
                                    if (isManual) {
                                        CORE.disableConstraints = true;
                                        CORE.constructionView3D.showMissingElements();
                                    }
                                    loadConstraints();
                                    // 3 - load results model (if exists)
                                    // if (loadResult.constructionResults) {
                                    //     // wait for RemoveAll function (because it resets ResultsModel)
                                    //     setTimeout(function() {
                                    //             try {
                                    //                 loadResultsModel();
                                    //             } catch (error) {
                                    //                 if (errorCustomCallback)
                                    //                     errorCustomCallback(error);
                                    //                 if (error.stack) console.error(error.stack);
                                    //             }
                                    //         },
                                    //         0);
                                    // } else {
                                    //     console
                                    //         .log("MODEL.ConstructionViewModel.load: no results model received (not yet calculated)");
                                    // }
                                    // 4 - load system preferences
                                    if (!isEmpty(loadResult.preferences) && !isEmpty(loadResult.preferencesResults)) {
                                        var preferences = JSON.parse(loadResult.preferences);
                                        var preferencesResults = JSON.parse(loadResult.preferencesResults);
                                        VIEWUI.viewOptions.unitDistance(preferences.unitDistance);
                                        VIEWUI.viewOptions.unitDistanceResults(preferencesResults.unitDistance);
                                        VIEWUI.viewOptions.unitDistancePrecision(preferences.unitDistancePrecision);
                                        VIEWUI.viewOptions.unitDistancePrecisionResults(preferencesResults.unitDistancePrecision);
                                        VIEWUI.viewOptions.unitForce(preferences.unitForce);
                                        VIEWUI.viewOptions.unitForceResults(preferencesResults.unitForce);
                                        VIEWUI.viewOptions.unitForcePrecision(preferences.unitForcePrecision);
                                        VIEWUI.viewOptions.unitForcePrecisionResults(preferencesResults.unitForcePrecision);
                                        VIEWUI.viewOptions.unitStress(preferencesResults.unitStress);
                                        VIEWUI.viewOptions.unitStressPrecision(preferencesResults.unitStressPrecision);
                                        VIEWUI.viewOptions.unitMomentForce(preferences.unitMomentForce);
                                        VIEWUI.viewOptions.unitMomentDistance(preferences.unitMomentDistance);
                                        VIEWUI.viewOptions.unitMomentForceResults(preferencesResults.unitMomentForce);
                                        VIEWUI.viewOptions.unitMomentDistanceResults(preferencesResults.unitMomentDistance);
                                        VIEWUI.viewOptions.unitMomentPrecision(preferences.unitMomentPrecision);
                                        VIEWUI.viewOptions.unitMomentPrecisionResults(preferencesResults.unitMomentPrecision);
                                        // VIEWUI.viewOptions.unitMomentPrecisionResults(preferencesResults.unitMomentPrecision);
                                        VIEWUI.viewOptions.unitPressureForce(preferences.unitPressureForce);
                                        VIEWUI.viewOptions.unitPressureSurface(preferences.unitPressureSurface);
                                        VIEWUI.viewOptions.unitPressurePrecision(preferences.unitPressurePrecision);
                                        VIEWUI.viewOptions.unitAngle(preferences.unitAngle);
                                        VIEWUI.viewOptions.unitAnglePrecision(preferences.unitAnglePrecision);
                                        VIEWUI.viewOptions.unitUtilization(preferences.unitUtilization);
                                        VIEWUI.viewOptions
                                            .unitUtilizationPrecision(preferences.unitUtilizationPrecision);
                                        VIEWUI.viewOptions.unitDisplacement(preferencesResults.unitDisplacement);
                                        VIEWUI.viewOptions
                                            .unitDisplacementPrecision(preferencesResults.unitDisplacementPrecision);
                                        VIEWUI.viewOptions
                                            .showRealElements(isEmpty(preferences.showRealElements) ?
                                                true :
                                                preferences.showRealElements);
                                        VIEWUI.viewOptions
                                            .supportsVisible(isEmpty(preferences.supportsVisible) ?
                                                true :
                                                preferences.supportsVisible);
                                        VIEWUI.viewOptions
                                            .loadsVisible(isEmpty(preferences.loadsVisible) ?
                                                true :
                                                preferences.loadsVisible);
                                        VIEWUI.viewOptions
                                            .measurementsVisible(true);
                                        VIEWUI.viewOptions
                                            .pushButtonsVisible(isEmpty(preferences.pushButtonsVisible) ?
                                                true :
                                                preferences.pushButtonsVisible);
                                        VIEWUI.viewOptions
                                            .boxesVisible(isEmpty(preferences.boxesVisible) ?
                                                true :
                                                preferences.boxesVisible);
                                    } else {
                                        throw new Error("SystemPreferences (loadResult.preferences) is null");
                                    }
                                } catch (error) {
                                    if (!CORE.viewerReloading) {
                                        VIEWUI.statusBar.setError("Load failed (processing error)");

                                        if (errorCustomCallback)
                                            errorCustomCallback(error);
                                        if (!isEmpty(error) && !isEmpty(error.stack)) console.error(error.stack);
                                    }
                                }
                            }
                            call(false);

                            function loadConstructionModel() {
                                var constructionProtoModel;
                                try {
                                    // var constructionModel = self.protoBuilder.ConstructionModel;
                                    // var constructionModelB64String = loadResult.constructionData;
                                    // var b64Length = ProtoBuf.util.base64.length(constructionModelB64String);
                                    // var outputBuffer = new Uint8Array(b64Length);
                                    // ProtoBuf.util.base64.decode(constructionModelB64String, outputBuffer, 0);
                                    // constructionProtoModel = constructionModel.decode(outputBuffer);
                                    constructionProtoModel = loadResult.constructionData;
                                } catch (error) {
                                    console
                                        .error("MODEL.ConstructionViewModel.load: proto message parsing for ConstructionModel failed, reason: " + error);
                                    VIEWUI.statusBar.setError("Load error (incompatible construction model)");
                                    throw error;
                                }
                                console.log("MODEL.ConstructionViewModel.load: proto message parsed successfully");
                                try {
                                    CORE.model.loadModel(constructionProtoModel);
                                } catch (error2) {
                                    console
                                        .error("MODEL.ConstructionViewModel.load: Error loading construction proto model into CORE.model, reason: " + error2);
                                    VIEWUI.statusBar.setError("Load error");
                                    throw error2;
                                }
                                console.log("MODEL.ConstructionViewModel.load: construction model loaded successfully");
                            }

                            function loadConstraints() {
                                try {
                                    CORE.model.constraints.data(loadResult.constraints);

                                } catch (error) {
                                    console
                                        .error("MODEL.ConstructionViewModel.load: Error setting constraints, reason: " +
                                            error);
                                    VIEWUI.statusBar.setError("Error setting constraints");
                                    throw error;
                                }
                                console.log("MODEL.ConstructionViewModel.load: constraints loaded successfully");
                            }

                            function loadResultsModel() {
                                var successCallback = function() {
                                    console.log("MODEL.ConstructionViewModel.load: results model loaded successfully");
                                    CORE.calculations
                                        .setCalculationSuccess("Calculation succeeded", loadResult.Exceptions);
                                }
                                var errorCallback = function(error) {
                                    console
                                        .error("MODEL.ConstructionViewModel.load: Error loading calculation results, reason: " + error);
                                }
                                CORE.calculations.loadResults(loadResult.constructionResults);
                            }
                        } else {
                            console
                                .info("MODEL.ConstructionViewModel.load: request returned no data, new job or missing info in db");
                        }
                        if (!isEmpty(customCallback)) {
                            var isNewJob = loadResult === null;
                            self.saveMode.mode = SAVE_MODE.NoConstraints;
                            customCallback(null, isNewJob); // null == no error
                        }
                    }

                    function errorCallback(error) {
                        console.error("MODEL.ConstructionViewModel.load: request failed");
                        VIEWUI.statusBar.setError("Load error (request failed)");
                        if (errorCustomCallback)
                            errorCustomCallback(error);
                        if (!isEmpty(error) && !isEmpty(error.stack)) console.error(error.stack);
                    }
                },
                loadModel: function(protoModel) {
                    console.log("CORE.load.loadModel: deleting all objects...");
                    try {
                        CORE.model.removeAll();
                    } catch (error) {
                        throw new Error("error while removing all objects: " + error);
                    }
                    CORE.model.resetCounters();
                    console.log("CORE.model.loadModel: all objects deleted");
                    var i;
                    // Load system preferences
                    try {
                        CORE.model.systemPreferences().load(protoModel.systemPreferences);
                    } catch (error3) {
                        throw new Error("error while loading system preferences: " + error3);
                    }
                    // Load of Nodes
                    for (i = 0; i < protoModel.nodes.length; i++) {
                        var newNode = new MODEL.Node();
                        newNode.load(protoModel.nodes[i], this);
                        this.addNode(newNode);
                    }
                    // Load of Bars
                    for (i = 0; i < protoModel.bars.length; i++) {
                        var newBar = new MODEL.Bar();
                        newBar.load(protoModel.bars[i], this);
                        this.addBar(newBar);
                    }
                    // Load of Connectors
                    for (i = 0; i < protoModel.connectors.length; i++) {
                        var newConnector = new MODEL.Connector();
                        newConnector.load(protoModel.connectors[i], this);
                        this.connectors.push(newConnector);
                    }
                    // Load of Loads
                    for (i = 0; i < protoModel.loads.length; i++) {
                        var newLoad;
                        switch (protoModel.loads[i].loadType) {
                            case "BarForce":
                                newLoad = new MODEL.load.bar.BarForce();
                                newLoad.load(protoModel.loads[i], this);
                                break;
                            case "BarUniform":
                                newLoad = new MODEL.load.bar.BarUniform();
                                newLoad.load(protoModel.loads[i], this);
                                break;
                            case "BarTrapezoidal":
                                newLoad = new MODEL.load.bar.BarTrapezoidal();
                                newLoad.load(protoModel.loads[i], this);
                                break;
                        }
                        this.addLoad(newLoad);
                    }
                    // Load of Boxes
                    if (!isEmpty(protoModel.boxes)) {
                        for (i = 0; i < protoModel.boxes.length; i++) {
                            var newBox = new MODEL.Box();
                            newBox.load(protoModel.boxes[i]);
                            this.addBox(newBox);
                        }
                    }
                    render();
                    console.log("CORE.load.loadModel: load completed");
                }
            }
        };
        MODEL.ConstructionViewModel.prototype = {}
    });