﻿var MODEL = MODEL || {};

define([],function () {
    var observableLatestValue = ko.utils.createSymbolOrString('_latestValue');
    MODEL.MaxMeasurementValue = 6;
    MODEL.Measurement = function (start, dir, length, validate) {
        this.type = MODEL.OBJECT_TYPE.Measurement;
        this.start = ko.observable(start);
        this.dir = ko.observable(dir);
        this.length = ko.observable(length);
        this.validate = validate;
        this.getEnd = function () {
            return this.start().add(this.dir().multiplyScalar(this.length()));
        };
        this.userData = {};
        this.watchModel = function (callback, unwatch) {
            ko.watch(this.start, false, callback, unwatch);
            ko.watch(this.dir, false, callback, unwatch);
            ko.watch(this.length, false, callback, unwatch);
        };
    };

// observsble list of nodes
    MODEL.Measurements = function (nodes) {
        
        // array of MODEL.Measurement
        this.xChain = ko.observableArray();
        this.yChain = ko.observableArray();
        this.zChain = ko.observableArray();

        this.getAllMeasurements = function () {
            return this.xChain().concat(this.yChain()).concat(this.zChain());
        }

        // event
        this.modelChange = ko.observable();
        // all nodes grouped by x value (sorted by this value)
        var xGroups = [];
        var yGroups = [];
        var zGroups = [];
        var nodesPosSub = {};
        var doNotMove = false;
        var ignoreNodePosChange = false;
        var createMesurement = function (start, dir, length, groups, prop, chain, min) {
            var validate = function(val) {
                val = Math.max(val, 0);
                var index = chain.indexOf(mesurement);
                var totalOtherSum = 0;
                for (i = 0; i < chain().length; i++) {
                    if(i !== index) {
                        totalOtherSum = totalOtherSum + chain()[i].length();
                    }
                }

                var multi = CORE.units.getMulti(VIEWUI.viewOptions.unitDistance());
                
                //MODEL.MaxMeasurementValue - len is total acceptable length for this element taking into account other lements in the chain

                var max = MODEL.MaxMeasurementValue - totalOtherSum;

                if(val * multi <= max)
                    return val;
                else
                    return max / multi;
            }
            var mesurement = new MODEL.Measurement(start, dir, length, validate);
            mesurement.userData.lengthSub = HELPERS.subscribePropertyChange(this, mesurement.length, "length", function (newVal, oldVal) {
                if (doNotMove) {
                    return;
                }
                if (newVal <= 0) {
                    throw "Mesurement length bust be greater than 0";
                }
                ignoreNodePosChange = true;
                var index = chain.indexOf(mesurement) + 1;
                var oldPos;
                var newPos;
                var i;

                //differed notification first update all values then notify
                var observables = [];
                for (i = index; i < groups.length; i++) {

                    var group = groups[i];
                    for (var j = 0; j < group.length; j++) {
                        oldPos = group[j].position();
                        newPos = new MODEL.types.Vector(
                            oldPos.x + (prop === 'x' ? newVal - oldVal : 0),
                            oldPos.y + (prop === 'y' ? newVal - oldVal : 0),
                            oldPos.z + (prop === 'z' ? newVal - oldVal : 0));
                        var observable = group[j].position;
                        observable.valueWillMutate();
                        observable[observableLatestValue] = newPos;
                        observables.push(observable);
                        //group[j].position(newPos);
                    }
                }
                for (var j = 0; j < observables.length; j++) {
                    var observable = observables[j];
                    observable.valueHasMutated();
                    var selectedObject = CORE.model.selectedObjects()[0];
                    if(typeof selectedObject !== 'undefined' && (selectedObject.type === "BarForce" || selectedObject.type === "BarTrapezoidal")) {
                        CORE.model.deselectAll();
                    }
                }

                for (i = index; i < chain().length; i++) {
                    oldPos = chain()[i].start();
                    newPos = new MODEL.types.Vector(
                        oldPos.x + (prop === 'x' ? newVal - oldVal : 0),
                        oldPos.y + (prop === 'y' ? newVal - oldVal : 0),
                        oldPos.z + (prop === 'z' ? newVal - oldVal : 0));
                    chain()[i].start(newPos);
                }
                ignoreNodePosChange = false;
            });
            return mesurement;
        };
        var updateChain = function (groups, prop, chain) {
            var min = {
                x: Number.MAX_VALUE,
                y: Number.MAX_VALUE,
                z: Number.MAX_VALUE
            };
            var i;
            for (i = 0; i < nodes().length; i++) {
                var pos = nodes()[i].position();
                min.x = Math.min(pos.x, min.x);
                min.y = Math.min(pos.y, min.y);
                min.z = Math.min(pos.z, min.z);
            }

            for (i = 1; i < groups.length; i++) {
                var startValue = groups[i - 1][0].position()[prop];
                var endValue = groups[i][0].position()[prop];
                var length = endValue - startValue;
                var start = new MODEL.types.Vector(
                    prop === 'x' ? startValue : min.x,
                    prop === 'y' ? startValue : min.y,
                    prop === 'z' ? startValue : min.z);
                var dir = new MODEL.types.Vector(
                    prop === 'x' ? 1 : 0,
                    prop === 'y' ? 1 : 0,
                    prop === 'z' ? 1 : 0);
                var existingMeasurement = chain()[i - 1];
                if (!isEmpty(existingMeasurement)) {
                    //change existing
                    if (!start.equals(existingMeasurement.start())) {
                        existingMeasurement.start(start);
                    }
                    doNotMove = true;
                    existingMeasurement.length(length);
                    doNotMove = false;
                } else {
                    //add new
                    chain.push(createMesurement(start, dir, length, groups, prop, chain, min));
                }
            }
            var toRemove = chain().length - groups.length + 1;
            if (toRemove > 0 && chain().length > 0) {
                var index = chain().length - toRemove;
                for (i = 0; i < toRemove; i++) {
                    chain()[index + i].userData.lengthSub.dispose();
                }
                chain.splice(index, toRemove);
            }
        };
        var removeNodeFromChain = function (groups, prop, chain, node) {
            for (var i = 0; i < groups.length; i++) {
                var group = groups[i];
                for (var j = 0; j < group.length; j++) {
                    if (node === group[j]) {
                        //remove node from group
                        group.splice(j, 1);
                        if (group.length <= 0) {
                            //remove empty group
                            groups.splice(i, 1);
                            updateChain(groups, prop, chain);
                        }
                        return;
                    }
                }
            }
            console.warn("MODEL.Measurement: node not found in chane");
        };
        var addNodeToChain = function (groups, prop, chain, node) {
            var i = 0;
            for (; i < groups.length; i++) {
                var group = groups[i];
                var groupValue = group[0].position()[prop];
                var nodeValue = node.position()[prop];
                if (Math.abs(nodeValue - groupValue) < 0.005) { //1/2mm
                    group.push(node);
                    break;
                } else if (nodeValue < groupValue) {
                    groups.splice(i, 0, [node]);
                    break;
                }
            }
            if (i === groups.length) {
                groups.push([node]);
            }
            updateChain(groups, prop, chain);
        };
        var onNodePositionChage = function (node, oldPos) {
            if (!ignoreNodePosChange) {
                if (node.position().x !== oldPos.x) {
                    removeNodeFromChain(xGroups, 'x', this.xChain, node);
                    addNodeToChain(xGroups, 'x', this.xChain, node);
                }
                if (node.position().y !== oldPos.y) {
                    removeNodeFromChain(yGroups, 'y', this.yChain, node);
                    addNodeToChain(yGroups, 'y', this.yChain, node);
                }
                if (node.position().z !== oldPos.z) {
                    removeNodeFromChain(zGroups, 'z', this.zChain, node);
                    addNodeToChain(zGroups, 'z', this.zChain, node);
                }
                this.updateStartPos();
            }
        };

        this.updateChainPos = function (chain, x, y, z){
            if(chain.length > 0){
                var startPos = chain[0].start();
                var delta = new MODEL.types.Vector( x - startPos.x, y - startPos.y, z - startPos.z);
                if(delta.length() > 0.001 ){
                    for (var i = 0; i < chain.length; i++) {
                        var measurement = chain[i];
                        measurement.start( measurement.start().add(delta));
                    }
                }
            }

        }

        this.updateStartPos = function() {
            if(xGroups.length > 0) { //not empty
                var minX = xGroups[0][0].position().x;
                var minY = yGroups[0][0].position().y;
                var minZ = zGroups[0][0].position().z;
                this.updateChainPos(this.xChain(), minX, minY, minZ);
                this.updateChainPos(this.yChain(), minX, minY, minZ);
                this.updateChainPos(this.zChain(), minX, minY, minZ);
            }
        }

        var onAddNode = function (node) {

            addNodeToChain(xGroups, 'x', this.xChain, node);
            addNodeToChain(yGroups, 'y', this.yChain, node);
            addNodeToChain(zGroups, 'z', this.zChain, node);
            this.updateStartPos();
            var self = this;
            nodesPosSub[node.id] = HELPERS.subscribePropertyChange(this, node.position, "node.position", function (newPos, oldPos) {
                onNodePositionChage.call(self, node, oldPos);
            });
        };
        var onRemoveNode = function (node) {
            removeNodeFromChain(xGroups, 'x', this.xChain, node);
            removeNodeFromChain(yGroups, 'y', this.yChain, node);
            removeNodeFromChain(zGroups, 'z', this.zChain, node);
            this.updateStartPos();
            if (!isEmpty(nodesPosSub[node.id])) {
                nodesPosSub[node.id].dispose();
            }
            delete nodesPosSub[node.id];
        };
        this.watchModel = function (callback, unwatch) {
            ko.watch(this.xChain, true, callback, unwatch);
            ko.watch(this.yChain, true, callback, unwatch);
            ko.watch(this.zChain, true, callback, unwatch);
        };
        // init
        var self = this;
        var willBeTriggered = false;
        this.watchModel(
            function () {
                if (!willBeTriggered) {
                    willBeTriggered = true;
                    setTimeout(function () {
                        self.modelChange.notifySubscribers();
                        willBeTriggered = false;
                    }, 1);
                }
            }, false);
        HELPERS.subscribeArrayChange(this, nodes, "nodes", onAddNode, onRemoveNode);

        for (var i = 0; i < nodes().length; i++) {
            onAddNode.call(this, nodes()[i]);
        }
        var measurementInfo = function (chain, groups) {
            var str = '';
            for (var i = 0; i < groups.length; i++) {
                str += '(' + groups[i].length + ')';
                if (i < chain.length) {
                    str += '--' + chain[i].length().toFixed(2) + '--';
                }
            }
            return str;
        };
        this.toString = function () {
            return 'X: ' + measurementInfo(this.xChain(), xGroups) +
                '\nY: ' + measurementInfo(this.yChain(), yGroups) +
                '\nZ: ' + measurementInfo(this.zChain(), zGroups);
        };
    };

    MODEL.TotalMeasurements = function (nodes) {
        var validate = function(val) {
            val = Math.max(val, 0);
            var multi = CORE.units.getMulti(VIEWUI.viewOptions.unitDistance());

            if(val * multi <= MODEL.MaxMeasurementValue)
                return val;
            else
                return MODEL.MaxMeasurementValue / multi;
        }

        // array of MODEL.Measurement
        this.xMeasurement = new MODEL.Measurement(new MODEL.types.Vector(0, 0, 0), new MODEL.types.Vector(1, 0, 0), 0, validate);
        this.yMeasurement = new MODEL.Measurement(new MODEL.types.Vector(0, 0, 0), new MODEL.types.Vector(0, 1, 0), 0, validate);
        this.zMeasurement = new MODEL.Measurement(new MODEL.types.Vector(0, 0, 0), new MODEL.types.Vector(0, 0, 1), 0, validate);

        var ignoreChange = false;

        var nodesPosSub = {};
        this.minmax = null;

        this.getAllMeasurements = function () {
            var all = [];
            if (this.xMeasurement.length() !== 0) {
                all.push(this.xMeasurement);
            }
            if (this.yMeasurement.length() !== 0) {
                all.push(this.yMeasurement);
            }
            if (this.zMeasurement.length() !== 0) {
                all.push(this.zMeasurement);
            }
            return all;
        }

        // event
        this.modelChange = ko.observable();

        this.onSizeChange = function (prop, newVal, oldVal) {
            if (!ignoreChange) {
                ignoreChange = true;
                this.minmax = getMinMax();

                var newSize = new MODEL.types.Vector(
                    prop === "x" ? newVal : this.minmax.max.x - this.minmax.min.x,
                    prop === "y" ? newVal : this.minmax.max.y - this.minmax.min.y,
                    prop === "z" ? newVal : this.minmax.max.z - this.minmax.min.z);
                var oldSize = new MODEL.types.Vector(
                    this.minmax.max.x - this.minmax.min.x,
                    this.minmax.max.y - this.minmax.min.y,
                    this.minmax.max.z - this.minmax.min.z);

                if (oldSize.x <= 0.001) {
                    newSize.x = oldSize.x = 1;
                }
                if (oldSize.y <= 0.001) {
                    newSize.y = oldSize.y = 1;
                }
                if (oldSize.z <= 0.001) {
                    newSize.z = oldSize.z = 1;
                }

                var observables = [];
                for (var i = 0; i < nodes().length; i++) {
                    var node = nodes()[i];
                    var observable = node.position;
                    var oldPos = observable();
                    var newPos = oldPos.sub(this.minmax.min).multiply(newSize).divide(oldSize);
                    if(isNaN(newPos.x)){
                        newPos = new MODEL.types.Vector(0, newPos.y, newPos.z)
                    }
                    if(isNaN(newPos.y)){
                        newPos.y = newPos = new MODEL.types.Vector(newPos.x, 0, newPos.z)
                    }
                    if(isNaN(newPos.z)){
                        newPos.z = newPos = new MODEL.types.Vector(newPos.x, newPos.y, 0)
                    }
                    newPos = newPos.add(this.minmax.min);
                    observable.valueWillMutate();
                    observable[observableLatestValue] = newPos;
                    observables.push(observable);
                }
                for (var j = 0; j < observables.length; j++) {
                    var observable = observables[j];
                    observable.valueHasMutated();
                }
                
                var selectedObject = CORE.model.selectedObjects()[0];
                if(typeof selectedObject !== 'undefined' && (selectedObject.type === "BarForce" || selectedObject.type === "BarTrapezoidal")) {
                    CORE.model.deselectAll();
                }
                ignoreChange = false;
            }
        }

        HELPERS.subscribePropertyChange(this, this.xMeasurement.length, "x size change", function (newVal, oldVal) {
            this.onSizeChange("x", newVal, oldVal)
        }.bind(this));
        HELPERS.subscribePropertyChange(this, this.yMeasurement.length, "y size change", function (newVal, oldVal) {
            this.onSizeChange("y", newVal, oldVal)
        }.bind(this));
        HELPERS.subscribePropertyChange(this, this.zMeasurement.length, "z size change", function (newVal, oldVal) {
            this.onSizeChange("z", newVal, oldVal)
        }.bind(this));

        var getMinMax = function () {
            var result = {
                max: {x: -Number.MAX_VALUE, y: -Number.MAX_VALUE, z: -Number.MAX_VALUE},
                min: {x: Number.MAX_VALUE, y: Number.MAX_VALUE, z: Number.MAX_VALUE},
                x: false,
                y: false,
                z: false
            }

            var xc = 0;
            var yc = 0;
            var zc = 0;
            for (var i = 0; i < nodes().length; i++) {
                var nodePos = nodes()[i].position();
                if (nodePos.x !== result.min.x && nodePos.x !== result.max.x) {
                    result.min.x = Math.min(result.min.x, nodePos.x);
                    result.max.x = Math.max(result.max.x, nodePos.x);
                    xc++;
                }
                if (nodePos.y !== result.min.y && nodePos.y !== result.max.y) {
                    result.min.y = Math.min(result.min.y, nodePos.y);
                    result.max.y = Math.max(result.max.y, nodePos.y);
                    yc++;
                }
                if (nodePos.z !== result.min.z && nodePos.z !== result.max.z) {
                    result.min.z = Math.min(result.min.z, nodePos.z);
                    result.max.z = Math.max(result.max.z, nodePos.z);
                    zc++;
                }
            }
            result.x = xc > 2;
            result.y = yc > 2;
            result.z = zc > 2;
            return result;
        }

        this.updateMeasurement = function (prop, measurementName, minmax) {
            var measurement = this[measurementName];
            var result = false;
            if (minmax[prop]) {
                var l = minmax.max[prop] - minmax.min[prop];
                if (isEmpty(measurement) || measurement.start()[prop] !== minmax.min[prop] || measurement.length !== l) {
                    measurement.length(l);
                    result = true;
                }
            } else {
                if (!isEmpty(measurement)) {
                    measurement.length(0);
                    result = true;
                }
            }
            measurement.start(new MODEL.types.Vector(minmax.min.x, minmax.min.y, minmax.min.z));
            return result;
        }

        this.update = function () {
            if (!ignoreChange) {
                ignoreChange = true;
                this.minmax = getMinMax();

                var changed = this.updateMeasurement("x", "xMeasurement", this.minmax);
                changed = this.updateMeasurement("y", "yMeasurement", this.minmax) || changed;
                changed = this.updateMeasurement("z", "zMeasurement", this.minmax) || changed;
                if (changed) {
                    this.modelChange.notifySubscribers();
                }
                ignoreChange = false;
            }
        };

        this.onAddNode = function (node) {
            nodesPosSub[node.id] = HELPERS.subscribePropertyChange(this, node.position, "node.position", this.update.bind(this));
            this.update();
        };

        this.onRemoveNode = function (node) {
            if (!isEmpty(nodesPosSub[node.id])) {
                nodesPosSub[node.id].dispose();
            }
            delete nodesPosSub[node.id];
            this.update();
        };

        HELPERS.subscribeArrayChange(this, nodes, "nodes", this.onAddNode, this.onRemoveNode);

        for (var i = 0; i < nodes().length; i++) {
            var node = nodes()[i];
            nodesPosSub[node.id] = HELPERS.subscribePropertyChange(this, node.position, "node.position", this.update.bind(this));
        }
        this.update();
    };
});