﻿var MODEL = MODEL || {};
var nodesCount = 0;

MODEL.Node = function(positionVector) { // positionVector = MODEL.types.Vector
    this.type = MODEL.OBJECT_TYPE.Node;
    this.id = 0;
    this.name = ko.observable(translation.NODE + " " + (nodesCount + 1));
    this.position = ko.observable(positionVector);
    this.supportId = ko.observable(0);
    this.connectorId = ko.observable(0);
    this.selected = ko.observable(false);
    this.createsBars = ko
        .observableArray(); // returns bars that are created using this node, implemented in ConstructionViewModel
    this.supportB = ko.observable(0.1);
    this.isFast = ko.observable(true);
    this.isSupport = ko.observable(false);
    this.isRotated = ko.observable(false);
    this.isRotatedChanged = ko.observable(false);
    this.attachedBars = ko.observableArray(0); //will not be saved
    this.orientation = {
        baseX: new MODEL.types.Vector(0, 0, 0),
        baseY: new MODEL.types.Vector(0, 0, 0),
        baseZ: new MODEL.types.Vector(0, 0, 0)
    };
    this.watchModel = function(callback, unwatch) {
        ko.watch(this.name, false, callback, unwatch);
        ko.watch(this.position, false, callback, unwatch);
        ko.watch(this.supportId, false, callback, unwatch);
        ko.watch(this.connectorId, false, callback, unwatch);
        ko.watch(this.supportB, false, callback, unwatch);
        ko.watch(this.isSupport, false, callback, unwatch);
        ko.watch(this.isFast, false, callback, unwatch);
        ko.watch(this.isRotatedChanged, false, callback, unwatch);
    };
    // NODE SAVE
    this.save = function() {
        this.position().x = Math.round(this.position().x * epsilonGrid) / epsilonGrid;
        this.position().y = Math.round(this.position().y * epsilonGrid) / epsilonGrid;
        this.position().z = Math.round(this.position().z * epsilonGrid) / epsilonGrid;

        var positionSaved = this.position().save();

        if (isNaN(positionSaved.x) || isNaN(positionSaved.y) || isNaN(positionSaved.z)) {
            console.error("Tried to save vector with NaN values! Changing them to 0...", this);
            if (isNaN(positionSaved.x)) positionSaved.x = 0;
            if (isNaN(positionSaved.y)) positionSaved.y = 0;
            if (isNaN(positionSaved.z)) positionSaved.z = 0;
        }

        this.isRotated(false);
        return {
            id: this.id,
            name: this.name(),
            connectorId: this.connectorId(),
            position: positionSaved,
            isSupport: this.isSupport(),
            baseX: this.orientation.baseX.save(),
            baseY: this.orientation.baseY.save(),
            baseZ: this.orientation.baseZ.save()
        };
    };
    // NODE LOAD
    this.load = function(object, construction) {
        this.id = object.id;
        this.name(object.name);
        
        if (isNaN(object.position.x) || isNaN(object.position.y) || isNaN(object.position.z)) {
            console.warn("Tried to load node position with NaN values! Changing them to 0...", this);
            if (isNaN(object.position.x)) object.position.x = 0;
            if (isNaN(object.position.y)) object.position.y = 0;
            if (isNaN(object.position.z)) object.position.z = 0;
        }

        this.position(new MODEL.types.Vector(object.position.x, object.position.y, object.position.z));

        if (object.supportId !== 0 && !isEmpty(object.supportId)) {
            var support = construction.getSupportById(object.supportId);
            if (isEmpty(support)) {
                console.error("Support with Id " +
                    object.supportId +
                    " was attached to node '" +
                    object.name +
                    "' but there is no support with that ID! so I removed it.");
                object.supportId = 0;
            } else {
                support.assignTo(this);
            }
        }
        this.connectorId(object.connectorId);
        this.isSupport(object.isSupport);
    };
    // NODE IS_FAST CHECKBOX CONTROLLER   
    this.showFastOptions = ko.observable(true);
};