﻿var MODEL = MODEL || {};

// there is a relation between node and bar which says that a node lies on a bar: bar.attachedNodes
// this class observes all nodes and bars and updates this relation
// so if you find that 
// - there in a node that lies on bar but is not attached to the bar
// - there is a node that is out of bar but is attached to the bar
// then problem is in this class :(
MODEL.NodeAttachController = function(nodes, bars) {
    // if its true the nodes on bar will be detachen if bar has moved instead moving them with bar
    var detachMode = false;
    this.setDetachMode = function(enable) { detachMode = enable; }
    // minimal distance betwenn node and bar in system units [m]
    var minDistance = 0.01;
    var minDistanceSquared = minDistance * minDistance;
    // subscribers on node positionchange
    var nodesPosSub = {};
    // subscribers on bar nodes (node1, node2) chnage
    var barNodesPosSub = {};
    var rfNodesPosSub = {};
    // helpers
    //----------------------------------------------------------
    // returns true if node shudd be on bars attachedNodes list  
    var attachTest = function(bar, node) {
        if (node === bar.node1() || node === bar.node2()) {
            return false;
        }
        var nodePos = node.position();
        var proj = bar.vectorProjection(nodePos);
        if (proj != null && proj.squaredDistanceTo(nodePos) < minDistanceSquared) {
            return true;
        } else {
            return false;
        }
    }
    var safeAttachNode = function(bar, node) {
        var index = bar.attachedNodes.indexOf(node);
        if (index === -1) {
            bar.attachedNodes.push(node);
        }
        index = node.attachedBars.indexOf(bar);
        if (index === -1) {
            node.attachedBars.push(bar);
        }
    }
    var safeDetachNode = function(bar, node) {
        var index = bar.attachedNodes.indexOf(node);
        if (index !== -1) {
            bar.attachedNodes.splice(index, 1);
        }
        index = node.attachedBars.indexOf(bar);
        if (index !== -1) {
            node.attachedBars.splice(index, 1);
        }
    }
    var safeReattachNode = function(bar, node) {
        if (attachTest(bar, node) === true) {
            safeAttachNode(bar, node);
        } else {
            safeDetachNode(bar, node);
        }
    }
    // event handlers
    //----------------------------------------------------------
    var onBarPositionChange = function(bar, oldNode1Pos, oldNode2Pos) {
        var newNode1Pos = bar.node1().position();
        var newNode2Pos = bar.node2().position();
        // update attached nodes
        var i;
        var node;
        for (i = 0; i < bar.attachedNodes().length; i++) {
            node = bar.attachedNodes()[i];
            if (detachMode) {
                // detach if not on bar any more
                safeReattachNode(bar, node);
            } else {
                // move it with bar
                var attachedNodePos = node.position();
                var t = oldNode1Pos.distanceTo(attachedNodePos) / oldNode1Pos.distanceTo(oldNode2Pos);
                node.position(newNode1Pos.add(newNode2Pos.sub(newNode1Pos).multiplyScalar(t)));
            }
        }
        // find nodes that now intersects
        for (i = 0; i < nodes().length; i++) {
            node = nodes()[i];
            safeReattachNode(bar, node);
        }
    }
    var onBarNodeChage = function(bar, oldNode1, oldNode2) {
        var oldNode1Pos = oldNode1.position();
        var oldNode2Pos = oldNode2.position();
        onBarPositionChange(bar, oldNode1Pos, oldNode2Pos);
    }
    var onNodePositionChage = function(node, oldPos) {
        // attach/detach this node
        var i, bar;
        for (i = 0; i < bars().length; i++) {
            bar = bars()[i];
            safeReattachNode(bar, node);
        }
        // handle bars that was moved with this node
        var nodeBars = node.createsBars();
        if (!isEmpty(nodeBars)) {
            for (i = 0; i < nodeBars.length; i++) {
                bar = nodeBars[i];
                var oldNode1Pos = node === bar.node1() ? oldPos : bar.node1().position();
                var oldNode2Pos = node === bar.node2() ? oldPos : bar.node2().position();
                onBarPositionChange(bar, oldNode1Pos, oldNode2Pos);
            }
        }
    }
    var onAddNode = function(node) {
        // attach/detach this node
        var i;
        for (i = 0; i < bars().length; i++) {
            var bar = bars()[i];
            safeReattachNode(bar, node);
        }
        nodesPosSub[node.id] = HELPERS.subscribePropertyChange(this, node.position, "node.position", function(newPos, oldPos) { onNodePositionChage(node, oldPos); });
    }
    var onRemoveNode = function(node) {
        // detach from bars
        var i;
        for (i = 0; i < bars().length; i++) {
            var bar = bars()[i];
            safeDetachNode(bar, node);
        }
        nodesPosSub[node.id].dispose();
        delete nodesPosSub[node.id];
    }
    var onAddBar = function(element) {
        for (var i = 0; i < nodes().length; i++) {
            var node = nodes()[i];
            safeReattachNode(element, node);
        }
        if (element.type === MODEL.OBJECT_TYPE.Bar) {
            barNodesPosSub[element.id] = {
                node1: HELPERS.subscribePropertyChange(this,
                    element.node1,
                    "bar.node1",
                    function (newNode, oldNode) { onBarNodeChage(element, oldNode, element.node2()); }),
                node2: HELPERS.subscribePropertyChange(this,
                    element.node2,
                    "bar.node2",
                    function (newNode, oldNode) { onBarNodeChage(element, element.node1(), oldNode); })
            }
        }
    }
    var onRemoveBar = function(element) {
        for (var i = 0; i < element.attachedNodes().length; i++) {
            var node = element.attachedNodes()[i];
            var index = node.attachedBars.indexOf(element);
            if (index !== -1) {
                node.attachedBars.splice(index, 1);
            }
        }
        if (element.type === MODEL.OBJECT_TYPE.Bar) {
            if (!isEmpty(barNodesPosSub[element.id])) {
                barNodesPosSub[element.id].node1.dispose();
                barNodesPosSub[element.id].node2.dispose();
                delete barNodesPosSub[element.id];
            }
        }
    }
    // init
    HELPERS.subscribeArrayChange(this, nodes, "nodes", onAddNode, onRemoveNode);
    HELPERS.subscribeArrayChange(this, bars, "bars", onAddBar, onRemoveBar);
    var i;
    for (i = 0; i < nodes().length; i++) {
        onAddNode(nodes()[i]);
    }
    for (i = 0; i < bars().length; i++) {
        onAddBar(bars()[i]);
    }
}