﻿var MODEL = MODEL || {};
// for 2d results view in bar properties
MODEL.ResultsWindowViewModel = function() {
    return {
        // public members
        nodes: ko.observableArray(),
        bars: ko.observableArray(),
        workplanes: ko.observableArray(),
        selectedObjects: ko.observableArray(),
        activeWorkplane: ko.observable(null),
        barsResults: ko.observable(null),
        lastXcoordinate: ko.observable(null),
        modelChange: ko.observable(), // event
        isActiveWorkplane: function(workplane) {
            return workplane === this.activeWorkplane();
        },
        getAllObjects: function() {
            var selectedObjects = [];
            var i;
            for (i = 0; i < this.nodes().length; i++) {
                selectedObjects.push(this.nodes()[i]);
            }
            for (i = 0; i < this.bars().length; i++) {
                selectedObjects.push(this.bars()[i]);
            }
            return selectedObjects;
        },
        removeObjects: function(objects) {
            objects = ko.unwrap(objects);
            while (objects.length > 0) {
                // common interface
                objects[0].remove();
            }
        },
        removeSelectedObjects: function() {
            this.removeObjects(this.selectedObjects);
        },
        getSelectedObjectsFromArray: function(modelArray) {
            modelArray = ko.unwrap(modelArray);
            var selectedObjects = [];
            for (var i = 0, ii = modelArray.length; i < ii; i++) {
                if (modelArray[i].selected()) {
                    selectedObjects.push(modelArray[i]);
                }
            }
            return selectedObjects;
        },
        addStartNode: function(node) {
            node.position(new MODEL.types.Vector(-1, 0, 0));
            this.addNode(node);
        },
        addEndNode: function(node) {
            node.position(new MODEL.types.Vector(-1, 0, 0));
            this.addNode(node);
        },
        //public methods
        addNode: function(node, modelInstance) {
            if (node.type !== MODEL.OBJECT_TYPE.Node) {
                throw "MODEL.addNode: tried to add a non-node object to the nodes array";
            }
            var self = this;
            if (!isEmpty(modelInstance)) {
                self = modelInstance;
            }
            if (node.id === 0) {
                nodesCount++;
                node.id = nodesCount;
            } else {
                nodesCount = Math.max(nodesCount, node.id);
            }
            if (node.id > nodesCount) {
                nodesCount = node.id;
            }
            if (node.name() === "" || node.name() === "Node 0") {
                node.name("Node " + nodesCount);
            }
            node = addElementToSet(self.nodes, node);
            node.remove = function() {
                self.removeNode(node);
            };
            return node;
        },
        removeNode: function(node) {
            if (!isElementInSet(this.nodes, node)) {
                throw "MODEL.removeNode: tried to remove node which is not in the nodes array";
            }
            var createsBars = node.createsBars();
            var i;
            for (i = 0; i < createsBars.length; i++) {
                if (isElementInSet(this.bars, createsBars[i])) {
                    // bar will remove itself from createsBars list
                    this.removeBar(createsBars[i]);
                }
            }
            if (typeof node.support === 'function' && !isEmpty(node.support())) {
                node.support().removeAssignment(node);
            }
            if (node.selected() && isElementInSet(this.selectedObjects, node)) {
                removeElementFromSet(this.selectedObjects, node);
            }
            if (isElementInSet(this.nodes, node)) {
                removeElementFromSet(this.nodes, node);
            }
        },
        addResultsBar: function(bar) {
            this.addBar(bar);
        },
        addBar: function(bar, modelInstance) {
            if (bar.type !== MODEL.OBJECT_TYPE.Bar) {
                throw "MODEL.addBar: tried to add a non-bar object to the bars array";
            }
            var self = this;
            if (!isEmpty(modelInstance)) {
                self = modelInstance;
            }
            if (bar.id === 0) {
                barsCount++;
                bar.id = barsCount;
            } else {
                barsCount = Math.max(barsCount, bar.id);
            }
            if (bar.id > barsCount) {
                barsCount = bar.id;
            }
            if (bar.name() === "" || bar.name() === "Bar 0") {
                bar.name("Bar " + barsCount);
            }
            if (self.nodes.indexOf(bar.node1()) === -1 ||
                self.nodes.indexOf(bar.node2()) === -1) {
                throw new Error("Cennot add bar which uses node that is not in nodes set");
            }
            bar = addElementToSet(self.bars, bar);
            bar.remove = function() {
                self.removeBar(bar);
            };
            return bar;
        },
        removeBar: function(bar) {
            if (!isElementInSet(this.bars, bar)) {
                throw "MODEL.removeBar: tried to remove bar which is not in the bars array";
            }
            if (bar.selected()) {
                removeElementFromSet(this.selectedObjects, bar);
            }
            if (isElementInSet(this.bars, bar)) {
                removeElementFromSet(this.bars, bar);
            }
            if (bar.node1().createsBars().length === 0 && isElementInSet(this.nodes, bar.node1())) {
                this.removeNode(bar.node1());
            }
            if (bar.node2().createsBars().length === 0 && isElementInSet(this.nodes, bar.node2())) {
                this.removeNode(bar.node2());
            }
        },
        addWorkplane: function(workplane, modelInstance) {
            if (workplane.type !== MODEL.OBJECT_TYPE.Workplane) {
                throw "MODEL.addWorkplane: tried to add a non-workplane object to the workplanes array";
            }
            var self = this;
            if (!isEmpty(modelInstance)) {
                self = modelInstance;
            }
            workplane = addElementToSet(self.workplanes, workplane);
            workplanesCount++;
            workplane.remove = function() {
                self.removeWorkplane(workplane);
            };
            return workplane;
        },
        removeWorkplane: function(workplane) {
            if (this.activeWorkplane() === workplane) {
                this.activeWorkplane(null);
            }
            removeElementFromSet(this.workplanes, workplane);
        },
        deselectAll: function() {
            var allObjects = this.getAllObjects();
            for (var i = 0, ii = allObjects.length; i < ii; i++) {
                allObjects[i].selected(false);
            }
        },
        init: function() {
            // type <--> object array binding
            this.modelArrays = {};
            this.modelArrays[MODEL.OBJECT_TYPE.Node] = this.nodes;
            this.modelArrays[MODEL.OBJECT_TYPE.Bar] = this.bars;
            this.modelArrays[MODEL.OBJECT_TYPE.Workplane] = this.workplanes;
            this.addFunctions = {};
            // need to pass model instance because 'this' is not reliable
            this.addFunctions[MODEL.OBJECT_TYPE.Node] = this.addNode;
            this.addFunctions[MODEL.OBJECT_TYPE.Bar] = this.addBar;
            this.addFunctions[MODEL.OBJECT_TYPE.Workplane] = this.addWorkplane;
        }
    }
};