﻿var MODEL = MODEL || {};
var nodeSupportsCount = 0;
var supportImageCache = [];

MODEL.NodeSupport = function(supportType, modelId, subType, id) {
    this.type = MODEL.OBJECT_TYPE.NodeSupport;
    this.id = id;
    this.name = ko.observable(supportType);
    this.supportType = ko.observable(supportType);
    this.kVector = ko.observable(new MODEL.types.ToggleableVector(0, 0, 0, true, true, true));
    this.hVector = ko.observable(new MODEL.types.ToggleableVector(0, 0, 0, true, true, true));
    this.rotation = ko.observable(new MODEL.types.Vector(0, 0, 0));
    this.selected = ko.observable(false);
    this.assignedTo = ko.observableArray();
    this.modelId = ko.observable(modelId);
    this.subType = ko.observable(subType);
    this.watchModel = function(callback, unwatch) {
        ko.watch(this.name, false, callback, unwatch);
        ko.watch(this.supportType, false, callback, unwatch);
        ko.watch(this.kVector, false, callback, unwatch);
        ko.watch(this.hVector, false, callback, unwatch);
        ko.watch(this.rotation, false, callback, unwatch);
    };
    this.save = function() {
        return {
            id: this.id,
            name: this.name(),
            supportType: this.supportType(),
            kVector: this.kVector().save(),
            hVector: this.hVector().save(),
            rotation: this.rotation().save()
        }
    }
    this.load = function() {
        // NYI
    }
    this.assignTo = function(nodeModelObject) {
        if (nodeModelObject.type !== MODEL.OBJECT_TYPE.Node) {
            console.error("MODEL.load.node.NodeForce.assignTo: passed nodeModelObject is not a Node");
            return;
        }
        var index = this.assignedTo.indexOf(nodeModelObject.id);
        if (index !== -1) {
            console.error("MODEL.Support.assignTo: passed nodeModelObject already exists in load.assignedTo array");
            return;
        }
        if (nodeModelObject.supportId() === this.id) {
            console.debug("MODEL.Support.assignTo: current support object already assigned to passed nodeModelObject");
            return;
        }
        // cleanup old assignment
        if (!isEmpty(nodeModelObject.supportId()) && nodeModelObject.supportId()) {
            var found = false;
            for (var i = 0; i < CORE.model.supports().length; i++) {
                if (CORE.model.supports()[i].id === nodeModelObject.supportId()) {
                    CORE.model.supports()[i].removeAssignment(nodeModelObject);
                    found = true;
                }
            }
            if (!found) {
                throw new Error("Support with index " + nodeModelObject.supportId() + " not found");
            }
        }
        this.assignedTo.push(nodeModelObject);
        nodeModelObject.supportId(this.id);
    };
    this.removeAssignment = function(nodeModelObject) {
        if (nodeModelObject.type !== MODEL.OBJECT_TYPE.Node) {
            console.error("MODEL.Support.removeAssignment: passed nodeModelObject is not a Node");
            return;
        }
        var indexInLoad = this.assignedTo.indexOf(nodeModelObject);
        if (indexInLoad === -1) {
            throw new Error("MODEL.Support.removeAssignment: passed nodeModelObject does not exist in support.assignedTo array");
        }
        if (nodeModelObject.supportId() !== ko.unwrap(this.id)) {
            throw new Error("MODEL.Support.removeAssignment: current support is not equal to passed nodeModelObject.support");
        }
        this.assignedTo.splice(indexInLoad, 1);
        nodeModelObject.supportId(0);
    };
    this.removeAllAssignments = function () {
        // remove all assignments
        while (this.assignedTo().length > 0) { 
            this.removeAssignment(this.assignedTo()[0]);
        }
    };
    this.subs = [];
    this.dispose = function () {
        for (var i = 0; i < this.subs.length; i++) {
            this.subs[i].dispose();
        }
    }
}