﻿var MODEL = MODEL || {};
MODEL.OBJECT_TYPE = {
    Node: "Node", Bar: "Bar", Workplane: "Workplane", Load: "Load", Support: "Support", LoadCase: "LoadCase",
    NodeForce: "NodeForce", NodeMoment: "NodeMoment", NodeSupport: "NodeSupport", NodeConnector: "NodeConnector",
    BarForce: "BarForce", BarUniform: "BarUniform", BarTrapezoidal: "BarTrapezoidal", BarProfile: "BarProfile", BarMaterial: "BarMaterial",
    Measurement: "Measurement", Fill: "Fill", Box: "Box", BoxDefinition: "BoxDefinition", Connector: "Connector"
}
MODEL.types = {
    Vector: function(x, y, z) {
        //immutable 
        Object.defineProperty(this, 'x', { value: x, writable: false, enumerable: true });
        Object.defineProperty(this, 'y', { value: y, writable: false, enumerable: true });
        Object.defineProperty(this, 'z', { value: z, writable: false, enumerable: true });
        this.toViewModel = function() {
            return {
                ModelObjectConstructor: this.constructor,
                x: ko.isObservable(this.x) ? this.x : ko.observable(this.x),
                y: ko.isObservable(this.y) ? this.y : ko.observable(this.y),
                z: ko.isObservable(this.z) ? this.z : ko.observable(this.z),
                toModelObject: function() {
                    return new this.ModelObjectConstructor(HELPERS.toNumber(this.x()), HELPERS.toNumber(this.y()), HELPERS.toNumber(this.z()));
                }
            }
        },
        this.save = function() {
            return {
                x: this.x,
                y: this.y,
                z: this.z
            }
        },
        this.getMaxAbsValue = function () {
            return Math.max(Math.abs(this.x), Math.abs(this.y), Math.abs(this.z));
        }
    },
    ThreeVectors: function(vector1, vector2, vector3) {
        // immutable 
        Object.defineProperty(this, 'vector1', {
            value: vector1,
            writable: false,
            enumerable: true
        });
        Object.defineProperty(this, 'vector2', {
            value: vector2,
            writable: false,
            enumerable: true
        });
        Object.defineProperty(this, 'vector3', {
            value: vector3,
            writable: false,
            enumerable: true
        });
        this.toViewModel = function() {
            return {
                ModelObjectConstructor: this.constructor,
                vector1: this.vector1.toViewModel(),
                vector2: this.vector2.toViewModel(),
                vector3: this.vector3.toViewModel(),
                getNestedProperties: function() {
                    return [this.vector1, this.vector2, this.vector3];
                },
                toModelObject: function() {
                    return new this.ModelObjectConstructor(this.vector1.toModelObject(), this.vector2.toModelObject(), this.vector3.toModelObject());
                }
            }
        }
    },
    ToggleableVector: function(x, y, z, xEnabled, yEnabled, zEnabled) {
        this.value = new MODEL.types.Vector(x, y, z);
        this.enabled = new MODEL.types.Vector(xEnabled, yEnabled, zEnabled);
        this.toViewModel = function() {
            return {
                ModelObjectConstructor: this.constructor,
                value: this.value.toViewModel(),
                enabled: this.enabled.toViewModel(),
                getNestedProperties: function() {
                    return [this.value, this.enabled];
                },
                toModelObject: function() {
                    var modelValueVector = this.value.toModelObject();
                    var enabledModelVector = this.enabled.toModelObject();
                    return new this.ModelObjectConstructor(
                        modelValueVector.x, modelValueVector.y, modelValueVector.z,
                        enabledModelVector.x, enabledModelVector.y, enabledModelVector.z
                    );
                }
            }
        }
        this.save = function () {
            return {
                value: this.value.save(),
                enabled: this.enabled.save()
            }
        }
    },
    ExtendedVector: function(vectorX, vectorY, vectorZ, distance) {
        this.vector = new MODEL.types.Vector(vectorX, vectorY, vectorZ);
        this.distance = distance;
        this.toViewModel = function() {
            return {
                ModelObjectConstructor: this.constructor,
                vector: this.vector.toViewModel(),
                distance: ko.observable(this.distance),
                getNestedProperties: function() {
                    return [this.vector, this.distance];
                },
                toModelObject: function() {
                    var vector = this.vector.toModelObject();
                    var distance = this.distance();
                    return new this.ModelObjectConstructor(
                        vector.x, vector.y, vector.z, HELPERS.toNumber(distance)
                    );
                }
            }
        }
        this.save = function () {
            return {
                vector: this.vector.save(),
                // always absolute
                distance: this.distance     
            }
        }
    },
    PositionVector: function (vectorX, vectorY, vectorZ, id, name) {
        this.position = new MODEL.types.Vector(vectorX, vectorY, vectorZ);
        this.id = id;
        this.name = ko.observable(name);
        this.type = "Point";
        this.toViewModel = function() {
            return {
                ModelObjectConstructor: this.constructor,
                position: this.position.toViewModel(),
                id: ko.observable(this.id),
                name: ko.observable(this.name()),
                type: this.type,
                getNestedProperties: function() {
                    return [this.position, this.id, this.name(), this.type];
                },
                toModelObject: function() {
                    var position = this.position.toModelObject();
                    var id = this.id();
                    var name = this.name();
                    var type = this.type;
                    return new this.ModelObjectConstructor(
                        position.x, position.y, position.z, HELPERS.toNumber(id), name, type
                    );
                }
            }
        }
    },
    BarAttachment: function (bar, x, isRf) {
        this.bar = bar;
        this.x = ko.observable(x);
        this.type = isEmpty(isRf) ? 0 : isRf;
        this.save = function () {
            if (isEmpty(this.bar)) {
                throw new Error("MODEL.types.BarAttachment: Tried to save BarAttachment with empty bar!");
            }
            if (isEmpty(this.x)) {
                throw new Error("MODEL.types.BarAttachment: Tried to save BarAttachment with empty x!");
            }
            return {
                barId: this.bar.id,
                x: this.x(),
                type: this.type
            }
        };
        this.watchModel = function (callback, unwatch) {
            ko.watch(this.x, false, callback, unwatch);
        };
    }
}
MODEL.types.Vector.prototype = {
    constructor: MODEL.types.Vector,
    dot: function(v) {
        return this.x * v.x + this.y * v.y + this.z * v.z;
    },
    squaredDistanceTo: function(vector) {
        return (this.x - vector.x) * (this.x - vector.x) + (this.y - vector.y) * (this.y - vector.y) + (this.z - vector.z) * (this.z - vector.z);
    },
    distanceTo: function(vector) {
        return Math.sqrt(this.squaredDistanceTo(vector));
    },
    sub: function(v) {
        return new MODEL.types.Vector(this.x - v.x, this.y - v.y, this.z - v.z);
    },
    add: function(v) {
        return new MODEL.types.Vector(this.x + v.x, this.y + v.y, this.z + v.z);
    },
    multiply: function(v) {
        return new MODEL.types.Vector(this.x * v.x, this.y * v.y, this.z * v.z);
    },
    divide: function(v) {
        return new MODEL.types.Vector(this.x / v.x, this.y / v.y, this.z / v.z);
    },
    multiplyScalar: function(s) {
        return new MODEL.types.Vector(this.x * s, this.y * s, this.z * s);
    },
    equals: function(v) {
        return this.x === v.x && this.y === v.y && this.z === v.z;
    },
    length: function() {
        return Math.sqrt(this.x * this.x + this.y * this.y + this.z * this.z);
    },
    normalize: function() {
        var l = 1 / this.length();
        return this.multiplyScalar(l);
    },
    addScalar: function(scalar) {
        return new MODEL.types.Vector(this.x + scalar, this.y + scalar, this.z + scalar);
    },
    clone: function () {
        return new MODEL.types.Vector(this.x, this.y, this.z);
    }
}