﻿var MODEL = MODEL || {};
var profileImageCache = ko.observableArray();
var connectorImageCache = ko.observableArray();
var materials = {};

function str2bytes(str) {
    var bytes = new Uint8Array(str.length);
    for (var i = 0; i < str.length; i++) {
        bytes[i] = str.charCodeAt(i);
    }
    return bytes;
}

MODEL.repository = {
    total: 0,
    start: null,
    models: {},
    modelsCallbacks: {},
    objectsToLoad: ko.observable(0),
    //models3dToLoad: [], // { id, callback }
    loadingPlaceholderCode: "<div class='VIEWUI_loadingPlaceholder VIEWUI_center'>" +
        "<span class='loader lightLoader'>" +
        "<span class='bounce1'></span>" +
        "<span class='bounce2'></span>" +
        "<span class='bounce3'></span>" +
        "</span>" +
        "</div>",
    get3DModelAsync: function(modelId, callback) {
        var db = openDatabase('3dViewerModels', '1.0', '3D viewer models', 838860800); // 100 MB in bytes
        var cacheModel;
        var self2 = this;
        db.transaction(function(tx) {
            tx.executeSql('CREATE TABLE IF NOT EXISTS Models (id unique, modelJson)');
        });
        db.transaction(function(tx) {
            tx.executeSql('SELECT * FROM Models WHERE id = ?', [modelId], function(tx, results) {

                if (results.rows.length > 0) cacheModel = results.rows.item(0);
                if (modelId < 1) {
                    callback(null);
                }
                var model = self2.models[modelId];
                if (cacheModel !== undefined) {
                    var model = JSON.parse(cacheModel.modelJson);
                    callback(model);
                } else {
                    if (isEmpty(self2.modelsCallbacks[modelId])) {
                        //first one that is askin for this model
                        self2.modelsCallbacks[modelId] = [callback];
                        self2.objectsToLoad(self2.objectsToLoad() + 1);
                        if (Number(parseFloat(modelId)) !== modelId) {
                            throw new Error("Cannot Get3DModel with id: " + modelId);
                        }
                        //if(CORE.constructionView3D.view3dInitialized())
                        //{

                        var url = apiUri + "ConstructionViewerStorage/Get3DModel";
                        console.log("Before request");


                        var self = self2;
                        VIEWUI.statusBar.startLoading();
                        $.ajax({
                                method: "GET",
                                url: HandleRequestUrl(url, "?id=" + modelId),
                                beforeSend: AddHeadersToRequest
                            })
                            .done(function(data) {
                                if (data !== null) {
                                    console.log("Loaded 3D model: " + modelId);
                                    var model3D = data;
                                    self.models[modelId] = model3D;
                                    var tempModel = JSON.stringify(model3D);
                                    db.transaction(function(tx) {
                                        tx.executeSql('INSERT INTO Models (id, modelJson) VALUES (?, ?)', [modelId, tempModel]);
                                    });
                                    if (!isEmpty(self.modelsCallbacks[modelId])) {
                                        for (var i = 0; i < self.modelsCallbacks[modelId].length; i++) {
                                            self.modelsCallbacks[modelId][i](model3D);
                                            self.objectsToLoad(self.objectsToLoad() - 1);
                                        }
                                        delete self.modelsCallbacks[modelId];
                                    }
                                } else {
                                    console.warn("3D model not Found: " + modelId);
                                    self.models[modelId] = "";
                                }
                                VIEWUI.statusBar.stopLoading();
                            })
                            .fail(function() {
                                console.warn("Failed to load 3D model");
                                self.models[modelId] = "";
                                for (var i = 0; i < self.modelsCallbacks[modelId].length; i++) {
                                    self.modelsCallbacks[modelId][i]("");
                                }
                            });
                        //}
                    } else {
                        //this model is in loading state, wait with others till it comes
                        self2.modelsCallbacks[modelId].push(callback);
                        self2.objectsToLoad(self2.objectsToLoad() + 1);
                    }
                }
            }, error => {
                console.error("error webDb: ", error);
            });
        });
    },
    load3DModels: function() { // load all needed models
        console.log("Before request");
        var self = this;
        VIEWUI.statusBar.startLoading();
        var url = apiUri + "ConstructionViewerStorage/Get3DModels";

        var modelIdsToLoad = Object.keys(MODEL.repository.modelsCallbacks);
        console.log("Requesing models:", modelIdsToLoad);
        $.ajax({
                method: "POST",
                url: HandleRequestUrl(url, ''),
                beforeSend: AddHeadersToRequest,
                data: JSON.stringify({ ids: modelIdsToLoad }),
                dataType: "json",
                contentType: "application/json",
                traditional: true
            })
            .done(function(data) {
                if (!isEmpty(data)) {
                    // if (data.length !== modelIdsToLoad.length) {
                    //     console.error("data.length !== modelIdsToLoad.length");
                    // }
                    for (var i = 0; i < data.length; i++) {
                        var modelId = data[i].id;
                        var model3D = data[i].model3D;
                        if (model3D.data === "") {
                            console.warn("3D model not Found: " + modelId);
                            self.models[modelId] = "";
                        }
                        self.models[modelId] = model3D;

                        if (!isEmpty(self.modelsCallbacks[modelId])) {
                            for (var j = 0; j < self.modelsCallbacks[modelId].length; j++) {
                                self.modelsCallbacks[modelId][j](model3D);
                                console.log("Loaded 3D model: " + modelId);
                                self.objectsToLoad(self.objectsToLoad() - 1);
                            }
                            delete self.modelsCallbacks[modelId];
                        }
                    }
                    // render(2);
                } else {
                    console.error("Get3DModels request returned empty response");
                }
                VIEWUI.statusBar.stopLoading();
            })
            .fail(function() {
                console.error("Failed to load 3D models");
                //self.models[modelId] = "";
                for (var i = 0; i < self.modelsCallbacks.length; i++) {
                    for (var j = 0; j < self.modelsCallbacks[i].length; j++) {
                        self.modelsCallbacks[i][j]("");
                    }
                }
                VIEWERLOADER.setError("Failed to load 3D models, request failed");
            });
    },
    get3DModel: function(modelId, callback) {
        if (modelId === 0) {
            return null;
        }
        var model = this.models[modelId];
        if (!isEmpty(model)) {
            return model;
        } else {
            if (isEmpty(this.modelsCallbacks[modelId])) {
                //first one that is askin for this model
                this.modelsCallbacks[modelId] = [callback];

                var url = apiUri + "ConstructionViewerStorage/Get3DModel";

                console.log("Before request");
                var self = this;
                VIEWUI.statusBar.startLoading();
                $.ajax({
                        method: "GET",
                        url: HandleRequestUrl(url, "?id=" + modelId),
                        beforeSend: AddHeadersToRequest
                    })
                    .done(function(model3D) {
                        if (!isEmpty(model3D)) {
                            console.log("Loaded 3D model: " + modelId);
                            var loader = new THREE.OBJLoader();
                            model3D.object3d = loader.parse(model3D.data);
                            self.models[modelId] = model3D;
                            for (var i = 0; i < self.modelsCallbacks[modelId].length; i++) {
                                if (!isEmpty(self.modelsCallbacks[modelId][i])) {
                                    self.modelsCallbacks[modelId][i](model3D);
                                }
                            }
                        } else {
                            console.warn("3D model not Found: " + modelId);
                            self.models[modelId] = "";
                        }
                        VIEWUI.statusBar.stopLoading();
                    })
                    .fail(function() {
                        console.warn("Failed to load 3D model");
                        self.models[modelId] = "";
                        for (var i = 0; i < self.modelsCallbacks[modelId].length; i++) {
                            self.modelsCallbacks[modelId][i]("");
                        }
                    });
            } else {
                //this model is in loading state, wait with others till it comes
                this.modelsCallbacks[modelId].push(callback);
            }
            return null;
        }
    },
    setLoadingPlaceholder: function(element) {
        if (!element) return;
        $(element).parent().prepend(MODEL.repository.loadingPlaceholderCode);
    },
    clearLoadingPlaceholder: function(element) {
        if (!element) return;
        $(element).parent().find(".VIEWUI_loadingPlaceholder").remove();
    },
    getProfileImage: function(profileId, element) {
        if (profileId === 0) {
            return null;
        }
        if (profileImageCache().length < 1) {
            MODEL.repository.setLoadingPlaceholder(element);
            var url = "/api/ConstructionViewerStorage/GetProfileImages";

            console.log("Before request");
            VIEWUI.statusBar.startLoading();
            $.ajax({
                    method: "GET",
                    url: url
                })
                .done(function(data) {
                    MODEL.repository.clearLoadingPlaceholder(element);
                    VIEWUI.statusBar.stopLoading();
                    if (profileImageCache().length < 1) {
                        if (!isEmpty(data)) {
                            console.log("Loading profile images successful");
                            profileImageCache(data);
                            return "data:image/png;base64," + profileImageCache()[profileId];
                        } else {
                            console.error("Loading profile images are empty!");
                            return null;
                        }
                    } else {
                        return "data:image/png;base64," + profileImageCache()[profileId];
                    }
                })
                .fail(function() {
                    console.error("Loading profile images failed");
                    VIEWUI.statusBar.stopLoading();
                    MODEL.repository.clearLoadingPlaceholder(element);
                    return null;
                });
        } else {
            return "data:image/png;base64," + profileImageCache()[profileId];
        }
        return null;
    },
    getConnectorImage: function(connectorId, element) {
        if (connectorId === 0 || connectorId === -1) {
            return null;
        }
        if (connectorImageCache().length < 1) {
            MODEL.repository.setLoadingPlaceholder(element);
            var url = "/api/ConstructionViewerStorage/GetConnectorImages";

            console.log("Before request");
            VIEWUI.statusBar.startLoading();
            $.ajax({
                    method: "GET",
                    url: url
                })
                .done(function(data) {
                    MODEL.repository.clearLoadingPlaceholder(element);
                    VIEWUI.statusBar.stopLoading();
                    if (connectorImageCache().length < 1) {
                        if (!isEmpty(data)) {
                            console.log("Loading connector images successful");
                            connectorImageCache(data);
                            return "data:image/png;base64," + connectorImageCache()[connectorId];
                        } else {
                            console.error("Loading connector images are empty!");
                            return null;
                        }
                    } else {
                        return "data:image/png;base64," + connectorImageCache()[connectorId];
                    }
                })
                .fail(function() {
                    console.error("Loading connector images failed");
                    VIEWUI.statusBar.stopLoading();
                    MODEL.repository.clearLoadingPlaceholder(element);
                    return null;
                });
        } else {
            return "data:image/png;base64," + connectorImageCache()[connectorId];
        }
        return null;
    },
    getSystemTypes: function(callback, errorCallback) { // TODO: wrap $.ajax
        VIEWUI.statusBar.stopLoading();
        // callback({});
        // return;
        console.log("Before request");
        var url = apiUri + "ConstructionViewerStorage/GetSystemTypes";
        VIEWUI.statusBar.startLoading();
        $.ajax({
                method: "GET",
                url: HandleRequestUrl(url, ""),
                beforeSend: AddHeadersToRequest
            })
            .done(function(data /*, textStatus, jqXHR*/ ) {
                if (!isEmpty(data)) {
                    console.log("Loading system types successful");
                    VIEWUI.statusBar.stopLoading();
                    callback(data);
                } else {
                    console.error("System types array is empty!");
                    VIEWUI.statusBar.stopLoading();
                    if (errorCallback) errorCallback(null);
                }
            })
            .fail(function(jqXhr, textStatus, errorThrown) {
                console.error("Loading system types failed");
                console.error(jqXhr.responseText);
                VIEWUI.statusBar.stopLoading();
                if (errorCallback) errorCallback(errorThrown);
            });
    },
    getSupportTypes: function(callback) {
        var url = "/api/ConstructionViewerStorage/GetSupportTypes";

        console.log("Before request");
        VIEWUI.statusBar.startLoading();
        $.ajax({
                method: "GET",
                url: url
            })
            .done(function(data) {
                if (!isEmpty(data)) {
                    console.log("Loading support types successful");
                    VIEWUI.statusBar.stopLoading();
                    callback(data);
                } else {
                    console.error("Loading support types are empty!");
                    VIEWUI.statusBar.stopLoading();
                    callback(null);
                }
            })
            .fail(function() {
                console.error("Loading support types failed");
                VIEWUI.statusBar.stopLoading();
                callback(null);
            });
    },
    loadMaterials: function() {
        var url = "/api/ConstructionViewerStorage/GetMaterials";

        console.log("Before request");
        VIEWUI.statusBar.startLoading();
        $.ajax({
                method: "GET",
                url: url
            })
            .done(function(data) {
                if (!isEmpty(data)) {
                    var loader = new THREE.MTLLoader();
                    var materialCreator;
                    console.log("Loading materials successful");
                    VIEWUI.statusBar.stopLoading();
                    for (var i = 0; i < data.length; i++) {
                        materialCreator = loader.parse(data[i]);
                        for (var materialName in materialCreator.materialsInfo) {
                            if (materialCreator.materialsInfo.hasOwnProperty(materialName)) {
                                materials[materialName] = materialCreator.materialsInfo[materialName];
                            }
                        }
                    }
                } else {
                    console.error("Loading materials are empty!");
                    VIEWUI.statusBar.stopLoading();
                    return null;
                }
                return null;
            })
            .fail(function() {
                console.error("Loading materials failed");
                VIEWUI.statusBar.stopLoading();
                return null;
            });
    },
    getTexture: function(name, callback) {
        var url = "/api/ConstructionViewerStorage/GetTexture?name=" + name;

        console.log("Before request");
        VIEWUI.statusBar.startLoading();
        $.ajax({
                method: "GET",
                url: url
            })
            .done(function(data) {
                if (!isEmpty(data)) {
                    console.log("Loading texture successful");
                    VIEWUI.statusBar.stopLoading();
                    callback(data);
                } else {
                    console.error("Loading texture is empty!");
                    VIEWUI.statusBar.stopLoading();
                    callback(null);
                }
            })
            .fail(function() {
                console.error("Loading texture failed");
                VIEWUI.statusBar.stopLoading();
                callback(null);
            });
    },
    getTranslation: function(callback) {
        var url = "/assets/i18n/" + CORE.options.language + ".json";

        console.log("Before request");
        VIEWUI.statusBar.startLoading();
        $.ajax({
                method: "GET",
                url: url
            })
            .done(function(data) {
                if (!isEmpty(data)) {
                    console.log("Loading translation successful");
                    VIEWUI.statusBar.stopLoading();
                    callback(data);
                } else {
                    console.error("Translation is empty!");
                    VIEWUI.statusBar.stopLoading();
                    callback(null);
                }
            })
            .fail(function() {
                console.error("Loading translation failed");
                VIEWUI.statusBar.stopLoading();
                callback(null);
            });
    },
    getModuleProfilesList: function(callback) {
        var url = "/api/ConstructionViewerStorage/GetModuleProfileList?systemId=" + CORE.options.systemId;

        console.log("Before request");
        VIEWUI.statusBar.startLoading();
        $.ajax({
                method: "GET",
                url: url
            })
            .done(function(data) {
                if (!isEmpty(data)) {
                    console.log("Loading module profiles list successful");
                    VIEWUI.statusBar.stopLoading();
                    callback(data);
                } else {
                    console.error("Loading module profiles list is empty!");
                    VIEWUI.statusBar.stopLoading();
                    callback(null);
                }
            })
            .fail(function() {
                console.error("Loading module profiles list failed");
                VIEWUI.statusBar.stopLoading();
                callback(null);
            });
    },
    exportToDxf: function(construction) {
        var div = document.getElementById('dxfLoader');
        div.style.display = 'block';
        VIEWUI.toolbar.exportToDxfInProgress = true;
        var windowUrl = window.URL || window.webkitURL;
        VIEWUI.statusBar.startLoading();

        CORE.webApiManager.uploadDataToServer(JSON.stringify(construction), (fileKey) => {
            $.ajax({
                url: HandleRequestUrl(apiUri + 'Construction/ExportToDxfFormUploadedFile', "?fileKey=" + fileKey),
                beforeSend: AddHeadersToRequest,
                method: "GET",
            }).done(function(taskInfo) {
                CORE.webApiManager.waitForTaskEnd(taskInfo.taskKey, () => MODEL.repository.GetDxf(taskInfo.taskKey));
            }).fail(function() {
                VIEWUI.statusBar.stopLoading();
            });
        })
    },
    GetDxf: function(taskKey) {
        var div = document.getElementById('dxfLoader');
        div.style.display = 'block';
        var windowUrl = window.URL || window.webkitURL;
        VIEWUI.statusBar.startLoading();

        $.ajax({
            url: HandleRequestUrl(apiUri + 'Construction/GetDxf', "?taskKey=" + taskKey),
            beforeSend: AddHeadersToRequest,
            method: "GET"
        }).done(function(data) {
            byteData = atob(data);
            bytes = str2bytes(byteData);
            var blob = new Blob([bytes], { type: 'application/zip' });
            var url = windowUrl.createObjectURL(blob);
            var link = document.createElement('a');
            link.href = url;
            link.download = 'construction.dxf.zip';
            document.body.appendChild(link);

            VIEWUI.statusBar.stopLoading();
            link.click();
            document.body.removeChild(link);
            windowUrl.revokeObjectURL(url);
            var div = document.getElementById('dxfLoader');
            div.style.display = 'none';
            VIEWUI.toolbar.exportToDxfInProgress = false;
        }).fail(function() {
            VIEWUI.statusBar.stopLoading();
        });
    }
};