﻿'use strict';
var VIEW3D = VIEW3D || {}

// class VIEW3D.ActiveWorkplane extendes THREE.Line
// binds to active workplane and shows it as frid
VIEW3D.ActiveWorkplane = function (activeWorkplaneModel, halfSize, step, color) {
    THREE.Object3D.call(this);
    this.halfSize = halfSize;
    this.planeMath = null;
    //this.planeGeometry = new THREE.PlaneGeometry(this.halfSize * 2, this.halfSize * 2);
    //this.planeMaterial = new THREE.MeshBasicMaterial({ color: 'lightgray', side: THREE.DoubleSide, opacity: 0.5, transparent: true });
    this.visible = false;
    this.model = activeWorkplaneModel == null ? null : activeWorkplaneModel();
    this.initialStep = 0.5;                // change grid density
    this.step = this.initialStep;          // distance between grid lines (is variable)
    this.initialDistance = 70;
    this.initialZoom = 6.86;
    this.margin = 0.1;                     // margin from window border
    this.initialised = false;

    var attributes = {
        position: {
            type: Float32Array,
            size: 3
        },
        color: {
            type: Float32Array,
            size: 3
        }
    };
    this.geometry = new VIEW3D.utils.ResizableNonIndexedGeometry(attributes, 10000);
    this.material = new THREE.LineBasicMaterial({ depthTest: false, depthWrite: false, vertexColors: THREE.VertexColors });

    this.lineSegments = new THREE.LineSegments(this.geometry, this.material);
    this.lineSegments.frustumCulled = false;
    this.add(this.lineSegments);

    //this.fogCamera = new THREE.OrthographicCamera(-0.99, 0.99, 0.99, -0.99, 0, 1);
    //this.fogScene = new THREE.Scene();
    this.workplaneScene = new THREE.Scene();
    this.skyboxScene = new THREE.Scene();
    this.skyboxCamera = new THREE.OrthographicCamera(-1000, 1000, 1000, -1000, -5001, 5001);

    this.gridLinesColor = new THREE.Color(color);
    this.centerLinesColor = new THREE.Color(0.4, 0.4, 0.4);
    //this.gridLinesColors = [this.gridLinesColor, this.gridLinesColor, this.gridLinesColor, this.gridLinesColor];
    //this.centerLinesColors = [this.centerLinesColor, this.centerLinesColor, this.centerLinesColor, this.centerLinesColor];

    var positionChangeSubscription = null;
    var self = this;

    this.init = function () {

        this.drawSkybox();

        this.initialised = true;
    }

    this.drawSkybox = function () {

        this.skyboxCamera.position.copy(camera.position); // copy from main camera
        this.skyboxCamera.up.copy(camera.up);
        this.skyboxCamera.rotation.copy(camera.rotation);
        this.skyboxCamera.lookAt(new THREE.Vector3(0, 0, 0));
        this.skyboxCamera.updateMatrix();
        this.skyboxCamera.updateProjectionMatrix();
        this.skyboxCamera.matrixAutoUpdate = false;

        function getTexture(direction) {

            var size = 64;

            var canvas = document.createElement('canvas');
            canvas.width = size;
            canvas.height = size;

            var context = canvas.getContext('2d');

            context.rect(0, 0, size, size);

            var skyColor = "#FFFFFF";
            var groundColor = "#E7E2D7";

            switch (direction) {
                case 'side':
                    //var gradient = context.createLinearGradient(0, 0, 0, size);
                    //gradient.addColorStop(0, skyColor);
                    //gradient.addColorStop(0.35, skyColor);
                    //gradient.addColorStop(0.65, groundColor);
                    //gradient.addColorStop(1, groundColor);
                    context.fillStyle = "#FFFFFF";   // Viewer background color
                    break;
                case 'ypos':
                    context.fillStyle = skyColor;
                    break;
                case 'yneg':
                    context.fillStyle = groundColor;
                    break;
            }
   
            context.fill();
            var texture = new THREE.Texture(canvas);
            texture.needsUpdate = true;

            return texture;
        }

        var directions = ["side", "side", "ypos", "yneg", "side", "side"];
        var sphereRadius = 2500;
        var skyGeometry = new THREE.SphereGeometry(sphereRadius, 32, 32);

        var loader = new THREE.TextureLoader();

        var materialArray = [];
        for (var i = 0; i < directions.length; i++) {

            materialArray.push(new THREE.MeshBasicMaterial({
                map: getTexture(directions[i]),
                side: THREE.BackSide
            }));
        }
        var skyBox = new THREE.Mesh(skyGeometry, materialArray);
        skyBox.up.copy(self.skyboxCamera.up);
        skyBox.rotation.x = Math.PI / 2;
        self.skyboxScene.add(skyBox);
        render();
    }


   this.get2dFrom3dPoint = function (point3d) {

       var topCamera = camera.clone(); // clone of the default camera, but set as always looking down ("top" view)

       var topRotation = new THREE.Euler(0, 0, 0);
       var topRotationMatrix = new THREE.Matrix4().makeRotationFromEuler(topRotation);
       topCamera.matrix.copy(topRotationMatrix);

       topCamera.updateMatrix();
       topCamera.updateProjectionMatrix();
       topCamera.updateMatrixWorld();

       topCamera.matrixWorldInverse.getInverse(topCamera.matrixWorld);  // why this is needed?

       var wvp = new THREE.Matrix4().multiplyMatrices(topCamera.projectionMatrix, topCamera.matrixWorldInverse);
       var position2d = point3d.clone().applyMatrix4(wvp);        // position3D -> position2D

       return new THREE.Vector2(
               canvas.clientWidth / 2 * (1 + position2d.x),
               canvas.clientHeight / 2 * (1 - position2d.y)
       );
   }

   this.get3dFrom2dPoint = function (point2d) {

       var raycaster = new THREE.Raycaster();

       raycaster.setFromCamera(point2d, camera);

       var intersects = raycaster.intersectObjects([this], false);

       if (intersects.length > 0)
           return intersects[0].point;
       else
           return null;
   }

   this.draw = function () {
        return;
       if (!this.initialised) return;      // not yet initialised

        this.clear();

        var safetyCounter = 0;
        this.step = this.initialStep;

        var point2d1 = this.get2dFrom3dPoint(new THREE.Vector3(0, 0, 0));
        var point2d2 = this.get2dFrom3dPoint(new THREE.Vector3(this.initialStep, 0, 0));

        var distance = point2d1.distanceTo(point2d2);
       //console.debug("distance 2d: " + distance);
        if (distance === 0) {
            console.log("Not ready to draw workplane grid, distance = 0 (viewer probably not visible on page)");
            return;
        }

        var testDistance = distance;

        var divideBy = 1;

        if (camera.zoom > this.initialZoom) {
            while (testDistance > this.initialDistance) {
                testDistance /= 2;
                divideBy *= 2;

                safetyCounter++; if (safetyCounter > 100) { throw new Error("safetyCounter > 100!"); }
            }

            if (divideBy > 1) divideBy /= 2;
        }

        safetyCounter = 0;

        if (camera.zoom < this.initialZoom) {
            while (testDistance < this.initialDistance) {
                testDistance *= 2;
                divideBy *= 0.5;

                safetyCounter++; if (safetyCounter > 100) { throw new Error("safetyCounter > 100!"); }
            }

            if (divideBy > 1) divideBy /= 0.5;
        }

        safetyCounter = 0;

        //console.debug("testDistance: " + testDistance);

        this.step = this.initialStep / divideBy;

        if (this.step < 0.01) this.step = 0.01;
        if (this.step === Infinity) this.step = 20;
       //console.debug("camera.zoom: " + camera.zoom);
       //console.debug("this.step: " + this.step);

       // limit number of lines if camera is close to parallel to the ground
        var cameraRotation = new THREE.Euler().setFromRotationMatrix(camera.matrix);
       
        if (Math.abs(Math.abs(cameraRotation.x) - Math.PI / 2) < 0.3    // are double abs necessary?
         || Math.abs(Math.abs(cameraRotation.y) - Math.PI / 2) < 0.3)
            this.step *= 2;
       

       // set halfSize according to bs
        var bs = CORE.view3D.cameraControl.getConstructionBS();
        var bsDistanceFromCenter = new THREE.Vector3(bs.center.x, bs.center.y, 0).distanceTo(new THREE.Vector3(0, 0, 0));
        halfSize = bs.radius + bsDistanceFromCenter + 2;

        if (bs.isModelEmpty)
            halfSize = 3.5;

        var maxHalfSize = 50;
        if (halfSize > maxHalfSize)
            halfSize = maxHalfSize;

       //adjust halfSize so that is is a multiply of this.step
        var fitsHowManyTimes = Math.floor(halfSize / this.step);

        halfSize = this.step * fitsHowManyTimes;

        this.geometry.changeSize((fitsHowManyTimes+fitsHowManyTimes+1)*4);

        var index = 0;
        for (var i = 0; i >= -halfSize; i -= this.step) {
            /*
            this.geometry.vertices.push(new THREE.Vector3(-halfSize, i, 0));
            this.geometry.vertices.push(new THREE.Vector3(halfSize, i, 0));
            this.geometry.vertices.push(new THREE.Vector3(i, -halfSize, 0));
            this.geometry.vertices.push(new THREE.Vector3(i, halfSize, 0));*/

            this.geometry.attributes.position.setXYZ(index    , -halfSize, i, 0)
            this.geometry.attributes.position.setXYZ(index+1  , halfSize, i, 0)
            this.geometry.attributes.position.setXYZ(index+2  , i, -halfSize, 0)
            this.geometry.attributes.position.setXYZ(index+3  , i,  halfSize, 0)

            var color = i === 0 ? this.centerLinesColor : this.gridLinesColor;
            this.geometry.attributes.color.setXYZ(index    , color.r, color.g, color.b)
            this.geometry.attributes.color.setXYZ(index+1  , color.r, color.g, color.b)
            this.geometry.attributes.color.setXYZ(index+2  , color.r, color.g, color.b)
            this.geometry.attributes.color.setXYZ(index+3  , color.r, color.g, color.b)
            index += 4;
        }

        for (var i = this.step; i <= halfSize; i += this.step) {
            this.geometry.attributes.position.setXYZ(index    , -halfSize, i, 0)
            this.geometry.attributes.position.setXYZ(index+1  , halfSize, i, 0)
            this.geometry.attributes.position.setXYZ(index+2  , i, -halfSize, 0)
            this.geometry.attributes.position.setXYZ(index+3  , i,  halfSize, 0)

            var color = this.gridLinesColor;
            this.geometry.attributes.color.setXYZ(index    , color.r, color.g, color.b)
            this.geometry.attributes.color.setXYZ(index+1  , color.r, color.g, color.b)
            this.geometry.attributes.color.setXYZ(index+2  , color.r, color.g, color.b)
            this.geometry.attributes.color.setXYZ(index+3  , color.r, color.g, color.b)
            index += 4;
        }

       this.geometry.attributes.position.needsUpdate = true;
       this.geometry.attributes.color.needsUpdate = true;
    }

   
   this.clear = function () {
       /*
       this.geometry.dispose();
       this.geometry = new THREE.Geometry();

       for (var i = 0; i < this.children.length; i++) {
           this.remove(this.children[i]);
       }
       */
   }

    this.draw();
    this.workplaneScene.add(this);
    CORE.model.modelChange.subscribe(function () { CORE.constructionView3D.activeWorkplane.draw(); });

    this.onPositionChange = function (threeVectors) { //threeVectors: MODEL.types.ThreeVectors
        this.planeMath = new THREE.Plane(new THREE.Vector3(0, 0, 0), 0);
        var point1 = HELPERS.toVector3(threeVectors.vector1);
        var point2 = HELPERS.toVector3(threeVectors.vector2);
        var point3 = HELPERS.toVector3(threeVectors.vector3);
        this.planeMath.setFromCoplanarPoints(point1, point2, point3);
        var normal = this.planeMath.normal;
        var position = this.position.clone();
        position.add(normal);
        this.lookAt(position);
        this.position.copy(point1);
        render();
    }
    this.onActiveWorkplaneChange = function (newWorkplane) {
        if (positionChangeSubscription != null) {
            positionChangeSubscription.dispose();
        }
        if (newWorkplane != null) {
            positionChangeSubscription = HELPERS.subscribePropertyChange(this, newWorkplane.position, "newWorkplane.position", this.onPositionChange);
            this.onPositionChange.call(this, newWorkplane.position());
            this.visible = true;
        } else {
            this.visible = false;
            this.planeMath = null;
            render();
        }
    }
    if (!isEmpty(activeWorkplaneModel)) {
        HELPERS.subscribePropertyChange(this, activeWorkplaneModel, "activeWorkplaneModel", this.onActiveWorkplaneChange);
        this.onActiveWorkplaneChange.call(this, activeWorkplaneModel());
    }


}
VIEW3D.ActiveWorkplane.prototype = Object.create(THREE.Object3D.prototype);

// overrid raycast
VIEW3D.ActiveWorkplane.prototype.raycast = function(raycaster, intersects) {
    if (this.planeMath != null) {
        var dir = raycaster.ray.direction.clone().multiplyScalar(10000.0);
        var start = raycaster.ray.origin.clone().sub(dir);
        var end = raycaster.ray.origin.clone().add(dir);
        var line = new THREE.Line3(start, end);
        var intersection = this.planeMath.intersectLine(line);
        if (intersection != undefined) {
            intersects.push({ point: intersection });
        }
    }
}