﻿'use strict';
var VIEW3D = VIEW3D || {}

define(['js/view3d/materials/VIEW3D.materials.ScreenSpace', 'js/view3d/utils/VIEW3D.utils'], function () {
    // class Arrows extends THREE.Mesh
    VIEW3D.Arrows = function (camera, maxLength, customMaxArrow) {
        var maxLength = maxLength || 1;
        THREE.Object3D.call(this);
        var arrowsData = {}; //arrows by id
        var lastArrowsId = 0;

        var attributes = {
            position: {
                type: Float32Array,
                size: 3
            },
            elementPos: {
                type: Float32Array,
                size: 3
            },
            color: {
                type: Float32Array,
                size: 3
            }
        };
        var mesh = new THREE.Mesh(
            new VIEW3D.utils.ResizableGeometry(attributes, 30000),
            new VIEW3D.materials.ScreenSpace()
        );
        mesh.frustumCulled = false;
        mesh.visible = false;
        this.add(mesh);

        this.dispose = function(){
            mesh.geometry.dispose();
        }

        var self = this;
        this.maxArrow = 0;
        this.minArrow = 0.08;
        var maxMoment = 0;
        /*
        var posArray = null;
        var indexArray = null;
        var elPosArray = null;
        var colorArray = null;
        */
        var addArrowToBuffers = function (startIndex, startVertex, arrow, collider) {
            var posArray = mesh.geometry.attributes.position.array;
            var elPosArray = mesh.geometry.attributes.elementPos.array;
            var colorArray = mesh.geometry.attributes.color.array;
            var indexArray = mesh.geometry.index.array;

            var x, y, z;
            var length = VIEW3D.Arrows.maxSize * arrow.dir.length() / self.maxArrow;

            if (length !== 0 && Math.abs(length) < self.minArrow){
                length = length > 0 ? self.minArrow : -self.minArrow;
            }

            length = length*CORE.loadModelScale();
             
            var base = VIEW3D.utils.getBaseFromDir(arrow.dir);
            var i;
            for (i = 0; i < arrowTemplate.indexArray.length; i++) {
                indexArray[startIndex + i] = arrowTemplate.indexArray[i] + startVertex;
            }
            for (i = 0; i < arrowTemplate.posArray.length; i += 3) {
                x = arrowTemplate.posArray[i + 0];
                y = arrowTemplate.posArray[i + 1];
                z = arrowTemplate.posArray[i + 2];
                //set length
                if (Math.abs(x) > 0.001) {
                    x += length - arrowTemplate.maxX;
                }
                //rotate to dir
                posArray[startVertex*3 + i + 0] = base.baseX.x * x + base.baseY.x * y + base.baseZ.x * z;
                posArray[startVertex*3 + i + 1] = base.baseX.y * x + base.baseY.y * y + base.baseZ.y * z;
                posArray[startVertex*3 + i + 2] = base.baseX.z * x + base.baseY.z * y + base.baseZ.z * z;
                elPosArray[startVertex*3 + i + 0] = arrow.pos.x;
                elPosArray[startVertex*3 + i + 1] = arrow.pos.y;
                elPosArray[startVertex*3 + i + 2] = arrow.pos.z;
                colorArray[startVertex*3 + i + 0] = arrow.color.x;
                colorArray[startVertex*3 + i + 1] = arrow.color.y;
                colorArray[startVertex*3 + i + 2] = arrow.color.z;
            }

            collider.addCollider(
                new VIEW3D.collision.ScreenSegmentCollider(
                    new THREE.Vector3(arrow.pos.x, arrow.pos.y, arrow.pos.z),
                    new THREE.Vector3(0, 0, 0),
                    new THREE.Vector3(arrow.pos.x, arrow.pos.y, arrow.pos.z),
                    new THREE.Vector3(base.baseX.x * length, base.baseX.y * length, base.baseX.z * length)
                ));
        }
        var addMomentToBuffers = function (startIndex, startVertex, arrow, collider) {
            var posArray = mesh.geometry.attributes.position.array;
            var elPosArray = mesh.geometry.attributes.elementPos.array;
            var colorArray = mesh.geometry.attributes.color.array;
            var indexArray = mesh.geometry.index.array;
            var x, y, z;
            var length = VIEW3D.Arrows.maxSize * arrow.dir.length() / maxMoment;
            var base = VIEW3D.utils.getBaseFromDir(arrow.dir);
            var i;
            for (i = 0; i < momentTemplate.indexArray.length; i++) {
                indexArray[startIndex + i] = momentTemplate.indexArray[i] + startVertex;
            }
            for (i = 0; i < momentTemplate.posArray.length; i += 3) {
                x = momentTemplate.posArray[i + 0];
                y = momentTemplate.posArray[i + 1];
                z = momentTemplate.posArray[i + 2];
                //set length
                if (y < 0 && z > 0) {
                    z += length - momentTemplate.maxY;
                } else {
                    v = new THREE.Vector3(-x, -y, -z);
                    v.normalize();
                    v.multiplyScalar(length - momentTemplate.maxY);
                    y -= v.y;
                    z -= v.z;
                }
                //rotate to dir
                posArray[startVertex*3 + i + 0] = base.baseX.x * x + base.baseY.x * y + base.baseZ.x * z;
                posArray[startVertex*3 + i + 1] = base.baseX.y * x + base.baseY.y * y + base.baseZ.y * z;
                posArray[startVertex*3 + i + 2] = base.baseX.z * x + base.baseY.z * y + base.baseZ.z * z;
                elPosArray[startVertex*3 + i + 0] = arrow.pos.x;
                elPosArray[startVertex*3 + i + 1] = arrow.pos.y;
                elPosArray[startVertex*3 + i + 2] = arrow.pos.z;
                colorArray[startVertex*3 + i + 0] = arrow.color.x;
                colorArray[startVertex*3 + i + 1] = arrow.color.y;
                colorArray[startVertex*3 + i + 2] = arrow.color.z;
            }
            for (i = 0; i < 3 * 2; i++) {
                var a1 = i * 2 * Math.PI / 8;
                var a2 = a1 + 2 * Math.PI / 8;
                var z1 = -Math.sin(a1) * length;
                var y1 = -Math.cos(a1) * length;
                var z2 = -Math.sin(a2) * length;
                var y2 = -Math.cos(a2) * length;
                collider.addCollider(
                    new VIEW3D.collision.ScreenSegmentCollider(
                        new THREE.Vector3(arrow.pos.x, arrow.pos.y, arrow.pos.z),
                        new THREE.Vector3(base.baseY.x * y1 + base.baseZ.x * z1, base.baseY.y * y1 + base.baseZ.y * z1, base.baseY.z * y1 + base.baseZ.z * z1),
                        new THREE.Vector3(arrow.pos.x, arrow.pos.y, arrow.pos.z),
                        new THREE.Vector3(base.baseY.x * y2 + base.baseZ.x * z2, base.baseY.y * y2 + base.baseZ.y * z2, base.baseY.z * y2 + base.baseZ.z * z2)
                    )
                );
            }
        }
        var addLineToBuffers = function (startIndex, startVertex, arrow, collider) {
            var posArray = mesh.geometry.attributes.position.array;
            var elPosArray = mesh.geometry.attributes.elementPos.array;
            var colorArray = mesh.geometry.attributes.color.array;
            var indexArray = mesh.geometry.index.array;
            var x, y, z;
            var dir1 = arrow.dir1.clone().multiplyScalar(VIEW3D.Arrows.maxSize / self.maxArrow);
            var dir2 = arrow.dir2.clone().multiplyScalar(VIEW3D.Arrows.maxSize / self.maxArrow);

            function validateMin(vector) {
                if (vector.x !== 0 && Math.abs(vector.x) < self.minArrow)
                    vector.x = vector.x > 0 ? self.minArrow : -self.minArrow;
                if (vector.y !== 0 && Math.abs(vector.y) < self.minArrow)
                    vector.y = vector.y > 0 ? self.minArrow : -self.minArrow;
                if (vector.z !== 0 && Math.abs(vector.z) < self.minArrow)
                    vector.z = vector.z > 0 ? self.minArrow : -self.minArrow;
            }

            function scaleVector(vec) {
                vec.multiplyScalar(CORE.loadModelScale());
            }

            validateMin(dir1);
            validateMin(dir2);
            scaleVector(dir1);
            scaleVector(dir2);

            var dir = new THREE.Vector3().subVectors(arrow.pos2, arrow.pos1);
            var base = VIEW3D.utils.getBaseFromDir(dir);
            var i;
            for (i = 0; i < lineTemplate.indexArray.length; i++) {
                indexArray[startIndex + i] = lineTemplate.indexArray[i] + startVertex;
            }
            for (i = 0; i < lineTemplate.posArray.length; i += 3) {
                x = lineTemplate.posArray[i + 0];
                y = lineTemplate.posArray[i + 1];
                z = lineTemplate.posArray[i + 2];
                //set length
                if (x > 0.0001) {
                    x = 0;
                    elPosArray[startVertex*3 + i + 0] = arrow.pos1.x;
                    elPosArray[startVertex*3 + i + 1] = arrow.pos1.y;
                    elPosArray[startVertex*3 + i + 2] = arrow.pos1.z;
                    posArray[startVertex*3 + i + 0] = base.baseX.x * x + base.baseY.x * y + base.baseZ.x * z + dir1.x;
                    posArray[startVertex*3 + i + 1] = base.baseX.y * x + base.baseY.y * y + base.baseZ.y * z + dir1.y;
                    posArray[startVertex*3 + i + 2] = base.baseX.z * x + base.baseY.z * y + base.baseZ.z * z + dir1.z;
                } else {
                    x = 0;
                    elPosArray[startVertex*3 + i + 0] = arrow.pos2.x;
                    elPosArray[startVertex*3 + i + 1] = arrow.pos2.y;
                    elPosArray[startVertex*3 + i + 2] = arrow.pos2.z;
                    posArray[startVertex*3 + i + 0] = base.baseX.x * x + base.baseY.x * y + base.baseZ.x * z + dir2.x;
                    posArray[startVertex*3 + i + 1] = base.baseX.y * x + base.baseY.y * y + base.baseZ.y * z + dir2.y;
                    posArray[startVertex*3 + i + 2] = base.baseX.z * x + base.baseY.z * y + base.baseZ.z * z + dir2.z;
                }
                colorArray[startVertex*3 + i + 0] = arrow.color.x;
                colorArray[startVertex*3 + i + 1] = arrow.color.y;
                colorArray[startVertex*3 + i + 2] = arrow.color.z;
            }
            collider.addCollider(
                new VIEW3D.collision.ScreenSegmentCollider(
                    new THREE.Vector3(arrow.pos1.x, arrow.pos1.y, arrow.pos1.z),
                    new THREE.Vector3(dir1.x, dir1.y, dir1.z),
                    new THREE.Vector3(arrow.pos2.x, arrow.pos2.y, arrow.pos2.z),
                    new THREE.Vector3(dir2.x, dir2.y, dir2.z)
                )
            );
        }

        var _parsedObjs = [];

        var getParsedObj = function (obj) {
            if (!_parsedObjs[obj]) {
                _parsedObjs[obj] = new THREE.OBJLoader().parse(obj);
            }
            return _parsedObjs[obj];
        };

        var createTemplate = function (obj, addToBuffers) {
            
            var geometry = getParsedObj(obj).children[0].geometry;
            VIEW3D.utils.optimize(geometry);
            var posArray = geometry.attributes.position.array;
            var vertCount = geometry.attributes.position.array.length/3;
            var indexArray = geometry.getIndex().array;
            var indexCount = indexArray.length;
            var maxX = 0;
            var maxY = 0;
            var maxZ = 0;
            for (var i = 0; i < posArray.length; i += 3) {
                posArray[i + 0] *= VIEW3D.Arrows.templateScale;
                posArray[i + 1] *= VIEW3D.Arrows.templateScale;
                posArray[i + 2] *= VIEW3D.Arrows.templateScale;
                maxX = Math.max(maxX, posArray[i]);
                maxY = Math.max(maxY, posArray[i + 1]);
                maxZ = Math.max(maxZ, posArray[i + 2]);
            }
            return {
                vertCount: vertCount,
                posArray: posArray,
                indexArray: indexArray,
                indexCount: indexCount,
                maxX: maxX,
                maxY: maxY,
                maxZ: maxZ,
                addToBuffers: addToBuffers
            }
        }
        var arrowTemplate = createTemplate(VIEW3D.Arrows.arrowObj, addArrowToBuffers);
        var momentTemplate = createTemplate(VIEW3D.Arrows.momentObj, addMomentToBuffers);
        var lineTemplate = createTemplate(VIEW3D.Arrows.lineObj, addLineToBuffers);
        var getTemplate = function (type) {
            switch (type) {
                case VIEW3D.Arrows.Arrow:
                    return arrowTemplate;
                case VIEW3D.Arrows.Moment:
                    return momentTemplate;
                case VIEW3D.Arrows.Line:
                    return lineTemplate;
                default:
                    throw "unknown arrow type";
            }
        }

        this.getPosMatrix = function () {
            return new THREE.Matrix4().multiplyMatrices(camera.projectionMatrix, camera.matrixWorldInverse).multiply(this.matrixWorld);
        }
        var getDirMatrix = function () {
            var matrixWorldInverse = camera.matrixWorldInverse.clone();
            //remove translation
            matrixWorldInverse.setPosition(new THREE.Vector3(0, 0, 0));
            var projectionMatrix = camera.projectionMatrix.clone();
            //remove zoom
            projectionMatrix.elements[0] /= projectionMatrix.elements[5];
            projectionMatrix.elements[5] = 1.0;
            return new THREE.Matrix4().multiplyMatrices(projectionMatrix, matrixWorldInverse);
        }
        this.collisionTest = function (point2D) {
            for (var id in arrowsData) {

                var params = {
                    mousePos: point2D,
                    matrix: this.getPosMatrix(),
                    matrixSreenOffset: getDirMatrix(),
                }

                var result = [];
                arrowsData[id].collider.intersectMousePos(params, result);
                if (result.length > 0) {
                    return arrowsData[id].userData
                }
            }

        }
        this.addArrows = function (arrows, userData) {
            if (arrows.length <= 0) {
                return 0;
            }
            var arrowsCollider = new VIEW3D.collision.DisjunctionCollider();

            var arrowsId = ++lastArrowsId;
            //how many vertices?
            var newVerticesCount = 0;
            var newIndicesCount = 0;
            var i;
            for (i = 0; i < arrows.length; i++) {
                newVerticesCount += getTemplate(arrows[i].type).vertCount;
                newIndicesCount += getTemplate(arrows[i].type).indexCount;

                arrows[i].originalColor = arrows[i].color;
                arrows[i].selectedColor = new THREE.Vector3(0.7, 0.35, 0.35);

                if (!isEmpty(userData) && typeof userData.selected === 'function' && userData.selected())
                    arrows[i].color = arrows[i].selectedColor;
            }

            var startVertex = mesh.geometry.verticesInUse;
            var startIndex = mesh.geometry.indicesInUse;

            //increase buffers size
            mesh.geometry.increaseSize(newIndicesCount, newVerticesCount);
            //add new arrows to buffer
            for (i = 0; i < arrows.length; i++) {
                var template = getTemplate(arrows[i].type);
                template.addToBuffers(startIndex, startVertex, arrows[i], arrowsCollider);
                startVertex += getTemplate(arrows[i].type).vertCount;
                startIndex += getTemplate(arrows[i].type).indexCount;
            }
            
            arrowsData[arrowsId] = {
                startVertex: mesh.geometry.verticesInUse - newVerticesCount,
                verticesCount: newVerticesCount,
                startIndex: mesh.geometry.indicesInUse - newIndicesCount,
                indicesCount: newIndicesCount,
                data: arrows,
                userData: userData,
                collider: arrowsCollider
            };
            this.updateGeometry();
            return arrowsId;
        }

        this.getCollider = function (arrowsId) {
            return arrowsData[arrowsId].collider;
        }

        this.removeArrows = function (arrowsId) {
            if (arrowsId <= 0) {
                return;
            }
            var arrowData = arrowsData[arrowsId];
            delete arrowsData[arrowsId];

            mesh.geometry.splice(arrowData.startIndex, arrowData.indicesCount, arrowData.startVertex, arrowData.verticesCount);

            for (var id in arrowsData) {
                if (arrowsData.hasOwnProperty(id)) {
                    if (id > arrowsId) {
                        arrowsData[id].startVertex -= arrowData.verticesCount;
                        arrowsData[id].startIndex -= arrowData.indicesCount;
                    }
                }
            }
            this.updateGeometry();
        }
        var willBeUpdated = false;
        this.updateGeometry = function () {
            var self = this;
            if (!willBeUpdated) {
                willBeUpdated = true;
                setTimeout(function () {
                    var newMaxArrow = 0;
                    var newMaxMoment = 0;
                    for (var id in arrowsData) {
                        if (arrowsData.hasOwnProperty(id)) {
                            var arrows = arrowsData[id];
                            for (var i = 0; i < arrows.data.length; i++) {
                                var data = arrows.data[i];
                                switch (data.type) {
                                    case VIEW3D.Arrows.Arrow:
                                        newMaxArrow = Math.max(newMaxArrow, data.dir.length());
                                        break;
                                    case VIEW3D.Arrows.Moment:
                                        newMaxMoment = Math.max(newMaxMoment, data.dir.length());
                                        break;
                                }
                            }
                        }
                    }
                    if (Math.abs(self.maxArrow - newMaxArrow) > 0.0000001 ||
                        Math.abs(maxMoment - newMaxMoment) > 0.0000001) {
                        self.maxArrow = Math.max(customMaxArrow || 0, newMaxArrow);
                        maxMoment = newMaxMoment;
                        self.updateAllArrows();
                    }
                    mesh.geometry.attributes.position.needsUpdate = true;
                    mesh.geometry.attributes.elementPos.needsUpdate = true;
                    mesh.geometry.attributes.color.needsUpdate = true;
                    mesh.geometry.index.needsUpdate = true;

                    mesh.visible = mesh.geometry.verticesInUse > 0;

                    render();
                    willBeUpdated = false;
                }, 5);
            }
        }
        this.updateAllArrows = function () {
            for (var id in arrowsData) {
                if (arrowsData.hasOwnProperty(id)) {
                    this.updateArrows(id);
                }
            }

            if (CORE.constructionView3D && CORE.constructionView3D.loadsObject3D)
                CORE.constructionView3D.loadsObject3D.fills.updateAllFills();  // update fills after calculation of arrows (refactor?)
        }
        this.updateArrows = function (arrowsId, newData) {
            if (isEmpty(newData)) {
                newData = arrowsData[arrowsId].data;
            }
            var startVertex = arrowsData[arrowsId].startVertex;
            var startIndex = arrowsData[arrowsId].startIndex;
            arrowsData[arrowsId].collider.removeAllColliders();
            for (var i = 0; i < newData.length; i++) {
                var template = getTemplate(newData[i].type);
                template.addToBuffers(startIndex, startVertex, newData[i], arrowsData[arrowsId].collider);
                startVertex += template.vertCount;
                startIndex += template.indexCount;
            }
            this.updateGeometry();
        }
        this.getArrowsData = function (arrowsId) {
            return arrowsData[arrowsId];
        }
        this.getGeometry = function () {
            return mesh.geometry;
        }

        // CORE.loadModelScale.subscribe(newVal => {
        //     self.updateAllArrows();
        // });
    }
    VIEW3D.Arrows.templateScale = 0.1;
    VIEW3D.Arrows.maxSize = 0.2;
    VIEW3D.Arrows.minSize = 0.08;
    VIEW3D.Arrows.Arrow = 0;
    VIEW3D.Arrows.Moment = 1;
    VIEW3D.Arrows.Line = 2;
    VIEW3D.Arrows.prototype = Object.create(THREE.Object3D.prototype);
    VIEW3D.Arrows.arrowObj =
        'v 0.000000 0.014299 -0.012736\n\
    v 2.000000 0.014299 -0.012736\n\
    v 0.000000 -0.014299 -0.012736\n\
    v 2.000000 -0.014299 -0.012736\n\
    v 0.000000 0.014299 0.012736\n\
    v 2.000000 0.014299 0.012736\n\
    v 0.000000 -0.014299 0.012736\n\
    v 2.000000 -0.014299 0.012736\n\
    v 1.940361 -0.143945 -0.003707\n\
    v 1.940361 -0.103112 0.096657\n\
    v 1.940361 -0.004534 0.138230\n\
    v 1.940361 0.094045 0.096657\n\
    v 1.940361 0.134877 -0.003707\n\
    v 1.940361 0.094045 -0.104071\n\
    v 1.940361 -0.004534 -0.145644\n\
    v 2.504914 -0.004534 -0.003707\n\
    v 1.940361 -0.103112 -0.104071\n\
    f 4 3 1\n\
    f 8 7 3\n\
    f 6 5 7\n\
    f 2 1 5\n\
    f 3 7 5\n\
    f 15 16 17\n\
    f 9 16 10\n\
    f 14 16 15\n\
    f 13 16 14\n\
    f 12 16 13\n\
    f 11 16 12\n\
    f 17 16 9\n\
    f 10 16 11\n\
    f 13 14 17\n\
    f 2 4 1\n\
    f 4 8 3\n\
    f 8 6 7\n\
    f 6 2 5\n\
    f 1 3 5\n\
    f 17 9 13\n\
    f 10 11 12\n\
    f 12 13 9\n\
    f 14 15 17\n\
    f 9 10 12\n\
    ';
    VIEW3D.Arrows.lineObj =
        'v 2.000000 0.012736 -0.014299\n\
    v 0.000000 0.012736 -0.014299\n\
    v 0.000000 0.012736 0.014299\n\
    v 2.000000 -0.012736 -0.014299\n\
    v 0.000000 -0.012736 -0.014299\n\
    v 2.000000 -0.012736 0.014299\n\
    v 0.000000 -0.012736 0.014299\n\
    v 2.000000 0.012736 0.014299\n\
    s off\n\
    f 1 2 3\n\
    f 4 5 2\n\
    f 6 7 5\n\
    f 8 3 7\n\
    f 2 5 7\n\
    f 8 1 3\n\
    f 1 4 2\n\
    f 4 6 5\n\
    f 6 8 7\n\
    f 3 2 7\n\
    ';

    VIEW3D.Arrows.momentObj =
        'v 0.000000 0.000000 -2.540000\n\
    v 0.030000 0.000000 -2.510000\n\
    v 0.000000 0.000000 -2.480000\n\
    v -0.030000 0.000000 -2.510000\n\
    v 0.000000 0.784903 -2.415684\n\
    v 0.030000 0.775632 -2.387152\n\
    v 0.000000 0.766362 -2.358620\n\
    v -0.030000 0.775632 -2.387152\n\
    v 0.000000 1.492975 -2.054903\n\
    v 0.030000 1.475341 -2.030633\n\
    v 0.000000 1.457708 -2.006362\n\
    v -0.030000 1.475341 -2.030633\n\
    v 0.000000 2.054903 -1.492974\n\
    v 0.030000 2.030633 -1.475340\n\
    v 0.000000 2.006362 -1.457707\n\
    v -0.030000 2.030633 -1.475340\n\
    v 0.000000 2.415684 -0.784904\n\
    v 0.030000 2.387152 -0.775633\n\
    v 0.000000 2.358620 -0.766363\n\
    v -0.030000 2.387152 -0.775633\n\
    v 0.000000 2.540000 -0.000000\n\
    v 0.030000 2.510000 -0.000000\n\
    v 0.000000 2.480000 -0.000000\n\
    v -0.030000 2.510000 -0.000000\n\
    v -0.000000 2.415684 0.784903\n\
    v 0.030000 2.387152 0.775633\n\
    v -0.000000 2.358620 0.766362\n\
    v -0.030000 2.387152 0.775633\n\
    v -0.000000 2.054903 1.492975\n\
    v 0.030000 2.030632 1.475341\n\
    v -0.000000 2.006362 1.457708\n\
    v -0.030000 2.030632 1.475341\n\
    v -0.000000 1.492974 2.054903\n\
    v 0.030000 1.475340 2.030633\n\
    v -0.000000 1.457707 2.006362\n\
    v -0.030000 1.475340 2.030633\n\
    v -0.000000 0.784903 2.415684\n\
    v 0.030000 0.775633 2.387152\n\
    v -0.000000 0.766362 2.358620\n\
    v -0.030000 0.775633 2.387152\n\
    v -0.000000 0.000000 2.540000\n\
    v 0.030000 0.000000 2.510000\n\
    v -0.000000 0.000000 2.480000\n\
    v -0.030000 0.000000 2.510000\n\
    v 0.000000 -2.540000 -0.000000\n\
    v 0.030000 -2.510000 -0.000000\n\
    v 0.000000 -2.480000 -0.000000\n\
    v -0.030000 -2.510000 -0.000000\n\
    v 0.000000 -2.415684 -0.784904\n\
    v 0.030000 -2.387152 -0.775633\n\
    v 0.000000 -2.358620 -0.766363\n\
    v -0.030000 -2.387152 -0.775633\n\
    v 0.000000 -2.054904 -1.492973\n\
    v 0.030000 -2.030634 -1.475340\n\
    v 0.000000 -2.006363 -1.457706\n\
    v -0.030000 -2.030634 -1.475340\n\
    v 0.000000 -1.492975 -2.054902\n\
    v 0.030000 -1.475342 -2.030632\n\
    v 0.000000 -1.457708 -2.006361\n\
    v -0.030000 -1.475342 -2.030632\n\
    v 0.000000 -0.784904 -2.415683\n\
    v 0.030000 -0.775633 -2.387151\n\
    v 0.000000 -0.766363 -2.358620\n\
    v -0.030000 -0.775633 -2.387151\n\
    v 0.004534 -0.000466 2.651673\n\
    v 0.004534 -0.565019 2.509736\n\
    v 0.103112 -0.000466 2.610100\n\
    v 0.143945 -0.000466 2.509736\n\
    v 0.103112 -0.000466 2.409372\n\
    v -0.094045 -0.000466 2.610100\n\
    v -0.134877 -0.000466 2.509736\n\
    v -0.094045 -0.000466 2.409372\n\
    v 0.004534 -0.000466 2.367799\n\
    s off\n\
    f 5 6 2\n\
    f 2 6 7\n\
    f 7 8 4\n\
    f 4 8 5\n\
    f 9 10 6\n\
    f 6 10 11\n\
    f 11 12 8\n\
    f 12 9 5\n\
    f 13 14 10\n\
    f 10 14 15\n\
    f 15 16 12\n\
    f 16 13 9\n\
    f 17 18 14\n\
    f 18 19 15\n\
    f 19 20 16\n\
    f 16 20 17\n\
    f 21 22 18\n\
    f 22 23 19\n\
    f 23 24 20\n\
    f 24 21 17\n\
    f 25 26 22\n\
    f 26 27 23\n\
    f 27 28 24\n\
    f 28 25 21\n\
    f 29 30 26\n\
    f 26 30 31\n\
    f 31 32 28\n\
    f 32 29 25\n\
    f 33 34 30\n\
    f 34 35 31\n\
    f 35 36 32\n\
    f 36 33 29\n\
    f 33 37 38\n\
    f 38 39 35\n\
    f 39 40 36\n\
    f 36 40 37\n\
    f 37 41 42\n\
    f 42 43 39\n\
    f 43 44 40\n\
    f 44 41 37\n\
    f 45 49 50\n\
    f 50 51 47\n\
    f 47 51 52\n\
    f 52 49 45\n\
    f 53 54 50\n\
    f 54 55 51\n\
    f 55 56 52\n\
    f 56 53 49\n\
    f 53 57 58\n\
    f 54 58 59\n\
    f 55 59 60\n\
    f 60 57 53\n\
    f 61 62 58\n\
    f 58 62 63\n\
    f 59 63 64\n\
    f 64 61 57\n\
    f 61 1 2\n\
    f 2 3 63\n\
    f 3 4 64\n\
    f 64 4 1\n\
    f 65 66 67\n\
    f 68 66 69\n\
    f 70 66 65\n\
    f 71 66 70\n\
    f 72 66 71\n\
    f 73 66 72\n\
    f 67 66 68\n\
    f 69 66 73\n\
    f 71 70 67\n\
    f 67 68 71\n\
    f 69 73 72\n\
    f 72 71 68\n\
    f 70 65 67\n\
    f 68 69 72\n\
    f 1 5 2\n\
    f 3 2 7\n\
    f 3 7 4\n\
    f 1 4 5\n\
    f 5 9 6\n\
    f 7 6 11\n\
    f 7 11 8\n\
    f 8 12 5\n\
    f 9 13 10\n\
    f 11 10 15\n\
    f 11 15 12\n\
    f 12 16 9\n\
    f 13 17 14\n\
    f 14 18 15\n\
    f 15 19 16\n\
    f 13 16 17\n\
    f 17 21 18\n\
    f 18 22 19\n\
    f 19 23 20\n\
    f 20 24 17\n\
    f 21 25 22\n\
    f 22 26 23\n\
    f 23 27 24\n\
    f 24 28 21\n\
    f 25 29 26\n\
    f 27 26 31\n\
    f 27 31 28\n\
    f 28 32 25\n\
    f 29 33 30\n\
    f 30 34 31\n\
    f 31 35 32\n\
    f 32 36 29\n\
    f 34 33 38\n\
    f 34 38 35\n\
    f 35 39 36\n\
    f 33 36 37\n\
    f 38 37 42\n\
    f 38 42 39\n\
    f 39 43 40\n\
    f 40 44 37\n\
    f 46 45 50\n\
    f 46 50 47\n\
    f 48 47 52\n\
    f 48 52 45\n\
    f 49 53 50\n\
    f 50 54 51\n\
    f 51 55 52\n\
    f 52 56 49\n\
    f 54 53 58\n\
    f 55 54 59\n\
    f 56 55 60\n\
    f 56 60 53\n\
    f 57 61 58\n\
    f 59 58 63\n\
    f 60 59 64\n\
    f 60 64 57\n\
    f 62 61 2\n\
    f 62 2 63\n\
    f 63 3 64\n\
    f 61 64 1\n\
    ';
});