﻿'use strict';
var VIEW3D = VIEW3D || {};

define(['js/view3d/materials/VIEW3D.materials.AxisLabel'], function () {
    // "view options"
    VIEW3D.AxisHelper = {
        renderer: null,
        camera: null,
        scene: null,
        camera2d: null,
        scene2d: null,
        arrows: null,
        cube: null,
        label: null,
        object3d: null,
        plane: null,
        getContainer: function () { return $("#axisHelperWebGlContainer"); },
        getMainContainer: function () { return $("#axisHelperWebGlContainer"); },
        cubeMaterialNormal: new THREE.MeshBasicMaterial({ color: 0xe0e0e0, transparent: false, opacity: 1, side: THREE.DoubleSide, depthWrite: true }),
        cubeEdgesMaterialNormal: new THREE.MeshBasicMaterial({ color: 0x7f7f7f, transparent: false, opacity: 1, side: THREE.DoubleSide, depthWrite: true }),
        cubeMaterialSelected: new THREE.MeshBasicMaterial({ color: 0x373737, transparent: false, opacity: 1, side: THREE.DoubleSide, depthWrite: true }),
        cubeEdgesLinesMaterial: new THREE.LineBasicMaterial({ color: 0xacacac/*0x6969fb*/, transparent: false, opacity: 1 }),
        arrows2dMaterialNormal: new THREE.MeshBasicMaterial({ color: 0x7f7f7f, transparent: false, opacity: 1, side: THREE.DoubleSide, depthWrite: false }),
        arrows2d: null,
        arrows2dColor: 0x0000ff,
        faceHighlighted: false,
        fontTexture: null,
        init: function () {
            this.initRenderer();
            this.init2dArrows();
            var arrowLength = 1.4;
            var arrowHeadSize = 1.1;
            var cubeSize = 1;
            var labelSize = 10;
            var labelOffset = 0.2;
            this.object3d = new THREE.Object3D();
            this.arrows = new this.Arrows(arrowLength, arrowHeadSize);
            this.cube = new this.Cube(cubeSize);
            this.fontTexture = new THREE.Texture(CORE.bitmapFont.texture.image);
            this.fontTexture.needsUpdate = true;
            this.labels = new THREE.Object3D();
            this.labels.add(new this.Label(new THREE.Vector3(arrowLength + labelOffset, 0, 0).addScalar(-0.5), new THREE.Color(1, 0, 0), "X", labelSize, this.fontTexture, this.screenSize));
            this.labels.add(new this.Label(new THREE.Vector3(0, arrowLength + labelOffset, 0).addScalar(-0.5), new THREE.Color(0, 1, 0), "Y", labelSize, this.fontTexture, this.screenSize));
            this.labels.add(new this.Label(new THREE.Vector3(0, 0, arrowLength + labelOffset).addScalar(-0.5), new THREE.Color(0, 0, 1), "Z", labelSize, this.fontTexture, this.screenSize));
            //this.labels.renderOrder = 1;
            //this.cube.up = this.cube.children[0].up = this.cube.children[1].up = new THREE.Vector3(0, 0, 1);
            this.object3d.add(this.arrows);
            this.object3d.add(this.cube);
            this.scene.add(this.labels);
            this.scene.add(this.object3d);

            //this.object3d.position.copy(camera.position); // copy from main camera
            this.object3d.up.copy(camera.up);
            //this.object3d.rotation.copy(camera.rotation);
            this.object3d.updateMatrix();
            this.camera.matrixAutoUpdate = false;

            this.initEventListeners();
        },
        initRenderer: function () {
            console.log("VIEW3D.AxisHelper.initRenderer() begin");
            var axisHelperWebGlContainer = this.getContainer()[0];
            if (axisHelperWebGlContainer === null || axisHelperWebGlContainer === 'undefined') {
                console.error("axisHelperWebGlContainer is null or undefined!");
            }
            this.scene = new THREE.Scene();
            var cubeCameraSize = 1.9; // smaller camera view = bigger object
            this.camera = new THREE.OrthographicCamera(-cubeCameraSize, cubeCameraSize, cubeCameraSize, -cubeCameraSize, -1000, 1000);
            //this.scene.add(this.camera);
            this.scene2d = new THREE.Scene();
            var arrows2dCameraSize = 0.9;
            this.camera2d = new THREE.OrthographicCamera(-arrows2dCameraSize, arrows2dCameraSize, arrows2dCameraSize, -arrows2dCameraSize, -1000, 1000);
            try {
                this.renderer = new THREE.WebGLRenderer({ antialias: true, alpha: true });
            } catch(err) {
                this.renderer = new THREE.WebGLRenderer({ antialias: false, alpha: true });
            }
            var height = parseInt(window.getComputedStyle(axisHelperWebGlContainer, null).getPropertyValue("height"));
            var width = parseInt(window.getComputedStyle(axisHelperWebGlContainer, null).getPropertyValue("width"));
            this.screenSize = ko.observable({ width: width, height: height });
            this.renderer.setSize(width, height);
            this.renderer.setClearColor(0x000000, 0); // set to 0 for transparent background
            $(axisHelperWebGlContainer).empty();
            axisHelperWebGlContainer.appendChild(this.renderer.domElement);
            this.renderer.autoClear = false;
            this.camera.position.copy(camera.position); // copy from main camera
            this.camera.up.copy(camera.up);
            this.camera.rotation.copy(camera.rotation);
            this.camera.lookAt(new THREE.Vector3(0, 0, 0));
            this.camera.updateMatrix();
            this.camera.updateProjectionMatrix();
            console.log("VIEW3D.AxisHelper.initRenderer() end");
        },
        init2dArrows: function () {

            // ARC ARROWS
            var leftArcStartAngle = 0.7 * Math.PI; // Starting point on circle
            var leftArcEndAngle = Math.PI; // End point on circle

            var leftArc = new THREE.Shape();
            leftArc.absarc(0, 0, 0.9, leftArcStartAngle, leftArcEndAngle, false);
            leftArc.absarc(0, 0, 0.8, leftArcEndAngle, leftArcStartAngle, true);

            var leftArcArrow = new THREE.Shape();
            leftArcArrow.moveTo(-0.9, 0);
            leftArcArrow.lineTo(-1, 0);
            leftArcArrow.lineTo(-0.85, -0.3);
            leftArcArrow.lineTo(-0.7, 0);
            leftArcArrow.lineTo(-0.8, 0);

            var leftArcGeometry = new THREE.ShapeGeometry([leftArc, leftArcArrow]);
            leftArcGeometry.rotateZ(-0.1 * Math.PI);

            var scale = 0.5;
            leftArcGeometry.scale(scale, scale, scale);
            leftArcGeometry.center();
            leftArcGeometry.translate(-0.5, 0.5, 0);       //after updating three.js, replace with:-0.5, 0.5, 0
            var rightArcGeometry = leftArcGeometry.clone();
            rightArcGeometry.rotateY(Math.PI);



            // DIR ARROWS 
            var triangleSide = 0.3;
            var triangleHeight = triangleSide * 0.8660254037844386;         // 0.866... =~ sqrt(3) / 2

            var leftArrow = new THREE.Shape();
            leftArrow.moveTo(0, 0);
            leftArrow.lineTo(0, triangleSide);
            leftArrow.lineTo(-triangleHeight, triangleSide / 2);
            leftArrow.lineTo(0, 0);

            var leftArrowGeometryBase = new THREE.ShapeGeometry(leftArrow);
            scale = 0.5;
            leftArrowGeometryBase.scale(scale, scale, scale);
            leftArrowGeometryBase.center();

            var distanceFromCenter = 0.75;

            var leftArrowGeometry = leftArrowGeometryBase.clone();
            leftArrowGeometry.translate(-distanceFromCenter, 0, 0);

            var rightArrowGeometry = leftArrowGeometryBase.clone();
            rightArrowGeometry.rotateZ(Math.PI);
            rightArrowGeometry.translate(distanceFromCenter, 0, 0);

            var upArrowGeometry = leftArrowGeometryBase.clone();
            upArrowGeometry.rotateZ(-0.5 * Math.PI);
            upArrowGeometry.translate(0, distanceFromCenter, 0);

            var downArrowGeometry = leftArrowGeometryBase.clone();
            downArrowGeometry.rotateZ(0.5 * Math.PI);
            downArrowGeometry.translate(0, -distanceFromCenter, 0);

            var dirArrowsGeometry = new THREE.Geometry();
            dirArrowsGeometry.merge(leftArrowGeometry);
            dirArrowsGeometry.merge(rightArrowGeometry);
            dirArrowsGeometry.merge(upArrowGeometry);
            dirArrowsGeometry.merge(downArrowGeometry);



            // MERGE ALL GEOMETRIES
            var shapeGeometry = new THREE.Geometry();
            shapeGeometry.merge(dirArrowsGeometry);
            shapeGeometry.merge(leftArcGeometry);
            shapeGeometry.merge(rightArcGeometry);


            var shapeMaterials = [];
            for (var i = 0; i < shapeGeometry.faces.length; i++) {
                shapeMaterials.push(this.arrows2dMaterialNormal);
                shapeGeometry.faces[i].materialIndex = i;
            }

            this.arrows2d = new THREE.Mesh(shapeGeometry, shapeMaterials);
            this.arrows2d.material.side = THREE.DoubleSide;     //fixing raycaster + MeshFaceMaterial bug in three.js

            this.scene2d.add(this.arrows2d);
        },
        Arrows: function (arrowLength, arrowHeadSize) {
            THREE.Object3D.call(this);
            var headsGeometry = new THREE.Geometry();
            var headsMaterial = new THREE.MeshBasicMaterial({
                vertexColors: THREE.FaceColors,
                depthWrite: true,
                depthTest: true
            });
            
            var lines = new THREE.Object3D();

            var addArrow = function (vec, color) {
                //1. add arrow head to headsGeometry
                var radius = 0.075 * arrowHeadSize;
                var height = 0.25 * arrowHeadSize;
                var segments = 8;
                var coneGeom = new THREE.CylinderGeometry(0, radius, height, segments);
                var v1 = new THREE.Vector3(0, 1, 0);
                var v2 = new THREE.Vector3(vec.x, vec.y, vec.z);
                v2.normalize();
                var q = new THREE.Quaternion().setFromUnitVectors(v1, v2);

                var m = new THREE.Matrix4();
                // read from the bottom
                m.multiply(new THREE.Matrix4().makeTranslation(vec.x - 0.5, vec.y - 0.5, vec.z - 0.5)); // move it to the end of vec
                m.multiply(new THREE.Matrix4().makeRotationFromQuaternion(q)); // rotate it so that it shows vec direction
                m.multiply(new THREE.Matrix4().makeTranslation(0, -height * 0.5, 0)); // move cone down so that it top is in (0,0,0)
                var startIndex = headsGeometry.faces.length;
                headsGeometry.merge(coneGeom, m);
                for (var i = startIndex; i < headsGeometry.faces.length; i++) {
                    headsGeometry.faces[i].color = color;
                }
                //2. add line to linesGeometry
                //linesGeometry.vertices.push(new THREE.Vector3(-0.5, -0.5, -0.5));
                //linesGeometry.vertices.push(new THREE.Vector3(vec.x - 0.5, vec.y - 0.5, vec.z - 0.5));
                //linesGeometry.colors.push(color);
                //linesGeometry.colors.push(color);

                var lineLength = vec.x || vec.y || vec.z;
                var lineThickness = 0.03;
                m = new THREE.Matrix4();
                m.multiply(new THREE.Matrix4().makeTranslation(vec.x - 0.5, vec.y - 0.5, vec.z - 0.5)); // move it to the end of vec
                m.multiply(new THREE.Matrix4().makeRotationFromQuaternion(q)); // rotate it so that it shows vec direction
                m.multiply(new THREE.Matrix4().makeTranslation(0, -(lineLength + 0.1) / 2, 0)); // move cone down so that it top is in (0,0,0)

                var linesGeometry = new THREE.CylinderBufferGeometry(lineThickness, lineThickness, lineLength - 0.1, 8);    // why - 0.1 is needed?
                linesGeometry.applyMatrix(m);

                var linesMaterial = new THREE.MeshBasicMaterial({
                    color: color,
                    depthWrite: true,
                    depthTest: true
                });

                lines.add(new THREE.Mesh(linesGeometry, linesMaterial));
            };
            addArrow(new MODEL.types.Vector(arrowLength, 0, 0), new THREE.Color(1, 0, 0));
            addArrow(new MODEL.types.Vector(0, arrowLength, 0), new THREE.Color(0, 1, 0));
            addArrow(new MODEL.types.Vector(0, 0, arrowLength), new THREE.Color(0, 0, 1));
            this.heads = new THREE.Mesh(headsGeometry, headsMaterial);
            this.lines = lines;
            this.lines.renderOrder = 100;
            this.heads.renderOrder = 100;
            this.add(this.lines);
            this.add(this.heads);
        },
        Cube: function () {
            THREE.Object3D.call(this);

            var cubeGeometry = new THREE.BoxGeometry(1, 1, 1, 3, 3, 3);

            var existingSegmentPosition = 0.1666666716337204;   // =~ 0.5 / 3

            for (var i = 0; i < cubeGeometry.vertices.length; i++) {
                if (Math.abs(cubeGeometry.vertices[i].x) === existingSegmentPosition)
                    cubeGeometry.vertices[i].x *= 2;

                if (Math.abs(cubeGeometry.vertices[i].y) === existingSegmentPosition)
                    cubeGeometry.vertices[i].y *= 2;

                if (Math.abs(cubeGeometry.vertices[i].z) === existingSegmentPosition)
                    cubeGeometry.vertices[i].z *= 2;
            }

            var cubeMaterials = [];

            for (i = 0; i < cubeGeometry.faces.length; i++) {
                var intersectionObject = VIEW3D.AxisHelper.getIntersectionObjectCube(i);
                if (intersectionObject.type === "edge") {
                    cubeMaterials.push(VIEW3D.AxisHelper.cubeEdgesMaterialNormal);
                    cubeGeometry.faces[i].isEdge = true;
                }
                else
                    cubeMaterials.push(VIEW3D.AxisHelper.cubeMaterialNormal);

                cubeGeometry.faces[i].materialIndex = i;
            }

            this.cube = new THREE.Mesh(cubeGeometry, cubeMaterials);
            //this.cube.renderOrder = 20;

            //var edgesHelper = new THREE.EdgesHelper(this.cube);     // temporary helper object
            //this.edgesHelper.renderOrder = 20;
            //this.edgesHelper.material.depthWrite = false;

            var edgePosition = existingSegmentPosition;

            var edgesGeometry = new THREE.Geometry();//edgesHelper.geometry.clone();       // take geometry from EdgesHelper

            edgesGeometry.vertices.push(
                new THREE.Vector3(-0.5, -0.5, -0.5),
                new THREE.Vector3(-0.5 + edgePosition, -0.5, -0.5),

                new THREE.Vector3(-0.5, -0.5, -0.5),
                new THREE.Vector3(-0.5, -0.5 + edgePosition, -0.5),

                new THREE.Vector3(-0.5, -0.5, -0.5),
                new THREE.Vector3(-0.5, -0.5, -0.5 + edgePosition),


                new THREE.Vector3(0.5, -0.5, -0.5),
                new THREE.Vector3(0.5 - edgePosition, -0.5, -0.5),

                new THREE.Vector3(0.5, -0.5, -0.5),
                new THREE.Vector3(0.5, -0.5 + edgePosition, -0.5),

                new THREE.Vector3(0.5, -0.5, -0.5),
                new THREE.Vector3(0.5, -0.5, -0.5 + edgePosition),


                new THREE.Vector3(-0.5, 0.5, -0.5),
                new THREE.Vector3(-0.5 + edgePosition, 0.5, -0.5),

                new THREE.Vector3(-0.5, 0.5, -0.5),
                new THREE.Vector3(-0.5, 0.5 - edgePosition, -0.5),

                new THREE.Vector3(-0.5, 0.5, -0.5),
                new THREE.Vector3(-0.5, 0.5, -0.5 + edgePosition),


                new THREE.Vector3(0.5, 0.5, -0.5),
                new THREE.Vector3(0.5 - edgePosition, 0.5, -0.5),

                new THREE.Vector3(0.5, 0.5, -0.5),
                new THREE.Vector3(0.5, 0.5 - edgePosition, -0.5),

                new THREE.Vector3(0.5, 0.5, -0.5),
                new THREE.Vector3(0.5, 0.5, -0.5 + edgePosition)

            );

            var edgesGeometry2 = edgesGeometry.clone();
            edgesGeometry2.rotateY(Math.PI);
            
            edgesGeometry.merge(edgesGeometry2);

            //EDGES GEOMETRY GENERATION (on-cube helper lines)

            this.edges = new THREE.LineSegments(edgesGeometry, VIEW3D.AxisHelper.cubeEdgesLinesMaterial);

            // ON-CUBE LABELS

            VIEW3D.AxisHelper.fontTexture = new THREE.Texture(CORE.bitmapFont.texture.image);   // why do I have to write this again?
            VIEW3D.AxisHelper.fontTexture.needsUpdate = true;
            VIEW3D.AxisHelper.fontTexture.anisotrophy = 16;

            var cubeLabelSize = 0.004;
            var texture = VIEW3D.AxisHelper.fontTexture;
            var screenSize = VIEW3D.AxisHelper.screenSize;
            this.cubeLabels = new THREE.Object3D();
            var cubeHalfSize = 0.51;
            var cubeLabelColor = 0x000000;//0x6969fb;
            var mult = cubeLabelSize / 0.005;
            this.cubeLabels.add(
                new VIEW3D.AxisHelper.CubeLabel(new THREE.Vector3(0, 0, cubeHalfSize), new THREE.Euler(Math.PI, 0, 0), cubeLabelColor, 'TOP', cubeLabelSize, texture, screenSize),
                new VIEW3D.AxisHelper.CubeLabel(new THREE.Vector3(0.68, 0, -0.41 / mult), new THREE.Euler(0, 0, Math.PI), cubeLabelColor, 'BOTTOM', cubeLabelSize, texture, screenSize),
                new VIEW3D.AxisHelper.CubeLabel(new THREE.Vector3(cubeHalfSize + 0.25, -0.185, -0.05 * mult), new THREE.Euler(-Math.PI / 2, -Math.PI / 2, 0), cubeLabelColor, 'RIGHT', cubeLabelSize, texture, screenSize),
                new VIEW3D.AxisHelper.CubeLabel(new THREE.Vector3(-0.31, 0.24, -0.05 / mult), new THREE.Euler(-Math.PI / 2, Math.PI / 2, 0), cubeLabelColor, 'LEFT', cubeLabelSize, texture, screenSize),
                new VIEW3D.AxisHelper.CubeLabel(new THREE.Vector3(0.01, -cubeHalfSize + 0.03, -0.04), new THREE.Euler(-Math.PI / 2, 0, 0), cubeLabelColor, 'FRONT', cubeLabelSize, texture, screenSize),
                new VIEW3D.AxisHelper.CubeLabel(new THREE.Vector3(0.44, cubeHalfSize + 0.08, -0.05), new THREE.Euler(-Math.PI / 2, Math.PI, 0), cubeLabelColor, 'BACK', cubeLabelSize, texture, screenSize)
                );

            this.add(this.cube);
            this.add(this.cubeLabels);
            this.add(this.edges);


            // SHADOW
            var planeGeometry = new THREE.PlaneGeometry(2, 2);

            var planeMaterial = new THREE.ShadowMaterial();
            planeMaterial.opacity = 0.3;

            var plane = new THREE.Mesh(planeGeometry, planeMaterial);

            plane.position.set(0, 0, -0.6);
            plane.renderOrder = 1;

            VIEW3D.AxisHelper.plane = plane;

            this.add(VIEW3D.AxisHelper.plane); // plane added separately, not included in this.object3d

            var light = new THREE.SpotLight(0xffffff, 1);//( color, intensity, distance, angle, penumbra, decay )

            light.position.set(-1, -1, 10);
            light.target.position.set(0, 0, 0);

            light.shadow.bias = -0.000222;
            //light.shadow.radius = 2;
            //light.shadow.mapSize.x = 1024;
            //light.shadow.mapSize.y = 1024;

            this.add(light);

            light.castShadow = true;
            this.cube.castShadow = true;
            plane.receiveShadow = true;

            VIEW3D.AxisHelper.renderer.shadowMap.enabled = true;
            VIEW3D.AxisHelper.renderer.shadowMap.type = THREE.PCFSoftShadowMap;
        },
        Label: function (vec, color, text, textSize, texture, screenSize) {

            THREE.Mesh.call(this, this.geometry, this.material);

            var bitmapFont = CORE.bitmapFont;
            var p1 = vec;
            var p2 = vec;
            this.geometry = new THREE.BufferGeometry();
            this.material = new VIEW3D.materials.AxisLabel(screenSize, texture, new THREE.Vector4(color.r, color.g, color.b, 1));
            this.geometry = bitmapFont.createText(text);
            this.geometry.addAttribute('begin', new THREE.BufferAttribute(undefined, 3));
            this.geometry.addAttribute('end', new THREE.BufferAttribute(undefined, 3));
            this.geometry.update({
                text: text.toString(),
                align: 'center',
                font: bitmapFont.font
            });
            var positions = this.geometry.attributes.position.array;
            var vertices = positions.length / 2;
            var halfW = this.geometry.layout._width / 2;
            var halfH = this.geometry.layout._height / 2;
            var begins = new Float32Array(vertices * 3);
            var ends = new Float32Array(vertices * 3);
            for (var i = 0; i < vertices; i++) {
                begins[i * 3] = p1.x;
                begins[i * 3 + 1] = p1.y;
                begins[i * 3 + 2] = p1.z;
                ends[i * 3] = p2.x;
                ends[i * 3 + 1] = p2.y;
                ends[i * 3 + 2] = p2.z;
                positions[i * 2] = (positions[i * 2] - halfW) * textSize / halfH;
                positions[i * 2 + 1] = -(positions[i * 2 + 1] + halfH) * textSize / halfH;
            }
            this.geometry.attributes.begin.array = begins;
            this.geometry.attributes.begin.needsUpdate = true;
            this.geometry.attributes.end.array = ends;
            this.geometry.attributes.end.needsUpdate = true;
        },

        CubeLabel: function(position, rotation, color, text, textSize, texture) {
            THREE.Mesh.call(this);

            var bitmapFont = CORE.bitmapFont;
            this.material = new THREE.MeshBasicMaterial({ map: texture, color: color, transparent: true });
            this.geometry = bitmapFont.createText(text);
            this.geometry.update({
                text: text,
                align: 'left',
                font: bitmapFont.font
            });

            this.rotation.copy(rotation);

            this.geometry.computeBoundingBox();

            var center = this.geometry.boundingBox.getCenter();
            var centerSmall = center.clone().multiplyScalar(textSize, textSize, textSize);

            this.scale.set(textSize, textSize, textSize);
            this.position.add(new THREE.Vector3(-centerSmall.x, centerSmall.y, centerSmall.z));               // center the text (why geometry.translate doesn't work?)
            this.position.add(position);
        },

        highlightFaces: function (facesArr) {

            for (var i = 0, ii = facesArr.length; i < ii; i++) {
                this.cube.cube.material[facesArr[i]] = this.cubeMaterialSelected;
                this.cube.cube.material[facesArr[i] + 1] = this.cubeMaterialSelected;
            }

            this.faceHighlighted = true;
        },

        highlight2dArrow: function (faceIndexFrom, faceIndexTo) {

            for (var i = faceIndexFrom, ii = faceIndexTo; i <= ii; i++) {
                this.arrows2d.material[i] = this.cubeMaterialSelected;
            }

            this.faceHighlighted = true;

        },

        clearHighlights: function () {
            if (this.faceHighlighted) {
                for (var i = 0; i < this.cube.cube.geometry.faces.length; i++) {                          // reset material to default
                    if (this.cube.cube.geometry.faces[i].isEdge)
                        this.cube.cube.material[i] = this.cubeEdgesMaterialNormal;
                    else
                        this.cube.cube.material[i] = this.cubeMaterialNormal;
                }

                for (i = 0; i < this.arrows2d.geometry.faces.length; i++) {
                    this.arrows2d.material[i] = this.arrows2dMaterialNormal;
                }

                this.faceHighlighted = false;
            }
        },

        getIntersectionObjectCube: function (intersectedFaceIndex) {

            var faceIndex = intersectedFaceIndex % 2 === 0 ? intersectedFaceIndex : intersectedFaceIndex - 1;

            var corners = [
                { faces: [0, 52, 76], direction: [-1, -1, -1] },
                { faces: [12, 58, 88], direction: [-1, 1, -1] },
                { faces: [4, 40, 90], direction: [-1, -1, 1] },
                { faces: [16, 70, 102], direction: [-1, 1, 1] },
                { faces: [18, 36, 94], direction: [1, -1, 1] },
                { faces: [22, 48, 72], direction: [1, -1, -1] },
                { faces: [30, 66, 106], direction: [1, 1, 1] },
                { faces: [34, 54, 84], direction: [1, 1, -1] }
            ];
            var edges = [
                { faces: [2, 46], direction: [-1, -1, 0] },
                { faces: [6, 82], direction: [-1, 0, -1] },
                { faces: [10, 96], direction: [-1, 0, 1] },
                { faces: [14, 64], direction: [-1, 1, 0] },
                { faces: [20, 42], direction: [1, -1, 0] },
                { faces: [24, 100], direction: [1, 0, 1] },
                { faces: [28, 78], direction: [1, 0, -1] },
                { faces: [32, 60], direction: [1, 1, 0] },
                { faces: [38, 92], direction: [0, -1, 1] },
                { faces: [50, 74], direction: [0, -1, -1] },
                { faces: [56, 86], direction: [0, 1, -1] },
                { faces: [68, 104], direction: [0, 1, 1] }
            ];
            var walls = [
                { faces: [8], direction: [-1, 0, 0] },
                { faces: [26], direction: [1, 0, 0] },
                { faces: [44], direction: [0, -1, 0] },
                { faces: [62], direction: [0, 1, 0] },
                { faces: [80], direction: [0, 0, -1] },
                { faces: [98], direction: [0, 0, 1] }
            ];

            // intersects center wall?
            for (var i = 0; i < walls.length; i++) {
                if (walls[i].faces.indexOf(faceIndex) !== -1) {
                    walls[i].type = "wall";
                    return walls[i];
                }
            }

            // intersects edge?
            for (i = 0; i < edges.length; i++) {
                if (edges[i].faces.indexOf(faceIndex) !== -1) {
                    edges[i].type = "edge";
                    return edges[i];
                }
            }

            // intersects corner?
            for (i = 0; i < corners.length; i++) {
                if (corners[i].faces.indexOf(faceIndex) !== -1) {
                    corners[i].type = "corner";
                    return corners[i];
                }
            }

            throw "VIEW3D.AxisHelper.getIntersectionObjectCube: No intersection object defined for faceIndex = " + intersectedFaceIndex;
        },

        getIntersectionObject2dArrow: function (intersectedFaceIndex) {

            var objects = [
                { faceIndexFrom: 0, faceIndexTo: 0, action: "left" },
                { faceIndexFrom: 1, faceIndexTo: 1, action: "right" },
                { faceIndexFrom: 2, faceIndexTo: 2, action: "up" },
                { faceIndexFrom: 3, faceIndexTo: 3, action: "down" },
                { faceIndexFrom: 4, faceIndexTo: 54, action: "rollLeft" },   //arc arrow
                { faceIndexFrom: 55, faceIndexTo: 105, action: "rollRight" }   //arc arrow
            ];

            for (var i = 0; i < objects.length; i++) {
                if (intersectedFaceIndex >= objects[i].faceIndexFrom && intersectedFaceIndex <= objects[i].faceIndexTo)
                    return objects[i];
            }

            throw "VIEW3D.AxisHelper.getIntersectionObject2dArrow: No intersection object defined for faceIndex = " + intersectedFaceIndex;
        },

        getCubeFaceIndexByMouse: function (event) {

            var container = this.getContainer();

            mouse = new THREE.Vector2();
            mouse.x = ((event.pageX - ($(container).offset().left)) / container[0].clientWidth) * 2 - 1;
            mouse.y = -((event.pageY - ($(container).offset().top)) / container[0].clientHeight) * 2 + 1;

            var raycaster = new THREE.Raycaster();
            raycaster.setFromCamera(mouse, this.camera);
            var dir = raycaster.ray.direction.clone();
            dir.multiplyScalar(camera.near);
            raycaster.ray.origin.add(dir);
            raycaster.linePrecision = 0.1;

            var intersectsFace = raycaster.intersectObjects([this.cube.cube], false);

            if (intersectsFace.length > 0)
                return intersectsFace[0].faceIndex;
            else
                return null;

        },

        get2dArrowByMouse: function (event) {

            var container = this.getContainer();

            mouse = new THREE.Vector2();
            mouse.x = ((event.pageX - ($(container).offset().left)) / container[0].clientWidth) * 2 - 1;
            mouse.y = -((event.pageY - ($(container).offset().top)) / container[0].clientHeight) * 2 + 1;

            var raycaster = new THREE.Raycaster();
            raycaster.setFromCamera(mouse, this.camera2d);
            var dir = raycaster.ray.direction.clone();
            dir.multiplyScalar(this.camera2d.near);
            raycaster.ray.origin.add(dir);
            var intersectsFace = raycaster.intersectObjects([this.arrows2d], false);

            if (intersectsFace.length > 0)
                return intersectsFace[0].faceIndex;
            else
                return null;
        },

        eventListeners: function () {

            var self = this;
            var rotateInterval = null;
            var rotateIntervalTime = 50;
            var rotateDistance = 5;

            return {
                onMouseMove: function (event) {
                    if (event.button === 1 || event.button === 2) { }

                    self.clearHighlights();                                                         // reset material to default

                    var intersectedFaceIndex = self.getCubeFaceIndexByMouse(event);
                    var intersectionObject;
                    if (intersectedFaceIndex !== null) {
                        //console.debug("VIEW3D.AxisHelper: 3d cube face:" + intersectedFaceIndex);

                        intersectionObject = self.getIntersectionObjectCube(intersectedFaceIndex);

                        self.highlightFaces(intersectionObject.faces);                              // highlight mouseovered faces

                    }
                    else {
                        intersectedFaceIndex = self.get2dArrowByMouse(event);

                        if (intersectedFaceIndex !== null) {
                            //console.debug("VIEW3D.AxisHelper: arrow face:" + intersectedFaceIndex);

                            intersectionObject = self.getIntersectionObject2dArrow(intersectedFaceIndex);

                            self.highlight2dArrow(intersectionObject.faceIndexFrom, intersectionObject.faceIndexTo);

                        }

                    }

                    renderAxisHelper();

                    return true;
                },

                onMouseUp: function (event) {
                    if (event.button === 1 || event.button === 2) {
                        event.preventDefault();
                        return false;
                    }

                    var intersectedFaceIndex = self.getCubeFaceIndexByMouse(event);

                    if (intersectedFaceIndex !== null) {

                        var intersectionObject = self.getIntersectionObjectCube(intersectedFaceIndex);

                        CORE.view3D.cameraControl.resetView(intersectionObject.direction[0], intersectionObject.direction[1], intersectionObject.direction[2], true);

                        if(!isEmpty(CORE.view3D.moveControl)) {
                            CORE.view3D.moveControl.updateTextBox(null);
                        }
                    }
                },

                onMouseDown: function (event) {
                    if (event.button === 0) {

                        var intersectedFaceIndex = self.get2dArrowByMouse(event);

                        if (intersectedFaceIndex !== null) {
                            //console.debug("VIEW3D.AxisHelper: arrow face:" + intersectedFaceIndex);

                            var intersectionObject = self.getIntersectionObject2dArrow(intersectedFaceIndex);

                            var executeRotate = function () {

                                switch (intersectionObject.action) {
                                    case "left": CORE.view3D.cameraControl.rotate(-rotateDistance, 0, 0, true); break;
                                    case "right": CORE.view3D.cameraControl.rotate(rotateDistance, 0, 0, true); break;
                                    case "up": CORE.view3D.cameraControl.rotate(0, -rotateDistance, 0, true); break;
                                    case "down": CORE.view3D.cameraControl.rotate(0, rotateDistance, 0, true); break;
                                    case "rollLeft": CORE.view3D.cameraControl.rotate(0, 0, -rotateDistance, true); break;
                                    case "rollRight": CORE.view3D.cameraControl.rotate(0, 0, rotateDistance, true); break;
                                    default: throw "VIEW3D.AxisHelper: intersectionObject.action unknown: " + intersectionObject.action;
                                }

                                render(MODE_TYPE.ROTATE);
                            };

                            executeRotate();

                            rotateInterval = setInterval(function () {

                                executeRotate();

                            }, rotateIntervalTime);

                            if (!isEmpty(CORE.view3D.moveControl)) {
                                CORE.view3D.moveControl.updateTextBox(null);
                            }
                        }
                    }
                },

                onMouseOut: function () {
                    self.clearHighlights();
                },

                onMouseUpDocument: function () {
                    clearInterval(rotateInterval);
                },

                preventDefault: function (event) {
                    event.preventDefault();
                }
            }
        },

        initEventListeners: function () {

            var self = this;

            if (typeof self.eventListeners === 'function')
                self.eventListeners = self.eventListeners();

            var container = this.getContainer();

            var rebindEvent = function (container, eventType, listener) {
                container.off(eventType, listener).on(eventType, listener);
            };

            rebindEvent(container, 'mousemove', self.eventListeners.onMouseMove);
            rebindEvent(container, 'mouseout', self.eventListeners.onMouseOut);
            rebindEvent(container, 'mouseup', self.eventListeners.onMouseUp);
            rebindEvent(container, 'mousedown', self.eventListeners.onMouseDown);
            rebindEvent(container, 'contextmenu', self.eventListeners.preventDefault);
            rebindEvent($(document), 'mouseup', self.eventListeners.onMouseUpDocument);

            //container.off('mousemove', self.eventListeners.onMouseMove).on('mousemove', onMouseMove);
            //container.off('mouseout', onMouseOut).on('mouseout', onMouseOut);
            //container.off('mouseup', onMouseUp).on('mouseup', onMouseUp);
            //container.off('mousedown', onMouseDown).on('mousedown', onMouseDown);
            //container.off('contextmenu', preventDefault).on('contextmenu', preventDefault);



            //document.removeEventListener('mouseup', onMouseUpDocument);
            //document.addEventListener('mouseup', onMouseUpDocument);

        }
    };
    VIEW3D.AxisHelper.Label.prototype = Object.create(THREE.Mesh.prototype);
    VIEW3D.AxisHelper.CubeLabel.prototype = Object.create(THREE.Mesh.prototype);
    VIEW3D.AxisHelper.Arrows.prototype = Object.create(THREE.Object3D.prototype);
    VIEW3D.AxisHelper.Cube.prototype = Object.create(THREE.Object3D.prototype);
});