﻿//special implementation of MeshTemplate (MultiMesh element)
//creates mesh that is made of repeated segments (segment is loaded from obj file)
var VIEW3D = VIEW3D || {};
VIEW3D.BarMeshTemplate = function(geometry) {
    this.getGeometry = function() {
        return geometry;
    };
    this.getBarLength = function(params) {
        var MemberConstraints = params.MemberConstraints;
        return MemberConstraints.length;
    };
    var getLoopsCount = function(params) {
        var MemberConstraints = params.MemberConstraints;
        var barLength = MemberConstraints.length;
        return Math.floor(barLength / template.length);
    };
    this.getIndicesCount = function(params) {
        var loops = getLoopsCount(params);
        return template.startInd.length + template.loopInd.length * loops;
    };
    this.getVerticesCount = function(params) {
        var loops = getLoopsCount(params);
        return (template.startPos.length + template.loopPos.length * loops) / 3;
    };
    this.getMatrix4FromBase = function(base) {
        var mat = new THREE.Matrix4();
        mat.set(
            base.baseX.x,
            base.baseY.x,
            base.baseZ.x,
            0,
            base.baseX.y,
            base.baseY.y,
            base.baseZ.y,
            0,
            base.baseX.z,
            base.baseY.z,
            base.baseZ.z,
            0,
            0,
            0,
            0,
            1);
        return mat;
    };
    this.transformPositions = function(mat, source, target, targetOffset, maxX) {
        // float array: 3 pre pos
        var me = mat.elements;
        for (var i = 0; i < source.length; i += 3) {
            var vx = Math.min(source[i + 0], maxX);
            var vy = source[i + 1];
            var vz = source[i + 2];
            var baseIndex = targetOffset + i;
            target[baseIndex + 0] = me[0] * vx + me[4] * vy + me[8] * vz + me[12];
            target[baseIndex + 1] = me[1] * vx + me[5] * vy + me[9] * vz + me[13];
            target[baseIndex + 2] = me[2] * vx + me[6] * vy + me[10] * vz + me[14];
        }
    };
    this.transformNormals = function(mat, source, target, targetOffset) {
        // float array: 3 pre pos
        var me = mat.elements;
        for (var i = 0; i < source.length; i += 3) {
            var vx = source[i + 0];
            var vy = source[i + 1];
            var vz = source[i + 2];
            var baseIndex = targetOffset + i;
            target[baseIndex + 0] = me[0] * vx + me[4] * vy + me[8] * vz;
            target[baseIndex + 1] = me[1] * vx + me[5] * vy + me[9] * vz;
            target[baseIndex + 2] = me[2] * vx + me[6] * vy + me[10] * vz;
        }
    };
    this.getTransformation = function(params) {
        var pt1 = params.MemberConstraints.orientation.origin;
        var base = params.MemberConstraints.orientation;
        var matOrientation = this.getMatrix4FromBase(base);
        var matStartPos = new THREE.Matrix4();
        matStartPos.makeTranslation(pt1.x, pt1.y, pt1.z);
        var matBase = new THREE.Matrix4();
        matBase.multiplyMatrices(matStartPos, matOrientation);
        return matBase;
    };
    this.writeAttributes = function(attrs, indices, sv, si, params) {
        var MemberConstraints = params.MemberConstraints;
        var pt1 = MemberConstraints.orientation.origin;
        var barLength = MemberConstraints.length;
        var loops = Math.floor(barLength / template.length);
        var verticesOffset = sv * 3;
        var indicesOffset = si;
        var base = MemberConstraints.orientation;
        var matOrientation = this.getMatrix4FromBase(base);
        var matStartPos = new THREE.Matrix4();
        matStartPos.makeTranslation(pt1.x, pt1.y, pt1.z);
        var matBase = new THREE.Matrix4();
        matBase.multiplyMatrices(matStartPos, matOrientation);
        // first segment
        var maxX = barLength;
        this.transformPositions(matBase, template.startPos, attrs.position.array, verticesOffset, maxX);
        this.transformNormals(matBase, template.startNor, attrs.normal.array, verticesOffset);
        var i;
        for (i = 0; i < template.startInd.length; i++) {
            indices.array[indicesOffset + i] = template.startInd[i] + verticesOffset / 3;
        }
        // rest of segments
        for (var j = 0; j < loops; j++) {
            maxX = barLength - (j + 1) * template.length;
            var matTranslation = new THREE.Matrix4();
            matTranslation.makeTranslation(template.length * (j + 1), 0, 0);
            var mat = new THREE.Matrix4();
            mat.multiplyMatrices(matBase, matTranslation);
            var targerOffset = verticesOffset + template.startPos.length + template.loopPos.length * j;
            this.transformPositions(mat, template.loopPos, attrs.position.array, targerOffset, maxX);
            this.transformNormals(mat, template.loopNor, attrs.normal.array, targerOffset);
            for (i = 0; i < template.loopInd.length; i++) {
                indices
                    .array[indicesOffset + i + template.startInd.length + template.loopInd.length * j] =
                    verticesOffset / 3 +
                    template.loopInd[i] +
                    template.startPos.length / 3 +
                    j * template.loopPos.length / 3;
            }
        }
    };
    this.comparePos = function(x1, x2, y1, y2, z1, z2) {
        var dx = x1 - x2;
        var dy = y1 - y2;
        var dz = z1 - z2;
        return dx * dx + dy * dy + dz * dz < 0.0000002;
    };
    this.compareNor = function(x1, x2, y1, y2, z1, z2) {
        return x1 * x2 + y1 * y2 + z1 * z2 > 0.8;
    };
    this.createTemplate = function(templateGeometry) {
        template = {};
        templateGeometry.computeBoundingBox();
        this.bbox = templateGeometry.boundingBox;
        var l = templateGeometry.boundingBox.max.x - templateGeometry.boundingBox.min.x;
        template.length = l;
        var indices = templateGeometry.index.array;
        var positions = templateGeometry.attributes.position.array;
        var normals = templateGeometry.attributes.normal.array;
        var i, j;

        // split
        var start = [];
        var middle = [];
        var end = [];
        for (i = 0; i < positions.length; i += 3) {
            found = false;/*
            for (j = 0; j < positions.length; j += 3) {
                if (this.comparePos(positions[i + 0] + l,
                        positions[j + 0],
                        positions[i + 1],
                        positions[j + 1],
                        positions[i + 2],
                        positions[j + 2]) &&
                    this.compareNor(normals[i + 0],
                        normals[j + 0],
                        normals[i + 1],
                        normals[j + 1],
                        normals[i + 2],
                        normals[j + 2])) {
                    found = true;
                    if (i < j) {
                        start.push(i);
                        end.push(j);
                    }
                    break;
                } else if (this.comparePos(positions[i + 0] - l,
                        positions[j + 0],
                        positions[i + 1],
                        positions[j + 1],
                        positions[i + 2],
                        positions[j + 2]) &&
                    this.compareNor(normals[i + 0],
                        normals[j + 0],
                        normals[i + 1],
                        normals[j + 1],
                        normals[i + 2],
                        normals[j + 2])) {
                    found = true;
                    if (i < j) {
                        start.push(j);
                        end.push(i);
                    }
                    break;
                }
            }*/
            if (!found) {
                middle.push(i);
            }
        }
        var templateSPos = new Float32Array(positions.length);
        var templateSNor = new Float32Array(positions.length);
        var templateSInd = new Uint32Array(indices.length);
        var index = 0;
        var oldIndex;
        for (i = 0; i < start.length; i++) {
            oldIndex = start[i];
            templateSPos[index * 3 + 0] = positions[oldIndex + 0];
            templateSPos[index * 3 + 1] = positions[oldIndex + 1];
            templateSPos[index * 3 + 2] = positions[oldIndex + 2];
            templateSNor[index * 3 + 0] = normals[oldIndex + 0];
            templateSNor[index * 3 + 1] = normals[oldIndex + 1];
            templateSNor[index * 3 + 2] = normals[oldIndex + 2];
            for (j = 0; j < indices.length; j++) {
                if (indices[j] === oldIndex / 3) {
                    templateSInd[j] = index;
                }
            }
            index++;
        }
        for (i = 0; i < middle.length; i++) {
            oldIndex = middle[i];
            templateSPos[index * 3 + 0] = positions[oldIndex + 0];
            templateSPos[index * 3 + 1] = positions[oldIndex + 1];
            templateSPos[index * 3 + 2] = positions[oldIndex + 2];
            templateSNor[index * 3 + 0] = normals[oldIndex + 0];
            templateSNor[index * 3 + 1] = normals[oldIndex + 1];
            templateSNor[index * 3 + 2] = normals[oldIndex + 2];
            for (j = 0; j < indices.length; j++) {
                if (indices[j] === oldIndex / 3) {
                    templateSInd[j] = index;
                }
            }
            index++;
        }
        for (i = 0; i < end.length; i++) {
            oldIndex = end[i];
            templateSPos[index * 3 + 0] = positions[oldIndex + 0];
            templateSPos[index * 3 + 1] = positions[oldIndex + 1];
            templateSPos[index * 3 + 2] = positions[oldIndex + 2];
            templateSNor[index * 3 + 0] = normals[oldIndex + 0];
            templateSNor[index * 3 + 1] = normals[oldIndex + 1];
            templateSNor[index * 3 + 2] = normals[oldIndex + 2];
            for (j = 0; j < indices.length; j++) {
                if (indices[j] === oldIndex / 3) {
                    templateSInd[j] = index;
                }
            }
            index++;
        }
        var templateLPos = new Float32Array((middle.length + end.length) * 3);
        var templateLNor = new Float32Array((middle.length + end.length) * 3);
        var templateLInd = new Uint32Array(indices.length);
        index = -start.length;
        for (i = 0; i < start.length; i++) {
            oldIndex = start[i];
            for (j = 0; j < indices.length; j++) {
                if (indices[j] === oldIndex / 3) {
                    templateLInd[j] = index;
                }
            }
            index++;
        }
        for (i = 0; i < middle.length; i++) {
            oldIndex = middle[i];
            templateLPos[index * 3 + 0] = positions[oldIndex + 0];
            templateLPos[index * 3 + 1] = positions[oldIndex + 1];
            templateLPos[index * 3 + 2] = positions[oldIndex + 2];
            templateLNor[index * 3 + 0] = normals[oldIndex + 0];
            templateLNor[index * 3 + 1] = normals[oldIndex + 1];
            templateLNor[index * 3 + 2] = normals[oldIndex + 2];
            for (j = 0; j < indices.length; j++) {
                if (indices[j] === oldIndex / 3) {
                    templateLInd[j] = index;
                }
            }
            index++;
        }
        for (i = 0; i < end.length; i++) {
            oldIndex = end[i];
            templateLPos[index * 3 + 0] = positions[oldIndex + 0];
            templateLPos[index * 3 + 1] = positions[oldIndex + 1];
            templateLPos[index * 3 + 2] = positions[oldIndex + 2];
            templateLNor[index * 3 + 0] = normals[oldIndex + 0];
            templateLNor[index * 3 + 1] = normals[oldIndex + 1];
            templateLNor[index * 3 + 2] = normals[oldIndex + 2];
            for (j = 0; j < indices.length; j++) {
                if (indices[j] === oldIndex / 3) {
                    templateLInd[j] = index;
                }
            }
            index++;
        }
        template.startPos = templateSPos;
        template.startNor = templateSNor;
        template.startInd = templateSInd;
        template.loopPos = templateLPos;
        template.loopNor = templateLNor;
        template.loopInd = templateLInd;
        return template;
    };
    var template = this.createTemplate(geometry);
};
VIEW3D.getBarTemplate = function(modelId, callback) {
    MODEL.repository.get3DModelAsync(modelId,
        function (model) {           
            var templateGeometry = MODEL.utils.Proto.getObject3dFromBase64(model.data, modelId).children[0].geometry;
            var verticesCount = templateGeometry.attributes.position.array.length / 3;
            templateGeometry.addAttribute("color", new THREE.BufferAttribute(new Float32Array(verticesCount * 3), 3));
            templateGeometry.computeBoundingBox();
            var template = new VIEW3D.BarMeshTemplate(templateGeometry);
            template.boundingBox = templateGeometry.boundingBox;
            //VIEW3D.utils.templates[modelId] = template;
            callback(template);
        });
};