﻿'use strict';
var VIEW3D = VIEW3D || {}

define(['js/view3d/materials/VIEW3D.materials.Metal', 'js/view3d/VIEW3D.MultiMesh', 'js/view3d/VIEW3D.RealCaps', 'js/view3d/VIEW3D.BarDxf.BarMeshTemplate'],
    function() {
        // class RealBars extends THREE.Object
        // binnds to an array of MODEL.Bar
        // and vizualizes it as a lines
        VIEW3D.BarDxf = function(bars, constraints) {
            var attributes = {
                position: { type: Float32Array, size: 3 },
                color: { type: Float32Array, size: 3 },
                normal: { type: Float32Array, size: 3 }
            };
            var material = new VIEW3D.materials.Metal();
            VIEW3D.MultiMesh.call(this, attributes, material);
            this.barsData = {};
            //init
            var collider = new VIEW3D.collision.CollidersContainer();
            this.getCollider = function() {
                return collider;
            };
            this.dispose = function() {
                this.barsObserver.dispose();
            }
            this.onBarSelectedChanged = function(bar) {
                if (bar.profileId() !== 0) {
                    var color = bar.selected() ? VIEW3D.selectionColor : VIEW3D.getProfileColor();
                    if (!isEmpty(this.barsData[bar.id])) {
                        var meshId = this.barsData[bar.id].meshId;
                        if (!isEmpty(meshId)) {
                            this.fillAttributeXYZ(meshId, "color", color.x, color.y, color.z);
                        }
                        render();
                    }
                }
            }
            this.removeBar = function(bar) {
                var barData = this.barsData[bar.id];
                if (!isEmpty(barData)) {
                    // nothing to remove
                    if (!isEmpty(barData.collider)) {
                        collider.removeCollider(barData.collider);
                    }
                    if (!isEmpty(barData.meshId)) {
                        this.removeMesh(barData.meshId);
                    }
                }
            }
            this.caps = new VIEW3D.RealCaps(constraints);
            this.addBar = function(bar) {
                var modelId = constraints.getBarProfileModelId(bar.id, bar.profileId());
                if (modelId === -1 || modelId === 0) {
                    console.warn("bar without model");
                    return;
                }
                this.barsData[bar.id] = {};
                var self = this;
                VIEW3D.getBarTemplate(modelId,
                    function(template) {
                        var barData = self.barsData[bar.id];
                        if (isEmpty(barData)) {
                            // looks like this bar is not in scene any more so leave
                            return;
                        }
                        var MemberConstraints = constraints.getMemberConstraints(bar.id);
                        if (isEmpty(MemberConstraints)) {
                            console.warn("Cant find Constraints for bar");
                            return;
                        }
                        var instanceParams = { bar: bar, MemberConstraints: MemberConstraints };
                        barData.meshId = self.addMeshFromTemplate(template, instanceParams);
                        var color = bar.selected() ? VIEW3D.selectionColor : VIEW3D.getProfileColor();
                        self.fillAttributeXYZ(barData.meshId, "color", color.x, color.y, color.z);
                        render();
                        // box collider
                        var bbox = template.boundingBox.clone();
                        bbox.max.x = template.getBarLength(instanceParams);
                        var boxCollider = new VIEW3D.collision
                            .BoxCollider(bbox, template.getTransformation(instanceParams));
                        boxCollider.view3D = this;
                        boxCollider.model = bar;
                        barData.collider = boxCollider;
                        collider.addCollider(boxCollider);
                        self.caps.addCaps(bar);
                    });
            }         
            //add to scene
            //this.add(this.caps);
            this.export = function() {
                var meshes = [];
                for (var index in this.barsData) {
                    var mesh, i;
                    if (!isEmpty(this.barsData[index].meshId)) {
                        mesh = this.getMeshData(this.barsData[index].meshId);
                        var vertices = [];
                        var geometry = [];
                        var max = (mesh.startVertex + mesh.vartexCount) * 3;
                        for (i = (mesh.startVertex * 3); i < max; i += 3) {
                            vertices.push(this.geometry.attributes.position.array[i]);
                            vertices.push(this.geometry.attributes.position.array[i + 1]);
                            vertices.push(this.geometry.attributes.position.array[i + 2]);
                        }
                        for (i = mesh.startIndex; i < mesh.indexCount + mesh.startIndex;) {
                            var tmp = [];
                            tmp.push(this.geometry.index.array[i++] - mesh.startVertex);
                            tmp.push(this.geometry.index.array[i++] - mesh.startVertex);
                            tmp.push(this.geometry.index.array[i++] - mesh.startVertex);
                            geometry.push(tmp);
                        }
                        var data = {
                            vertices: vertices,
                            faces: geometry,
                            type: 0 // Bar
                        };
                        meshes.push(data);
                    }
                }
                return meshes;
            };
            for (var i = 0; i < bars().length; i++) {
                var bar = bars()[i];
                this.addBar(bar);
            }
        };
        VIEW3D.BarDxf.prototype = Object.create(VIEW3D.MultiMesh.prototype);
    });