'use strict';
var VIEW3D = VIEW3D || {}

define(['js/view3d/VIEW3D.Edit.MoveControl', 'js/view3d/VIEW3D.Edit.RotationControl'], function () {
    rotObj = CORE.res.obj["rotation"];

    VIEW3D.BarEdit = function(bar, allBars, constraints) {

        var collider = new VIEW3D.collision.CollidersContainer();
        this.getCollider = function () {
            return collider;
        };

        THREE.Object3D.call(this);

        this.calculateExtermas = function(isPositive, isHorizontal) {
            var node1Id = bar.node1().id;
            var node2Id = bar.node2().id;
            for (var i = 0; i < allBars.length; i++) {
                var itBar = allBars[i];
                if(bar.id !== itBar.id) {
                    var otherNodePos = null;
                    if(itBar.node1().id === node1Id || itBar.node1().id === node2Id) {
                        otherNodePos = itBar.node2().position();
                    }
                    if(itBar.node2().id === node1Id || itBar.node2().id === node2Id) {
                        otherNodePos = itBar.node1().position();
                    }
                    if(otherNodePos !== null) {
                        var diff = 0;
                        if(isHorizontal) {//z difference
                            diff = otherNodePos.z - bar.node1().position().z;
                        }
                        else {//vertical so x difference
                            diff = otherNodePos.x - bar.node1().position().x;
                        }
                        diff = 0.8*diff;
                        if(diff > 0) {
                            if(diff < 0.1) {
                                diff = 0;
                            }
                            if(isPositive){
                                return { max : 100 , min : -diff };
                            }
                            else {
                                return { max : diff , min : -100 };
                            }
                        }
                        else {
                            if(diff > -0.1) {
                                diff = 0;
                            }
                            if(isPositive){
                                return { max : -diff , min : -100 };
                            }
                            else {
                                return { max : 100 , min : diff };
                            }
                        }
                    }
                }
            }
            return {max : 100, min : -100};
        }

        this.moveNodes = function(isHorizontal, baseValue, nodeOffset) {
            var nodes = CORE.model.nodes();
            for (var i = 0; i < nodes.length; i++) {
                var node = nodes[i];
                var nodePos = node.position();
                if(isHorizontal && Math.abs(nodePos.z - baseValue) < 0.00001) {
                    var newZ = nodePos.z + nodeOffset.z;
                    if(VIEW3D.SnapGridDefinition.snapToGrid()) {
                        newZ = VIEW3D.SnapZToGrid(newZ);
                    }
                    node.position(new MODEL.types.Vector(nodePos.x + nodeOffset.x, nodePos.y + nodeOffset.y, newZ));
                }
                if(!isHorizontal && Math.abs(nodePos.x - baseValue) < 0.00001) {
                    var newX = nodePos.x + nodeOffset.x;
                    if(VIEW3D.SnapGridDefinition.snapToGrid()) {
                        newX = VIEW3D.SnapXToGrid(newX);
                    }
                    node.position(new MODEL.types.Vector(newX, nodePos.y + nodeOffset.y, nodePos.z + nodeOffset.z));
                }
            }
        }

        this.createMove = function (isPositive) {
            var pos1 = HELPERS.toVector3(bar.node1().position());
            var pos2 = HELPERS.toVector3(bar.node2().position());
            var center = new THREE.Vector3().addVectors(pos1, pos2).divideScalar(2);
            var isHorizontal = Math.abs(pos1.z - pos2.z) < 0.00001;
            var offset = isPositive ? 1 : -1;
            var offsetVec = isHorizontal ? new THREE.Vector3(0,0,offset) : new THREE.Vector3(offset,0,0);
            var secPoint = new THREE.Vector3().addVectors(center, offsetVec);

            var sub = new THREE.Vector3().subVectors(secPoint, center);
            var length = sub.length();
            var moveSpace = VIEW3D.utils.getTransformationFromSegment(secPoint, center);
            var self = this;
            var rotCtrl = new VIEW3D.Edit.MoveControl(length, moveSpace, center, secPoint, allBars, new THREE.Vector4(219/255, 55/255, 58/255, 1.0), function (newLength) {
                if(Math.abs(newLength - length) > 0.001) {
                    var lengthDiff = newLength - length;
                    var nodeOffset = offsetVec.clone().multiplyScalar(-1*lengthDiff);
                    var oldNode1Pos = bar.node1().position();
                    self.moveNodes(isHorizontal, isHorizontal ? oldNode1Pos.z : oldNode1Pos.x, nodeOffset);
                }
            }, this.clearAllImputs.bind(this), pos1, pos2, offsetVec, this.calculateExtermas(isPositive, isHorizontal));
            this.add(rotCtrl.mesh);
            collider.addCollider(rotCtrl.collider);
            return rotCtrl;
        }

        this.controls = [];
        this.disposeControls = function() {
            for (var i = 0; i < this.controls.length; i++) {
                var control = this.controls[i];
                control.dispose();
                collider.removeCollider(control.collider);
                this.remove(control.mesh)
            }
            this.controls = [];
        }

        this.update = function() {
            this.disposeControls();


            this.controls.push( this.createMove(false));
            this.controls.push( this.createMove(true));
        }

        this.clearAllImputs = function () {
            for (var i = 0; i < this.controls.length; i++) {
                var control = this.controls[i];
                control.setGrayed(true);
                control.inputs.clearInputs();
            }
        }


        this.update();
        this.waitForConstraintsAndUpdate = function() {
            var sub = HELPERS.subscribePropertyChange( this, constraints.data, "constraints.data", function () {
                sub.dispose();
                this.update();
            });
        }

        var pos1Sub = HELPERS.subscribePropertyChange(this, bar.node1().position, "node 1 pos", this.waitForConstraintsAndUpdate );
        var pos2Sub = HELPERS.subscribePropertyChange(this, bar.node2().position, "node 2 pos", this.waitForConstraintsAndUpdate );
        var node1Sub = HELPERS.subscribePropertyChange(this, bar.node1, "node 1", function(){
            pos1Sub.dispose();
            pos1Sub = HELPERS.subscribePropertyChange(this, bar.node1().position, "node 1 pos", this.waitForConstraintsAndUpdate );
            this.update();
        } );
        var node2Sub = HELPERS.subscribePropertyChange(this, bar.node2, "node 2", function(){
            pos2Sub.dispose();
            pos2Sub = HELPERS.subscribePropertyChange(this, bar.node2().position, "node 1 pos", this.waitForConstraintsAndUpdate );
            this.update();
        } );

        this.dispose = function() {
            pos1Sub.dispose();
            pos2Sub.dispose();
            node1Sub.dispose();
            node2Sub.dispose();
            this.disposeControls();
        }
    }
    VIEW3D.BarEdit.prototype = Object.create(THREE.Object3D.prototype);
});
