﻿'use strict';
var VIEW3D = VIEW3D || {}

// class VIEW3D.BarForce
// binds to MODEL.load.node.BarForce and creates geometry
// that represents that BarForce model. Once BarForce model
// is assigned to Bar this class creates new Object3D and
// adds it to the parent
VIEW3D.BarForce = function(barForceModel, arrows, inputs) {
    var instances = []; // synchronnized with barForceModel.assignedTo
    this.arrows = arrows;
    this.inputs = inputs;
    var collider = new VIEW3D.collision.DisjunctionCollider();
    collider.model = barForceModel;
    this.getCollider = function () {
        return collider;
    }

    var getLoadPosition = function (barAndPosition) {
        var bar = barAndPosition.bar;
        var node1Position = HELPERS.toVector3(bar.node1().position());
        var node2Position = HELPERS.toVector3(bar.node2().position());
        var distvec = new THREE.Vector3().subVectors(node2Position, node1Position);
        /*if (barForceModel.coordinatesAbsolute()) {*/
        var length = distvec.length();
        if (length < barAndPosition.x()) {
            distvec.normalize();
            distvec.multiplyScalar(length);
        } else {
            distvec.normalize();
            distvec.multiplyScalar(barAndPosition.x());
        }
        /*} else {
         distvec.multiplyScalar(barAndPosition.x());
         /* }*/
        distvec.add(node1Position);
        return distvec;
    }
    var getArrowsData = function (barAndPosition) {
        var pos = getLoadPosition(barAndPosition);
        var rotation = barForceModel.rotation();
        var q = new THREE.Quaternion().setFromEuler(new THREE.Euler(rotation.x, rotation.y, rotation.z, 'XYZ'));
        var v = HELPERS.toVector3(barForceModel.forces());
        v.applyQuaternion(q);
        var arrowsData = [];
        if (v.x !== 0) {
            arrowsData.push({
                dir: new THREE.Vector3(v.x, 0, 0),
                pos: new THREE.Vector3(pos.x, pos.y, pos.z),
                color: new THREE.Vector3(1, 0, 0),
                type: VIEW3D.Arrows.Arrow
            });
        }
        if (v.y !== 0) {
            arrowsData.push({
                dir: new THREE.Vector3(0, v.y, 0),
                pos: new THREE.Vector3(pos.x, pos.y, pos.z),
                color: new THREE.Vector3(0, 1, 0),
                type: VIEW3D.Arrows.Arrow
            });
        }
        if (v.z !== 0) {
            arrowsData.push({
                dir: new THREE.Vector3(0, 0, v.z),
                pos: new THREE.Vector3(pos.x, pos.y, pos.z),
                color: new THREE.Vector3(0, 0, 1),
                type: VIEW3D.Arrows.Arrow
            });
        }
        var m = barForceModel.moments();
        if (m.x !== 0) {
            arrowsData.push({
                dir: new THREE.Vector3(m.x, 0, 0),
                pos: new THREE.Vector3(pos.x, pos.y, pos.z),
                color: new THREE.Vector3(1, 0, 0),
                type: VIEW3D.Arrows.Moment
            });
        }
        if (m.y !== 0) {
            arrowsData.push({
                dir: new THREE.Vector3(0, m.y, 0),
                pos: new THREE.Vector3(pos.x, pos.y, pos.z),
                color: new THREE.Vector3(0, 1, 0),
                type: VIEW3D.Arrows.Moment
            });
        }
        if (m.z !== 0) {
            arrowsData.push({
                dir: new THREE.Vector3(0, 0, m.z),
                pos: new THREE.Vector3(pos.x, pos.y, pos.z),
                color: new THREE.Vector3(0, 0, 1),
                type: VIEW3D.Arrows.Moment
            });
        }
        return arrowsData;
    }

    var getInputsData = function(barAndPosition) {
        var pos = getLoadPosition(barAndPosition);
        var rotation = barForceModel.rotation();
        var q = new THREE.Quaternion().setFromEuler(new THREE.Euler(rotation.x, rotation.y, rotation.z, 'XYZ'));
        var v = HELPERS.toVector3(barForceModel.forces());
        v.applyQuaternion(q);

        var inputsData = [];
        if (v.x !== 0) {
            inputsData.push({
                dir: new THREE.Vector3(v.x, 0, 0),
                pos: new THREE.Vector3(pos.x, pos.y, pos.z),
                val: ko.observable(v.x),
                offset : 10
            });
        }
        if (v.y !== 0) {
            inputsData.push({
                dir: new THREE.Vector3(0, v.y, 0),
                pos: new THREE.Vector3(pos.x, pos.y, pos.z),
                val: ko.observable(v.y),
                offset : 10
            });
        }
        if (v.z !== 0) {
            inputsData.push({
                dir: new THREE.Vector3(0, 0, v.z),
                pos: new THREE.Vector3(pos.x, pos.y, pos.z),
                val: ko.observable(v.z),
                offset : 10
            });
        }

        return inputsData;
    }

    var removeInstance = function(index) {
        if(instances[index].arrowsId !== 0) {
            collider.removeCollider(arrows.getCollider(instances[index].arrowsId));
        }
        arrows.removeArrows(instances[index].arrowsId);
        inputs.removeInput(instances[index].arrowsId);
        instances[index].positionChangeSubscription1.dispose();
        instances[index].positionChangeSubscription2.dispose();
        instances[index].positionChangeSubscription3.dispose();
        instances[index].positionChangeSubscription4.dispose();
        instances[index].positionChangeSubscription5.dispose();
    }
    var onNodePositionChange = function(index) {
        removeInstance(index);
        var node = barForceModel.barAttachments()[index];
        onBarAssigned(node, index);
    }
    var onBarAssigned = function (barAndPosition, index) {
        if (!isEmpty(barAndPosition) && !isEmpty(barAndPosition.bar)) {
            var instance = {};
            var arrowsData = getArrowsData(barAndPosition);
            instance.arrowsId = arrows.addArrows(arrowsData, barForceModel);
            var inputData = getInputsData(barAndPosition);
            inputs.addInputs(inputData, instance.arrowsId);

            instance.positionChangeSubscription1 = HELPERS.subscribePropertyChange(VIEW3D.BarForce, barAndPosition.bar.node1, "bar.node1", onNodePositionChange, index);
            instance.positionChangeSubscription2 = HELPERS.subscribePropertyChange(VIEW3D.BarForce, barAndPosition.bar.node2, "bar.node2", onNodePositionChange, index);
            instance.positionChangeSubscription3 = HELPERS.subscribePropertyChange(VIEW3D.BarForce, barAndPosition.bar.node1().position, "bar.node1.position", onNodePositionChange, index);
            instance.positionChangeSubscription4 = HELPERS.subscribePropertyChange(VIEW3D.BarForce, barAndPosition.bar.node2().position, "bar.node2.position", onNodePositionChange, index);
            instance.positionChangeSubscription5 = HELPERS.subscribePropertyChange(VIEW3D.BarForce, barAndPosition.x, "bar.node1", onNodePositionChange, index);
            instances[index] = instance;
            if (instance.arrowsId !== 0) {
                collider.addCollider(arrows.getCollider(instance.arrowsId));
            }
        }
    }
    var onBarUnassigned = function (node, index) {
        removeInstance(index);
        instances.splice(index, 1);
    }
    var onPropertyChanged = function() {
        for (var i = 0; i < barForceModel.barAttachments().length; i++) {
            var instance = instances[i];
            if(instance.arrowsId !== 0) {
                collider.removeCollider(arrows.getCollider(instance.arrowsId));
            }
            inputs.removeInput(instance.arrowsId);
            arrows.removeArrows(instance.arrowsId);
            var barAndPosition = barForceModel.barAttachments()[i];
            var arrowsData = getArrowsData(barAndPosition);
            var inputData = getInputsData(barAndPosition);
            instance.arrowsId = arrows.addArrows(arrowsData, barForceModel);
            inputs.addInputs(inputData, instance.arrowsId);
            if(instance.arrowsId !== 0) {
                collider.addCollider(arrows.getCollider(instance.arrowsId));
            }
        }
    }

    var onLoadSelected = function (val) {

        var newColor;
        for (var i = 0; i < instances.length; i++) {
            var arrowData = this.arrows.getArrowsData(instances[i].arrowsId);
            if (isEmpty(arrowData)) continue;
            arrowData = arrowData.data;

            for (var j = 0; j < arrowData.length; j++) {

                if (val)  newColor = arrowData[j].selectedColor;
                else newColor = arrowData[j].originalColor;

                arrowData[j].color = newColor;
            }
        }
        this.arrows.updateAllArrows();
        render();
    }

    var assignedSub = HELPERS.subscribeArrayChange(this, barForceModel.barAttachments, "MODEL.loads.barForce.assignedTo", onBarAssigned, onBarUnassigned);
    var forcePropSub = HELPERS.subscribePropertyChange(this, barForceModel.forces, "barForceModel.forces", onPropertyChanged);
    var momentPropSub = HELPERS.subscribePropertyChange(this, barForceModel.moments, "barForceModel.moments", onPropertyChanged);
    var selectSub = HELPERS.subscribePropertyChange(this, barForceModel.selected, "MODEL.loads.bar.assignedTo", onLoadSelected);
    //var _rotationPropSub = HELPERS.subscribePropertyChange(this, barForceModel.rotation, "barForceModel.rotation", onPropertyChanged);
    //var xASub = HELPERS.subscribePropertyChange(this, barForceModel.xAbsolute, "barForceModel.xAbsolute", onPropertyChanged);
    //var xRSub = HELPERS.subscribePropertyChange(this, barForceModel.xRelative, "barForceModel.xRelative", onPropertyChanged);
    //var coordinateSub = HELPERS.subscribePropertyChange(this, barForceModel.coordinatesAbsolute, "barForceModel.coordinatesAbsolute", onPropertyChanged);
    this.clean = function() {
        for (var i = 0; i < instances.length; i++) {
            removeInstance(i);
        }
        instances = [];
        assignedSub.dispose();
        forcePropSub.dispose();
        momentPropSub.dispose();
        selectSub.dispose();
        //_rotationPropSub.dispose();
        //xASub.dispose();
        //xRSub.dispose();
        //coordinateSub.dispose();
    }
    this.getAllArrowsIds = function() {
        var ids = [];
        for (var i = 0; i < instances.length; i++) {
            ids.push(instances[i].arrowsId);
        }
        return ids;
    }
    for (var index = 0; index < barForceModel.barAttachments().length; index++) {
        onBarAssigned(barForceModel.barAttachments()[index], index);
    }
    HELPERS.subscribePropertyChange(this, barForceModel.coordinatesAbsolute, "VIEW3D.BarForce.coordinatesAbsolute", function() {
        VIEWUI.spinners.refresh("#VIEWUI_coordinateX", 1);
    });

    CORE.loadModelScale.subscribe(newVal => {
        onPropertyChanged();
    });
};