﻿'use strict';
var VIEW3D = VIEW3D || {}

define([], function () {
// class VIEW3D.BarTrapezoidal
// binds to MODEL.load.node.BarTrapezoidal and creates geometry
// that represents that BarTrapezoidal model. Once BarTrapezoidal model
// is assigned to Bar this class creates new Object3D and
// adds it to the parent
VIEW3D.BarTrapezoidal = function (barTrapezoidalModel, arrows, fills, inputs) { //
    this.arrows = arrows;
    this.inputs = inputs;
    var inputCount = 100000;
    var instances = []; // synchronnized with barTrapezoidalModel.assignedTo

    var collider = new VIEW3D.collision.DisjunctionCollider();
    collider.model = barTrapezoidalModel;
    this.getCollider = function () {
        return collider;
    }

    var getArrowsData = function (node1Position, node2Position) {
        var arrowsData = [];
        var p1 = barTrapezoidalModel.points()[0]();
        var p2 = barTrapezoidalModel.points()[1]();
        var p1Vector = p1.vector; // forces vectors
        var p2Vector = p2.vector;
        var p1DistanceProp = p1.distance; // proportional distance from bar start node to p1 [0-1]
        var p2DistanceProp = p2.distance; // proportional distance from bar start node to p2 [0-1]
        if (p1DistanceProp !== p2DistanceProp) { // overlapping points are ignored
            if (p1Vector.x !== 0 || p2Vector.x !== 0) {
                addArrowRow(new THREE.Vector3(p1Vector.x, 0, 0), new THREE.Vector3(p2Vector.x, 0, 0));
            }
            if (p1Vector.y !== 0 || p2Vector.y !== 0) {
                addArrowRow(new THREE.Vector3(0, p1Vector.y, 0), new THREE.Vector3(0, p2Vector.y, 0));
            }
            if (p1Vector.z !== 0 || p2Vector.z !== 0) {
                addArrowRow(new THREE.Vector3(0, 0, p1Vector.z), new THREE.Vector3(0, 0, p2Vector.z));
            }
        }

        function addArrowRow(vec1, vec2) {
            var addArrow = function (pos, v) {
                if (v.x !== 0) {
                    arrowsData.push({ dir: new THREE.Vector3(v.x, 0, 0), pos: new THREE.Vector3(pos.x, pos.y, pos.z), color: new THREE.Vector3(1, 0, 0), type: VIEW3D.Arrows.Arrow });
                }
                if (v.y !== 0) {
                    arrowsData.push({ dir: new THREE.Vector3(0, v.y, 0), pos: new THREE.Vector3(pos.x, pos.y, pos.z), color: new THREE.Vector3(0, 1, 0), type: VIEW3D.Arrows.Arrow });
                }
                if (v.z !== 0) {
                    arrowsData.push({ dir: new THREE.Vector3(0, 0, v.z), pos: new THREE.Vector3(pos.x, pos.y, pos.z), color: new THREE.Vector3(0, 0, 1), type: VIEW3D.Arrows.Arrow });
                }
            }
            var addLine = function (pos1, pos2, v1, v2) {
                if (v1.x !== 0 || v2.x !== 0) {
                    arrowsData.push({ dir1: new THREE.Vector3(v1.x, 0, 0), dir2: new THREE.Vector3(v2.x, 0, 0), pos1: new THREE.Vector3(pos1.x, pos1.y, pos1.z), pos2: new THREE.Vector3(pos2.x, pos2.y, pos2.z), color: new THREE.Vector3(1, 0, 0), type: VIEW3D.Arrows.Line });
                }
                if (v1.y !== 0 || v2.y !== 0) {
                    arrowsData.push({ dir1: new THREE.Vector3(0, v1.y, 0), dir2: new THREE.Vector3(0, v2.y, 0), pos1: new THREE.Vector3(pos1.x, pos1.y, pos1.z), pos2: new THREE.Vector3(pos2.x, pos2.y, pos2.z), color: new THREE.Vector3(0, 1, 0), type: VIEW3D.Arrows.Line });
                }
                if (v1.z !== 0 || v2.z !== 0) {
                    arrowsData.push({ dir1: new THREE.Vector3(0, 0, v1.z), dir2: new THREE.Vector3(0, 0, v2.z), pos1: new THREE.Vector3(pos1.x, pos1.y, pos1.z), pos2: new THREE.Vector3(pos2.x, pos2.y, pos2.z), color: new THREE.Vector3(0, 0, 1), type: VIEW3D.Arrows.Line });
                }
            }
            
            var sub = new THREE.Vector3().subVectors(node2Position, node1Position);
            var barLength = node1Position.distanceTo(node2Position);

            var p1X = p1DistanceProp;
            var p2X = p2DistanceProp;

            var p1Vec = node1Position.clone().add(sub.clone().setLength(p1X)); // vertices on bar, p1x and p2x away from node1
            var p2Vec = node1Position.clone().add(sub.clone().setLength(p2X));
            var subp = new THREE.Vector3().subVectors(p2Vec, p1Vec);

            //draw arrows at the start and end nodes of the bar
            addArrow(p1Vec, vec1);
            addArrow(p2Vec, vec2);
            addLine(p1Vec, p2Vec, vec1, vec2);

            var partialBarLength = Math.abs(p2X - p1X); // distance between two points
            var distance = 1; // distance between arrows, will get rounded so that the bar's length will be it's multiple
            var quotient = partialBarLength / distance;
            var fits = Math.floor(quotient); // fits that many arrows
            var doesntFit = quotient % 1; // the remaining part of bar (too small to fit an arrow)
            var enlargeDistanceBy = doesntFit / fits; // enlarge distance by
            distance += enlargeDistanceBy;

            function getMinDescaled() {
                var minScaled = CORE.constructionView3D ?  CORE.constructionView3D.loadsObject3D.arrows.minArrow : 0.08;
                var maxArrow = CORE.model.getMaxAbsLoadValue();

                var minDescaled = (minScaled * maxArrow) / VIEW3D.Arrows.maxSize;

                return minDescaled;
            }

            function validateMin(vec) {
                
                var minDescaled = getMinDescaled();

                if (Math.abs(vec.z) < minDescaled && vec.x !== 0) vec.x = vec.x > 0 ? minDescaled : -minDescaled;
                if (Math.abs(vec.z) < minDescaled && vec.y !== 0) vec.y = vec.y > 0 ? minDescaled : -minDescaled;
                if (Math.abs(vec.z) < minDescaled && vec.z !== 0) vec.z = vec.z > 0 ? minDescaled : -minDescaled;
                return vec;
            }

            //draw the rest of the arrows
            for (var i = 0 + distance, ii = partialBarLength; i < ii; i += distance) {
                var startVertex;
                var endVertex;
                var subVec1Vec2 = new THREE.Vector3().subVectors(validateMin(vec1), validateMin(vec2));
                var subpNew = subp.clone();
                var subVec1Vec2Part = subVec1Vec2.clone().divideScalar(fits); // needed to calculate arrowEndVectorTransform
                var vec;

                if (subVec1Vec2.x > 0 || subVec1Vec2.y > 0 || subVec1Vec2.z > 0) { // choose the smaller vector as start point (this probably could have been done without the if...)
                    startVertex = p2Vec; // if the difference is positive, take p2vec
                    endVertex = p1Vec;
                    vec = vec2;
                } else {
                    startVertex = p1Vec; // otherwise, take p1vec
                    endVertex = p2Vec;
                    vec = vec1;
                    subpNew.negate();
                    subVec1Vec2Part.negate();
                }

                //draw arrows from smallest to largest
                var arrowStartVertex = startVertex.clone().sub(subpNew.clone().setLength(i));
                var arrowEndVectorTransform = vec.clone().add(subVec1Vec2Part.clone().multiplyScalar(i / distance)); // arrowStartVertex + arrowEndVectorTransform = arrowEndVertex

                if (arrowEndVectorTransform.length() < getMinDescaled()) continue; // don't draw an arrow if smaller than the line (for loads with force1 === 0 && force2 !== 0)

                addArrow(arrowStartVertex, arrowEndVectorTransform);
            }
        }

        return arrowsData;
    }

    var createFillObjects = function (barTrapezoidalModel, barAttachment) {

        var object = new THREE.Object3D();
        var fills = [];

        var node1pos = barAttachment.bar.node1().position();
        var node2pos = barAttachment.bar.node2().position();
        var force1 = barTrapezoidalModel.points()[0]().vector;      // force values as vectors
        var force2 = barTrapezoidalModel.points()[1]().vector;
        var distance1 = barTrapezoidalModel.points()[0]().distance;
        var distance2 = barTrapezoidalModel.points()[1]().distance;
        var sub = HELPERS.toVector3(node2pos.sub(node1pos));
        var v1 = node1pos.add(sub.clone().setLength(distance1));    // positions on bar in world space, distance1 and distance2 away from node1
        var v2 = node1pos.add(sub.clone().setLength(distance2));

        var isZero = function(x) {
            return Math.abs(x) < 0.00001;
        }

        if (!isZero(force1.x) || !isZero(force2.x)) {
            var fill = {
                force1pos: v1,
                force2pos: v2,
                force1value: new MODEL.types.Vector(force1.x, 0, 0),
                force2value: new MODEL.types.Vector(force2.x, 0, 0),
                color: { x: 1, y: 0, z: 0 },
                type: VIEW3D.LoadFills.Fill
            }

            fills.push(fill);
        }

        if (!isZero(force1.y) || !isZero(force2.y)) {
            var fill = {
                force1pos: v1,
                force2pos: v2,
                force1value: new MODEL.types.Vector(0, force1.y, 0),
                force2value: new MODEL.types.Vector(0, force2.y, 0),
                color: { x: 0, y: 1, z: 0 },
                type: VIEW3D.LoadFills.Fill
            }

            fills.push(fill);
        }

        if (!isZero(force1.z) || !isZero(force2.z)) {
            var fill = {
                force1pos: v1,
                force2pos: v2,
                force1value: new MODEL.types.Vector(0, 0, force1.z),
                force2value: new MODEL.types.Vector(0, 0, force2.z),
                color: { x: 0, y: 0, z: 1 },
                type: VIEW3D.LoadFills.Fill
            }

            fills.push(fill);
        }

        return fills;
    }

    var getInputsData = function(n1p, n2p) {
        
        var p1 = barTrapezoidalModel.points()[0]();
        var p2 = barTrapezoidalModel.points()[1]();
        var v = p1.vector;
        var sub = new THREE.Vector3().subVectors(n2p, n1p);
        var node1 = n1p.clone().add(sub.clone().setLength(p1.distance));
        var node2 = n1p.clone().add(sub.clone().setLength(p2.distance));
        var pos = new THREE.Vector3((node1.x + node2.x)/2, (node1.y + node2.y)/2, (node1.z + node2.z)/2);
        var inputsData = [];
        if (v.x !== 0) {
            inputsData.push({
                dir: new THREE.Vector3(v.x, 0, 0),
                pos: pos,
                val: ko.observable(v.x),
                offset : 0
            });
        }
        if (v.y !== 0) {
            inputsData.push({
                dir: new THREE.Vector3(0, v.y, 0),
                pos: pos,
                val: ko.observable(v.y),
                offset : 0
            });
        }
        if (v.z !== 0) {
            inputsData.push({
                dir: new THREE.Vector3(0, 0, v.z),
                pos: pos,
                val: ko.observable(v.z),
                offset : 0
            });
        }

        return inputsData;
    }

    var onPropertyChanged = function () {
        for (var i = 0; i < barTrapezoidalModel.barAttachments().length; i++) {
            var bar = barTrapezoidalModel.barAttachments()[i].bar;
            var instance = instances[i];
            if (instance.arrowsId !== 0) {
                collider.removeCollider(arrows.getCollider(instance.arrowsId));
            }
            inputs.removeInput(instance.fillsId + inputCount)
            arrows.removeArrows(instance.arrowsId);
            fills.removeFills(instance.fillsId);
            var arrowsData = getArrowsData(HELPERS.toVector3(bar.node1().position()), HELPERS.toVector3(bar.node2().position()));
            var inputData = getInputsData(HELPERS.toVector3(bar.node1().position()), HELPERS.toVector3(bar.node2().position()));
            instance.arrowsId = arrows.addArrows(arrowsData, barTrapezoidalModel);
            var fillsData = createFillObjects(barTrapezoidalModel, barTrapezoidalModel.barAttachments()[i]);
            instance.fillsId = fills.addFills(fillsData);
            inputs.addInputs(inputData, instance.fillsId + inputCount, "DistLoad");
            if (instance.arrowsId !== 0) {
                collider.addCollider(arrows.getCollider(instance.arrowsId));
            }
        }
    }

    var removeInstance = function (index) {
        if (instances[index].arrowsId !== 0) {
            collider.removeCollider(arrows.getCollider(instances[index].arrowsId));
        }
        if (instances[index]) {     // sometimes instance is already removed (when MODEL.LoadsController is first to remove an instance)
            arrows.removeArrows(instances[index].arrowsId);
            inputs.removeInput(instances[index].fillsId + inputCount);
            fills.removeFills(instances[index].fillsId);
            instances[index].positionChangeSubscription1.dispose();
            instances[index].positionChangeSubscription2.dispose();
            instances[index].positionChangeSubscription3.dispose();
            instances[index].positionChangeSubscription4.dispose();
        }
    }
    var onNodePositionChange = function (index) {
        removeInstance(index);
        var node = barTrapezoidalModel.barAttachments()[index];
        onBarAssigned(node, index);
    }

    var onBarAssigned = function (barAttachment, index) {

        if (!isEmpty(barAttachment) && !isEmpty(barAttachment.bar))  // if bar exists, draw the load
        {
            var instance = {};
            var arrowsData = getArrowsData(HELPERS.toVector3(barAttachment.bar.node1().position()), HELPERS.toVector3(barAttachment.bar.node2().position()));
            instance.arrowsId = arrows.addArrows(arrowsData, barTrapezoidalModel);
            var fillsData = createFillObjects(barTrapezoidalModel, barAttachment);
            instance.fillsId = fills.addFills(fillsData);
            var inputData = getInputsData(HELPERS.toVector3(barAttachment.bar.node1().position()), HELPERS.toVector3(barAttachment.bar.node2().position()));
            inputs.addInputs(inputData, instance.fillsId + inputCount, "DistLoad")

            instance.positionChangeSubscription1 = HELPERS.subscribePropertyChange(VIEW3D.BarTrapezoidal, barAttachment.bar.node1, "bar.node1", onNodePositionChange, index);
            instance.positionChangeSubscription2 = HELPERS.subscribePropertyChange(VIEW3D.BarTrapezoidal, barAttachment.bar.node2, "bar.node2", onNodePositionChange, index);
            instance.positionChangeSubscription3 = HELPERS.subscribePropertyChange(VIEW3D.BarTrapezoidal, barAttachment.bar.node1().position, "bar.node1.position", onNodePositionChange, index);
            instance.positionChangeSubscription4 = HELPERS.subscribePropertyChange(VIEW3D.BarTrapezoidal, barAttachment.bar.node2().position, "bar.node2.position", onNodePositionChange, index);
            instances[index] = instance;
            if (instance.arrowsId !== 0) {
                collider.addCollider(arrows.getCollider(instance.arrowsId));
            }
            //parent.add(instance);
            render();
        } else {
            console.error("VIEW3D.BarTrapezoidal: Bar trapezoidal with id = " + barTrapezoidalModel.id + " does not have a bar assigned in barAndPosition[" + index + "]");
        }
    }


    var onBarUnassigned = function (node, index) {
        removeInstance(index);
        instances.splice(index, 1);
    }

    var onLoadSelected = function (val) {

        var newColor;
        for (var i = 0; i < instances.length; i++) {
            var arrowData = this.arrows.getArrowsData(instances[i].arrowsId);
            if (isEmpty(arrowData)) continue;
            arrowData = arrowData.data;

            for (var j = 0; j < arrowData.length; j++) {

                if (val)  newColor = arrowData[j].selectedColor;
                else newColor = arrowData[j].originalColor;

                arrowData[j].color = newColor;
            }
        }
        this.arrows.updateAllArrows();
        render();
    }

    var assignedSub = HELPERS.subscribeArrayChange(this, barTrapezoidalModel.barAttachments, "MODEL.loads.bar.assignedTo", onBarAssigned, onBarUnassigned);
    var p1PropSub = HELPERS.subscribePropertyChange(this, barTrapezoidalModel.points()[0], "barTrapezoidalModel.points()[0]", onPropertyChanged);
    var p2PropSub = HELPERS.subscribePropertyChange(this, barTrapezoidalModel.points()[1], "barTrapezoidalModel.points()[1]", onPropertyChanged);
    var selectSub = HELPERS.subscribePropertyChange(this, barTrapezoidalModel.selected, "MODEL.loads.bar.assignedTo", onLoadSelected);

    //var _rotationPropSub = HELPERS.subscribePropertyChange(this, barTrapezoidalModel.rotation, "barTrapezoidalModel.rotation", onPropertyChanged);

    this.clean = function () {
        for (var i = 0; i < instances.length; i++) {
            removeInstance(i);
        }
        instances = [];
        assignedSub.dispose();
        selectSub.dispose();
        p1PropSub.dispose();
        p2PropSub.dispose();
        //_rotationPropSub.dispose();
        render();
    }
    this.getAllArrowsIds = function () {
        var ids = [];
        for (var i = 0; i < instances.length; i++) {
            ids.push(instances[i].arrowsId);
        }
        return ids;
    }
    for (var index = 0; index < barTrapezoidalModel.barAttachments().length; index++) {
        onBarAssigned(barTrapezoidalModel.barAttachments()[index], index);
    }
    HELPERS.subscribePropertyChange(this, barTrapezoidalModel.coordinatesAbsolute, "VIEW3D.BarTrapezoidal.coordinatesAbsolute", function () {
        VIEWUI.spinners.refresh("#VIEWUI_coordinateX", 1);
    });

    CORE.loadModelScale.subscribe(newVal => {
        onPropertyChanged();
    });
};

});