﻿'use strict';
var VIEW3D = VIEW3D || {}

define([], function () {
// class VIEW3D.BarUniform
// binds to MODEL.load.node.BarUniform and creates geometry
// that represents that BarUniform model. Once BarUniform model
// is assigned to Bar this class creates new Object3D and
// adds it to the parent
VIEW3D.BarUniform = function(barUniformModel, arrows, fills) { //
    var instances = []; // synchronnized with barUniformModel.assignedTo
    this.arrows = arrows;

    var collider = new VIEW3D.collision.DisjunctionCollider();
    collider.model = barUniformModel;
    this.getCollider = function () {
        return collider;
    }

    var getArrowsData = function(node1Position, node2Position) {
        var arrowsData = [];
        var v = HELPERS.toVector3(barUniformModel.forces());

        function addArrowRow() {
            var addArrow = function(pos) {
                var v = barUniformModel.forces();
                if (v.x !== 0) {
                    arrowsData.push({
                        dir: new THREE.Vector3(v.x, 0, 0),
                        pos: new THREE.Vector3(pos.x, pos.y, pos.z),
                        color: new THREE.Vector3(1, 0, 0),
                        type: VIEW3D.Arrows.Arrow
                    });
                }
                if (v.y !== 0) {
                    arrowsData.push({
                        dir: new THREE.Vector3(0, v.y, 0),
                        pos: new THREE.Vector3(pos.x, pos.y, pos.z),
                        color: new THREE.Vector3(0, 1, 0),
                        type: VIEW3D.Arrows.Arrow
                    });
                }
                if (v.z !== 0) {
                    arrowsData.push({
                        dir: new THREE.Vector3(0, 0, v.z),
                        pos: new THREE.Vector3(pos.x, pos.y, pos.z),
                        color: new THREE.Vector3(0, 0, 1),
                        type: VIEW3D.Arrows.Arrow
                    });
                }
            }
            var addLine = function(pos1, pos2) {
                var v = barUniformModel.forces();
                if (v.x !== 0) {
                    arrowsData.push({
                        dir1: new THREE.Vector3(v.x, 0, 0),
                        dir2: new THREE.Vector3(v.x, 0, 0),
                        pos1: new THREE.Vector3(pos1.x, pos1.y, pos1.z),
                        pos2: new THREE.Vector3(pos2.x, pos2.y, pos2.z),
                        color: new THREE.Vector3(1, 0, 0),
                        type: VIEW3D.Arrows.Line
                    });
                }
                if (v.y !== 0) {
                    arrowsData.push({
                        dir1: new THREE.Vector3(0, v.y, 0),
                        dir2: new THREE.Vector3(0, v.y, 0),
                        pos1: new THREE.Vector3(pos1.x, pos1.y, pos1.z),
                        pos2: new THREE.Vector3(pos2.x, pos2.y, pos2.z),
                        color: new THREE.Vector3(0, 1, 0),
                        type: VIEW3D.Arrows.Line
                    });
                }
                if (v.z !== 0) {
                    arrowsData.push({
                        dir1: new THREE.Vector3(0, 0, v.z),
                        dir2: new THREE.Vector3(0, 0, v.z),
                        pos1: new THREE.Vector3(pos1.x, pos1.y, pos1.z),
                        pos2: new THREE.Vector3(pos2.x, pos2.y, pos2.z),
                        color: new THREE.Vector3(0, 0, 1),
                        type: VIEW3D.Arrows.Line
                    });
                }
            }
            //draw arrows at the start and end nodes of the bar
            addArrow(node1Position);
            addArrow(node2Position);
            addLine(node1Position, node2Position);
            var barLength = node1Position.distanceTo(node2Position);
            var distance = 1; // distance between arrows, will get rounded so that the bar's length will be it's multiple
            var quotient = barLength / distance;
            var fits = Math.floor(quotient); // fits that many arrows
            var doesntFit = quotient % 1; // the remaining part of bar (too small to fit an arrow)
            var enlargeDistanceBy = doesntFit / fits; // enlarge distance by
            distance += enlargeDistanceBy;
            //draw the rest of the arrows
            var sub = new THREE.Vector3().subVectors(node2Position, node1Position);
            for (var i = 0 + distance, ii = barLength; i < ii; i += distance) {
                var newVertex = node1Position.clone().add(sub.setLength(i));
                addArrow(newVertex);
            }
        }

        if (v.x !== 0.0) {
            addArrowRow(new THREE.Vector3(v.x, 0, 0), new THREE.Color(1, 0, 0));
        }
        if (v.y !== 0.0) {
            addArrowRow(new THREE.Vector3(0, v.y, 0), new THREE.Color(0, 1, 0));
        }
        if (v.z !== 0.0) {
            addArrowRow(new THREE.Vector3(0, 0, v.z), new THREE.Color(0, 0, 1));
        }
        return arrowsData;
    }

    var createFillObjects = function (barTrapezoidalModel, barAttachment) {

        var object = new THREE.Object3D();
        var fills = [];

        var node1pos = barAttachment.bar.node1().position();
        var node2pos = barAttachment.bar.node2().position();
        var force = barTrapezoidalModel.forces();            // force values as vectors
        var v1 = node1pos;                                   // positions on bar in world space
        var v2 = node2pos;

        var isZero = function (x) {
            return Math.abs(x) < 0.00001;
        }

        if (!isZero(force.x)) {
            var fill = {
                force1pos: v1,
                force2pos: v2,
                force1value: new MODEL.types.Vector(force.x, 0, 0),
                force2value: new MODEL.types.Vector(force.x, 0, 0),
                color: { x: 1, y: 0, z: 0 },
                type: VIEW3D.LoadFills.Fill
            }

            fills.push(fill);
        }

        if (!isZero(force.y)) {
            var fill = {
                force1pos: v1,
                force2pos: v2,
                force1value: new MODEL.types.Vector(0, force.y, 0),
                force2value: new MODEL.types.Vector(0, force.y, 0),
                color: { x: 0, y: 1, z: 0 },
                type: VIEW3D.LoadFills.Fill
            }

            fills.push(fill);
        }

        if (!isZero(force.z)) {
            var fill = {
                force1pos: v1,
                force2pos: v2,
                force1value: new MODEL.types.Vector(0, 0, force.z),
                force2value: new MODEL.types.Vector(0, 0, force.z),
                color: { x: 0, y: 0, z: 1 },
                type: VIEW3D.LoadFills.Fill
            }

            fills.push(fill);
        }

        return fills;
    }

    var removeInstance = function(index) {
        if(instances[index].arrowsId !== 0) {
            collider.removeCollider(arrows.getCollider(instances[index].arrowsId));
        }
        arrows.removeArrows(instances[index].arrowsId);
        fills.removeFills(instances[index].fillsId);
        instances[index].positionChangeSubscription1.dispose();
        instances[index].positionChangeSubscription2.dispose();
        instances[index].positionChangeSubscription3.dispose();
        instances[index].positionChangeSubscription4.dispose();
    }
    var onNodePositionChange = function(index) {
        removeInstance(index);
        var node = barUniformModel.barAttachments()[index];
        onBarAssigned(node, index);
    }
    var onBarAssigned = function (barAttachment, index) {
        if (!isEmpty(barAttachment)) {

            var instance = {};
            var arrowsData = getArrowsData(HELPERS.toVector3(barAttachment.bar.node1().position()), HELPERS.toVector3(barAttachment.bar.node2().position()));
            instance.arrowsId = arrows.addArrows(arrowsData, barUniformModel);
            var fillsData = createFillObjects(barUniformModel, barAttachment);
            instance.fillsId = fills.addFills(fillsData);
            instance.positionChangeSubscription1 = HELPERS.subscribePropertyChange(VIEW3D.BarUniform, barAttachment.bar.node1, "bar.node1", onNodePositionChange, index);
            instance.positionChangeSubscription2 = HELPERS.subscribePropertyChange(VIEW3D.BarUniform, barAttachment.bar.node2, "bar.node2", onNodePositionChange, index);
            instance.positionChangeSubscription3 = HELPERS.subscribePropertyChange(VIEW3D.BarUniform, barAttachment.bar.node1().position, "bar.node1.position", onNodePositionChange, index);
            instance.positionChangeSubscription4 = HELPERS.subscribePropertyChange(VIEW3D.BarUniform, barAttachment.bar.node2().position, "bar.node2.position", onNodePositionChange, index);
            instances[index] = instance;
            if (instance.arrowsId !== 0) {
                collider.addCollider(arrows.getCollider(instance.arrowsId));
            }
        }
    }
    var onBarUnassigned = function(node, index) {
        removeInstance(index);
        instances.splice(index, 1);
    }
    var onPropertyChanged = function() {
        for (var i = 0; i < barUniformModel.barAttachments().length; i++) {
            var instance = instances[i];
            if(instance.arrowsId !== 0) {
                collider.removeCollider(arrows.getCollider(instance.arrowsId));
            }
            arrows.removeArrows(instance.arrowsId);
            fills.removeFills(instance.fillsId);
            var bar = barUniformModel.barAttachments()[i].bar;
            var arrowsData = getArrowsData(HELPERS.toVector3(bar.node1().position()), HELPERS.toVector3(bar.node2().position()));
            instance.arrowsId = arrows.addArrows(arrowsData, barUniformModel);
            var fillsData = createFillObjects(barUniformModel, barUniformModel.barAttachments()[i]);
            instance.fillsId = fills.addFills(fillsData);
            if(instance.arrowsId !== 0) {
                collider.addCollider(arrows.getCollider(instance.arrowsId));
            }
        }
    }

    var onLoadSelected = function (val) {

        var newColor;
        for (var i = 0; i < instances.length; i++) {
            var arrowData = this.arrows.getArrowsData(instances[i].arrowsId);
            if (isEmpty(arrowData)) continue;
            arrowData = arrowData.data;

            for (var j = 0; j < arrowData.length; j++) {

                if (val) newColor = arrowData[j].selectedColor;
                else newColor = arrowData[j].originalColor;

                arrowData[j].color = newColor;
            }
        }
        this.arrows.updateAllArrows();
        render();
    }

    var assignedSub = HELPERS.subscribeArrayChange(this, barUniformModel.barAttachments, "MODEL.loads.node.assignedTo", onBarAssigned, onBarUnassigned);
    var forcePropSub = HELPERS.subscribePropertyChange(this, barUniformModel.forces, "barUniformModel.forces", onPropertyChanged);
    var selectSub = HELPERS.subscribePropertyChange(this, barUniformModel.selected, "MODEL.loads.bar.assignedTo", onLoadSelected);

    this.clean = function() {
        for (var i = 0; i < instances.length; i++) {
            removeInstance(i);
        }
        instances = [];
        assignedSub.dispose();
        forcePropSub.dispose();
        selectSub.dispose();
        //_rotationPropSub.dispose();
        render();
    }
    this.getAllArrowsIds = function() {
        var ids = [];
        for (var i = 0; i < instances.length; i++) {
            ids.push(instances[i].arrowsId);
        }
        return ids;
    }
    //var _rotationPropSub = HELPERS.subscribePropertyChange(this, barUniformModel.rotation, "barUniformModel.rotation", onPropertyChanged);
    for (var index = 0; index < barUniformModel.barAttachments().length; index++) {
        onBarAssigned(barUniformModel.barAttachments()[index], index);
    }

    CORE.loadModelScale.subscribe(newVal => {
        onPropertyChanged();
    });
};

});