﻿'use strict';
var VIEW3D = VIEW3D || {}

define(['js/view3d/utils/VIEW3D.utils', 'js/view3d/materials/VIEW3D.materials.ChartEdge', 'js/view3d/materials/VIEW3D.materials.Chart'], function() {

    VIEW3D.BarsResults = function(barResultsModel, viewOptions, isResultsWindow) {
        this.barResultsModel = barResultsModel;
        this.viewOptions = viewOptions; // set in properties or by parameter
        this.isResultsWindow = isResultsWindow;
        var isResults = false;
        var fxChartObject = null;
        var mxChartObject = null;
        var fyChartObject = null;
        var fzChartObject = null;
        var myChartObject = null;
        var mzChartObject = null;
        var uxChartObject = null;
        var uyChartObject = null;
        var uzChartObject = null;
        var chartColorDisp = new THREE.Color(0, 38.0 / 255.0, 255.0 / 255.0);
        var chartColorFx = new THREE.Color(0, 153.0 / 255.0, 204.0 / 255.0);
        var chartColorFy = new THREE.Color(0, 153.0 / 255.0, 55.0 / 255.0);
        var chartColorFz = new THREE.Color(178.0 / 255.0, 0, 255.0 / 255.0);
        var chartColorMx = new THREE.Color(255.0 / 255.0, 216.0 / 255.0, 0);
        var chartColorMy = new THREE.Color(127.0 / 255.0, 0, 0);
        var chartColorMz = new THREE.Color(255.0 / 255.0, 0, 0);
        

        //static members
        VIEW3D.BarsResults.opacity = !isResultsWindow ? 0.3 : 0.5;
        this.displacementColor = chartColorDisp;
        VIEW3D.BarsResults.edgeColor = new THREE.Color(0.4, 0.4, 0.4);
        this.fxPositiveColor = chartColorFx; //new THREE.Color(0.0, 1.0, 0.0);   // colors are inverted, but they work (?)
        this.fxNegativeColor = chartColorFx; //new THREE.Color(1.0, 0.0, 0.0);
        this.mxPositiveColor = chartColorMx; //new THREE.Color(1.0, 1.0, 0.0);
        this.mxNegativeColor = chartColorMx; //new THREE.Color(0.0, 0.0, 1.0);
        this.fyPositiveColor = chartColorFy; //new THREE.Color(1.0, 1.0, 0.0);
        this.fyNegativeColor = chartColorFy; //new THREE.Color(0.0, 0.0, 1.0);
        this.fzPositiveColor = chartColorFz; //new THREE.Color(1.0, 1.0, 0.0);
        this.fzNegativeColor = chartColorFz; //new THREE.Color(0.0, 0.0, 1.0);
        this.myPositiveColor = chartColorMy; //new THREE.Color(1.0, 0.0, 0.0);
        this.myNegativeColor = chartColorMy; //new THREE.Color(0.0, 1.0, 0.0);
        this.mzPositiveColor = chartColorMz; //new THREE.Color(1.0, 0.0, 0.0);
        this.mzNegativeColor = chartColorMz; //new THREE.Color(0.0, 1.0, 0.0);
        this.chartColorUx = new THREE.Color(1, 0, 0);
        this.chartColorUy = new THREE.Color(0, 1, 0);
        this.chartColorUz = new THREE.Color(0, 0, 1);

        var Labels = function() {
            this.labels = [];
            this.addLabel = function(dir, baseX, baseY, baseZ, p1, pos, val, type) {
                if (val !== 0) {
                    this.labels.push({
                        pos: new THREE.Vector3(
                            p1.x + baseX.x * pos,
                            p1.y + baseY.x * pos,
                            p1.z + baseZ.x * pos
                        ),
                        dir: new THREE.Vector3(
                            dir.x * val,
                            dir.y * val,
                            dir.z * val
                        ),
                        text: convertUnitsResults(val, type)
                    });
                }
            }
            this.AddExtremes = function(maxDir, minDir, baseX, baseY, baseZ, p1, extremum, type) {
                this.addLabel(maxDir, baseX, baseY, baseZ, p1, extremum.maxPos, extremum.maxVal, type);
                if (extremum.minVal !== extremum.maxVal) {
                    this.addLabel(minDir, baseX, baseY, baseZ, p1, extremum.minPos, extremum.minVal, type);
                }
            }
            this.createObject = function(scale) {
                var structures = VIEW3D.utils.Labeling.createStructures();
                for (var i = 0; i < this.labels.length; i++) {
                    var label = this.labels[i];
                    VIEW3D.utils.Labeling.createLabel(structures, label, scale);
                }
                // TODO fix render order for labels
                structures.labelsObject.material.depthTest = false;
                return structures.labelsObject;
            }
        }
        this.Chart = function(positiveColor, nagativeColor) {
            this.positions = [];
            this.dirs = [];
            this.edgePositions = [];
            this.colors = [];
            this.edgeDirs = [];
            this.indices = [];
            this.positiveColor = positiveColor;
            this.nagativeColor = nagativeColor;
            this.lastValue = null;
            this.maxValue = 0.0;
            this.isResultsWindow = isResultsWindow;
            this.labels = new Labels();
            this.AddValue = function(x, value, dir, baseX, baseY, baseZ, p1, extremum) {
                this.maxValue = Math.max(Math.abs(value), this.maxValue); // max value for all bars
                if (!isEmpty(extremum)) {
                    this.extremum = extremum;
                }
                var x0 = baseX.x * x + p1.x;
                var y0 = baseY.x * x + p1.y;
                var z0 = baseZ.x * x + p1.z;
                var x1 = x0 + dir.x * value;
                var y1 = y0 + dir.y * value;
                var z1 = z0 + dir.z * value;
                this.edgePositions.push(x0);
                this.edgePositions.push(y0);
                this.edgePositions.push(z0);
                this.edgeDirs.push(dir.x);
                this.edgeDirs.push(dir.y);
                this.edgeDirs.push(dir.z);
                this.edgeDirs.push(value);
                var c;
                if (this.lastValue != null) {
                    var index = (this.positions.length / 3);
                    if (value * this.lastValue < 0) { // crossing point 0
                        var t = this.lastValue / (this.lastValue - value);
                        var lastValue = this.dirs[index * 4 - 1];
                        var cx = (this.positions[index * 3 - 3] + dir.x * lastValue) * (1.0 - t) + x1 * t;
                        var cy = (this.positions[index * 3 - 2] + dir.y * lastValue) * (1.0 - t) + y1 * t;
                        var cz = (this.positions[index * 3 - 1] + dir.z * lastValue) * (1.0 - t) + z1 * t;
                        this.positions.push(cx);
                        this.positions.push(cy);
                        this.positions.push(cz);
                        this.positions.push(cx);
                        this.positions.push(cy);
                        this.positions.push(cz);
                        this.dirs.push(0);
                        this.dirs.push(0);
                        this.dirs.push(0);
                        this.dirs.push(0);
                        this.dirs.push(0);
                        this.dirs.push(0);
                        this.dirs.push(0);
                        this.dirs.push(0);
                        c = this.lastValue > 0 ? this.positiveColor : this.nagativeColor;
                        this.colors.push(c.r);
                        this.colors.push(c.g);
                        this.colors.push(c.b);
                        c = value > 0 ? this.positiveColor : this.nagativeColor;
                        this.colors.push(c.r);
                        this.colors.push(c.g);
                        this.colors.push(c.b);
                        this.indices.push(index - 2);
                        this.indices.push(index - 1);
                        this.indices.push(index);
                        this.indices.push(index + 3);
                        this.indices.push(index + 2);
                        this.indices.push(index + 1);
                    } else {
                        this.indices.push(index - 2);
                        this.indices.push(index - 1);
                        this.indices.push(index + 1);
                        this.indices.push(index - 2);
                        this.indices.push(index + 1);
                        this.indices.push(index);
                    }
                }
                this.positions.push(x0);
                this.positions.push(y0);
                this.positions.push(z0);
                this.positions.push(x0);
                this.positions.push(y0);
                this.positions.push(z0);
                this.dirs.push(dir.x);
                this.dirs.push(dir.y);
                this.dirs.push(dir.z);
                this.dirs.push(0);
                this.dirs.push(dir.x);
                this.dirs.push(dir.y);
                this.dirs.push(dir.z);
                this.dirs.push(value);
                c = value > 0 ? this.positiveColor : this.nagativeColor;
                this.colors.push(c.r); // no gradient
                this.colors.push(c.g);
                this.colors.push(c.b);
                this.colors.push(c.r);
                this.colors.push(c.g);
                this.colors.push(c.b);
                this.lastValue = value;
            };
            this.CreateChartObject = function(edgeIndices) {
                var object = new THREE.Object3D();
                var scaleMaxValue = Math.max(Math.abs(this.extremum.maxVal), Math.abs(this.extremum.minVal)); // calculate max value for results window
                if (scaleMaxValue < 0.000001 && isResultsWindow) return object; // don't draw when force is smaller than epsilon
                var scale;
                if (this.isResultsWindow) { // when results window
                    scale = scaleMaxValue < 0.000001 ? 1.0 : 1.0 / scaleMaxValue;
                    scale *= 2; // multiply by results window height
                } else {
                    scale = this.maxValue < 0.000001 ? 1.0 : 1.0 / this.maxValue; // when normal 3d view
                    scale *= 0.3;
                }
                var chartGeometry = new THREE.BufferGeometry();
                chartGeometry.setIndex(new THREE.BufferAttribute(new Uint16Array(this.indices), 1));
                chartGeometry.addAttribute('position', new THREE.BufferAttribute(new Float32Array(this.positions), 3));
                chartGeometry.addAttribute('color', new THREE.BufferAttribute(new Float32Array(this.colors), 3));
                chartGeometry.addAttribute('dir', new THREE.BufferAttribute(new Float32Array(this.dirs), 4));
                var depthWrite = isResultsWindow ? true : false;
                var chartMesh = new THREE.Mesh(chartGeometry, new VIEW3D.materials.Chart(VIEW3D.BarsResults.opacity, scale, depthWrite));
                object.add(chartMesh);
                var chartEdgeGeometry = new THREE.BufferGeometry();
                chartEdgeGeometry.setIndex(new THREE.BufferAttribute(new Uint16Array(edgeIndices), 1));
                chartEdgeGeometry.addAttribute('position', new THREE.BufferAttribute(new Float32Array(this.edgePositions), 3));
                chartEdgeGeometry.addAttribute('dir', new THREE.BufferAttribute(new Float32Array(this.edgeDirs), 4));
                var edgeMaterial = new VIEW3D.materials.ChartEdge();
                edgeMaterial.uniforms.scale.value = scale; //this.maxValue == 0.0 ? 1.0 : 1.0 / this.maxValue;
                edgeMaterial.uniforms.lineColor.value = VIEW3D.BarsResults.edgeColor;
                var chartEdgeLine = new THREE.LineSegments(chartEdgeGeometry, edgeMaterial);
                object.add(chartEdgeLine);
                var labelsObject = this.labels.createObject(scale);
                object.add(labelsObject);
                return object;
            }
        }
        THREE.Object3D.call(this);
        this.onResultsChange = function() {
            while (this.children.length > 0) {
                this.remove(this.children[0]);
            }
            var barsResults = this.barResultsModel();
            this.isResults = !(isEmpty(barsResults) || barsResults.length < 1 );
            if (isEmpty(barsResults) || barsResults.length < 1 || !CORE.isResultsScreen) {
                render();
                return;
            }
            var linesPerSpline = 20.0;
            var displacemetPositions = [];
            var displacemetDirs = [];
            var edgeIndices = [];
            var currIndex = 0;
            var maxDisplacement = 0.0;
            var displacementLabels = new Labels();
            var fxChart = new this.Chart(this.fxPositiveColor, this.fxNegativeColor);
            var mxChart = new this.Chart(this.mxPositiveColor, this.mxNegativeColor);
            var fyChart = new this.Chart(this.fyPositiveColor, this.fyNegativeColor);
            var fzChart = new this.Chart(this.fzPositiveColor, this.fzNegativeColor);
            var myChart = new this.Chart(this.myPositiveColor, this.myNegativeColor);
            var mzChart = new this.Chart(this.mzPositiveColor, this.mzNegativeColor);
            var uxChart = new this.Chart(this.chartColorUx, this.chartColorUx);
            var uyChart = new this.Chart(this.chartColorUy, this.chartColorUy);
            var uzChart = new this.Chart(this.chartColorUz, this.chartColorUz);

            for (var barIndex = 0; barIndex < barsResults.length; barIndex++) {
                var barResults = barsResults[barIndex];
                var bar = barResults.bar;
                var p1 = bar.node1().position();
                var splines = barResults.data.splines;
                var extrema = barResults.data.extrema;
                var baseX = barResults.data.baseX;
                var baseY = barResults.data.baseY;
                var baseZ = barResults.data.baseZ;
                var startPos = 0.0;
                var barMaxDispDir = new THREE.Vector3();
                var barMaxDisp = 0;
                var barMinDispDir = new THREE.Vector3();
                var barMinDisp = Infinity;
                for (var splineIndex = 0; splineIndex < splines.length; splineIndex++) {
                    var spline = splines[splineIndex];
                    fxChart.lastValue = null;
                    mxChart.lastValue = null;
                    fyChart.lastValue = null;
                    fzChart.lastValue = null;
                    myChart.lastValue = null;
                    mzChart.lastValue = null;
                    uxChart.lastValue = null;
                    uyChart.lastValue = null;
                    uzChart.lastValue = null;
                    for (var lineIndex = 0; lineIndex < linesPerSpline + 1; lineIndex++) {
                        var splineT = (spline.end - startPos) * lineIndex / linesPerSpline; // 0 to spline lenth
                        var barT = startPos + splineT; // 0 to bar lenth
                        var x = barT;
                        var y = 0;
                        var z = 0;
                        //displacemen
                        var dx = polynomialValue(spline.UX, barT);
                        var dy = polynomialValue(spline.UYRZ, barT);
                        var dz = polynomialValue(spline.UZRY, barT);
                        var l = Math.sqrt(dx * dx + dy * dy + dz * dz);
                        if (!isResultsWindow) {
                            //normalize
                            if (l > 0) {
                                dx /= l;
                                dy /= l;
                                dz /= l;
                            }
                        }
                        if (isResultsWindow) {
                            baseX = new MODEL.types.Vector(1, 0, 0);
                            baseY = new MODEL.types.Vector(0, 1, 0);
                            baseZ = new MODEL.types.Vector(0, 0, 1);
                        }
                        var posx = baseX.x * x + baseX.y * y + baseX.z * z + p1.x;
                        var posy = baseY.x * x + baseY.y * y + baseY.z * z + p1.y;
                        var posz = baseZ.x * x + baseZ.y * y + baseZ.z * z + p1.z;
                        var dispx = baseX.x * dx + baseX.y * dy + baseX.z * dz;
                        var dispy = baseY.x * dx + baseY.y * dy + baseY.z * dz;
                        var dispz = baseZ.x * dx + baseZ.y * dy + baseZ.z * dz;
                        if (maxDisplacement < l) {
                            maxDisplacement = l;
                        }
                        if (barMaxDisp < l) {
                            barMaxDispDir.x = dispx;
                            barMaxDispDir.y = dispy;
                            barMaxDispDir.z = dispz;
                            barMaxDisp = l;
                        }
                        if (barMinDisp > l) {
                            barMinDispDir.x = dispx;
                            barMinDispDir.y = dispy;
                            barMinDispDir.z = dispz;
                            barMinDisp = l;
                        }
                        if (this.isResultsWindow) {
                            dispx = 0;
                            dispy = 1;
                            dispz = 0;
                        }

                        if (lineIndex > 0 || splineIndex > 0) {
                            edgeIndices.push(currIndex);
                        } else {
                            displacemetPositions.push(posx);
                            displacemetPositions.push(posy);
                            displacemetPositions.push(posz);
                            displacemetDirs.push(dispx);
                            displacemetDirs.push(dispy);
                            displacemetDirs.push(dispz);
                            displacemetDirs.push(l);
                            mxChart.AddValue(x, 0, { x: 0, y: 0, z: 0 }, baseX, baseY, baseZ, p1);
                            mzChart.AddValue(x, 0, { x: 0, y: 0, z: 0 }, baseX, baseY, baseZ, p1);
                            myChart.AddValue(x, 0, { x: 0, y: 0, z: 0 }, baseX, baseY, baseZ, p1);
                            fxChart.AddValue(x, 0, { x: 0, y: 0, z: 0 }, baseX, baseY, baseZ, p1);
                            fyChart.AddValue(x, 0, { x: 0, y: 0, z: 0 }, baseX, baseY, baseZ, p1);
                            fzChart.AddValue(x, 0, { x: 0, y: 0, z: 0 }, baseX, baseY, baseZ, p1);
                            uxChart.AddValue(x, 0, { x: 0, y: 0, z: 0 }, baseX, baseY, baseZ, p1);
                            uyChart.AddValue(x, 0, { x: 0, y: 0, z: 0 }, baseX, baseY, baseZ, p1);
                            uzChart.AddValue(x, 0, { x: 0, y: 0, z: 0 }, baseX, baseY, baseZ, p1);
                            edgeIndices.push(currIndex);
                            currIndex++;
                            edgeIndices.push(currIndex);
                        }


                        displacemetPositions.push(posx);
                        displacemetPositions.push(posy);
                        displacemetPositions.push(posz);
                        displacemetDirs.push(dispx);
                        displacemetDirs.push(dispy);
                        displacemetDirs.push(dispz);
                        displacemetDirs.push(l);
                        fxChart.AddValue(
                            x,
                            polynomialValue(spline.FX, barT), !isResultsWindow ? {
                                x: baseX.z,
                                y: baseY.z,
                                z: baseZ.z
                            } : {
                                x: 0,
                                y: 1,
                                z: 0
                            },
                            baseX,
                            baseY,
                            baseZ,
                            p1,
                            extrema.fx[0]);
                        mxChart.AddValue(
                            x,
                            polynomialValue(spline.MX, barT), !isResultsWindow ? {
                                x: baseX.z,
                                y: baseY.z,
                                z: baseZ.z
                            } : {
                                x: 0,
                                y: 1,
                                z: 0
                            },
                            baseX,
                            baseY,
                            baseZ,
                            p1,
                            extrema.mx[0]);
                        uxChart.AddValue(
                            x,
                            polynomialValue(spline.UX, barT), !isResultsWindow ? {
                                x: baseX.z,
                                y: baseY.z,
                                z: baseZ.z
                            } : {
                                x: 0,
                                y: 1,
                                z: 0
                            },
                            baseX,
                            baseY,
                            baseZ,
                            p1,
                            extrema.ux[0]);
                        var fymZderiv = spline.FYMZ.slice(1, spline.FYMZ.length);
                        fymZderiv[1] *= 2;
                        fymZderiv[2] *= 3;
                        fyChart.AddValue(
                            x,
                            polynomialValue(fymZderiv, barT), !isResultsWindow ? {
                                x: -baseX.y,
                                y: -baseY.y,
                                z: -baseZ.y
                            } : {
                                x: 0,
                                y: -1,
                                z: 0
                            },
                            baseX,
                            baseY,
                            baseZ,
                            p1,
                            extrema.fy[0]);
                        var fzmYderiv = spline.FZMY.slice(1, spline.FZMY.length);
                        fzmYderiv[1] *= 2;
                        fzmYderiv[2] *= 3;
                        fzChart.AddValue(
                            x,
                            polynomialValue(fzmYderiv, barT), !isResultsWindow ? {
                                x: baseX.z,
                                y: baseY.z,
                                z: baseZ.z
                            } : {
                                x: 0,
                                y: 1,
                                z: 0
                            },
                            baseX,
                            baseY,
                            baseZ,
                            p1,
                            extrema.fz[0]);
                        myChart.AddValue(
                            x,
                            polynomialValue(spline.FZMY, barT), !isResultsWindow ? {
                                x: baseX.z,
                                y: baseY.z,
                                z: baseZ.z
                            } : {
                                x: 0,
                                y: 1,
                                z: 0
                            },
                            baseX,
                            baseY,
                            baseZ,
                            p1,
                            extrema.my[0]);
                        mzChart.AddValue(
                            x,
                            polynomialValue(spline.FYMZ, barT), !isResultsWindow ? {
                                x: baseX.y,
                                y: baseY.y,
                                z: baseZ.y
                            } : {
                                x: 0,
                                y: 1,
                                z: 0
                            },
                            baseX,
                            baseY,
                            baseZ,
                            p1,
                            extrema.mz[0]);
                        uyChart.AddValue(
                            x,
                            polynomialValue(spline.UYRZ, barT), !isResultsWindow ? {
                                x: baseX.z,
                                y: baseY.z,
                                z: baseZ.z
                            } : {
                                x: 0,
                                y: 1,
                                z: 0
                            },
                            baseX,
                            baseY,
                            baseZ,
                            p1,
                            extrema.uy[0]);
                        uzChart.AddValue(
                            x,
                            polynomialValue(spline.UZRY, barT), !isResultsWindow ? {
                                x: baseX.y,
                                y: baseY.y,
                                z: baseZ.y
                            } : {
                                x: 0,
                                y: 1,
                                z: 0
                            },
                            baseX,
                            baseY,
                            baseZ,
                            p1,
                            extrema.uz[0]);
                        var lastPointOnBar = (lineIndex === linesPerSpline) && (splineIndex === splines.length - 1);
                        if (lastPointOnBar) {

                            displacemetPositions.push(posx);
                            displacemetPositions.push(posy);
                            displacemetPositions.push(posz);
                            displacemetDirs.push(dispx);
                            displacemetDirs.push(dispy);
                            displacemetDirs.push(dispz);
                            displacemetDirs.push(l);
                            mxChart.AddValue(x, 0, { x: 0, y: 0, z: 0 }, baseX, baseY, baseZ, p1);
                            mzChart.AddValue(x, 0, { x: 0, y: 0, z: 0 }, baseX, baseY, baseZ, p1);
                            myChart.AddValue(x, 0, { x: 0, y: 0, z: 0 }, baseX, baseY, baseZ, p1);
                            fxChart.AddValue(x, 0, { x: 0, y: 0, z: 0 }, baseX, baseY, baseZ, p1);
                            fyChart.AddValue(x, 0, { x: 0, y: 0, z: 0 }, baseX, baseY, baseZ, p1);
                            fzChart.AddValue(x, 0, { x: 0, y: 0, z: 0 }, baseX, baseY, baseZ, p1);
                            uxChart.AddValue(x, 0, { x: 0, y: 0, z: 0 }, baseX, baseY, baseZ, p1);
                            uyChart.AddValue(x, 0, { x: 0, y: 0, z: 0 }, baseX, baseY, baseZ, p1);
                            uzChart.AddValue(x, 0, { x: 0, y: 0, z: 0 }, baseX, baseY, baseZ, p1);
                            edgeIndices.push(currIndex);
                            currIndex++;
                            edgeIndices.push(currIndex);
                        } else {
                            edgeIndices.push(currIndex);
                        }

                        currIndex++;
                    }
                    startPos = spline.end; // start of next spline
                }
                //Extremes
                if (!isResultsWindow) {
                    displacementLabels.AddExtremes(
                        barMaxDispDir,
                        barMinDispDir,
                        baseX,
                        baseY,
                        baseZ,
                        p1,
                        extrema.disp[0],
                        "Displacement");
                    fxChart.labels.AddExtremes({
                            x: baseX.z,
                            y: baseY.z,
                            z: baseZ.z
                        }, {
                            x: baseX.z,
                            y: baseY.z,
                            z: baseZ.z
                        },
                        baseX,
                        baseY,
                        baseZ,
                        p1,
                        extrema.fx[0],
                        "Force");
                    mxChart.labels.AddExtremes({
                            x: baseX.z,
                            y: baseY.z,
                            z: baseZ.z
                        }, {
                            x: baseX.z,
                            y: baseY.z,
                            z: baseZ.z
                        },
                        baseX,
                        baseY,
                        baseZ,
                        p1,
                        extrema.mx[0],
                        "Moment");
                    fyChart.labels.AddExtremes({
                            x: -baseX.y,
                            y: -baseY.y,
                            z: -baseZ.y
                        }, {
                            x: -baseX.y,
                            y: -baseY.y,
                            z: -baseZ.y
                        },
                        baseX,
                        baseY,
                        baseZ,
                        p1,
                        extrema.fy[0],
                        "Force");
                    fzChart.labels.AddExtremes({
                            x: baseX.z,
                            y: baseY.z,
                            z: baseZ.z
                        }, {
                            x: baseX.z,
                            y: baseY.z,
                            z: baseZ.z
                        },
                        baseX,
                        baseY,
                        baseZ,
                        p1,
                        extrema.fz[0],
                        "Force");
                    myChart.labels.AddExtremes({
                            x: baseX.z,
                            y: baseY.z,
                            z: baseZ.z
                        }, {
                            x: baseX.z,
                            y: baseY.z,
                            z: baseZ.z
                        },
                        baseX,
                        baseY,
                        baseZ,
                        p1,
                        extrema.my[0],
                        "Moment");
                    mzChart.labels.AddExtremes({
                            x: -baseX.y,
                            y: -baseY.y,
                            z: -baseZ.y
                        }, {
                            x: -baseX.y,
                            y: -baseY.y,
                            z: -baseZ.y
                        },
                        baseX,
                        baseY,
                        baseZ,
                        p1,
                        extrema.mz[0],
                        "Moment");
                }
            }
            var displacementGeometry = new THREE.BufferGeometry();
            displacementGeometry.setIndex(new THREE.BufferAttribute(new Uint16Array(edgeIndices), 1));
            displacementGeometry.addAttribute('position', new THREE.BufferAttribute(new Float32Array(displacemetPositions), 3));
            displacementGeometry.addAttribute('dir', new THREE.BufferAttribute(new Float32Array(displacemetDirs), 4));
            var scale;
            if (this.isResultsWindow) { // when results window
                var dispExtremum = this.barResultsModel()[0].data.extrema.disp[0];
                var scaleMaxValue = Math.max(Math.abs(dispExtremum.maxVal), Math.abs(dispExtremum.minVal)); // calculate max value for results window
                scale = scaleMaxValue === 0.0 ? 1.0 : 1.0 / scaleMaxValue; // temporarily using maxDisplacement
                scale *= 2; // multiply by results window height
            } else {
                scale = maxDisplacement === 0.0 ? 1.0 : 1.0 / maxDisplacement; // when normal 3d view
                scale *= 0.3;
            }
            var edgeMaterial = new VIEW3D.materials.ChartEdge(scale);
            edgeMaterial.uniforms.scale.value = scale;
            edgeMaterial.uniforms.lineColor.value = this.displacementColor;
            this.displacement = new THREE.LineSegments(displacementGeometry, edgeMaterial);
            if (this.viewOptions.showCalcDisplacement()) {
                this.add(this.displacement);
            }
            this.displacement.add(displacementLabels.createObject(scale));
            this.forces = new THREE.Object3D();
            fxChartObject = fxChart.CreateChartObject(edgeIndices);
            this.forces.fx = (fxChartObject);
            fyChartObject = fyChart.CreateChartObject(edgeIndices);
            this.forces.fy = (fyChartObject);
            fzChartObject = fzChart.CreateChartObject(edgeIndices);
            this.forces.fz = (fzChartObject);
            mxChartObject = mxChart.CreateChartObject(edgeIndices);
            this.forces.mx = (mxChartObject);
            myChartObject = myChart.CreateChartObject(edgeIndices);
            this.forces.my = (myChartObject);
            mzChartObject = mzChart.CreateChartObject(edgeIndices);
            this.forces.mz = (mzChartObject);
            uxChartObject = uxChart.CreateChartObject(edgeIndices);
            this.forces.ux = (uxChartObject);
            uyChartObject = uyChart.CreateChartObject(edgeIndices);
            this.forces.uy = (uyChartObject);
            uzChartObject = uzChart.CreateChartObject(edgeIndices);
            this.forces.uz = (uzChartObject);
            // remove unnecessary?
            if (this.viewOptions.showCalcForceFx()) {
                this.add(this.forces.fx);
            } else {
                this.remove(this.forces.fx);
            }
            if (this.viewOptions.showCalcForceFy()) {
                this.add(this.forces.fy);
            } else {
                this.remove(this.forces.fy);
            }
            if (this.viewOptions.showCalcForceFz()) {
                this.add(this.forces.fz);
            } else {
                this.remove(this.forces.fz);
            }
            if (this.viewOptions.showCalcForceMx()) {
                this.add(this.forces.mx);
            } else {
                this.remove(this.forces.mx);
            }
            if (this.viewOptions.showCalcForceMy()) {
                this.add(this.forces.my);
            } else {
                this.remove(this.forces.my);
            }
            if (this.viewOptions.showCalcForceMz()) {
                this.add(this.forces.mz);
            } else {
                this.remove(this.forces.mz);
            }
            if (this.viewOptions.showCalcForceUx()) {
                this.add(this.forces.ux);
            } else {
                this.remove(this.forces.ux);
            }
            if (this.viewOptions.showCalcForceUy()) {
                this.add(this.forces.uy);
            } else {
                this.remove(this.forces.uy);
            }
            if (this.viewOptions.showCalcForceUz()) {
                this.add(this.forces.uz);
            } else {
                this.remove(this.forces.uz);
            }
            render();
        }
        var polynomialValue = function(factors, x) {
            var pow = 0;
            var val = 0;
            for (var i = 0; i < factors.length; i++) {
                val += factors[i] * Math.pow(x, pow);
                pow++;
            }
            return val;
        }
        HELPERS.subscribePropertyChange(this, this.barResultsModel, "barResultsModel", this.onResultsChange);
        HELPERS.subscribePropertyChange(this, VIEWUI.viewOptions.unitDisplacement, "VIEWUI.viewOptions.unitDisplacement", this.onResultsChange);
        HELPERS.subscribePropertyChange(this, VIEWUI.viewOptions.unitForce, "5000", this.onResultsChange);
        HELPERS.subscribePropertyChange(this, VIEWUI.viewOptions.unitStress, "VIEWUI.viewOptions.unitStress", this.onResultsChange);
        HELPERS.subscribePropertyChange(this, VIEWUI.viewOptions.unitMomentForce, "VIEWUI.viewOptions.unitMomentForce", this.onResultsChange);
        HELPERS.subscribePropertyChange(this, VIEWUI.viewOptions.unitForcePrecision, "VIEWUI.viewOptions.unitForcePrecision", this.onResultsChange);
        HELPERS.subscribePropertyChange(this, VIEWUI.viewOptions.unitStressPrecision, "VIEWUI.viewOptions.unitStressPrecision", this.onResultsChange);
        HELPERS.subscribePropertyChange(this, VIEWUI.viewOptions.unitMomentPrecision, "VIEWUI.viewOptions.unitMomentPrecision", this.onResultsChange);
        HELPERS.subscribePropertyChange(this, VIEWUI.viewOptions.unitPressurePrecision, "VIEWUI.viewOptions.unitPressurePrecision", this.onResultsChange);
        HELPERS.subscribePropertyChange(this, VIEWUI.viewOptions.unitDisplacementPrecision, "VIEWUI.viewOptions.unitDisplacementPrecision", this.onResultsChange);
        this.onResultsChange();
        this.subscribeViewOptions = function() {
            HELPERS.subscribePropertyChange(this, this.viewOptions.showCalcDisplacement, "this.viewOptions.showCalcDisplacement", function(showCalcDisplacement) {
                if (showCalcDisplacement && this.displacement && this.isResults) {
                    this.add(this.displacement);
                } else {
                    this.remove(this.displacement);
                }
                render();
            });

            HELPERS.subscribePropertyChange(this, this.viewOptions.showCalcForceFx, "this.viewOptions.showCalcForceFx", function(showCalcForceFx) {
                if (showCalcForceFx && this.forces && this.isResults) {
                    this.add(this.forces.fx);
                } else if (this.forces) {
                    this.remove(this.forces.fx);
                }
                render();
            });

            HELPERS.subscribePropertyChange(this, this.viewOptions.showCalcForceFy, "this.viewOptions.showCalcForceFy", function(showCalcForceFy) {
                if (showCalcForceFy && this.forces && this.isResults) {
                    this.add(this.forces.fy);
                } else if (this.forces) {
                    this.remove(this.forces.fy);
                }
                render();
            });

            HELPERS.subscribePropertyChange(this, this.viewOptions.showCalcForceFz, "this.viewOptions.showCalcForceFz", function(showCalcForceFz) {
                if (showCalcForceFz && this.forces && this.isResults) {
                    this.add(this.forces.fz);
                } else if (this.forces) {
                    this.remove(this.forces.fz);
                }
                render();
            });

            HELPERS.subscribePropertyChange(this, this.viewOptions.showCalcForceMx, "this.viewOptions.showCalcForceMx", function(showCalcForceMx) {
                if (showCalcForceMx && this.forces && this.isResults) {
                    this.add(this.forces.mx);
                } else if (this.forces) {
                    this.remove(this.forces.mx);
                }
                render();
            });

            HELPERS.subscribePropertyChange(this, this.viewOptions.showCalcForceMy, "this.viewOptions.showCalcForceMy", function(showCalcForceMy) {
                if (showCalcForceMy && this.forces && this.isResults) {
                    this.add(this.forces.my);
                } else if (this.forces) {
                    this.remove(this.forces.my);
                }
                render();
            });

            HELPERS.subscribePropertyChange(this, this.viewOptions.showCalcForceMz, "this.viewOptions.showCalcForceMz", function(showCalcForceMz) {
                if (showCalcForceMz && this.forces && this.isResults) {
                    this.add(this.forces.mz);
                } else if (this.forces) {
                    this.remove(this.forces.mz);
                }
                render();
            });

            HELPERS.subscribePropertyChange(this, this.viewOptions.showCalcForceUx, "this.viewOptions.showCalcForceUx", function(showCalcForceUx) {
                if (showCalcForceUx && this.forces && this.isResults) {
                    this.add(this.forces.ux);
                } else if (this.forces) {
                    this.remove(this.forces.ux);
                }
                render();
            });

            HELPERS.subscribePropertyChange(this, this.viewOptions.showCalcForceUy, "this.viewOptions.showCalcForceUy", function(showCalcForceUy) {
                if (showCalcForceUy && this.forces && this.isResults) {
                    this.add(this.forces.uy);
                } else if (this.forces) {
                    this.remove(this.forces.uy);
                }
                render();
            });

            HELPERS.subscribePropertyChange(this, this.viewOptions.showCalcForceUz, "this.viewOptions.showCalcForceUz", function(showCalcForceUz) {
                if (showCalcForceUz && this.forces && this.isResults) {
                    this.add(this.forces.uz);
                } else if (this.forces) {
                    this.remove(this.forces.uz);
                }
                render();
            });
        }
        this.subscribeViewOptions();

        this.dispose = function() {
            for (var i = 0; i < this.children.length; i++) {
                if (this.children[i].dispose)
                    this.children[i].dispose();
                if (this.children[i].geometry)
                    this.children[i].geometry.dispose();
            }
        }
    }
    VIEW3D.BarsResults.prototype = Object.create(THREE.Object3D.prototype);
});