﻿'use strict';
var VIEW3D = VIEW3D || {}

VIEW3D.Box = function(box) {
    THREE.Object3D.call(this);
    this.model = box;
    var material;
    this.mesh = null;
    this.edgesHelper = null;

    this.updateModel = function () {
        if (!isEmpty(this.mesh)) {
            this.remove(this.mesh);
            this.remove(this.edgesHelper);
        }

        var size = box.size();
        var geometry = new THREE.BoxBufferGeometry(size.x, size.y, size.z);
        if (box.selected()) {
            material = new THREE.MeshBasicMaterial({ color: 'rgb(255, 25, 25)', transparent: true, opacity: 0.3 });
        } else {
            material = new THREE.MeshBasicMaterial({ color: 0x3D85C6, transparent: true, opacity: 0.8 });
        }
        this.mesh = new THREE.Mesh(geometry, material);
        var center = box.origin();
        this.mesh.position.set(center.x, center.y, center.z);
        this.add(this.mesh);
        this.edgesHelper = new THREE.EdgesHelper(this.mesh, 0x27557E, 0.1);
        this.add(this.edgesHelper);
        render();

        var o = this.model.origin();
        var s = this.model.size();
        var bb = {
            min: {
                x: o.x - s.x/2,
                y: o.y - s.y/2,
                z: o.z - s.z/2
            },
            max: {
                x: o.x + s.x/2,
                y: o.y + s.y/2,
                z: o.z + s.z/2
            }
        }
        this.collider = new VIEW3D.collision.BoxCollider(bb, new THREE.Matrix4());
        this.collider.model = this.model;

    }

    this.getCollider = function () {
        return this.collider
    }
}
VIEW3D.Box.prototype = Object.create(THREE.Object3D.prototype);