﻿'use strict';
var VIEW3D = VIEW3D || {}

define(['js/view3d/VIEW3D.Box'], function () {
    VIEW3D.Boxes = function (boxModel) {
        THREE.Object3D.call(this);
        var self = this;
        this.list = [];
        var collider = new VIEW3D.collision.CollidersContainer();
        this.getCollider = function () {
            return collider;
        }
        var reDraw = function (id) {
            for (var i = 0; i < self.list.length; i++) {
                if (self.list[i].id === id) {
                    var box3D = self.list[i].element;
                    collider.removeCollider(box3D.getCollider());
                    self.list[i].element.updateModel();
                    collider.addCollider(box3D.getCollider());
                }
            }
        }
        var onBoxAdded = function (box) {
            var box3D = new VIEW3D.Box(box);
            box3D.updateModel();
            collider.addCollider(box3D.getCollider());
            self.add(box3D);
            var subs = [];
            var selectSub = HELPERS.subscribePropertyChange(self, box.selected, "box.selected", reDraw, box.id);
            subs.push(selectSub);
            var sizeSub = HELPERS.subscribePropertyChange(self, box.size, "box.size", reDraw, box.id);
            subs.push(sizeSub);
            var originSub = HELPERS.subscribePropertyChange(self, box.origin, "box.origin", reDraw, box.id);
            subs.push(originSub);
            self.list.push({ element: box3D, id: box.id, subs: subs });
        }
        var onBoxRemoved = function (box) {
            for (var i = 0; i < self.list.length; i++) {
                if (self.list[i].id === box.id) {
                    for (var k = 0; k < self.list[i].subs.length; k++) {
                        self.list[i].subs[k].dispose();
                    }
                    collider.removeCollider(self.list[i].element.getCollider());
                    self.remove(self.list[i].element);
                    self.list.splice(i, 1);
                    break;
                }
            }
        }
        var boxesSub = HELPERS.subscribeArrayChange(this, boxModel, "MODEL.boxes", onBoxAdded, onBoxRemoved);
        for (var index = 0; index < CORE.model.boxes().length; index++) {
            onBoxAdded(CORE.model.boxes()[index]);
        }
        this.dispose = function () {
            boxesSub.dispose();
        }
    }
    VIEW3D.Boxes.prototype = Object.create(THREE.Object3D.prototype);
});