﻿'use strict';
var VIEW3D = VIEW3D || {}

define([
    'js/view3d/VIEW3D.Fills', 'js/view3d/VIEW3D.Loads', 'js/view3d/VIEW3D.SupportsNode', 'js/view3d/VIEW3D.MissingConnector', 'js/view3d/VIEW3D.MissingProfile', 'js/view3d/VIEW3D.MissingElement',
    'js/view3d/VIEW3D.RealNodes',
    'js/view3d/VIEW3D.PushButtons', 'js/view3d/VIEW3D.Boxes', 'js/view3d/VIEW3D.RealSupportsNode'
], function() {
    // class LineBars extends THREE.Object3D
    // binds to MODEL.Construction
    // it is a root Object3D of a construction 
    VIEW3D.Construction = function(model) {
        THREE.Object3D.call(this);
        this.view3dInitialized = ko.observable(false);
        this.editMode = ko.observable(false);
        this.activeWorkplane = new VIEW3D.ActiveWorkplane(model.activeWorkplane, 50, 1, 'lightgray');
        this.workplanes = [];
        for(var i = 0 ; i < model.workplanes().length ; i++) {
            this.workplanes.push(new VIEW3D.ActiveWorkplane(ko.observable(model.workplanes()[i]), 50, 1, 'lightgray'));
        }
        this.nodesObject3D = VIEWUI.viewOptions.showRealElements() ? new VIEW3D.RealNodes(model.nodes, model.constraints) : new VIEW3D.PointNodeCloud(model.nodes);
        this.barsObject3D = VIEWUI.viewOptions.showRealElements() ? new VIEW3D.RealBars(model.bars, model.constraints, MODEL.repository) : new VIEW3D.LineBars(model.bars);
        this.barsResults = new VIEW3D.BarsResults(model.barsResults, VIEWUI.viewOptions, false); // false - isResultsWindow
        this.loadsObject3D = VIEWUI.viewOptions.loadsVisible() ? new VIEW3D.Loads(model.loads) : null;

        this.fillsObject3D = new VIEW3D.Fills(model.fills);

        this.boxesObject3D = VIEWUI.viewOptions.boxesVisible ? new VIEW3D.Boxes(model.boxes) : null;

        this.pushButtons = VIEWUI.viewOptions.showRealElements() && VIEWUI.viewOptions.pushButtonsVisible() ? new VIEW3D.PushButtons(model.constraints, model) : null;

        this.externalModels = new VIEW3D.ExternalModels();

        this.editBarsObject3D = null;
        this.editNodesObject3D = null;
        this.add(this.nodesObject3D);
        this.add(this.barsObject3D);
        if (!isEmpty(this.loadsObject3D)) {
            this.add(this.loadsObject3D);
        }
        this.add(this.fillsObject3D);
        this.add(this.barsResults);
        if (!isEmpty(this.boxesObject3D)) {
            this.add(this.boxesObject3D);
        }
        if (!isEmpty(this.pushButtons)) {
            this.add(this.pushButtons);
        }
        this.add(this.externalModels);

        var collider = new VIEW3D.collision.CollidersContainer();
        this.addColliders = function() {
            //important to keep proper order
            //nodes before bars means that collision with node has higher priority
            // collider.addCollider(this.missingConnectors.getCollider());
            // collider.addCollider(this.missingProfiles.getCollider());
            if (!isEmpty(this.editNodesObject3D)) {
                collider.addCollider(this.editNodesObject3D.getCollider());
                collider.addCollider(this.editBarsObject3D.getCollider());
            }
            collider.addCollider(this.nodesObject3D.getCollider());
            collider.addCollider(this.barsObject3D.getCollider());
            if (VIEWUI.viewOptions.loadsVisible() === true) {
                collider.addCollider(this.loadsObject3D.getCollider());
            }
            collider.addCollider(this.boxesObject3D.getCollider());
            collider.addCollider(this.externalModels.getCollider());
        }
        this.removeColliders = function() {
            collider.removeCollider(this.barsObject3D.getCollider());
            collider.removeCollider(this.nodesObject3D.getCollider());
            if (collider.containsCollider(this.loadsObject3D.getCollider())) {
                collider.removeCollider(this.loadsObject3D.getCollider());
            }
            if (!isEmpty(this.editNodesObject3D)) {
                collider.removeCollider(this.editNodesObject3D.getCollider());
            }
            if (!isEmpty(this.editBarsObject3D)) {
                collider.removeCollider(this.editBarsObject3D.getCollider());
            }
            collider.removeCollider(this.boxesObject3D.getCollider());
            collider.removeCollider(this.externalModels.getCollider());
        }
        this.showMissingElements = function() {
            // this.missingElement = new VIEW3D.MissingElement(CORE.screenSize, model);
            // this.add(this.missingElement);
            // collider.addCollider(this.missingElement.getCollider());
        };
        this.getCollider = function() {
            return collider;
        };
        HELPERS.subscribePropertyChange(this, this.editMode, "editMode", function(editMode) {
            this.removeColliders();
            if (!isEmpty(this.editNodesObject3D)) {
                this.remove(this.editNodesObject3D);
                this.editNodesObject3D.dispose();
                this.editNodesObject3D = null;
            }
            if (!isEmpty(this.editBarsObject3D)) {
                this.remove(this.editBarsObject3D);
                this.editBarsObject3D.dispose();
                this.editBarsObject3D = null;
            }
            if (editMode && VIEWUI.viewOptions.showRealElements) {
                this.editNodesObject3D = new VIEW3D.PointNodeCloud(model.nodes);
                this.editBarsObject3D = new VIEW3D.LineBars(model.bars);
                this.add(this.editNodesObject3D);
                this.add(this.editBarsObject3D);
            }
            this.addColliders();
            render();
        });
        HELPERS.subscribePropertyChange(this, VIEWUI.viewOptions.showRealElements, "VIEWUI.viewOptions.showRealElements", function(showRealElements) {
            this.removeColliders();
            this.remove(this.barsObject3D);
            this.barsObject3D.dispose();
            this.barsObject3D = null;
            this.remove(this.nodesObject3D);
            this.nodesObject3D.dispose();
            this.nodesObject3D = null;
            if(!isEmpty(this.supportsObject3D)){
                this.remove(this.supportsObject3D);
                this.supportsObject3D.dispose();
                this.supportsObject3D = null;
            }
            if (!isEmpty(this.editNodesObject3D)) {
                this.remove(this.editNodesObject3D);
                this.editNodesObject3D.dispose();
                this.editNodesObject3D = null;
            }
            if (!isEmpty(this.editBarsObject3D)) {
                this.remove(this.editBarsObject3D);
                this.editBarsObject3D.dispose();
                this.editBarsObject3D = null;
            }
            if (!isEmpty(this.pushButtons)) {
                this.remove(this.pushButtons);
                this.pushButtons.dispose();
                this.pushButtons = null;
            }
            if (showRealElements) {
                this.barsObject3D = new VIEW3D.RealBars(model.bars, model.constraints, MODEL.repository);
                this.nodesObject3D = new VIEW3D.RealNodes(model.nodes, model.constraints);
                // this.supportsObject3D = new VIEW3D.RealSupportsNode(model.nodes, model.constraints);
                this.pushButtons = new VIEW3D.PushButtons(model.constraints, model);
                if (this.editMode()) {
                    this.editNodesObject3D = new VIEW3D.PointNodeCloud(model.nodes);
                    this.editBarsObject3D = new VIEW3D.LineBars(model.bars);
                }
            } else {
                this.barsObject3D = new VIEW3D.LineBars(model.bars);
                this.nodesObject3D = new VIEW3D.PointNodeCloud(model.nodes);
                this.supportsObject3D = new VIEW3D.SupportsNode(model.supports);
            }
            this.add(this.barsObject3D);
            this.add(this.nodesObject3D);
            if(!isEmpty(this.supportsObject3D)){
                this.add(this.supportsObject3D);
            }
            if (!isEmpty(this.editNodesObject3D)) {
                this.add(this.editNodesObject3D);
            }
            if (!isEmpty(this.editBarsObject3D)) {
                this.add(this.editBarsObject3D);
            }
            if (!isEmpty(this.pushButtons) && VIEWUI.viewOptions.pushButtonsVisible()) {
                this.add(this.pushButtons);
            }
            this.addColliders();
            render();
        });
        HELPERS.subscribePropertyChange(this, VIEWUI.viewOptions.supportsVisible, "supportsVisible", function(supportsVisible) {
            if (supportsVisible && this.children.indexOf(this.supportsObject3D) === -1) {
                if(!isEmpty(this.supportsObject3D)){
                    this.add(this.supportsObject3D);
                    render();
                }
            } else {
                if(!isEmpty(this.supportsObject3D)){
                    this.remove(this.supportsObject3D);
                    render();
                }
            }
        });
        HELPERS.subscribePropertyChange(this, VIEWUI.viewOptions.loadsVisible, "loadsVisible", function(loadsVisible) {
            this.removeColliders();
            if (loadsVisible && this.children.indexOf(this.loadsObject3D) === -1) {
                this.loadsObject3D.inputs.setInputVisibility(true);
                this.add(this.loadsObject3D);
                render();
            } else {
                this.loadsObject3D.inputs.setInputVisibility(false);
                this.remove(this.loadsObject3D);
                render();
            }
            this.addColliders();
        });
        HELPERS.subscribePropertyChange(this, VIEWUI.viewOptions.pushButtonsVisible, "pushButtonsVisible", function(pushButtonsVisible) {
            if (VIEWUI.viewOptions.showRealElements()) {
                this.removeColliders();
                if (pushButtonsVisible && this.children.indexOf(this.pushButtons) === -1) {
                    this.add(this.pushButtons);
                    render();
                } else {
                    this.remove(this.pushButtons);
                    render();
                }
                this.addColliders();
            }
        });
        HELPERS.subscribePropertyChange(this, VIEWUI.viewOptions.boxesVisible, "boxesVisible", function(boxesVisible) {
            if (boxesVisible && this.children.indexOf(this.boxesObject3D) === -1) {
                this.add(this.boxesObject3D);
                render();
            } else {
                this.remove(this.boxesObject3D);
                render();
            }
        });

        // this.view3dInitialized.subscribe(function (newVal) {
        //     if (newVal) {
        //         MODEL.repository.load3DModels();
        //     }
        // });

        this.view3dInitialized(true);
        this.addColliders();
        render();
    }
    VIEW3D.Construction.prototype = Object.create(THREE.Object3D.prototype);
});

