
define(['js/view3d/materials/VIEW3D.materials.ScreenSpace'], function () {
    rotObj = CORE.res.obj["rotation"];
    VIEW3D = VIEW3D || {};
    VIEW3D.Edit = VIEW3D.Edit || {};
    var controlsSize = 0.2;
    var arrowGeometry = rotObj.getObjectByName('arrow').geometry;
    arrowGeometry.applyMatrix(new THREE.Matrix4().makeScale(controlsSize,controlsSize,controlsSize));
    arrowGeometry.addAttribute('hightlightPosition', new THREE.Float32BufferAttribute(arrowGeometry.attributes.position.array.length,3));
    var centerGeometry = rotObj.getObjectByName('center').geometry;
    centerGeometry.applyMatrix(new THREE.Matrix4().makeScale(controlsSize,controlsSize,controlsSize));
    centerGeometry.addAttribute('hightlightPosition', new THREE.Float32BufferAttribute(arrowGeometry.attributes.position.array.length,3));

    var lineGeometry = new THREE.BufferGeometry();
    var linePositions = new Float32Array( 2 * 3 ); // 3 vertices per point
    lineGeometry.addAttribute( 'position', new THREE.BufferAttribute( linePositions, 3 ) );
    var lineMaterial = new THREE.LineBasicMaterial( { color: 0x888888 } );
    var line = new THREE.Line( lineGeometry,  lineMaterial );

    var setLineStartPos = function(startPos) {
        var positions = line.geometry.attributes.position.array;
        positions[0] = startPos.x;
        positions[1] = startPos.y;
        positions[2] = startPos.z;
        positions[3] = startPos.x;
        positions[4] = startPos.y;
        positions[5] = startPos.z;
        line.geometry.attributes.position.needsUpdate = true;
    }

    var setLineEndPos = function(endPos) {
        var positions = line.geometry.attributes.position.array;
        positions[3] = endPos.x;
        positions[4] = endPos.y;
        positions[5] = endPos.z;
        line.geometry.attributes.position.needsUpdate = true;
    }

    VIEW3D.Edit.MoveControl = function (length, moveSpace, pos, lineStart, allBars, color,  onChange, clearAllInputs, fromVec, toVec, offsetVec, extermas) {
        this.type = "EditControl"
        this.moveSpace = moveSpace;
        this.pos = pos;
        this.startLength = length;
        this.length = ko.observable(length);
        //this.inputs = new VIEW3D.MoveInputs(camera, CORE.screenSize);

        if(!isEmpty(lineStart)){
            //setLineStartPos(lineStart);
        }

        var material = new VIEW3D.materials.EditControl(color);
        material.transparent = true;

        this.mesh = new THREE.Mesh(arrowGeometry, material);
        this.mesh.matrixAutoUpdate = false;
        this.mesh.matrix = moveSpace.clone();
        this.mesh.matrix.setPosition(pos);
        this.mesh.renderOrder =  1e20;
        this.mesh.frustumCulled = false;
        var centerMaterial = new VIEW3D.materials.EditControl(new THREE.Vector4(0.3,0.3,0.3, 1));
        centerMaterial.transparent = true;
        var centerMesh = new THREE.Mesh(centerGeometry, centerMaterial);
        centerMesh.renderOrder =  1e20;
        centerMesh.frustumCulled = false;
        this.mesh.add(centerMesh);


        //collider
        var offset1 = new THREE.Vector3(controlsSize * 0.2, 0, 0);
        offset1.applyMatrix4(moveSpace);
        var offset2 = new THREE.Vector3(controlsSize * 0.8, 0, 0);
        offset2.applyMatrix4(moveSpace);
        this.collider = new VIEW3D.collision.ScreenSegmentCollider(pos, offset1, pos, offset2);
        this.collider.model = this;

        var findSnapPoints = function () {
            var result = [];
            var ray = new THREE.Ray(pos, new THREE.Vector3(1, 0, 0).applyMatrix4(moveSpace));
            var ray2 = ray.clone();
            ray2.direction.multiplyScalar(-1); //opposite direction
            for (var i = 0; i < allBars.length; i++) {
                var snapBar = allBars[i];
                //if (snapBar !== bar) {
                    var v0 = HELPERS.toVector3(snapBar.node1().position());
                    var v1 = HELPERS.toVector3(snapBar.node2().position());
                    var pointOnBar = new THREE.Vector3();
                    var dist = ray.distanceSqToSegment(v0, v1, undefined, pointOnBar);
                    if (dist < 0.001) {
                        result.push(pos.distanceTo(pointOnBar));
                    }
                    //opposite direction
                    dist = ray2.distanceSqToSegment(v0, v1, undefined, pointOnBar);
                    if (dist < 0.001) {
                        result.push(-pos.distanceTo(pointOnBar));
                    }
                //}
            }
            return result;
        }


        var sub = this.length.subscribe(function (newVal) {
            this.mesh.matrix = this.moveSpace.clone();
            var offset = newVal - this.startLength;
            var newPos = new THREE.Vector3(offset, 0, 0).applyMatrix4(this.moveSpace).add(pos);
            this.mesh.matrix.setPosition(newPos);
            setLineEndPos(newPos);

            //update collider
            this.collider.pos1 = newPos;
            this.collider.pos2 = newPos;
            render();
        }.bind(this));


        this.getRayDist = function (ray) {
            var u = ray.direction;
            var v = new THREE.Vector3(1, 0, 0).applyMatrix4(this.moveSpace);
            var w = new THREE.Vector3().subVectors(ray.origin, this.pos);
            var a = u.dot(u);         // always >= 0
            var b = u.dot(v);
            var c = v.dot(v);         // always >= 0
            var d = u.dot(w);
            var e = v.dot(w);
            var D = a * c - b * b;  // always >= 0
            var tN, tD = D;         // tc = tN / tD, default tD = D >= 0
            if (D > 0.001) {        // the lines are almost parallel
                tN = (a * e - b * d);
                return tN / tD
            } else {
                return undefined;
            }
        }

        this.setGrayed = function (grayed) {
            material.uniforms.controlColor.value.w = grayed ? 0.4 : 1.0;
            centerMaterial.uniforms.controlColor.value.w = grayed ? 0.4 : 1.0;
        }
        var calcCubeLength = function(val1, val2) {
            var val = Math.abs(val1 - val2);
            if(val < 0.00001) return 0.05;
            return val;
        }
        cube = new THREE.Mesh( new THREE.CubeGeometry(calcCubeLength(toVec.x, fromVec.x), calcCubeLength(toVec.y, fromVec.y), calcCubeLength(toVec.z, fromVec.z)), new THREE.MeshBasicMaterial({color: 0xff0000}) );
        cube.position.x = pos.x;
        cube.position.y = pos.y;
        cube.position.z = pos.z;
                
        this.startDrag = function (ray) {
            //clearAllInputs();
            this.setGrayed(false);

            //this.inputs.add(this, onChange);
            this.startDist = this.getRayDist(ray);
            this.snapDists = findSnapPoints();
            //if(!isEmpty(lineStart)){
            //    this.mesh.parent.add(line);
            //}
            scene.add( cube );
        }

        this.modifyRayToSnappingLines = function (ray) {
            var newray = { origin : ray.origin, direction : ray.direction };
            if(VIEW3D.SnapGridDefinition.snapToGrid() === true) {
                //snap to grid
                newray.origin = VIEW3D.SnapToGrid(ray.origin);
            }

            return newray;
        }
        this.drag = function (ray) {
            var maxSnapDistanceInPixels = 20;
            var maxSnapDistance = maxSnapDistanceInPixels / camera.projectionMatrix.elements[5] / CORE.screenSize().height;
            if (this.startDist !== undefined) {
                var dist = this.getRayDist(ray) - this.startDist;
                if(dist > extermas.max) {
                    dist = extermas.max;
                }
                if(dist < extermas.min) {
                    dist = extermas.min;
                }
                if (Math.abs(dist - this.startDist) > 0.001) {
                    var newLength = this.startLength + dist;// Math.round((this.startLength + dist) * epsilonGrid) / epsilonGrid;
                    this.length(newLength);
                    var currentOffsetVec = offsetVec.clone().multiplyScalar(-1*(newLength - this.startLength));
                    var newPosition = new THREE.Vector3(pos.x + currentOffsetVec.x, pos.y + currentOffsetVec.y, pos.z + currentOffsetVec.z);
                    if(VIEW3D.SnapGridDefinition.snapToGrid() === true) {
                        if(Math.abs(currentOffsetVec.x) > 0.00001) {
                            newPosition.x = VIEW3D.SnapXToGrid(newPosition.x);
                        }
                        if(Math.abs(currentOffsetVec.z) > 0.00001) {
                            newPosition.z = VIEW3D.SnapZToGrid(newPosition.z);
                        }
                    }

                    cube.position.x = newPosition.x;
                    cube.position.y = newPosition.y;
                    cube.position.z = newPosition.z;
                }
            }
        }

        this.endDrag = function () {
            onChange(this.length());
            if(!isEmpty(line.parent)) {
                line.parent.remove(line);
            }
            scene.remove( cube );
        }

        this.dispose = function () {
            if(!isEmpty(line.parent)) {
                line.parent.remove(line);
            }
            sub.dispose();
            //this.inputs.dispose();
        }

    }
});