define(['js/view3d/materials/VIEW3D.materials.ScreenSpace', 'js/view3d/VIEW3D.Angle'], function () {
    rotObj = CORE.res.obj["rotation"];
    VIEW3D = VIEW3D || {};
    VIEW3D.Edit = VIEW3D.Edit || {};
    var controlsSize = 0.2;
    var circleGeometry = rotObj.getObjectByName('circle').geometry;
    circleGeometry.applyMatrix(new THREE.Matrix4().makeScale(controlsSize,controlsSize,controlsSize));
    circleGeometry.addAttribute('hightlightPosition', new THREE.Float32BufferAttribute(circleGeometry.attributes.position.array.length,3));


    VIEW3D.Edit.RotationControl = function (angle, rotSpace, pos, onChange, angleOffset, clearAllInputs) {
        this.type = "EditControl"
        this.rotSpace = rotSpace;
        this.pos = pos;
        this.angle = ko.observable(angle);
        this.inputs = new VIEW3D.RotationInputs(camera, CORE.screenSize);
        this.angleOffset = angleOffset;


        var localRot = new THREE.Matrix4().makeRotationZ(angle + angleOffset);
        var transform = new THREE.Matrix4().multiplyMatrices(rotSpace, localRot);

        var material = new VIEW3D.materials.EditControl(new THREE.Vector4(0.5, 0.5, 0.5, 1.0));
        material.transparent = true;
        this.mesh = new THREE.Mesh(circleGeometry, material);
        this.mesh.matrixAutoUpdate = false;
        this.mesh.matrix = transform.clone();
        this.mesh.matrix.setPosition(pos);
        this.mesh.renderOrder = 1e20;//render as a last one -> always on top


        //collider
        var offset = new THREE.Vector3(controlsSize, 0, 0);
        offset.applyMatrix4(transform);
        this.collider = new VIEW3D.collision.ScreenPointCollider(pos, offset);
        this.collider.model = this;

        var angleSub = this.angle.subscribe(function (newVal) {
            //update mesh
            var localRot = new THREE.Matrix4().makeRotationZ(newVal + angleOffset);
            var newTransform = new THREE.Matrix4().multiplyMatrices(rotSpace, localRot);
            this.mesh.matrix = newTransform.clone();
            this.mesh.matrix.setPosition(pos);

            //update collider
            var offset = new THREE.Vector3(controlsSize, 0, 0);
            offset.applyMatrix4(newTransform);
            this.collider.offset = offset;
            render();
        }.bind(this));

        this.setGrayed = function (grayed) {
            material.uniforms.controlColor.value.w = grayed ? 0.4 : 1.0;
        }

        this.startDrag = function () {
            clearAllInputs();
            this.setGrayed(false);

            this.inputs.add(this, onChange);
            this.mesh.geometry = circleGeometry;
            this.angleVis = new VIEW3D.Angle();
            this.mesh.add(this.angleVis);
            this.dragStartAngle = angle;
        }

        this.drag = function (ray) {
            var rotSpace = this.rotSpace.clone();
            //rotSpace.transpose();
            rotSpace.setPosition(this.pos.clone());
            var inv = new THREE.Matrix4().getInverse(rotSpace);
            var localRay = ray.clone();
            localRay.applyMatrix4(inv);

            if (localRay.direction.z !== 0.0) {
                var t = -localRay.origin.z / localRay.direction.z;
                var x = localRay.origin.x + localRay.direction.x * t;
                var y = localRay.origin.y + localRay.direction.y * t;
                var angle = Math.atan2(y, x);

                var round = angle / (Math.PI / 4);
                round = Math.round(round);
                round *= Math.PI / 4;
                if (Math.abs(round - angle) < (Math.PI / 180) * 5) {//+-4st
                    angle = round;
                }

                angle -= this.angleOffset;
                var a = -angle + this.dragStartAngle;
                while(a > Math.PI) a-=Math.PI*2;
                while(a <-Math.PI) a+=Math.PI*2;
                this.angleVis.setAngle(a);

                this.angle(angle);
            }
        }

        this.endDrag = function () {
            onChange(this.angle());
            this.mesh.remove(this.angleVis);
        }

        this.dispose = function () {
            angleSub.dispose();
            this.mesh.remove(this.angleVis);
            this.inputs.dispose();
        }

    }

});