﻿'use strict';
var VIEW3D = VIEW3D || {}

VIEW3D.Fill = function (fill) {   
    THREE.Object3D.call(this);
    var geometry = new THREE.Geometry();
    var material;
    var i;
    var count = fill.positions().length;   
    if (fill.positions().length > 1) {     
        for (i = 0; i < fill.positions().length; i++) {
            geometry.vertices.push(fill.positions()[i].position);
        }
        geometry.vertices.push(fill.positions()[0].position);
        material = new THREE.LineBasicMaterial({ color: 0x0 });
        var newLane = new THREE.Line(geometry, material, THREE.LineStrip);
        this.add(newLane);
        // MESH
        if (count > 2) {
            var pairPointsTransform = [];
            var resultsPairPointsTransform = [];
            var triangulation = new VIEW3D.utils.Triangulation();
            geometry = new THREE.Geometry();
            material = new THREE.MeshBasicMaterial({
                color: 0x66CCFF,
                transparent: true,
                opacity: 0.8,
                side: THREE.DoubleSide
            });
            var transform = new VIEW3D.Transformation3D();
            transform.SetData(fill.positions()[0].position, fill.positions()[1].position, fill.positions()[count - 1].position);
            for (i = 0; i < count; i++) {
                pairPointsTransform[i] = new VIEW3D.VectorPair(fill.positions()[i].position, transform.TransformToLocal(fill.positions()[i].position));
            }
            triangulation.process(pairPointsTransform, resultsPairPointsTransform);
            if (resultsPairPointsTransform.length > 2) {
                for (i = 0; i < resultsPairPointsTransform.length; i++) {
                    geometry.vertices.push(resultsPairPointsTransform[i].vec1);
                }
                for (i = 0; i < resultsPairPointsTransform.length / 3; i++) {
                    geometry.faces.push(new THREE.Face3(i * 3 + 0, i * 3 + 1, i * 3 + 2));
                }
                var mesh = new THREE.Mesh(geometry, material);
                this.add(mesh);
                render();
            }
        }
    }
}
VIEW3D.Transformation3D = function() {
    this.mBaseX = new MODEL.types.Vector(1, 0, 0);  // transform matrix
    this.mBaseY = new MODEL.types.Vector(0, 1, 0);  // transform matrix
    this.mBaseZ = new MODEL.types.Vector(0, 0, 1);  // transform matrix
    this.mOrigin = new MODEL.types.Vector(0, 0, 0); // origin of local coordinate system
    this.SetData = function (vecX, vecY, vecZ) {
        var vectX = this.Subtract(vecY, vecX);
        var vectY = this.Subtract(vecZ, vecX);
        vectX = this.Normalize(vectX);
        vectY = this.Normalize(vectY);
        var vectZ = this.CrossProduct(vectX, vectY);
        vectZ = this.Normalize(vectZ);
        vectY = this.CrossProduct(vectZ, vectX);
        this.mBaseX = vectX;
        this.mBaseY = vectY;
        this.mBaseZ = vectZ;
        this.mOrigin = vecX;
    }
    this.Subtract = function (point1, point2) {
        var x = point1.x - point2.x;
        var y = point1.y - point2.y;
        var z = point1.z - point2.z;
        return new MODEL.types.Vector(x, y, z);
    }
    this.Normalize = function(vec){
        var num = Math.sqrt(((vec.x * vec.x) + (vec.y * vec.y)) + (vec.z * vec.z));
        var x = vec.x / num;
        var y = vec.y / num;
        var z = vec.z / num;
        return new MODEL.types.Vector(x, y, z);
    }
    this.CrossProduct = function (vector1, vector2) {
        var x = (vector1.y * vector2.z) - (vector1.z * vector2.y);
        var y = (vector1.z * vector2.x) - (vector1.x * vector2.z);
        var z = (vector1.x * vector2.y) - (vector1.y * vector2.x);
        return new MODEL.types.Vector(x, y, z);
    }
    this.TransformToLocal = function(vec) {
        var tmpX = vec.x - this.mOrigin.x;
        var tmpY = vec.y - this.mOrigin.y;
        var tmpZ = vec.z - this.mOrigin.z;
        var x = this.mBaseX.x * tmpX + this.mBaseX.y * tmpY + this.mBaseX.z * tmpZ;
        var y = this.mBaseY.x * tmpX + this.mBaseY.y * tmpY + this.mBaseY.z * tmpZ;
        var z = this.mBaseZ.x * tmpX + this.mBaseZ.y * tmpY + this.mBaseZ.z * tmpZ;
        return new MODEL.types.Vector(x, y, z);
    }
}
VIEW3D.VectorPair = function(vec1, vec2) {
    this.vec1 = new MODEL.types.Vector(vec1.x, vec1.y, vec1.z);
    this.vec2 = new MODEL.types.Vector(vec2.x, vec2.y, vec2.z);
}
VIEW3D.Fill.prototype = Object.create(THREE.Object3D.prototype);