﻿"use strict";
var VIEW3D = VIEW3D || {};
define(["js/view3d/VIEW3D.RealBars", "js/view3d/materials/VIEW3D.materials.Highlight"],
    function() {

        VIEW3D.Highlight = function(renderer, mouse3D, view3D, barCut, barSnap, constraints) { // THREE.WebGLRenderer
            var scene1 = new THREE.Scene();
            var scene2 = new THREE.Scene();
            var selectColor = new THREE.Vector3(0.1, 0.6, 0.1);
            var trimColor = new THREE.Vector3(0.6, 0.1, 0.1);
            var extendColor = new THREE.Vector3(0.1, 0.1, 0.6);
            var addMesh = function(x, y, color, geometry, Object3DConstructor, transform, useSimpleMaterial) {
                var materialHighlight = new VIEW3D.materials.Highlight({
                    color: color,
                    offset: new THREE.Vector3(x, y, 0)
                });

                materialHighlight.defaultAttributeValues["vertexOffset"] = [0, 0, 0];
                materialHighlight.defaultAttributeValues["position"] = [0, 0, 0];
                materialHighlight.defaultAttributeValues["vertexOffset"] = [0, 0, 0];
                if(useSimpleMaterial) {
                    mesh = new Object3DConstructor(geometry, materialHighlight);
                }
                else{
                    mesh = new Object3DConstructor(geometry, [materialHighlight]);
                }
                mesh.frustumCulled = false;
                if (!isEmpty(transform)) {
                    if (mesh instanceof THREE.Vector3) {
                        mesh.position.copy(transform);
                    } else {
                        mesh.matrix = transform;
                        mesh.matrixAutoUpdate = false;
                    }
                }
                scene2.add(mesh);
            };
            var cleareScene = function(scene) {
                while (scene.children.length > 0) {
                    scene.remove(scene.children[0]);
                }
            };
            var addGeometryHightlight = function(geometry, Object3DConstructor, transform, useSimpleMaterial, color) {
                var materialHighlight = new VIEW3D.materials.Highlight({ color: new THREE.Vector3(0.7, 0.7, 0.7) });
                materialHighlight.defaultAttributeValues["vertexOffset"] = [0, 0, 0];
                var mesh = new Object3DConstructor(geometry, materialHighlight);
                mesh.frustumCulled = false;
                if (!isEmpty(transform)) {
                    if (mesh instanceof THREE.Vector3) {
                        mesh.position.copy(transform);
                    } else {
                        mesh.matrix = transform;
                        mesh.matrixAutoUpdate = false;
                    }
                }
                scene1.add(mesh);
                var width = parseInt(window.getComputedStyle(webGlContainer, null).getPropertyValue("width"));
                var height = parseInt(window.getComputedStyle(webGlContainer, null).getPropertyValue("height"));
                var offsetX = 3 / width;
                var offsetY = 3 / height;
                color = color || selectColor;
                addMesh(offsetX, offsetY, color, geometry, Object3DConstructor, transform, useSimpleMaterial);
                addMesh(-offsetX, offsetY, color, geometry, Object3DConstructor, transform, useSimpleMaterial);
                addMesh(offsetX, -offsetY, color, geometry, Object3DConstructor, transform, useSimpleMaterial);
                addMesh(-offsetX, -offsetY, color, geometry, Object3DConstructor, transform, useSimpleMaterial);
            };
            this.render = function(camera) {
                this.updateHighlight();
                if (scene1.children.length > 0) {
                    var gl = renderer.context;
                    gl.colorMask(false, false, false, false);
                    gl.depthMask(false);
                    gl.enable(gl.STENCIL_TEST);
                    gl.stencilOp(gl.REPLACE, gl.REPLACE, gl.REPLACE);
                    gl.stencilFunc(gl.ALWAYS, 1, 0xffffffff);
                    gl.clearStencil(0);
                    renderer.render(scene1, camera);
                    gl.colorMask(true, true, true, true);
                    gl.depthMask(true);
                    gl.stencilOp(gl.REPLACE, gl.REPLACE, gl.REPLACE);
                    gl.stencilFunc(gl.NOTEQUAL, 1, 0xffffffff);
                    gl.disable(gl.DEPTH_TEST);
                    renderer.render(scene2, camera);
                    gl.enable(gl.DEPTH_TEST);
                    gl.disable(gl.STENCIL_TEST);
                }
            };
            // EDIT
            var addEditControl = function(editControl) {
                var ctrlGeom = editControl.mesh.geometry;
                var geom = new THREE.BufferGeometry();
                //geom.addAttribute('position', new THREE);
                geom.addAttribute("vertexOffset", ctrlGeom.attributes.position);
                geom.addAttribute("position", ctrlGeom.attributes.hightlightPosition);
                geom.setIndex(ctrlGeom.getIndex());
                addGeometryHightlight(geom, THREE.Mesh, editControl.mesh.matrix, true);
            };
            var addMoveArrow = function(arrowDir) {
                var arrows = CORE.view3D.moveControl.arrows3d;
                var arrowsIds = CORE.view3D.moveControl
                    .getArrowsIdsForArrow(arrowDir); // pass CORE.view3D.moveControl as parameter instead?
                var arrowsGeom = arrows.getGeometry();
                var geom = new THREE.BufferGeometry();
                geom.addAttribute("position", arrowsGeom.attributes.elementPos);
                geom.addAttribute("vertexOffset", arrowsGeom.attributes.position);
                geom.setIndex(arrowsGeom.getIndex());
                for (var i = 0; i < arrowsIds.length; i++) {
                    var data = arrows.getArrowsData(arrowsIds[i]);
                    geom.addGroup(data.startIndex, data.indicesCount, 0);
                }
                addGeometryHightlight(geom, THREE.Mesh, arrows.position);
            };
            // MISSING ELEMENTS
            var circleGeometry = new THREE.CircleBufferGeometry(30, 20, 0, Math.PI * 2);
            circleGeometry.addAttribute("vertexOffset2D", circleGeometry.attributes.position);
            circleGeometry.removeAttribute("position");
            var circle2Geometry = new THREE.CircleBufferGeometry(34, 20, 0, Math.PI * 2);
            circle2Geometry.addAttribute("vertexOffset2D", circle2Geometry.attributes.position);
            circle2Geometry.removeAttribute("position");
            var addMissingElement = function(element) {
                var vec1, vec2;
                var position = new THREE.Vector3();
                var material = new VIEW3D.materials.Highlight({ color: new THREE.Vector3(0.7, 0.7, 0.7) });
                material.defaultAttributeValues["position"] = [0, 0, 0];
                var mesh = new THREE.Mesh(circleGeometry, [material]);
                var material2 = new VIEW3D.materials.Highlight({ color: selectColor });
                material2.defaultAttributeValues["position"] = [0, 0, 0];
                var mesh2 = new THREE.Mesh(circle2Geometry, [material2]);
                switch (element.type) {
                case MODEL.OBJECT_TYPE.Bar:
                    vec1 = element.node1().position();
                    vec2 = element.node2().position();
                    position = new THREE.Vector3().addVectors(vec2, vec1).multiplyScalar(0.5);
                    break;
                case MODEL.OBJECT_TYPE.Node:
                    position = element.position();
                    break;
                case MODEL.OBJECT_TYPE.Support:
                    position = element.position();
                    break;
                }
                mesh.position.set(position.x, position.y, position.z);
                mesh2.position.set(position.x, position.y, position.z);
                scene1.add(mesh);
                scene2.add(mesh2);
            };
            // NODE: SIMPLE VIEW
            var addPointNode = function(node) {
                var geometry = new THREE.Geometry();
                geometry.vertices.push(HELPERS.toVector3(node.position)
                );
                var material = new THREE.PointsMaterial({
                    size: CORE.options.nodeSize,
                    sizeAttenuation: false
                });
                var point = new THREE.Points(geometry, [material]);
                scene1.add(point);
                var c = new THREE.Color();
                c.setRGB(selectColor.x, selectColor.y, selectColor.z);
                material = new THREE.PointsMaterial({
                    size: CORE.options.nodeSize + 3,
                    color: c,
                    sizeAttenuation: false
                });
                point = new THREE.Points(geometry, [material]);
                scene2.add(point);
            };
            // NODE: REAL VIEW
            var addRealNode = function(node) {
                var realNodes = view3D.nodesObject3D;
                var nodeData = realNodes.nodesData[node.id];
                if (!isEmpty(nodeData) && !isEmpty(nodeData.meshId)) {
                    var meshData = realNodes.getMeshData(nodeData.meshId);
                    if (!isEmpty(meshData)) {
                        var rnGeo = realNodes.geometry;
                        var geom = new THREE.BufferGeometry();
                        geom.addAttribute("position", rnGeo.attributes.position);
                        geom.setIndex(rnGeo.getIndex());
                        geom.addGroup(meshData.startIndex, meshData.indexCount, 0);
                        if (!isEmpty(nodeData.secondaryMeshId)) {
                            meshData = realNodes.getMeshData(nodeData.secondaryMeshId);
                            if (!isEmpty(meshData)) {
                                geom.addGroup(meshData.startIndex, meshData.indexCount, 0);
                            }
                        }
                        addGeometryHightlight(geom, THREE.Mesh);
                    }
                }
            };
            // NODE: MISSING NODE
            var addMissingNode = function(node, position) {
                var material = new VIEW3D.materials.Highlight({ color: new THREE.Vector3(0.7, 0.7, 0.7) });
                material.defaultAttributeValues["position"] = [0, 0, 0];
                var mesh = new THREE.Mesh(circleGeometry, [material]);
                mesh.position.set(position.x, position.y, position.z);
                scene1.add(mesh);
                var material2 = new VIEW3D.materials.Highlight({ color: selectColor });
                material2.defaultAttributeValues["position"] = [0, 0, 0];
                var mesh2 = new THREE.Mesh(circle2Geometry, [material2]);
                mesh2.position.set(position.x, position.y, position.z);
                scene2.add(mesh2);
            };
            // NODE: CONTROL
            var addNode = function(node) {
                var noConnector = false;
                var nodesView = view3D.nodesObject3D;
                for (var i = 0; i < CORE.model.missingElements().length; i++) {
                    if (CORE.model.missingElements()[i].Type === 1) {
                        if (node.id === CORE.model.missingElements()[i].ID) {
                            addMissingElement(node);
                            noConnector = true;
                        }
                    }
                }
                if (!noConnector) {
                    if (VIEW3D.RealNodes && nodesView instanceof VIEW3D.RealNodes) {
                        if (!(node.connectorId() > 0)) {
                            addMissingNode(node, node.position());
                        } else {
                            addRealNode(node);
                        }
                    } else if (VIEW3D.PointNodeCloud && nodesView instanceof VIEW3D.PointNodeCloud) {
                        if (!(node.connectorId() > 0) && node.createsBars().length > 1) {
                            addMissingNode(node, node.position());
                        } else {
                            addPointNode(node);
                        }
                    }
                }
            };
            // BAR: SIMPLE VIEW
            var addLine = function(p1, p2, color) {
                // TODO reuse the buffer
                var geometry = new THREE.Geometry();
                geometry.vertices.push(
                    p1,
                    p2
                );
                addGeometryHightlight(geometry, THREE.LineSegments, null, true, color);
            };
            // BAR: REAL VIEW
            var profiles = {};
            var getProfileGeom = function(modelId) {
                var profileGeom = profiles[modelId];
                if (isEmpty(profileGeom)) {
                    var orginalGeom = VIEW3D.utils.getTemplateGeometry(modelId);
                    if (isEmpty(orginalGeom)) {
                        return null;
                    }
                    profileGeom = new THREE.BufferGeometry();
                    //find edges
                    var edges = [];
                    var pos = orginalGeom.attributes.position;
                    var ind = orginalGeom.index.array;
                    var trCount = ind.length / 3;
                    for (var tr = 0; tr <= trCount; tr++) {
                        var i0 = ind[tr * 3 + 0];
                        var i1 = ind[tr * 3 + 1];
                        var i2 = ind[tr * 3 + 2];
                        var x0 = pos.getX(i0);
                        var x1 = pos.getX(i1);
                        var x2 = pos.getX(i2);
                        if (x0 < 0.001) {
                            if (x1 < 0.001) {
                                edges.push(i0);
                                edges.push(i1);
                            } else if (x2 < 0.001) {
                                edges.push(i2);
                                edges.push(i0);
                            }
                        } else if (x1 < 0.001 && x2 < 0.001) {
                            edges.push(i1);
                            edges.push(i2);
                        }
                    }
                    var edgesCount = edges.length / 2;
                    //var posArray = new Float32Array(edgesCount*6);
                    var prefilePosition = new THREE.BufferAttribute(new Float32Array(edgesCount * 6 * 3), 3);
                    //create triangles
                    for (var ed = 0; ed < edgesCount; ed++) {
                        var s = edges[ed * 2 + 0];
                        var e = edges[ed * 2 + 1];
                        var sy = pos.getY(s);
                        var sz = pos.getZ(s);
                        var ey = pos.getY(e);
                        var ez = pos.getZ(e);
                        prefilePosition.setXYZ(ed * 6 + 0, 0, sy, sz);
                        prefilePosition.setXYZ(ed * 6 + 2, 1, sy, sz);
                        prefilePosition.setXYZ(ed * 6 + 1, 0, ey, ez);
                        prefilePosition.setXYZ(ed * 6 + 4, 1, sy, sz);
                        prefilePosition.setXYZ(ed * 6 + 3, 1, ey, ez);
                        prefilePosition.setXYZ(ed * 6 + 5, 0, ey, ez);
                    }
                    profileGeom.addAttribute("position", prefilePosition);
                    profiles[modelId] = profileGeom;
                }
                return profileGeom;
            };
            var addProfile = function(bar, v1, v2, color) {
                var modelId = constraints.getBarProfileModelId(bar.id, bar.profileId());
                if (modelId === -1) {
                    return false;
                }
                var profileGeom = getProfileGeom(modelId);
                if (isEmpty(profileGeom)) {
                    return false;
                }
                var MemberConstraints = constraints.getMemberConstraints(bar.id);
                var transform = VIEW3D.utils.getMatrix4FromBase(MemberConstraints.orientation);
                var start = v2;
                var end = v1;
                if (bar.node1().position().squaredDistanceTo(v1) < bar.node1().position().squaredDistanceTo(v2)) {
                    start = v1;
                    end = v2;
                }
                var mirror = false;
                var prfileDir = end.clone().sub(start);
                var barDir = bar.node2().position().sub(bar.node1().position());
                if (prfileDir.dot(barDir) < 0) {
                    mirror = true;
                }
                var bl = bar.getLength();
                var brl = MemberConstraints.length;
                var offset = bar.node1().position().distanceTo(start);
                offset = brl * offset / bl;
                var l = v1.distanceTo(v2);
                l = brl * l / bl;
                if (mirror) {
                    l = -l;
                }
                //transform.setPosition(start);
                transform.multiplyMatrices(transform, new THREE.Matrix4().makeTranslation(offset, 0, 0));
                transform.multiplyMatrices(transform, new THREE.Matrix4().makeScale(l, 1, 1));
                addGeometryHightlight(profileGeom, THREE.Mesh, transform, true, color);
                return true;

            };
            // BAR: CONTROL
            var addBar = function(bar) {
                var noProfile = false;
                var barsView = view3D.barsObject3D;
                for (var i = 0; i < CORE.model.missingElements().length; i++) {
                    if (CORE.model.missingElements()[i].Type === 0) {
                        if (bar.id === CORE.model.missingElements()[i].ID) {
                            addMissingElement(bar);
                            noProfile = true;
                        }
                    }
                }
                if (!noProfile) {
                    var drawLine = true;
                    if (barsView instanceof VIEW3D.RealBars) { //re
                        drawLine = !addProfile(bar, bar.node1().position(), bar.node2().position(), selectColor);
                    }
                    if (drawLine) {
                        addLine(bar.node1().position(), bar.node2().position(), selectColor);
                    }
                }
            };
            // LOADS
            var addLoad = function(load) {
                var arrows = view3D.loadsObject3D.arrows;
                var load3D = view3D.loadsObject3D.loads[load.type][load.id];
                var arrowsIds = load3D.getAllArrowsIds();
                var arrowsGeom = arrows.getGeometry();
                var geom = new THREE.BufferGeometry();
                geom.addAttribute("position", arrowsGeom.attributes.elementPos);
                geom.addAttribute("vertexOffset", arrowsGeom.attributes.position);
                geom.setIndex(arrowsGeom.getIndex());
                for (var i = 0; i < arrowsIds.length; i++) {
                    var data = arrows.getArrowsData(arrowsIds[i]);
                    geom.addGroup(data.startIndex, data.indicesCount, 0);
                }
                addGeometryHightlight(geom, THREE.Mesh);
            };

            var addCustomMesh = function (object) {
                if (isEmpty(object)) return;

                var object3d = view3D.stpPartsObject3D.stpPartsData[object.id];
                if (isEmpty(object3d)) return;

                var meshGeom = object3d.mesh.geometry;
                if (isEmpty(meshGeom))
                    throw new Error("mesh.geometry is empty");

                var geom = new THREE.BufferGeometry();
                geom.addAttribute("position", meshGeom.attributes.position);
                geom.setIndex(meshGeom.getIndex());

                var data = { startIndex: 0, indexCount: meshGeom.getIndex().count };
                geom.addGroup(data.startIndex, data.indexCount, 0);
                addGeometryHightlight(geom, THREE.Mesh);
            }
            // BOX
            var addBox = function(box) {
                var box3D = null;
                var list = view3D.boxesObject3D.list;
                for (var i = 0; i < list.length; i++) {
                    if (list[i].id === box.id) {
                        box3D = list[i].element;
                    }
                }
                if (box3D !== null && typeof box3D !== "undefined") {
                    var geometry = box3D.mesh.geometry;
                    var geom = new THREE.BufferGeometry();
                    geom.addAttribute("position", geometry.attributes.position);
                    geom.setIndex(geometry.getIndex());
                    addGeometryHightlight(geom, THREE.Mesh, box3D.mesh.matrix, true);
                }
            };

            // SUPPORT
            var addReinforcement = function(reinforcementResult) {
                var geom = CORE.constructionView3D.reinforcementsObject3D.getGeometry(reinforcementResult.id);
                addGeometryHightlight(geom, THREE.Mesh, null, true);
            };
            // BAR EDITS
            var addBarCut = function(cutData) {
                var barsView = view3D.barsObject3D;
                var drawLine = true;
                if (barsView instanceof VIEW3D.RealBars) {
                    drawLine = !addProfile(cutData.bar, cutData.leftEnd, cutData.rightEnd, trimColor);
                }
                if (drawLine) {
                    addLine(cutData.leftEnd, cutData.rightEnd, trimColor);
                }
            };
            var addBarExtension = function(extendData) {
                var barsView = view3D.barsObject3D;
                for (var i = 0; i < extendData.length; i++) {
                    var data = extendData[i];
                    var drawLine = true;
                    if (barsView instanceof VIEW3D.RealBars) { //re
                        drawLine = !addProfile(data.barToExtend,
                            HELPERS.toVector3(data.node.position()),
                            data.extendPos,
                            extendColor);
                    }
                    if (drawLine) {
                        addLine(HELPERS.toVector3(data.node.position()), data.extendPos, extendColor);
                    }
                }
            };
            var needsUpdate = false;
            // HIGHLIGHT CONTROLER
            this.updateHighlight = function() {
                if (!needsUpdate) {
                    return;
                }
                cleareScene(scene1);
                cleareScene(scene2);
                var cutData = null;
                if (barCut) {
                    cutData = barCut.cutData();
                }
                if (!isEmpty(cutData)) {
                    addBarCut(cutData);
                } else {
                    var extendData = null;
                    if (barSnap) {
                        extendData = barSnap.extendData();
                    }
                    if (!isEmpty(extendData)) {
                        addBarExtension(extendData);
                    }
                    //get object geometry
                    var object = mouse3D.object();
                    if (!isEmpty(object)) {
                        switch (object.type) {
                        case MODEL.OBJECT_TYPE.Node:
                            addNode(object);
                            break;
                        case MODEL.OBJECT_TYPE.Bar:
                            addBar(object);
                            break;
                        case MODEL.OBJECT_TYPE.BarForce:
                        case MODEL.OBJECT_TYPE.BarUniform:
                        case MODEL.OBJECT_TYPE.BarTrapezoidal:
                            addLoad(object);
                            break;
                        case "MoveToolArrow":
                            addMoveArrow(object.dir);
                            break;
                        case "EditControl":
                            addEditControl(object);
                            break;
                        case "CustomMesh":
                            addCustomMesh(object);
                            break;
                        // case MODEL.OBJECT_TYPE.Reinforcement:
                        //     addReinforcement(object);
                        //     break;
                        case MODEL.OBJECT_TYPE.Box:
                            addBox(object);
                            break;
                        }
                    }
                }
                if (!isEmpty(CORE.view3D.moveControl) && CORE.view3D.moveControl.isMoving) {
                    // should be done in VIEW3D.controls.Move.js
                    addMoveArrow(CORE.view3D.moveControl.currentDir);
                }
                needsUpdate = false;
            };
            mouse3D.object.subscribe(function() {
                needsUpdate = true;
                render();
            });
            if (barCut)
                barCut.cutData.subscribe(function() {
                    needsUpdate = true;
                    render();
                });
            if (barSnap)
                barSnap.extendData.subscribe(function() {
                    needsUpdate = true;
                    render();
                });
        };
    });