'use strict';
var VIEW3D = VIEW3D || {};

define(['js/view3d/materials/VIEW3D.materials.InstancedProfile'],function () {
    /*
    if ( renderer.extensions.get( 'ANGLE_instanced_arrays' ) === false ) {
        throw new Error( "Instanced arrays not supported by hardware.");
    }
    */
    VIEW3D.InstancedProfile = function(segmentTemplate, isBackFace) {

        var segmentLength = Math.round(segmentTemplate.boundingBox.max.x*1000)/1000;

        var material = new VIEW3D.materials.InstancedProfile({isBackFace});
        if(isBackFace) material.side = THREE.BackSide;
        var geometry = new THREE.InstancedBufferGeometry();

        geometry.addAttribute( 'position', segmentTemplate.geometry.attributes.position );
        geometry.addAttribute( 'normal', segmentTemplate.geometry.attributes.normal );
        geometry.setIndex( segmentTemplate.geometry.index );

        THREE.Mesh.call(this, geometry, material);
        this.frustumCulled = false;
        this.visible = false;

        this.getInstancesSpace = function(){
            if(isEmpty(this.geometry.attributes.segPos)){
                return 0;
            }
            return this.geometry.attributes.segPos.count;
        }

        this.instancesInUse = 0;
        var instanceMargine = 1000;
        var instanceAttributes = {
            segPos : { type: Float32Array, size: 4 },
            rotation : { type: Float32Array, size: 4 },
            color : { type: Float32Array, size: 3 },
            };

        this.increaseSize = function(instances) {
            if(this.getInstancesSpace() < instances + this.instancesInUse) {  // index buffer too small

                var newInstanceBufferSize = instances + this.instancesInUse + instanceMargine;
                this.geometry.dispose();//release resources
                for (var attr in instanceAttributes) {
                    if (instanceAttributes.hasOwnProperty(attr)) {
                        this.geometry.addAttribute(attr, VIEW3D.utils.increaseAttribute(
                            this.geometry.attributes[attr],
                            newInstanceBufferSize,
                            instanceAttributes[attr].type,
                            instanceAttributes[attr].size,
                            THREE.InstancedBufferAttribute
                        ));
                        this.geometry.attributes[attr].needsUpdate = true;
                    }
                }
            }

            this.instancesInUse += instances;
            this.geometry.maxInstancedCount = this.instancesInUse;
        }

        this.spliceInstances = function(startInstance, instances) {
            for (var attr in instanceAttributes) {
                if (instanceAttributes.hasOwnProperty(attr)) {
                    this.geometry.addAttribute(attr, VIEW3D.utils.bufferLeftShiftRange(
                        this.geometry.attributes[attr],
                        startInstance,
                        instances,
                        instanceAttributes[attr].type,
                        instanceAttributes[attr].size,
                        undefined,
                        this.instancesInUse));
                    this.geometry.attributes[attr].needsUpdate = true;
                }
            }

            this.instancesInUse -= instances;
            this.geometry.maxInstancedCount = this.instancesInUse;
        }



        this.addBar = function(transform, length, color, barData) {
            var instances = Math.ceil(length / segmentLength);
            var startInstance = this.instancesInUse;
            barData.startInstance = startInstance;
            barData.instances = instances;
            this.increaseSize(instances);

            var rotQuat = new THREE.Quaternion().setFromRotationMatrix(transform);
            var pos = new THREE.Vector3().setFromMatrixPosition(transform);
            var dir = new THREE.Vector3(1, 0, 0).applyQuaternion(rotQuat);

            var segPos = geometry.attributes.segPos;
            var rotation = geometry.attributes.rotation;
            var col = geometry.attributes.color;

            for (var i = 0; i < instances; i++) {
                var index = startInstance + i;
                var p = dir.clone();
                p.multiplyScalar(segmentLength * i);
                p.add(pos);
                segPos.setXYZW(index, p.x, p.y, p.z, length - segmentLength * i);
                rotation.setXYZW(index, rotQuat.x, rotQuat.y, rotQuat.z, rotQuat.w);
                col.setXYZ(index, color.x, color.y, color.z);
            }
            segPos.needsUpdate = true;
            rotation.needsUpdate = true;
            col.needsUpdate = true;

            this.visible = this.instancesInUse > 0;
        }

        this.removeBar = function(barData, allBarsData, modelId) {
            this.spliceInstances(barData.startInstance, barData.instances)
            for (var barId in allBarsData) {
                var bd = allBarsData[barId];
                if(modelId === bd.modelId && bd.startInstance > barData.startInstance) {
                    bd.startInstance -= barData.instances;
                }
            }
            this.visible = this.instancesInUse > 0;
        }

        this.setColor = function (barData, color) {
            var col = geometry.attributes.color;
            for (var i = barData.startInstance; i < barData.startInstance + barData.instances; i++) {
                col.setXYZ(i, color.x, color.y, color.z);
            }
            col.needsUpdate = true;
        }

    }
    VIEW3D.InstancedProfile.prototype = Object.create(THREE.Mesh.prototype);

});
