﻿'use strict';
var VIEW3D = VIEW3D || {}

// class PointNode 
// binds to a single MODEL.Bar and 
// modifies line vertices (positions and color) from LineBars that ware assigned to it
VIEW3D.LineBar = function(lineBars, barModel, originalColor, selectionColor) {
    this.lineBars = lineBars;
    this.barModel = barModel;

    var collider = new VIEW3D.collision.SegmentCollider(
        HELPERS.toVector3(barModel.node1().position()),
        HELPERS.toVector3(barModel.node2().position())
        );
    collider.view3D = this;
    collider.model = barModel;

    this.getCollider = function () {
        return collider;
    }

    this.select = function(multiSelection) {
        console.log('VIEW3D.LineBar: onSelect');
        this.barModel.selected(!this.barModel.selected() || !multiSelection);
        return true;
    };
    this.onBarSelectedChanged = function(newValue) {
        console.log('VIEW3D.LineBar: onBarSelectedChanged, new value: ' + newValue);
        try {
            var color = newValue ? selectionColor : originalColor;
            this.lineBars.geometry.attributes.color.setXYZ(this.startIndex, color.r, color.g, color.b);
            this.lineBars.geometry.attributes.color.setXYZ(this.startIndex+1, color.r, color.g, color.b);
            this.lineBars.geometry.attributes.color.needsUpdate = true;
            render();
        } catch (err) {
            console.error("Node selection failed! Reason: " + err);
        }
    };
    this.onNodePositionChanged = function() {
        this.changeVertexPosition();
    };

    this.changeVertexPosition = function() {
        var p1 = HELPERS.toVector3(barModel.node1().position());
        var p2 = HELPERS.toVector3(barModel.node2().position());
        collider.set(p1, p2);
        try {
            this.lineBars.geometry.attributes.position.setXYZ(this.startIndex, p1.x, p1.y, p1.z);
            this.lineBars.geometry.attributes.position.setXYZ(this.startIndex+1, p2.x, p2.y, p2.z);
            this.lineBars.geometry.attributes.position.needsUpdate = true;
            render();
        } catch (err) {
            console.error("Node position change failed! Reason: " + err);
        }
    };
    this.onNode1Changed = function(newValue) {
        this.node1PosSub.dispose();
        if (!isEmpty(newValue)) { //can happen if we are removing this bar
            this.changeVertexPosition();
            this.node1PosSub = HELPERS.subscribePropertyChange(this, this.barModel.node1().position, "bar.node1.position", this.onNodePositionChanged);
        }
    };
    this.onNode2Changed = function(newValue) {
        this.node2PosSub.dispose();
        if (!isEmpty(newValue)) { //can happen if we are removing this bar
            this.changeVertexPosition();
            this.node2PosSub = HELPERS.subscribePropertyChange(this, this.barModel.node2().position, "bar.node2.position", this.onNodePositionChanged);
        }
    };
    this.dispose = function() {
        this.selectionSub.dispose();
        this.node1PosSub.dispose();
        this.node2PosSub.dispose();
        this.node1Sub.dispose();
        this.node2Sub.dispose();
    }
    this.selectionSub = HELPERS.subscribePropertyChange(this, this.barModel.selected, "bar.selected", this.onBarSelectedChanged);
    this.node1PosSub = HELPERS.subscribePropertyChange(this, this.barModel.node1().position, "bar.node1.position", this.onNodePositionChanged);
    this.node2PosSub = HELPERS.subscribePropertyChange(this, this.barModel.node2().position, "bar.node2.position", this.onNodePositionChanged);
    this.node1Sub = HELPERS.subscribePropertyChange(this, this.barModel.node1, "bar.node1", this.onNode1Changed);
    this.node2Sub = HELPERS.subscribePropertyChange(this, this.barModel.node2, "bar.node2", this.onNode2Changed);
}