﻿'use strict';
var VIEW3D = VIEW3D || {}

define(['js/view3d/VIEW3D.LineBar', 'js/view3d/utils/VIEW3D.utils.ResizableGeometry'], function () {
    // class LineBars extends THREE.Line
    // binnds to an array of MODEL.Bar
    // and vizualizes it as a lines
    VIEW3D.LineBars = function (bars, isResultsWindow) {
        //private members
        var lineBars = {}; //dictionary of VIEW3D.LineBar
        //static members
        this.originalColor = !isResultsWindow ? new THREE.Color(0.4, 0.4, 0.4) : new THREE.Color(0.4, 0.4, 0.4);
        this.selectionColor = new THREE.Color(1, 0, 0);

        //init
        var collider = new VIEW3D.collision.CollidersContainer();
        this.getCollider = function () {
            return collider;
        };

        this.init = function () {
            for (var i = 0; i < bars().length; i++) {
                var bar = bars()[i];
                this.onBarAdded(bar, i);
            }
        };

        var attributes = {
            position: {
                type: Float32Array,
                size: 3
            },
            color: {
                type: Float32Array,
                size: 3
            }
        };

        var geometry = new VIEW3D.utils.ResizableNonIndexedGeometry(attributes, 1000);
        var material = new THREE.LineBasicMaterial({
            vertexColors: THREE.VertexColors,
            side: THREE.DoubleSide,
            transparent: true
        });

        THREE.LineSegments.call(this, geometry, material);
        this.visible = false;
        this.frustumCulled = false;

        this.onBarAdded = function (barModelObject) {
            var startIndex = this.geometry.verticesInUse;
            this.geometry.increaseSize(2);
            var n1 = barModelObject.node1().position();
            var n2 = barModelObject.node2().position();
            var pos1 = new THREE.Vector3(n1.x, n1.y, n1.z);
            var pos2 = new THREE.Vector3(n2.x, n2.y, n2.z);
            var color = barModelObject.selected() ? this.selectionColor : this.originalColor;
            this.geometry.attributes.position.setXYZ(startIndex, pos1.x, pos1.y, pos1.z);
            this.geometry.attributes.position.setXYZ(startIndex+1, pos2.x, pos2.y, pos2.z);
            this.geometry.attributes.color.setXYZ(startIndex, color.r, color.g, color.b);
            this.geometry.attributes.color.setXYZ(startIndex+1, color.r, color.g, color.b);

            var lineBar = new VIEW3D.LineBar(this, barModelObject, this.originalColor, this.selectionColor);
            lineBar.startIndex = startIndex;
            lineBars[barModelObject.id] = lineBar;
            collider.addCollider(lineBar.getCollider());
            this.visible = true;
            render();
        }

        this.onBarRemoved = function (bar, index) {
            var lineBar = lineBars[bar.id];
            this.geometry.splice(lineBar.startIndex, 2);

            collider.removeCollider(lineBar.getCollider());
            delete lineBars[bar.id];

            for (var id in lineBars) {
                var lb = lineBars[id]
                if(lb.startIndex > lineBar.startIndex) {
                    lb.startIndex -=2;
                }
            }

            if (this.geometry.verticesInUse === 0) {
                this.visible = false;
            }
            render();
        }

        this.dispose = function () {
            for (var i = 0; i < lineBars.length; i++) {
                lineBars[i].dispose();
            }
            this.barsListSub.dispose();
        }
        this.barsListSub = HELPERS.subscribeArrayChange(this, bars, "MODEL.bars", this.onBarAdded, this.onBarRemoved);
        this.init();
    }
    VIEW3D.LineBars.prototype = Object.create(THREE.LineSegments.prototype);
});