﻿'use strict';
var VIEW3D = VIEW3D || {}

define(['js/view3d/materials/VIEW3D.materials.LineLabel'], function () {

    VIEW3D.LineLabel = function (pos1, pos2, aspectRatio, bitmapFont, color, customText) {
        this.observePoints = ko.isObservable(pos1);
        this.pos1 = pos1;
        this.pos2 = pos2;
        this.material = new VIEW3D.materials.LineLabel(aspectRatio, bitmapFont.texture, color);
        bitmapFont.texture.needsUpdate = true;
        this.getLengthText = function () {
            if (customText) return customText;

            var p1 = HELPERS.toVector3(pos1);
            var p2 = HELPERS.toVector3(pos2);
            var length = p1.sub(p2).length();
            return length.toFixed(2);
        }
        this.geometry = bitmapFont.createText();
        this.geometry.addAttribute('begin', new THREE.BufferAttribute(undefined, 3));
        this.geometry.addAttribute('end', new THREE.BufferAttribute(undefined, 3));
        this.updateText = function () {
            var text = this.getLengthText();
            this.geometry.update({
                text: text,
                align: 'center',
                font: bitmapFont.font
            });
            var p1 = HELPERS.toVector3(pos1);
            var p2 = HELPERS.toVector3(pos2);
            var positions = this.geometry.attributes.position.array;
            var vertices = positions.length / 2;
            var halfW = this.geometry.layout._width / 2;
            var halfH = this.geometry.layout._height / 2;
            var begins = new Float32Array(vertices * 3);
            var ends = new Float32Array(vertices * 3);
            for (var i = 0; i < vertices; i++) {
                begins[i * 3 + 0] = p1.x;
                begins[i * 3 + 1] = p1.y;
                begins[i * 3 + 2] = p1.z;
                ends[i * 3 + 0] = p2.x;
                ends[i * 3 + 1] = p2.y;
                ends[i * 3 + 2] = p2.z;
                positions[i * 2 + 0] = (positions[i * 2 + 0] - halfW) * 0.0015;
                positions[i * 2 + 1] = (positions[i * 2 + 1] + halfH) * -0.0015;
            }
            this.geometry.attributes.begin.array = begins;
            this.geometry.attributes.begin.needsUpdate = true;
            this.geometry.attributes.end.array = ends;
            this.geometry.attributes.end.needsUpdate = true;
            this.geometry.boundingSphere = new THREE.Sphere();
            this.geometry.boundingSphere.setFromPoints([p1, p2]); //label is visible if line is visible
        }
        this.updateText();
        THREE.Mesh.call(this, this.geometry, this.material);
        if (this.observePoints) {
            this.pos1Sub = HELPERS.subscribePropertyChange(this, this.pos1, "pos1", this.updateText);
            this.pos2Sub = HELPERS.subscribePropertyChange(this, this.pos2, "pos2", this.updateText);
        }
        this.dispose = function () {
            if (this.observePoints) {
                this.pos1Sub.dispose();
                this.pos2Sub.dispose();
            }
            this.geometry.dispose();
            this.material.dispose();
        }
    };
    VIEW3D.LineLabel.prototype = Object.create(THREE.Mesh.prototype);
});