'use strict';
var VIEW3D = VIEW3D || {}
define(['js/view3d/VIEW3D.Measurements', 'js/view3d/materials/VIEW3D.materials.LineMeasurements', 'js/view3d/VIEW3D.Inputs', 'js/view3d/utils/VIEW3D.utils.ResizableGeometry'], function () {

    //defined in VIEW3D.Measurements
    var coneGeometry = VIEW3D.MeasurementConeGeometry;


    VIEW3D.LineMeasurements = function (measurementsProvider, modelChangeEvent, aspectRatio, camera, config, mode) {
        var lines = null;
        var heads = null;

        //lines
        var lineAttributes = {
            position: {
                type: Float32Array,
                size: 3
            },
            offset: {
                type: Float32Array,
                size: 1
            }
        };

        var linesGeometry = new VIEW3D.utils.ResizableNonIndexedGeometry(lineAttributes);

        var headAttributes = {
            position: {
                type: Float32Array,
                size: 3
            },
            vertexOffset: {
                type: Float32Array,
                size: 3
            },
            offset: {
                type: Float32Array,
                size: 1
            }
        };

        var headsGeometry = new VIEW3D.utils.ResizableGeometry(headAttributes);


        var config = config || {
                measurementOffset0: 30,
                measurementOffset1: 80,
                measurementOffset2: 110,
                textOffset: -50,
                color: VIEW3D.editMeasurementColor
        }

        var createLinesGeometry = function (measurements) {
            var vertices = 6 * measurements.length;
            linesGeometry.changeSize(vertices);
            var positions = linesGeometry.getAttribute('position');
            var offsets = linesGeometry.getAttribute('offset');

            var d0 = config.measurementOffset0;
            var d1 = config.measurementOffset1;
            var d2 = config.measurementOffset2;

            var drawIndex = 0;
            for (var i = 0; i < measurements.length ; i++) {
                var measurement = measurements[i];
                if(measurement.length() < 0.001) {
                    continue;
                }
                var p1 = measurement.start();
                var p2 = measurement.getEnd();

                var si = drawIndex * 6; //statr index
                positions.setXYZ(si + 0, p1.x, p1.y, p1.z);
                positions.setXYZ(si + 1, p2.x, p2.y, p2.z);
                positions.setXYZ(si + 2, p1.x, p1.y, p1.z);
                positions.setXYZ(si + 3, p1.x, p1.y, p1.z);
                positions.setXYZ(si + 4, p2.x, p2.y, p2.z);
                positions.setXYZ(si + 5, p2.x, p2.y, p2.z);

                offsets.setX(si + 0, d1);
                offsets.setX(si + 1, d1);
                offsets.setX(si + 2, d0);
                offsets.setX(si + 3, d2);
                offsets.setX(si + 4, d0);
                offsets.setX(si + 5, d2);
                drawIndex++;
            }
            linesGeometry.setDrawRange(0, drawIndex * 6);
            return linesGeometry;
        }

        var createHeadsGeometry = function(measurements) {
            var verticesInCone = coneGeometry.attributes.position.array.length / 3;
            var indicesInCone = coneGeometry.index.array.length;
            var verticesCount = 2 * measurements.length * verticesInCone;
            var indicesCount = 2 * measurements.length * indicesInCone;
            headsGeometry.changeSize(indicesCount, verticesCount);


            var vertexOffset = headsGeometry.getAttribute('vertexOffset');
            var position = headsGeometry.getAttribute('position');
            var offsets = headsGeometry.getAttribute('offset');
            var primitivesCount = 0;
            var drawIndex = 0;
            for (var i = 0; i < measurements.length; i++) {
                var mesurement = measurements[i];
                if(mesurement.length() < 0.001) {
                    continue;

                }
                primitivesCount += indicesInCone * 2;
                var p1 = mesurement.start();
                var p2 = mesurement.getEnd();
                var mDir = mesurement.dir();

                var d1 = HELPERS.toVector3( mDir );
                var d2 = d1.clone().multiplyScalar(-1);

                var base1 = VIEW3D.utils.getBaseFromDir(d1);
                var transform1 = VIEW3D.utils.getRotationMatrix4FromBase(base1);
                //transform1.setPosition(p1);

                var base2 = VIEW3D.utils.getBaseFromDir(d2);
                var transform2 = VIEW3D.utils.getRotationMatrix4FromBase(base2);
                //transform2.setPosition(p2);

                var srcPositions = coneGeometry.attributes.position;
                for(var vi = 0; vi < verticesInCone; vi++) {
                    var destIndex = drawIndex*verticesInCone*2 + vi;
                    var pp1 = new THREE.Vector3(srcPositions.getX(vi), srcPositions.getY(vi), srcPositions.getZ(vi));
                    var pp2 = pp1.clone();
                    pp1.applyMatrix4(transform1);
                    position.setXYZ(destIndex, p1.x, p1.y, p1.z);
                    vertexOffset.setXYZ(destIndex, pp1.x, pp1.y, pp1.z);
                    offsets.setX(destIndex, config.measurementOffset1);

                    destIndex = drawIndex*verticesInCone*2 + verticesInCone + vi;
                    pp2.applyMatrix4(transform2);
                    position.setXYZ(destIndex, p2.x, p2.y, p2.z);
                    vertexOffset.setXYZ(destIndex, pp2.x, pp2.y, pp2.z);
                    offsets.setX(destIndex, config.measurementOffset1);
                }

                var srcIndices = coneGeometry.index.array;
                var destIndices = headsGeometry.index.array;
                var si1 = drawIndex*verticesInCone*2;
                var si2 = drawIndex*verticesInCone*2+verticesInCone;
                for(var ii = 0; ii < indicesInCone; ii++) {
                    var destIndex = drawIndex*indicesInCone*2 + ii;
                    destIndices[destIndex] = srcIndices[ii] + si1;
                    destIndex += indicesInCone;
                    destIndices[destIndex] = srcIndices[ii] + si2;
                }

                drawIndex++;
            }
            headsGeometry.setDrawRange(0, primitivesCount);
            return headsGeometry;
        }

        var inputs = new VIEW3D.LineMeasurementInputs(camera, CORE.screenSize, mode, config.textAnchor, config.textOffset);
        var createInputs = function (measurements, measDir) {
            inputs.measDir = measDir;
            for (var i = 0; i < measurements.length ; i++) {
                inputs.add(measurements[i]);
            }
        }

        this.direction = new THREE.Vector3(0,0,1);
        this.setDirection = function (dir) {
            this.direction.copy(dir);
            this.onModelChange();
        }

        this.lineDirection = new THREE.Vector3(1,0,0);
        this.setLineDirection = function (measurements) {
            this.lineDirection.set(1,0,0);
            for (var i = 0; i < measurements.length; i++) {
                var mesurement = measurements[i];
                if (mesurement.length() > 0) {
                    this.lineDirection.copy(mesurement.dir())
                }
            }
        }

        this.onModelChange = function () {
            if (lines !== null) {
                this.remove(lines);
                //TODO reuse gl buffers
                lines = null;
                inputs.clearAll();
                this.remove(heads);
                heads = null;
            }

            //var dir = new THREE.Vector3(1,0,0);
            //var dir2 = new THREE.Vector3(0,0,1);

            var allMeasurements = measurementsProvider();

            this.setLineDirection(allMeasurements);

            if (allMeasurements.length !== 0) {
                lines = new THREE.LineSegments(
                    createLinesGeometry(allMeasurements),
                    new VIEW3D.materials.LineMeasurements(config.color, CORE.screenSize, this.lineDirection, this.direction ));
                lines.renderOrder = 1000;
                lines.frustumCulled = false;
                this.add(lines);

                createInputs(allMeasurements, this.direction );

                heads = new THREE.Mesh(
                    createHeadsGeometry(allMeasurements),
                    new VIEW3D.materials.LineMeasurements(config.color, CORE.screenSize, this.lineDirection, this.direction ));
                heads.frustumCulled = false;
                heads.renderOrder = 1000;
                this.add(heads);

                render();
            }
        }

//        this.visible = true;
        this.show = function (visible) {
            inputs.show(visible);
        }

        //init
        THREE.Object3D.call(this);
        modelChangeEvent.subscribe(this.onModelChange.bind(this));
        this.onModelChange();

    }

    VIEW3D.LineMeasurements.prototype = Object.create(THREE.Object3D.prototype);
});
