﻿'use strict';
var VIEW3D = VIEW3D || {}

define(['js/view3d/VIEW3D.BarForce', 'js/view3d/VIEW3D.BarUniform', 'js/view3d/VIEW3D.BarTrapezoidal', 'js/view3d/VIEW3D.Arrows', 'js/view3d/VIEW3D.LoadFills' , 'js/view3d/VIEW3D.LoadInputs'], function () {
    // class VIEW3D.Loads extends THREE.Object3D
    // all kinds of loads will add a new THREE.Object3D
    // to this object when they are assigned to model element
    VIEW3D.Loads = function (loadsModel) {
        THREE.Object3D.call(this);
        this.loads = {}; //dictionary by load id
        this.arrows = new VIEW3D.Arrows(camera);
        this.inputs = new VIEW3D.LoadInputs(camera);
        this.fills = new VIEW3D.LoadFills(camera);

        this.add(this.arrows);
        //this.add(this.inputs);
        this.add(this.fills);

        var collider = new VIEW3D.collision.CollidersContainer();

        this.getCollider = function () {
            return collider;
        }

        var onLoadAdded = function (load) {
            var load3D;
            switch (load.type) {
                case MODEL.OBJECT_TYPE.NodeForce:
                    load3D = new VIEW3D.NodeForce(load, this, this.arrows);
                    break;
                case MODEL.OBJECT_TYPE.BarForce:
                    load3D = new VIEW3D.BarForce(load, this.arrows, this.inputs);
                    break;
                case MODEL.OBJECT_TYPE.BarUniform:
                    load3D = new VIEW3D.BarUniform(load, this.arrows, this.fills);
                    break;
                case MODEL.OBJECT_TYPE.BarTrapezoidal:
                    load3D = new VIEW3D.BarTrapezoidal(load, this.arrows, this.fills, this.inputs);
                    break;
                default:
                    throw new Error("unknown Load type");
                    break;
            }
            collider.addCollider(load3D.getCollider());
            this.loads[load.type] = isEmpty(this.loads[load.type]) ? {} : this.loads[load.type];
            this.loads[load.type][load.id] = load3D;
        }
        var onLoadRemoved = function (load) {
            this.loads[load.type][load.id].clean();
            delete this.loads[load.type][load.id];
        }
        
        HELPERS.subscribeArrayChange(this, loadsModel, "MODEL.loads", onLoadAdded, onLoadRemoved);
        //Init
        for (var i = 0; i < loadsModel().length; i++) {
            onLoadAdded.call(this, loadsModel()[i], i);
        }
    }
    VIEW3D.Loads.prototype = Object.create(THREE.Object3D.prototype);
});