﻿'use strict';
var VIEW3D = VIEW3D || {}

define(['js/view3d/materials/VIEW3D.materials.Measurement', 'js/view3d/VIEW3D.LineLabel'], function () {

    VIEW3D.Measurement = function (pos1, pos2, aspectRatio) {
        this.pos1 = pos1;
        this.pos2 = pos2;
        this.geometry = new THREE.BufferGeometry();
        this.material = new VIEW3D.materials.Measurement(aspectRatio);
        var vertices = 6;
        var positions = new Float32Array(vertices * 3);
        var offsets = new Float32Array(vertices * 2);
        var ends = new Float32Array(vertices * 3);
        this.geometry.addAttribute('position', new THREE.BufferAttribute(positions, 3));
        this.geometry.addAttribute('end', new THREE.BufferAttribute(ends, 3));
        this.updateNodesPos = function () {
            var p1 = this.pos1();
            var p2 = this.pos2();
            positions[0 * 3 + 0] = p1.x;
            positions[0 * 3 + 1] = p1.y;
            positions[0 * 3 + 2] = p1.z;
            ends[0 * 3 + 0] = p2.x;
            ends[0 * 3 + 1] = p2.y;
            ends[0 * 3 + 2] = p2.z;
            positions[1 * 3 + 0] = p2.x;
            positions[1 * 3 + 1] = p2.y;
            positions[1 * 3 + 2] = p2.z;
            ends[1 * 3 + 0] = p1.x;
            ends[1 * 3 + 1] = p1.y;
            ends[1 * 3 + 2] = p1.z;
            positions[2 * 3 + 0] = p1.x;
            positions[2 * 3 + 1] = p1.y;
            positions[2 * 3 + 2] = p1.z;
            ends[2 * 3 + 0] = p2.x;
            ends[2 * 3 + 1] = p2.y;
            ends[2 * 3 + 2] = p2.z;
            positions[3 * 3 + 0] = p1.x;
            positions[3 * 3 + 1] = p1.y;
            positions[3 * 3 + 2] = p1.z;
            ends[3 * 3 + 0] = p2.x;
            ends[3 * 3 + 1] = p2.y;
            ends[3 * 3 + 2] = p2.z;
            positions[4 * 3 + 0] = p2.x;
            positions[4 * 3 + 1] = p2.y;
            positions[4 * 3 + 2] = p2.z;
            ends[4 * 3 + 0] = p1.x;
            ends[4 * 3 + 1] = p1.y;
            ends[4 * 3 + 2] = p1.z;
            positions[5 * 3 + 0] = p2.x;
            positions[5 * 3 + 1] = p2.y;
            positions[5 * 3 + 2] = p2.z;
            ends[5 * 3 + 0] = p1.x;
            ends[5 * 3 + 1] = p1.y;
            ends[5 * 3 + 2] = p1.z;
            this.geometry.attributes.position.needsUpdate = true;
            this.geometry.attributes.end.needsUpdate = true;
        }
        this.updateNodesPos();
        //offsets are static
        offsets[0 * 2 + 0] = 0.0;
        offsets[0 * 2 + 1] = 0.04;
        offsets[1 * 2 + 0] = 0.0;
        offsets[1 * 2 + 1] = 0.04;
        offsets[2 * 2 + 0] = 0.0;
        offsets[2 * 2 + 1] = 0.0;
        offsets[3 * 2 + 0] = 0.0;
        offsets[3 * 2 + 1] = 0.07;
        offsets[4 * 2 + 0] = 0.0;
        offsets[4 * 2 + 1] = 0.0;
        offsets[5 * 2 + 0] = 0.0;
        offsets[5 * 2 + 1] = 0.07;
        this.geometry.addAttribute('offset', new THREE.BufferAttribute(offsets, 2));
        this.pos1Sub = HELPERS.subscribePropertyChange(this, this.pos1, "pos1", this.updateNodesPos);
        this.pos2Sub = HELPERS.subscribePropertyChange(this, this.pos2, "pos2", this.updateNodesPos);
        this.dispose = function () {
            this.pos1Sub.dispose();
            this.pos2Sub.dispose();
            this.geometry.dispose();
            this.material.dispose();
        }
        THREE.LineSegments.call(this, this.geometry, this.material);
        this.frustumCulled = false;
        this.label = new VIEW3D.LineLabel(this.pos1, this.pos2, CORE.aspectRatio, CORE.bitmapFont, VIEW3D.measurementColor);
        this.add(this.label);
    };
    VIEW3D.Measurement.prototype = Object.create(THREE.LineSegments.prototype);
});