﻿'use strict';
var VIEW3D = VIEW3D || {}

define(['js/view3d/materials/VIEW3D.materials.Measurements',
        'js/view3d/VIEW3D.Arrowheads', 'js/view3d/VIEW3D.Inputs', 'js/view3d/utils/VIEW3D.utils.ResizableGeometry'], function () {

    var coneProps = coneProps || {radius : 7, height: 30, radialSegments: 12}
    //var coneProps = coneProps || {radius : 0.010, height: 0.040, radialSegments: 12}
    VIEW3D.MeasurementConeGeometry = new THREE.ConeBufferGeometry( coneProps.radius, coneProps.height, coneProps.radialSegments );
    var coneGeometry = VIEW3D.MeasurementConeGeometry
    coneGeometry.applyMatrix(
        new THREE.Matrix4().makeRotationZ(Math.PI/2).multiply(
        new THREE.Matrix4().makeTranslation(0,-coneProps.height/2,0)
        )
    );


    VIEW3D.Measurements = function (measurementsProvider, modelChangeEvent, aspectRatio, camera, config, mode) {

        var self = this;
        var lines = null;
        var heads = null;
        var allMeasurements = [];
        var data = [];

        //lines
        var lineAttributes = {
            position: {
                type: Float32Array,
                size: 3
            },
            offset: {
                type: Float32Array,
                size: 1
            },
            dir1: {
                type: Float32Array,
                size: 3
            },
            dir2: {
                type: Float32Array,
                size: 3
            }
        };

        var linesGeometry = new VIEW3D.utils.ResizableNonIndexedGeometry(lineAttributes);

        var headAttributes = {
            position: {
                type: Float32Array,
                size: 3
            },
            elementPos: {
                type: Float32Array,
                size: 3
            },
            offset: {
                type: Float32Array,
                size: 1
            },
            dir1: {
                type: Float32Array,
                size: 3
            },
            dir2: {
                type: Float32Array,
                size: 3
            }
        };

        var headsGeometry = new VIEW3D.utils.ResizableGeometry(headAttributes);


        var config = config || {
                measurementOffset0: 0.20,
                measurementOffset1: 0.27,
                measurementOffset2: 0.30,
                textAnchor: 0.27,
                textOffset: -18,
                color: VIEW3D.measurementColor
            }
        var inputs = new VIEW3D.MeasurementInputs(camera, CORE.screenSize, mode, config.textAnchor, config.textOffset );

        this.setFocusOnFirstInput = function() {
            inputs.setFocusOnFirstInput();
        }

        var createLinesGeometry = function (measurements) {
            var vertices = 6 * measurements.length;
            linesGeometry.changeSize(vertices);
            var positions = linesGeometry.getAttribute('position');
            var offsets = linesGeometry.getAttribute('offset');
            var dirs1 = linesGeometry.getAttribute('dir1');
            var dirs2 = linesGeometry.getAttribute('dir2');


            var d0 = config.measurementOffset0;
            var d2 = config.measurementOffset2;
            var d1 = config.measurementOffset1;

            var drawIndex = 0;
            for (var i = 0; i < measurements.length ; i++) {
                var measurement = measurements[i];
                if(measurement.length() < 0.001) {
                    continue;
                }
                var p1 = measurement.start();
                var p2 = measurement.getEnd();
                var dir1 = new THREE.Vector3(0, 0, -1);
                var dir2 = new THREE.Vector3(0, -1, 0);
                var dx = Math.abs(p2.x - p1.x);
                var dy = Math.abs(p2.y - p1.y) * 0.999;
                var dz = Math.abs(p2.z - p1.z) * 0.999;
                if (dy > dx && dy > dz) {
                    dir1.set(0, 0, -1);
                    dir2.set(-1, 0, 0);
                } else if (dz > dx && dz > dy) {
                    dir1.set(0, -1, 0);
                    dir2.set(-1, 0, 0);
                }

                var si = drawIndex * 6; //statr index
                positions.setXYZ(si + 0, p1.x, p1.y, p1.z);
                positions.setXYZ(si + 1, p2.x, p2.y, p2.z);
                positions.setXYZ(si + 2, p1.x, p1.y, p1.z);
                positions.setXYZ(si + 3, p1.x, p1.y, p1.z);
                positions.setXYZ(si + 4, p2.x, p2.y, p2.z);
                positions.setXYZ(si + 5, p2.x, p2.y, p2.z);

                offsets.setX(si + 0, d1);
                offsets.setX(si + 1, d1);
                offsets.setX(si + 2, d0);
                offsets.setX(si + 3, d2);
                offsets.setX(si + 4, d0);
                offsets.setX(si + 5, d2);

                for (var j = 0; j < 14; j++) {
                    dirs1.setXYZ(si + j, dir1.x, dir1.y, dir1.z);
                    dirs2.setXYZ(si + j, dir2.x, dir2.y, dir2.z);
                }
                drawIndex++;
            }
            linesGeometry.setDrawRange(0, drawIndex * 6);
            return linesGeometry;
        }

        var createInputs = function (measurements) {
            for (var i = 0; i < measurements.length ; i++) {
                inputs.add(measurements[i]);
            }
        }

        var createHeadsGeometry = function(measurements) {
            var verticesInCone = coneGeometry.attributes.position.array.length / 3;
            var indicesInCone = coneGeometry.index.array.length;
            var verticesCount = 2 * measurements.length * verticesInCone;
            var indicesCount = 2 * measurements.length * indicesInCone;
            headsGeometry.changeSize(indicesCount, verticesCount);
            /*
            var geom = new THREE.BufferGeometry();
            geom.addAttribute('position', new THREE.BufferAttribute(new Float32Array(verticesCount * 3), 3));
            geom.addAttribute('elementPos', new THREE.BufferAttribute(new Float32Array(verticesCount * 3), 3));
            geom.addAttribute('offset', new THREE.BufferAttribute(new Float32Array(verticesCount * 1), 1));
            geom.addAttribute('dir1', new THREE.BufferAttribute(new Float32Array(verticesCount * 3), 3));
            geom.addAttribute('dir2', new THREE.BufferAttribute(new Float32Array(verticesCount * 3), 3));
            var TypeArray = indicesCount > 65535 ? Uint32Array : Uint16Array;
            var indices = new TypeArray( indicesCount );
            geom.setIndex( new THREE.BufferAttribute( indices, 1 ));
            */

            var positions = headsGeometry.getAttribute('position');
            var elementPos = headsGeometry.getAttribute('elementPos');
            var offsets = headsGeometry.getAttribute('offset');
            var dirs1 = headsGeometry.getAttribute('dir1');
            var dirs2 = headsGeometry.getAttribute('dir2');
            var primitivesCount = 0;
            var drawIndex = 0;
            for (var i = 0; i < measurements.length; i++) {
                var mesurement = measurements[i];
                if(mesurement.length() < 0.001) {
                    continue;

                }
                primitivesCount += indicesInCone * 2;
                var p1 = mesurement.start();
                var p2 = mesurement.getEnd();
                var mDir = mesurement.dir();
                var dir1 = new THREE.Vector3(0, 0, -1);
                var dir2 = new THREE.Vector3(0, -1, 0);
                var dx = Math.abs(p2.x - p1.x);
                var dy = Math.abs(p2.y - p1.y) * 0.999;
                var dz = Math.abs(p2.z - p1.z) * 0.999;
                if (dy > dx && dy > dz) {
                    dir1.set(0, 0, -1);
                    dir2.set(-1, 0, 0);
                } else if (dz > dx && dz > dy) {
                    dir1.set(0, -1, 0);
                    dir2.set(-1, 0, 0);
                }

                var d1 = HELPERS.toVector3( mDir );
                var d2 = d1.clone().multiplyScalar(-1);

                var base1 = VIEW3D.utils.getBaseFromDir(d1);
                var transform1 = VIEW3D.utils.getRotationMatrix4FromBase(base1);
                //transform1.setPosition(p1);

                var base2 = VIEW3D.utils.getBaseFromDir(d2);
                var transform2 = VIEW3D.utils.getRotationMatrix4FromBase(base2);
                //transform2.setPosition(p2);

                var srcPositions = coneGeometry.attributes.position;
                for(var vi = 0; vi < verticesInCone; vi++) {
                    var destIndex = drawIndex*verticesInCone*2 + vi;
                    var pp1 = new THREE.Vector3(srcPositions.getX(vi), srcPositions.getY(vi), srcPositions.getZ(vi));
                    var pp2 = pp1.clone();
                    pp1.applyMatrix4(transform1);
                    elementPos.setXYZ(destIndex, p1.x, p1.y, p1.z);
                    positions.setXYZ(destIndex, pp1.x, pp1.y, pp1.z);
                    offsets.setX(destIndex, config.measurementOffset1);
                    dirs1.setXYZ(destIndex, dir1.x, dir1.y, dir1.z);
                    dirs2.setXYZ(destIndex, dir2.x, dir2.y, dir2.z);

                    destIndex = drawIndex*verticesInCone*2 + verticesInCone + vi;
                    pp2.applyMatrix4(transform2);
                    elementPos.setXYZ(destIndex, p2.x, p2.y, p2.z);
                    positions.setXYZ(destIndex, pp2.x, pp2.y, pp2.z);
                    offsets.setX(destIndex, config.measurementOffset1);
                    dirs1.setXYZ(destIndex, dir1.x, dir1.y, dir1.z);
                    dirs2.setXYZ(destIndex, dir2.x, dir2.y, dir2.z);
                }

                var srcIndices = coneGeometry.index.array;
                var destIndices = headsGeometry.index.array;
                var si1 = drawIndex*verticesInCone*2;
                var si2 = drawIndex*verticesInCone*2+verticesInCone;
                for(var ii = 0; ii < indicesInCone; ii++) {
                    var destIndex = drawIndex*indicesInCone*2 + ii;
                    destIndices[destIndex] = srcIndices[ii] + si1;
                    destIndex += indicesInCone;
                    destIndices[destIndex] = srcIndices[ii] + si2;
                }

                drawIndex++;
            }
            headsGeometry.setDrawRange(0, primitivesCount);
            return headsGeometry;
        }

        this.onModelChange = function () {
            //console.log("----------messurements chaged-------------");
            //console.log(messurements.toString());

            allMeasurements = measurementsProvider();
            data = [];
            console.debug("MeasurementsCount = " + allMeasurements.length);

            if (lines !== null) {
                this.remove(lines);
                lines = null;
                inputs.clearAll();
                this.remove(heads);
                heads = null;
            }

            if (allMeasurements.length !== 0) {
                lines = new THREE.LineSegments(
                    createLinesGeometry(allMeasurements),
                    new VIEW3D.materials.Measurements(config.color));
                lines.frustumCulled = false;
                this.add(lines);

                createInputs(allMeasurements);

                heads = new THREE.Mesh(
                    createHeadsGeometry(allMeasurements),
                    new VIEW3D.materials.MeasurementsArrows(config.color, CORE.screenSize));
                heads.frustumCulled = false;
                this.add(heads);
            }

            render();
        }
//        this.visible = true;
        this.show = function (visible) {
             inputs.show(visible);
        }

        //init
        THREE.Object3D.call(this);
        modelChangeEvent.subscribe(function () { self.onModelChange(); });
        this.onModelChange();
    }

    VIEW3D.Measurements.prototype = Object.create(THREE.Object3D.prototype);
});