﻿'use strict';
var VIEW3D = VIEW3D || {}
define(['js/view3d/utils/VIEW3D.utils'], function () {
//Represents sicgle meash that can be added to VIEW3D.MultiMesh
VIEW3D.MeshTemplate = function () {
	this.geometry = null;
    this.getIndicesCount = function() {
             return this.geometry.getIndex().array.length;
        },
    this.getVerticesCount = function() {
         return this.geometry.attributes.position.array.length / 3;
    },
    this.setGeometryFrom3DObject = function (model3DId) {
        if (!isEmpty(model3DId)) {
            this.geometry = MODEL.repository.get3DModel(model3DId).object3d.children[0].geometry;
            if (isEmpty(this.geometry.getIndex())) {
                VIEW3D.utils.optimize(this.geometry);
            }
        }
    }
    this.setGeometry = function (geometry) {
        this.geometry = geometry;
        if (isEmpty(this.geometry.getIndex())) {
            VIEW3D.utils.optimize(geometry);
        }
    }
	this.getGeometry = function () {
		return this.geometry;
	}
}

VIEW3D.MeshTemplate.utils = {
	writeGeometryAttributes: function (geometry, attrs, indices, sv, si, transformation) {
		//index
		var i;
		for (i = 0; i < geometry.getIndex().array.length; i++) {
			indices.array[si + i] = geometry.getIndex().array[i] + sv;
		}
		var posSrc = geometry.attributes.position.array;
		var posDest = attrs.position.array;

		var copyNormals = !isEmpty(attrs.normal) && !isEmpty(geometry.attributes.normal);
		var normalDest = copyNormals ? attrs.normal.array : null;
		var normalSrc = copyNormals ? geometry.attributes.normal.array : null;

		var copyUvs = !isEmpty(geometry.attributes.uv) && !isEmpty(attrs.uv);
		var uvSrc = copyUvs ? geometry.attributes.uv.array : null;
		var uvDest = copyUvs ? attrs.uv.array : null;

		for (i = 0; i < posSrc.length / 3; i++) {
			//position
			var e = transformation.elements;
			var x = posSrc[i * 3 + 0];
			var y = posSrc[i * 3 + 1];
			var z = posSrc[i * 3 + 2];
			posDest[(sv + i) * 3 + 0] = e[0] * x + e[4] * y + e[8] * z + e[12];
			posDest[(sv + i) * 3 + 1] = e[1] * x + e[5] * y + e[9] * z + e[13];
			posDest[(sv + i) * 3 + 2] = e[2] * x + e[6] * y + e[10] * z + e[14];
			//uv
			if (copyUvs) {
				uvDest[(sv + i) * 2 + 0] = uvSrc[i * 2 + 0];
				uvDest[(sv + i) * 2 + 1] = uvSrc[i * 2 + 1];
			}
			if (copyNormals) {
				var x = normalSrc[i * 3 + 0];
				var y = normalSrc[i * 3 + 1];
				var z = normalSrc[i * 3 + 2];
				normalDest[(sv + i) * 3 + 0] = e[0] * x + e[4] * y + e[8] * z;
				normalDest[(sv + i) * 3 + 1] = e[1] * x + e[5] * y + e[9] * z;
				normalDest[(sv + i) * 3 + 2] = e[2] * x + e[6] * y + e[10] * z;
			}
		}
	}
}
	
VIEW3D.MeshTemplate.prototype = Object.create({});

VIEW3D.MeshTemplate.prototype.writeAttributes = function (attrs, indices,sv, si, transformation) {
	VIEW3D.MeshTemplate.utils.writeGeometryAttributes(this.geometry, attrs, indices,sv, si, transformation)
}

VIEW3D.ColorableMeshTemplate = function () {
	VIEW3D.MeshTemplate.call(this)
}

VIEW3D.ColorableMeshTemplate.prototype = Object.create(VIEW3D.MeshTemplate.prototype);

VIEW3D.ColorableMeshTemplate.prototype.writeAttributes = function (attrs, indices, sv, si, params) {
	VIEW3D.MeshTemplate.prototype.writeAttributes.call(this, attrs, indices, sv, si, params.transform);
	var colorDest = attrs.color;
	var verticesCount = this.getVerticesCount();
	var x = params.color.x;
	var y = params.color.y;
	var z = params.color.z;
	for (var i = sv; i < sv + verticesCount; i++) {
		colorDest.setXYZ(i, x, y, z);
	}
}

//////
VIEW3D.MultiMaterialTemplate = function (templateObject, config) {
	this.templateObject = templateObject;
	this.modelParams = JSON.parse(config);
	this.optimize();
	this.boundingBox = new THREE.Box3();
	for (var i = 0; i < templateObject.children.length; i++) {
		var child = templateObject.children[i];
		var geom = child.geometry;
		geom.computeBoundingBox();
		this.boundingBox.min.min( geom.boundingBox.min );
		this.boundingBox.max.max( geom.boundingBox.max );
	}
}

VIEW3D.MultiMaterialTemplate.prototype = {
	optimize: function () {
		for (var i = 0; i < this.templateObject.children.length; i++) {
			var geometry = this.templateObject.children[i].geometry;
			if (isEmpty(geometry.index)) {
				VIEW3D.utils.optimize(geometry);
			}
		}
	},

	getIndicesCount: function () {
		var amount = 0;
		for (var i = 0; i < this.templateObject.children.length; i++) {
			var geometry = this.templateObject.children[i].geometry;
			amount += geometry.index.count;
		}
		return amount;
	},

	getVerticesCount: function () {
		var amount = 0;
		for (var i = 0; i < this.templateObject.children.length; i++) {
			var geometry = this.templateObject.children[i].geometry;
			amount += geometry.attributes.position.count;
		}
		return amount;
	},

	writeAttributes: function (attributes, index, startVertex, startIndex, params, mesh) {
		var meshStartVertex = startVertex;
		var meshstartIndex = startIndex;
		for (var i = 0; i < this.templateObject.children.length; i++) {
			var templateMesh = this.templateObject.children[i];
			var geometry = templateMesh.geometry;
			this.writeGeometryAttributes(geometry, attributes, index, meshStartVertex, meshstartIndex, params);
			var material = materials[this.modelParams.Materials[templateMesh.name]];
			//var color = params.color.clone().multiply(new THREE.Vector3(material.kd[0], material.kd[1], material.kd[2]));
			var color = new THREE.Vector3(material.kd[0], material.kd[1], material.kd[2]);
			for (var j = meshStartVertex; j < meshStartVertex + geometry.attributes.position.count; j++) {
				attributes.color.setXYZ(j, color.x, color.y, color.z);
			}
			meshStartVertex += geometry.attributes.position.count;
			meshstartIndex += geometry.index.count;
		}

		var template = this;
		mesh.setColors = function (diffuse, multimesh) {
			var meshData = multimesh.getMeshData(this);
			var meshStartVertex = meshData.startVertex;
			var meshstartIndex = meshData.startIndex;
			for (var i = 0; i < template.templateObject.children.length; i++) {
				var templateMesh = template.templateObject.children[i];
				var geometry = templateMesh.geometry;
				var material = materials[template.modelParams.Materials[templateMesh.name]];
				var color = diffuse.clone().multiply(new THREE.Vector3(material.kd[0], material.kd[1], material.kd[2]));
				for (var j = meshStartVertex; j < meshStartVertex + geometry.attributes.position.count; j++) {
					attributes.color.setXYZ(j, color.x, color.y, color.z);
				}
				attributes.color.needsUpdate = true;
				meshStartVertex += geometry.attributes.position.count;
				meshstartIndex += geometry.index.count;
			}
		}
	},

	writeGeometryAttributes: function (geometry, attrs, indices, sv, si, params) {
		//index
		var i;
		for (i = 0; i < geometry.getIndex().array.length; i++) {
			indices.array[si + i] = geometry.getIndex().array[i] + sv;
		}
		var posSrc = geometry.attributes.position.array;
		var posDest = attrs.position.array;

		var copyNormals = !isEmpty(attrs.normal) && !isEmpty(geometry.attributes.normal);
		var normalDest = copyNormals ? attrs.normal.array : null;
		var normalSrc = copyNormals ? geometry.attributes.normal.array : null;

		var copyUvs = !isEmpty(geometry.attributes.uv) && !isEmpty(attrs.uv);
		var uvSrc = copyUvs ? geometry.attributes.uv.array : null;
		var uvDest = copyUvs ? attrs.uv.array : null;

		var pos = new THREE.Vector3()

		var e = params.elements;
		for (i = 0; i < posSrc.length / 3; i++) {
			//position
			pos.fromArray(posSrc, i * 3);
			var x = pos.x;
			var y = pos.y;
			var z = pos.z;
			posDest[(sv + i) * 3 + 0] = e[0] * x + e[4] * y + e[8] * z + e[12];
			posDest[(sv + i) * 3 + 1] = e[1] * x + e[5] * y + e[9] * z + e[13];
			posDest[(sv + i) * 3 + 2] = e[2] * x + e[6] * y + e[10] * z + e[14];
			//uv
			if (copyUvs) {
				uvDest[(sv + i) * 2 + 0] = uvSrc[i * 2 + 0];
				uvDest[(sv + i) * 2 + 1] = uvSrc[i * 2 + 1];
			}
			if (copyNormals) {
				var x = normalSrc[i * 3 + 0];
				var y = normalSrc[i * 3 + 1];
				var z = normalSrc[i * 3 + 2];
				normalDest[(sv + i) * 3 + 0] = e[0] * x + e[4] * y + e[8] * z;
				normalDest[(sv + i) * 3 + 1] = e[1] * x + e[5] * y + e[9] * z;
				normalDest[(sv + i) * 3 + 2] = e[2] * x + e[6] * y + e[10] * z;
			}
		}
	}
}



// VIEW3D.MeshTemplateExt allows to set per mesh data -> will be stored in per vertex data
VIEW3D.MeshTemplateExt = function () {
	VIEW3D.MeshTemplate.call(this)
}

VIEW3D.MeshTemplateExt.prototype = Object.create(VIEW3D.MeshTemplate.prototype);

VIEW3D.MeshTemplateExt.prototype.writeAttributes = function (attrs, indices, sv, si, params) {
	VIEW3D.MeshTemplate.prototype.writeAttributes.call(this, attrs, indices, sv, si, params.transform);
	var attributeValue = params.attributeValue;
	var verticesCount = this.getVerticesCount();
	for (var attributeName in attributeValue) {
		var attrDest = attrs[attributeName];
		var attrValue = attributeValue[attributeName];
		var x = attrValue.x;
		var y = attrValue.y;
		var z = attrValue.z;
		for (var i = sv; i < sv + verticesCount; i++) {
			attrDest.setXYZ(i, x, y, z);
		}
	}
}
});