﻿'use strict';
var VIEW3D = VIEW3D || {}
var geometryScaled = false;
define(['js/view3d/controls/VIEW3D.controls.SelectArea', 'js/view3d/VIEW3D.MultiMesh'],
    function() {
        VIEW3D.MissingConnector = function(screenSize, nodes) {
            var size = 30.0; //in pixels
            //vertex format
            var attributes = {
                position: { type: Float32Array, size: 3 },
                begin: { type: Float32Array, size: 3 },
                uv: { type: Float32Array, size: 2 }
            };
            var collider = new VIEW3D.collision.CollidersContainer();
            this.getCollider = function() {
                return collider;
            };
            //texture
            var texture = CORE.res.texture["warning"];
            //material
            var material = new VIEW3D.materials.AxisLabel(screenSize, texture, new THREE.Vector4(1, 1, 1, 0.5));
            VIEW3D.MultiMesh.call(this, attributes, material);
            var meshTemplate = {
                geometry: null,
                getGeometry: function() {
                    if (this.geometry === null) {
                        var obj = CORE.res.obj["warning"];
                        this.geometry = obj.children[0].geometry;
                        if (isEmpty(this.geometry.getIndex())) {
                            VIEW3D.utils.optimize(this.geometry);
                        }
                        var posArray = this.geometry.attributes.position.array;
                        var count = posArray.length / 3;
                        if (!geometryScaled) {
                            for (var i = 0; i < count; i++) {
                                posArray[i * 3 + 0] *= size;
                                posArray[i * 3 + 1] *= size;
                                posArray[i * 3 + 2] = -0.0002;
                            }
                            geometryScaled = true;
                        }
                    }
                    return this.geometry;
                },
                getIndicesCount: function() {
                    return this.getGeometry().getIndex().count;
                },
                getVerticesCount: function() {
                    return this.getGeometry().attributes.position.array.length / 3;
                },
                writeAttributes: function(attrs, indices, sv, si, position) {
                    var geometry = this.getGeometry();
                    var i;
                    var indCount = geometry.getIndex().count;
                    var indSrc = geometry.getIndex().array;
                    for (i = 0; i < indCount; i++) {
                        indices.array[si + i] = indSrc[i] + sv;
                    }

                    attrs.uv.array.set(geometry.attributes.uv.array, sv * 2);

                    var verCount = geometry.attributes.position.array.length / 3;
                    var beginArray = attrs.begin.array;
                    var posDest = attrs.position.array;
                    var posSrc = geometry.attributes.position.array;
                    for (i = 0; i < verCount; i++) {
                        beginArray[(sv + i) * 3 + 0] = position.x;
                        beginArray[(sv + i) * 3 + 1] = position.y;
                        beginArray[(sv + i) * 3 + 2] = position.z;
                        posDest[(sv + i) * 3 + 0] = posSrc[i * 3 + 0];
                        posDest[(sv + i) * 3 + 1] = posSrc[i * 3 + 1];
                        posDest[(sv + i) * 3 + 2] = posSrc[i * 3 + 2];
                    }
                }
            }
            var nodesData = {};
            var self = this;
            var onPositionChange = function(node) {
                var data = nodesData[node.id];
                if (!isEmpty(data.mesh)) {
                    self.removeMesh(data.mesh);
                    data.mesh = self.addMeshFromTemplate(meshTemplate, node.position());
                }
            }
            var isMissingConnector = function(node) {
                return false;
            }
            var addNodeCollider = function(node) {
                var data = nodesData[node.id];
                data.collider = new VIEW3D.collision.GlobalPointCollider(HELPERS.toVector3(node.position()), size);
                data.collider.model = node;
                data.collider.missingConnector = true;
                collider.addCollider(data.collider);
            }
            var removeNodeCollider = function(node) {
                var data = nodesData[node.id];
                if (!isEmpty(data)) {
                    if (!isEmpty(data.collider)) {
                        collider.removeCollider(data.collider);
                    }
                    delete data.collider;
                }
            }
            var onConnectorChange = function(node) {
                var data = nodesData[node.id];
                if (!isEmpty(data.mesh) && !isMissingConnector(node)) {
                    self.removeMesh(data.mesh);
                    data.mesh = null;
                    removeNodeCollider(node);
                } else if (isEmpty(data.mesh) && isMissingConnector(node)) {
                    data.mesh = self.addMeshFromTemplate(meshTemplate, node.position());
                    addNodeCollider(node);
                }

            }
            var addNode = function(node) {
                var mesh = null;
                var nodeData = {};
                nodesData[node.id] = nodeData;
                if (isMissingConnector(node)) {
                    mesh = self.addMeshFromTemplate(meshTemplate, node.position());
                    addNodeCollider(node);
                }
                var subPos = HELPERS.subscribePropertyChange(self,
                    node.position,
                    "node.positio",
                    function() { onPositionChange(node) });
                var subConn = HELPERS.subscribePropertyChange(self,
                    node.connectorId,
                    "node.connectorId",
                    function() { onConnectorChange(node) });
                var subBars = HELPERS.subscribeArrayChange(self,
                    node.createsBars,
                    "node.createsBars",
                    function() { onConnectorChange(node) },
                    function() { onConnectorChange(node) });
                var subAttachedBars = HELPERS
                    .subscribeArrayChange(self,
                        node.attachedBars,
                        "node.attachedBars",
                        function() { onConnectorChange(node) },
                        function() { onConnectorChange(node) });

                nodeData.mesh = mesh;
                nodeData.subPos = subPos;
                nodeData.subConn = subConn;
                nodeData.subBars = subBars;
                nodeData.subAttachedBars = subAttachedBars;
            }
            var removeNode = function(node) {
                removeNodeCollider(node);
                var data = nodesData[node.id];
                if (!isEmpty(data)) {
                    if (!isEmpty(data.mesh)) {
                        self.removeMesh(data.mesh);
                    }
                    data.subPos.dispose();
                    data.subConn.dispose();
                    data.subBars.dispose();
                    data.subAttachedBars.dispose();
                    delete nodesData[node.id];
                }
            }
            for (var i = 0; i < nodes().length; i++) {
                var node = nodes()[i];
                addNode(node, i);
            }
            HELPERS.subscribeArrayChange(this, nodes, "MODEL.node", addNode, removeNode);
        }
        VIEW3D.MissingConnector.prototype = Object.create(VIEW3D.MultiMesh.prototype);
    });