﻿"use strict";
var VIEW3D = VIEW3D || {};
define(["js/view3d/VIEW3D.MultiMesh"],
    function() {

        VIEW3D.MissingElement = function(screenSize, model) {
            THREE.Object3D.call(this);
            var size = 30.0; // in pixels
            var barsData = {};
            var rfsData = {};
            var barsToDraw = ko.observableArray();
            var rfsToDraw = ko.observableArray();
            // vertex format
            var attributes = {
                position: { type: Float32Array, size: 3 },
                begin: { type: Float32Array, size: 3 },
                uv: { type: Float32Array, size: 2 }
            };
            var collider = new VIEW3D.collision.CollidersContainer();
            this.getCollider = function() {
                return collider;
            };
            // texture
            var textureNode = CORE.res.texture["warning"];
            var textureBar = CORE.res.texture["warning"];
            // material
            var materialNode = new VIEW3D.materials.AxisLabel(screenSize, textureNode, new THREE.Vector4(1, 1, 1, 0.5));
            var materialBar = new VIEW3D.materials.AxisLabel(screenSize, textureBar, new THREE.Vector4(1, 1, 1, 0.5));
            // mesh
            this.multiMeshNode = new VIEW3D.MultiMesh(attributes, materialNode);
            this.add(this.multiMeshNode);
            this.multiMeshBar = new VIEW3D.MultiMesh(attributes, materialBar);
            this.add(this.multiMeshBar);

            var meshTemplate = {
                geometry: null,
                getGeometry: function() {
                    if (this.geometry === null) {
                        var obj = CORE.res.obj["warning"];
                        this.geometry = obj.children[0].geometry;
                        if (isEmpty(this.geometry.getIndex())) {
                            VIEW3D.utils.optimize(this.geometry);
                        }
                        var posArray = this.geometry.attributes.position.array;
                        var count = posArray.length / 3;
                        if (!geometryScaled) {
                            for (var i = 0; i < count; i++) {
                                posArray[i * 3 + 0] *= size;
                                posArray[i * 3 + 1] *= size;
                                posArray[i * 3 + 2] = -0.0002;
                            }
                            geometryScaled = true;
                        }
                    }
                    return this.geometry;
                },
                getIndicesCount: function() {
                    return this.getGeometry().getIndex().count;
                },
                getVerticesCount: function() {
                    return this.getGeometry().attributes.position.array.length / 3;
                },
                writeAttributes: function(attrs, indices, sv, si, position) {
                    var geometry = this.getGeometry();
                    var i;
                    var indCount = geometry.getIndex().count;
                    var indSrc = geometry.getIndex().array;
                    for (i = 0; i < indCount; i++) {
                        indices.array[si + i] = indSrc[i] + sv;
                    }
                    attrs.uv.array.set(geometry.attributes.uv.array, sv * 2);
                    var verCount = geometry.attributes.position.array.length / 3;
                    var beginArray = attrs.begin.array;
                    var posDest = attrs.position.array;
                    var posSrc = geometry.attributes.position.array;
                    for (i = 0; i < verCount; i++) {
                        beginArray[(sv + i) * 3 + 0] = position.x;
                        beginArray[(sv + i) * 3 + 1] = position.y;
                        beginArray[(sv + i) * 3 + 2] = position.z;
                        posDest[(sv + i) * 3 + 0] = posSrc[i * 3 + 0];
                        posDest[(sv + i) * 3 + 1] = posSrc[i * 3 + 1];
                        posDest[(sv + i) * 3 + 2] = posSrc[i * 3 + 2];
                    }
                }
            };
            var elementsData = [];
            var self = this;
            var getPosition;
            var onPositionChange = function(element) {
                var data = elementsData[element.type + element.id];
                if (!isEmpty(data.mesh)) {
                    self.removeMesh(data.mesh);
                    switch (element.type) {
                    case MODEL.OBJECT_TYPE.Bar:
                        data.mesh = self.multiMeshBar.addMeshFromTemplate(meshTemplate, getPosition(element));
                        break;
                    case MODEL.OBJECT_TYPE.Node:
                        data.mesh = self.multiMeshNode.addMeshFromTemplate(meshTemplate, getPosition(element));
                        break;
                    }
                }
            };
            var addElementCollider = function(element) {
                var data = elementsData[element.type + element.id];
                switch (element.type) {
                case MODEL.OBJECT_TYPE.Node:
                    data.collider = new VIEW3D.collision
                        .GlobalPointCollider(HELPERS.toVector3(getPosition(element)), size);
                    break;
                default:
                    data.collider = new VIEW3D.collision
                        .GlobalPointCollider(getPosition(element), size);
                    break;
                }
                data.collider.model = element;
                collider.addCollider(data.collider);
            };
            var removeElementCollider = function(element) {
                var data = elementsData[element.type + element.id];
                if (!isEmpty(data.collider)) {
                    collider.removeCollider(data.collider);
                }
                delete data.collider;
            };
            var addCustomCollider = function(element) {
                var data;
                if (element.type === MODEL.OBJECT_TYPE.Bar) {
                    data = barsData[element.id];
                } else {
                    data = rfsData[element.id];
                }
                var pos = HELPERS.toVector(new THREE.Vector3()
                    .addVectors(element.node2().position(), element.node1().position())
                    .multiplyScalar(0.5));
                data.collider = new VIEW3D.collision.GlobalPointCollider(HELPERS.toVector3(pos), size);
                data.collider.model = element;
                collider.addCollider(data.collider);
            };
            var addCustomElement = function(element, self) {
                var subPos1, subPos2;
                if (element.type === MODEL.OBJECT_TYPE.Bar) {
                    var barData = {};
                    barsData[element.id] = barData;
                    addCustomCollider(element);
                    barsToDraw.push(element);
                    subPos1 = HELPERS.subscribePropertyChange(self,
                        element.node1().position,
                        "bar.node1.position",
                        function() { onPositionChange(element) });
                    subPos2 = HELPERS.subscribePropertyChange(self,
                        element.node2().position,
                        "bar.node2.position",
                        function() { onPositionChange(element) });
                    barData.subPos1 = subPos1;
                    barData.subPos2 = subPos2;
                }
            };
            var removeCustomElement = function(element) {
                var data, index, i;
                if (element.type === MODEL.OBJECT_TYPE.Bar) {
                    data = barsData[element.id];
                    if (!isEmpty(data)) {
                        data.subPos1.dispose();
                        data.subPos2.dispose();
                        delete barsData[element.id];
                    }
                    index = -1;
                    for (i = 0; i < barsToDraw().length; i++) {
                        if (barsToDraw()[i] === element) {
                            index = i;
                            break;
                        }
                    }
                    if (index !== -1) {
                        barsToDraw.splice(index, 1);
                    }
                } else {
                    data = rfsData[element.id];
                    if (!isEmpty(data)) {
                        data.subPos1.dispose();
                        data.subPos2.dispose();
                        delete rfsData[element.id];
                    }
                    index = -1;
                    for (i = 0; i < rfsToDraw().length; i++) {
                        if (rfsToDraw()[i] === element) {
                            index = i;
                            break;
                        }
                    }
                    if (index !== -1) {
                        rfsToDraw.splice(index, 1);
                    }
                }
            };            
            var onModelChange = function(element) {
                var data = elementsData[element.type + element.id];
                if (!isEmpty(data.mesh)) {
                    switch (element.type) {
                    case MODEL.OBJECT_TYPE.Bar:
                        data.mesh = self.multiMeshBar.removeMesh(data.mesh);
                        data.subPos1.dispose();
                        data.subPos2.dispose();
                        data.subProf.dispose();
                        removeCustomElement(element);
                        break;
                    case MODEL.OBJECT_TYPE.Node:
                        data.mesh = self.multiMeshNode.removeMesh(data.mesh);
                        data.subConn.dispose();
                        data.subPos.dispose();
                        break;

                    }
                    data.mesh = 0;
                    removeElementCollider(element);
                } else if (isEmpty(data.mesh)) {
                    switch (element.type) {
                    case MODEL.OBJECT_TYPE.Bar:
                        data.mesh = self.multiMeshBar.addMeshFromTemplate(meshTemplate, getPosition(element));
                        break;
                    case MODEL.OBJECT_TYPE.Node:
                        data.mesh = self.multiMeshNode.addMeshFromTemplate(meshTemplate, getPosition(element));
                        break;
                    }
                    addElementCollider(element);
                }
            };
            this.removeElement = function (data) {
                var type = null;
                switch (data.Type) {
                    case 0:
                        type = MODEL.OBJECT_TYPE.Bar;
                        break;
                    case 1:
                        type = MODEL.OBJECT_TYPE.Node;
                        break;
                    case 3:
                        type = MODEL.OBJECT_TYPE.Support;
                        break;
                }
                var element = {
                    id: data.ID,
                    type: type
                }
                onModelChange(element);
            }
            getPosition = function(element) {
                var position = null;
                switch (element.type) {
                case MODEL.OBJECT_TYPE.Bar:
                case MODEL.OBJECT_TYPE.Node:
                    position = element.position();
                    break;

                case MODEL.OBJECT_TYPE.Support:
                    position = element.position();
                    //var subSuppPos = HELPERS
                    //    .subscribePropertyChange(self,
                    //        element.position,
                    //        "node.position",
                    //        function() { onPositionChange(element) });
                    //data.subPos = subSuppPos;
                    //var subSupp = HELPERS
                    //    .subscribePropertyChange(self,
                    //        element.supportId,
                    //        "node.supportId",
                    //        function() { onModelChange(element) });
                    //data.subConn = subSupp;
                    break;
                }
                return position;
            };

            var addElement = function(element) {
                var elementData = {};
                var string = element.type + element.id;
                elementsData[string] = elementData;
                var position = getPosition(element);
                var data = elementsData[element.type + element.id];
                var mesh, subPos1, subPos2, subProf;
                switch (element.type) {
                case MODEL.OBJECT_TYPE.Bar:
                    mesh = self.multiMeshBar.addMeshFromTemplate(meshTemplate, position);
                    subPos1 = HELPERS
                        .subscribePropertyChange(self,
                            element.node1().position,
                            "bar.node1().position",
                            function() { onPositionChange(element) });
                    data.subPos1 = subPos1;
                    subPos2 = HELPERS
                        .subscribePropertyChange(self,
                            element.node2().position,
                            "bar.node2().position",
                            function() { onPositionChange(element) });
                    data.subPos2 = subPos2;
                    subProf = HELPERS
                        .subscribePropertyChange(self,
                            element.profileId,
                            "bar.profileId",
                            function() { onModelChange(element) });
                    data.subProf = subProf;
                    break;
                case MODEL.OBJECT_TYPE.Node:
                    mesh = self.multiMeshNode.addMeshFromTemplate(meshTemplate, position);
                    var subNodePos = HELPERS
                        .subscribePropertyChange(self,
                            element.position,
                            "node.position",
                            function() { onPositionChange(element) });
                    data.subPos = subNodePos;
                    var subCon = HELPERS
                        .subscribePropertyChange(self,
                            element.connectorId,
                            "node.connectorId",
                            function() { onModelChange(element) });
                    data.subConn = subCon;
                    break;
                }
                elementData.mesh = mesh;
                addElementCollider(element);
            };
            for (var i = 0; i < model.missingElements().length; i++) {
                var element = model.missingElements()[i];
                var newElement = null;
                var tryAdd = true;
                switch (element.Type) {
                case 0:
                    newElement = model.getBarById(element.ID);
                    addCustomElement(newElement, this);
                    break;
                case 1:
                    newElement = model.getNodeById(element.ID);
                    break;
                case 3:
                    newElement = model.getSupportById(element.ID);
                    break;
                default:
                    console.warn("Found not yet implemented missing element with ID: " +
                        element.ID +
                        " and Type " +
                        element.Type);
                    tryAdd = false;
                    break;
                }
                if (tryAdd) {
                    addElement(newElement);
                }
            }
            var barsView = new VIEW3D.LineBars(barsToDraw, false);
            var rfsView = new VIEW3D.LineBars(rfsToDraw, false);
            collider.addCollider(barsView.getCollider());
            collider.addCollider(rfsView.getCollider());
            scene.add(barsView);
            scene.add(rfsView);
        };
        VIEW3D.MissingElement.prototype = Object.create(THREE.Object3D.prototype);
    });