﻿'use strict';
var VIEW3D = VIEW3D || {}
var geometryScaled = false;
define(['js/view3d/controls/VIEW3D.controls.SelectArea', 'js/view3d/VIEW3D.MultiMesh'],
    function() {
        VIEW3D.MissingProfile = function(screenSize, bars) {
            var size = 30.0; //in pixels
            var barsToDraw = ko.observableArray();
            //vertex format
            var attributes = {
                position: { type: Float32Array, size: 3 },
                begin: { type: Float32Array, size: 3 },
                uv: { type: Float32Array, size: 2 }
            };
            var collider = new VIEW3D.collision.CollidersContainer();
            this.getCollider = function() {
                return collider;
            };
            //texture
            var texture = CORE.res.texture["warning"];
            //material
            var material = new VIEW3D.materials.AxisLabel(screenSize, texture, new THREE.Vector4(1, 1, 1, 0.5));
            VIEW3D.MultiMesh.call(this, attributes, material);
            var meshTemplate = {
                geometry: null,
                getGeometry: function() {
                    if (this.geometry === null) {
                        var obj = CORE.res.obj["warning"];
                        this.geometry = obj.children[0].geometry;
                        if (isEmpty(this.geometry.getIndex())) {
                            VIEW3D.utils.optimize(this.geometry);
                        }
                        var posArray = this.geometry.attributes.position.array;
                        var count = posArray.length / 3;
                        if (!geometryScaled) {
                            for (var i = 0; i < count; i++) {
                                posArray[i * 3 + 0] *= size;
                                posArray[i * 3 + 1] *= size;
                                posArray[i * 3 + 2] = -0.0002;
                            }
                            geometryScaled = true;
                        }
                    }
                    return this.geometry;
                },
                getIndicesCount: function() {
                    return this.getGeometry().getIndex().count;
                },
                getVerticesCount: function() {
                    return this.getGeometry().attributes.position.array.length / 3;
                },
                writeAttributes: function(attrs, indices, sv, si, position) {
                    var geometry = this.getGeometry();
                    var i;
                    var indCount = geometry.getIndex().count;
                    var indSrc = geometry.getIndex().array;
                    for (i = 0; i < indCount; i++) {
                        indices.array[si + i] = indSrc[i] + sv;
                    }
                    attrs.uv.array.set(geometry.attributes.uv.array, sv * 2);
                    var verCount = geometry.attributes.position.array.length / 3;
                    var beginArray = attrs.begin.array;
                    var posDest = attrs.position.array;
                    var posSrc = geometry.attributes.position.array;
                    for (i = 0; i < verCount; i++) {
                        beginArray[(sv + i) * 3 + 0] = position.x;
                        beginArray[(sv + i) * 3 + 1] = position.y;
                        beginArray[(sv + i) * 3 + 2] = position.z;
                        posDest[(sv + i) * 3 + 0] = posSrc[i * 3 + 0];
                        posDest[(sv + i) * 3 + 1] = posSrc[i * 3 + 1];
                        posDest[(sv + i) * 3 + 2] = posSrc[i * 3 + 2];
                    }
                }
            }
            var barsData = {};
            var self = this;
            var onPositionChange = function(bar) {
                var data = barsData[bar.id];
                if (!isEmpty(data.mesh)) {
                    self.removeMesh(data.mesh);
                    var pos = HELPERS.toVector(new THREE.Vector3()
                        .addVectors(bar.node2().position(), bar.node1().position())
                        .multiplyScalar(0.5));
                    data.mesh = self.addMeshFromTemplate(meshTemplate, pos);
                }
            }
            var addBarCollider = function (bar) {
                var data = barsData[bar.id];
                var pos = HELPERS.toVector(new THREE.Vector3()
                        .addVectors(bar.node2().position(), bar.node1().position())
                        .multiplyScalar(0.5));
                data.collider = new VIEW3D.collision.GlobalPointCollider(HELPERS.toVector3(pos), size);
                data.collider.model = bar;
                data.collider.missingProfile = true;
                collider.addCollider(data.collider);
            }
            var removeBarCollider = function (bar) {
                var data = barsData[bar.id];
                if (!isEmpty(data)) {
                    if (!isEmpty(data.collider)) {
                        collider.removeCollider(data.collider);
                    }
                    delete data.collider;
                }
            }
            var onProfileChange = function (bar) {
                var data = barsData[bar.id];
                if (bar.profileId() !== 0) {
                    self.removeMesh(data.mesh);
                    data.mesh = null;
                    removeBarCollider(bar);
                    var index = -1;
                    for (var i = 0; i < barsToDraw().length; i++) {
                        if (barsToDraw()[i] === bar) {
                            index = i;
                            break;
                        }
                    }
                    if (index !== -1) {
                        barsToDraw.splice(index, 1);
                    }
                } else {
                    var pos = HELPERS.toVector(new THREE.Vector3()
                        .addVectors(bar.node2().position(), bar.node1().position())
                        .multiplyScalar(0.5));
                    data.mesh = self.addMeshFromTemplate(meshTemplate, pos);
                    addBarCollider(bar);
                    barsToDraw.push(bar);
                }
            }
            var addBar = function(bar) {
                var barData = {};
                barsData[bar.id] = barData;
                if (bar.profileId() === 0) {
                    var pos = HELPERS.toVector(new THREE.Vector3()
                        .addVectors(bar.node2().position(), bar.node1().position())
                        .multiplyScalar(0.5));
                    var mesh = self.addMeshFromTemplate(meshTemplate, pos);
                    addBarCollider(bar);
                    barsToDraw.push(bar);
                    barData.mesh = mesh;
                }
                var subPos1 = HELPERS.subscribePropertyChange(self,
                    bar.node1().position,
                    "bar.node1.position",
                    function() { onPositionChange(bar) });
                var subPos2 = HELPERS.subscribePropertyChange(self,
                    bar.node2().position,
                    "bar.node2.position",
                    function() { onPositionChange(bar) });
                var subProf = HELPERS.subscribePropertyChange(self,
                    bar.profileId,
                    "bar.profileId",
                    function() { onProfileChange(bar) });
                barData.subPos1 = subPos1;
                barData.subPos2 = subPos2;
                barData.subProf = subProf;
            }
            var removeBar = function(bar) {
                var data = barsData[bar.id];
                if (!isEmpty(data)) {
                    if (!isEmpty(data.mesh)) {
                        self.removeMesh(data.mesh);
                    }
                    data.subPos1.dispose();
                    data.subPos2.dispose();
                    data.subProf.dispose();
                    delete barsData[bar.id];
                }
                var index = -1;
                for (var i = 0; i < barsToDraw().length; i++) {
                    if (barsToDraw()[i] === bar) {
                        index = i;
                        break;
                    }
                }
                if (index !== -1) {
                    barsToDraw.splice(index, 1);
                }
            }
            for (var i = 0; i < bars().length; i++) {
                var bar = bars()[i];
                if (bar.profileId() === 0) {
                    addBar(bar, i);
                }
            }
            var view = new VIEW3D.LineBars(barsToDraw, false);
            collider.addCollider(view.getCollider());
            scene.add(view);
            HELPERS.subscribeArrayChange(this, bars, "MODEL.bar", addBar, removeBar);
        }
        VIEW3D.MissingProfile.prototype = Object.create(VIEW3D.MultiMesh.prototype);
    });