﻿'use strict';
var VIEW3D = VIEW3D || {}

define(['js/view3d/utils/VIEW3D.utils.ResizableGeometry'],function(){
VIEW3D.MultiMesh = function(attributes, material) {
    //getIndex() = { type: Uint32Array, size: 1 }; //must have
    THREE.Mesh.call(this, new VIEW3D.utils.ResizableGeometry(attributes), material);
    this.frustumCulled = false; //no visibility test
    this.visible = false; //do not redner empty buffers
    var lastMeshId = 0;
    var meshes = {};

    //////////////////////////////////////////////////////////////////////////////////////////
    //PUBLIC
    this.addMeshFromTemplate = function(meshTemplate, params) {
        var indexCount = meshTemplate.getIndicesCount(params);
        var vartexCount = meshTemplate.getVerticesCount(params);
        if (indexCount === 0 || vartexCount === 0) {
            return 0;
        }
        var meshId = ++lastMeshId;
        var startVertex = this.geometry.verticesInUse;
        var startIndex = this.geometry.indicesInUse;

        //make place for new vertices
        this.geometry.increaseSize(indexCount, vartexCount);

        //ask meshTemplate to fill the data
        var mesh = {id:meshId};
        meshTemplate.writeAttributes(this.geometry.attributes, this.geometry.index, startVertex, startIndex, params, mesh);

        //show me since there is at least one mesh in buffers
        this.visible = true;
        meshes[meshId] = {
            indexCount: indexCount,
            vartexCount: vartexCount,
            startVertex: startVertex,
            startIndex: startIndex
        }
        render();
        return mesh;
    }
    this.removeMesh = function(mesh) {
        if (isEmpty(mesh) || isEmpty(mesh.id) || mesh.id === 0) {
            return;
        }
        var meshData = meshes[mesh.id];
        if (!isEmpty(meshData)) {
            delete meshes[mesh.id];

            this.geometry.splice(meshData.startIndex, meshData.indexCount, meshData.startVertex, meshData.vartexCount);

            //update "older" meshes
            for (var id in meshes) {
                if (meshes.hasOwnProperty(id)) {
                    if (id > mesh.id) {
                        meshes[id].startIndex -= meshData.indexCount;
                        meshes[id].startVertex -= meshData.vartexCount;
                    }
                }
            }
            //hide if there is no more meshes
            this.visible = Object.keys(meshes).length > 0;
            render();
        }
    }
    
    this.getMeshData = function(mesh) {
        return meshes[mesh.id];
    }

    this.fillAttributeXYZ = function (mesh, name, x,y,z) {
        var attribute = this.geometry.attributes[name];
        var meshData = meshes[mesh.id];
        for(var i = meshData.startVertex; i < meshData.startVertex + meshData.vartexCount; i++){
            attribute.setXYZ(i, x,y,z);
        }
        attribute.needsUpdate = true;
    }

    this.dispose = function () {
        this.geometry.dispose();
    }
}
VIEW3D.MultiMesh.prototype = Object.create(THREE.Mesh.prototype);
});