'use strict';
var VIEW3D = VIEW3D || {}

define(['js/view3d/VIEW3D.Edit.MoveControl', 'js/view3d/VIEW3D.Edit.RotationControl'], function () {
    VIEW3D.PositionEdit = function (position, allBars) {
        var collider = new VIEW3D.collision.CollidersContainer();
        this.getCollider = function () {
            return collider;
        };

        THREE.Object3D.call(this);

        this.createDirMove = function (dir, color) {
            var value = 0;
            var moveSpace = null;
            var pos = HELPERS.toVector3(position());
            switch (dir){
                case 'x':
                    value = pos.x;
                    moveSpace = new THREE.Matrix4().set(
                        1, 0, 0, 0,
                        0, 1, 0, 0,
                        0, 0, 1, 0,
                        0, 0, 0, 1);
                    break;
                case 'y':
                    value = pos.y;
                    moveSpace = new THREE.Matrix4().set(
                        0,-1, 0, 0,
                        1, 0, 0, 0,
                        0, 0, 1, 0,
                        0, 0, 0, 1);
                    break;
                case 'z':
                    value = pos.z;
                    moveSpace = new THREE.Matrix4().set(
                        0, 0,-1, 0,
                        0, 1, 0, 0,
                        1, 0, 0, 0,
                        0, 0, 0, 1);
                    break;
            }
            //var pos = HELPERS.toVector3(position());
            var ctrl = new VIEW3D.Edit.MoveControl(value, moveSpace, pos, null, allBars, color, function (newVal) {
                if(Math.abs(newVal - value) > 0.001) {
                    var newPos = HELPERS.toVector(new THREE.Vector3(newVal-value).applyMatrix4(moveSpace) );
                    position(position().add(newPos));
                }
            }, this.clearAllImputs.bind(this));
            this.add(ctrl.mesh);
            collider.addCollider(ctrl.collider);
            return ctrl;
        }

        this.controls = [];
        this.disposeControls = function() {
            for (var i = 0; i < this.controls.length; i++) {
                var control = this.controls[i];
                control.dispose();
                collider.removeCollider(control.collider);
                this.remove(control.mesh)
            }
            this.controls = [];
        }

        this.clearAllImputs = function () {
            for (var i = 0; i < this.controls.length; i++) {
                var control = this.controls[i];
                control.setGrayed(true);
                control.inputs.clearInputs();
            }
        }

        this.update = function() {
            this.disposeControls();
            this.controls.push( this.createDirMove( 'x', new THREE.Vector4(1, 0, 0, 1)));
            this.controls.push( this.createDirMove( 'y', new THREE.Vector4(0, 1, 0, 1)));
            this.controls.push( this.createDirMove( 'z', new THREE.Vector4(0, 0, 1, 1)));
        }
        this.update();


        var posSub = HELPERS.subscribePropertyChange(this, position, "position", this.update );

        this.dispose = function() {
            posSub.dispose();
            this.disposeControls();
        }
    }
    VIEW3D.PositionEdit.prototype = Object.create(THREE.Object3D.prototype);
});