﻿'use strict';
var VIEW3D = VIEW3D || {}

// class VIEW3D.NodeForce
// binds to MODEL.load.node.NodeForce and creates geometry
// that represents that NodeForce model. Once NodeForce model
// is assigned to Node this class creates new Object3D and
// adds it to the parent
VIEW3D.NodeForce = function(nodeForceModel, parent, arrows) { //
    var arrowsId = -1;
    var instances = []; // synchronnized with nodeForceModel.assignedTo
    var addLoadArrows = function(pos) {
        var rotation = nodeForceModel.rotation();
        var q = new THREE.Quaternion().setFromEuler(new THREE.Euler(rotation.x, rotation.y, rotation.z, 'XYZ'));
        var v = nodeForceModel.forces();
        v.applyQuaternion(q);
        var arrows = [
            {
                dir: new THREE.Vector3(v.x, 0, 0),
                pos: new THREE.Vector3(pos.x, pos.y, pos.z),
                color: THREE.Vector3(1, 0, 0)
            },
            {
                dir: new THREE.Vector3(0, v.y, 0),
                pos: new THREE.Vector3(pos.x, pos.y, pos.z),
                color: THREE.Vector3(0, 1, 0)
            },
            {
                dir: new THREE.Vector3(0, 0, v.z),
                pos: new THREE.Vector3(pos.x, pos.y, pos.z),
                color: THREE.Vector3(0, 0, 1)
            }
        ];
        arrowsId = arrows.addArrow(arrows);
        return { arrowsId: arrowsId };
    }
    var resetLoadArrows = function(instance, pos) {
        arrows.removeArrows(instance.arrowsId);
        addLoadArrows(pos);
    }
    var onNodePositionChange = function(pos) {
        for (var index = 0; index < nodeForceModel.assignedTo().length; index++) {
            if (pos === nodeForceModel.assignedTo()[index].position()) {
                resetLoadArrows(instances[index], pos);
                render();
                break;
            }
        }
    }
    var onNodeAssigned = function(node, index) {
        var instance = createInstance(node.position());
        instance.positionChangeSubscription = HELPERS.subscribePropertyChange(VIEW3D.NodeForce, node.position, "node.position", onNodePositionChange);
        instances[index] = instance;
        parent.add(instance);
        render();
    }
    var onNodeUnassigned = function(node, index) {
        arrows.removeArrows(instances[index].arrowsId);
        instances[index].positionChangeSubscription.dispose();
        instances.splice(index, 1);
        render();
    }
    var onPropertyChane = function() {
        var i;
        for (i = 0; i < instances.length; i++) {
            arrows.removeArrows(instances[i].arrowsId);
            instances[i].positionChangeSubscription.dispose();
        }
        instances = [];
        for (i = 0; i < nodeForceModel.assignedTo().length; i++) {
            var node = nodeForceModel.assignedTo()[i];
            onNodeAssigned(node, i);
        }
        render();
    }
    var assignedSub = HELPERS.subscribeArrayChange(this, nodeForceModel.assignedTo, "MODEL.loads.node.assignedTo", onNodeAssigned, onNodeUnassigned);
    var forcePropSub = HELPERS.subscribePropertyChange(this, nodeForceModel.forces, "nodeForceModel.forces", onPropertyChane);
    this.clean = function() {
        for (var index = 0; index < instances.length; index++) {
            parent.remove(instances[index]);
            instances[index].positionChangeSubscription.dispose();
        }
        instances = [];
        assignedSub.dispose();
        forcePropSub.dispose();
        render();
    }
    for (var index = 0; index < nodeForceModel.assignedTo().length; index++) {
        onNodeAssigned(nodeForceModel.assignedTo()[index], index);
    }
};