var VIEW3D = VIEW3D || {}

define(['js/view3d/utils/VIEW3D.utils'], function () {

    VIEW3D.PipeTemplate = function (templateObject, config) {
        this.templateObject = templateObject;
        this.modelParams = JSON.parse(config);
        //this.optimize();
        var bb = new THREE.Box3();
        for (var i = 0; i < templateObject.children.length; i++) {
            var child = templateObject.children[i];               
            var geom = child.geometry;
            geom.computeBoundingBox();
            bb.expandByPoint(geom.boundingBox.min);
            bb.expandByPoint(geom.boundingBox.max);
        }
        this.boundingBox = bb;
        return;
    }

    VIEW3D.PipeTemplate.prototype = {
        getIndicesCount: function () {
            var amount = 0;
            for (var i = 0; i < this.templateObject.children.length; i++) {
                var geometry = this.templateObject.children[i].geometry;
                amount += geometry.index.count;
            }
            return amount;
        },

        getVerticesCount: function () {
            var amount = 0;
            for (var i = 0; i < this.templateObject.children.length; i++) {
                var geometry = this.templateObject.children[i].geometry;
                amount += geometry.attributes.position.count;
            }
            return amount;
        },

        writeAttributes: function (attributes, index, startVertex, startIndex, params, mesh) {
            var meshStartVertex = startVertex;
            var meshstartIndex = startIndex;
            for (var i = 0; i < this.templateObject.children.length; i++) {
                var templateMesh = this.templateObject.children[i];
                var geometry = templateMesh.geometry;
                VIEW3D.PipeTemplate.writeGeometryAttributes(geometry, attributes, index, meshStartVertex, meshstartIndex, params, this.modelParams);
                var material = materials[this.modelParams.Materials[templateMesh.name]];
                var color = params.color.clone().multiply(new THREE.Vector3(1, 1, 1));
                //var color = params.color.clone().multiply(new THREE.Vector3(material.kd[0], material.kd[1], material.kd[2]));
                for (var j = meshStartVertex; j < meshStartVertex + geometry.attributes.position.count; j++) {
                    attributes.color.setXYZ(j, color.x, color.y, color.z);
                }
                meshStartVertex += geometry.attributes.position.count;
                meshstartIndex += geometry.index.count;
            }

            var pipeTemplate = this;
            mesh.setColors = function (diffuse, multimesh) {
                var meshData = multimesh.getMeshData(this);
                var meshStartVertex = meshData.startVertex;
                var meshstartIndex = meshData.startIndex;
                for (var i = 0; i < pipeTemplate.templateObject.children.length; i++) {
                    var templateMesh = pipeTemplate.templateObject.children[i];
                    var geometry = templateMesh.geometry;
                    var material = materials[pipeTemplate.modelParams.Materials[templateMesh.name]];
                    var color = diffuse.clone().multiply(new THREE.Vector3(material.kd[0], material.kd[1], material.kd[2]));
                    for (var j = meshStartVertex; j < meshStartVertex + geometry.attributes.position.count; j++) {
                        attributes.color.setXYZ(j, color.x, color.y, color.z);
                    }
                    attributes.color.needsUpdate = true;
                    meshStartVertex += geometry.attributes.position.count;
                    meshstartIndex += geometry.index.count;
                }
            }
        },
    }

    VIEW3D.PipeTemplate.resizeVertex = function (pos, modelParams, d) {
        var h = 0.010;
        var c = modelParams.C;
        var dd = pos.z - c;
        if ((dd < h && dd > -h) ||
            pos.x > modelParams.R ||
            pos.x < -modelParams.R) {
            //skaluj tylko w lewo i prawo
            c = pos.z;
        } else {
            if (dd < 0) {
                var a = -h / (modelParams.R - h)
                var mm = a * dd + modelParams.R * a;
            }
        }

        pos.z -= c;
        var l = Math.sqrt(pos.x * pos.x + pos.z * pos.z);
        var mul = d / l;
        var dx = pos.x * mul;
        var dz = pos.z * mul;

        pos.x += dx;
        pos.z += dz;
        pos.z += c;
        if (modelParams.C !== 0) {
            pos.z += d;
        }
    }

    VIEW3D.PipeTemplate.writeGeometryAttributes = function (geometry, attrs, indices, sv, si, params, modelParams) {

        //index
        var i;
        for (i = 0; i < geometry.getIndex().array.length; i++) {
            indices.array[si + i] = geometry.getIndex().array[i] + sv;
        }
        var posSrc = geometry.attributes.position.array;
        var posDest = attrs.position.array;

        var copyNormals = !isEmpty(attrs.normal) && !isEmpty(geometry.attributes.normal);
        var normalDest = copyNormals ? attrs.normal.array : null;
        var normalSrc = copyNormals ? geometry.attributes.normal.array : null;

        var copyUvs = !isEmpty(geometry.attributes.uv) && !isEmpty(attrs.uv);
        var uvSrc = copyUvs ? geometry.attributes.uv.array : null;
        var uvDest = copyUvs ? attrs.uv.array : null;

        var pos = new THREE.Vector3()

        var resize = !isEmpty(modelParams.S);
        var d = (params.size - modelParams.S) * 0.5;
        var e = params.transform.elements;
        for (i = 0; i < posSrc.length / 3; i++) {
            //position
            pos.fromArray(posSrc, i * 3);
            if (resize) {
                VIEW3D.PipeTemplate.resizeVertex(pos, modelParams, d);
            }
            var x = pos.x;
            var y = pos.y;
            var z = pos.z;
            posDest[(sv + i) * 3 + 0] = e[0] * x + e[4] * y + e[8] * z + e[12];
            posDest[(sv + i) * 3 + 1] = e[1] * x + e[5] * y + e[9] * z + e[13];
            posDest[(sv + i) * 3 + 2] = e[2] * x + e[6] * y + e[10] * z + e[14];
            //uv
            if (copyUvs) {
                uvDest[(sv + i) * 2 + 0] = uvSrc[i * 2 + 0];
                uvDest[(sv + i) * 2 + 1] = uvSrc[i * 2 + 1];
            }
            if (copyNormals) {
                var x = normalSrc[i * 3 + 0];
                var y = normalSrc[i * 3 + 1];
                var z = normalSrc[i * 3 + 2];
                normalDest[(sv + i) * 3 + 0] = e[0] * x + e[4] * y + e[8] * z;
                normalDest[(sv + i) * 3 + 1] = e[1] * x + e[5] * y + e[9] * z;
                normalDest[(sv + i) * 3 + 2] = e[2] * x + e[6] * y + e[10] * z;
            }
        }
    }

});