'use strict';
var VIEW3D = VIEW3D || {}

// class PointNode 
// binnds to a single MODEL.Node and 
// modifies position and color from PointNodeCloud that was assigned to it
VIEW3D.PointNode = function (pointNodeCloud, nodeModel) {

    //
    var collider = new VIEW3D.collision.GlobalPointCollider(HELPERS.toVector3(nodeModel.position()));
    collider.view3D = this;
    collider.model = nodeModel;

    this.getCollider = function() {
        return collider;
    }

    this.nodeModel = nodeModel;
    this.select = function(multiSelection) {
        console.log('PointNode: select');
        this.nodeModel.selected(!this.nodeModel.selected() || !multiSelection);
        return true;
    }
    this.onNodeSelectedChanged = function(newValue) {
        try {
            var color = newValue ? VIEW3D.PointNodeCloud.selectionColor : VIEW3D.PointNodeCloud.originalColor;
            pointNodeCloud.geometry.attributes.color.setXYZ(this.index, color.r, color.g, color.b);
            pointNodeCloud.geometry.attributes.color.needsUpdate = true;
            render();
        } catch (err) {
            console.error("Node selection failed! Reason: " + err);
        }
    },
    this.onNodePositionChanged = function(newValue) {
        try {
            collider.pos = HELPERS.toVector3(newValue);
            pointNodeCloud.geometry.attributes.position.setXYZ(this.index, newValue.x, newValue.y, newValue.z);
            pointNodeCloud.geometry.attributes.position.needsUpdate = true;
            render();
        } catch (err) {
            console.error("Node position change failed! Reason: " + err);
        }
    },
    this.dispose = function() {
        this.selectionSub.dispose();
        this.positionSub.dispose();
    }
    this.selectionSub = HELPERS.subscribePropertyChange(this, this.nodeModel.selected, "node.selected", this.onNodeSelectedChanged);
    this.positionSub = HELPERS.subscribePropertyChange(this, this.nodeModel.position, "node.position", this.onNodePositionChanged);

};