'use strict';
var VIEW3D = VIEW3D || {}

define(['js/view3d/VIEW3D.PointNode', 'js/view3d/collision/VIEW3D.collision.MouseColliders', 'js/view3d/utils/VIEW3D.utils.ResizableGeometry'], function () {
    // class PiontNodeCloud extends THREE.Points
    // binds to an array of MODEL.Node  
    // and visualizes it as a points
    VIEW3D.PointNodeCloud = function (nodes) {
        // nodes: ko.observableArray of MODEL.Node
        //private members
        var pointNodes = {}; // dictionary of VIEW3D.PointNode it is synchronizzed with buffers in this.geometry
        //static members
        VIEW3D.PointNodeCloud.originalColor = new THREE.Color(0, 0, 0);
        VIEW3D.PointNodeCloud.selectionColor = new THREE.Color(1, 0, 0);
        //init
        this.init = function () {
            for (var i = 0; i < nodes().length; i++) {
                var node = nodes()[i];
                this.onNodeAdded(node, i);
            }
        };

        var attributes = {
            position: {
                type: Float32Array,
                size: 3
            },
            color: {
                type: Float32Array,
                size: 3
            }
        };
        
        var geometry = new VIEW3D.utils.ResizableNonIndexedGeometry(attributes, 1000);
        var material = new THREE.PointsMaterial({
            size: CORE.options.nodeSize,
            vertexColors: THREE.VertexColors,
            sizeAttenuation: false
        });

        THREE.Points.call(this, geometry, material);

        this.frustumCulled = false;
        this.visible = false;
        var collider = new VIEW3D.collision.CollidersContainer();
        this.getCollider = function () {
            return collider;
        };

        this.onNodeAdded = function (nodeModel) {
            //ad geometry
            var index = this.geometry.verticesInUse;
            this.geometry.increaseSize(1);
            var pos = nodeModel.position();
            var color = VIEW3D.PointNodeCloud.originalColor.clone();
            this.geometry.attributes.position.setXYZ(index, pos.x, pos.y, pos.z);
            this.geometry.attributes.color.setXYZ(index, color.r, color.g, color.b);

            //add node events handler
            var pointNode = new VIEW3D.PointNode(this, nodeModel);
            pointNode.index = index;
            pointNodes[nodeModel.id] = pointNode;

            //add collider
            collider.addCollider(pointNode.getCollider());
            this.visible = true;
            render();
        }

        this.onNodeRemoved = function (node) {
            var pointNode = pointNodes[node.id];
            this.geometry.splice(pointNode.index, 1);
            
            collider.removeCollider(pointNode.getCollider());
            delete pointNodes[node.id];

            for (var pnid in pointNodes) {
                var pn = pointNodes[pnid]
                if(pn.index > pointNode.index) {
                    pn.index--;
                }
            }
            
            if (this.geometry.verticesInUse === 0) {
                this.visible = false;
            }
            render();
        }
        
        this.dispose = function () {
            for (var i = 0; i < pointNodes.length; i++) {
                pointNodes[i].dispose();
            }
            this.barsListSub.dispose();
        }

        this.barsListSub = HELPERS.subscribeArrayChange(this, nodes, "model.nodes", this.onNodeAdded, this.onNodeRemoved);
        this.init();
    };
    VIEW3D.PointNodeCloud.prototype = Object.create(THREE.Points.prototype);
});