'use strict';
var VIEW3D = VIEW3D || {};
define(['js/view3d/VIEW3D.MultiMesh'], function() {
    var base32;
    VIEW3D.PushButtons = function(constraints, construction) { //construction to get the node
        var attributes = {
            position: { type: Float32Array, size: 3 },
            color: { type: Float32Array, size: 3 },
            normal: { type: Float32Array, size: 3 }
        };
        var material = new VIEW3D.materials.Metal();
        VIEW3D.MultiMesh.call(this, attributes, material);
        var currentVersion = 0;
        var pushButtons = {};
        //there is no id of push button so we will calculate unique id (NO COLLISIONS!)
        var getPushButtonHash = function(pushButon) {
            var buffer = new ArrayBuffer(4 * ((3 * 3) + 3 + 3));
            var floats = new Float32Array(buffer);
            var i = 0;
            floats[i++] = pushButon.baseZ.x;
            floats[i++] = pushButon.baseZ.y;
            floats[i++] = pushButon.baseZ.z;
            floats[i++] = pushButon.baseX.x;
            floats[i++] = pushButon.baseX.y;
            floats[i++] = pushButon.baseX.z;
            floats[i++] = pushButon.baseY.x;
            floats[i++] = pushButon.baseY.y;
            floats[i++] = pushButon.baseY.z;
            floats[i++] = pushButon.baseZ.x;
            floats[i++] = pushButon.baseZ.y;
            floats[i++] = pushButon.baseZ.z;
            floats[i++] = pushButon.origin.x;
            floats[i++] = pushButon.origin.y;
            floats[i++] = pushButon.origin.z;
            return base32(buffer);
        };
        var getTransforHash = function(transform) {
            var buffer = new ArrayBuffer(4 * (2 + 3));
            var floats = new Float32Array(buffer);
            var i = 0;
            floats[i++] = transform.baseX.x;
            floats[i++] = transform.baseY.y;
            floats[i++] = transform.origin.x;
            floats[i++] = transform.origin.y;
            floats[i++] = transform.origin.z;
            return base32(buffer);
        };
        /*var getArmTransformation = function (armId) {
            var connectorConfig = {
                "1": { x: 0.055, y:0, z:0.0},
                "2": { x: 0.055, y:0, z:0.0},
            }
            var arm = connectorConfig[armId];
            return new THREE.Matrix4().setPosition(arm);
        }*/
        var getConnector = function (elementId, elementType) {
            switch(elementType) {
                case "Node":
                    var node = construction.getNodeById(elementId);
                    return constraints.getConnector(elementId, node.connectorId(), elementType);
            }
            return null;
        };
        var self = this;
        var addPushButtonMeshes = function (hash, elementType, isDouble, templateId, configD) {
            var pushButton = pushButtons[hash];
            if (isEmpty(pushButton)) {
                console.error("Push button not found! Hash: " + hash);
                return;
            }
            var connector = getConnector(Number(pushButton.nodeId), elementType);
            if (pushButton && !isEmpty(connector)) {
                var modelId;
                if (isDouble) {
                    modelId = connector.model3DId_d;
                } else {
                    modelId = connector.model3DId;
                }
                var self = this;
                //MODEL.repository.get3DModelAsync(modelId, function (model) {

                var model = null;
                var template = null;

                var templateRequestFinished = false;
                var modelRequestFinished = false;

                var callback = function () {

                    var config;
                    try {
                        config = JSON.parse(model.config);
                    }
                    catch (ex) {
                        console.error("model.config JSON error!", ex);
                    }
                    if (!isEmpty(config)) {
                        var armConfig = config.Arms[pushButton.armId];
                        if (isEmpty(armConfig)) {
                            console.debug("DEBUG: Missing arm configuration in model: + " + modelId + " arm: " + pushButton.armId);
                            armConfig = [];
                        }
                        for (var i = 0; i < armConfig.PushButtons.length; i++) {
                            var rot;
                            var pushButtonConfg = armConfig.PushButtons[i];
                            var pushButtonPos = new THREE.Vector3(pushButtonConfg.Position.x, pushButtonConfg.Position.y, pushButtonConfg.Position.z);
                            var orientation = VIEW3D.utils.getRotationMatrix4FromBase(pushButton.pushButtonConstraints);
                            var position = VIEW3D.utils.getTranslationMatrix4FromBase(pushButton.pushButtonConstraints);
                            if (isEmpty(pushButtonConfg.Rot)) {
                                rot = 0;
                            } else {
                                rot = pushButtonConfg.Rot;
                            }
                            var rotation = new THREE.Matrix4().makeRotationX(rot);
                            if (!isEmpty(configD)) {
                                pushButtonPos.x -= configD;
                            }
                            pushButtonPos.applyMatrix4(orientation);
                            pushButtonPos.applyMatrix4(position);
                            //var nodeModel = CORE.model.getNodeById(parseInt(pushButton.nodeId)); //TO BE REMOVED
                            //var nodeTransform = new THREE.Matrix4().makeTranslation(nodeModel.position().x, nodeModel.position().y, nodeModel.position().z);
                            //pushButtonPos.applyMatrix4(nodeTransform);
                            orientation.multiply(rotation);
                            orientation.setPosition(pushButtonPos);
                            pushButton.meshes.push(
                                this.addMeshFromTemplate(template, {
                                    transform: orientation,
                                    color: new THREE.Vector3(0.8, 0.6, 0.4)
                                }));
                        }
                    }
                }



                MODEL.repository.get3DModelAsync(modelId, function (model_) {
                    model = model_;
                    modelRequestFinished = true;
                    if (templateRequestFinished && modelRequestFinished) callback.call(self);
                });
                if (templateId !== -1) {
                    VIEW3D.utils.getColorableTemplate(templateId,
                        function(template_) {
                            template = template_;
                            templateRequestFinished = true;
                            if (templateRequestFinished && modelRequestFinished) callback.call(self);
                        });
                }


                //});
            }
        };

        var addModelMesh = function (nodeModel, nodeConstrs, armId) {         
            var pushButton = nodeConstrs.pushButtons[armId];
            var connector = getConnector(nodeModel.id, "Node");
            if (!isEmpty(connector)) {
                if (connector.pushButton.model3DId !== -1) {
                    var hash = getPushButtonHash(pushButton) + nodeModel.connectorId().toString();
                    var existing = pushButtons[hash];
                    var modelId;
                    if (!isEmpty(existing)) {
                        //we have it then just updaqte the version
                        existing.version = currentVersion;
                    } else {
                        pushButton = {
                            version: currentVersion,
                            pushButtonConstraints: pushButton,
                            nodeConstraints: nodeConstrs,
                            nodeId: nodeModel.id, //TO BE REMOVED
                            armId: armId,
                            meshes: []
                        };
                        connector = getConnector(Number(pushButton.nodeId), "Node");
                        if (!isEmpty(connector)) {
                            modelId = connector.pushButton.model3DId;
                            if (modelId !== -1) {
                                pushButtons[hash] = pushButton;
                                addPushButtonMeshes.call(this, hash, "Node", false, modelId);
                            }
                        }
                        //its new one -> add mesh
                    }
                    console.log("push button hash:" + hash);
                    if (nodeConstrs.isDouble) {
                        pushButton = nodeConstrs.pushButtons_d[armId];
                        hash = getPushButtonHash(pushButton) + nodeModel.connectorId().toString();
                        existing = pushButtons[hash];
                        if (!isEmpty(existing)) {
                            //we have it then just updaqte the version
                            existing.version = currentVersion;
                        } else {
                            pushButton = {
                                version: currentVersion,
                                pushButtonConstraints: pushButton,
                                nodeConstraints: nodeConstrs,
                                nodeId: nodeModel.id, //TO BE REMOVED
                                armId: armId,
                                meshes: []
                            };
                            connector = getConnector(Number(pushButton.nodeId), "Node");
                            if (!isEmpty(connector)) {
                                modelId = connector.pushButton.model3DId;
                                if (modelId !== -1) {
                                    pushButtons[hash] = pushButton;
                                    addPushButtonMeshes.call(this, hash, "Node", true, modelId);
                                }
                            }
                            //its new one -> add mesh
                        }
                        console.log("push button hash:" + hash);
                    }
                }
            }
        }

        var updatePushButtons = function(constraintsData) {
            currentVersion++;
            var pushButton, existing, modelId, armId, hash;
            for (var nodeId in constraintsData.nodeConstraints) {
                var nodeConstrs = constraintsData.nodeConstraints[nodeId];
                var nodeModel = construction.getNodeById(Number(nodeId));
                if (nodeModel === null) return;
                //bars push buttons
                for (armId in nodeConstrs.pushButtons) {
                    pushButton = nodeConstrs.pushButtons[armId];
                    addModelMesh.call(self, nodeModel, nodeConstrs, armId);
                }
            }
            var connector;

            //delete push buttons that are not in constraints
            var toDelete = [];
            var meshIndex, mesh;
            for (hash in pushButtons) {
                pushButton = pushButtons[hash];
                if (pushButton.version !== currentVersion) {
                    toDelete.push(hash);
                    for (meshIndex = 0; meshIndex < pushButton.meshes.length; meshIndex++) {
                        mesh = pushButton.meshes[meshIndex];
                        self.removeMesh(mesh);
                    }
                }
            }
            var i;
            for (i = 0; i < toDelete.length; i++) {
                hash = toDelete[i];
                delete pushButtons[hash];
            }
        };
        //var constrainsSub = HELPERS.subscribePropertyChange(this, constraints.data, "constrainsData", function(data) {
        //    updatePushButtons(data);
        //});
        var constrainsSub = HELPERS.subscribePropertyChange(this, constraints.constraintsApplied, "constraintsApplied", function (data) {
            updatePushButtons.call(this, constraints.data());
        });
        this.dispose = function() {
            constrainsSub.dispose();
        };
        if (!isEmpty(constraints.data())) {
            updatePushButtons.call(this, constraints.data());
        }
        this.export = function () {
            var meshes = [];
            var vertices = [];
            var geometry = [];
            var tmp = [];
            var tmpMin = 0;
            var index, mesh, i, k, data, max;
            for (index in pushButtons) {
                if (!isEmpty(pushButtons[index].meshes)) {
                    tmpMin = 0;
                    vertices = [];
                    geometry = [];
                    for (k = 0; k < pushButtons[index].meshes.length; k++) {
                        mesh = this.getMeshData(pushButtons[index].meshes[k]);                   
                        max = (mesh.startVertex + mesh.vartexCount) * 3;
                        for (i = (mesh.startVertex * 3); i < max; i += 3) {
                            vertices.push(this.geometry.attributes.position.array[i]);
                            vertices.push(this.geometry.attributes.position.array[i + 1]);
                            vertices.push(this.geometry.attributes.position.array[i + 2]);
                        }
                        for (i = mesh.startIndex; i < mesh.indexCount + mesh.startIndex;) {
                            tmp = [];
                            tmp.push(this.geometry.index.array[i++] - mesh.startVertex + tmpMin);
                            tmp.push(this.geometry.index.array[i++] - mesh.startVertex + tmpMin);
                            tmp.push(this.geometry.index.array[i++] - mesh.startVertex + tmpMin);
                            geometry.push(tmp);
                        }
                        tmpMin += mesh.vartexCount;
                    }                   
                    data = {
                        vertices: vertices,
                        faces: geometry,
                        type: 4 // Pushbutton
                    };
                    meshes.push(data);
                }
            }
            return meshes;
        };
    };
    VIEW3D.PushButtons.prototype = Object.create(VIEW3D.MultiMesh.prototype);
    var Base32Map = ['A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P', 'Q', 'R', 'S', 'U', 'W', 'X', 'Y', 'Z', 'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h'];
    base32 = function(buffer) {
        var result = '';
        var u8 = new Uint8Array(buffer);
        for (var i = 0; i < u8.length; i++) {
            var byte = u8[i];
            result += Base32Map[byte >> 4];
            result += Base32Map[byte & 0xf];
        }
        return result;
    };
});