﻿"use strict";
var VIEW3D = VIEW3D || {};
define([
        "js/view3d/materials/VIEW3D.materials.Metal", "js/view3d/VIEW3D.MultiMesh", "js/view3d/VIEW3D.InstancedProfile",
        "js/view3d/VIEW3D.RealCaps"
    ],
    function() {
        // class RealBars extends THREE.Object
        // binnds to an array of MODEL.Bar
        // and vizualizes it as a lines
        VIEW3D.RealBars = function(bars, constraints) {
            THREE.Object3D.call(this);
            this.barsData = {};
            this.pofilesGraphcs = {};
            // init
            var collider = new VIEW3D.collision.CollidersContainer();
            this.getCollider = function() {
                return collider;
            };
            this.dispose = function() {
                this.barsObserver.dispose();
            };
            this.onBarSelectedChanged = function(bar) {
                if (bar.profileId() !== 0) {
                    var color = bar.selected() ? VIEW3D.selectionColor : VIEW3D.getProfileColor();
                    var barData = this.barsData[bar.id];
                    if (!isEmpty(barData)) {
                        var instancedProfile = this.pofilesGraphcs[barData.modelId];
                        instancedProfile.setColor(barData, color);
                        render();
                    }
                }
            };
            this.removeBar = function(bar) {
                var barData = this.barsData[bar.id];
                if (!isEmpty(barData)) {
                    // nothing to remove
                    if (!isEmpty(barData.collider)) {
                        collider.removeCollider(barData.collider);
                    }
                    var instancedProfile = this.pofilesGraphcs[barData.modelId];
                    if (!isEmpty(instancedProfile)) {
                        instancedProfile.removeBar(barData, this.barsData, barData.modelId);
                    }
                    delete this.barsData[bar.id];
                }
            };
            this.getProfileContainer = function(modelId, template) {
                var instancedProfile = this.pofilesGraphcs[modelId];
                if (instancedProfile === undefined) {
                    this.pofilesGraphcs[modelId] = instancedProfile = new VIEW3D.InstancedProfile(template);
                    this.add(instancedProfile);
                }
                return instancedProfile;
            };
            this.addBar = function(bar) {
                var modelId = constraints.getBarProfileModelId(bar.id, bar.profileId());
                if (modelId === -1 || modelId === 0) {
                    console.warn("Bar id: " + bar.id + " has no 3D model");
                    return;
                }
                this.barsData[bar.id] = {};
                var self = this;
                VIEW3D.utils.getTemplate(modelId,
                    function(template) {
                        var barData = self.barsData[bar.id];
                        if (isEmpty(barData)) {
                            // looks like this bar is not in scene any more so leave
                            return;
                        }
                        var MemberConstraints = constraints.getMemberConstraints(bar.id);
                        if (isEmpty(MemberConstraints)) {
                            console.warn("Cant find Constraints for bar");
                            return;
                        }
                        barData.modelId = modelId;
                        var instancedProfile = self.getProfileContainer(modelId, template);
                        var transform = VIEW3D.utils.getMatrix4FromBase(MemberConstraints.orientation);
                        var color = bar.selected() ? VIEW3D.selectionColor : VIEW3D.getProfileColor();
                        instancedProfile.addBar(transform, MemberConstraints.length, color, barData);
                        render();
                        // box collider
                        var bbox = template.boundingBox.clone();
                        bbox.max.x = MemberConstraints.length;
                        var boxCollider = new VIEW3D.collision.BoxCollider(bbox, transform);
                        boxCollider.view3D = this;
                        boxCollider.model = bar;
                        barData.collider = boxCollider;
                        collider.addCollider(boxCollider);
                    });
            };
            var _self = this;
            var _caps = new VIEW3D.RealCaps(constraints);
            // add caps to scene
            this.add(_caps);
            this.barsObserver = new VIEW3D.RealBars.BarsObserver(
                bars,
                constraints.data,
                function(bar) {
                    _self.addBar(bar);
                    _caps.addCaps(bar);
                },
                function(bar) {
                    _self.removeBar(bar);
                    _caps.removeCaps(bar);
                },
                function(bar) {
                    _self.removeBar(bar);
                    _self.addBar(bar);
                    _caps.removeCaps(bar);
                    _caps.addCaps(bar);
                },
                function(bar) { _self.onBarSelectedChanged(bar) }
            );
        };
        VIEW3D.RealBars.prototype = Object.create(THREE.Object3D.prototype);
        // will call onChange if something happend to bar, which requires redraw
        // also prevents from redrawing single bar multiple times in single frame
        // observes:
        // -node1, node2
        // -node2.position, node2.position
        // -profileId
        // -bar orientation returen by constraints
        VIEW3D.RealBars.BarObserver = function(bar, constrainsData, onChange, onSelected, missingConstraints) {
            var willBeCalled = false;
            var trigger = function() {
                if (willBeCalled === false) {
                    willBeCalled = true;
                    setTimeout(function() {
                            console.log("Updating Bar " + bar.id);
                            onChange(bar);
                            willBeCalled = false;
                        },
                        1);
                }
            };
            var triggerOnConstraints = missingConstraints;
            var reDrawSub = HELPERS.subscribePropertyChange(this,
                CORE.model.bars,
                "CORE.model.bars",
                function() { triggerOnConstraints = true });
            var pos1ChangeSub = HELPERS.subscribePropertyChange(this,
                bar.node1().position,
                "bar.selected",
                function() {
                    triggerOnConstraints = true;
                });
            var pos2ChangeSub = HELPERS.subscribePropertyChange(this,
                bar.node2().position,
                "bar.selected",
                function() {
                    triggerOnConstraints = true;
                });
            var node1ChangeSub = HELPERS.subscribePropertyChange(this,
                bar.node1,
                "bar.node1",
                function() {
                    pos1ChangeSub.dispose();
                    if (!isEmpty(bar.node1())) {
                        pos1ChangeSub = HELPERS.subscribePropertyChange(this,
                            bar.node1().position,
                            "bar.selected",
                            function() {
                                triggerOnConstraints = true;
                            });
                    }
                    trigger();
                });
            var node2ChangeSub = HELPERS.subscribePropertyChange(this,
                bar.node2,
                "bar.node2",
                function() {
                    pos2ChangeSub.dispose();
                    if (!isEmpty(bar.node2())) {
                        pos2ChangeSub = HELPERS.subscribePropertyChange(this,
                            bar.node2().position,
                            "bar.selected",
                            function() {
                                triggerOnConstraints = true;
                            });
                    }
                    trigger();
                });
            var profileChangeSub = HELPERS.subscribePropertyChange(this,
                bar.profileId,
                "bar.profile",
                function() {
                    triggerOnConstraints = true;
                });
            var getCurrentOrientation = function() {
                var data = constrainsData();
                if (!isEmpty(data)) {
                    var MemberConstraints = data.MemberConstraints[bar.id];
                    if (!isEmpty(MemberConstraints)) {
                        return MemberConstraints.orientation;
                    }
                }
                return null;
            };
            var getCurrentLength = function() {
                var data = constrainsData();
                if (!isEmpty(data)) {
                    var MemberConstraints = data.MemberConstraints[bar.id];
                    if (!isEmpty(MemberConstraints)) {
                        return MemberConstraints.length;
                    }
                }
                return null;
            };
            var lastOrientation = getCurrentOrientation();
            var lastLength = getCurrentLength();
            var constrainsSub = HELPERS.subscribePropertyChange(this,
                constrainsData,
                "constrainsData",
                function() {
                    var currentLength = getCurrentLength();
                    var currentOrientation = getCurrentOrientation();
                    if (triggerOnConstraints) {
                        triggerOnConstraints = false;
                        lastOrientation = currentOrientation;
                        lastLength = currentLength;
                        trigger();
                        return;
                    }
                    if (currentOrientation === lastOrientation && currentLength === lastLength) {
                        return;
                    }
                    if (!isEmpty(currentOrientation) && !isEmpty(lastOrientation)) {
                        if (currentOrientation.baseX.x === lastOrientation.baseX.x &&
                            currentOrientation.baseX.y === lastOrientation.baseX.y &&
                            currentOrientation.baseX.z === lastOrientation.baseX.z &&
                            currentOrientation.baseY.x === lastOrientation.baseY.x &&
                            currentOrientation.baseY.y === lastOrientation.baseY.y &&
                            currentOrientation.baseY.z === lastOrientation.baseY.z &&
                            currentOrientation.baseZ.x === lastOrientation.baseZ.x &&
                            currentOrientation.baseZ.y === lastOrientation.baseZ.y &&
                            currentOrientation.baseZ.z === lastOrientation.baseZ.z &&
                            currentOrientation.origin.x === lastOrientation.origin.x &&
                            currentOrientation.origin.y === lastOrientation.origin.y &&
                            currentOrientation.origin.z === lastOrientation.origin.z &&
                            currentLength === lastLength) {
                            return;
                        }
                    }
                    triggerOnConstraints = false;
                    lastOrientation = currentOrientation;
                    lastLength = currentLength;
                    trigger();
                });
            var selectionSub = HELPERS.subscribePropertyChange(this,
                bar.selected,
                "bar.selected",
                function() { onSelected(bar); });
            this.dispose = function() {
                reDrawSub.dispose();
                pos1ChangeSub.dispose();
                pos2ChangeSub.dispose();
                node1ChangeSub.dispose();
                node2ChangeSub.dispose();
                profileChangeSub.dispose();
                constrainsSub.dispose();
                selectionSub.dispose();
            };
        };
        VIEW3D.RealBars.BarsObserver = function(bars, constrainsData, addBar, removeBar, braChange, selectionChange) {
            var barsData = {};
            this.onBarAdded = function(bar) {
                var missingConstraints = isEmpty(constrainsData() && constrainsData().MemberConstraints[bar.id]);
                // TODO nodeObserver
                if (!missingConstraints) {
                    addBar(bar);
                }
                barsData[bar.id] = new VIEW3D.RealBars
                    .BarObserver(bar, constrainsData, braChange, selectionChange, missingConstraints);
            };
            this.onBarRemoved = function(bar) {
                removeBar(bar);
                barsData[bar.id].dispose();
                delete barsData[bar.id];
            };
            this.barsListSub = HELPERS.subscribeArrayChange(this, bars, "bars", this.onBarAdded, this.onBarRemoved);
            for (var i = 0; i < bars().length; i++) {
                var bar = bars()[i];
                this.onBarAdded(bar, i);
            }
            this.dispose = function() {
                this.barsListSub.dispose();
                for (var barId in barsData) {
                    barsData[barId].dispose();
                }
                barsData = {};
            };
        };
    });