'use strict';
var VIEW3D = VIEW3D || {}

define(['js/view3d/VIEW3D.MultiMesh'], function() {

    VIEW3D.RealCaps = function(constraints) {
        var attributes = {
            position: {
                type: Float32Array,
                size: 3
            },
            color: {
                type: Float32Array,
                size: 3
            },
            normal: {
                type: Float32Array,
                size: 3
            }
        };
        // texture
        var material = new VIEW3D.materials.Metal();
        VIEW3D.MultiMesh.call(this, attributes, material);

        this.capsData = {}

        this.addCaps = function(bar) {
            this.capsData[bar.id] = {};
            var self = this;
            var profile = constraints.getBarProfile(bar.id, bar.profileId());
            if (profile.model3DIdCap !== 0 && profile !== -1) {
                VIEW3D.utils.getTemplate(profile.model3DIdCap, function(template) {
                    var capData = self.capsData[bar.id];
                    if (isEmpty(capData)) {
                        // looks like this bar is not in scene any more so leave
                        return;
                    }
                    var MemberConstraints = constraints.getMemberConstraints(bar.id);
                    if (isEmpty(MemberConstraints)) {
                        console.warn("Cant find Constraints for bar");
                        return;
                    }
                    var transform;
                    if (profile.isCapNode1) {
                        transform = VIEW3D.utils.getMatrix4FromBase(MemberConstraints.orientation);
                        transform = new THREE.Matrix4().multiplyMatrices(transform, new THREE.Matrix4().makeRotationZ(Math.PI));
                        capData.mesh1Id = self.addMeshFromTemplate(template, transform);
                    }
                    if (profile.isCapNode2) {
                        transform = VIEW3D.utils.getMatrix4FromBase(MemberConstraints.orientation);
                        transform = new THREE.Matrix4().multiplyMatrices(transform, new THREE.Matrix4().makeTranslation(MemberConstraints.length, 0, 0));
                        capData.mesh2Id = self.addMeshFromTemplate(template, transform);
                    }
                }, function(er){
                    console.warn("Cannot read cap model: " + profile.model3DIdCap + " for bar: " + bar.is + " with ptofile: " + profile.model3DId);
                });
            }
        }
        this.removeCaps = function(bar) {
            var capData = this.capsData[bar.id];
            if (isEmpty(capData)) {
                // nothing to remove
                return;
            }
            if (!isEmpty(capData.mesh1Id)) {
                this.removeMesh(capData.mesh1Id);
            }
            if (!isEmpty(capData.mesh2Id)) {
                this.removeMesh(capData.mesh2Id);
            }
            delete this.capsData[bar.id];
        }
        this.export = function () {
            var meshes = [];
            for (var index in this.capsData) {
                var data;
                var mesh, i, vertices, geometry, max, tmp;
                if (!isEmpty(this.capsData[index].mesh1Id)) {
                    mesh = this.getMeshData(this.capsData[index].mesh1Id);
                    vertices = [];
                    geometry = [];
                    max = (mesh.startVertex + mesh.vartexCount) * 3;
                    for (i = (mesh.startVertex * 3); i < max; i += 3) {
                        vertices.push(this.geometry.attributes.position.array[i]);
                        vertices.push(this.geometry.attributes.position.array[i + 1]);
                        vertices.push(this.geometry.attributes.position.array[i + 2]);
                    }
                    for (i = mesh.startIndex; i < mesh.indexCount + mesh.startIndex;) {
                        tmp = [];
                        tmp.push(this.geometry.index.array[i++] - mesh.startVertex);
                        tmp.push(this.geometry.index.array[i++] - mesh.startVertex);
                        tmp.push(this.geometry.index.array[i++] - mesh.startVertex);
                        geometry.push(tmp);
                    }
                    data = {
                        vertices: vertices,
                        faces: geometry,
                        type: 8 // Bar Cap
                    };
                    meshes.push(data);
                }   
                
                if (!isEmpty(this.capsData[index].mesh2Id)) {
                    mesh = this.getMeshData(this.capsData[index].mesh2Id);
                    vertices = [];
                    geometry = [];
                    max = (mesh.startVertex + mesh.vartexCount) * 3;
                    for (i = (mesh.startVertex * 3); i < max; i += 3) {
                        vertices.push(this.geometry.attributes.position.array[i]);
                        vertices.push(this.geometry.attributes.position.array[i + 1]);
                        vertices.push(this.geometry.attributes.position.array[i + 2]);
                    }
                    for (i = mesh.startIndex; i < mesh.indexCount + mesh.startIndex;) {
                        tmp = [];
                        tmp.push(this.geometry.index.array[i++] - mesh.startVertex);
                        tmp.push(this.geometry.index.array[i++] - mesh.startVertex);
                        tmp.push(this.geometry.index.array[i++] - mesh.startVertex);
                        geometry.push(tmp);
                    }
                    data = {
                        vertices: vertices,
                        faces: geometry,
                        type: 8 // Bar Cap
                    };
                    meshes.push(data);
                }
            }
            return meshes;
        };
    }
    VIEW3D.RealCaps.prototype = Object.create(VIEW3D.MultiMesh.prototype);
});