﻿"use strict";
var VIEW3D = VIEW3D || {};
define(["js/view3d/VIEW3D.MultiMesh"],
    function() {
        // class RealNodes extends THREE.Object
        // binnds to an array of MODEL.Bar
        // and vizualizes it as a lines
        VIEW3D.RealNodes = function(nodes, constraints) {
            var attributes = {
                position: { type: Float32Array, size: 3 },
                color: { type: Float32Array, size: 3 },
                normal: { type: Float32Array, size: 3 }
            };
            var material = new VIEW3D.materials.Metal();
            VIEW3D.MultiMesh.call(this, attributes, material);
            this.mesh = null;
            this.nodesData = {};
            this.templates = {};
            // init
            var collider = new VIEW3D.collision.CollidersContainer();
            this.getCollider = function() {
                return collider;
            };
            this.dispose = function() {
                for (var id in this.nodesData) {
                    if (!isEmpty(this.nodesData[id].mesh)) {
                        scene.remove(this.nodesData[id].mesh);
                        this.nodesData[id].mesh = null;
                    }
                    if (!isEmpty(this.nodesData[id].labelObject)) {
                        scene.remove(this.nodesData[id].labelObject);
                        this.nodesData[id].mesh = null;
                    }
                }
                render();
                this.nodesObserver.dispose();
            };
            this.onNodeSelectedChanged = function(node) {
                var color;
                color = node.selected() ? VIEW3D.selectionColor : VIEW3D.getConnectorColor();
                var meshId = this.nodesData[node.id].meshId;
                if (!isEmpty(meshId)) {
                    this.fillAttributeXYZ(meshId, "color", color.x, color.y, color.z);
                }
                meshId = this.nodesData[node.id].secondaryMeshId;
                if (!isEmpty(meshId)) {
                    this.fillAttributeXYZ(meshId, "color", color.x, color.y, color.z);
                }
                meshId = this.nodesData[node.id].mesh;
                if (!isEmpty(meshId)) {
                    if (node.selected()) {
                        meshId.material.uniforms.diffuseColor.value = VIEW3D.selectionColor;
                    } else {
                        meshId.material.uniforms.diffuseColor.value = new THREE.Vector3(0.2, 0.2, 0.2);
                    }
                }
                render();
            };
            this.removeNode = function(node) {
                var nodeData = this.nodesData[node.id];
                if (!isEmpty(nodeData)) {
                    if (!isEmpty(nodeData.collider)) {
                        collider.removeCollider(nodeData.collider);
                    }
                    if (!isEmpty(nodeData.mesh)) {
                        scene.remove(nodeData.mesh);
                    }
                    if (!isEmpty(nodeData.meshId)) {
                        this.removeMesh(nodeData.meshId);
                    }
                    if (!isEmpty(nodeData.secondaryMeshId)) {
                        this.removeMesh(nodeData.secondaryMeshId);
                    }
                    if (!isEmpty(nodeData.labelObject)) {
                        scene.remove(nodeData.labelObject);
                    }
                }
            };
            this.createEmptyNode = function(node) {
                var pt = node.position();
                var transform = new THREE.Matrix4();
                transform.makeTranslation(pt.x, pt.y, pt.z);
                var boxCollider = new VIEW3D.collision.GlobalPointCollider(HELPERS.toVector3(node.position()), 30);
                boxCollider.view3D = this;
                boxCollider.model = node;
                collider.addCollider(boxCollider);
                var mesh = null;
                var skip = false;
                if (!isEmpty(this.nodesData[node.id])) {
                    scene.remove(this.nodesData[node.id].mesh);
                }
                if (node.createsBars().length === 0 && !skip) {
                    var geometry = new THREE.SphereGeometry(0.02, 16, 16);
                    var material = new VIEW3D.materials.Metal({ diffuseColor: new THREE.Vector3(0.2, 0.2, 0.2) });
                    mesh = new THREE.Mesh(geometry, material);
                    mesh.position.set(node.position().x, node.position().y, node.position().z);
                    scene.add(mesh);
                    render();
                }
                this.nodesData[node.id] = {
                    // model
                    mesh: mesh,
                    collider: boxCollider
                };
            };
            var getMatrix4FromBase = function(base) {
                var mat = new THREE.Matrix4();
                mat.set(
                    base.baseX.x,
                    base.baseY.x,
                    base.baseZ.x,
                    base.origin.x,
                    base.baseX.y,
                    base.baseY.y,
                    base.baseZ.y,
                    base.origin.y,
                    base.baseX.z,
                    base.baseY.z,
                    base.baseZ.z,
                    base.origin.z,
                    0,
                    0,
                    0,
                    1);
                return mat;
            };
            this.addMesh = function(node, modelId, isSecond) {
                var self = this;
                VIEW3D.utils.getColorableTemplate(modelId,
                    function(template) {
                        var nodeData = self.nodesData[node.id];
                        if (isEmpty(nodeData)) {
                            // looks like this node is not in scene any more so leave
                            return;
                        }
                        var nodeConstraints = constraints.getNodeConstraints(node.id);
                        if (isEmpty(nodeConstraints)) {
                            console.warn("Cant find Constraints for node");
                            return;
                        }
                        // TRANSFORMATION
                        var base = isSecond ? nodeConstraints.orientation_d : nodeConstraints.orientation;
                        var matOrientation;
                        if (!isEmpty(base)) {
                            matOrientation = getMatrix4FromBase(base);
                        } else {
                            console.warn("null orientation for node " + node.name());
                            matOrientation = THREE.Matrix4();
                        }
                        var color = node.selected() ? VIEW3D.selectionColor : VIEW3D.getConnectorColor();
                        var meshId = self.addMeshFromTemplate(template,
                        {
                            transform: matOrientation,
                            color: color
                        });
                        nodeData.collider.addCollider(new VIEW3D.collision
                            .BoxCollider(template.boundingBox, matOrientation));
                        if (!isSecond) {
                            nodeData.meshId = meshId;
                        } else {
                            nodeData.secondaryMeshId = meshId;

                        }
                    });
            };
            this.addNode = function(node) {
                var models = constraints.getConnectorModelIds(node.id, node.connectorId());
                if (models.length === 0) {
                    this.createEmptyNode(node);
                } else {
                    var nodeCollider = new VIEW3D.collision.DisjunctionCollider();
                    nodeCollider.model = node;
                    collider.addCollider(nodeCollider);
                    this.nodesData[node.id] = {
                        collider: nodeCollider
                    };
                    if (models.length > 0) {
                        if(models[0] >= 0){
                            this.addMesh(node, models[0], false);
                        }
                    }
                    if (models.length > 1) {
                        if(models[1] >= 0){
                            this.addMesh(node, models[1], true);
                        }
                    }
                }
                // this.addLabel(node);
            };

            this.addLabel = function(node) {
                var scale = 1;
                var nodePos = node.position();
                var label = {
                    pos: new THREE.Vector3(nodePos.x, nodePos.y,nodePos.z),
                    dir: new THREE.Vector3(0, 0, 0.06),
                    text: String(node.id)
                };

                var structures = VIEW3D.utils.Labeling.createStructures(new THREE.Vector4(0.2, 0.3, 1.0, 1));

                VIEW3D.utils.Labeling.createLabel(structures, label, scale);
                structures.labelsObject.material.depthTest = false;
                structures.labelsObject.material.uniforms.scale.value = 2;

                scene.add(structures.labelsObject);
                var nodeData = this.nodesData[node.id];
                nodeData.labelObject = structures.labelsObject;
            }

            var _self = this;
            this.nodesObserver = new VIEW3D.RealNodes.NodesObserver(
                nodes,
                constraints.data,
                function(node) { _self.addNode(node); },
                function(node) { _self.removeNode(node); },
                function(node) {
                    _self.removeNode(node);
                    _self.addNode(node);
                },
                function(node) { _self.onNodeSelectedChanged(node) }
            );
            this.export = function() {
                var meshes = [];
                for (var index in this.nodesData) {
                    var mesh, i;
                    if (!isEmpty(this.nodesData[index].meshId)) {
                        mesh = this.getMeshData(this.nodesData[index].meshId);
                        var type;
                        var node = CORE.model.getNodeById(Number(index));
                        if (!isEmpty(node)) {
                            var support = CORE.model.getSupportById(node.supportId());
                            type = 1; // Connector
                            var vertices = [];
                            var geometry = [];
                            var max = (mesh.startVertex + mesh.vartexCount) * 3;
                            for (i = (mesh.startVertex * 3); i < max; i += 3) {
                                vertices.push(this.geometry.attributes.position.array[i]);
                                vertices.push(this.geometry.attributes.position.array[i + 1]);
                                vertices.push(this.geometry.attributes.position.array[i + 2]);
                            }
                            for (i = mesh.startIndex; i < mesh.indexCount + mesh.startIndex;) {
                                var tmp = [];
                                tmp.push(this.geometry.index.array[i++] - mesh.startVertex);
                                tmp.push(this.geometry.index.array[i++] - mesh.startVertex);
                                tmp.push(this.geometry.index.array[i++] - mesh.startVertex);
                                geometry.push(tmp);
                            }
                            var data = {
                                vertices: vertices,
                                faces: geometry,
                                type: type
                            };
                            meshes.push(data);
                        }
                    }
                    if (!isEmpty(this.nodesData[index].secondaryMeshId)) {
                        mesh = this.getMeshData(this.nodesData[index].secondaryMeshId);
                        var type;
                        var node = CORE.model.getNodeById(Number(index));
                        if (!isEmpty(node)) {
                            var support = CORE.model.getSupportById(node.supportId());
                            type = 1; // Connector
                            var vertices = [];
                            var geometry = [];
                            var max = (mesh.startVertex + mesh.vartexCount) * 3;
                            for (i = (mesh.startVertex * 3) ; i < max; i += 3) {
                                vertices.push(this.geometry.attributes.position.array[i]);
                                vertices.push(this.geometry.attributes.position.array[i + 1]);
                                vertices.push(this.geometry.attributes.position.array[i + 2]);
                            }
                            for (i = mesh.startIndex; i < mesh.indexCount + mesh.startIndex;) {
                                var tmp = [];
                                tmp.push(this.geometry.index.array[i++] - mesh.startVertex);
                                tmp.push(this.geometry.index.array[i++] - mesh.startVertex);
                                tmp.push(this.geometry.index.array[i++] - mesh.startVertex);
                                geometry.push(tmp);
                            }
                            var data = {
                                vertices: vertices,
                                faces: geometry,
                                type: type
                            };
                            meshes.push(data);
                        }
                    }
                }
                return meshes;
            };
        };
        VIEW3D.RealNodes.prototype = Object.create(VIEW3D.MultiMesh.prototype);
        // will call onChange if something happend to node, which requires redraw
        // also prevents from redrawing single node multiple times in single frame
        // observs:
        // -position
        // -connectorId
        // -node orientation returen by constraints
        // -selection
        // will not trigger if constraints are mot ready
        VIEW3D.RealNodes.NodeObserver = function(node, constrainsData, onChange, onSelChange, missingConstraints) {
            var willBeCalled = false;
            var trigger = function() {
                if (willBeCalled === false) {
                    willBeCalled = true;
                    setTimeout(function() {
                            console.log("Updating Node " + node.id);
                            onChange(node);
                            willBeCalled = false;
                        },
                        1);
                }
            };
            this.triggerOnConstraints = missingConstraints;
            var reDrawSub = HELPERS.subscribePropertyChange(this,
                CORE.model.nodes,
                "CORE.model.nodes",
                function() { this.triggerOnConstraints = true }, this);
            var selChangeSub = HELPERS.subscribePropertyChange(this,
                node.selected,
                "node.selected",
                function() { onSelChange(node); });
            var posChangeSub = HELPERS.subscribePropertyChange(this,
                node.position,
                "node.position",
                function() { this.triggerOnConstraints = true }, this);
            var connectorChangeSub = HELPERS
                .subscribePropertyChange(this,
                    node.connectorId,
                    "node.connectorId",
                    function() { this.triggerOnConstraints = true }, this);
            var getCurrentOrientation = function() {
                var data = constrainsData();
                if (!isEmpty(data)) {
                    var MemberConstraints = data.nodeConstraints[node.id];
                    if (!isEmpty(MemberConstraints)) {
                        return MemberConstraints.orientation;
                    }
                }
                return null;
            };
            var getCurrentOrientationDouble = function() {
                var data = constrainsData();
                if (!isEmpty(data)) {
                    var MemberConstraints = data.nodeConstraints[node.id];
                    if (!isEmpty(MemberConstraints)) {
                        return MemberConstraints.orientation_d;
                    }
                }
                return null;
            };
            var lastOrientation = getCurrentOrientation();
            var lastOrientationDouble;
            var isDouble;
            if (!isEmpty(constrainsData() && constrainsData().nodeConstraints[node.id])) {
                isDouble = constrainsData().nodeConstraints[node.id].isDouble;
                if (isDouble) {
                    lastOrientationDouble = getCurrentOrientationDouble();
                }
            }
            var constrainsSub = HELPERS.subscribePropertyChange(this,
                constrainsData,
                "constrainsData",
                function() {
                    if (this.triggerOnConstraints) {
                        this.triggerOnConstraints = false;
                        trigger();
                        return;
                    }
                    var currentOrientation = getCurrentOrientation();
                    if (isDouble) {
                        var currentOrientationDouble = getCurrentOrientationDouble();
                        if (currentOrientation === lastOrientation && currentOrientationDouble === lastOrientationDouble
                        ) {
                            return;
                        }
                        if (!isEmpty(currentOrientation) &&
                            !isEmpty(lastOrientation) &&
                            !isEmpty(currentOrientationDouble) &&
                            !isEmpty(lastOrientationDouble)) {
                            if (currentOrientation.baseX.x === lastOrientation.baseX.x &&
                                currentOrientation.baseX.y === lastOrientation.baseX.y &&
                                currentOrientation.baseX.z === lastOrientation.baseX.z &&
                                currentOrientation.baseY.x === lastOrientation.baseY.x &&
                                currentOrientation.baseY.y === lastOrientation.baseY.y &&
                                currentOrientation.baseY.z === lastOrientation.baseY.z &&
                                currentOrientation.baseZ.x === lastOrientation.baseZ.x &&
                                currentOrientation.baseZ.y === lastOrientation.baseZ.y &&
                                currentOrientation.baseZ.z === lastOrientation.baseZ.z &&
                                currentOrientationDouble.baseX.x === lastOrientationDouble.baseX.x &&
                                currentOrientationDouble.baseX.y === lastOrientationDouble.baseX.y &&
                                currentOrientationDouble.baseX.z === lastOrientationDouble.baseX.z &&
                                currentOrientationDouble.baseY.x === lastOrientationDouble.baseY.x &&
                                currentOrientationDouble.baseY.y === lastOrientationDouble.baseY.y &&
                                currentOrientationDouble.baseY.z === lastOrientationDouble.baseY.z &&
                                currentOrientationDouble.baseZ.x === lastOrientationDouble.baseZ.x &&
                                currentOrientationDouble.baseZ.y === lastOrientationDouble.baseZ.y &&
                                currentOrientationDouble.baseZ.z === lastOrientationDouble.baseZ.z &&
                                currentOrientation.origin.x === lastOrientationDouble.origin.x &&
                                currentOrientation.origin.y === lastOrientationDouble.origin.y &&
                                currentOrientation.origin.z === lastOrientationDouble.origin.z) {
                                return;
                            }
                        }
                        lastOrientation = currentOrientation;
                        lastOrientationDouble = currentOrientationDouble;
                    } else {
                        if (currentOrientation === lastOrientation) {
                            return;
                        }
                        if (!isEmpty(currentOrientation) && !isEmpty(lastOrientation)) {
                            if (currentOrientation.baseX.x === lastOrientation.baseX.x &&
                                currentOrientation.baseX.y === lastOrientation.baseX.y &&
                                currentOrientation.baseX.z === lastOrientation.baseX.z &&
                                currentOrientation.baseY.x === lastOrientation.baseY.x &&
                                currentOrientation.baseY.y === lastOrientation.baseY.y &&
                                currentOrientation.baseY.z === lastOrientation.baseY.z &&
                                currentOrientation.baseZ.x === lastOrientation.baseZ.x &&
                                currentOrientation.baseZ.y === lastOrientation.baseZ.y &&
                                currentOrientation.baseZ.z === lastOrientation.baseZ.z &&
                                currentOrientation.origin.x === lastOrientation.origin.x &&
                                currentOrientation.origin.y === lastOrientation.origin.y &&
                                currentOrientation.origin.z === lastOrientation.origin.z) {
                                return;
                            }
                        }
                        lastOrientation = currentOrientation;
                    }
                    trigger();
                }, this);
            this.dispose = function() {
                reDrawSub.dispose();
                selChangeSub.dispose();
                posChangeSub.dispose();
                connectorChangeSub.dispose();
                constrainsSub.dispose();
            };
        };
        VIEW3D.RealNodes.NodesObserver = function(nodes,
            constrainsData,
            addNode,
            removeNode,
            nodeChange,
            selectionChange) {
            var nodesData = {};
            this.onNodeAdded = function(node) {
                var missingConstraints = isEmpty(constrainsData() && constrainsData().nodeConstraints[node.id]);
                // TODO nodeObserver
                if (!missingConstraints) {
                    addNode(node);
                }
                nodesData[node.id] = new VIEW3D.RealNodes
                    .NodeObserver(node, constrainsData, nodeChange, selectionChange, true);
            };
            this.onNodeRemoved = function(node) {
                if (!isEmpty(nodesData[node.id])) {
                    removeNode(node);
                    nodesData[node.id].dispose();
                    delete nodesData[node.id];
                }
            };
            this.barsListSub = HELPERS.subscribeArrayChange(this, nodes, "nodes", this.onNodeAdded, this.onNodeRemoved);
            for (var i = 0; i < nodes().length; i++) {
                var node = nodes()[i];
                this.onNodeAdded(node, i);
            }
            this.dispose = function() {
                this.barsListSub.dispose();
                for (var nodeId in nodesData) {
                    nodesData[nodeId].dispose();
                }
                nodesData = {};
            };
        };
    });