﻿"use strict";
var VIEW3D = VIEW3D || {};
define(["js/view3d/VIEW3D.MultiMesh", "js/view3d/VIEW3D.MeshTemplate"],
    function() {
        VIEW3D.RealSupportsNode = function(nodes, constraints) {
            THREE.Object3D.call(this);
            var instances = {};
            this.addSupportInstance = function(node) {
                instances[node.id] = {
                    transform: {}
                };
                var modelId = constraints.getNodeSupportModelId(node);
                if (modelId === -1) {
                    console
                    .warn("cannot find support for node: nodeId: " +
                        node.id +
                        " suppostId: " +
                        node
                        .supportId());
                    return;
                }
                getSupportTemplate(modelId,
                    function(template) {
                        var supportData = instances[node.id];
                        if (supportData) {
                            var nodeConstraints = constraints.getNodeConstraints(node.id);
                            if (isEmpty(nodeConstraints)) {
                                console.warn("Cant find Constraints for node");
                                return;
                            }
                            supportData.transform = VIEW3D.utils.getMatrix4FromBase(nodeConstraints.orientation);
                            if (!isEmpty(nodeConstraints.orientation_d)) {
                                var o1 = nodeConstraints.orientation_d.origin;
                                var o2 = nodeConstraints.orientation.origin;
                                supportData.transform.setPosition({
                                    x: (o1.x + o2.x) / 2,
                                    y: (o1.y + o2.y) / 2,
                                    z: (o1.z + o2.z) / 2
                                });
                            }
                            supportData.material = template.config.material;
                            var params;
                            if (template instanceof ScalableTemplate) {
                                params = {
                                    transform: supportData.transform,
                                    size: node.supportB()
                                };
                            } else {
                                params = supportData.transform;
                            }
                            supportData.meshId = this.multimeshes[supportData.material]
                                .addMeshFromTemplate(template, params);
                        }
                    }.bind(this));
            };
            this.removeSupportInstance = function(node) {
                var data = instances[node.id];
                if (!isEmpty(data)) {
                    delete instances[node.id];
                    if (data.meshId != undefined) {
                        this.multimeshes[data.material].removeMesh(data.meshId);
                    }
                }
            };
            //-----------------------------------------------------
            // Init
            //-----------------------------------------------------
            this.updateSupports = function() {
                for (var i = 0; i < nodes().length; i++) {
                    var node = nodes()[i];
                    var supportId = node.supportId();
                    var supportData = instances[node.id];
                    if (supportId === 0 && supportData !== undefined) {
                        this.removeSupportInstance(node);
                    }
                    if (supportId !== 0) {
                        if (supportData === undefined) {
                            this.addSupportInstance(node);
                        } else {
                            var nodeConstraints = constraints.data().nodeConstraints[node.id];
                            if ((!isEmpty(supportData.transform.elements) && !VIEW3D.utils.transtomAndMatrixAreEqual(nodeConstraints.orientation, supportData.transform)) ||
                                supportData.supportId !== supportId) {
                                //need to redraw
                                this.removeSupportInstance(node);
                                this.addSupportInstance(node);
                            }
                        }
                    }
                }
            };
            // ---------------------------------------------------------------
            // Graphics configuration
            // ---------------------------------------------------------------
            //var texture = CORE.res.texture["support"];
            this.multimeshes = {
                textured: new VIEW3D.MultiMesh({
                        position: { type: Float32Array, size: 3 },
                        uv: { type: Float32Array, size: 2 }
                    },
                    new THREE.MeshBasicMaterial({
                        //map: texture
                    })),
                metal: new VIEW3D.MultiMesh({
                        position: { type: Float32Array, size: 3 },
                        normal: { type: Float32Array, size: 3 }
                    },
                    new VIEW3D.materials.Metal({
                        diffuseColor: VIEW3D.getProfileColor()
                    })
                )
            };
            this.add(this.multimeshes.textured);
            this.add(this.multimeshes.metal);
            this.updateSupports();
            var subRemoveNode = HELPERS.subscribeArrayChange(this,
                nodes,
                "nodes",
                function() {},
                this.removeSupportInstance);
            var subConstraints = HELPERS
                .subscribePropertyChange(this, constraints.data, "constraints.data", this.updateSupports);
            this.dispose = function() {
                subRemoveNode.dispose();
                subConstraints.dispose();
            };
        };
        VIEW3D.RealSupportsNode.prototype = Object.create(THREE.Object3D.prototype);
        var getSupportTemplate = function(modelId, callback) {
            var template = VIEW3D.utils.templates[modelId];
            if (!isEmpty(template)) {
                callback(template);
                return;
            }
            MODEL.repository.get3DModelAsync(modelId,
                function(model) {
                    var templateGeometry = MODEL.utils.Proto.getBufferGeometryFromBase64(model.data, modelId);
                    var verticesCount = templateGeometry.attributes.position.array.length / 3;
                    templateGeometry.addAttribute("color",
                        new THREE.BufferAttribute(new Float32Array(verticesCount * 3), 3));
                    var config = JSON.parse(model.config);
                    // scalable support
                    if (!isEmpty(config.S)) {
                        template = new ScalableTemplate(templateGeometry, config);
                    } else {
                        template = new VIEW3D.MeshTemplate();
                        template.setGeometry(templateGeometry);
                        template.config = config;
                    }
                    VIEW3D.utils.templates[modelId] = template;
                    callback(template);
                });
        };
        var ScalableTemplate = function(geometry, config) {
            this.geometry = geometry;
            this.config = config;
        };
        ScalableTemplate.prototype = {
            getIndicesCount: function() {
                return this.geometry.index.count;
            },
            getVerticesCount: function() {
                return this.geometry.attributes.position.count;
            },
            writeAttributes: function(attrs, indices, sv, si, params) {
                // index
                var geometry = this.geometry;
                var i;
                for (i = 0; i < geometry.getIndex().array.length; i++) {
                    indices.array[si + i] = geometry.getIndex().array[i] + sv;
                }
                var posSrc = geometry.attributes.position.array;
                var posDest = attrs.position.array;
                var copyNormals = !isEmpty(attrs.normal) && !isEmpty(geometry.attributes.normal);
                var normalDest = copyNormals ? attrs.normal.array : null;
                var normalSrc = copyNormals ? geometry.attributes.normal.array : null;
                var pos = new THREE.Vector3();
                var d = (params.size - this.config.S) * 0.5;
                var e = params.transform.elements;
                for (i = 0; i < posSrc.length / 3; i++) {
                    // position
                    pos.fromArray(posSrc, i * 3);
                    var x = posSrc[i * 3 + 0];
                    var y = posSrc[i * 3 + 1];
                    var z = posSrc[i * 3 + 2];
                    if (x > this.config.T) {
                        x += d;
                    } else if (x < -this.config.T) {
                        x -= d;
                    }
                    posDest[(sv + i) * 3 + 0] = e[0] * x + e[4] * y + e[8] * z + e[12];
                    posDest[(sv + i) * 3 + 1] = e[1] * x + e[5] * y + e[9] * z + e[13];
                    posDest[(sv + i) * 3 + 2] = e[2] * x + e[6] * y + e[10] * z + e[14];
                    x = normalSrc[i * 3 + 0];
                    y = normalSrc[i * 3 + 1];
                    z = normalSrc[i * 3 + 2];
                    normalDest[(sv + i) * 3 + 0] = e[0] * x + e[4] * y + e[8] * z;
                    normalDest[(sv + i) * 3 + 1] = e[1] * x + e[5] * y + e[9] * z;
                    normalDest[(sv + i) * 3 + 2] = e[2] * x + e[6] * y + e[10] * z;
                }
            }
        };
    });