﻿'use strict';
var VIEW3D = VIEW3D || {};

define(['js/view3d/VIEW3D.LineBars', 'js/view3d/VIEW3D.BarsResults'], function () {
    // class LineBars extends THREE.Object3D
    // binds to MODEL.Construction
    // it is a root Object3D of a construction 
    VIEW3D.ResultsWindow = VIEW3D.ResultsWindow || {};
    VIEW3D.ResultsWindow.Results = function (model) {
        THREE.Object3D.call(this);
        this.nodesObject3D = new VIEW3D.PointNodeCloud(model.nodes);
        this.barsObject3D = new VIEW3D.LineBars(model.bars, true); // true - isResultsWindow
        this.barsResults = new VIEW3D.BarsResults(model.barsResults, VIEWUI.properties.resultsViewOptions, true); // viewOptions - default preset, will be overwritten later, true - IsResultsWindow
        this.coordinateIndicator = null;
        this.add(this.nodesObject3D);
        this.add(this.barsObject3D);
        this.add(this.barsResults);
        this.dispose = function () {
            for(var i=0;i<this.children.length;i++)
            {
                this.children[i].dispose();
            }
        }
        this.CoordinateIndicator = function () {
            var rectX = 0.05, rectY = 0.5;
            var rectXHalf = rectX / 2, rectYHalf = rectY / 2;
            var rectShape = new THREE.Shape();
            rectShape.moveTo(-rectXHalf, -rectYHalf);
            rectShape.lineTo(rectXHalf, -rectYHalf);
            rectShape.lineTo(rectXHalf, rectYHalf);
            rectShape.lineTo(-rectXHalf, rectYHalf);
            rectShape.lineTo(-rectXHalf, -rectYHalf);
            var rectGeom = new THREE.ShapeGeometry(rectShape);
            var rectMaterial = new THREE.MeshBasicMaterial({
                color: 0xff0000
            });
            var mesh = new THREE.Mesh(rectGeom, rectMaterial);
            mesh.position.x = -1.95; // lower limit
            return mesh;
        }
        this.coordinateIndicator = this.CoordinateIndicator();
        resultsWindowGridScene.add(this.coordinateIndicator); // not this, resultsWindowGridScene doesn't require scaling
        this.onMouseUp = function (event) {
            var mouse = new THREE.Vector2();
            mouse.x = ((event.pageX - ($(resultsWebGlContainer).offset().left)) / resultsWebGlContainer.clientWidth) * 2 - 1;
            console.log("mouse.x: " + mouse.x);
            var node1Position = HELPERS.toVector3(model.bars()[0].node1().position());
            var node2Position = HELPERS.toVector3(model.bars()[0].node2().position());
            var barLength = +node1Position.distanceTo(node2Position);
            var normalizedMouseX = (mouse.x + 1) / 2; // normalize [0, 1]
            var coordinate = barLength * normalizedMouseX;
            model.lastXcoordinate(coordinate);
        }
        
        HELPERS.subscribePropertyChange(this, model.lastXcoordinate, "model.lastXcoordinate", function () {
            this.updateCoordinateIndicator();
        });
        this.updateCoordinateIndicator = function () {
            var node1Position = HELPERS.toVector3(model.bars()[0].node1().position());
            var node2Position = HELPERS.toVector3(model.bars()[0].node2().position());
            var barLength = +node1Position.distanceTo(node2Position);
            var lastXcoordinate = model.lastXcoordinate(); // [0, barLength]
            var positionInSpaceX = (lastXcoordinate / barLength) * (1.95 * 2) - 1.95; // [-2, 2]
            if (positionInSpaceX > 1.95) { // upper limit
                positionInSpaceX = 1.95;
            }
            if (positionInSpaceX < -1.95) { // lower limit
                positionInSpaceX = -1.95;
            }
            this.coordinateIndicator.position.x = positionInSpaceX;
            render();
        }

        this.init = function() {
            $(resultsWebGlContainer).click(this.onMouseUp);
        }
    }
    VIEW3D.ResultsWindow.Results.prototype = Object.create(THREE.Object3D.prototype);
});