'use strict';
var VIEW3D = VIEW3D || {}

VIEW3D.SnapGridDefinition = {
    showGrid : ko.observable(true),
    snapToGrid : ko.observable(true),
    xSpacing : ko.observable(0.25),
    zSpacing : ko.observable(0.25),
    xGridOrigin : ko.observable(0),
    zGridOrigin : ko.observable(0),
    meshObject : null,
    type : "SnapingLinesGrid",
};
function OnSnapDefinitionChanged(newValue) {
    VIEW3D.HideSnapGrid();
    VIEW3D.SnapGridDefinition.meshObject = null;
    VIEW3D.ShowSnapGrid();
    render();
}
VIEW3D.SnapGridDefinition.showGrid.subscribe(OnSnapDefinitionChanged);
VIEW3D.SnapGridDefinition.snapToGrid.subscribe(OnSnapDefinitionChanged);
VIEW3D.SnapGridDefinition.xSpacing.subscribe(OnSnapDefinitionChanged);
VIEW3D.SnapGridDefinition.zSpacing.subscribe(OnSnapDefinitionChanged);
VIEW3D.SnapGridDefinition.xGridOrigin.subscribe(OnSnapDefinitionChanged);
VIEW3D.SnapGridDefinition.zGridOrigin.subscribe(OnSnapDefinitionChanged);

VIEW3D.GetSnapGridDefinition = function() {
    var vm = {
        showGrid : VIEW3D.SnapGridDefinition.showGrid,
        snapToGrid : VIEW3D.SnapGridDefinition.snapToGrid,
        xSpacing : VIEW3D.SnapGridDefinition.xSpacing,
        zSpacing : VIEW3D.SnapGridDefinition.zSpacing,
        xGridOrigin : VIEW3D.SnapGridDefinition.xGridOrigin,
        zGridOrigin : VIEW3D.SnapGridDefinition.zGridOrigin,
        type : "SnapingLinesGrid",
        name : ko.observable("")
    };

    vm.Finish = function() {
        VIEW3D.HideSnapGrid();
        VIEWUI.toolbar.switchMode(MODE.SELECT);
        CORE.view3D.cameraControl.resetView(-1, 1, -1, true);
        CORE.model.deselectAll();
        VIEWUI.properties.hideCustomPropertyTab();
        CORE.view3D.moveControl.updateTextBox(null);
        VIEW3D.AxisHelper.getContainer().show();
        CORE.setShowCheckboxesPanel(true);
        vm.showMenuItems(true);
        CORE.view3D.cameraControl.rotationEnabled = true;
    }

    return vm;
}

VIEW3D.SnapToGrid = function(vec) {
    return new THREE.Vector3(VIEW3D.SnapXToGrid(vec.x), vec.y, VIEW3D.SnapZToGrid(vec.z));
}

VIEW3D.SnapXToGrid = function(x) {
    var xSpacing = parseFloat(VIEW3D.SnapGridDefinition.xSpacing());
    var xOrigin = parseFloat(VIEW3D.SnapGridDefinition.xGridOrigin());
    var newX = Math.round((x - xOrigin) / xSpacing) * xSpacing + xOrigin;
    return newX;
}

VIEW3D.SnapZToGrid = function(z) {
    var zSpacing = parseFloat(VIEW3D.SnapGridDefinition.zSpacing());
    var zOrigin = parseFloat(VIEW3D.SnapGridDefinition.zGridOrigin());
    var newZ = Math.round((z - zOrigin) / zSpacing) * zSpacing + zOrigin;
    return newZ;
}

VIEW3D.CreateSnapGrid = function() {
    var lineColor = {r : 211/255, g:211/255, b: 211/255};
    var vertices = [], colors = [];
    var xGridOrigin = parseFloat(VIEW3D.SnapGridDefinition.xGridOrigin());
    var xSpacing = parseFloat(VIEW3D.SnapGridDefinition.xSpacing());
    var zGridOrigin = parseFloat(VIEW3D.SnapGridDefinition.zGridOrigin());
    var zSpacing = parseFloat(VIEW3D.SnapGridDefinition.zSpacing());

    if(VIEW3D.SnapGridDefinition.showGrid() === true) {
        var bb = CORE.view3D.cameraControl.getConstructionBS();
        var yCoord = 0;
        var boundingBoxFactor = 4*bb.radius;
        boundingBoxFactor = Math.max(boundingBoxFactor, 2);
        for (var i = 0; i < boundingBoxFactor/Math.min(xSpacing, zSpacing); i++) {
            vertices.push(xGridOrigin + xSpacing*i, yCoord, -boundingBoxFactor);
            vertices.push(xGridOrigin + xSpacing*i, yCoord, boundingBoxFactor);
            vertices.push(xGridOrigin - xSpacing*i, yCoord, -boundingBoxFactor);
            vertices.push(xGridOrigin - xSpacing*i, yCoord, boundingBoxFactor);
    
            vertices.push(-boundingBoxFactor, yCoord ,zGridOrigin + zSpacing*i);
            vertices.push(boundingBoxFactor, yCoord ,zGridOrigin + zSpacing*i);
            vertices.push(-boundingBoxFactor, yCoord ,zGridOrigin - zSpacing*i);
            vertices.push(boundingBoxFactor, yCoord ,zGridOrigin - zSpacing*i);
    
    
            for (var j = 0; j < 8; j++) {
                colors.push(lineColor.r, lineColor.g, lineColor.b);      
            }
        }
    }

    var geometry = new THREE.BufferGeometry();
	geometry.addAttribute( 'position', new THREE.Float32BufferAttribute( vertices, 3 ) );
	geometry.addAttribute( 'color', new THREE.Float32BufferAttribute( colors, 3 ) );

    var material = new THREE.LineBasicMaterial( { vertexColors: THREE.VertexColors } );
    
    var mesh = new THREE.LineSegments( geometry, material, THREE.LinePieces );

    return mesh;
}

VIEW3D.ShowSnapGrid = function() {
    if(VIEW3D.SnapGridDefinition.meshObject === null) {
        VIEW3D.SnapGridDefinition.meshObject = VIEW3D.CreateSnapGrid();
    }
    scene.add(VIEW3D.SnapGridDefinition.meshObject);
}

VIEW3D.HideSnapGrid = function() {
    if(VIEW3D.SnapGridDefinition.meshObject !== null) {
        scene.remove(VIEW3D.SnapGridDefinition.meshObject);
    }
}