﻿'use strict';
var VIEW3D = VIEW3D || {}

define(['js/view3d/VIEW3D.Support'],
    function() {
        // class VIEW3D.SupportsNode extends THREE.Object3D
        VIEW3D.SupportsNode = function(supportsModel) {
            THREE.Object3D.call(this);
            var instances = [];
            this.onNodePositionChange = function(node, index) {
                var dir = new THREE.Vector3(0, 0, 1);
                if (node.createsBars().length > 0) {
                    dir = new THREE.Vector3().subVectors(HELPERS.toVector3(node.createsBars()[0].node1().position()),
                        HELPERS.toVector3(node.createsBars()[0].node2().position()));
                }
                var base = VIEW3D.utils.getBaseFromDir(dir);
                var mat = new THREE.Matrix4();
                mat.set(
                    base.baseX.x,
                    base.baseY.x,
                    base.baseZ.x,
                    0,
                    base.baseX.y,
                    base.baseY.y,
                    base.baseZ.y,
                    0,
                    base.baseX.z,
                    base.baseY.z,
                    base.baseZ.z,
                    0,
                    0,
                    0,
                    0,
                    1);
                if (!isEmpty(instances[index])) {
                    if (!isEmpty(instances[index].positionChangeSubscription)) {
                        instances[index].positionChangeSubscription.dispose();
                    }
                    this.remove(instances[index]);
                }
                instances[index] = new VIEW3D.Support(node.position(), mat);
                instances[index].positionChangeSubscription = HELPERS
                    .subscribePropertyChange(this,
                        node.position,
                        "node.position",
                        function () { this.onNodePositionChange(node, index) });
                this.add(instances[index]);
                render();
            }
            this.onNodeAssigned = function(node, index) {
                var dir = new THREE.Vector3(0, 0, 1);
                if (node.createsBars().length > 0) {
                    dir = new THREE.Vector3().subVectors(HELPERS.toVector3(node.createsBars()[0].node1().position()),
                        HELPERS.toVector3(node.createsBars()[0].node2().position()));
                }
                var base = VIEW3D.utils.getBaseFromDir(dir);
                var mat = new THREE.Matrix4();
                mat.set(
                    base.baseX.x,
                    base.baseY.x,
                    base.baseZ.x,
                    0,
                    base.baseX.y,
                    base.baseY.y,
                    base.baseZ.y,
                    0,
                    base.baseX.z,
                    base.baseY.z,
                    base.baseZ.z,
                    0,
                    0,
                    0,
                    0,
                    1);
                var instance = new VIEW3D.Support(node.position(), mat);
                instance.positionChangeSubscription = HELPERS
                    .subscribePropertyChange(this,
                        node.position,
                        "node.position",
                        function() { this.onNodePositionChange(node, index) });
                instances[index] = instance;
                this.add(instance);
                render();
            }
            this.onNodeUnassigned = function(node, index) {
                if (instances[index]) {
                    if (!isEmpty(instances[index].positionChangeSubscription)) {
                        instances[index].positionChangeSubscription.dispose();
                    }
                    this.remove(instances[index]);
                    instances.splice(index, 1);
                }
                render();
            }
            var onSupportAdded = function(supportModel) {
                supportModel.subs.push(HELPERS
                    .subscribeArrayChange(this,
                        supportModel.assignedTo,
                        "MODEL.loads.node.assignedTo",
                        this.onNodeAssigned,
                        this.onNodeUnassigned));
                for (var index = 0; index < supportModel.assignedTo().length; index++) {
                    this.onNodeAssigned(supportModel.assignedTo()[index], index);
                }
            }
            var onSupportRemoved = function(load, index) {
                // never happens?
            }
            var sub = HELPERS.subscribeArrayChange(this,
                supportsModel,
                "MODEL.supports",
                onSupportAdded,
                onSupportRemoved);
            this.dispose = function () {
                var i;
                sub.dispose();
                for (i = 0; i < instances.length; i++) {
                    if (!isEmpty(instances[i].positionChangeSubscription)) {
                        instances[i].positionChangeSubscription.dispose();
                    }
                    if (typeof instances[i].dispose === 'function') instances[i].dispose();
                    this.remove(instances[i]);
                }
                for (i = 0; i < supportsModel().length; i++) {
                    for (var k = 0; k < supportsModel()[i].subs.length; k++) {
                        supportsModel()[i].subs[k].dispose();
                    }
                }
            }
            for (var i = 0; i < supportsModel().length; i++) {
                onSupportAdded.call(this, supportsModel()[i], i);
            }
        }
        VIEW3D.SupportsNode.prototype = Object.create(THREE.Object3D.prototype);
    });