﻿'use strict';
var VIEW3D = VIEW3D || {}
VIEW3D.collision = VIEW3D.collision || {}
VIEW3D.collision.debugMode = false;
VIEW3D.collision.debugMeshes = [];

///////////////////////////////////////////////
///////////////////////////////////////////////
VIEW3D.collision.GlobalPointCollider = function (pos, size) {
    this.pos = pos;
    this.size = isEmpty(size) ? 10 : size;
};

VIEW3D.collision.GlobalPointCollider.prototype = {
    //size : 10,
    intersectRegion: function (params, result) {
        var onScreen = this.pos.clone();
        onScreen.applyMatrix4(params.matrix);
        if (onScreen.x > params.region.max.x) return;
        if (onScreen.y > params.region.max.y) return;
        if (onScreen.x < params.region.min.x) return;
        if (onScreen.y < params.region.min.y) return;

        //add to result
        result.push({
        	collider: this,
        	containedEntirely: true
        });
    },
    
    intersectMousePos:function (params, result) {
        var onScreen = this.pos.clone();
        onScreen.applyMatrix4(params.matrix);
        var dx = (onScreen.x - params.mousePos.x) / params.pixelSize.x;
        var dy = (onScreen.y - params.mousePos.y) / params.pixelSize.y;
        if (dx * dx + dy * dy <= this.size * this.size) {
            result.push({
                collider: this,
                position: this.pos.clone(),
            });
        }
    }
};

VIEW3D.collision.ScreenPointCollider = function (pos3d, offset, size) {
    this.pos3d = pos3d;
    this.offset = offset;
    this.size = isEmpty(size) ? 10 : size;
};

VIEW3D.collision.ScreenPointCollider.prototype = {
    //size : 10,
    intersectRegion: function (params, result) {

    },
    to2D: function(pos, offset, posMat, dirMat) {
        var p = pos.clone();
        p.applyMatrix4(posMat);
        var d = offset.clone();
        d.applyMatrix4(dirMat);
        p.add(d);
        return p;
    },

    intersectMousePos:function (params, result) {
        var onScreen = this.to2D(this.pos3d, this.offset, params.matrix, params.matrixSreenOffset);
        var dx = (onScreen.x - params.mousePos.x) / params.pixelSize.x;
        var dy = (onScreen.y - params.mousePos.y) / params.pixelSize.y;
        if (dx * dx + dy * dy <= this.size * this.size) {
            result.push({
                collider: this,
                //position: this.pos.clone(),
            });
        }
    }
};



///////////////////////////////////////////////
///////////////////////////////////////////////
//based on http://www.cs.utah.edu/~awilliam/box/
VIEW3D.collision.SegmentCollider = function (pos1, pos2) {
    this.set(pos1, pos2);
}

VIEW3D.collision.SegmentCollider.prototype = {
    size: 7,
    set:function(pos1, pos2){
        this.origin = pos1.clone();
        this.direction = pos2.clone().sub(pos1);
    },
    intersectRegion: function (params, result) {
        var tmin, tmax, tymin, tymax;   
        var dir = this.direction.clone().applyMatrix4( params.matrixDir );
        var origin = this.origin.clone().applyMatrix4(params.matrix);

        if (dir.x >= 0) {
            tmin = (params.region.min.x - origin.x) / dir.x;
            tmax = (params.region.max.x - origin.x) / dir.x;
        }
        else {
            tmin = (params.region.max.x - origin.x) / dir.x;
            tmax = (params.region.min.x - origin.x) / dir.x;
        }
        if (dir.y >= 0) {
            tymin = (params.region.min.y - origin.y) / dir.y;
            tymax = (params.region.max.y - origin.y) / dir.y;
        }
        else {
            tymin = (params.region.max.y - origin.y) / dir.y;
            tymax = (params.region.min.y - origin.y) / dir.y;
        }

        if ( (tmin > tymax) || (tymin > tmax) )
            return;
        if (tymin > tmin)
            tmin = tymin;
        if (tymax < tmax)
            tmax = tymax;
        if ((tmin < 1) && (tmax > 0)) {
            result.push({
                collider: this,
                containedEntirely: tmin < 0 && tmax > 1
            });
        }

    },

    intersectMousePos: function (params, result) {
        var t;
        var dir = this.direction.clone().applyMatrix4(params.matrixDir);
        var origin = this.origin.clone().applyMatrix4(params.matrix);
        if (Math.abs(dir.x) > Math.abs(dir.y)) {
            t = (params.mousePos.x - origin.x) / dir.x;
        } else {
            t = (params.mousePos.y - origin.y) / dir.y;
        }
        if (t < 0 || t > 1) {
            return; // no collision
        }

        var pos = dir.clone().multiplyScalar(t).add(origin);
        var dx = (pos.x - params.mousePos.x) / params.pixelSize.x;
        var dy = (pos.y - params.mousePos.y) / params.pixelSize.y;
        if (dx * dx + dy * dy <= this.size * this.size) {
            result.push({
                collider: this,
                position: this.direction.clone().multiplyScalar(t).add(this.origin),
            });
        }
    }
}

///////////////////////////////////////////////
VIEW3D.collision.BoxCollider = function (box, transform) {
    this.box = box;
    this.boxVerticrs = [
        new THREE.Vector3( box.min.x, box.min.y, box.min.z),
        new THREE.Vector3( box.max.x, box.min.y, box.min.z),
        new THREE.Vector3( box.max.x, box.max.y, box.min.z),
        new THREE.Vector3( box.min.x, box.max.y, box.min.z),
        new THREE.Vector3( box.min.x, box.min.y, box.max.z),
        new THREE.Vector3( box.max.x, box.min.y, box.max.z),
        new THREE.Vector3( box.max.x, box.max.y, box.max.z),
        new THREE.Vector3( box.min.x, box.max.y, box.max.z),
    ];
    this.transform = transform;
    this.inverse = new THREE.Matrix4().getInverse(transform);
}

VIEW3D.collision.BoxCollider.prototype = {
    intersectMousePos: function (params, result) {
        //transform ray to box local space
        var ray = params.ray.clone();
        ray.applyMatrix4(this.inverse);
        var pos = ray.intersectBox(this.box);
        if (pos !== null && typeof pos !== 'undefined') {
            //switch to world space
            pos.applyMatrix4(this.transform);
            //console.debug("BoxCollider: HIT!");
            result.push({
                collider: this,
                position: pos,
            });
        }
    },

    create2dConvex: function (dir, viewProj) {
        var transform = new THREE.Matrix4().multiplyMatrices(viewProj, this.transform);
        var e = this.inverse.elements;
        var gx = dir.x;
        var gy = dir.y;
        var gz = dir.z;

        var x = e[0] * gx + e[4] * gy + e[8] * gz;
        var y = e[1] * gx + e[5] * gy + e[9] * gz;
        var z = e[2] * gx + e[6] * gy + e[10] * gz;

        //normalize for z = 0
        var l = Math.sqrt(x * x + y * y);
        x /= l;
        y /= l;


        if (z < -0.999) {
            //top view
            return [
                this.boxVerticrs[4].clone().applyMatrix4(transform),
                this.boxVerticrs[5].clone().applyMatrix4(transform),
                this.boxVerticrs[6].clone().applyMatrix4(transform),
                this.boxVerticrs[7].clone().applyMatrix4(transform),
            ]
        } else if (z < -0.001) {
            if (x < -0.999) {
                //0
                return [
                    this.boxVerticrs[1].clone().applyMatrix4(transform),
                    this.boxVerticrs[2].clone().applyMatrix4(transform),
                    this.boxVerticrs[7].clone().applyMatrix4(transform),
                    this.boxVerticrs[4].clone().applyMatrix4(transform),
                ]
            }
            else if (y < -0.999) {
                //90
                return [
                    this.boxVerticrs[2].clone().applyMatrix4(transform),
                    this.boxVerticrs[3].clone().applyMatrix4(transform),
                    this.boxVerticrs[4].clone().applyMatrix4(transform),
                    this.boxVerticrs[5].clone().applyMatrix4(transform),
                ]
            }
            else if (x > 0.999) {
                //180
                return [
                    this.boxVerticrs[3].clone().applyMatrix4(transform),
                    this.boxVerticrs[0].clone().applyMatrix4(transform),
                    this.boxVerticrs[5].clone().applyMatrix4(transform),
                    this.boxVerticrs[6].clone().applyMatrix4(transform),
                ]
            }
            else if (y > 0.999) {
                //270
                return [
                    this.boxVerticrs[0].clone().applyMatrix4(transform),
                    this.boxVerticrs[1].clone().applyMatrix4(transform),
                    this.boxVerticrs[6].clone().applyMatrix4(transform),
                    this.boxVerticrs[7].clone().applyMatrix4(transform),
                ]
            }
            else if (x < 0) {
                if (y < 0) {
                    //<0:90>
                    return [
                        this.boxVerticrs[1].clone().applyMatrix4(transform),
                        this.boxVerticrs[2].clone().applyMatrix4(transform),
                        this.boxVerticrs[3].clone().applyMatrix4(transform),
                        this.boxVerticrs[7].clone().applyMatrix4(transform),
                        this.boxVerticrs[4].clone().applyMatrix4(transform),
                        this.boxVerticrs[5].clone().applyMatrix4(transform),
                    ]
                } else {
                    //<270:360>
                    return [
                        this.boxVerticrs[0].clone().applyMatrix4(transform),
                        this.boxVerticrs[1].clone().applyMatrix4(transform),
                        this.boxVerticrs[2].clone().applyMatrix4(transform),
                        this.boxVerticrs[6].clone().applyMatrix4(transform),
                        this.boxVerticrs[7].clone().applyMatrix4(transform),
                        this.boxVerticrs[4].clone().applyMatrix4(transform),
                    ]
                }
            }
            else if (x > 0) {
                if (y < 0) {
                    //<90:180>
                    return [
                        this.boxVerticrs[2].clone().applyMatrix4(transform),
                        this.boxVerticrs[3].clone().applyMatrix4(transform),
                        this.boxVerticrs[0].clone().applyMatrix4(transform),
                        this.boxVerticrs[4].clone().applyMatrix4(transform),
                        this.boxVerticrs[5].clone().applyMatrix4(transform),
                        this.boxVerticrs[6].clone().applyMatrix4(transform),
                    ]
                } else {
                    //<270:360>
                    return [
                        this.boxVerticrs[3].clone().applyMatrix4(transform),
                        this.boxVerticrs[0].clone().applyMatrix4(transform),
                        this.boxVerticrs[1].clone().applyMatrix4(transform),
                        this.boxVerticrs[5].clone().applyMatrix4(transform),
                        this.boxVerticrs[6].clone().applyMatrix4(transform),
                        this.boxVerticrs[7].clone().applyMatrix4(transform),
                    ]
                }
            }
        } else if (z < 0.001) {
            if (x < -0.999) {
                //0
                return [
                    this.boxVerticrs[1].clone().applyMatrix4(transform),
                    this.boxVerticrs[2].clone().applyMatrix4(transform),
                    this.boxVerticrs[6].clone().applyMatrix4(transform),
                    this.boxVerticrs[5].clone().applyMatrix4(transform),
                ]
            }
            else if (y < -0.999) {
                //90
                return [
                    this.boxVerticrs[2].clone().applyMatrix4(transform),
                    this.boxVerticrs[3].clone().applyMatrix4(transform),
                    this.boxVerticrs[7].clone().applyMatrix4(transform),
                    this.boxVerticrs[6].clone().applyMatrix4(transform),
                ]
            }
            else if (x > 0.999) {
                //180
                return [
                    this.boxVerticrs[3].clone().applyMatrix4(transform),
                    this.boxVerticrs[0].clone().applyMatrix4(transform),
                    this.boxVerticrs[4].clone().applyMatrix4(transform),
                    this.boxVerticrs[7].clone().applyMatrix4(transform),
                ]
            }
            else if (y > 0.999) {
                //270
                return [
                    this.boxVerticrs[0].clone().applyMatrix4(transform),
                    this.boxVerticrs[1].clone().applyMatrix4(transform),
                    this.boxVerticrs[5].clone().applyMatrix4(transform),
                    this.boxVerticrs[4].clone().applyMatrix4(transform),
                ]
            }
            else if (x < 0) {
                if (y < 0) {
                    //<0:90>
                    return [
                        this.boxVerticrs[1].clone().applyMatrix4(transform),
                        this.boxVerticrs[3].clone().applyMatrix4(transform),
                        this.boxVerticrs[7].clone().applyMatrix4(transform),
                        this.boxVerticrs[5].clone().applyMatrix4(transform),
                    ]
                } else {
                    //<270:360>
                    return [
                        this.boxVerticrs[0].clone().applyMatrix4(transform),
                        this.boxVerticrs[2].clone().applyMatrix4(transform),
                        this.boxVerticrs[6].clone().applyMatrix4(transform),
                        this.boxVerticrs[4].clone().applyMatrix4(transform),
                    ]
                }
            }
            else if (x > 0) {
                if (y < 0) {
                    //<90:180>
                    return [
                        this.boxVerticrs[2].clone().applyMatrix4(transform),
                        this.boxVerticrs[0].clone().applyMatrix4(transform),
                        this.boxVerticrs[4].clone().applyMatrix4(transform),
                        this.boxVerticrs[6].clone().applyMatrix4(transform),
                    ]
                } else {
                    //<180:270>
                    return [
                        this.boxVerticrs[3].clone().applyMatrix4(transform),
                        this.boxVerticrs[1].clone().applyMatrix4(transform),
                        this.boxVerticrs[5].clone().applyMatrix4(transform),
                        this.boxVerticrs[7].clone().applyMatrix4(transform),
                    ]
                }
            }
        } else if (z < 0.999) {
            if (x < -0.999) {
                //0
                return [
                    this.boxVerticrs[0].clone().applyMatrix4(transform),
                    this.boxVerticrs[3].clone().applyMatrix4(transform),
                    this.boxVerticrs[6].clone().applyMatrix4(transform),
                    this.boxVerticrs[5].clone().applyMatrix4(transform),
                ]
            }
            else if (y < -0.999) {
                //90
                return [
                    this.boxVerticrs[0].clone().applyMatrix4(transform),
                    this.boxVerticrs[1].clone().applyMatrix4(transform),
                    this.boxVerticrs[6].clone().applyMatrix4(transform),
                    this.boxVerticrs[7].clone().applyMatrix4(transform),
                ]
            }
            else if (x > 0.999) {
                //180
                return [
                    this.boxVerticrs[2].clone().applyMatrix4(transform),
                    this.boxVerticrs[1].clone().applyMatrix4(transform),
                    this.boxVerticrs[4].clone().applyMatrix4(transform),
                    this.boxVerticrs[7].clone().applyMatrix4(transform),
                ]
            }
            else if (y > 0.999) {
                //270
                return [
                    this.boxVerticrs[3].clone().applyMatrix4(transform),
                    this.boxVerticrs[2].clone().applyMatrix4(transform),
                    this.boxVerticrs[5].clone().applyMatrix4(transform),
                    this.boxVerticrs[4].clone().applyMatrix4(transform),
                ]
            }
            else if (x < 0) {
                if (y < 0) {
                    //<0:90>
                    return [
                        this.boxVerticrs[0].clone().applyMatrix4(transform),
                        this.boxVerticrs[3].clone().applyMatrix4(transform),
                        this.boxVerticrs[7].clone().applyMatrix4(transform),
                        this.boxVerticrs[6].clone().applyMatrix4(transform),
                        this.boxVerticrs[5].clone().applyMatrix4(transform),
                        this.boxVerticrs[1].clone().applyMatrix4(transform),
                    ]
                } else {
                    //<270:360>
                    return [
                        this.boxVerticrs[3].clone().applyMatrix4(transform),
                        this.boxVerticrs[2].clone().applyMatrix4(transform),
                        this.boxVerticrs[6].clone().applyMatrix4(transform),
                        this.boxVerticrs[5].clone().applyMatrix4(transform),
                        this.boxVerticrs[4].clone().applyMatrix4(transform),
                        this.boxVerticrs[0].clone().applyMatrix4(transform),
                    ]
                }
            }
            else if (x > 0) {
                if (y < 0) {
                    //<90:180>
                    return [
                        this.boxVerticrs[1].clone().applyMatrix4(transform),
                        this.boxVerticrs[0].clone().applyMatrix4(transform),
                        this.boxVerticrs[4].clone().applyMatrix4(transform),
                        this.boxVerticrs[7].clone().applyMatrix4(transform),
                        this.boxVerticrs[6].clone().applyMatrix4(transform),
                        this.boxVerticrs[2].clone().applyMatrix4(transform),
                    ]
                } else {
                    //<180:270>
                    return [
                        this.boxVerticrs[2].clone().applyMatrix4(transform),
                        this.boxVerticrs[1].clone().applyMatrix4(transform),
                        this.boxVerticrs[5].clone().applyMatrix4(transform),
                        this.boxVerticrs[4].clone().applyMatrix4(transform),
                        this.boxVerticrs[7].clone().applyMatrix4(transform),
                        this.boxVerticrs[3].clone().applyMatrix4(transform),
                    ]
                }
            }
        } else {
            //looking from bottom
            return [
                this.boxVerticrs[0].clone().applyMatrix4(transform),
                this.boxVerticrs[3].clone().applyMatrix4(transform),
                this.boxVerticrs[2].clone().applyMatrix4(transform),
                this.boxVerticrs[1].clone().applyMatrix4(transform),
            ]
        }
        return null;
    },

    regionConvexTest: function(region, convex){
        var left = true, right = true, top = true, bottom = true;
        for(var i = 0; (i < convex.length) && (left || right || top || bottom); i++){
            var c = convex[i];
            if(c.x > region.min.x) {
                left = false;
            }
            if(c.x < region.max.x) {
                right = false;
            }
            if(c.y > region.min.y) {
                top = false;
            }
            if(c.y < region.max.y) {
                bottom = false
            }
        }
        //entire convex is on left, right, top or bottom
        if(left || right || top || bottom) {
            return false;
        }

        //is there an edge with entire region on the left
        for(var i = 1; i < convex.length; i++) {
            var s = convex[i-1];//edge start
            var e = convex[i];//edge end
            //edge direction
            var dx = e.x - s.x;
            var dy = e.y - s.y;
            var testX;
            var testY;
            if(dx > 0) {
                if(dy > 0) {
                    //test against left upper conner
                    testX = region.min.x;
                    testY = region.max.y;
                }
                else {
                    //test against right upper conner
                    testX = region.max.x;
                    testY = region.max.y;
                }
            } else {
                if(dy > 0 ) {
                    //test against left lower conner
                    testX = region.min.x;
                    testY = region.min.y;
                }
                else {
                    //test against right lower conner
                    testX = region.max.x;
                    testY = region.min.y;
                }
            }
            //test vector direction
            var dtx = testX - s.x;
            var dty = testY - s.y;

            //crosss product 2D
            var cross = dx * dty - dy * dtx;
            if(cross < 0) {
                return false;
            }
        }

        return true;
    },

    regionContainsConvex: function(region, convex) {
        for (var i = 0; (i < convex.length); i++) {
            var c = convex[i];
            if (c.x > region.max.x) {
                return false;
            }
            if (c.x < region.min.x) {
                return false;
            }
            if (c.y > region.max.y) {
                return false;
            }
            if (c.y < region.min.y) {
                return false
            }
        }
        return true;
    },
    intersectRegion: function (params, result) {
        var dir = params.cameraDir;

        var convex = this.create2dConvex(dir, params.matrix);

        var collides = this.regionConvexTest(params.region, convex);
        var contains = collides && this.regionContainsConvex(params.region, convex);


        if(collides) {
            result.push({
                collider: this,
                containedEntirely: contains
            });
        }


        if(VIEW3D.collision.debugMode) {
            var geometry = new THREE.Geometry();
            geometry.vertices = convex;
            for (var i = 0; i < convex.length - 2; i++) {
                geometry.faces.push(new THREE.Face3(0, 2 + i, 1 + i));
            }
            var color = new THREE.Vector3(0.5, 0.5, 0.5);
            if (contains) {
                color = new THREE.Vector3(1, 0, 0);
            }
            else if (collides) {
                color = new THREE.Vector3(0, 1, 0);
            }
            var debugMesh = new THREE.Mesh(geometry, new VIEW3D.materials.ZoomArea(0.2, CORE.aspectRatio(), color));
            debugMesh.renderOrder = 1000;
            debugMesh.frustumCulled = false;
            scene.add(debugMesh);
            VIEW3D.collision.debugMeshes.push(debugMesh);
        }
    }
}
///////////////////////////////////////////////
VIEW3D.collision.ScreenSegmentCollider = function (pos1, pos1offset, pos2, pos2offset) {
    this.pos1 = pos1;
    this.pos1offset = pos1offset;
    this.pos2 = pos2;
    this.pos2offset = pos2offset;
}

VIEW3D.collision.ScreenSegmentCollider.prototype = {

    intersectRegion: function (params) {
        //throw new Error("Not implemented");
    },
    to2D: function(pos, offset, posMat, dirMat) {
        var p = pos.clone();
        p.applyMatrix4(posMat);
        var d = offset.clone();
        d.applyMatrix4(dirMat);
        p.add(d);
        return p;
    },

    intersectMousePos: function (params, result) {
        var s = this.to2D(this.pos1, this.pos1offset, params.matrix, params.matrixSreenOffset);
        var e = this.to2D(this.pos2, this.pos2offset, params.matrix, params.matrixSreenOffset);
        var r = 0.01;
        var d = { x: e.x - s.x, y: e.y - s.y }
        var f = { x: s.x - params.mousePos.x, y: s.y - params.mousePos.y }
        var a = d.x * d.x + d.y * d.y;
        var b = 2.0 * (f.x * d.x + f.y * d.y);
        var c = f.x * f.x + f.y * f.y - r * r;
        var discriminant = b * b - 4 * a * c;
        if (discriminant > 0) {
            // ray didn't totally miss sphere,
            // so there is a solution to
            // the equation.
            discriminant = Math.sqrt(discriminant);
            // either solution may be on or off the ray so need to test both
            // t1 is always the smaller value, because BOTH discriminant and
            // a are nonnegative.
            var t1 = (-b - discriminant) / (2 * a);
            var t2 = (-b + discriminant) / (2 * a);
            // 3x HIT cases:
            //          -o->             --|-->  |            |  --|->
            // Impale(t1 hit,t2 hit), Poke(t1 hit,t2>1), ExitWound(t1<0, t2 hit),
            // 3x MISS cases:
            //       ->  o                     o ->              | -> |
            // FallShort (t1>1,t2>1), Past (t1<0,t2<0), CompletelyInside(t1<0, t2>1)
            if (t1 >= 0 && t1 <= 1) {
                // t1 is the intersection, and it's closer than t2
                // (since t1 uses -b - discriminant)
                // Impale, Poke
                result.push({
                    collider: this,
                    position: this.pos1 // not sure if it's correct
                });
                return;
            }
            // here t1 didn't intersect so we are either started
            // inside the sphere or completely past it
            if (t2 >= 0 && t2 <= 1) {
                // ExitWound
                result.push({
                    collider: this,
                    position: this.pos2 // not sure if it's correct
                });
                return;
            }
        }
    }
}

///////////////////////////////////////////////
VIEW3D.collision.CollidersContainer = function () {
    this.lastId = 1;
    this.coliders = {};
}

VIEW3D.collision.CollidersContainer.prototype = {
    addCollider : function (collider) {
        collider.id = this.lastId++;
        this.coliders[collider.id] = collider;
    },

    removeCollider : function (collider) {
        delete this.coliders[collider.id];
    },

    removeAllColliders : function () {
        this.coliders = [];
    },
	
	removeColliderIfExists: function (collider) {
        if (this.containsCollider(collider))
            this.removeCollider(collider);
    },

    containsCollider : function (collider) {
        return !isEmpty(this.coliders[collider.id]);
    },
    region2DIntersection : function (region, camera, pixelSize) {
        var matrix = new THREE.Matrix4();
        matrix.multiplyMatrices(camera.projectionMatrix, camera.matrixWorldInverse);
        
        var matrixDir = new THREE.Matrix4();
        var matrixWorldInverse = camera.matrixWorldInverse.clone();
        //remove translation
        matrixWorldInverse.setPosition(new THREE.Vector3(0, 0, 0));
        //remove zoom
        matrixDir.multiplyMatrices(camera.projectionMatrix, matrixWorldInverse);

        var params = {
        	region: region,
        	matrix: matrix,
        	matrixDir: matrixDir,
        	pixelSize: pixelSize,
            cameraDir: camera.getWorldDirection()
        }

        if( VIEW3D.collision.debugMode ) {
            for (var i = 0; i < VIEW3D.collision.debugMeshes.length; i++) {
                scene.remove(VIEW3D.collision.debugMeshes[i]);
            }
            VIEW3D.collision.debugMeshes = [];
        }
        var result = [];

        this.intersectRegion(params, result);

        return result;
    },

    position2DIntersection : function (position, camera, pixelSize) {
        var matrix = new THREE.Matrix4();
        matrix.multiplyMatrices(camera.projectionMatrix, camera.matrixWorldInverse);

        var matrixDir = new THREE.Matrix4();
        var matrixWorldInverse = camera.matrixWorldInverse.clone();
        //remove translation
        matrixWorldInverse.setPosition(new THREE.Vector3(0, 0, 0));
        //remove zoom
        matrixDir.multiplyMatrices(camera.projectionMatrix, matrixWorldInverse);

        //dir matrix witout zoom (for screen colliders)
        matrixWorldInverse = camera.matrixWorldInverse.clone();
        //remove translation
        matrixWorldInverse.setPosition(new THREE.Vector3(0, 0, 0));
        var projectionMatrix = camera.projectionMatrix.clone();
        //remove zoom
        projectionMatrix.elements[0] /= projectionMatrix.elements[5];
        projectionMatrix.elements[5] = 1.0;
        var matrixSreenOffset = new THREE.Matrix4().multiplyMatrices(projectionMatrix, matrixWorldInverse);

        var raycaster = new THREE.Raycaster();
        raycaster.setFromCamera(position, camera);
        var dir = raycaster.ray.direction.clone();
        dir.multiplyScalar(camera.near);
        raycaster.ray.origin.add(dir);

        var params = {
            mousePos: position,
            matrix: matrix,
            matrixDir: matrixDir,
            matrixSreenOffset: matrixSreenOffset,
            pixelSize: pixelSize,
            ray: raycaster.ray
        };

        var result = [];
        this.intersectMousePos(params, result);
        result = this.discardOutOfRange(result);
        this.sortIntersections(result);

        return result;
    },

    discardOutOfRange : function (results) {
        var filteredResults = [];
        var box = new THREE.Box3(Extension.clippingLow, Extension.clippingHigh);
        for (var i = 0 ; i < results.length ; i++) {
            var res = results[i];
            if(box.containsPoint(res.position)) {
                filteredResults.push(res);
            }
        }

        return filteredResults;
    },

    intersectMousePos : function (params, result) {
        for (var id in this.coliders) {
            this.coliders[id].intersectMousePos(params, result);
        }
    },

    intersectRegion : function (params, result) {
        for (var id in this.coliders) {
            this.coliders[id].intersectRegion(params, result);
        }
    },

    sortIntersections: function (result) {
        result.sort(function (a, b) {
            var cameraPosition = camera.getWorldPosition();
            var aDistance = a.position.distanceTo(cameraPosition);
            var bDistance = b.position.distanceTo(cameraPosition);
            return aDistance - bDistance;   // if < 0, "a" comes first
        });
    }
}

////////////////////////////////////////////////
//return hem self if one of his children is colliding
VIEW3D.collision.DisjunctionCollider = function() {
    VIEW3D.collision.CollidersContainer.call(this);
}

VIEW3D.collision.DisjunctionCollider.prototype = Object.create( VIEW3D.collision.CollidersContainer.prototype );
VIEW3D.collision.DisjunctionCollider.prototype.constructor =  VIEW3D.collision.DisjunctionCollider;

VIEW3D.collision.DisjunctionCollider.prototype.intersectMousePos = function (params, result) {
    var childrenResult = []
    for (var id in this.coliders) {
        this.coliders[id].intersectMousePos(params, childrenResult);
        if(childrenResult.length > 0) {
            result.push({ collider: this, position: childrenResult[0].position });
            return;
        }
    }
}

VIEW3D.collision.DisjunctionCollider.prototype.intersectRegion = function (params, result) {

    var childrenResult = []
    for (var id in this.coliders) {
        this.coliders[id].intersectRegion(params, childrenResult);
    }
    if(childrenResult.length === 0) {
        //no collision!!!!
        return;
    }
    var containedEntirely = true;
    for (var i = 0; i < childrenResult.length; i++) {
        var obj = childrenResult[i];
        if(!obj.containedEntirely){
            containedEntirely = false;
            break;
        }
    }
    result.push({
        collider: this,
        containedEntirely: containedEntirely
    });
}
