'use strict';
var VIEW3D = VIEW3D || {}
VIEW3D.controls = VIEW3D.controls || {}

define(['js/helpers/HELPERS'], function() {

    VIEW3D.controls.BarCut = function (mouse3D, construction) {

        this.cutData = ko.observable(null);

        var mouseSubPos = null;
        var mouseSubObj = null;
        this.enable = function() {
            HELPERS.safeDispose(mouseSubPos);
            HELPERS.safeDispose(mouseSubObj   );
            mouseSubPos = HELPERS.subscribePropertyChange(this, mouse3D.position, "mouse3D.position", this.onMouse3DChange);
            mouseSubObj = HELPERS.subscribePropertyChange(this, mouse3D.object, "mouse3D.position", this.onMouse3DChange);
        }

        this.disable = function() {
            mouseSubPos = HELPERS.safeDispose(mouseSubPos);
            mouseSubObj = HELPERS.safeDispose(mouseSubObj);
            this.setCutData(null);
        }

        var getBarIntersections = function(bar, bars) {
            var result = [];
            var v0 = HELPERS.toVector3( bar.node1().position() );
            var v1 = HELPERS.toVector3( bar.node2().position() );
            for (var i = 0; i < bars.length; i++) {
                var otherBar = bars[i];
                if(otherBar !== bar) {
                    var w0 = HELPERS.toVector3( otherBar.node1().position() );
                    var w1 = HELPERS.toVector3( otherBar.node2().position() );

                    var intersection = VIEW3D.utils.segmentsIntersection(v0, v1, w0, w1);
                    if(!isEmpty(intersection)){
                        result.push(intersection);
                    }
                }
            }
            return result;
        }

        this.setCutData = function(cutData) {
            if(this.cutData()==null && cutData == null) {
                return;
            }else if(this.cutData()==null || cutData == null){
                this.cutData(cutData);
            }else if(
                !this.cutData().leftEnd.equals(cutData.leftEnd) ||
                !this.cutData().rightEnd.equals(cutData.rightEnd) ||
                !this.cutData().bar !== cutData.bar) {
                this.cutData(cutData);
            }
        }

        this.onMouse3DChange = function () {
            var object = mouse3D.object();
            var position = mouse3D.position();
            if(isEmpty(object) || object.type !== MODEL.OBJECT_TYPE.Bar || isEmpty(position)) {
                this.setCutData(null);
                return false;
            }

            var pos = HELPERS.toVector3(position);
            var onBarPos = HELPERS.toVector3(object.vectorProjection(new MODEL.types.Vector(pos.x, pos.y, pos.z)));
            var barVec = object.node2().position().sub(object.node1().position())
            var longestCoord = 'x';
            if( Math.abs( barVec.x ) <  Math.abs( barVec.y ) ) {
                longestCoord = 'y';
            }
            if( Math.abs( barVec[longestCoord] ) <  Math.abs( barVec.z ) ) {
                longestCoord = 'z';
            }


            var orginalLeftNode = object.node1;
            var orginalRightNode = object.node2;
            if( orginalLeftNode().position()[longestCoord] >  orginalRightNode().position()[longestCoord]) {
                orginalLeftNode = object.node2;
                orginalRightNode = object.node1;
            }
            var leftEnd = HELPERS.toVector3(orginalLeftNode().position());
            var rightEnd = HELPERS.toVector3(orginalRightNode().position());

            var intersections = getBarIntersections(object, construction.bars());
            var setCut = false;
            for (var i = 0; i < intersections.length; i++) {
                var intersection = intersections[i];
                if( onBarPos[longestCoord] > intersection[longestCoord] && leftEnd[longestCoord] < intersection[longestCoord] ) {
                    leftEnd = intersection;
                    setCut = true;
                }
                if( onBarPos[longestCoord] < intersection[longestCoord] && rightEnd[longestCoord] > intersection[longestCoord] ) {
                    rightEnd = intersection;
                    setCut = true;
                }

            }

            this.setCutData(setCut ? {leftEnd:leftEnd, rightEnd:rightEnd, bar:object, orginalLeftNode:orginalLeftNode(), orginalRightNode:orginalRightNode()} : null);
            return false;
        }

        this.onMouseUp = function (event) {
            if (isEmpty(CORE.options.mode) && event.target.parentElement.id !== "webGlContainer") {
                return false;
            }
            if (event.button === 0) {
                var object = mouse3D.object();

                var cutData = this.cutData();

                if(!isEmpty(cutData)){
                    if (cutData.leftEnd.distanceToSquared(cutData.orginalLeftNode.position()) > 0.00001) {
                        var node2 = construction.addNode(new MODEL.Node(HELPERS.toVector(cutData.leftEnd)));
                        var newBar = new MODEL.Bar(cutData.orginalLeftNode, node2);
                        construction.addBar(newBar);
                    }

                    if (cutData.rightEnd.distanceToSquared(cutData.orginalRightNode.position()) > 0.00001) {
                        var node1 = construction.addNode(new MODEL.Node(HELPERS.toVector(cutData.rightEnd)));
                        var newBar = new MODEL.Bar(node1, cutData.orginalRightNode);
                        construction.addBar(newBar);
                    }
                    construction.removeBar(cutData.bar);
                }

                return true;
            }
            return false;
        }
    }
});
