﻿'use strict';
var VIEW3D = VIEW3D || {}
VIEW3D.controls = VIEW3D.controls || {}

define(['js/view3d/VIEW3D.Arrow3D'], function() {
    VIEW3D.controls.BarForceEdit = function(parent, bars, barForceModel, mouse3D, onBarElementEdit) {
        var Mode = { BarSelection: 0, OnBaLocalization: 1 }
        var enabled = false;
        var mode = Mode.BarSelection;
        var arrow3D = null;
        this.bar = null;
        this.x = 0;
        this.enable = function() {
            if (enabled) {
                return;
            }
            enabled = true;
            mode = Mode.BarSelection;
            if (elementsPropertiesCache.barForceModel !== 0) {
                barForceModel(elementsPropertiesCache.barForceModel);
            }
        }
        this.disable = function() {
            if (!enabled) {
                return;
            }
            enabled = false;
            if (!isEmpty(arrow3D)) {
                arrow3D.dispose();
                scene.remove(arrow3D);
            }
            onBarElementEdit.clearMesurements();
            mode = Mode.BarSelection;
        }
        this.onMouseMove = function() {
            if (enabled === false || mode !== Mode.OnBaLocalization) {
                return false;
            }
            var ray = mouse3D.raycaster.ray;
            var pos = mouse3D.getNearestToRay(this.bar, ray, true);
            var offset = new THREE.Vector3().subVectors(pos, this.bar.node1().position());
            this.x = offset.length();
            onBarElementEdit.setBar(this.bar, this.x, null);
            arrow3D.updatePos(pos);
            render();
            return false;
        }
        this.onMouseUp = function(event) {
            if (enabled === false || event.target.parentElement.id !== "webGlContainer") {
                return false;
            }
            var x;
            var self = this;
            switch (mode) {
            case Mode.BarSelection:
                var object = mouse3D.object();
                if (!isEmpty(object) && object.type === MODEL.OBJECT_TYPE.Bar) {
                    this.bar = object;
                    var clickPosition = mouse3D.position();
                    this.originalOffset = new THREE.Vector3().subVectors(clickPosition, this.bar.node1().position());
                    this.x = this.originalOffset.length();
                    var tmp = new MODEL.types.BarAttachment(this.bar, x);
                    barForceModel().barAttachments().push(tmp);
                    barForceModel().barAttachments().splice(0, 1); // remove default bar attachment
                    if (this.checkBar()) {
                        this.lastLoad = barForceModel;
                        onBarElementEdit.setBar(this.bar, this.x, function(newX) {
                            var convertedX = convertUnits(newX, 'Distance');
                            self.lastLoad().barAttachments()[0].x(convertedX);
                        });
                        var MemberConstraints = CORE.model.constraints.getMemberConstraints(this.bar.id);
                        arrow3D = new VIEW3D.Arrow3D(barForceModel(), VIEW3D.utils.getMatrix4FromBase(MemberConstraints.orientation), mouse3D.position());
                        arrow3D.updatePos(clickPosition);
                        scene.add(arrow3D);
                        mode = Mode.OnBaLocalization;
                        return true;
                    }
                    return false;
                }
                return false;
            case Mode.OnBaLocalization:
                var oldLoad = new MODEL.load.bar.BarForce();
                oldLoad.forces(barForceModel().forces());
                oldLoad.moments(barForceModel().moments());
                elementsPropertiesCache.barForceModel = oldLoad;
                var newLoad = new MODEL.load.bar.BarForce();
                newLoad.forces(barForceModel().forces());
                newLoad.moments(barForceModel().moments());
                newLoad.name(barForceModel().name());
                newLoad.loadCase(barForceModel().loadCase());
                
                var barAttachment = newLoad.barAttachments()[0];
                barAttachment.x(this.x);
                barAttachment.bar = this.bar;
                CORE.model.addLoad(newLoad);
                mode = Mode.BarSelection;
                scene.remove(arrow3D);
                arrow3D.dispose();
                arrow3D = null;
                onBarElementEdit.clearMesurements();
                return true;
            }
            return false;
        }
        this.checkBar = function() {
            for (var i = 0; i < CORE.model.loads().length; i++) {
                var load = CORE.model.loads()[i];
                if (load.type === MODEL.OBJECT_TYPE.BarForce)
                    for (var j = 0; j < load.barAttachments().length; j++) {
                        var bar = load.barAttachments()[j].bar;
                        var newAttachment = barForceModel().barAttachments()[barForceModel().barAttachments().length - 1];
                        if (bar.id === newAttachment.bar.id) {
                            if (load.barAttachments()[0].x() === newAttachment.x()) {
                                console.error("VIEW3D.controls.BarForceEdit.checkBar: You can't add the same load twice!");
                                return false;
                            }
                        }
                    }
            }
            return true;
        }
    }
});