'use strict';
var VIEW3D = VIEW3D || {}
VIEW3D.controls = VIEW3D.controls || {}

define([], function() {

    VIEW3D.controls.BarSnap = function (mouse3D, bars) {

        var mouseSubObj = null;
        this.enable = function() {
            mouseSubObj = HELPERS.subscribePropertyChange(this, mouse3D.object, "mouse3D.position", this.onMouse3DChange);
        }

        this.disable = function() {
            mouseSubObj = HELPERS.safeDispose(mouseSubObj);
            this.setExtendData(null);
        }

        this.extendData = ko.observable();

        this.setExtendData = function(extendData) {
            if(this.extendData()!==null || extendData !== null) {
                this.extendData(extendData);
            }
        }


        var findExtendData = function(startNode, endNode, barToExtend, bar) {
            if(endNode.createsBars().length === 1 && endNode.attachedBars().length === 0) {
                var startPos = startNode.position;
                var endPos = endNode.position;
                var p0 = HELPERS.toVector3(startPos());
                var p1 = HELPERS.toVector3(endPos());
                var rayDir = p1.clone().sub(p0);
                rayDir.normalize();
                //var middlePos = p0.add(p1).multiplyScalar(0.5);
                var ray = new THREE.Ray(p1, rayDir);
                var v0 = HELPERS.toVector3(bar.node1().position());
                var v1 = HELPERS.toVector3(bar.node2().position());
                var pointOnBar = new THREE.Vector3();
                var dist = ray.distanceSqToSegment(v0, v1, undefined, pointOnBar);
                if (dist < 0.001) {
                    /*
                    endPos(HELPERS.toVector(pointOnBar));
                    */
                    var vec1 = new THREE.Vector3().subVectors(p1, p0);
                    var dir1 = vec1.multiplyScalar(1 / vec1.length());
                    var vec2 = new THREE.Vector3().subVectors(v1, v0);
                    var dir2 = vec2.multiplyScalar(1 / vec2.length());
                    var cross = new THREE.Vector3().crossVectors(dir1, dir2).length();
                    if (cross === 0) {
                        var dist1 = new THREE.Vector3().subVectors(bar.node1().position(), p0).length();
                        var dist2 = new THREE.Vector3().subVectors(bar.node2().position(), p0).length();
                        if (dist1 > dist2) {
                            pointOnBar = bar.node2().position();
                        } else {
                            pointOnBar = bar.node1().position();
                        }
                    }
                    return{node:endNode, extendPos: pointOnBar, barToExtend: barToExtend};
                }
            }
        }


        this.onMouse3DChange = function () {
            var bar = mouse3D.object();
            if(isEmpty(bar) || bar.type !== MODEL.OBJECT_TYPE.Bar) {
                this.setExtendData(null);
                return;
            }

            var dataArray = [];
            for (var i = 0; i < bars.length; i++) {
                var snapBar = bars[i];
                if(snapBar !== bar) {
                    var data = findExtendData(snapBar.node1(), snapBar.node2(), snapBar, bar);
                    if(!isEmpty(data)) {
                        dataArray.push(data);
                    }
                    data = findExtendData(snapBar.node2(), snapBar.node1(), snapBar, bar);
                    if(!isEmpty(data)) {
                        dataArray.push(data);
                    }
                }
            }

            this.setExtendData( dataArray.length > 0 ? dataArray : null);
        }
        /*
        var snapSegmentEndToBar = function(startNode, endNode, bar) {
            if(endNode.createsBars().length === 1 && endNode.attachedBars().length === 0) {
                var startPos = startNode.position;
                var endPos = endNode.position;
                var p0 = HELPERS.toVector3(startPos());
                var p1 = HELPERS.toVector3(endPos());
                var rayDir = p1.clone().sub(p0);
                rayDir.normalize();
                //var middlePos = p0.add(p1).multiplyScalar(0.5);
                var ray = new THREE.Ray(p1, rayDir);
                var v0 = HELPERS.toVector3(bar.node1().position());
                var v1 = HELPERS.toVector3(bar.node2().position());
                var pointOnBar = new THREE.Vector3();
                var dist = ray.distanceSqToSegment(v0, v1, undefined, pointOnBar);
                if (dist < 0.001) {
                    endPos(HELPERS.toVector(pointOnBar));
                }
            }
        }
        */

        this.onMouseUp = function (event) {
            if (isEmpty(CORE.options.mode) && event.target.parentElement.id !== "webGlContainer") {
                return false;
            }
            if (event.button === 0) {
                var extendData = this.extendData();
                if( !isEmpty(extendData) ) {
                    for (var i = 0; i < extendData.length; i++) {
                        var data = extendData[i];
                        data.node.position(HELPERS.toVector( data.extendPos));
                    }
                }
                return true;

            }
            return false;
        }
    }
});
