﻿
'use strict';
var VIEW3D = VIEW3D || {}
VIEW3D.controls = VIEW3D.controls || {}

VIEW3D.controls.BarTrapezoidalEdit = function (parent, selectedObjects, barTrapezoidalModel, mouse3D, onBarElementEdit) {
    var Mode = { BarSelection: 0, OnBaLocalization: 1 , OnBaLocalization2: 2}
    var enabled = false;
    var mode = Mode.BarSelection;
    var arrowContainer = null;
    var fill = null;
    this.bar = null;
    this.x = 0;
    this.enable = function () {
        if (enabled) {
            return;
        }
        enabled = true;
        mode = Mode.BarSelection;
        if (elementsPropertiesCache.barTrapezoidalModel !== 0) {
            barTrapezoidalModel(elementsPropertiesCache.barTrapezoidalModel);
        }
    }
    this.disable = function() {
        if (!enabled) {
            return;
        }
        enabled = false;
        scene.remove(this.fillsContainer);

        if(!isEmpty(arrowContainer)) {
            scene.remove(arrowContainer);
            arrowContainer.dispose();
            arrowContainer = null;
        }

        fill = null;
        this.bar = null;
        onBarElementEdit.clearMesurements();
        mode = Mode.BarSelection;
    }
    var createFillObjects = function (barTrapezoidalModel, barAttachment) {
        var fills = [];
        var fill;
        var node1Pos = barAttachment.bar.node1().position();
        var node2Pos = barAttachment.bar.node2().position();
        var force1 = barTrapezoidalModel.points()[0]().vector;      // force values as vectors
        var force2 = barTrapezoidalModel.points()[1]().vector;
        var distance1 = barTrapezoidalModel.points()[0]().distance;
        var distance2 = barTrapezoidalModel.points()[1]().distance;
        var sub = HELPERS.toVector3(node2Pos.sub(node1Pos));
        var v1 = node1Pos.add(sub.clone().setLength(distance1));    // positions on bar in world space, distance1 and distance2 away from node1
        var v2 = node1Pos.add(sub.clone().setLength(distance2));
        var isZero = function (x) {
            return Math.abs(x) < 0.00001;
        }
        if (!isZero(force1.x) || !isZero(force2.x)) {
            fill = {
                force1pos: v1,
                force2pos: v2,
                force1value: new MODEL.types.Vector(force1.x, 0, 0),
                force2value: new MODEL.types.Vector(force2.x, 0, 0),
                color: { x: 1, y: 0, z: 0 },
                type: VIEW3D.LoadFills.Fill
            };
            fills.push(fill);
        }
        if (!isZero(force1.y) || !isZero(force2.y)) {
            fill = {
                force1pos: v1,
                force2pos: v2,
                force1value: new MODEL.types.Vector(0, force1.y, 0),
                force2value: new MODEL.types.Vector(0, force2.y, 0),
                color: { x: 0, y: 1, z: 0 },
                type: VIEW3D.LoadFills.Fill
            };
            fills.push(fill);
        }
        if (!isZero(force1.z) || !isZero(force2.z)) {
            fill = {
                force1pos: v1,
                force2pos: v2,
                force1value: new MODEL.types.Vector(0, 0, force1.z),
                force2value: new MODEL.types.Vector(0, 0, force2.z),
                color: { x: 0, y: 0, z: 1 },
                type: VIEW3D.LoadFills.Fill
            };
            fills.push(fill);
        }
        return fills;
    }
    this.onMouseMove = function () {
        if (enabled === false || mode === Mode.BarSelection) {
            return false;
        }
        var i;
        var ray = mouse3D.raycaster.ray;
        var pos = mouse3D.getNearestToRay(this.bar, ray, true);
        var offset = new THREE.Vector3().subVectors(pos, this.bar.node1().position());
        this.x = offset.length();
        switch (mode) {
            case Mode.OnBaLocalization:
                onBarElementEdit.setBar(this.bar, this.x, null);
                arrowContainer.updatePos(pos);
            break;
            case Mode.OnBaLocalization2:
                onBarElementEdit.setBarTrapezoid(this.bar, barTrapezoidalModel().points()[0]().distance, this.x);
                arrowContainer.updatePosP2(pos);
                if (isEmpty(fill)) {
                    this.fillsContainer = new VIEW3D.LoadFills(camera, 1, CORE.constructionView3D.loadsObject3D.arrows.maxArrow);
                    fill = createFillObjects(barTrapezoidalModel(), barTrapezoidalModel().barAttachments()[0]);
                    this.fillsContainer.addFills(fill, undefined);
                    scene.add(this.fillsContainer);
                }
                for(i = 0; i < fill.length; i++) {
                    fill[i].force2pos = pos;
                }
                this.fillsContainer.updateAllFills();
                break;
        }
        render();
        return false;
    }
    this.onMouseUp = function (event) {
        if (enabled === false || event.target.parentElement.id !== "webGlContainer") {
            return false;
        }
        var i, x;
        var self = this;
        switch (mode) {
            case Mode.BarSelection:
                var object = mouse3D.object();
                if (!isEmpty(object) && object.type === MODEL.OBJECT_TYPE.Bar) {
                    this.bar = object;
                    var clickPosition = mouse3D.position();
                    this.originalOffset = new THREE.Vector3().subVectors(clickPosition, this.bar.node1().position());
                    this.x = this.originalOffset.length()
                    var tmp = new MODEL.types.BarAttachment(this.bar, x);
                    barTrapezoidalModel().barAttachments().push(tmp);
                    barTrapezoidalModel().barAttachments().splice(0, 1); // remove default bar attachment
                    if (this.checkBar()) {
                        this.lastLoad = barTrapezoidalModel;
                        onBarElementEdit.setBar(this.bar, this.x, function (newX) {
                            var convertedNewX = convertUnits(newX, 'Distance');
                            self.lastLoad().barAttachments()[0].x(newX);
                        });
                        this.MemberConstraints = CORE.model.constraints.getMemberConstraints(this.bar.id);
                        arrowContainer = new VIEW3D.Arrow3D(barTrapezoidalModel(), VIEW3D.utils.getMatrix4FromBase(this.MemberConstraints.orientation), mouse3D.position());
                        scene.add(arrowContainer);
                        mode = Mode.OnBaLocalization;
                        return true;
                    }
                    return false;
                }
                return false;
            case Mode.OnBaLocalization:
                this.newLoad = new MODEL.load.bar.BarTrapezoidal();
                this.newLoad.points()[0](barTrapezoidalModel().points()[0]());
                this.newLoad.points()[0]().distance = this.x;
                var barAttachment = this.newLoad.barAttachments()[0];
                barAttachment.x(this.x);
                barAttachment.bar = this.bar;
                mode = Mode.OnBaLocalization2;
                onBarElementEdit.clearMesurements();
                return true;
            case Mode.OnBaLocalization2:
                var oldLoad = new MODEL.load.bar.BarTrapezoidal();
                oldLoad.points()[0](barTrapezoidalModel().points()[0]());
                oldLoad.points()[1](barTrapezoidalModel().points()[1]());
                elementsPropertiesCache.barTrapezoidalModel = oldLoad;

                // set first point
                var vector = barTrapezoidalModel().points()[0]().vector;
                var x = barTrapezoidalModel().points()[0]().distance;
                var clonedPoint = new MODEL.types.ExtendedVector(vector.x, vector.y, vector.z, x);
                this.newLoad.points()[0](clonedPoint);

                // set second point
                var vector = barTrapezoidalModel().points()[1]().vector;
                var clonedPoint = new MODEL.types.ExtendedVector(vector.x, vector.y, vector.z, this.x);
                this.newLoad.points()[1](clonedPoint);
                this.newLoad.name(barTrapezoidalModel().name());
                this.newLoad.loadCase(barTrapezoidalModel().loadCase());
                CORE.model.addLoad(this.newLoad);
                mode = Mode.BarSelection;
                scene.remove(arrowContainer);
                scene.remove(this.fillsContainer);
                arrowContainer.dispose();
                arrowContainer = null;
                fill = null;
                onBarElementEdit.clearMesurements();
                return true;
        }
        return false;
    }
    this.checkBar = function () {
        for (var i = 0; i < CORE.model.loads().length; i++) {
            var load = CORE.model.loads()[i];
            if (load.type === MODEL.OBJECT_TYPE.BarTrapezoidal)
                for (var j = 0; j < load.barAttachments().length; j++) {
                    var bar = load.barAttachments()[j].bar;
                    var newAttachment = barTrapezoidalModel().barAttachments()[barTrapezoidalModel().barAttachments().length - 1];
                    if (bar.id === newAttachment.bar.id) {
                        if (load.barAttachments()[0].x() === newAttachment.x()) {
                            console.error("VIEW3D.controls.BarTapezoidalEdit.checkBar: You can't add the same load twice!");
                            return false;
                        }
                    }
                }
        }
        return true;
    }
}