﻿"use strict";
var VIEW3D = VIEW3D || {};
VIEW3D.controls = VIEW3D.controls || {};
VIEW3D.controls.BoxEdit = function(parent, cursor3D, mouse3D, boxModel) {
    var editMode = { FirstPoint: 0, SecondPoint: 1, ThirdPoint: 2 };
    var enabled = false;
    var mode = editMode.FirstPoint;
    this.material = new THREE.MeshBasicMaterial({ color: 0x3D85C6, transparent: true, opacity: 0.8 });
    this.plane = null;
    this.enable = function() {
        if (enabled) {
            return;
        }
        enabled = true;
        mode = editMode.FirstPoint;
    };
    this.disable = function() {
        if (!enabled) {
            return;
        }
        enabled = false;
        if (!isEmpty(this.mesh)) {
            scene.remove(this.mesh);
            this.mesh = null;
        }
        if (!isEmpty(this.edgesHelper)) {
            scene.remove(this.edgesHelper);
            this.edgesHelper = null;
        }
        mode = editMode.FirstPoint;
    };
    this.updateVisualization = function() {
        if (!isEmpty(this.mesh)) {
            scene.remove(this.mesh);
            this.mesh = null;
            scene.remove(this.edgesHelper);
            this.edgesHelper = null;
        }
        var a = Math.abs(boxModel().p1().x - boxModel().p2().x);
        var b = Math.abs(boxModel().p1().y - boxModel().p2().y);
        var c = Math.abs(boxModel().p1().z - boxModel().p2().z);
        var geometry = new THREE.BoxBufferGeometry(a, b, c);
        var center = new THREE.Vector3().addVectors(boxModel().p1(), boxModel().p2()).multiplyScalar(0.5);
        this.mesh = new THREE.Mesh(geometry, this.material);
        this.mesh.position.set(center.x, center.y, center.z);
        scene.add(this.mesh);
        if (boxModel().p1().x !== boxModel().p2().x && boxModel().p1().y !== boxModel().p2().y) {
            this.edgesHelper = new THREE.EdgesHelper(this.mesh, 0x27557E, 0.1);
            scene.add(this.edgesHelper);
        }
    };
    this.onMouseMove = function() {
        if (enabled === false) {
            return false;
        }
        var posX, posY, posZ, ray;
        switch (mode) {
        case editMode.FirstPoint:
            posX = Math.round(cursor3D.position.x * epsilonGrid) / epsilonGrid;
            posY = Math.round(cursor3D.position.y * epsilonGrid) / epsilonGrid;
            posZ = Math.round(cursor3D.position.z * epsilonGrid) / epsilonGrid;
            boxModel().p1(new MODEL.types.Vector(posX, posY, posZ));
            boxModel().p2(new MODEL.types.Vector(posX, posY, posZ));
            return true;
        case editMode.SecondPoint:
            if (!isEmpty(cursor3D.object) &&
                (cursor3D.object.type === MODEL.OBJECT_TYPE.Bar || cursor3D.object.type === MODEL.OBJECT_TYPE.Node)) {
                posX = Math.round(cursor3D.position.x * epsilonGrid) / epsilonGrid;
                posY = Math.round(cursor3D.position.y * epsilonGrid) / epsilonGrid;
                boxModel().p2(new MODEL.types.Vector(posX, posY, boxModel().p1().z));
            } else {
                ray = mouse3D.raycaster.ray;
                var position = ray.intersectPlane(this.plane);
                posX = Math.round(position.x * epsilonGrid) / epsilonGrid;
                posY = Math.round(position.y * epsilonGrid) / epsilonGrid;
                boxModel().p2(new MODEL.types.Vector(posX, posY, boxModel().p1().z));
            }
            this.updateVisualization();
            return true;
        case editMode.ThirdPoint:
            if (isEmpty(posZ)) {
                posZ = new THREE.Vector3();
            }
            ray = mouse3D.raycaster.ray;
            var v1 = new THREE.Vector3(boxModel().p2().x, boxModel().p2().y, -1000);
            var v2 = new THREE.Vector3(boxModel().p2().x, boxModel().p2().y, 1000);
            ray.distanceSqToSegment(v1, v2, undefined, posZ);
            boxModel().p2(new MODEL.types.Vector(boxModel().p2().x, boxModel().p2().y, posZ.z));

            this.updateVisualization();
            return true;
        }
        render();
        return false;
    };
    var boxDefinitionToBox = function(boxDefinition) {
        var box = new MODEL.Box();
        var center = new THREE.Vector3().addVectors(boxDefinition.p1(), boxDefinition.p2()).multiplyScalar(0.5);
        var size = new THREE.Vector3().subVectors(boxDefinition.p1(), boxDefinition.p2());
        box.origin(HELPERS.toVector(center));
        box.size(new MODEL.types.Vector(Math.abs(size.x), Math.abs(size.y), Math.abs(size.z)));
        return box;
    };
    this.onMouseUp = function(event) {
        if (enabled === false || event.target.parentElement.id !== "webGlContainer") {
            return false;
        }
        switch (mode) {
        case editMode.FirstPoint:
            mode = editMode.SecondPoint;
            this.plane = new THREE.Plane(new THREE.Vector3(0, 0, 1), -boxModel().p1().z);
            return true;
        case editMode.SecondPoint:
            mode = editMode.ThirdPoint;
            return true;
        case editMode.ThirdPoint:
            CORE.model.addBox(boxDefinitionToBox(boxModel()));
            this.plane = null;
            scene.remove(this.mesh);
            this.mesh = null;
            scene.remove(this.edgesHelper);
            this.edgesHelper = null;
            boxModel(new MODEL.BoxDefinition());
            mode = editMode.FirstPoint;
            return true;
        }
        return false;
    };
};