﻿"use strict";
var VIEW3D = VIEW3D || {};
VIEW3D.controls = VIEW3D.controls || {};
define([
        "js/view3d/VIEW3D.Highlight", "js/view3d/controls/VIEW3D.controls.NodeDraw",
        "js/view3d/controls/VIEW3D.controls.OrthogonalDraw",
        "js/view3d/controls/VIEW3D.controls.Select", "js/view3d/controls/VIEW3D.controls.snap",
        "js/view3d/controls/VIEW3D.controls.NodeForceEdit",
        "js/view3d/controls/VIEW3D.controls.SupportEdit", "js/view3d/controls/VIEW3D.controls.BarForceEdit",
        "js/view3d/controls/VIEW3D.controls.BarUniformEdit",
        "js/view3d/controls/VIEW3D.controls.BarTrapezoidalEdit", "js/view3d/controls/VIEW3D.controls.Trackball",
        "js/view3d/controls/VIEW3D.controls.Move",
        "js/view3d/controls/VIEW3D.controls.ZoomArea", "js/view3d/controls/VIEW3D.controls.Mouse3D",
        "js/view3d/controls/VIEW3D.controls.MeasurementEdit",
        "js/view3d/controls/VIEW3D.controls.SelectArea", "js/core/CORE.calculations",
        "js/view3d/controls/VIEW3D.controls.Edit",
        "js/view3d/controls/VIEW3D.controls.MoveConstruction",
        "js/view3d/controls/VIEW3D.controls.OnBarElementEdit", "js/view3d/controls/VIEW3D.controls.SelectionEditTools",
        "js/view3d/controls/VIEW3D.controls.BarCut",
        "js/view3d/controls/VIEW3D.controls.BarSnap", "js/view3d/controls/VIEW3D.controls.BoxEdit",
        "js/view3d/controls/VIEW3D.controls.OrthogonalHelper"
    ],
    function() {
        VIEW3D.controls.ControlsManager = function(scene,
            camera,
            axisHelperNamespace,
            domElement,
            model,
            view3D,
            mode,
            measurementsView,
            aspectRatio) {
            //helpers
            var snap = new VIEW3D.controls.Snap(scene, camera, CORE.constructionView3D);
            var mouse3D = new VIEW3D.controls.Mouse3D(camera, view3D, measurementsView, canvas, "ControlsManager");
            var cursor3D = new VIEW3D.controls.Cursor3D(scene, camera, view3D, mouse3D);
            var barCut = new VIEW3D.controls.BarCut(mouse3D, model);
            var barSnap = new VIEW3D.controls.BarSnap(mouse3D, model.bars());
            CORE.highlight = new VIEW3D.Highlight(renderer, mouse3D, view3D, barCut, barSnap, model.constraints);
            //var orthogonalHelper = new VIEW3D.controls.OrthogonalHelper(scene, cursor3D);
            var orthogonalHelper = new VIEW3D.controls.OrthogonalPlaneHelper(scene, cursor3D);
            var onBarElementEdit = new VIEW3D.controls.OnBarElementEdit(scene, mouse3D);
            cursor3D.orthogonalHelper = orthogonalHelper;
            //controls
            var controls = {
                select: {
                    enable: false,
                    control: new VIEW3D.controls.Select(camera, model, mouse3D)
                },
                orthogonalDraw: {
                    enable: false,
                    control: new VIEW3D.controls.OrthogonalDraw(scene,
                        model,
                        model.definingObject,
                        cursor3D,
                        orthogonalHelper)
                },
                nodeDraw: {
                    enable: false,
                    control: new VIEW3D.controls.NodeDraw(scene,
                        model.definingObject,
                        model,
                        cursor3D,
                        orthogonalHelper)
                },
                nodeForceEdit: {
                    enable: false,
                    control: new VIEW3D.controls.NodeForceEdit(scene, model.selectedObjects, model.definingObject)
                },
                barForceEdit: {
                    enable: false,
                    control: new VIEW3D.controls.BarForceEdit(scene,
                        model.bars,
                        model.definingObject,
                        mouse3D,
                        onBarElementEdit)
                },
                barUniformEdit: {
                    enable: false,
                    control: new VIEW3D.controls
                        .BarUniformEdit(scene, model.selectedObjects, model.definingObject, mouse3D)
                },
                barTrapezoidalEdit: {
                    enable: false,
                    control: new VIEW3D.controls
                        .BarTrapezoidalEdit(scene,
                            model.selectedObjects,
                            model.definingObject,
                            mouse3D,
                            onBarElementEdit)
                },
                nodeSupportEdit: {
                    enable: false,
                    control: new VIEW3D.controls
                        .SupportEdit(scene, model.selectedObjects, model.definingObject, mouse3D)
                },
                propertiesWindow: {
                    enable: false,
                    control: VIEWUI.properties
                },
                cameraControl: {
                    enable: false,
                    control: new VIEW3D.controls.Trackball(camera, axisHelperNamespace, domElement, model, mouse3D)
                },
                move: {
                    enable: false,
                    control: new VIEW3D.controls.Move(camera, model, view3D, mouse3D, snap, aspectRatio)
                },
                zoomArea: {
                    enable: false,
                    control: new VIEW3D.controls.ZoomArea(camera, axisHelperNamespace, domElement, scene)
                },
                mouse3d: {
                    enable: false,
                    control: mouse3D
                },
                cursor3D: {
                    enable: false,
                    control: cursor3D
                },
                measurementEdit: {
                    enable: false,
                    control: new VIEW3D.controls.MeasurementEdit(mouse3D)
                },
                selectArea: {
                    enable: false,
                    control: new VIEW3D.controls.SelectArea(camera, domElement, view3D, model)
                },
                edit: {
                    enable: false,
                    control: new VIEW3D.controls.Edit(model.selectedObjects, mouse3D)
                },
                moveConstruction: {
                    enable: false,
                    control: new VIEW3D.controls.MoveConstruction(mouse3D)
                },
                onBarElementEdit: {
                    enable: false,
                    control: onBarElementEdit
                },
                selectionEditTools: {
                    enable: false,
                    control: new VIEW3D.controls.SelectionEditTools(model.selectedObjects, onBarElementEdit)
                },
                barCut: {
                    enable: false,
                    control: barCut
                },
                barSnap: {
                    enable: false,
                    control: barSnap
                },
                boxEdit: {
                    enable: false,
                    control: new VIEW3D.controls.BoxEdit(scene, cursor3D, mouse3D, model.definingObject)
                },
                orthogonalHelper: {
                    enable: false,
                    control: orthogonalHelper
                }
            };
            this.getControls = function () { return controls; };
            // access to the camera is required from the outside (CORE.view3D.cameraControl)
            this.cameraControl = controls.cameraControl
                .control;
            this.zoomArea = controls.zoomArea.control;
            this.moveControl = controls.move.control;
            this.mouse3d = controls.mouse3d.control;
            var inputHandlersQueue = [];
            var updateStatus = function(newMode) {
                inputHandlersQueue = [];
                for (var control in controls) {
                    if (controls.hasOwnProperty(control)) {
                        controls[control].enable = false;
                    }
                }
                // turn it on for all modes
                controls.mouse3d.enable = true;
                inputHandlersQueue.push(controls.mouse3d.control);
                var editmode = false;
                switch (newMode) {
                case MODE.DRAW_BAR:
                    controls.cameraControl.enable = true;
                    inputHandlersQueue.push(controls.cameraControl.control);
                    controls.cursor3D.enable = true;
                    inputHandlersQueue.push(controls.cursor3D.control);
                    controls.orthogonalDraw.enable = true;
                    inputHandlersQueue.push(controls.orthogonalDraw.control);
                    controls.propertiesWindow.enable = true;
                    inputHandlersQueue.push(controls.orthogonalHelper.control);
                    controls.orthogonalHelper.enable = true;
                    editmode = true;
                    break;
                case MODE.DRAW_NODE:
                    controls.cursor3D.enable = true;
                    inputHandlersQueue.push(controls.cursor3D.control);
                    controls.cameraControl.enable = true;
                    inputHandlersQueue.push(controls.cameraControl.control);
                    controls.nodeDraw.enable = true;
                    inputHandlersQueue.push(controls.nodeDraw.control);
                    controls.propertiesWindow.enable = true;
                    inputHandlersQueue.push(controls.orthogonalHelper.control);
                    controls.orthogonalHelper.enable = true;
                    editmode = true;
                    break;
                case MODE.SELECT_AREA:
                    controls.selectArea.enable = true;
                    inputHandlersQueue.push(controls.selectArea.control);
                    controls.propertiesWindow.enable = true;
                    break;
                case MODE.SELECT:
                    controls.move.enable = false;
                    controls.cursor3D.enable = false;
                    controls.selectArea.enable = true;
                    inputHandlersQueue.push(controls.selectArea.control);
                    controls.cameraControl.enable = true;
                    controls.cameraControl.control.rotationEnabled = true;
                    inputHandlersQueue.push(controls.cameraControl.control);
                    controls.measurementEdit.enable = true;
                    inputHandlersQueue.push(controls.measurementEdit.control);
                    controls.select.enable = true;
                    controls.select.control.forceMultiSelection = false;
                    inputHandlersQueue.push(controls.select.control);
                    controls.selectionEditTools.enable = true;
                    controls.onBarElementEdit.enable = true;
                    inputHandlersQueue.push(controls.onBarElementEdit.control);
                    controls.propertiesWindow.enable = true;
                    break;
                case MODE.CUT:
                    controls.move.enable = false;
                    controls.cursor3D.enable = false;
                    controls.cameraControl.enable = true;
                    controls.cameraControl.control.rotationEnabled = true;
                    inputHandlersQueue.push(controls.cameraControl.control);
                    controls.barCut.enable = true;
                    inputHandlersQueue.push(controls.barCut.control);
                    break;
                case MODE.SNAP:
                    controls.move.enable = false;
                    controls.cursor3D.enable = false;
                    controls.cameraControl.enable = true;
                    controls.cameraControl.control.rotationEnabled = true;
                    inputHandlersQueue.push(controls.cameraControl.control);
                    controls.barSnap.enable = true;
                    inputHandlersQueue.push(controls.barSnap.control);
                    break;
                case MODE.MOVE:
                    controls.move.enable = true;
                    inputHandlersQueue.push(controls.move.control);
                    controls.cameraControl.enable = true;
                    controls.cameraControl.control.rotationEnabled = true;
                    inputHandlersQueue.push(controls.cameraControl.control);
                    controls.measurementEdit.enable = true;
                    inputHandlersQueue.push(controls.measurementEdit.control);
                    controls.select.enable = true;
                    controls.select.control.forceMultiSelection = false;
                    inputHandlersQueue.push(controls.select.control);
                    controls.propertiesWindow.enable = true;
                    break;
                case MODE.EDIT_NODE_FORCE:
                    controls.cameraControl.enable = true;
                    inputHandlersQueue.push(controls.cameraControl.control);
                    controls.select.enable = true;
                    controls.select.control.forceMultiSelection = true;
                    controls.nodeForceEdit.enable = true;
                    inputHandlersQueue.push(controls.select.control);
                    controls.propertiesWindow.enable = true;
                    break;
                case MODE.EDIT_NODE_MOMENT:
                    controls.cameraControl.enable = true;
                    inputHandlersQueue.push(controls.cameraControl.control);
                    controls.select.enable = true;
                    controls.select.control.forceMultiSelection = true;
                    controls.nodeForceEdit.enable = true;
                    inputHandlersQueue.push(controls.select.control);
                    controls.propertiesWindow.enable = true;
                    break;
                case MODE.EDIT_BAR_FORCE:
                    controls.cameraControl.enable = true;
                    inputHandlersQueue.push(controls.cameraControl.control);
                    controls.barForceEdit.enable = true;
                    inputHandlersQueue.push(controls.barForceEdit.control);
                    controls.propertiesWindow.enable = true;
                    controls.onBarElementEdit.enable = true;
                    inputHandlersQueue.push(controls.onBarElementEdit.control);
                    break;
                case MODE.EDIT_BAR_UNIFORM:
                    controls.cameraControl.enable = true;
                    inputHandlersQueue.push(controls.cameraControl.control);
                    controls.barUniformEdit.enable = true;
                    inputHandlersQueue.push(controls.barUniformEdit.control);
                    controls.propertiesWindow.enable = true;
                    break;
                case MODE.EDIT_BAR_TRAPEZOIDAL:
                    controls.cameraControl.enable = true;
                    inputHandlersQueue.push(controls.cameraControl.control);
                    controls.barTrapezoidalEdit.enable = true;
                    inputHandlersQueue.push(controls.barTrapezoidalEdit.control);
                    controls.propertiesWindow.enable = true;
                    controls.onBarElementEdit.enable = true;
                    inputHandlersQueue.push(controls.onBarElementEdit.control);
                    break;
                case MODE.EDIT_SUPPORT:
                    controls.cameraControl.enable = true;
                    inputHandlersQueue.push(controls.cameraControl.control);
                    controls.nodeSupportEdit.enable = true;
                    inputHandlersQueue.push(controls.nodeSupportEdit.control);
                    controls.propertiesWindow.enable = true;
                    break;
                case MODE.ZOOM_AREA:
                    controls.propertiesWindow.enable = true;
                    controls.cameraControl.enable = true;
                    controls.cameraControl.control.rotationEnabled = false;
                    inputHandlersQueue.push(controls.cameraControl.control);
                    controls.zoomArea.enable = true;
                    inputHandlersQueue.push(controls.zoomArea.control);
                    controls.select.enable = true;
                    controls.select.control.forceMultiSelection = false;
                    inputHandlersQueue.push(controls.select.control);
                    break;
                case MODE.EDIT:
                    controls.edit.enable = true;
                    inputHandlersQueue.push(controls.edit.control);
                    controls.measurementEdit.enable = true;
                    inputHandlersQueue.push(controls.measurementEdit.control);
                    controls.cameraControl.enable = true;
                    controls.cameraControl.control.rotationEnabled = true;
                    inputHandlersQueue.push(controls.cameraControl.control);
                    controls.select.enable = true;
                    controls.select.control.forceMultiSelection = false;
                    inputHandlersQueue.push(controls.select.control);
                    break;
                case MODE.MOVE_CONSTRUCTION:
                    controls.moveConstruction.enable = true;
                    inputHandlersQueue.push(controls.moveConstruction.control);
                    break;
                case MODE.EDIT_BOX:
                    controls.cursor3D.enable = true;
                    inputHandlersQueue.push(controls.cursor3D.control);
                    controls.cameraControl.enable = true;
                    inputHandlersQueue.push(controls.cameraControl.control);
                    controls.boxEdit.enable = true;
                    inputHandlersQueue.push(controls.boxEdit.control);
                    controls.propertiesWindow.enable = true;
                    break;
                }
                view3D.editMode(editmode);
                var controlName;
                for (controlName in controls) {
                    if (controls.hasOwnProperty(controlName)) {
                        if (!controls[controlName].enable) {
                            controls[controlName].control.disable();
                        }
                    }
                }
                for (controlName in controls) {
                    if (controls.hasOwnProperty(controlName)) {
                        if (controls[controlName].enable) {
                            controls[controlName].control.enable();
                        }
                    }
                }
                view3D.editMode(editmode);
            };
            var onMouseUp = function(event) {
                // Do not process if clicked on scrollbar (chrome B.U.G?)
                if (event.target.tagName === "HTML" || event.target.tagName !== "CANVAS") {
                    return;
                }
                for (var i = 0; i < inputHandlersQueue.length; i++) {
                    if (inputHandlersQueue[i].onMouseUp !== undefined) {
                        if (inputHandlersQueue[i].onMouseUp(event) === true) {
                            break;
                        }
                    }
                }
                event.preventDefault();
            };
            var onMouseDown = function(event) {
                // user just clicked on view3d div so we want to still focus from any active
                // element this will update model if user was editing something and clicks on view 3d
                if (event.target.tagName !== "CANVAS") {
                    return;
                }
                document.activeElement.blur();
                for (var i = 0; i < inputHandlersQueue.length; i++) {
                    if (inputHandlersQueue[i].onMouseDown !== undefined) {
                        if (inputHandlersQueue[i].onMouseDown(event) === true) {
                            break;
                        }
                    }
                }
                event.preventDefault();
            };
            var lastMouseMoveEvent = null;
            var onMouseMove = function(event) {
                lastMouseMoveEvent = event;
            };
            this.processLastMouseMoveEvent = function() {
                if (lastMouseMoveEvent !== null) {
                    for (var i = 0; i < inputHandlersQueue.length; i++) {
                        if (inputHandlersQueue[i].onMouseMove !== undefined) {

                            if (inputHandlersQueue[i].onMouseMove(lastMouseMoveEvent) === true) {
                                break;
                            }
                        }
                    }
                }
                lastMouseMoveEvent = null;
            };
            var onMouseWheel = function(event) {
                for (var i = 0; i < inputHandlersQueue.length; i++) {
                    if (inputHandlersQueue[i].onMouseWheel !== undefined) {
                        if (inputHandlersQueue[i].onMouseWheel(event) === true) {
                            break;
                        }
                    }
                }
                event.stopPropagation();
                event.preventDefault();
            };
            var onKeyUp = function(event) {
                if (VIEWUI.contextMenu.onKeyUp(event)) {
                    event.stopPropagation();
                    event.preventDefault();
                    return;
                }
                for (var i = 0; i < inputHandlersQueue.length; i++) {
                    if (inputHandlersQueue[i].onKeyUp !== undefined) {
                        if (inputHandlersQueue[i].onKeyUp(event) === true) {
                            event.stopPropagation();
                            event.preventDefault();
                            return;
                        }
                    }
                }
                HELPERS.inputState.onKeyUp(event);
            };
            window.addEventListener("keyup", onKeyUp);
            domElement.addEventListener("mousedown", onMouseDown);
            window.addEventListener("mouseup", onMouseUp);
            window.addEventListener("mousemove", onMouseMove);
            domElement.addEventListener("mousewheel", onMouseWheel); // TODO: change to wheel event
            domElement.addEventListener("DOMMouseScroll", onMouseWheel); // for firefox, delete later
            this.updateStatusSub = HELPERS.subscribePropertyChange(this, mode, "mode", updateStatus);
            updateStatus(mode());

            this.disposeSubscriptions = function () {
                window.removeEventListener("keyup", onKeyUp);
                domElement.removeEventListener("mousedown", onMouseDown);
                window.removeEventListener("mouseup", onMouseUp);
                window.removeEventListener("mousemove", onMouseMove);
                domElement.removeEventListener("mousewheel", onMouseWheel); // TODO: change to wheel event
                domElement.removeEventListener("DOMMouseScroll", onMouseWheel); // for firefox, delete later
                this.updateStatusSub.dispose();
            }
        };
    });
