﻿'use strict';
var VIEW3D = VIEW3D || {}
var CURSOR = {
    NONE: -1,
    ON_NODE: 0,
    ON_BAR: 1,
    ON_BAR_MIDDLE: 2,
    PERPENDICULAR: 3,
    COUNT: 4
}
VIEW3D.controls = VIEW3D.controls || {}

define(['js/view3d/materials/VIEW3D.materials.Cursor3D', 'js/view3d/VIEW3D.SnapingGuides'], function () {

    VIEW3D.controls.Cursor3D = function (parent, camera, constructionView, mouse3D) {
        this.position = new THREE.Vector3(0, 0, 0);
        this.object = null; //null, node, bar
        this.changeEvent = ko.observable();
        this.cursorType = CURSOR.NONE;
        //this.orthoPoint = null;
        this.guideLines = ko.observableArray(); // of VIEW3D.controls.Cursor3D.GuideLines
        this.mouse3D = mouse3D;
        this.addCrossGuide = function (pos) {
            pos = pos.clone();
            this.guideLines.push(new VIEW3D.controls.Cursor3D.GuideLines( pos , new THREE.Vector3(1,0,0)));
            this.guideLines.push(new VIEW3D.controls.Cursor3D.GuideLines( pos , new THREE.Vector3(0,1,0)));
            this.guideLines.push(new VIEW3D.controls.Cursor3D.GuideLines( pos , new THREE.Vector3(0,0,1)));
        }

        this.clearGuides = function () {
            this.guideLines([]);
        }

        this.addVerticalGuide = function (pos) {
            pos = pos.clone();
            this.guideLines.push(new VIEW3D.controls.Cursor3D.GuideLines( pos , new THREE.Vector3(0,0,1)));
        }

        this.getObjectType = function () {
            return this.object !== null ? this.object.type : null;
        }
        this.snapNodes = null; // if is not null then snaping is enabled
        this.startNode = null; // start node in bar drawing
        var enabled = false;
        var self = this;
        var snap = new VIEW3D.controls.Snap(scene, camera, constructionView);
        var snapView = new VIEW3D.SnapingGuides(snap);
        parent.add(snapView);
        var createMesh = function () {
            var geometry = new THREE.BufferGeometry();
            var halfSize = 0.03;
            var pos = new Float32Array(6 * 2);
            pos[0 * 2 + 0] = -halfSize;
            pos[0 * 2 + 1] = -halfSize;
            pos[1 * 2 + 0] = halfSize;
            pos[1 * 2 + 1] = -halfSize;
            pos[2 * 2 + 0] = -halfSize;
            pos[2 * 2 + 1] = halfSize;
            pos[3 * 2 + 0] = halfSize;
            pos[3 * 2 + 1] = -halfSize;
            pos[4 * 2 + 0] = halfSize;
            pos[4 * 2 + 1] = halfSize;
            pos[5 * 2 + 0] = -halfSize;
            pos[5 * 2 + 1] = halfSize;
            var uv = new Float32Array(6 * 2);
            var w = 1.0 / CURSOR.COUNT;
            uv[0 * 2 + 0] = 0;
            uv[0 * 2 + 1] = 0.0;
            uv[1 * 2 + 0] = w;
            uv[1 * 2 + 1] = 0.0;
            uv[2 * 2 + 0] = 0;
            uv[2 * 2 + 1] = 1.0;
            uv[3 * 2 + 0] = w;
            uv[3 * 2 + 1] = 0.0;
            uv[4 * 2 + 0] = w;
            uv[4 * 2 + 1] = 1.0;
            uv[5 * 2 + 0] = 0;
            uv[5 * 2 + 1] = 1.0;
            geometry.addAttribute('position', new THREE.BufferAttribute(pos, 2));
            geometry.addAttribute('uv', new THREE.BufferAttribute(uv, 2));
            var material = new VIEW3D.materials.Cursor3D(CORE.aspectRatio, CORE.res.texture["3dcursors"]);
            var mesh = new THREE.Mesh(geometry, material);
            mesh.frustumCulled = false;
            return mesh;
        }
        var mesh = createMesh();
        parent.add(mesh);
        mesh.visible = false;
        var getBarMiddle = function (bar) {
            return HELPERS.toVector3(bar.node1().position).add(HELPERS.toVector3(bar.node2().position)).multiplyScalar(0.5);
        }
        var updateCursor = function () {
            if (self.cursorType === CURSOR.NONE) {
                mesh.visible = false;
            } else {
                mesh.visible = true;
                mesh.material.uniforms.pos3D.value = self.position;
                mesh.material.uniforms.uOffset.value = self.cursorType / CURSOR.COUNT;
            }
            render();
        }
        var projectionToStartNode = function (bar) {
            if (!isEmpty(self.startNode)) {
                return bar.vectorProjection(self.startNode.position());
            }
            return null;
        }
        var onMousePosChange = function () {
            // if(self.guideLines().length > 0) {
            //     console.log("Lines ON");
            // } else {
            //     console.log("Lines OFF");
            // }

            //snap.hide();
            var threshold = 0.4 / camera.zoom;
            var object = mouse3D.object();
            var pos = isEmpty(mouse3D.position()) ? null : HELPERS.toVector3(mouse3D.position());
            //console.log('Cursor3D : X : ' + pos.x + ', Y :' + pos.y + ', Z :' + pos.z );
            if (!isEmpty(object) && !isEmpty(pos)) {
                if (Array.isArray(object)) {
                    // on two bars
                    self.position = pos;
                    self.object = object[0];
                    self.cursorType = CURSOR.ON_BAR;
                } else if (object.type === MODEL.OBJECT_TYPE.Node) {
                    //on node
                    self.position = HELPERS.toVector3(object.position());
                    self.object = object;
                    self.cursorType = CURSOR.ON_NODE;
                } else if (object.type === MODEL.OBJECT_TYPE.Bar) {
                    //on one bar
                    self.object = object;
                    var barMiddle = getBarMiddle(object);
                    var onBarPos = HELPERS.toVector3(object.vectorProjection(new MODEL.types.Vector(pos.x, pos.y, pos.z)));

                    if (onBarPos.distanceTo(barMiddle) < threshold) {
                        // on bar center
                        self.position = barMiddle;
                        self.cursorType = CURSOR.ON_BAR_MIDDLE;
                    } else {
                        var proj = projectionToStartNode(object);
                        //perpendicular to start node?
                        if (proj !== null && proj.squaredDistanceTo(onBarPos) < threshold * threshold) {
                            self.position = HELPERS.toVector3(proj);
                            self.cursorType = CURSOR.PERPENDICULAR;
                        } else {
                            if (self.snapNodes !== null) {
                                // snap to any node but keep it on bar
                                var s = object.node1().position();
                                var e = object.node2().position();
                                var dir = new THREE.Vector3(e.x - s.x, e.y - s.y, e.z - s.z);
                                dir.normalize();
                                snap.snap(onBarPos, dir, threshold, self.snapNodes);
                            }
                            self.position = onBarPos;
                            self.cursorType = CURSOR.ON_BAR;
                        }
                    }
                } 
                // else if ( self.guideLines().length > 0 ) {
                //     //set position on ortho cross
                //     self.snapToGuideLine( CORE.constructionView3D.activeWorkplane.step );
                //     self.cursorType = CURSOR.NONE;
                //     self.object = null;
                // }
                else {
                    if(mouse3D.drawingPlane != null) {
                        var newPos = mouse3D.calculateDrawingPlaneIntersection();
                        if(newPos !== undefined) pos = newPos;
                    }
                    pos = self.orthogonalHelper.calculateCurrentPointPosition(pos);
                    //workplane
                    self.object = object;
                    //console.log('Snap workplane Cursor3d Before X : ' + pos.x + ', Y :' + pos.y + ', Z :' + pos.z );
                    if (self.snapNodes !== null) {

                        var hasSnapped = snap.snapPointOnPlane(pos, mouse3D.getActivePlaneDef(), self.snapNodes, threshold, CORE.constructionView3D.activeWorkplane.step);
                    }
                    //console.log('Snap workplane Cursor3d After X : ' + pos.x + ', Y :' + pos.y + ', Z :' + pos.z );
                    self.position = pos;
                    self.cursorType = CURSOR.NONE;
                }
            } else {
                //no object under moise
                self.object = null;
                self.cursorType = CURSOR.NONE;
            }
            self.changeEvent.notifySubscribers();
            updateCursor();
        }

        var mouse3DSub = null;
        this.enable = function () {
            if (enabled === true) {
                return;
            }
            mouse3DSub = mouse3D.position.subscribe(onMousePosChange);
            enabled = true;
        }
        this.disable = function () {
            if (enabled === false) {
                return;
            }
            mouse3DSub.dispose();
            enabled = false;
            mesh.visible = false;
            snap.clearGuides();
            //snap.hide();
            render();
        }
        this.isEnabled = function() {
            return enabled;
        }

        this.snapToGuideLine = function (gridstep) {

            var v0 = new THREE.Vector3();
            var v1 = new THREE.Vector3();
            var offset = new THREE.Vector3();
            var ray = mouse3D.raycaster.ray;
            var pos = new THREE.Vector3();
            var precision = 0.2 / camera.zoom;
            var minDist = Number.MAX_VALUE;

            for (var i = 0; i < this.guideLines().length; i++) {
                var line = this.guideLines()[i];
                offset.copy(line.dir).multiplyScalar(10000);
                v0.copy(line.origin).add(offset);
                v1.copy(line.origin).add(offset.multiplyScalar(-1));
                var dist = ray.distanceSqToSegment(v0, v1, null, pos);
                if (dist < minDist) {
                    if(!snap.snap(pos, line.dir, precision, this.snapNodes)){
                        if(line.dir.x > 0.999) {
                            var nearestGridX = Math.round(pos.x / gridstep) * gridstep;
                            if (Math.abs(nearestGridX - pos.x) <= precision) {
                                pos.x = nearestGridX;
                            }
                        }
                        if(line.dir.y > 0.999) {
                            var nearestGridY = Math.round(pos.y / gridstep) * gridstep;
                            if (Math.abs(nearestGridY - pos.y) <= precision) {
                                pos.y = nearestGridY;
                            }
                        }
                        if(line.dir.z > 0.999) {
                            var nearestGridZ = Math.round(pos.z / gridstep) * gridstep;
                            if (Math.abs(nearestGridZ - pos.z) <= precision) {
                                pos.z = nearestGridZ;
                            }
                        }

                    }
                    this.position.copy(pos);
                    minDist = dist;
                }
            }
        }
    }

    VIEW3D.controls.Cursor3D.GuideLines = function (origin, dir) {
        //immutable
        Object.defineProperty(this, 'origin', { value: origin, writable: false, enumerable: true });
        Object.defineProperty(this, 'dir', { value: dir, writable: false, enumerable: true });
    }
});
