'use strict';
var VIEW3D = VIEW3D || {};
VIEW3D.controls = VIEW3D.controls || {};

define(['js/view3d/VIEW3D.BarEdit', 'js/view3d/VIEW3D.NodeEdit'],function(){
    VIEW3D.controls.EditMap = {}; // model type -> edit control class
    VIEW3D.controls.Edit = function(selectedObjects, mouse3D) {

        var currentEditControl = null;
        var selectionSub = null;
        var onSelectionChange = function() {
            if(currentEditControl !== null) {
                currentEditControl.dispose();
                currentEditControl = null;
            }

            if(selectedObjects().length === 1) {
                var selectedObject = selectedObjects()[0];
                var editControl = VIEW3D.controls.EditMap[selectedObject.type];
                if(!isEmpty(editControl)) {
                    currentEditControl = new editControl(selectedObject, mouse3D);
                    currentEditControl.initialize();
                }
            }
        };

        this.enable = function() {
            selectionSub = HELPERS.subscribeArrayChange(this, selectedObjects, "selectedObjects", onSelectionChange, onSelectionChange);
            onSelectionChange();
        }

        this.disable = function() {
            if(currentEditControl !== null) {
                currentEditControl.dispose();
                currentEditControl = null;
            }
            if(selectionSub !== null) {
                selectionSub.dispose();
                selectionSub = null;
            }
        }

        this.onMouseDown = function (event) {
            if(currentEditControl !== null) {
                return currentEditControl.onMouseDown(event);
            }
            return false;
        }
        this.onMouseUp = function (event) {
            if(currentEditControl !== null) {
                return currentEditControl.onMouseUp(event);
            }
            return false;
        }
        this.onMouseMove = function (event) {
            if(currentEditControl !== null) {
                return currentEditControl.onMouseMove(event);
            }
            return false;
        }

    };

    VIEW3D.controls.EditBar = function(bar, mouse3D) {
        var control3d = null;
        var editControl = null;
        var drag = false;

        this.initialize = function () {
            //VIEWUI.barEdit.show(bar);
            control3d = new VIEW3D.BarEdit(bar, CORE.model.bars(), CORE.model.constraints);
            scene.add(control3d);
            mouse3D.controllsCollisions.addCollider(control3d.getCollider());
        };

        this.onMouseDown = function (event) {
            var objectUnderMouse = mouse3D.object()
            if(objectUnderMouse && objectUnderMouse.type ==="EditControl") {
                editControl = objectUnderMouse;
                editControl.startDrag(mouse3D.raycaster.ray);
                drag = true
                return true;
            }
            return false;
        }

        this.onMouseUp = function (event) {
            var result = false;
            if(editControl !== null) {
                if(drag) {
                    editControl.endDrag();
                    result = true;
                }
            }
            drag = false;
            return result;
        }

        this.onMouseMove = function (event) {
            if(drag) {
                var ray = mouse3D.raycaster.ray;
                editControl.drag(ray);
                return true;
            }
            return false;
        }


        this.dispose = function() {
            mouse3D.controllsCollisions.removeCollider(control3d.getCollider());
            if(editControl) {
                editControl.dispose();
            }
            scene.remove(control3d);
            editControl = null;
            drag = false;
        }

    };

    VIEW3D.controls.EditMap[MODEL.OBJECT_TYPE.Bar] = VIEW3D.controls.EditBar;


    // VIEW3D.controls.EditNode = function(node, mouse3D) {
    //     var control3d = null;
    //     var editControl = null;
    //     var drag = false;

    //     this.initialize = function () {
    //         var ignore = false;
    //         if (!ignore) {
    //             control3d = new VIEW3D.PositionEdit(node.position, CORE.model.bars());
    //             scene.add(control3d);
    //             mouse3D.controllsCollisions.addCollider(control3d.getCollider());
    //         }
    //     };

    //     this.onMouseDown = function (event) {
    //         var objectUnderMouse = mouse3D.object()
    //         if(objectUnderMouse && objectUnderMouse.type ==="EditControl") {
    //             editControl = objectUnderMouse;
    //             editControl.startDrag(mouse3D.raycaster.ray);
    //             drag = true
    //             return true;
    //         }
    //         return false;
    //     }

    //     this.onMouseUp = function (event) {
    //         var result = false;
    //         if(editControl !== null) {
    //             if(drag) {
    //                 editControl.endDrag();
    //                 result = true;
    //             }
    //         }
    //         drag = false;
    //         return result;
    //     }

    //     this.onMouseMove = function (event) {
    //         if(drag) {
    //             var ray = mouse3D.raycaster.ray;
    //             editControl.drag(ray);
    //             return true;
    //         }
    //         return false;
    //     }


    //     this.dispose = function () {
    //         if (!isEmpty(control3d)) {
    //             mouse3D.controllsCollisions.removeCollider(control3d.getCollider());
    //             if (editControl) {
    //                 editControl.dispose();
    //             }
    //             scene.remove(control3d);
    //             editControl = null;
    //             drag = false;
    //         }
    //     }

    // };

    // VIEW3D.controls.EditBox = function(box, mouse3D) {
    //     var control3d = null;
    //     var editControl = null;
    //     var drag = false;

    //     this.initialize = function () {
    //         control3d = new VIEW3D.PositionEdit(box.origin, CORE.model.bars());
    //         scene.add(control3d);
    //         mouse3D.controllsCollisions.addCollider(control3d.getCollider());
    //     };

    //     this.onMouseDown = function (event) {
    //         var objectUnderMouse = mouse3D.object()
    //         if(objectUnderMouse && objectUnderMouse.type ==="EditControl") {
    //             editControl = objectUnderMouse;
    //             editControl.startDrag(mouse3D.raycaster.ray);
    //             drag = true
    //             return true;
    //         }
    //         return false;
    //     }

    //     this.onMouseUp = function (event) {
    //         var result = false;
    //         if(editControl !== null) {
    //             if(drag) {
    //                 editControl.endDrag();
    //                 result = true;
    //             }
    //         }
    //         drag = false;
    //         return result;
    //     }

    //     this.onMouseMove = function (event) {
    //         if(drag) {
    //             var ray = mouse3D.raycaster.ray;
    //             editControl.drag(ray);
    //             return true;
    //         }
    //         return false;
    //     }


    //     this.dispose = function () {
    //         if (!isEmpty(control3d)) {
    //             mouse3D.controllsCollisions.removeCollider(control3d.getCollider());
    //             if (editControl) {
    //                 editControl.dispose();
    //             }
    //             scene.remove(control3d);
    //             editControl = null;
    //             drag = false;
    //         }
    //     }

    // };

    VIEW3D.controls.EditMap[MODEL.OBJECT_TYPE.Bar] = VIEW3D.controls.EditBar;
    // VIEW3D.controls.EditMap[MODEL.OBJECT_TYPE.Node] = VIEW3D.controls.EditNode;
    // VIEW3D.controls.EditMap[MODEL.OBJECT_TYPE.Box] = VIEW3D.controls.EditBox;

});