﻿'use strict';
var VIEW3D = VIEW3D || {}
VIEW3D.controls = VIEW3D.controls || {}


VIEW3D.controls.Mouse3D = function (camera, constructionView, measurementsView, canvasElement, viewName) {  // viewName - for debugging
    this.canvasElement = canvasElement || canvas;      // default - 3d view canvas
    this.position = ko.observable(new MODEL.types.Vector());
    this.object = ko.observable(null);
    this.viewName = viewName;
    this.lastPageX = null;
    this.lastPageY = null;

    var enabled = false;
    var self = this;
    var _resetObject = false;

    var collisionRoot = new VIEW3D.collision.CollidersContainer();

    this.controllsCollisions  = new VIEW3D.collision.CollidersContainer();
    this.constructionColliders = constructionView.getCollider();

    collisionRoot.addCollider(this.controllsCollisions);
    collisionRoot.addCollider(this.constructionColliders);

    this.enable = function() {
        if (enabled === true) {
            return;
        }
        enabled = true;
    }
    this.disable = function() {
        if (enabled === false) {
            return;
        }
        enabled = false;
    }

    var pixelHalfSize = function () {
        return {
            x: 1.0 / self.canvasElement.clientWidth,
            y: 1.0 / self.canvasElement.clientHeight
        }
    }

    //returns nearest point to ray on bar 
    this.getNearestToRay = function(bar, ray, clip) {
        var p0 = bar.node1().position();
        var pD = bar.node2().position().sub(p0);
        var q0 = new MODEL.types.Vector(ray.origin.x, ray.origin.y, ray.origin.z);
        var qD = new MODEL.types.Vector(ray.direction.x, ray.direction.y, ray.direction.z);
        var w0 = p0.sub(q0);
        var a = pD.dot(pD);
        var b = pD.dot(qD);
        var c = qD.dot(qD);
        var d = pD.dot(w0);
        var e = qD.dot(w0);
        var sc = (b * e - c * d) / (a * c - b * b);
        if(clip) {
            sc = Math.min(Math.max(sc, 0),1);
        }
        return p0.add(pD.multiplyScalar(sc));
    }

    var findCollision = function (raycaster, mouse) {


        //move tool arrows
        if (!isEmpty(CORE.view3D.moveControl)) {
            var intersection = CORE.view3D.moveControl.getArrow(mouse); // circular dependency workaround
            if (intersection != null) {
                self.object(intersection.userData);  // "arrow" object is set
                self.position(new THREE.Vector3().setFromMatrixPosition(intersection.matrixWorld));
                //console.log("OVER MOVE ARROW");
                return;
            }
        }


        var halfSize = pixelHalfSize();

        //console.time("mouseCollisionTest");

        var intersections = collisionRoot.position2DIntersection(mouse, camera, halfSize);
        //console.timeEnd("mouseCollisionTest");

        if (intersections.length > 0) {
            var firstIntersection = intersections[0];
            if (!isEmpty(firstIntersection.collider.model)) {
                var model = firstIntersection.collider.model;
                switch (model.type) {
                    case MODEL.OBJECT_TYPE.Node:
                        //WARNING! set object before position because cursor 3d is watching position
                        //and if its changed than assumes that that object is set
                        self.object(model);
                        self.position(model.position());//node position
                        break;
                    case MODEL.OBJECT_TYPE.Bar:
                        self.object(model);
                        self.position(firstIntersection.position);
                        break;
                    default:
                        model.lastMouseOverPosition = firstIntersection.position;
                        self.object(model);
                        self.position(firstIntersection.position);
                        break;

                }

                if (_resetObject) {
                    setTimeout(function () {
                        self.object.valueHasMutated();  // for refreshing highlight when changing bar's length
                        _resetObject = false;
                    }, 300);
                }

                return;
            }
        }

        // if(self.drawingPlane !== null)  {

        // }


        if (!isEmpty(constructionView.activeWorkplane)) {
            var intersects = raycaster.intersectObject(constructionView.activeWorkplane);
            if (intersects.length > 0) {
                var pos = intersects[0].point;
                self.position(new MODEL.types.Vector(pos.x, pos.y, pos.z));
                // console.log('Mouse3d Active plane X : ' + pos.x + ', Y :' + pos.y + ', Z :' + pos.z );
                if (self.object() !== constructionView.activeWorkplane.model) {
                    self.object(constructionView.activeWorkplane.model);
                }
                // console.log("OVER WORKPLANE");
                return;
            }
        }
        for(var i = 0 ; i < constructionView.workplanes.length ; i++) {
            var plane = constructionView.workplanes[i];

            var intersects = raycaster.intersectObject(plane);
            if (intersects.length > 0) {
                var pos = intersects[0].point;
                self.position(new MODEL.types.Vector(pos.x, pos.y, pos.z));
                if (self.object() !== plane.model) {
                    self.object(plane.model);
                }
                return;
            }
        }

        //self.position -> no change
        self.object(null);
    }
    var createRaycaster = function(mouse) {
        var raycaster = new THREE.Raycaster();
        raycaster.setFromCamera(mouse, camera);
        var precision = 0.2 / camera.zoom;
        raycaster.params.Points.threshold = precision * 2;
        raycaster.linePrecision = precision;
        return raycaster;
    }
    //var updateCursor = function () {
    //    if (self.cursorType == CURSOR.NONE) {
    //        mesh.visible = false;
    //    } else {
    //        mesh.visible = true;
    //        mesh.material.uniforms.pos3D.value = self.position;
    //        mesh.material.uniforms.uOffset.value = self.cursorType / CURSOR.COUNT;
    //    }
    //}
    this.onMouseMove = function(event) {
        self.mouseX = event.pageX - ($(self.canvasElement).offset().left);
        self.mouseY = event.pageY - ($(self.canvasElement).offset().top);
        
        if (enabled === false || event.target !== self.canvasElement) {
            return false;
        }
        self.lastPageX = event.pageX;
        self.lastPageY = event.pageY;

        var mouse = new THREE.Vector2();
        mouse.x = ((event.pageX - ($(self.canvasElement).offset().left)) / self.canvasElement.clientWidth) * 2 - 1;
        mouse.y = -((event.pageY - ($(self.canvasElement).offset().top)) / self.canvasElement.clientHeight) * 2 + 1;
        
        this.raycaster = createRaycaster(mouse);
        findCollision(this.raycaster, mouse);
        return false;
    }

    this.onMouseOut = function () { // clear e.g. highlight when mouse out of renderer area
        this.object(null);
    }

    this.getActivePlaneDef = function() {
        if(self.drawingPlane === null) {
            return constructionView.activeWorkplane.planeMath;
        }
        return self.drawingPlane;
    }
    this.drawingPlane = null;

    this.calculateDrawingPlaneIntersection = function() {
        var dir = self.raycaster.ray.direction.clone().multiplyScalar(10000.0);
        var start = self.raycaster.ray.origin.clone().sub(dir);
        var end = self.raycaster.ray.origin.clone().add(dir);
        var line = new THREE.Line3(start, end);
        var intersection = self.drawingPlane.intersectLine(line);

        return intersection;
    }

    CORE.model.modelChange.subscribe(function () {      // for refreshing highlight when changing model
        _resetObject = true;
        if (self.lastPageX !== null) self.onMouseMove({ pageX: self.lastPageX, pageY: self.lastPageY });
    })
    
}