﻿'use strict';
var VIEW3D = VIEW3D || {}
VIEW3D.controls = VIEW3D.controls || {}

define(['js/view3d/VIEW3D.EditedElements', 'js/view3d/VIEW3D.Arrows'], function () {

    VIEW3D.controls.Move = function (camera, model, constructionView3D, mouse3d, snap, aspectRatio) {
        var enabled = false;
        var visualization = null;
        var startPos = null;
        var target = null;
        var startNode = null;
        var startData = null;
        var data = null;
        this.subscriptions = [];
        this.isMoving = false;
        this.subscription1 = null;
        //this.previousMouseX = null;
        //this.previousMouseY = null;
        //this.currentArrow = null;   // currently moving arrow
        this.currentDir = null      // currently moving axis
        this.valueFromInput = ko.observable(0);  // value from the textbox
        this.initialDistance = 0;   // distance when moving for the first time
        this.ignoreValueFromInputChange = false;    // ignore input change if changed by the control, not by user
        this.isTextBoxVisible = false;
        this.hasCameraMoved = false;    // true if user moved the camera by moving the mouse

        var mouseStart = { x: 0, y: 0 };

        this.getOffset = function (event, mouseClickPos) {
            var mouse = new THREE.Vector2();
            mouse.x = ((event.pageX - ($(canvas).offset().left)) / canvas.clientWidth) * 2 - 1;
            mouse.y = -((event.pageY - ($(canvas).offset().top)) / canvas.clientHeight) * 2 + 1;
            var offset;

            var raycaster = new THREE.Raycaster();
            raycaster.setFromCamera(mouse, camera);
            var ray = raycaster.ray;

            var lineStart = startPos.clone();
            var lineEnd = startPos.clone();

            switch (this.currentDir) {
                case "x":
                    lineStart.x += 10000;
                    lineEnd.x -= 10000;
                    break;
                case "y":
                    lineStart.y += 10000;
                    lineEnd.y -= 10000;
                    break;
                case "z":
                    lineStart.z += 10000;
                    lineEnd.z -= 10000;
                    break;
            }
            var pos = new THREE.Vector3();
            var minDist = ray.distanceSqToSegment(lineStart, lineEnd, null, pos);
            pos.sub(mouseClickPos);
            var posX = Math.round(pos.x * epsilonGrid) / epsilonGrid;
            var posY = Math.round(pos.y * epsilonGrid) / epsilonGrid;
            var posZ = Math.round(pos.z * epsilonGrid) / epsilonGrid;
            switch (this.currentDir) {
                case 'x':
                    offset = new THREE.Vector3(posX, 0, 0);
                    break;
                case 'y':
                    offset = new THREE.Vector3(0, posY, 0);
                    break;
                case 'z':
                    offset = new THREE.Vector3(0, 0, posZ);
                    break;
            }
            return offset;
        }

        this.getOffsetVector = function (offsetValue) {

            var offsetVector = new THREE.Vector3(0, 0, 0);
            offsetVector[this.currentDir] = offsetValue;

            return offsetVector;
        }

        var getNodesToMove = function () {
            var result = [];
            for (var i = 0; i < model.nodes().length; i++) {
                var node = model.nodes()[i];
                if (node.selected()) {
                    result.push(node);
                    continue;
                }
                var bars = node.createsBars();
                for (var j = 0; j < bars.length; j++) {
                    if (bars[j].selected()) {
                        result.push(node);
                        break;
                    }
                }
            }
            return result;
        }
        var getBarsMove = function () {
            var result = [];
            for (var i = 0; i < model.bars().length; i++) {
                var j;
                var bar = model.bars()[i];
                var moveStart = bar.node1().selected();
                for (j = 0; j < bar.node1().createsBars().length; j++) {
                    if (bar.node1().createsBars()[j].selected()) {
                        moveStart = true;
                    }
                }
                var moveEnd = bar.node2().selected();
                for (j = 0; j < bar.node2().createsBars().length; j++) {
                    if (bar.node2().createsBars()[j].selected()) {
                        moveEnd = true;
                    }
                }
                if (bar.selected() || moveStart || moveEnd) {
                    result.push({
                        start: {
                            x: bar.node1().position().x,
                            y: bar.node1().position().y,
                            z: bar.node1().position().z,
                            move: bar.selected() || moveStart
                        },
                        end: {
                            x: bar.node2().position().x,
                            y: bar.node2().position().y,
                            z: bar.node2().position().z,
                            move: bar.selected() || moveEnd
                        }
                    });
                }
            }
            return result;
        }
        this.enable = function () {
            if (enabled === true) {
                return;
            }
            enabled = true;
            this.subscription1 = HELPERS.subscribeArrayChange(this, model.selectedObjects, "MODEL.loads.node.assignedTo", this.onObjectSelected, this.onObjectSelected);
            this.onObjectSelected();
        }
        this.disable = function () {
            if (enabled === false) {
                return;
            }
            enabled = false;

            if (!isEmpty(this.subscription1))
                this.subscription1.dispose();

            HELPERS.removeFromScene(this.arrows3d, scene);
            render();

            this.updateTextBox(null);
        }

        this.updateTextBox = function (arrowsPosition, textBoxValue) {

            if (this.currentDir !== null) {
                if (!isEmpty(textBoxValue) && this.valueFromInput() !== textBoxValue) {
                    this.ignoreValueFromInputChange = true;
                    this.valueFromInput(textBoxValue);
                }
            }

            if (arrowsPosition != null) {   // this condition is equivalent to "textBox is visible"
                //var mouse = new THREE.Vector2();
                //mouse.x = (event.pageX - ($(canvas).offset().left));
                //mouse.y = (event.pageY - ($(canvas).offset().top));

                //mouse.x += 20; mouse.y += 20;

                if (!this.currentDir) { console.error("VIEW3D.controls.Move: this.currentDir is null!"); return; }

                var offset = 4.5; // when camera.zoom = 1, offset = 4.5 looks ok
                arrowsPosition[this.currentDir] += offset * (1 / camera.zoom);

                var wvp = new THREE.Matrix4().multiplyMatrices(camera.projectionMatrix, camera.matrixWorldInverse);

                var arrowsPosition2d = arrowsPosition.clone().applyMatrix4(wvp);        // position3D -> position2D
                //arrowsPosition2d.x *= aspectRatio();

                VIEWUI.moveEdit.edit(
                        this.valueFromInput,    // observable
                        canvas.clientWidth / 2 * (1 + arrowsPosition2d.x),
                        canvas.clientHeight / 2 * (1 - arrowsPosition2d.y) - 15 // 15 = half of the input's height
                );

                this.isTextBoxVisible = true;
            }
            else {
                if (!isEmpty(VIEWUI.moveEdit)) {
                    VIEWUI.moveEdit.viewModel.cancel();
                }
                this.isTextBoxVisible = false;
                this.initialDistance = 0;
                this.ignoreValueFromInputChange = true;
                this.valueFromInput(0);
                this.ignoreValueFromInputChange = false;
            }

        }

        this.onMouseDown = function (event) {   // delete?
            if (enabled === false || event.button === 1 || event.button === 2) {
                return false;
            }
            mouseStart = new THREE.Vector2();
            mouseStart.x = ((event.pageX - ($(canvas).offset().left)) / canvas.clientWidth) * 2 - 1;
            mouseStart.y = -((event.pageY - ($(canvas).offset().top)) / canvas.clientHeight) * 2 + 1;

            var target = mouse3d.object();
            var modelTarget = model.selectedObjects()[0];

            if (!isEmpty(target) && target.type === "MoveToolArrow") {
                if (modelTarget != null && modelTarget.selected() === true) {
                    //if (modelTarget.type === MODEL.OBJECT_TYPE.Node) { ... }
                    startNode = modelTarget;
                    startPos = this.getArrowPosition();

                    if (!isEmpty(this.currentDir) && target.dir !== this.currentDir)
                        this.updateTextBox(null, 0);         // reset textbox to 0 if different arrow clicked

                    this.currentDir = target.dir;

                    var raycaster = new THREE.Raycaster();
                    raycaster.setFromCamera(mouseStart, camera);
                    var ray = raycaster.ray;

                    var lineStart = startPos.clone();
                    var lineEnd = startPos.clone();

                    lineStart[this.currentDir] += 10000;
                    lineEnd[this.currentDir] -= 10000;

                    var pos = new THREE.Vector3();
                    var minDist = ray.distanceSqToSegment(lineStart, lineEnd, null, pos);

                    this.currentMouseClickPos = pos;

                } else {
                    startPos = null;
                }

                if (!isEmpty(target) && target.type === "MoveToolArrow") {
                    console.log("MOVE " + target.dir);

                    this.previousMouseX = ((event.pageX - ($(canvas).offset().left)) / canvas.clientWidth) * 2 - 1;
                    this.previousMouseY = -((event.pageY - ($(canvas).offset().top)) / canvas.clientHeight) * 2 + 1;
                    this.isMoving = true;

                    return true;
                }
                return false;
            }

            //this.updateTextBox(null);
        }

        this.onMouseUp = function (event, inputOkClicked) {
            if (enabled === false) {
                return false;
            }
            var target = mouse3d.object();

            if ((!isEmpty(target) && target.type === "MoveToolArrow") || this.isMoving || inputOkClicked) {
                this.isMoving = false;

                var newArrowsPosition = this.getArrowPosition();
                this.updateArrowsPosition(newArrowsPosition);
                this.updateTextBox(newArrowsPosition);

                if (startPos != null || inputOkClicked) {
                    var offset;
                    if (isEmpty(this.valueFromInput()))
                        offset = this.getOffset(event, this.currentMouseClickPos);
                    else
                        offset = this.getOffsetVector(this.valueFromInput() - this.initialDistance);
                    if (offset.lengthSq() > 0.0) {
                        var nodestoMove = getNodesToMove();
                        for (var i = 0; i < nodestoMove.length; i++) {
                            var node = nodestoMove[i];
                            var pos = HELPERS.toVector3(node.position());
                            var newPos = pos.clone().add(offset);

                            node.position(new MODEL.types.Vector(newPos.x, newPos.y, newPos.z));

                            this.initialDistance = this.valueFromInput();
                        }
                    }
                    /*
                    if (startNode != null && snap.result.snapNode != null) {
                        model.mergeNodes(snap.result.snapNode, startNode);
                    }
                    */
                    if (visualization != null) {
                        visualization.dispose();
                        scene.remove(visualization);
                        visualization = null;
                    }
                    startPos = null;
                    render();
                    startNode = null;
                    snap.hide();

                    var newArrowsPosition = this.getArrowPosition();
                    this.updateTextBox(newArrowsPosition);


                    return true;
                }

                return true;
            }

            if (!this.hasCameraMoved)        // if clicked on workplane
                this.updateTextBox(null);   // hide textbox (move this?)

            this.isMoving = false;
            this.hasCameraMoved = false;
            return false;
        }


        this.onMouseMove = function (event) {
            if (enabled === false) {
                return false;
            }

            if (this.isMoving) {

                if (startPos != null) {
                    if (visualization == null) {
                        visualization = new VIEW3D.EditedElements();
                        scene.add(visualization); //todo replace scene
                        startData = { lines: getBarsMove() };
                        data = { lines: getBarsMove() };
                        visualization.init(data);
                    }
                    var offset = this.getOffset(event, this.currentMouseClickPos);
                    for (var i = 0; i < data.lines.length; i++) {
                        var line = data.lines[i];
                        var startLine = startData.lines[i];
                        if (line.start.move) {
                            line.start.x = startLine.start.x + offset.x;
                            line.start.y = startLine.start.y + offset.y;
                            line.start.z = startLine.start.z + offset.z;
                        }
                        if (line.end.move) {
                            line.end.x = startLine.end.x + offset.x;
                            line.end.y = startLine.end.y + offset.y;
                            line.end.z = startLine.end.z + offset.z;
                        }
                        visualization.updateLines();
                    }

                    var newArrowsPosition = this.getArrowPosition().add(offset);
                    this.updateArrowsPosition(newArrowsPosition);
                    this.updateTextBox(newArrowsPosition, offset[this.currentDir] + this.initialDistance);

                    return true;
                }


                return true;
            }
            else {
                if ((event.button === 2 || event.button === 1) && this.isTextBoxVisible)  //only update textbox position when rotating or paning the camera
                {
                    var newArrowsPosition = this.getArrowPosition();
                    this.updateTextBox(newArrowsPosition);
                    this.hasCameraMoved = true;
                }
                //console.log("is not moving");
            }
            return false;
        }

        this.onValueFromInputChanged = function () {    // when "ok" clicked
            if (!this.ignoreValueFromInputChange)       // ignore input change if changed by the control, not by user
                this.onMouseUp(null, true);

            this.ignoreValueFromInputChange = false;
        }

        this.onMouseWheel = function () {
            if (enabled === false) {
                return false;
            }

            if (this.isTextBoxVisible) {
                var newArrowsPosition = this.getArrowPosition();
                this.updateTextBox(newArrowsPosition);
            }
        }



        this.arrows3d = null;
        this.arrowId = null;
        this.xArrowIds = [];
        this.yArrowIds = [];
        this.zArrowIds = [];

        this.getArrowsIdsForArrow = function (dir) {
            return this[dir + "ArrowIds"];
        }

        this.init = function () {

            this.arrows3d = new VIEW3D.Arrows(camera);
            var arrowData = null;

            arrowData = [{
                dir: new THREE.Vector3(1, 0, 0),
                pos: new THREE.Vector3(0, 0, 0),
                color: new THREE.Vector3(1, 0, 0),
                type: VIEW3D.Arrows.Arrow
            }];
            this.xArrowIds.push(this.arrows3d.addArrows(arrowData, { type: 'MoveToolArrow', dir: 'x' }));

            arrowData = [{
                dir: new THREE.Vector3(-1, 0, 0),
                pos: new THREE.Vector3(0, 0, 0),
                color: new THREE.Vector3(1, 0, 0),
                type: VIEW3D.Arrows.Arrow
            }];
            this.xArrowIds.push(this.arrows3d.addArrows(arrowData, { type: 'MoveToolArrow', dir: 'x' }));

            arrowData = [{
                dir: new THREE.Vector3(0, 1, 0),
                pos: new THREE.Vector3(0, 0, 0),
                color: new THREE.Vector3(0, 1, 0),
                type: VIEW3D.Arrows.Arrow
            }];
            this.yArrowIds.push(this.arrows3d.addArrows(arrowData, { type: 'MoveToolArrow', dir: 'y' }));

            arrowData = [{
                dir: new THREE.Vector3(0, -1, 0),
                pos: new THREE.Vector3(0, 0, 0),
                color: new THREE.Vector3(0, 1, 0),
                type: VIEW3D.Arrows.Arrow
            }];
            this.yArrowIds.push(this.arrows3d.addArrows(arrowData, { type: 'MoveToolArrow', dir: 'y' }));

            arrowData = [{
                dir: new THREE.Vector3(0, 0, 1),
                pos: new THREE.Vector3(0, 0, 0),
                color: new THREE.Vector3(0, 0, 1),
                type: VIEW3D.Arrows.Arrow
            }];
            this.zArrowIds.push(this.arrows3d.addArrows(arrowData, { type: 'MoveToolArrow', dir: 'z' }));

            arrowData = [{
                dir: new THREE.Vector3(0, 0, -1),
                pos: new THREE.Vector3(0, 0, 0),
                color: new THREE.Vector3(0, 0, 1),
                type: VIEW3D.Arrows.Arrow
            }];
            this.zArrowIds.push(this.arrows3d.addArrows(arrowData, { type: 'MoveToolArrow', dir: 'z' }));

            HELPERS.subscribePropertyChange(this, this.valueFromInput, "VIEW3D.controls.Move.valueFromInput", this.onValueFromInputChanged);

        }

        this.getArrow = function (mouse2d) {
            if (!enabled || model.selectedObjects().length === 0) return null;

            var userData = this.arrows3d.collisionTest(mouse2d);
            if (userData)
                return { userData: userData, matrixWorld: this.arrows3d.matrixWorld }
            else
                return null;
        }

        this.onObjectSelected = function () {
            if (enabled === false) {
                return false;
            }

            if (model.selectedObjects().length === 0) {
                HELPERS.removeFromScene(this.arrows3d, scene);
            }
            else {
                this.updateArrowsPosition(this.getArrowPosition());

                HELPERS.addToScene(this.arrows3d, scene);
                render();

                for (var i = 0, ii = this.subscriptions.length; i < ii; i++) {  // remove previous subscriptions
                    this.subscriptions[i].dispose();
                }
                this.subscriptions = [];

                var selectedObjects = model.selectedObjects();

                for (var i = 0, ii = selectedObjects.length; i < ii; i++) {       // subscribe on moving objects
                    if (selectedObjects[i].type === MODEL.OBJECT_TYPE.Node) {
                        this.subscriptions.push(HELPERS.subscribePropertyChange(this, selectedObjects[i].position, "controls.move.onObjectSelected",
                            this.onObjectsPositionChanged));
                    }

                    if (selectedObjects[i].type === MODEL.OBJECT_TYPE.Bar) {
                        this.subscriptions.push(HELPERS.subscribePropertyChange(this, selectedObjects[i].node1().position, "controls.move.onObjectSelected",
                            this.onObjectsPositionChanged));
                        this.subscriptions.push(HELPERS.subscribePropertyChange(this, selectedObjects[i].node2().position, "controls.move.onObjectSelected",
                            this.onObjectsPositionChanged));
                    }
                }
            }
        }
        /*this.onObjectDeselected = function (node, index) {
            removeInstance(index);
            instances.splice(index, 1);
        }*/

        this.onObjectsPositionChanged = function () {
            this.updateArrowsPosition(this.getArrowPosition());
        }

        this.updateArrowsPosition = function (arrowPosition) {
            this.arrows3d.position.set(arrowPosition.x, arrowPosition.y, arrowPosition.z);
        }

        this.getArrowPosition = function () {                               // find center point of the bounding box

            var pos = new THREE.Vector3();

            var min = new THREE.Vector3(Infinity, Infinity, Infinity);      // min (bottom) vertex of the bounding box
            var max = new THREE.Vector3(-Infinity, -Infinity, -Infinity);   // max (top) vertex of the bounding box

            var selectedObjects = model.selectedObjects();

            var addMinMax = function (vertex) {
                if (vertex.x < min.x) min.x = vertex.x;
                if (vertex.y < min.y) min.y = vertex.y;
                if (vertex.z < min.z) min.z = vertex.z;

                if (vertex.x > max.x) max.x = vertex.x;
                if (vertex.y > max.y) max.y = vertex.y;
                if (vertex.z > max.z) max.z = vertex.z;
            }

            // find min and max of all selected vertices' positions
            for (var i = 0, ii = selectedObjects.length; i < ii; i++) {
                if (selectedObjects[i].type === MODEL.OBJECT_TYPE.Node) {
                    addMinMax(selectedObjects[i].position());
                }

                if (selectedObjects[i].type === MODEL.OBJECT_TYPE.Bar) {
                    addMinMax(selectedObjects[i].node1().position());
                    addMinMax(selectedObjects[i].node2().position());
                }
            }

            pos.set((min.x + max.x) / 2, (min.y + max.y) / 2, (min.z + max.z) / 2);

            return pos;
        }
        this.getSelected3dObjects = function () {
            constructionView3D.nodes();

        }

        this.init();

    }


});