'use strict';
var VIEW3D = VIEW3D || {};
VIEW3D.controls = VIEW3D.controls || {};

VIEW3D.controls.MoveConstruction = function(mouse3D) {
    var drag = false;

    this.enable = function() {

    }

    this.disable = function() {

    }

    this.onMouseDown = function (event) {
        console.log('MoveConst - mouse down');
        drag = true;
        this.startDrag();
        render();
        return true;
    }

    this.onMouseUp = function (event) {
        console.log('MoveConst - mouse up');
        drag = false;
        this.endDrag();
        render();
        return true;
    }

    this.onMouseMove = function (event) {
        if(drag === true) {
            console.log('MoveConst - mouse move');
            this.drag();
            return true;
        }
        return false;
    }

    this.getRayDist = function (ray) {
        var u = ray.direction;
        var v = new THREE.Vector3(1, 0, 0).applyMatrix4(this.moveSpace);
        var w = new THREE.Vector3().subVectors(ray.origin, this.pos);
        var a = u.dot(u);         // always >= 0
        var b = u.dot(v);
        var c = v.dot(v);         // always >= 0
        var d = u.dot(w);
        var e = v.dot(w);
        var D = a * c - b * b;  // always >= 0
        var tN, tD = D;         // tc = tN / tD, default tD = D >= 0
        if (D > 0.001) {        // the lines are almost parallel
            tN = (a * e - b * d);
            return tN / tD
        } else {
            return undefined;
        }
    }

    this.startDrag = function () {
        var ray = mouse3D.raycaster.ray;
        this.pos = ray.origin;
        this.moveSpace = VIEW3D.utils.getTransformationFromSegment(new THREE.Vector3(this.pos.x -1, this.pos.y, this.pos.z), this.pos)
        this.startDist = this.getRayDist(ray);
        this.models = this.createConstructionModel();
        this.addConstructionToScene(this.models);
        //snaping points later
        //this.snapDists = findSnapPoints();
    }

    this.drag = function () {
        var ray = mouse3D.raycaster.ray;
        if (this.startDist !== undefined) {
            var dist = this.calculateOffset();
            if (Math.abs(dist - this.startDist) > 0.001) {
                console.log('Drag - dist = ' + dist);
                this.moveConstructionModel(this.models, dist)
                render();
            }
        }
    }

    this.endDrag = function () {
        this.removeConstructionFromScene(this.models);
        this.moveNodesByOffset(this.calculateOffset());
    }

    this.calculateOffset = function() {
        var ray = mouse3D.raycaster.ray;
        var newRay = { direction: ray.direction , origin : new THREE.Vector3(ray.origin.x, ray.origin.y, ray.origin.z)}
        // if(VIEW3D.SnapGridDefinition.snapToGrid() === true) {
        //     newRay.origin.x = VIEW3D.SnapXToGrid(newRay.origin.x);
        // }
        return this.getRayDist(newRay) - this.startDist;
    }

    this.moveNodesByOffset = function (offset) {
        var nodes = CORE.model.nodes();
        var actualOffset = this.calculateSnappingOffset(this.models, offset);
        for (var i = 0; i < nodes.length; i++){
            var node = nodes[i];
            var oldPosition = node.position();
            var xPos = oldPosition.x + actualOffset;
            
            node.position(new MODEL.types.Vector(xPos, oldPosition.y, oldPosition.z));
        }
    }

    this.addConstructionToScene = function(models) {
        for (var i = 0; i < models.length; i++){
            var model = models[i];
            scene.add(model);
        }
    }

    this.removeConstructionFromScene = function(models) {
        for (var i = 0; i < models.length; i++){
            var model = models[i];
            scene.remove(model);
        }
    }

    this.calculateSnappingOffset = function(models, offset) {
        for (var i = 0; i < models.length; i++){
            var model = models[i];
            if(!model.isHorizontal) {
                var xPos = model.originalPosition.x + offset;
                if(VIEW3D.SnapGridDefinition.snapToGrid() === true) {
                    xPos = VIEW3D.SnapXToGrid(xPos);
                }
                return xPos - model.originalPosition.x;
            }
        }

        return offset;
    }

    this.moveConstructionModel = function(models, offset) {
        var actualOffset = this.calculateSnappingOffset(models, offset);

        for (var i = 0; i < models.length; i++){
            var model = models[i];
            this.moveBarModel(model, actualOffset);
        }
    }

    this.moveBarModel = function(cube, offset) {
        var xPos = cube.originalPosition.x + offset;
        cube.position.x = xPos;
        cube.position.y = cube.originalPosition.y;
        cube.position.z = cube.originalPosition.z;
    }

    this.createBarModel = function(bar) {
        var node1 = bar.node1().position();
        var node2 = bar.node2().position();

        var calcCubeLength = function(val1, val2) {
            var val = Math.abs(val1 - val2);
            if(val < 0.00001) return 0.05;
            return val;
        }
        var cube = new THREE.Mesh( new THREE.CubeGeometry(calcCubeLength(node1.x, node2.x), calcCubeLength(node1.y, node2.y), calcCubeLength(node1.z, node2.z)), new THREE.MeshBasicMaterial({color: 0xff0000}) );
        var center = new THREE.Vector3().addVectors(node1, node2).divideScalar(2);
        cube.originalPosition = center;
        this.moveBarModel(cube, 0);
        cube.isHorizontal = Math.abs(node1.z - node2.z) < 0.00001;
        return cube;
    }

    this.createConstructionModel = function() {
        var models = [];
        var bars = CORE.model.bars();
        for (var i = 0; i < bars.length; i++){
            var bar = bars[i];
            var cube = this.createBarModel(bar);
            models.push(cube);
        }
        return models;
    }
};