﻿'use strict';
var VIEW3D = VIEW3D || {}
VIEW3D.controls = VIEW3D.controls || {}

define(['js/view3d/VIEW3D.Highlight'], function () {

    /*
    var NodeDrawMode = {
        FindStartPos : 0,
        FindHeight : 1
    }
    */

    VIEW3D.controls.NodeDraw = function(scene, nodeModel, construction, cursor3D, orthogonalHelper) {

        var visualization = new VIEW3D.controls.NodeDrawVisualization( );

        this.isEnabled = false;
        this.enable = function() {
            if(!this.isEnabled) {
                this.isEnabled = true;
                cursor3D.snapNodes = construction.nodes();
                scene.add(visualization);
                visualization.setPosition(0, 0, 0);
            }
        }

        this.disable = function() {
            if(this.isEnabled) {
                this.isEnabled = false;
                cursor3D.snapNodes = null;
                scene.remove(visualization);
                render();
            }
        }

        this.setNodePosition = function() {
            var posX = Math.round(cursor3D.position.x * epsilonGrid) / epsilonGrid;
            var posY = Math.round(cursor3D.position.y * epsilonGrid) / epsilonGrid;
            var posZ = Math.round(cursor3D.position.z * epsilonGrid) / epsilonGrid;
            visualization.setPosition(posX, posY, posZ);
            nodeModel().position(new MODEL.types.Vector(posX, posY, posZ));
        }

        this.onMouseMove = function(event) {
            if (isEmpty(CORE.options.mode) && event.target.parentElement.id !== "webGlContainer") {
                return false;
            }
            this.setNodePosition();
            return false;
        }

        this.onMouseUp = function(event) {
            if (event.which === 3 || (isEmpty(CORE.options.mode) && event.target.parentElement.id !== "webGlContainer")) {
                return false;
            }
            if(event.button === 0) {
                construction.deselectAll();
                construction.addNode(nodeModel());
                nodeModel(new MODEL.Node(new MODEL.types.Vector(nodeModel().position().x, nodeModel().position().y, nodeModel().position().z)));
                orthogonalHelper.hide();
                return true;
            }
            return false;
        }
    }


    //UTILS
    VIEW3D.controls.NodeDrawVisualization = function () {
        var dotGeometry = new THREE.BufferGeometry();
        dotGeometry.addAttribute("position", new THREE.BufferAttribute(new Float32Array([0,0,0]), 3));
        var dotMaterial = new THREE.PointsMaterial({
            size: CORE.options.nodeSize,
            vertexColors: THREE.VertexColors,
            sizeAttenuation: false
        });

        THREE.Points.call( this, dotGeometry, dotMaterial );
        this.frustumCulled = false;

        this.setPosition = function(x, y, z) {
            this.position.set(x, y, z);
            this.updateMatrixWorld();
        }
    }
    VIEW3D.controls.NodeDrawVisualization.prototype = Object.create(THREE.Points.prototype);
})
