'use strict';
var VIEW3D = VIEW3D || {}
VIEW3D.controls = VIEW3D.controls || {}
define(['js/view3d/VIEW3D.LineMeasurements'],function () {

VIEW3D.controls.OnBarElementEdit = function(parent3D, mouse3D) {
    var mesurements = [];
    var changeEvent = ko.observable();
    var mesurementsView3D = new VIEW3D.LineMeasurements(
        function() { return mesurements },
        changeEvent,
        CORE.aspectRatio, camera,
        undefined,
        CORE.mode
    );

    this.modelSub = null;

    this.setBar = function(bar, startDist, onChange) {
        var startPoint = bar.node1().position();
        var endPoint = bar.node2().position();
        var dist = startDist;
        var vec = endPoint.sub(startPoint);
        var totalLength = vec.length();
        if (totalLength >= startDist) {
            var dir = vec.multiplyScalar(1 / totalLength); // normalized vec dir
            var onSegmentPoint = dir.multiplyScalar(dist).add(startPoint);
            var mesurement1 = new MODEL.Measurement(
                startPoint,
                dir,
                dist,
                function(newValue) {
                    if (newValue < 0) {
                        return 0;
                    }
                    var multi = CORE.units.getMulti(VIEWUI.viewOptions.unitDistance());
                    var valBaseUnit = newValue * multi;

                    if (valBaseUnit > totalLength) {
                        return totalLength;
                    } else {
                        return newValue;
                    }
                }
            );
            var mesurement2 = new MODEL.Measurement(
                onSegmentPoint,
                dir,
                totalLength - dist,
                function (newValue) {
                    if (newValue < 0) {
                        return 0;
                    }

                    var multi = CORE.units.getMulti(VIEWUI.viewOptions.unitDistance());
                    var valBaseUnit = newValue * multi;

                    if (valBaseUnit > totalLength) {
                        return totalLength;
                    } else {
                        return newValue;
                    }
                }
            );
            mesurements = [];
            mesurements.push(mesurement1);
            mesurements.push(mesurement2);
            mesurement1.length.subscribe(function(length) {
                mesurement2.length(totalLength - length);
                mesurement2.start(dir.multiplyScalar(length).add(startPoint));
                changeEvent.notifySubscribers(length);
                if(onChange) onChange(length);
            });
            mesurement2.length.subscribe(function(length) {
                mesurement1.length(totalLength - length);
                mesurement2.start(dir.multiplyScalar(totalLength - length).add(startPoint));
                //changeEvent.notifySubscribers(totalLength - length);
            });
            changeEvent.notifySubscribers(dist);
        }

        var self = this;
        if(!this.modelSub) this.modelSub = CORE.model.modelChange.subscribe(function() {
            //self.setBar(bar, startDist, onChange);
        });
    }
    this.setBarTrapezoid = function(bar, p1, p2, onChangeP1, onChangeP2) {
        this.x1Edited = false;
        this.x2Edited = false;
        this.x3Edited = false;
        var node1 = bar.node1().position();
        var node2 = bar.node2().position();
        // Bar total length
        var vec = node2.sub(node1);
        var totalLength = vec.length();
        if (totalLength >= p1 && totalLength >= p2) {
            if (p1 < 0) {
                p1 = 0;
            }
            if (p2 < 0) {
                p2 = 0;
            }
            // Choose which node is main one
            var startX1, endX3, startX2, startX3, dir;
            if (p1 < p2) {
                startX1 = node1;
                endX3 = node2;
                dir = vec.multiplyScalar(1 / totalLength);
                startX2 = startX1.add(dir.multiplyScalar(p1));
                startX3 = startX1.add(dir.multiplyScalar(p2));
            } else {
                startX1 = node2;
                endX3 = node1;
                vec = node1.sub(node2);
                dir = vec.multiplyScalar(1 / totalLength);
                startX2 = endX3.sub(dir.multiplyScalar(p1));
                startX3 = endX3.sub(dir.multiplyScalar(p2));
            }
            // Points position
            var lengthX1 = new THREE.Vector3().subVectors(startX2, startX1).length();
            var lengthX2 = new THREE.Vector3().subVectors(startX3, startX2).length();
            var lengthX3 = new THREE.Vector3().subVectors(endX3, startX3).length();
            var mesurement1 = new MODEL.Measurement(
                startX1,
                dir,
                lengthX1,
                function(newValue) {
                    var val;

                    if (newValue < 0) {
                        return 0;
                    }

                    var multi = CORE.units.getMulti(VIEWUI.viewOptions.unitDistance());
                    var valBaseUnit = newValue * multi;
                    var max = lengthX1 + lengthX2;

                    if (valBaseUnit > max / multi) {
                        return max;
                    } else {
                        return newValue;
                    }
                }
            );
            var mesurement2 = new MODEL.Measurement(
                startX2,
                dir,
                lengthX2,
                function (newValue) {
                   var val;
    
                   if (newValue < 0) {
                       return 0;
                   }
    
                   var multi = CORE.units.getMulti(VIEWUI.viewOptions.unitDistance());
                   var valBaseUnit = newValue * multi;
                   var max = totalLength;
    
                   if (valBaseUnit > max / multi) {
                       return max;
                   } else {
                       return newValue;
                   }
                }
            );
            var mesurement3 = new MODEL.Measurement(
                startX3,
                dir,
                lengthX3,
                function(newValue) {
                    var val;

                    if (newValue < 0) {
                        return 0;
                    }

                    var multi = CORE.units.getMulti(VIEWUI.viewOptions.unitDistance());
                    var valBaseUnit = newValue * multi;
                    var max = totalLength - lengthX1;

                    if (valBaseUnit > max / multi) {
                        return max;
                    } else {
                        return newValue;
                    }
                }
            );
            mesurements = [];
            mesurements.push(mesurement1);
            mesurements.push(mesurement2);
            mesurements.push(mesurement3);
            var self = this;
            mesurement1.length.subscribe(function(length) {
                if (!self.x2Edited) {
                    self.x2Edited = true;
                    mesurement2.start(startX1.add(dir.multiplyScalar(length)));
                    mesurement2.length(totalLength - length - mesurement3.length());
                    changeEvent.notifySubscribers();
                    onChangeP1(mesurement1.length());
                    self.x2Edited = false;
                }
            });
            mesurement2.length.subscribe(function(length) {
                var mod1, mod3;
                if (!self.x1Edited && !self.x2Edited && !self.x3Edited) {
                    mod1 = mesurement1.length() + (totalLength - (length + mesurement3.length() + mesurement1.length())) / 2;
                    mod3 = mesurement3.length() + (totalLength - (length + mesurement3.length() + mesurement1.length())) / 2;
                } else {
                    mod1 = mesurement1.length();
                    mod3 = mesurement3.length();
                }
                if (length >= totalLength) {
                    mod1 = 0;
                    mod3 = 0;
                }
                if (!self.x1Edited) {
                    self.x1Edited = true;
                    mesurement1.length(totalLength - length - mod3);
                    changeEvent.notifySubscribers();
                    self.x1Edited = false;
                }
                if (!self.x2Edited) {
                    self.x2Edited = true;
                    mesurement2.start(startX1.add(dir.multiplyScalar(mod1)));
                    changeEvent.notifySubscribers();
                    onChangeP1(mesurement1.length());
                    self.x2Edited = false;
                }
                if (!self.x3Edited) {
                    self.x3Edited = true;
                    mesurement3.length(totalLength - length - mod1);
                    mesurement3.start(mesurement2.start().add(dir.multiplyScalar(mesurement2.length())));
                    changeEvent.notifySubscribers();
                    onChangeP2(mesurement1.length() + mesurement2.length());
                    self.x3Edited = false;
                }
            });
            mesurement3.length.subscribe(function(length) {
                if (!self.x2Edited) {
                    self.x2Edited = true;
                    mesurement2.length(totalLength - length - mesurement1.length());
                    changeEvent.notifySubscribers();
                    self.x2Edited = false;
                    onChangeP2(mesurement1.length() + mesurement2.length());
                }
            });
            changeEvent.notifySubscribers();
        }

        if(!this.modelSub) this.modelSub = CORE.model.modelChange.subscribe(function() {
            //self.setBarTrapezoid(bar, p1, p2, onChangeP1, onChangeP2);
        });
    }

    this.setDimensionBetweenPoints = function(bar, posList, posCurrent, onChange) {

        mesurements = [];

        var node1Pos = bar.node1().position();
        var node2Pos = bar.node2().position();
        var vector = node2Pos.sub(node1Pos);
        var posCurrent_ = posCurrent;
        var length = vector.length();
        var dirVect = vector.multiplyScalar(1 / length); // normalized vec dir
        var nodeCurrent = dirVect.multiplyScalar(posCurrent_).add(node1Pos);
        
        var m = [];
        m.push({ len: 0, pos: node1Pos });
        m.push({ len: bar.getLength(), pos: node2Pos });

        for (var i = 0; i < posList.length; i++) {
            var posLocal = posList[i];

            var offset = dirVect.multiplyScalar(posLocal).add(node1Pos);
            m.push({ len: posLocal, pos: offset });
        }

        m.sort(function (a, b) { return a.len - b.len });

        for (var i = 0; i < m.length - 1; i++) {
            var posObj1 = m[i];
            var posObj2 = m[i + 1];

            if (posCurrent <= posObj2.len && posCurrent >= posObj1.len) {
                var lenMax = posObj2.len - posObj1.len;
                var dist = posCurrent - posObj1.len;

                var mesurement1 = this.setDimensionDouble(posObj1.pos, dirVect, dist, lenMax);
                var mesurement2 = this.setDimensionDouble(nodeCurrent, dirVect, posObj2.len - posCurrent, lenMax);
                mesurements.push(mesurement1);
                mesurements.push(mesurement2);

                mesurement1.length.subscribe(function (length) {
                    mesurement2.length(lenMax - length);
                    mesurement2.start(dirVect.multiplyScalar(length).add(posObj1.pos));
                    changeEvent.notifySubscribers(length);
                    if (onChange) onChange(posObj1.len+length);
                });
                mesurement2.length.subscribe(function (length) {
                    mesurement1.length(lenMax - length);
                    mesurement2.start(dirVect.multiplyScalar(lenMax - length).add(posObj1.pos));
                });
                changeEvent.notifySubscribers(dist);
                return;
            }
        }
    }
    this.setDimensionDouble = function (start, dir, len, maxLength) {
        return new MODEL.Measurement(
                        start,
                        dir,
                        len,
                        function (newValue) {
                            var val;
                            if (newValue < 0) {
                                val = 0;
                            }

                            var multi = CORE.units.getMulti(VIEWUI.viewOptions.unitDistance());
                            var valBaseUnit = val * multi;

                            if (valBaseUnit > maxLength) {
                                val = maxLength;
                            } else {
                                val = valBaseUnit;
                            }
                            return val;
                        }
                    );
    }

    this.setMeasurementDirection = function(dir) {
        mesurementsView3D.setDirection(dir);
    }

    this.enable = function() {
        parent3D.add(mesurementsView3D);
    }
    this.disable = function() {
        parent3D.remove(mesurementsView3D);
        mesurements = [];
        changeEvent.notifySubscribers();
        if(this.modelSub) { 
            this.modelSub.dispose();
            this.modelSub = null;
        }
    }
    this.clearMesurements = function() {
        if (mesurements.length > 0) {
            mesurements = [];
            changeEvent.notifySubscribers();
        }
        if(this.modelSub) {
            this.modelSub.dispose();
            this.modelSub = null;
        }
    }
    this.onMouseUp = function(event) {
        if ((isEmpty(CORE.options.mode) && event.target.parentElement.id !== "webGlContainer")) {
            return false;
        }
        var object = mouse3D.object();
        if (!isEmpty(object) && object.type === MODEL.OBJECT_TYPE.Measurement) {
            VIEWUI.measurementEdit.edit(
                object,
                canvas.clientWidth / 2 * (1 + mouse3D.position().x), // mouse3D - this is not the position of the mouse! (but it is set by the VIEW3D.Measurements.getLabel function)
                canvas.clientHeight / 2 * (1 - mouse3D.position().y)
            );
        }
        return false;
    }
}

});
