﻿'use strict';
var VIEW3D = VIEW3D || {}
VIEW3D.controls = VIEW3D.controls || {}
VIEW3D.controls.isMultiDrawActive = false;
define(['js/view3d/controls/VIEW3D.controls.OrthogonalHelper', 'js/view3d/VIEW3D.PointNodeCloud'], function () {

    VIEW3D.controls.OrthogonalDraw = function (parent, construction, barModel, cursor3D, orthogonalHelper) {
        //barModel will be updated so that you can see the changes on properties dialog
        var self = this;
        //THREE.Object3D.call(this);
        var OD_STATE = {
            DRAW_FIRST_POINT: 0,
            DRAW_SECONT_POINT: 1
        }
        var enabled = false;
        //var currentNode = null;
        var state = OD_STATE.DRAW_FIRST_POINT;
        orthogonalHelper.state = state;
        var orthoSub = null;
        var vizualization = new VIEW3D.controls.OrthogonalDrawVisualization();

        //called by helper when user enters value by keyboard
        this.newPositionFromOrthoHelper = function (pos) {
            this.setCurrentNodePosition(HELPERS.toVector(pos));
            this.nextStep();
        }

        this.setCurrentNodePosition = function (pos) {
            switch (state){
                case OD_STATE.DRAW_FIRST_POINT:
                    barModel().node1().position(pos);
                    break;
                case OD_STATE.DRAW_SECONT_POINT:
                    barModel().node2().position(pos);
                    break;
                default:
                    throw Error("Unknown orthogonal draw state: " + state)
            }
        }


        var reset = function () {
            var node1 = new MODEL.Node(new MODEL.types.Vector(0, 0, 0));
            var node2 = new MODEL.Node(new MODEL.types.Vector(0, 0, 0));
            var newBar = new MODEL.Bar(node1, node2);
            barModel(newBar);
            state = OD_STATE.DRAW_FIRST_POINT;
            orthogonalHelper.state = state;
            vizualization.update(true, node1.position());
            cursor3D.snapNodes = construction.nodes();
            cursor3D.startNode = null;
        }

        this.enable = function () {
            if (enabled === true) {
                return;
            }
            enabled = true;
            orthoSub = orthogonalHelper.changeEvent.subscribe(this.newPositionFromOrthoHelper.bind(this));
            reset();
            parent.add(vizualization);
        }

        this.disable = function () {
            if (enabled === false) {
                return;
            }
            enabled = false;
            cursor3D.snapNodes = null;
            cursor3D.startNode = null;
            if(!isEmpty(orthoSub)) {
                orthoSub.dispose();
                orthoSub = null;
            }
            parent.remove(vizualization);
        }

        this.calculateCurrentPointPosition = function(pos) {
            return orthogonalHelper.calculateCurrentPointPosition(pos);
        }

        this.onMouseMove = function (event) {
            if (enabled === false || (isEmpty(CORE.options.mode) && (event.target.parentElement && event.target.parentElement.id !== "webGlContainer"))) {
                return false;
            }
            //var pos = this.calculateCurrentPointPosition(cursor3D.position);
            var pos = cursor3D.position;
            var x = Math.round(pos.x * epsilonGrid) / epsilonGrid;
            var y = Math.round(pos.y * epsilonGrid) / epsilonGrid;
            var z = Math.round(pos.z * epsilonGrid) / epsilonGrid;
            //console.log('Orto draw X : ' + x + ', Y :' + y + ', Z :' + z );
            this.setCurrentNodePosition(new MODEL.types.Vector(x,y,z));

            vizualization.update(state === OD_STATE.DRAW_FIRST_POINT, barModel().node1().position(), barModel().node2().position());

            render();
            return false;
        }
        //var addNode1 = false;

        var validateBarModel = function() {
            var p1 = barModel().node1().position();
            var p2 = barModel().node2().position();
            if (p1.sub(p2).length() < 0.001 || p1.sub(p2).length() > 100) {
                return false;
            }
            return true;
        }

        function filterNodes(nodes, exludePos) {
            var newNodes = [];
            for (var i = 0; i < nodes.length; i++) {
                var node = nodes[i];
                var pos = node.position();
                if(Math.abs(exludePos.x - pos.x) < 0.000001 && 
                    Math.abs(exludePos.y - pos.y) < 0.000001 &&
                    Math.abs(exludePos.z - pos.z) < 0.000001) {

                }
                else {
                    newNodes.push(node);
                }
            }
            return newNodes;
        }

        this.nextStep = function () {
            if (state === OD_STATE.DRAW_FIRST_POINT) {
                //automatically enable helper on first node
                orthogonalHelper.setPosition(HELPERS.toVector3(barModel().node1().position()));
                orthogonalHelper.show();

                //cursor3d
                //var nodesToSnap = construction.nodes();
                //var currentNodeIndex = nodesToSnap.indexOf(barModel().node1());
                //if(currentNodeIndex > -1 ) nodesToSnap.splice(currentNodeIndex, 1);
                cursor3D.snapNodes = filterNodes(construction.nodes(), barModel().node1().position());// construction.nodes().concat([barModel().node1()]); //add new node to snap
                cursor3D.startNode = barModel().node1(); //enables perpendicular snapping
                barModel().node2().position((barModel().node1().position()));
                state = OD_STATE.DRAW_SECONT_POINT;
                orthogonalHelper.state = state;
            } else if (state === OD_STATE.DRAW_SECONT_POINT && validateBarModel()) {
                //add nodes and bar to the model
                //NOTE: addNode will return an existing node if a new one is on the same position
                var nodeInModel = construction.addNode(barModel().node1());
                barModel().node1(nodeInModel);
                nodeInModel = construction.addNode(barModel().node2());
                barModel().node2(nodeInModel);
                construction.addBar(barModel());

                //disable helper
                orthogonalHelper.hide();

                //reset snapping points
                cursor3D.snapNodes = construction.nodes();
                cursor3D.startNode = null; //switch off perpendicular snapping

                reset();
                // if(VIEW3D.controls.isMultiDrawActive === true) {
                //     self.nextStep();
                // }
            }
            vizualization.update(state === OD_STATE.DRAW_FIRST_POINT, barModel().node1().position(), barModel().node2().position());
        }

        this.onMouseUp = function (event) {
            if (enabled === false || (isEmpty(CORE.options.mode) && (event.target.parentElement && event.target.parentElement.id !== "webGlContainer"))) {
                return false;
            }
            if (event.button === 0) {
                this.nextStep();
                return true;
            }
            return false;
        }

        this.onKeyUp = function (event){
            switch(event.keyCode){
                case 27://esc
                    if(state !== OD_STATE.DRAW_FIRST_POINT) {
                        orthogonalHelper.hide();
                        reset();
                        return true;
                    }
            }
            return false;
        }

    }

    //utils
    VIEW3D.controls.OrthogonalDrawVisualization = function () {
        THREE.Object3D.call(this);

        this.positions = new THREE.BufferAttribute(new Float32Array(6), 3);
        this.positions.dynamic = true;
        var points = new THREE.Points(
            new THREE.BufferGeometry(),
            new THREE.PointsMaterial({
                size: CORE.options.nodeSize,
                vertexColors: THREE.VertexColors,
                sizeAttenuation: false
            }));

        points.geometry.addAttribute("position", this.positions);
        points.frustumCulled = false;

        var line = new THREE.Line(
            new THREE.BufferGeometry(),
            new THREE.LineBasicMaterial({
                color: new THREE.Color(0,0,0)
            }));
        line.geometry.addAttribute("position", this.positions);
        line.frustumCulled = false;
        line.visible = false;

        points.geometry.setDrawRange(0,0);
        line.geometry.setDrawRange(0,0);

        this.add(points);
        this.add(line);

        this.update = function(singleNode, p1, p2) {
            this.positions.setXYZ(0, p1.x,p1.y,p1.z);
            if(singleNode){
                points.geometry.setDrawRange(0,1);
                line.geometry.setDrawRange(0,0);
                line.visible = false;
            } else {
                this.positions.setXYZ(1, p2.x,p2.y,p2.z);
                points.geometry.setDrawRange(0,2);
                line.geometry.setDrawRange(0,2);
                line.visible = true;
            }
            this.positions.needsUpdate = true;
            render();
        }

    }

    VIEW3D.controls.OrthogonalDrawVisualization.prototype = Object.create(THREE.Object3D.prototype);
});