'use strict';
var VIEW3D = VIEW3D || {}
VIEW3D.controls = VIEW3D.controls || {}

define(['js/view3d/VIEW3D.Measurement'], function () {

    VIEW3D.controls.OrthogonalHelper = function (scene, cursor3D) {
        //static members
        VIEW3D.controls.OrthogonalDraw.guideXColor = new THREE.Color(1, 0.5, 0.5);
        VIEW3D.controls.OrthogonalDraw.guideYColor = new THREE.Color(0.5, 1, 0.5);
        VIEW3D.controls.OrthogonalDraw.guideZColor = new THREE.Color(0.5, 0.5, 1);
        //VIEW3D.controls.OrthogonalDraw.beamColor = new THREE.Color(0, 0, 0);


        this.endPos = new THREE.Vector3(0, 0, 0);
        this.startPos = new THREE.Vector3(0, 0, 0);

        var geometry = new THREE.BufferGeometry();
        geometry.addAttribute("color", new THREE.BufferAttribute( new Float32Array(6*3),3));
        geometry.addAttribute("position", new THREE.BufferAttribute( new Float32Array(6*3),3));
        var cx = VIEW3D.controls.OrthogonalDraw.guideXColor;
        var cy = VIEW3D.controls.OrthogonalDraw.guideYColor;
        var cz = VIEW3D.controls.OrthogonalDraw.guideZColor;
        geometry.attributes.color.setXYZ(0, cx.r, cx.g, cx.b);
        geometry.attributes.color.setXYZ(1, cx.r, cx.g, cx.b);
        geometry.attributes.color.setXYZ(2, cy.r, cy.g, cy.b);
        geometry.attributes.color.setXYZ(3, cy.r, cy.g, cy.b);
        geometry.attributes.color.setXYZ(4, cz.r, cz.g, cz.b);
        geometry.attributes.color.setXYZ(5, cz.r, cz.g, cz.b);
        geometry.attributes.position.setXYZ(0, -1, 0, 0);
        geometry.attributes.position.setXYZ(1, +1, 0, 0);
        geometry.attributes.position.setXYZ(2, 0, -1, 0);
        geometry.attributes.position.setXYZ(3, 0, +1, 0);
        geometry.attributes.position.setXYZ(4, 0, 0, -1);
        geometry.attributes.position.setXYZ(5, 0, 0, +1);

        var material = new THREE.LineBasicMaterial({
            vertexColors: THREE.VertexColors
        });

        //THREE.LineSegments.call(this, geometry, material);
        this.frustumCulled = false;

        var visible = false;

        this.OnMeasurementChange = function (val) {
            var newEndPos = this.startPos.clone().add( this.endPos.clone().sub(this.startPos).normalize().multiplyScalar(val));
            this.changeEvent.notifySubscribers(newEndPos);
        }

        var measurement = new VIEW3D.controls.OrthogonalHelper.Measurement(scene, this.OnMeasurementChange.bind(this));

        this.changeEvent = ko.observable();

        var gfx = new THREE.LineSegments(geometry, material);
        gfx.frustumCulled = false;

        this.setPosition = function (pos) {
            gfx.position.set(pos.x,pos.y,pos.z);
            //this.updateMatrixWorld();
            this.startPos.set(pos.x,pos.y,pos.z);

        }

        this.calculateCurrentPointPosition = function(pos) {
            return pos;
        }

        this.onCursorChange = function () {
            //change black line
            this.endPos.copy(cursor3D.position);
            geometry.verticesNeedUpdate = true;
            measurement.set(this.startPos, this.endPos);
        }

        this.show = function () {
            cursor3D.clearGuides();
            cursor3D.addCrossGuide(this.startPos);
            scene.add(gfx);
            visible = true;
            if(isEmpty(cursor3dSub)) {
                cursor3dSub = cursor3D.changeEvent.subscribe(this.onCursorChange.bind(this));
            }
        }

        this.hide = function () {
            cursor3D.clearGuides();
            scene.remove(gfx);
            visible = false;
            if(!isEmpty(cursor3dSub)) {
                cursor3dSub.dispose();
                cursor3dSub = null;
            }
            measurement.clean();
        }

        var cursor3dSub = null;

        this.enabled = false;
        this.enable = function () {
            this.enabled = true;
        }

        this.disable = function () {
            this.hide();
            this.enabled = false;
        }


        this.onKeyUp = function (event){
            switch(event.keyCode){
                case 32://space
                    this.hide();
                    this.setPosition(HELPERS.toVector3(cursor3D.position));
                    this.show();
                    return true;
                case 27://esc
                    if(visible) {
                        this.hide();
                        return true;
                    }
            }

            return false;
        }
    }

    VIEW3D.controls.OrthogonalHelper.prototype = Object.create(THREE.LineSegments.prototype);

    VIEW3D.controls.OrthogonalHelper.Measurement = function (scene, onChaneg) {

        this.changeEvent = ko.observable();
        var measurements = [];
        this.view = new VIEW3D.Measurements(
            function() {
                return measurements
            },
            this.changeEvent,
            CORE.aspectRatio, camera,
            {
                measurementOffset0: 0.00,
                measurementOffset1: 0.12,
                measurementOffset2: 0.16,
                textAnchor: 0.0,
                textOffset: -25,
                color: VIEW3D.editMeasurementColor
            },
            CORE.mode
        );

        scene.add(this.view);
        this.measurementSub = null;

        this.onMeasurementChange = function (newVal) {
            onChaneg(newVal)
        }

        this.set = function (pos1, pos2) {
            if(!isEmpty(this.measurementSub)) {
                this.measurementSub.dispose();
            }
            var sub = pos2.clone().sub(pos1);
            var distSq = sub.lengthSq();
            if(distSq < 0.001 * 0.001) {
                measurements = [];
                this.changeEvent.notifySubscribers();
            } else {
                var dir = sub.normalize();
                var measurement = new MODEL.Measurement(HELPERS.toVector(pos1), HELPERS.toVector(dir), Math.sqrt(distSq))
                measurements = [measurement];
                this.changeEvent.notifySubscribers();
                this.view.setFocusOnFirstInput();
                this.measurementSub = HELPERS.subscribePropertyChange(this, measurement.length, "measurement.length", this.onMeasurementChange);
            }
        }

        this.clean = function () {
            if(!isEmpty(this.measurementSub)) {
                this.measurementSub.dispose();
            }
            measurements = [];
            this.changeEvent.notifySubscribers();
        }

    }
});