'use strict';
var VIEW3D = VIEW3D || {}
VIEW3D.controls = VIEW3D.controls || {}

define(['js/view3d/VIEW3D.Measurement'], function () {

    VIEW3D.controls.OrthogonalPlaneHelper = function (scene, cursor3D) {
        function rotateGeometry(geometry, rotX, rotY, rotZ) {
            geometry.rotateX(rotX);
            geometry.rotateY(rotY);
            geometry.rotateZ(rotZ);
        }

        function createPlaneModel(rotX, rotY, rotZ) {
            var geometry = new THREE.PlaneGeometry( 0.3, 0.3);
            rotateGeometry(geometry, rotX, rotY, rotZ);
            var material = Extension.MATERIAL.createSheet(0x98fb98, 0.5, true, true);
            var plane = new THREE.Mesh( geometry, material );
            return plane;
        }

        function createMainPoints(rotX, rotY, rotZ) {
            var dotGeometry = new THREE.Geometry();

            dotGeometry.vertices.push(new THREE.Vector3( 0, -0.15, 0));
            dotGeometry.vertices.push(new THREE.Vector3( 0, 0.15, 0));
            dotGeometry.vertices.push(new THREE.Vector3( 0.15, 0, 0));
            dotGeometry.vertices.push(new THREE.Vector3( -0.15, 0, 0));

            rotateGeometry(dotGeometry, rotX, rotY, rotZ);
            var dotMaterial = new THREE.PointsMaterial( { size: 5, sizeAttenuation: false, color : 0xff0000 } );
            var dot = new THREE.Points( dotGeometry, dotMaterial );
            return dot;
        }

        function createSecondaryPoints(rotX, rotY, rotZ) {
            var dotGeometry2 = new THREE.Geometry();
            dotGeometry2.vertices.push(new THREE.Vector3( -0.15, -0.15, 0));
            dotGeometry2.vertices.push(new THREE.Vector3( -0.15, 0.15, 0));
            dotGeometry2.vertices.push(new THREE.Vector3( 0.15, -0.15, 0));
            dotGeometry2.vertices.push(new THREE.Vector3( 0.15, 0.15, 0));

            rotateGeometry(dotGeometry2, rotX, rotY, rotZ);
            var dotMaterial2 = new THREE.PointsMaterial( { size: 3, sizeAttenuation: false, color : 0xff0000 } );
            var dot2 = new THREE.Points( dotGeometry2, dotMaterial2 );
            return dot2;
        }

        function createLineGeometry(rotX, rotY, rotZ, from, to) {
            var geometry = new THREE.Geometry();
            geometry.vertices.push(from );
            geometry.vertices.push(to );
            rotateGeometry(geometry, rotX, rotY, rotZ);

            return geometry;
        }

        function createLines(rotX, rotY, rotZ) {
            var material = new THREE.LineBasicMaterial( { color: 0x778899} );
            var materialLight = new THREE.LineBasicMaterial( { color: 0x778899, transparent : true, opacity : 0.3 } );
            var line1 = new THREE.Line( createLineGeometry(rotX, rotY, rotZ, new THREE.Vector3( -0.15, -0.15, 0), new THREE.Vector3( 0.15, 0.15, 0)), materialLight );
            var line2 = new THREE.Line( createLineGeometry(rotX, rotY, rotZ, new THREE.Vector3( -0.15, 0.15, 0), new THREE.Vector3( 0.15, -0.15, 0)), materialLight );
            var line3 = new THREE.Line( createLineGeometry(rotX, rotY, rotZ, new THREE.Vector3( -0.15, 0, 0), new THREE.Vector3( 0.15, 0, 0)), material );
            var line4 = new THREE.Line( createLineGeometry(rotX, rotY, rotZ, new THREE.Vector3( 0, -0.15, 0), new THREE.Vector3( 0, 0.15, 0)), material );


            var group = new THREE.Group();
            group.add(line1);
            group.add(line2);
            group.add(line3);
            group.add(line4);

            return group;
        }

        function createPlaneObject(rotX, rotY, rotZ) {
            var plane = createPlaneModel(rotX, rotY, rotZ);
            var dot = createMainPoints(rotX, rotY, rotZ);
            var dot2 = createSecondaryPoints(rotX, rotY, rotZ);
            var lines = createLines(rotX, rotY, rotZ);
            var group = new THREE.Group();
            group.add(plane);
            group.add(dot);
            group.add(dot2);
            group.add(lines);
            return group;
        }

        this.endPos = new THREE.Vector3(0, 0, 0);
        this.startPos = new THREE.Vector3(0, 0, 0);

        var material = new THREE.LineBasicMaterial({
            vertexColors: THREE.VertexColors
        });

        //THREE.LineSegments.call(this, geometry, material);
        this.frustumCulled = false;

        var visible = false;

        this.OnMeasurementChange = function (val) {
            var newEndPos = this.startPos.clone().add( this.endPos.clone().sub(this.startPos).normalize().multiplyScalar(val));
            this.changeEvent.notifySubscribers(newEndPos);
        }

        var measurement = new VIEW3D.controls.OrthogonalHelper.Measurement(scene, this.OnMeasurementChange.bind(this));

        this.changeEvent = ko.observable();
        this.planes = [];
        this.planes.push(createPlaneObject(0,0,0)); //XY
        this.planes.push(createPlaneObject(1.57,0,0)); //XZ
        this.planes.push(createPlaneObject(0,1.57,0)); //YZ
        this.activePlaneIndex = 1;

        

        this.setPosition = function (pos) {
            for(var i = 0 ; i < this.planes.length ; i++) {
                var plane = this.planes[i];
                plane.position.set(pos.x,pos.y,pos.z);
            }

            this.startPos.set(pos.x,pos.y,pos.z);
        }


        this.onCursorChange = function () {
            //change black line
            this.endPos.copy(cursor3D.position);
            //geometry.verticesNeedUpdate = true;
            measurement.set(this.startPos, this.endPos);
        }
        this.calculateMathPlaneConstant = function(index) {
            switch(index){
                case 0:
                    return -this.startPos.z;
                case 1:
                    return -this.startPos.y;
                case 2:
                    return -this.startPos.x;
            }
        }
        this.setMathPlane = function(index) {
            var normal = new THREE.Vector3(index === 2 ? 1 : 0, index === 1 ? 1 : 0 ,index === 0 ? 1 : 0);
            var constant = this.calculateMathPlaneConstant(index);
            var plane = new THREE.Plane(normal, constant);
            cursor3D.mouse3D.drawingPlane = plane;
        }

        this.updatePlaneVisiblity = function(show) {
            for(var i = 0 ; i < this.planes.length ; i++) {
                var plane = this.planes[i];
                if(i === this.activePlaneIndex && show === true) {
                    scene.add(plane);
                } else {
                    scene.remove(plane);
                }
            }

            if(show) {
                this.setMathPlane(this.activePlaneIndex);
            } else {
                cursor3D.mouse3D.drawingPlane = null;
            }
        }

        this.testLineDistance = function(pos, vec1, vec2) {
            var line1 = new THREE.Line3(vec1, vec2);
            var clPoint = line1.closestPointToPoint(pos, false);
            var dist = pos.distanceTo(clPoint);
            return { distance : dist, point : clPoint, isCloseEnough : dist < 0.075};
        }

        this.sortDistanceResults = function(res1, res2) {
            return res1.distance - res2.distance;
        }

        this.calculateXYPosition = function(pos) {
            var results = [];
            results.push(this.testLineDistance(pos, new THREE.Vector3(this.startPos.x, -10, this.startPos.z ),
            new THREE.Vector3(this.startPos.x, 10, this.startPos.z)));

            results.push(this.testLineDistance(pos, new THREE.Vector3(-10, this.startPos.y, this.startPos.z ),
            new THREE.Vector3(10, this.startPos.y, this.startPos.z)));

            results.push(this.testLineDistance(pos, new THREE.Vector3(this.startPos.x - 10, this.startPos.y - 10, this.startPos.z ),
            new THREE.Vector3(this.startPos.x + 10, this.startPos.y + 10, this.startPos.z)));

            results.push(this.testLineDistance(pos, new THREE.Vector3(this.startPos.x - 10, this.startPos.y + 10, this.startPos.z ),
            new THREE.Vector3(this.startPos.x + 10, this.startPos.y - 10, this.startPos.z)));

            results.sort(this.sortDistanceResults);
            if(results[0].isCloseEnough || this.state === 1) {
                return results[0].point;
            }
            return pos;
        }

        this.calculateXZPosition = function(pos) {
            var results = [];

            results.push(this.testLineDistance(pos, new THREE.Vector3(this.startPos.x, this.startPos.y, -10 ),
            new THREE.Vector3(this.startPos.x, this.startPos.y, 10)));

            results.push(this.testLineDistance(pos, new THREE.Vector3(-10, this.startPos.y, this.startPos.z ),
            new THREE.Vector3(10, this.startPos.y, this.startPos.z)));

            results.push(this.testLineDistance(pos, new THREE.Vector3(this.startPos.x - 10, this.startPos.y, this.startPos.z -10 ),
            new THREE.Vector3(this.startPos.x + 10, this.startPos.y, this.startPos.z + 10)));

            results.push(this.testLineDistance(pos, new THREE.Vector3(this.startPos.x - 10, this.startPos.y, this.startPos.z + 10 ),
            new THREE.Vector3(this.startPos.x + 10, this.startPos.y, this.startPos.z -10)));
  
            results.sort(this.sortDistanceResults);
            
            if(results[0].isCloseEnough || this.state === 1) {
                return results[0].point;
            }
            return pos;
        }

        this.calculateYZPosition = function(pos) {
            var results = [];

            results.push(this.testLineDistance(pos, new THREE.Vector3(this.startPos.x, -10, this.startPos.z ),
            new THREE.Vector3(this.startPos.x, 10, this.startPos.z)));

            results.push(this.testLineDistance(pos, new THREE.Vector3(this.startPos.x, this.startPos.y, -10 ),
            new THREE.Vector3(this.startPos.x, this.startPos.y, 10)));

            results.push(this.testLineDistance(pos, new THREE.Vector3(this.startPos.x, this.startPos.y - 10, this.startPos.z - 10 ),
            new THREE.Vector3(this.startPos.x, this.startPos.y + 10, this.startPos.z + 10)));

            results.push(this.testLineDistance(pos, new THREE.Vector3(this.startPos.x, this.startPos.y - 10, this.startPos.z + 10 ),
            new THREE.Vector3(this.startPos.x, this.startPos.y + 10, this.startPos.z - 10)));

            results.sort(this.sortDistanceResults);
            
            if(results[0].isCloseEnough || this.state === 1) {
                return results[0].point;
            }
            return pos;
        }

        this.calculateCurrentPointPosition = function(pos) {
            if(this.activePlaneIndex === 0 ) {
                return this.calculateXYPosition(pos);
            }
            if(this.activePlaneIndex === 1 ) {
                return this.calculateXZPosition(pos);
            }
            if(this.activePlaneIndex === 2 ) {
                return this.calculateYZPosition(pos);
            }
            
            return pos;
        }

        this.show = function () {
            cursor3D.clearGuides();
            cursor3D.addCrossGuide(this.startPos);
            this.updatePlaneVisiblity(true);
            visible = true;
            if(isEmpty(cursor3dSub)) {
                cursor3dSub = cursor3D.changeEvent.subscribe(this.onCursorChange.bind(this));
            }
        }

        this.hide = function () {
            cursor3D.clearGuides();
            this.updatePlaneVisiblity(false);
            visible = false;
            if(!isEmpty(cursor3dSub)) {
                cursor3dSub.dispose();
                cursor3dSub = null;
            }
            measurement.clean();
        }

        var cursor3dSub = null;

        this.enabled = false;
        this.enable = function () {
            this.enabled = true;
        }

        this.disable = function () {
            this.hide();
            this.enabled = false;
        }


        this.onKeyUp = function (event){
            switch(event.keyCode){
                case 81:
                case 113://q for now
                    this.activePlaneIndex += 1;
                    if(this.activePlaneIndex >= this.planes.length) this.activePlaneIndex = 0;
                    this.updatePlaneVisiblity(visible);
                    render();
                    return true;
                case 32://space
                    this.hide();
                    this.setPosition(HELPERS.toVector3(cursor3D.position));
                    this.show();
                    return true;
                case 27://esc
                    if(visible) {
                        this.hide();
                        return true;
                    }
            }

            return false;
        }
    }

    VIEW3D.controls.OrthogonalPlaneHelper.prototype = Object.create(THREE.LineSegments.prototype);
});