﻿'use strict';
var VIEW3D = VIEW3D || {}
VIEW3D.controls = VIEW3D.controls || {}

VIEW3D.controls.SelectArea = function (camera, domElement, constructionView, construction) {
    var button = {
        LEFT: 1,
        MIDDLE: 4
    };
    this.box = null;
    this.boxStarted = null; // true if box drawing has started
    this.boxInScene = false;
    this.boxStartVector = null; // coords of first mouse click
    this.boxEndVector = null;
    this.rotationEnabled = true;

    var mouseDownOn3DView = false;

    this.enable = function () {
        this.enabled = true;
    }
    this.disable = function () {
        this.enabled = true;
    }
    this.init = function () {
        var geometry = new THREE.Geometry();
        geometry.vertices.push(
            new THREE.Vector3(0, 0, 0),
            new THREE.Vector3(0.5, 0, 0),
            new THREE.Vector3(0.5, 0.5, 0),
            new THREE.Vector3(0, 0.5, 0)
        );
        geometry.faces.push(new THREE.Face3(0, 1, 2), new THREE.Face3(0, 2, 3));
        this.box = new THREE.Mesh(geometry, new VIEW3D.materials.ZoomArea(0.5, CORE.aspectRatio(), new THREE.Vector3(0, 0, 0)));
        this.box.renderOrder = 1000;
        this.box.frustumCulled = false;
    }
    this.init();
    this.onMouseUp = function(event) {
        if (event.which === 1) {
            mouseDownOn3DView = false;
            if (this.boxStarted && this.boxInScene) {
                scene.remove(this.box);
                this.boxInScene = false;
            }
            this.boxStarted = false;
            //console.log(this.boxStartVector);
            //console.log(this.boxEndVector);
            if (this.boxEndVector !== null) {
                this.selectArea();
                this.boxEndVector = null;
                CORE.mode(MODE.SELECT);
                return true;
            }
        }
        return false;
    }
    this.getDomElementArea = function () {
        var box = domElement.getBoundingClientRect();
        var d = domElement.ownerDocument.documentElement;
        return {
            left: box.left + window.pageXOffset - d.clientLeft,
            top: box.top + window.pageYOffset - d.clientTop,
            width: box.width,
            height: box.height
        }
    }
    this.containsEntireMode = function () {
        //left to right -> select elements that are entirely in selection box
        //right to left -> select even if only part of element is in selection box
        return this.boxEndVector.x > this.boxStartVector.x;
    }

    this.onMouseDown = function (event) {
        mouseDownOn3DView = event.target.parentElement.id === "webGlContainer";
        return false;
    }

    this.onMouseMove = function (event) {
        if ((event.buttons & button.LEFT) !== 0) {
            if ($(event.target).parents('#VIEWUI_propertiesContainer').length === 0 &&
                mouseDownOn3DView
            ) {
            var offsetX, offsetY;
                if (!this.boxStarted) {
                    offsetX = ((event.pageX - $(webGlContainer).offset().left) / $(webGlContainer).width()) * 2 - 1;
                    offsetY = ((event.pageY - $(webGlContainer).offset().top) / $(webGlContainer).height()) * 2 - 1;
                    this.boxStartVector = new THREE.Vector2(offsetX, -offsetY);
                    this.boxEndVector = this.boxStartVector.clone();
                    this.box.geometry.vertices[0].x = this.boxStartVector.x;
                    this.box.geometry.vertices[0].y = this.boxStartVector.y;
                    this.box.geometry.vertices[1].y = this.boxStartVector.y;
                    this.box.geometry.vertices[3].x = this.boxStartVector.x;
                    this.boxStarted = true;
                    this.boxEndVector = null;
                    console.log("BOX STARTED");
                } else {
                    offsetX = ((event.pageX - $(webGlContainer).offset().left) / $(webGlContainer).width()) * 2 - 1;
                    offsetY = ((event.pageY - $(webGlContainer).offset().top) / $(webGlContainer).height()) * 2 - 1;
                    this.boxEndVector = new THREE.Vector2(offsetX, -offsetY);
                    this.box.geometry.vertices[1].x = this.boxEndVector.x;
                    this.box.geometry.vertices[2].x = this.boxEndVector.x;
                    this.box.geometry.vertices[2].y = this.boxEndVector.y;
                    this.box.geometry.vertices[3].y = this.boxEndVector.y;
                    this.box.geometry.verticesNeedUpdate = true;
                    this.box.material.uniforms.color.value = this.containsEntireMode() ?
                        new THREE.Vector3(0.5, 1, 0.5) :
                        new THREE.Vector3(0.3, 0.7, 1);
                    //this.box.material.uniforms.opacity = 0.5;
                    //this.box.material.needsUpdate = true;
                    if (!this.boxInScene) {
                        scene.add(this.box);
                        this.boxInScene = true;
                    }
                    //console.log("BOX CONTINUING");
                    render();
                }
            }
        }
        return false;
    }

    var pixelHalfSize = function () {
        return {
            x: 1.0 / canvas.clientWidth,
            y: 1.0 / canvas.clientHeight
        }
    }

    this.selectArea = function () {

        // deselect
        if (!HELPERS.inputState.selectMultiple) {
            construction.deselectAll();
        }

        var region = {
            min: {
                x: Math.min(this.boxStartVector.x, this.boxEndVector.x),
                y: Math.min(this.boxStartVector.y, this.boxEndVector.y),
            },
            max: {
                x: Math.max(this.boxStartVector.x, this.boxEndVector.x),
                y: Math.max(this.boxStartVector.y, this.boxEndVector.y),
            }
        }

        var intersections = constructionView.getCollider().region2DIntersection(region, camera, pixelHalfSize);

        for (var i = 0; i < intersections.length; i++) {
            if(!this.containsEntireMode() || intersections[i].containedEntirely){
                intersections[i].collider.model.selected(true);
            }
        }
        render();
        return true;
    }

};