﻿'use strict';
var VIEW3D = VIEW3D || {}
VIEW3D.controls = VIEW3D.controls || {}

var MODE_TYPE = {
    NONE: 0,
    ROTATE: 1,
    PAN: 2,
    ZOOM: 3
};

VIEW3D.controls.Trackball = function (camera, axisHelperNamespace, domElement, construction, mouse3D) {
    var modeType = {
        NONE: 0,
        ROTATE: 1,
        PAN: 2
    };
    var button = {
        LEFT: 1,
        RIGHT: 2,
        MIDDLE: 4
    };
    this.camera = camera;
    //this.axisHelperCamera = axisHelperNamespace.camera;
    //this.axisHelper = VIEW3D.AxisHelper.object3d; //doesn't hold reference, can't be used //axisHelperNamespace.object3d;   //axisHelperNamespace doesn't work, need to use static field
    this.camera.matrixAutoUpdate = false;
    //this.axisHelper.matrixAutoUpdate = false;
    this.domElement = domElement;
    this.rotationEnabled = true;
    this._mouseDrag = false;
    this._draggingStartedOnCanvas = false;

    this.pivotPoint = new THREE.Vector3(0, 0, 0);
    this.pivotPointGeometry = new THREE.Geometry();
    this.pivotPointMesh = new THREE.Points( this.pivotPointGeometry, new THREE.PointsMaterial( { color: 0xff0000, size: 5, sizeAttenuation: false } ) );
    this.pivotPointMesh.renderOrder = 1000000; this.pivotPointMesh.frustumCulled = false;//needed to properly display for all camera angles
    this.pivotPointMesh.visible = false;
    CORE.constructionView3D.externalModels.addModel({ ElementId : "RotationPivotPointMesh" }, [this.pivotPointMesh], false);

    this.enable = function () {
        this.enabled = true;
    }
    this.disable = function () {
        this.enabled = true;
    }
    this.onMouseUp = function (event) {
        var result = this._mouseDrag;
        this._mouseDrag = event.buttons > 0;
        this._draggingStartedOnCanvas = false;
        this.pivotPointMesh.visible = false;
        render();
        return result;
    }
    this.getDomElementArea = function () {
        var box = this.domElement.getBoundingClientRect();
        var d = this.domElement.ownerDocument.documentElement;
        return {
            left: box.left + window.pageXOffset - d.clientLeft,
            top: box.top + window.pageYOffset - d.clientTop,
            width: box.width,
            height: box.height
        }
    }
    this.getConstructionBS = function (isForReport) {
        var bb = new THREE.Box3();
        bb.setFromObject(CORE.constructionView3D);

        var bbWidth = bb.max.x - bb.min.x;
        var bbHeight = bb.max.y - bb.min.y;
        var bbDepth = bb.max.z - bb.min.z;
        var largestDimention = Math.max(bbWidth, bbHeight, bbDepth);

        var sphereRadius = largestDimention / 2;
        //if (sphereRadius < 1) sphereRadius = 1;

        var bs = new THREE.Sphere(bb.center(), sphereRadius);

        return bs;

        // var minX, minY, minZ, maxX, maxY, maxZ, nodePos;
        // for (var i = 0; !isEmpty(construction.nodes) && i < construction.nodes().length; i++) {
        //     nodePos = construction.nodes()[i].position();
        //     if (i === 0) {
        //         minX = maxX = nodePos.x;
        //         minY = maxY = nodePos.y;
        //         minZ = maxZ = nodePos.z;
        //     } else {
        //         if (nodePos.x > maxX) {
        //             maxX = nodePos.x;
        //         } else if (nodePos.x < minX) {
        //             minX = nodePos.x;
        //         }
        //         if (nodePos.y > maxY) {
        //             maxY = nodePos.y;
        //         } else if (nodePos.y < minY) {
        //             minY = nodePos.y;
        //         }
        //         if (nodePos.z > maxZ) {
        //             maxZ = nodePos.z;
        //         } else if (nodePos.z < minZ) {
        //             minZ = nodePos.z;
        //         }
        //     }
        // }

        // for (var i = 0; i < Extension.externalBoundingBox.length; i++) {
        //     if (i === 0 && construction.nodes().length === 0) {
        //         minX = maxX = 0;
        //         minY = maxY = 0;
        //         minZ = maxZ = 0;
        //     }
        //     var bbox = Extension.externalBoundingBox[i];
        //     if (bbox.max.x > maxX) {
        //         maxX = bbox.max.x;
        //     } else if (bbox.min.x < minX) {
        //         minX = bbox.min.x;
        //     }
        //     if (bbox.max.y > maxY) {
        //         maxY = bbox.max.y;
        //     } else if (bbox.min.y < minY) {
        //         minY = bbox.min.y;
        //     }
        //     if (bbox.max.z > maxZ) {
        //         maxZ = bbox.max.z;
        //     } else if (bbox.min.z < minZ) {
        //         minZ = bbox.min.z;
        //     }
        // }

        // var vecMin = new THREE.Vector3(minX, minY, minZ);
        // var vecMax = new THREE.Vector3(maxX, maxY, maxZ);
        // if (vecMin.x === vecMax.x && isForReport === true) {
        //     vecMin.x -= 1;
        //     vecMax.x += 1;
        // }
        // if (vecMin.y === vecMax.y && isForReport === true) {
        //     vecMin.y -= 1;
        //     vecMax.y += 1;
        // }
        // if (vecMin.z === vecMax.z && isForReport === true) {
        //     vecMin.z -= 1;
        //     vecMax.z += 1;
        // }
        // var bb = new THREE.Box3(vecMin, vecMax);
        // var bs = bb.getBoundingSphere();

        // if (construction.nodes().length === 0)
        //     bs.isModelEmpty = true;
        // else
        //     bs.isModelEmpty = false;

        // return bs;
    }
    this.onMouseMove = function (event) {
        //if (event.target.parentElement.id !== "webGlContainer") {
        //    return false;
        //}

        var mode = modeType.NONE;

        if (this._draggingStartedOnCanvas) {     // check if dragging started with a click on canvas

            if (this.rotationEnabled) {
                if ((event.buttons & button.RIGHT) !== 0) {
                    mode = modeType.ROTATE;
                } else if ((event.buttons & button.MIDDLE) !== 0) {
                    mode = modeType.PAN;
                }
            } else {
                if ((event.buttons & button.MIDDLE) !== 0) {
                    mode = modeType.PAN;
                }
            }

        }


        if (mode === modeType.ROTATE) {
            this.rotate(-(event.pageX - this.lastPageX), -(event.pageY - this.lastPageY), 0);
            render(MODE_TYPE.ROTATE);
        }
        if (mode === modeType.PAN) {
            this.pan(this.lastPageX - event.pageX, this.lastPageY - event.pageY);
            render(MODE_TYPE.PAN);
        }
        if (this.lastPageX !== event.pageX && this.lastPageY !== event.pageY) {
            this._mouseDrag = event.buttons > 1;
        }
        this.lastPageX = event.pageX;
        this.lastPageY = event.pageY;
        //if(mode !== modeType.NONE) {
        //    //render(modeType.NONE);
        //    camera.changeEvent.notifySubscribers();
        //}
        return /*this._mouseDrag*/ false;
    }
    this.rotate = function (angleX, angleY, angleZ, viaCube) {

        if (viaCube)
            this.findPivotPoint(true);

        this.pivotPointMesh.visible = true;
        var rot = new THREE.Quaternion();
        //get target object bounding sphere
        /*
        var bs = this.getConstructionBS();
        if (isNaN(bs.center.x)) {
            bs.center.set(0, 0, 0);
        }
        */
        //var distanceFromBBcenter = bs.center.distanceTo(this.camera.position);    vector.setFromMatrixPosition(cameraMatrix) -> distanceTo bs.center
        var el = this.camera.matrix.elements;
        var dirY = new THREE.Vector3(el[4], el[5], el[6]); // camera "up" vector in word space (normalized)
        var dirX = new THREE.Vector3(el[0], el[1], el[2]); // camera "right" vector in word space (normalized)
        var dirZ = new THREE.Vector3(el[8], el[9], el[10]);
        //rotate around camera's "up" vector
        var rotY = new THREE.Matrix4();
        rot.setFromAxisAngle(dirY, angleX * 0.01); // TODO: rotate slower when the camera is zoomed
        rotY.makeRotationFromQuaternion(rot);
        //rotate around camera's "right" vector
        var rotX = new THREE.Matrix4();
        rot.setFromAxisAngle(dirX, angleY * 0.01);
        rotX.makeRotationFromQuaternion(rot);
        //rotate around camera's "lookAt" direction
        var rotZ = new THREE.Matrix4();
        rot.setFromAxisAngle(dirZ, angleZ * 0.01);
        rotZ.makeRotationFromQuaternion(rot);
        //move pivot point to (0,0,0)
        var translate = new THREE.Matrix4();
        translate.makeTranslation(-this.pivotPoint.x, -this.pivotPoint.y, -this.pivotPoint.z);
        this.camera.matrix.multiplyMatrices(translate, this.camera.matrix);

        //apply rotations
        this.camera.matrix.multiplyMatrices(rotY, this.camera.matrix);
        this.camera.matrix.multiplyMatrices(rotX, this.camera.matrix);
        this.camera.matrix.multiplyMatrices(rotZ, this.camera.matrix);
        VIEW3D.utils.normalizeMatrix(this.camera.matrix);


        var axisHelperCamera = VIEW3D.AxisHelper.camera;
        axisHelperCamera.matrix.multiplyMatrices(rotY, axisHelperCamera.matrix);
        axisHelperCamera.matrix.multiplyMatrices(rotX, axisHelperCamera.matrix);
        axisHelperCamera.matrix.multiplyMatrices(rotZ, axisHelperCamera.matrix);
        VIEW3D.utils.normalizeMatrix(axisHelperCamera.matrix);
        axisHelperCamera.matrixWorldNeedsUpdate = true;

        var skyboxCamera = CORE.constructionView3D.activeWorkplane.skyboxCamera;
        skyboxCamera.matrix.multiplyMatrices(rotY, skyboxCamera.matrix);
        skyboxCamera.matrix.multiplyMatrices(rotX, skyboxCamera.matrix);
        skyboxCamera.matrix.multiplyMatrices(rotZ, skyboxCamera.matrix);
        skyboxCamera.matrixWorldNeedsUpdate = true;
        skyboxCamera.updateProjectionMatrix();

        //move pivot point to previous position
        translate.makeTranslation(this.pivotPoint.x, this.pivotPoint.y, this.pivotPoint.z);
        this.camera.matrix.multiplyMatrices(translate, this.camera.matrix);
        this.camera.updateMatrixWorld(true);
        this.camera.matrixWorldInverse.getInverse(camera.matrixWorld);


        CORE.constructionView3D.activeWorkplane.draw();
        camera.changeEvent.notifySubscribers();
    }
    this.pan = function (offsetX, offsetY) {
        var el = this.camera.matrix.elements;
        var dirY = new THREE.Vector3(el[4], el[5], el[6]); // camera "up" vector in word space (normalized)
        var dirX = new THREE.Vector3(el[0], el[1], el[2]); // camera "right" vector in word space (normalized)
        var area = this.getDomElementArea();
        //normalized offset - relative to window size
        offsetX = offsetX / area.width;
        offsetY = offsetY / area.height;
        //offset in scene units (meters) in camera local space
        offsetX *= (this.camera.right - this.camera.left) / this.camera.zoom;
        offsetY *= (this.camera.bottom - this.camera.top) / this.camera.zoom;
        //3d offsets in parent (word) space 
        dirX.multiplyScalar(offsetX);
        dirY.multiplyScalar(offsetY);
        //apply offset
        var translate = new THREE.Matrix4();
        translate.makeTranslation(dirX.x + dirY.x, dirX.y + dirY.y, dirX.z + dirY.z);
        this.camera.matrix.multiplyMatrices(translate, this.camera.matrix);
        this.camera.updateMatrixWorld(true);
        this.camera.matrixWorldInverse.getInverse(camera.matrixWorld);

        //CORE.constructionView3D.activeWorkplane.draw();
        camera.changeEvent.notifySubscribers();
    }

    this.findPivotPoint = function (setAsModelGeometryCenter) {

        if (setAsModelGeometryCenter) {  // if rotated by cube

            var bs = this.getConstructionBS();

            if (bs) {
                this.pivotPoint = bs.center;
            }

        }
        else {      // if rotated by dragging mouse

            //mouse3D.ray.get
            if (!mouse3D.raycaster) {
                this.pivotPoint.set(0, 0, 0);
                return;
            }
            if (isEmpty(mouse3D.position())) {
                this.pivotPoint.set(0, 0, 0);
            } else if (!isEmpty(mouse3D.object())) {
                this.pivotPoint.copy(mouse3D.position());
            }
            else {
                var bs = this.getConstructionBS();

                if (bs) {
                    this.pivotPoint = bs.center;
                }
                // var ray = mouse3D.raycaster.ray;
                // var bars = construction.bars();
                // var minDist = Number.MAX_VALUE;
                // for (var i = 0; i < bars.length; i++) {
                //     var bar = bars[i];
                //     var v0 = HELPERS.toVector3(bar.node1().position());
                //     var v1 = HELPERS.toVector3(bar.node2().position());
                //     var onRay = new THREE.Vector3();
                //     var dist = ray.distanceSqToSegment(v0, v1, onRay);
                //     if (dist < minDist) {
                //         minDist = dist;
                //         this.pivotPoint.copy(onRay);
                //     }
                // }
            }
        }
    }

    this.onMouseDown = function (event) {
        // prevent dragging if clicked on properties/popup window
        if ($(event.target).closest(domElement).length !== 0)
            this._draggingStartedOnCanvas = true;
        else
            this._draggingStartedOnCanvas = false;

        if (this._draggingStartedOnCanvas) {
            this.findPivotPoint();
            this.pivotPointGeometry.vertices = [this.pivotPoint.clone()];
            this.pivotPointGeometry.verticesNeedUpdate = true;
            this.pivotPointGeometry.computeBoundingSphere();//for some reason needed to properly display for all camera angles
        }

        return this._mouseDrag;
    }
    this.zoomIn = function () { // zoom without mouse wheel
        this.camera.zoom *= 1.0 + 0.3;
        this.camera.updateProjectionMatrix();
        render(MODE_TYPE.ZOOM);

        CORE.constructionView3D.activeWorkplane.draw();
        camera.changeEvent.notifySubscribers();
    }
    this.zoomOut = function () {
        this.camera.zoom *= 1.0 - 0.3;
        this.camera.updateProjectionMatrix();
        render(MODE_TYPE.ZOOM);

        CORE.constructionView3D.activeWorkplane.draw();
        camera.changeEvent.notifySubscribers();
    }
    this.onMouseWheel = function (event) {
        var el = this.camera.matrix.elements;
        var dirY = new THREE.Vector3(el[4], el[5], el[6]); // camera "up" vector in word space (normalized)
        var dirX = new THREE.Vector3(el[0], el[1], el[2]); // camera "right" vector in word space (normalized)
        //zoom multiplier
        var delta = 0;
        if (event.wheelDelta) {
            // WebKit / Opera / Explorer 9
            delta = event.wheelDelta / 2000;
        } else if (event.detail) {
            // Firefox
            delta = - event.detail / 50;
        }
        //apply new zoom
        this.camera.zoom *= 1.0 + delta;
        this.camera.updateProjectionMatrix();
        // normalized mouse position in screen (2D) space (values between -1 and 1)
        var area = this.getDomElementArea();
        var offsetX = ((event.pageX - area.left) / area.width) * 2 - 1;
        var offsetY = ((event.pageY - area.top) / area.height) * 2 - 1;
        //size of delta in screen space
        offsetX *= this.camera.right * delta / this.camera.zoom;
        offsetY *= this.camera.bottom * delta / this.camera.zoom;
        //offset in parent (world) 3D space
        dirX.multiplyScalar(offsetX);
        dirY.multiplyScalar(offsetY);
        //apply offset in parent space
        var translate = new THREE.Matrix4();
        translate.makeTranslation(dirX.x + dirY.x, dirX.y + dirY.y, dirX.z + dirY.z);
        this.camera.matrix.multiplyMatrices(translate, this.camera.matrix);
        this.camera.updateMatrixWorld(true);
        this.camera.matrixWorldInverse.getInverse(camera.matrixWorld);
        render(MODE_TYPE.ZOOM);

        CORE.constructionView3D.activeWorkplane.draw();
        camera.changeEvent.notifySubscribers();
        return true;
    }
    this.resetView = function (x, y, z, animate, quaternion, originalZoom, stepZoom, stepTranslation) {       // animate, quaternion, zoom - optional (for animation)

        if (isEmpty(x) && isEmpty(y) && isEmpty(z)) {
            this.camera.position.set(10, -10, 10);
            this.camera.up.set(0, 0, 1);
            this.camera.lookAt(new THREE.Vector3(0, 0, 0));
            this.camera.updateMatrix();
            this.zoomAll();
            return;
        }

        var forward = new THREE.Vector3(-x, -y, -z); //get target object bounding sphere
        var bs = this.getConstructionBS(); //change camera orientation
        forward.normalize();

        var up = new THREE.Vector3(0, 0, 1); // camera "up" vector in word space (normalized)
        var right = new THREE.Vector3(); // camera "right" vector in word space (normalized)
        right.crossVectors(up, forward).normalize();

        if (right.lengthSq() < 0.1) { // means that dirZ = (0,1,0);
            up.set(0, 1, 0);
            right.crossVectors(up, forward);
        } else {
            up.crossVectors(forward, right);
        }

        //position: 500 units back form target 
        var pos = forward.clone(); // is normalize
        pos.multiplyScalar(500);
        pos.add(bs.center);

        var finalRot = new THREE.Matrix4().set(
            right.x, up.x, forward.x, bs.center.x,  //pos.x
            right.y, up.y, forward.y, bs.center.y,  //pos.y
            right.z, up.z, forward.z, bs.center.z,  //pos.z
            0, 0, 0, 1
        );


        if (animate) {
            var self = this;
            var originalQ = new THREE.Quaternion().setFromRotationMatrix(this.camera.matrix);
            var finalQ = new THREE.Quaternion().setFromRotationMatrix(finalRot);

            var camsize = Math.min(camera.right - camera.left, camera.top - camera.bottom);
            var originalZoom = !isEmpty(originalZoom) ? originalZoom : this.camera.zoom;
            var finalZoom = camsize / (bs.radius * 2.2);

            var originalPosition = new THREE.Vector3().setFromMatrixPosition(this.camera.matrix);
            var finalPosition = new THREE.Vector3().setFromMatrixPosition(finalRot);

            var steps = 20;
            var duration = 25;
            var i = 1;

            //cubic easing in/out - acceleration until halfway, then deceleration
            //c: current time, b: start value, c: change in value (final value minus start value), d: duration
            var easeInOutCubic = function (t, b, c, d) {
                t /= d / 2;
                if (t < 1) return c / 2 * t * t * t * t + b;
                t -= 2;
                return -c / 2 * (t * t * t * t - 2) + b;
            };

            var animationInterval = setInterval(function () {

                var dir = this.camera.getWorldDirection();

                if (i <= steps) {

                    var step = i * (1 / steps); // [0 - 1]
                    var targetQ = new THREE.Quaternion();

                    THREE.Quaternion.slerp(originalQ, finalQ, targetQ, easeInOutCubic(step, 0, 1, 1));    // 0.1, 0.2, ... , 1

                    var stepZoom = easeInOutCubic(step, originalZoom, finalZoom - originalZoom, 1);

                    var easeInOutCoef = easeInOutCubic(step, 0, 1.8, 1);
                    var stepTranslation = new THREE.Vector3().subVectors(finalPosition, originalPosition).multiplyScalar((1 / steps)/* *easeInOutCoef */);    // uncomment to enable translation easing

                    self.resetView(x, y, z, false, targetQ, originalZoom, stepZoom, stepTranslation);

                    i++;
                }
                else {
                    self.resetView(x, y, z);
                    clearInterval(animationInterval);
                    i = 0;
                }

            }, duration);

            return true;
        }


        if (!quaternion) {  // use direction vector for rotation
            this.camera.matrix.copy(finalRot);

            //change zoom in the way that it shows entire target object
            //OrthographicCamera only!!!
            var camsize = Math.min(camera.right - camera.left, camera.top - camera.bottom);
            this.camera.zoom = camsize / (bs.radius * 2.2);
        }
        else {              // use quaternion for rotation (when animating)
            var currentPosition = new THREE.Vector3().setFromMatrixPosition(this.camera.matrix);
            this.camera.matrix.identity();                           // reset rotation component
            this.camera.matrix.setPosition(currentPosition);         // restore position component (how to do in one step?)

            var translation = new THREE.Matrix4().makeTranslation(stepTranslation.x, stepTranslation.y, stepTranslation.z);
            this.camera.matrix.multiply(translation);

            var rotFromQuaternion = new THREE.Matrix4().makeRotationFromQuaternion(quaternion);
            this.camera.matrix.multiply(rotFromQuaternion);

            this.camera.zoom = stepZoom;
        }

        var rot = new THREE.Matrix4().extractRotation(this.camera.matrix);

        var axisHelperCamera = VIEW3D.AxisHelper.camera;
        axisHelperCamera.matrix.copy(rot);
        axisHelperCamera.updateProjectionMatrix();
        axisHelperCamera.matrixWorldNeedsUpdate = true;

        var skyboxCamera = CORE.constructionView3D.activeWorkplane.skyboxCamera;
        skyboxCamera.matrix.copy(rot);
        skyboxCamera.updateProjectionMatrix();
        skyboxCamera.matrixWorldNeedsUpdate = true;

        this.camera.matrixWorldNeedsUpdate = true;
        this.camera.updateProjectionMatrix();

        render(MODE_TYPE.ROTATE);

        CORE.constructionView3D.activeWorkplane.draw();
        if (!isEmpty(camera.changeEvent)) {
            camera.changeEvent.notifySubscribers();
        }
        return true;
    }
    this.zoomAll = function (isForReport) {
        var scaleFactor = 3.0;
        var bs = this.getConstructionBS(isForReport);
        var pos = bs.center.clone().multiplyScalar(1);
        this.camera.matrix.setPosition(pos);
        this.camera.matrixWorldNeedsUpdate = true;
        this.camera.updateProjectionMatrix();
        var camsize = Math.min(camera.right - camera.left, camera.top - camera.bottom);
        if (bs.radius === 0) bs.radius = 1; // prevent division by 0 (when model contains only one node or is empty)
        this.camera.zoom = camsize / (bs.radius * scaleFactor);
        this.camera.updateProjectionMatrix();

        CORE.constructionView3D.activeWorkplane.draw();
        render(MODE_TYPE.ZOOM);
        if (typeof camera.changeEvent !== 'undefined') {
            camera.changeEvent.notifySubscribers();
            setTimeout(function () {
                camera.changeEvent.notifySubscribers();    // strange bug, sometimes not working
            }, 100);
        }
    }
};