﻿'use strict';
var VIEW3D = VIEW3D || {}
VIEW3D.controls = VIEW3D.controls || {}

define(['js/view3d/materials/VIEW3D.materials.ZoomArea'], function () {

    VIEW3D.controls.ZoomArea = function (camera, axisHelperNamespace, domElement, sceneParam) {
        var button = {
            LEFT: 1,
            MIDDLE: 4
        };
        this.camera = camera;
        this.domElement = domElement;
        this.box = null;
        this.boxStarted = null; // true if box drawing has started
        this.boxInScene = false;
        this.boxStartVector = null; // coords of first mouse click
        this.boxEndVector = null;
        this.rotationEnabled = true;
        this._mouseDrag = false;
        this.scene = sceneParam || scene;
        this.enable = function () {
            this.enabled = true;
        }
        this.disable = function () {
            this.enabled = true;
        }
        this.init = function () {
            var geometry = new THREE.Geometry();
            geometry.vertices.push(
                new THREE.Vector3(0, 0, 0),
                new THREE.Vector3(0.5, 0, 0),
                new THREE.Vector3(0.5, 0.5, 0),
                new THREE.Vector3(0, 0.5, 0)
            );
            geometry.faces.push(new THREE.Face3(0, 1, 2), new THREE.Face3(0, 2, 3));
            this.box = new THREE.Mesh(geometry, new VIEW3D.materials.ZoomArea(0.3, CORE.aspectRatio(), new THREE.Vector3(66 / 255, 210 / 255, 1)));
            this.box.renderOrder = 1000;
            this.box.frustumCulled = false;
        }
        this.init();
        this.onMouseUp = function (event) {
            console.log("ON MOUSE UP");
            var result = this._mouseDrag;
            this._mouseDrag = event.buttons > 0;
            if (event.which === 1) {
                if (this.boxStarted && this.boxInScene) {
                    this.scene.remove(this.box);
                    this.boxInScene = false;
                }
                this.boxStarted = false;
                //console.log("BOX STOPPED");
                //console.log(this.boxStartVector);
                //console.log(this.boxEndVector);
                if (this.boxEndVector !== null && this.boxStartVector.distanceTo(this.boxEndVector) > 0.1) {    //ignore when window too small
                    this.zoomToArea();
                    this.boxEndVector = null;
                    CORE.mode(MODE.SELECT);
                    return true;
                }
            }
            return result;
        }
        this.getDomElementArea = function () {
            var box = this.domElement.getBoundingClientRect();
            var d = this.domElement.ownerDocument.documentElement;
            return {
                left: box.left + window.pageXOffset - d.clientLeft,
                top: box.top + window.pageYOffset - d.clientTop,
                width: box.width,
                height: box.height
            }
        }
        this.onMouseMove = function (event) {
            if ((event.buttons & button.LEFT) !== 0) {
                var offsetX, offsetY;
                if (!this.boxStarted) {
                    offsetX = ((event.pageX - $(this.domElement).offset().left) / $(this.domElement).width()) * 2 - 1;
                    offsetY = ((event.pageY - $(this.domElement).offset().top) / $(this.domElement).height()) * 2 - 1;
                    this.boxStartVector = new THREE.Vector2(offsetX, -offsetY);
                    this.boxEndVector = this.boxStartVector.clone();
                    this.box.geometry.vertices[0].x = this.boxStartVector.x;
                    this.box.geometry.vertices[0].y = this.boxStartVector.y;
                    this.box.geometry.vertices[1].y = this.boxStartVector.y;
                    this.box.geometry.vertices[3].x = this.boxStartVector.x;


                    this.boxStarted = true;
                    this.boxEndVector = null;
                    console.log("BOX STARTED");
                } else {
                    offsetX = ((event.pageX - $(this.domElement).offset().left) / $(this.domElement).width()) * 2 - 1;
                    offsetY = ((event.pageY - $(this.domElement).offset().top) / $(this.domElement).height()) * 2 - 1;
                    this.boxEndVector = new THREE.Vector2(offsetX, -offsetY);
                    this.box.geometry.vertices[1].x = this.boxEndVector.x;
                    this.box.geometry.vertices[2].x = this.boxEndVector.x;
                    this.box.geometry.vertices[2].y = this.boxEndVector.y;
                    this.box.geometry.vertices[3].y = this.boxEndVector.y;


                    this.box.geometry.verticesNeedUpdate = true;
                    if (!this.boxInScene) {
                        this.scene.add(this.box);
                        this.boxInScene = true;
                    }
                    console.log("BOX CONTINUING");
                    render();
                }
            }
            return false;
        }
        this.zoomToArea = function () {
            // ==== translate ====
            var webGlContainerWidth = $(this.domElement).width();
            var webGlContainerHeight = $(this.domElement).height();
            var zoomWindowCenterXNorm = (this.boxStartVector.x + this.boxEndVector.x) / 2; //[-1, 1]  // box center coordinates
            var zoomWindowCenterYNorm = (this.boxStartVector.y + this.boxEndVector.y) / 2; //[-1, 1]
            var zoomWindowCenterNormVector = new THREE.Vector3(
                zoomWindowCenterXNorm,
                zoomWindowCenterYNorm,
                0.5);
            var zoomWindowCenterVector = new THREE.Vector3(
                (zoomWindowCenterXNorm) / 2 * webGlContainerWidth,
                (zoomWindowCenterYNorm) / 2 * webGlContainerHeight,
                0.5);
            console.info("zoomWindowCenterNormVector =", zoomWindowCenterNormVector);
            
            var translationVector = zoomWindowCenterVector.clone().multiplyScalar(0.04 / this.camera.zoom);
            var translationM = new THREE.Matrix4().makeTranslation(translationVector.x, translationVector.y, 0);
            this.camera.matrix.multiply(translationM);
            this.camera.matrixWorldNeedsUpdate = true;
            this.camera.updateProjectionMatrix();
            // ==== zoom ====
            var zoomWindowWidthNorm = Math.abs(this.boxStartVector.x - this.boxEndVector.x) / 2; //[0, 1]    // normalized box dimensions
            var zoomWindowHeightNorm = Math.abs(this.boxStartVector.y - this.boxEndVector.y) / 2; //[0, 1]
            var zoomFactor = zoomWindowWidthNorm; // zoom according to the largest dimension
            if (zoomWindowHeightNorm > zoomWindowWidthNorm) {
                zoomFactor = zoomWindowHeightNorm;
            }
            this.camera.zoom /= zoomFactor;
            this.camera.updateProjectionMatrix();

            CORE.constructionView3D.activeWorkplane.draw();
            var self = this;
            setTimeout(function() {
                self.camera.changeEvent.notifySubscribers();
                setTimeout(function() {
                    self.camera.changeEvent.notifySubscribers();    // strange bug, sometimes not working
                }, 100);
            }, 0);
            render();
            return true;
        }
        //this.dollyIn = function () { // zoom without mouse wheel
        //    this.camera.zoom *= 1.0 + 0.3;
        //    this.camera.updateProjectionMatrix();
        //},
        //    this.dollyOut = function () {
        //        this.camera.zoom *= 1.0 - 0.3;
        //        this.camera.updateProjectionMatrix();
        //    },
        //    this.onMouseWheel = function (event) {
        //        var el = this.camera.matrix.elements;
        //        var dirY = new THREE.Vector3(el[4], el[5], el[6]); // camera "up" vector in word space (normalized)
        //        var dirX = new THREE.Vector3(el[0], el[1], el[2]); // camera "right" vector in word space (normalized)
        //        //zoom multiplier
        //        var delta = 0;
        //        if (event.wheelDelta) {
        //            // WebKit / Opera / Explorer 9
        //            delta = event.wheelDelta / 4000;
        //        } else if (event.detail) {
        //            // Firefox
        //            delta = - event.detail / 300;
        //        }
        //        //apply new zoom
        //        this.camera.zoom *= 1.0 + delta;
        //        this.camera.updateProjectionMatrix();
        //        // normalized mouse position in screen (2D) space (values between -1 and 1)
        //        var area = this.getDomElementArea();
        //        var offsetX = ((event.pageX - area.left) / area.width) * 2 - 1;
        //        var offsetY = ((event.pageY - area.top) / area.height) * 2 - 1;
        //        //size of delta in screen space
        //        offsetX *= this.camera.right * delta / this.camera.zoom;
        //        offsetY *= this.camera.bottom * delta / this.camera.zoom;
        //        //offset in parent (world) 3D space
        //        dirX.multiplyScalar(offsetX);
        //        dirY.multiplyScalar(offsetY);
        //        //apply offset in parent space
        //        var translate = new THREE.Matrix4();
        //        translate.makeTranslation(dirX.x + dirY.x, dirX.y + dirY.y, dirX.z + dirY.z);
        //        this.camera.matrix.multiplyMatrices(translate, this.camera.matrix);
        //        this.camera.matrixWorldNeedsUpdate = true;
        //        render();
        //        return true;
        //    }
        this.getBS = function () {
            var bb = new THREE.Box3();
            bb.setFromObject(this.targetObject);
            var bs = bb.getBoundingSphere();
            if (isNaN(bs.center.x)) {
                bs.center.set(0, 0, 0);
                bs.radius = 5;
            }
            return bs;
        }
    };
});