﻿'use strict';
var VIEW3D = VIEW3D || {}
VIEW3D.controls = VIEW3D.controls || {}

VIEW3D.controls.Snap = function(/*parent, camera, constructionView3D*/) {
    this.plane = null;
    this.snapingPoints = [];
    this.snapedPosition = null;
    this.snapingPointsChange = ko.observable();

    function checkIfPointIsOnPlane(point, plane) {
        var dist = plane.distanceToPoint(point);
        return Math.abs(dist) < 0.01;
    }

    var snapAlternativeVersion = function (pos, dir, snapDist, nodes, snapLine) {
        var threshold = snapDist * snapDist;
        var minDist = 10000.0;
        var result = null;
        var snapTo = null;
        for (var i = 0; i < nodes.length; i++) {
            var p = nodes[i].position();
            var point = new THREE.Vector3(p.x, p.y, p.z);
            var dx, dy, dz, dist;

            if(this.plane === null || checkIfPointIsOnPlane(point, this.plane)) {
                dist = new THREE.Vector3(0, 0, 0);
                if(this.plane.normal.x > 0.1) {
                    if(Math.abs(point.y - pos.y) < 0.02 || Math.abs(point.z - pos.z) < 0.02) {
                        if (minDist > point.distanceToSquared(pos)) {
                            minDist = point.distanceToSquared(pos);
                            result = dist;
                            snapTo = p;
                        }
                    }
                }
                if(this.plane.normal.y > 0.1) {
                    if(Math.abs(point.x - pos.x) < 0.02 || Math.abs(point.z - pos.z) < 0.02) {
                        if (minDist > point.distanceToSquared(pos)) {
                            minDist = point.distanceToSquared(pos);
                            result = dist;
                            snapTo = p;
                        }
                    }
                }
                if(this.plane.normal.z > 0.1) {
                    if(Math.abs(point.x - pos.x) < 0.02 || Math.abs(point.y - pos.y) < 0.02) {
                        if (minDist > point.distanceToSquared(pos)) {
                            minDist = point.distanceToSquared(pos);
                            result = dist;
                            snapTo = p;
                        }
                    }
                }
            }
        }
        if (snapTo != null) {
            pos.add(result);
            this.snapingPoints.push(HELPERS.toVector3( snapTo ) );
            this.snapedPosition = HELPERS.toVector3(pos);
            return true;
        }
        return false;
    }

    var snap = function (pos, dir, snapDist, nodes, snapLine) {
        var threshold = snapDist * snapDist;
        var minDist = 10000.0;
        var result = null;
        var snapTo = null;
        for (var i = 0; i < nodes.length; i++) {
            var p = nodes[i].position();
            var point = new THREE.Vector3(p.x, p.y, p.z);
            var dx, dy, dz, dist;
            if (dir.x > 0.1 || dir.x < -0.1) {
                dx = point.x - pos.x;
                dy = (dx * dir.y) / dir.x;
                dz = (dx * dir.z) / dir.x;
                dist = new THREE.Vector3(dx, dy, dz);
                if (threshold > dist.lengthSq()) {
                    if (minDist > point.distanceToSquared(pos)) {
                        minDist = point.distanceToSquared(pos);
                        result = dist;
                        snapTo = p;
                    }
                }
            }
            if (dir.y > 0.1 || dir.y < -0.1) {
                dy = point.y - pos.y;
                dx = (dy * dir.x) / dir.y;
                dz = (dy * dir.z) / dir.y;
                dist = new THREE.Vector3(dx, dy, dz);
                if (threshold > dist.lengthSq()) {
                    if (minDist > point.distanceToSquared(pos)) {
                        minDist = point.distanceToSquared(pos);
                        result = dist;
                        snapTo = p;
                    }
                }
            }
            if (dir.z > 0.1 || dir.z < -0.1) {
                dz = point.z - pos.z;
                dx = (dz * dir.x) / dir.z;
                dy = (dz * dir.y) / dir.z;
                dist = new THREE.Vector3(dx, dy, dz);
                if (threshold > dist.lengthSq()) {
                    if (minDist > point.distanceToSquared(pos)) {
                        minDist = point.distanceToSquared(pos);
                        result = dist;
                        snapTo = p;
                    }
                }
            }
        }
        if (snapTo != null) {
            pos.add(result);
            this.snapingPoints.push(HELPERS.toVector3( snapTo ) );
            this.snapedPosition = HELPERS.toVector3(pos);
            return true;
        }
        return false;
    }

    this.snap = function(pos, dir, snapDist, nodes) {
        this.snapingPoints = [];
        this.snapedPosition = null;
        snap.call(this, pos, dir, snapDist, nodes);
        this.snapingPointsChange.notifySubscribers();
    }

    this.clearGuides = function () {
        this.snapingPoints = [];
        this.snapedPosition = null;
        this.snapingPointsChange.notifySubscribers();
    }

    this.snapPointOnPlane = function(pos, plane, nodes, precision, gridstep) {
        var planeNormal = plane.normal;
        var hasSnapped = false;
        var xDir = new THREE.Vector3();
        var yDir = new THREE.Vector3();
        xDir.crossVectors(new THREE.Vector3(0, 0, 1), planeNormal);
        if (xDir.length() < 0.0001) {
            xDir.crossVectors(new THREE.Vector3(0, 1, 0), planeNormal);
        }
        yDir.crossVectors(xDir, planeNormal);
        console.log('xDir : ' + xDir.x + ' , ' + xDir.y + ' , ' + xDir.z + ' , ' );
        console.log('yDir : ' + yDir.x + ' , ' + yDir.y + ' , ' + yDir.z + ' , ' );
        this.snapingPoints = [];
        this.snapedPosition = null;
        this.plane = plane;
        if(!snap.call(this, pos, xDir, precision, nodes)) {
            var nearestGridX = Math.round(pos.x / gridstep) * gridstep;
            if (Math.abs(nearestGridX - pos.x) <= precision) {
                pos.x = nearestGridX;
                hasSnapped = true;
            }
        }
        if(!snap.call(this, pos, yDir, precision, nodes)){
            var nearestGridY = Math.round( pos.y / gridstep ) * gridstep;
            if(Math.abs(nearestGridY - pos.y) <= precision) {
                pos.y = nearestGridY;
                hasSnapped = true;
            }
        }
        this.snapingPointsChange.notifySubscribers();
        return hasSnapped;
    }
}