﻿'use strict';
var VIEW3D = VIEW3D || {}
VIEW3D.materials = VIEW3D.materials || {}

VIEW3D.materials.AxisLabel = function(screenSize, texture, color) {
    THREE.ShaderMaterial.call(this, {
        transparent: true,
        uniforms: {
            screenSize: {
                type: "v2",
                value: new THREE.Vector2(screenSize().width, screenSize().height)
            },
            map: {
                type: "t",
                value: texture
            },
            color: {
                type: "v4",
                value: color
            },
            scale: {
                value: 1.0
            },
            offsetX: {
                value: 0.0
            }
        },
        vertexShader:
            'uniform vec2 screenSize;\n\
                attribute vec3 begin;\n\
                varying vec2 vUV;\n\
                uniform float scale;\n\
                uniform float offsetX;\n\
                \n\
                void main() {\n\
                    vec4 pos = projectionMatrix * modelViewMatrix * vec4( begin, 1.0 );\n\
                    pos.x += scale * (position.x - offsetX) / screenSize.x; \n\
                    pos.y += scale * position.y / screenSize.y; \n\
                    pos.z += position.z; \n\
                    gl_Position = pos;\n\
                    vUV = uv;\n\
            }',
        fragmentShader:
            'uniform sampler2D map;\n\
             varying vec2 vUV;\n\
             uniform vec4 color;\n\
             \n\
             void main() {\n\
                vec4 texture = texture2D( map, vUV );\n\
                vec4 c = texture * color;\n\
                if(c.w < 0.1)\n\
                    discard;\n\
                gl_FragColor = c;\n\
            }'
    });
    this.onScreenSizeChanged = function(newVal) {
        this.uniforms.screenSize.value = new THREE.Vector2(newVal.width, newVal.height);
    }
    HELPERS.subscribePropertyChange(this, screenSize, "screenSize", this.onScreenSizeChanged);
}
VIEW3D.materials.AxisLabel.prototype = Object.create(THREE.ShaderMaterial.prototype);