﻿'use strict';
var VIEW3D = VIEW3D || {}
VIEW3D.materials = VIEW3D.materials || {}

VIEW3D.materials.Cursor3D = function(aspectRatio, texture) {
    THREE.ShaderMaterial.call(this, {
        fog: false,
        transparent: true,
        uniforms: {
            aspectRatio: {
                type: "f",
                value: aspectRatio()
            },
            map: {
                type: "t",
                value: texture
            },
            pos3D: {
                type: "v3",
                value: new THREE.Vector3()
            },
            uOffset: {
                type: "f",
                value: 0.0
            }
        },
        vertexShader:
            'uniform float aspectRatio;\n\
		        uniform vec3 pos3D;\n\
                attribute vec3 begin;\n\
                varying vec2 vUV;\n\
				\n\
				void main() {\n\
					vec4 pos = projectionMatrix * modelViewMatrix * vec4( pos3D, 1.0 );\n\
					pos.x += position.x / aspectRatio; \n\
					pos.y += position.y; \n\
					pos.z += position.z - 0.1; \n\
					gl_Position = pos;\n\
                    vUV = uv;\n\
			}',
        fragmentShader:
            'uniform sampler2D map;\n\
             uniform float uOffset;\n\
             varying vec2 vUV;\n\
             \n\
             void main() {\n\
                vec2 uv = vUV;\n\
                uv.x += uOffset;\n\
                vec4 texture = texture2D( map, uv );\n\
				gl_FragColor = texture;\n\
			}'
    });
    this.onScreenSizeChanged = function(newVal) {
        this.uniforms.aspectRatio.value = newVal;
    }
    HELPERS.subscribePropertyChange(this, aspectRatio, "aspectRatio", this.onScreenSizeChanged);
}
VIEW3D.materials.Cursor3D.prototype = Object.create(THREE.ShaderMaterial.prototype);