﻿'use strict';
var VIEW3D = VIEW3D || {}
VIEW3D.materials = VIEW3D.materials || {}

VIEW3D.materials.Highlight = function(params) {
    if (isEmpty(params.offset)) params.offset = new THREE.Vector3(0, 0, 0);
    if (isEmpty(params.color)) params.color = new THREE.Vector3(1, 1, 1);
    this.size = CORE.options.nodeSize;
    var screenSize = CORE.screenSize;
    THREE.ShaderMaterial.call(this, {
        side: THREE.DoubleSide,
        uniforms: {
            offset: {
                type: "v3",
                value: params.offset
            },
            color: {
                type: "v3",
                value: params.color
            },
            screenSize: {
                type: "v2",
                value: new THREE.Vector2(screenSize().width, screenSize().height)
            },
        },
        vertexShader:
            'uniform vec2 screenSize;\n\
             attribute vec3 vertexOffset;\n\
             attribute vec3 vertexOffset2D;\n\
                uniform vec3 offset;\n\
                \n\
                void main(){\n\
                    vec4 pos = projectionMatrix * modelViewMatrix * vec4(position, 1.0);\n\
                    \n\
                    vec4 vertexOffset = modelViewMatrix * vec4( vertexOffset, 0.0 );\n\
                    vertexOffset = (projectionMatrix * vertexOffset);\n\
                    vertexOffset.xy /= projectionMatrix[1][1];//divide by zoom\n\
                    \n\
                    pos.xyz += vertexOffset.xyz;\n\
                    pos.xyz += offset;\n\
                    pos.xy += vertexOffset2D.xy / screenSize.xy;\n\
                    gl_Position = pos;\n\
                }',
        fragmentShader:
            'uniform vec3 color;\n\
             \n\
             void main() {\n\
                gl_FragColor = vec4( color, 1.0 );\n\
            }'
    });

    this.defaultAttributeValues = {
        'position': [ 0, 0, 0 ],
        'vertexOffset': [ 0, 0, 0 ],
        'vertexOffset2D': [ 0, 0, 0 ]
    };

    this.onScreenSizeChanged = function(newVal) {
        this.uniforms.screenSize.value = new THREE.Vector2(newVal.width, newVal.height);
    }
    HELPERS.subscribePropertyChange(this, screenSize, "screenSize", this.onScreenSizeChanged);

    //this.depthTest = false;
    //this.depthWrite = false;


}
VIEW3D.materials.Highlight.prototype = Object.create(THREE.ShaderMaterial.prototype);