﻿'use strict';
var VIEW3D = VIEW3D || {};
VIEW3D.materials = VIEW3D.materials || {};

VIEW3D.materials.LineLabel = function(aspectRatio, texture, color, offset) {

    THREE.ShaderMaterial.call(this, {
        fog: false,
        transparent: true,
        uniforms: {
            aspectRatio: {
                value: aspectRatio()
            },
            map: {
                value: texture
            },
            offset: {
                value: offset
            },
            color: {
                value: color || new THREE.Vector4(0, 0, 1, 1)   // default blue
            }
        },
        vertexShader:
            'uniform float aspectRatio;\n\
                uniform float offset;\n\
                attribute vec3 begin;\n\
                attribute vec3 end;\n\
                varying vec2 vUV;\n\
                \n\
                void main() {\n\
                    vec4 pos = projectionMatrix * modelViewMatrix * vec4( begin, 1.0 );\n\
                    vec4 end2d = projectionMatrix * modelViewMatrix * vec4( end, 1.0 );\n\
                    vec2 p = pos.xy / vec2(1, aspectRatio);\n\
                    vec2 e = end2d.xy / vec2(1, aspectRatio);\n\
                    vec2 middle = (end2d.xy - pos.xy) * 0.5; \n\
                    pos.xy += middle;\n\
                    \n\
                    vec3 xDir = vec3( e.xy - p.xy, 0);\n\
                    vec3 zDir = cross(xDir, vec3(-p.xy + vec2(0.00001,0.00001),0));\n\
                    vec3 yDir = cross(xDir, zDir);\n\
                    yDir = normalize(yDir);\n\
                    mat2 trans = mat2(yDir.y, -yDir.x, yDir.x, yDir.y);\n\
                    vec2 w = trans * vec2(0.0, offset);\n\
                    w.x /= aspectRatio;\n\
                    pos.xy +=  w; \n\
                    \n\
                    xDir = vec3( e.xy - p.xy, 0);\n\
                    zDir = cross(xDir, vec3(0.00001,-1.00001,0.0));\n\
                    yDir = cross(xDir, zDir);\n\
                    yDir = normalize(yDir);\n\
                    trans = mat2(yDir.y, -yDir.x, yDir.x, yDir.y);\n\
                    w = trans * position.xy;\n\
                    w.x /= aspectRatio;\n\
                    pos.xy +=  w; \n\
                    \n\
                    gl_Position = pos;\n\
                    vUV = uv;\n\
            }',
        fragmentShader:
            'uniform sampler2D map;\n\
             varying vec2 vUV;\n\
             uniform vec4 color;\n\
             \n\
             void main() {\n\
                vec4 texture = texture2D( map, vUV );\n\
                gl_FragColor = texture * color;\n\
            }'
    });
    this.onScreenSizeChanged = function(newVal) {
        this.uniforms.aspectRatio.value = newVal;
    };
    HELPERS.subscribePropertyChange(this, aspectRatio, "aspectRatio", this.onScreenSizeChanged);
};
VIEW3D.materials.LineLabel.prototype = Object.create(THREE.ShaderMaterial.prototype);