'use strict';
var VIEW3D = VIEW3D || {}
VIEW3D.materials = VIEW3D.materials || {}



VIEW3D.materials.LineMeasurements = function(color, screenSize, lineDir, measDir)
{
    var calcZoomParam = function() {
        var zoom = CORE.view3D.cameraControl.camera.zoom;
        var zoomProc = (zoom + 65)/100.0;
        var zoomParam = Math.pow(zoomProc,2.55);
        return zoomParam;
    };

    THREE.ShaderMaterial.call(this, {
        depthTest: false,
        uniforms: {
            offsetScale: {
                type: "f",
                value: calcZoomParam()
            },
            color: {
                type: "v4",
                value: color || VIEW3D.measurementColor
            },
            screenSize: {
                type: "v2",
                value: new THREE.Vector2(screenSize().width, screenSize().height)
            },
            lineDir: {
                type: "v3",
                value: lineDir
            },
            measDir: {
                type: "v3",
                value: measDir
            }
        },
        vertexShader: '\
                attribute float offset;\n\
                attribute vec3 vertexOffset;\n\
                uniform vec2 screenSize;\n\
                uniform vec3 lineDir;\n\
                uniform vec3 measDir;\n\
                uniform float offsetScale;\n\
                \n\
                void main() {\n\
                    vec3 csDir = (modelViewMatrix * vec4(lineDir, 0)).xyz;\n\
                    if(csDir.x<-0.0001) {\n\
                        csDir *= -1.0;\n\
                    }\n\
                    vec3 rqDir = (modelViewMatrix * vec4(measDir, 0)).xyz;\n\
                    float cross = csDir.x * rqDir.y - csDir.y * rqDir.x;\n\
                    if(cross>-0.0001 && rqDir.x*rqDir.x + rqDir.y*rqDir.y > 0.001) {\n\
                        csDir *= -1.0;\n\
                    }\n\
                    \n\
                    vec2 dir = vec2(csDir.y, -csDir.x);\n\
                    dir = normalize(dir);\n\
                    \n\
                    dir.xy *= offsetScale*offset/screenSize;\n\
                    vec4 pos = projectionMatrix * modelViewMatrix * vec4(position,1);\n\
                    pos.xy += dir.xy;\n\
                    \n\
                    vec4 vo = modelViewMatrix * vec4( vertexOffset, 0.0 );\n\
                    vo = (projectionMatrix * vo);\n\
                    vo.xy /= projectionMatrix[1][1];//divide by zoom\n\
                    vo.xy /= screenSize.yy;\n\
                    \n\
                    pos.xy += vo.xy;\n\
                    gl_Position = pos;\n\
            }',
        fragmentShader: 'uniform vec4 color;\n\
            void main() {\n\
                gl_FragColor = color;\n\
            }'
    });
    this.defaultAttributeValues = {
        'vertexOffset': [ 0, 0, 0 ],
    };

    this.onScreenSizeChanged = function (newVal) {
        this.uniforms.screenSize.value = new THREE.Vector2(newVal.width, newVal.height);
    }
    var self = this;
    this.onCameraChanged = function () {
        self.uniforms.offsetScale.value = calcZoomParam();
    }
    HELPERS.subscribePropertyChange(this, screenSize, "aspectRatio", this.onScreenSizeChanged);
    CORE.view3D.cameraControl.camera.changeEvent.subscribe(this.onCameraChanged);
}

VIEW3D.materials.LineMeasurements.prototype = Object.create(THREE.ShaderMaterial.prototype);