﻿'use strict';
var VIEW3D = VIEW3D || {}
VIEW3D.materials = VIEW3D.materials || {}

VIEW3D.materials.Measurement = function(aspectRatio) {
    THREE.ShaderMaterial.call(this, {
        uniforms: {
            aspectRatio: {
                type: "f",
                value: aspectRatio()
            },
            color: {
                type: "v4",
                value: VIEW3D.measurementColor
            }
        },
        vertexShader:
            'uniform float aspectRatio;\n\
                attribute vec2 offset;\n\
                attribute vec3 end;\n\
                \n\
                void main() {\n\
                    vec4 pos = projectionMatrix * modelViewMatrix * vec4( position, 1.0 );\n\
                    vec4 end2d = projectionMatrix * modelViewMatrix * vec4( end, 1.0 );\n\
                    vec2 p = pos.xy / vec2(1, aspectRatio);\n\
                    vec2 e = end2d.xy / vec2(1, aspectRatio);\n\
                    vec3 xDir = vec3( e.xy - p.xy, 0);\n\
                    vec3 zDir = cross(xDir, vec3(-p.xy + vec2(0.00001,0.00001),0));\n\
                    vec3 yDir = cross(xDir, zDir);\n\
                    yDir = normalize(yDir);\n\
                    mat2 trans = mat2(yDir.y, yDir.x, yDir.x, yDir.y);\n\
                    vec2 w = trans * offset;\n\
                    w.x /= aspectRatio;\n\
                    pos.xy += w.xy;\n\
                    //pos.z = -1.0;\n\
                    //pos.w = 1.0;\n\
                    gl_Position = pos;\n\
            }',
        fragmentShader:
            'uniform vec4 color;\n\
            void main() {\n\
                gl_FragColor = color;\n\
            }'
    });
    this.onScreenSizeChanged = function(newVal) {
        this.uniforms.aspectRatio.value = newVal;
    }
    HELPERS.subscribePropertyChange(this, aspectRatio, "aspectRatio", this.onScreenSizeChanged);
}
VIEW3D.materials.Measurement.prototype = Object.create(THREE.ShaderMaterial.prototype);