'use strict';
var VIEW3D = VIEW3D || {}
VIEW3D.materials = VIEW3D.materials || {}

VIEW3D.materials.MeasurementLabel = function(aspectRatio, texture, color) {
    THREE.ShaderMaterial.call(this, {
        fog: false,
        transparent: true,
        uniforms: {
            aspectRatio: {
                type: "f",
                value: aspectRatio()
            },
            map: {
                type: "t",
                value: texture
            },
            offset3D: {
                type: 'f',
                value: 0.005
            },
            color: {
                type: "v4",
                value: color || VIEW3D.measurementColor
            }
        },
        vertexShader:
            'uniform float aspectRatio;\n\
                uniform float offset3D;\n\
                attribute vec3 begin;\n\
                attribute vec3 dir1;\n\
                attribute vec3 dir2;\n\
                attribute vec3 end;\n\
                varying vec2 vUV;\n\
                \n\
                void main() {\n\
                    vec3 camDir = vec3(modelViewMatrix[0].z,modelViewMatrix[1].z,modelViewMatrix[2].z);\n\
                    float dot1 = dot(dir1, camDir);\n\
                    float dot2 = dot(dir2, camDir);\n\
                    vec3 dir;\
                    if(abs(dot1) < abs(dot2)){\n\
                        dir = dir1;\n\
                    }else{\n\
                        dir = dir2;\n\
                    }\n\
                    vec4 pos = vec4( begin + dir * offset3D, 1.0 );\n\
                    pos = projectionMatrix * modelViewMatrix * pos;\n\
                    vec4 end2d = vec4( end + dir * offset3D, 1.0 );\n\
                    end2d = projectionMatrix * modelViewMatrix * end2d;\n\
                    vec2 p = pos.xy / vec2(1, aspectRatio);\n\
                    vec2 e = end2d.xy / vec2(1, aspectRatio);\n\
                    vec2 middle = (end2d.xy - pos.xy) * 0.5; \n\
                    pos.xy += middle;\n\
                    vec3 xDir = vec3( e.xy - p.xy, 0);\n\
                    vec3 zDir = cross(xDir, vec3(0.00001,-1.00001,0.0));\n\
                    vec3 yDir = cross(xDir, zDir);\n\
                    yDir = normalize(yDir);\n\
                    mat2 trans = mat2(yDir.y, -yDir.x, yDir.x, yDir.y);\n\
                    vec2 w = trans * position.xy;\n\
                    w.x /= aspectRatio;\n\
                    pos.xy +=  w; \n\
                    \n\
                    gl_Position = pos;\n\
                    vUV = uv;\n\
            }',
        fragmentShader:
            'uniform vec4 color;\n\
            uniform sampler2D map;\n\
             varying vec2 vUV;\n\
             \n\
             void main() {\n\
                vec4 texture = texture2D( map, vUV );\n\
                gl_FragColor = texture * color;\n\
            }'
    });
    this.onScreenSizeChanged = function(newVal) {
        this.uniforms.aspectRatio.value = newVal;
    }
    HELPERS.subscribePropertyChange(this, aspectRatio, "aspectRatio", this.onScreenSizeChanged);
}
VIEW3D.materials.MeasurementLabel.prototype = Object.create(THREE.ShaderMaterial.prototype);    