'use strict';
var VIEW3D = VIEW3D || {}
VIEW3D.materials = VIEW3D.materials || {}

VIEW3D.materials.Measurements = function(color) {
    THREE.ShaderMaterial.call(this, {
        uniforms: {
            color: {
                type: "v4",
                value: color || VIEW3D.measurementColor
            }
        },
        vertexShader:
            '   attribute float offset;\n\
                attribute vec3 dir1;\n\
                attribute vec3 dir2;\n\
                \n\
                void main() {\n\
                    vec3 camDir = vec3(modelViewMatrix[0].z,modelViewMatrix[1].z,modelViewMatrix[2].z);\n\
                    float dot1 = dot(dir1, camDir);\n\
                    float dot2 = dot(dir2, camDir);\n\
                    vec3 dir;\
                    if(abs(dot1) < abs(dot2)){\n\
                        dir = dir1;\n\
                    }else{\n\
                        dir = dir2;\n\
                    }\n\
                    vec4 pos = vec4( position + dir * offset, 1.0 );\n\
                    pos = projectionMatrix * modelViewMatrix * pos;\n\
                    gl_Position = pos;\n\
            }',
        fragmentShader:
            'uniform vec4 color;\n\
            void main() {\n\
                gl_FragColor = color;\n\
            }'
    });
}
VIEW3D.materials.Measurements.prototype = Object.create(THREE.ShaderMaterial.prototype);

VIEW3D.materials.MeasurementsArrows = function(color, screenSize) {
    THREE.ShaderMaterial.call(this, {
        uniforms: {
            color: {
                type: "v4",
                value: color || VIEW3D.measurementColor
            },
            screenSize: {
                type: "v2",
                value: new THREE.Vector2(screenSize().width, screenSize().height)
            }
        },
        vertexShader:
            '   attribute float offset;\n\
                attribute vec3 dir1;\n\
                attribute vec3 dir2;\n\
                attribute vec3 elementPos;\n\
                uniform vec2 screenSize;\n\
                \n\
                void main() {\n\
                    vec3 camDir = vec3(modelViewMatrix[0].z,modelViewMatrix[1].z,modelViewMatrix[2].z);\n\
                    float dot1 = dot(dir1, camDir);\n\
                    float dot2 = dot(dir2, camDir);\n\
                    vec3 dir;\
                    if(abs(dot1) < abs(dot2)){\n\
                        dir = dir1;\n\
                    }else{\n\
                        dir = dir2;\n\
                    }\n\
                    vec4 pos = vec4( elementPos + dir * offset, 1.0 );\n\
                    pos = projectionMatrix * modelViewMatrix * pos;\n\
                    \n\
                    vec4 offset = modelViewMatrix * vec4( position, 0.0 );\n\
                    offset = (projectionMatrix * offset);\n\
                    offset.xy /= projectionMatrix[1][1];//divide by zoom\n\
                    offset.xy /= screenSize.yy;\n\
                    \n\
                    pos.xy += offset.xy;\n\
                    gl_Position = pos;\n\
            }',
        fragmentShader:
            'uniform vec4 color;\n\
            void main() {\n\
                gl_FragColor = color;\n\
            }'
    });
    this.onScreenSizeChanged = function(newVal) {
        this.uniforms.screenSize.value = new THREE.Vector2(newVal.width, newVal.height);
    }
    HELPERS.subscribePropertyChange(this, screenSize, "aspectRatio", this.onScreenSizeChanged);

}
VIEW3D.materials.MeasurementsArrows.prototype = Object.create(THREE.ShaderMaterial.prototype);