﻿'use strict';
var VIEW3D = VIEW3D || {}
VIEW3D.materials = VIEW3D.materials || {}

VIEW3D.materials.Metal = function (params) {
    if (params === undefined || params === null) {
        params = {};
    }
    if (params.ambientColor === undefined) {
        params.ambientColor = new THREE.Vector3(0.0, 0.0, 0.0);
    }
    if (params.diffuseColor === undefined) {
        params.diffuseColor = new THREE.Vector3(1.0, 1.0, 1.0);
    }
    if (params.specColor === undefined) {
        params.specColor = new THREE.Vector3(0.2, 0.2, 0.2);
    }
    THREE.ShaderMaterial.call(this, {
        vertexColors: THREE.VertexColors,
        fog: false,
        //wireframe: true,
        uniforms: {
            ambientColor: {
                type: "v3",
                value: params.ambientColor
            },
            diffuseColor: {
                type: "v3",
                value: params.diffuseColor
            },
            specColor: {
                type: "v3",
                value: params.specColor
            }
        },
        vertexShader:
            '   varying vec3 normalInterp;\n\
	            varying vec3 vertPos;\n\
	            varying vec3 vertColor;\n\
                \n\
	            void main(){\n\
		            gl_Position = projectionMatrix * modelViewMatrix * vec4(position, 1.0);\n\
		            vec4 vertPos4 = modelViewMatrix * vec4(position, 1.0);\n\
		            vertPos = vec3(vertPos4) / vertPos4.w;\n\
		            normalInterp = vec3(modelViewMatrix* vec4(normal, 0.0));\n\
		            vertColor = color;\n\
	            }',
        fragmentShader:
            'precision lowp float; \n\
                \n\
                varying vec3 normalInterp;\n\
                varying vec3 vertPos;\n\
	            varying vec3 vertColor;\n\
                \n\
	            const vec3 lightPos = vec3(0.0,0.0,1.0);\n\
	            uniform vec3 ambientColor;\n\
	            uniform vec3 diffuseColor;\n\
	            uniform vec3 specColor;\n\
                \n\
	            void main() {\n\
                \n\
		        vec3 normal = normalize(normalInterp);\n\
		        vec3 lightDir = lightPos;\n\
                \n\
		        float lambertian = abs(dot(lightDir,normal));\n\
                lambertian = pow(lambertian, 0.6);\n\
                \n\
		        vec3 viewDir = normalize(-vertPos);\n\
                \n\
		        // this is blinn phong\n\
		        vec3 halfDir = normalize(lightDir + viewDir);\n\
		        float specAngle = abs(dot(halfDir, normal));\n\
                float specular = pow(specAngle, 10.0);\n\
                \n\
		        gl_FragColor = vec4(ambientColor +\n\
							        lambertian * diffuseColor * vertColor+\n\
							        specular * specColor, 1.0);\n\
	            }'
    });
}
VIEW3D.materials.Metal.prototype = Object.create(THREE.ShaderMaterial.prototype);