﻿'use strict';
var VIEW3D = VIEW3D || {}
VIEW3D.materials = VIEW3D.materials || {}

VIEW3D.materials.ScreenSpace = function (opacity, scale) {
    THREE.ShaderMaterial.call(this, {
        fog: false,
        transparent: !isEmpty(opacity),
        vertexColors: THREE.VertexColors,
        uniforms: {
            opacity: {
                type: "f",
                value: !isEmpty(opacity) ? opacity : 1
            },
            scale: {
                type: "f",
                value: !isEmpty(scale) ? scale : 1
            }
        },

        vertexShader:
            'attribute vec3 elementPos;\n\
                varying vec3 vColor;\n\
                uniform float scale;\n\
                \n\
                void main() {\n\
                    vec4 pos = modelViewMatrix * vec4( elementPos, 1.0 );\n\
                    pos = (projectionMatrix * pos);\n\
                    \n\
                    vec4 offset = modelViewMatrix * vec4( position * scale, 0.0 );\n\
                    offset = (projectionMatrix * offset);\n\
                    offset.xy /= projectionMatrix[1][1];//divide by zoom\n\
                    \n\
                    pos.xyz += offset.xyz;\n\
                    gl_Position = pos;\n\
                    vColor=color;\n\
                }',
        fragmentShader:
            'uniform float opacity;\n\
            varying vec3 vColor;\n\
            \n\
             void main() {\n\
                gl_FragColor = vec4( vColor, opacity );\n\
            }'
    });
}
VIEW3D.materials.ScreenSpace.prototype = Object.create(THREE.ShaderMaterial.prototype);

VIEW3D.materials.EditControl = function (color) {
    THREE.ShaderMaterial.call(this, {
        fog: false,
        transparent: true,
        //color: color,

        uniforms: {
            controlColor: {
                type: "v4",
                value: color
            },
        },
        vertexShader:
                'varying vec4 vColor;\n\
                 uniform vec4 controlColor;\n\
                 varying vec3 vNormal;\n\
                \n\
                void main() {\n\
                    vec4 pos = modelViewMatrix * vec4( 0.0, 0.0 , 0.0 , 1.0 );\n\
                    vNormal = normalize((modelViewMatrix * vec4( normal , 0.0 )).xyz);\n\
                    pos = (projectionMatrix * pos);\n\
                    pos.z -= 0.2; \n\
                    \n\
                    vec4 offset = modelViewMatrix * vec4( position , 0.0 );\n\
                    offset = (projectionMatrix * offset);\n\
                    offset.xy /= projectionMatrix[1][1];//divide by zoom\n\
                    \n\
                    pos.xyz += offset.xyz;\n\
                    gl_Position = pos;\n\
                    //gl_Normal = nor;\n\
                    vColor=controlColor;\n\
                }',
        fragmentShader:
            '\n\
            varying vec4 vColor;\n\
            varying vec3 vNormal;\n\
            \n\
             void main() {\n\
                gl_FragColor = vec4( vColor.rgb * dot(vNormal, vec3(0,0,1)), vColor.a );\n\
            }'
    });
}
VIEW3D.materials.EditControl.prototype = Object.create(THREE.ShaderMaterial.prototype);

VIEW3D.materials.Angle = function (color, angle) {
    THREE.ShaderMaterial.call(this, {
        fog: false,
        transparent: true,
        side: THREE.DoubleSide,

        uniforms: {
            controlColor: {
                type: "v4",
                value: color
            },
            angle: {
                type: "f",
                value: angle
            },
        },
        vertexShader:
            'varying vec4 vColor;\n\
             uniform vec4 controlColor;\n\
             uniform float angle;\n\
            \n\
            void main() {\n\
                vec4 pos = modelViewMatrix * vec4( 0.0, 0.0 , 0.0 , 1.0 );\n\
                pos = (projectionMatrix * pos);\n\
                vec4 offset = modelViewMatrix * vec4( position.x * cos(position.y*angle), position.x * sin(position.y*angle), 0.0, 0.0 );\n\
                offset = (projectionMatrix * offset);\n\
                offset.xy /= projectionMatrix[1][1];//divide by zoom\n\
                \n\
                pos.xyz += offset.xyz;\n\
                pos.z -= 0.2;\n\
                gl_Position = pos;\n\
                vColor = controlColor;\n\
            }',
        fragmentShader:
            '\n\
            varying vec4 vColor;\n\
            \n\
             void main() {\n\
                gl_FragColor = vColor;\n\
            }'
    });
}
VIEW3D.materials.Angle.prototype = Object.create(THREE.ShaderMaterial.prototype);