"use strict";
var VIEW3D = VIEW3D || {};
VIEW3D.utils = VIEW3D.utils || {};
VIEW3D.utils.Labeling = VIEW3D.utils.Labeling || {};

VIEW3D.utils.Labeling.createSimpleLabelMesh = function(text, position) {
    var structs = VIEW3D.utils.Labeling.createStructures();
    structs.labelsObject.material.depthTest = false;
    VIEW3D.utils.Labeling.createLabel(structs, { text : text, dir : new THREE.Vector3(0, 0, 0), pos : position }, 1, false);
    return structs.labelsObject;
}

VIEW3D.utils.Labeling.createStructures = function(color, textureName) {
    var actualTextureName = textureName || "extremum";
    var obj = CORE.res.obj["extremum"];
    var g = obj.children[0].geometry;
    if (isEmpty(g.getIndex())) {
        VIEW3D.utils.optimize(g);
    }
    var bitmapFont = new VIEW3D.BitmapFont(VIEW3D.BitmapFont.extremumData, CORE.res.texture[actualTextureName]);
    //vertex format
    var attributes = {
        position: {
            type: Float32Array,
            size: 3
        },
        begin: {
            type: Float32Array,
            size: 3
        },
        uv: {
            type: Float32Array,
            size: 2
        }
    };
    var labelsObject = new VIEW3D.MultiMesh(attributes, new VIEW3D.materials.AxisLabel(CORE.screenSize, bitmapFont.texture, color || new THREE.Vector4(0.9, 0.9, 1.0, 1)));
    return { labelsObject : labelsObject  , bitmapFont : bitmapFont, g : g };
}

VIEW3D.utils.Labeling.createLabel = function(structures, label, scale, mirror) {
    var geometry = structures.bitmapFont.createText();
    geometry.update({
        text: label.text,
        align: 'center',
        font: structures.bitmapFont.font
    });

    var maxX = 0;
    for (var k = 0; k < geometry.attributes.position.array.length / 2; k++) {
        var x = geometry.attributes.position.array[k * 2] - 15.0;
        maxX = Math.max(x, maxX);
    }
    // console.log(label.text + " -> " + String(maxX));
    var mesh = {
        getIndicesCount: function() {
            return geometry.getIndex().array.length;
        },
        getVerticesCount: function() {
            return geometry.attributes.position.array.length / 2;
        },
        writeAttributes: function(attrs, indices, sv, si) {
            //index
            var i;
            for (i = 0; i < geometry.getIndex().array.length; i++) {
                indices.array[si + i] = geometry.getIndex().array[i] + sv;
            }
            var posSrc = geometry.attributes.position.array;
            var posDest = attrs.position.array;
            var uvSrc = geometry.attributes.uv.array;
            var uvDest = attrs.uv.array;
            var beginDest = attrs.begin.array;
            var x = label.pos.x + label.dir.x * scale;
            var y = label.pos.y + label.dir.y * scale;
            var z = label.pos.z + label.dir.z * scale;
            for (i = 0; i < posSrc.length / 2; i++) {
                var tx = posSrc[i * 2 + 0] - 15.0;
                posDest[(sv + i) * 3 + 0] = tx;
                posDest[(sv + i) * 3 + 1] = -(posSrc[i * 2 + 1] - 25.0);
                posDest[(sv + i) * 3 + 2] = 0.0;
                uvDest[(sv + i) * 2 + 0] = uvSrc[i * 2 + 0];
                uvDest[(sv + i) * 2 + 1] = uvSrc[i * 2 + 1];
                beginDest[(sv + i) * 3 + 0] = x;
                beginDest[(sv + i) * 3 + 1] = y;
                beginDest[(sv + i) * 3 + 2] = z;
            }
        }
    }
    var backgrpundTemplate = {
        getIndicesCount: function() {
            return structures.g.getIndex().array.length;
        },
        getVerticesCount: function() {
            return structures.g.attributes.position.array.length / 3;
        },
        writeAttributes: function(attrs, indices, sv, si) {
            //index
            var i;
            for (i = 0; i < structures.g.getIndex().array.length; i++) {
                indices.array[si + i] = structures.g.getIndex().array[i] + sv;
            }
            var posSrc = structures.g.attributes.position.array;
            var posDest = attrs.position.array;
            var uvSrc = structures.g.attributes.uv.array;
            var uvDest = attrs.uv.array;
            var beginDest = attrs.begin.array;
            var x = label.pos.x + label.dir.x * scale;
            var y = label.pos.y + label.dir.y * scale;
            var z = label.pos.z + label.dir.z * scale;
            var minTx = 1000000;
            var maxTx = -1000000;
            for (i = 0; i < posSrc.length / 2; i++) {
                var tx = posSrc[i * 3 + 0];
                if (tx > 0) {
                    tx += maxX - 60;
                }
                if(tx != undefined) minTx = Math.min(tx, minTx);
                if(tx != undefined) maxTx = Math.max(tx, maxTx);
                
                posDest[(sv + i) * 3 + 0] = tx;
                posDest[(sv + i) * 3 + 1] = posSrc[i * 3 + 1];
                posDest[(sv + i) * 3 + 2] = 0.0001;
                uvDest[(sv + i) * 2 + 0] = uvSrc[i * 2 + 0];
                uvDest[(sv + i) * 2 + 1] = uvSrc[i * 2 + 1];
                beginDest[(sv + i) * 3 + 0] = x;
                beginDest[(sv + i) * 3 + 1] = y;
                beginDest[(sv + i) * 3 + 2] = z;
            }
            var offsetX = 0;
            if(mirror) {
                var offsetX = maxTx - minTx;
            } 
            structures.labelsObject.material.uniforms.offsetX.value = offsetX;//.toFixed(4);
        }
    }

    structures.labelsObject.addMeshFromTemplate(backgrpundTemplate);
    structures.labelsObject.addMeshFromTemplate(mesh);
}