'use strict';
var VIEW3D = VIEW3D || {}
VIEW3D.utils = VIEW3D.utils || {}

define([],function () {

    VIEW3D.utils.ResizableGeometry = function (attributes) {
        THREE.BufferGeometry.call(this);
        var verticesMargine = 100000;
        var indexMargine = 200000;

        this.verticesInUse = 0;
        this.indicesInUse = 0;

        this.getIndicesSpace = function(){
            if(isEmpty(this.index)){
                return 0;
            }
            return this.index.count;
        }

        this.getVerticesSpace = function(){
            if(isEmpty(this.attributes.position)){
                return 0;
            }
            return this.attributes.position.count;
        }

        this.changeSize = function(indices, vertices) {
            if(this.getVerticesSpace() < vertices ||      // vertex buffers too small
                this.getIndicesSpace() < indices){      // vertex buffers too small
                var newIndexBufferSize = indices + indexMargine;
                var newVertexBufferSize = vertices + verticesMargine;
                this.dispose();//release resources
                this.setIndex(new THREE.BufferAttribute(new Uint32Array(newIndexBufferSize), 1), 1);
                for (var attr in attributes) {
                    if (attributes.hasOwnProperty(attr)) {
                        var Type = attributes[attr].type;
                        var elementSize = attributes[attr].size;
                        var size = newVertexBufferSize*elementSize;
                        this.addAttribute(attr, new THREE.BufferAttribute(new Type(size), elementSize));
                    }
                }
            }

            //TODO: limit update range (right now its not working in THREEjs for non dynamic buffers? )
            this.index.needsUpdate = true;
            for (var attr in attributes) {
                if (attributes.hasOwnProperty(attr)) {
                    this.attributes[attr].needsUpdate = true;
                }
            }

            this.verticesInUse = vertices;
            this.indicesInUse = indices;
            this.drawRange.count = this.indicesInUse;
        }

        this.increaseSize = function(indices, vertices) {
            if(this.getVerticesSpace() < vertices + this.verticesInUse      // vertex buffers too small
                || this.getIndicesSpace() < indices + this.indicesInUse) {  // index buffer too small

                var newIndexBufferSize = indices + this.indicesInUse + indexMargine;
                var newVertexBufferSize = vertices + this.verticesInUse + verticesMargine;
                this.dispose();//release resources
                this.setIndex(VIEW3D.utils.increaseAttribute(this.getIndex(), newIndexBufferSize, Uint32Array, 1));
                for (var attr in attributes) {
                    if (attributes.hasOwnProperty(attr)) {
                        this.addAttribute(attr, VIEW3D.utils.increaseAttribute(
                            this.attributes[attr],
                            newVertexBufferSize,
                            attributes[attr].type,
                            attributes[attr].size));
                    }
                }
            }

            //TODO: limit update range (right now its not working in THREEjs for non dynamic buffers? )
            this.index.needsUpdate = true;
            for (var attr in attributes) {
                if (attributes.hasOwnProperty(attr)) {
                    this.attributes[attr].needsUpdate = true;
                }
            }

            this.verticesInUse += vertices;
            this.indicesInUse += indices;
            this.drawRange.count = this.indicesInUse;
        }

        this.splice = function(startIndex, indices, startVertex, vertices) {
            //Do we want to have buffer reduction?
            this.setIndex(
                VIEW3D.utils.bufferLeftShiftRange(
                    this.getIndex(),
                    startIndex,
                    indices,
                    Uint32Array,
                    1,
                    vertices,
                    this.indicesInUse
                ));

            for (var attr in attributes) {
                if (attributes.hasOwnProperty(attr)) {
                    this.addAttribute(attr, VIEW3D.utils.bufferLeftShiftRange(
                        this.attributes[attr],
                        startVertex,
                        vertices,
                        attributes[attr].type,
                        attributes[attr].size,
                        undefined,
                        this.verticesInUse));
                }
            }

            //TODO: limit update range (right now its not working in THREEjs for non dynamic buffers? )
            this.index.needsUpdate = true;
            for (var attr in attributes) {
                if (attributes.hasOwnProperty(attr)) {
                    this.attributes[attr].needsUpdate = true;
                }
            }

            this.verticesInUse -= vertices;
            this.indicesInUse -= indices;
            this.drawRange.count = this.indicesInUse;
        }

    }
    VIEW3D.utils.ResizableGeometry.prototype = Object.create(THREE.BufferGeometry.prototype);


    VIEW3D.utils.ResizableNonIndexedGeometry = function (attributes, verticesMargine) {
        THREE.BufferGeometry.call(this);
        var verticesMargine = verticesMargine || 100000 ;

        this.verticesInUse = 0;


        this.getVerticesSpace = function(){
            if(isEmpty(this.attributes.position)){
                return 0;
            }
            return this.attributes.position.count;
        }

        //will not copy old data
        this.changeSize = function(vertices) {
            if(this.getVerticesSpace() < vertices){      // vertex buffers too small
                var newVertexBufferSize = vertices + verticesMargine;
                this.dispose();//release resources
                for (var attr in attributes) {
                    if (attributes.hasOwnProperty(attr)) {
                        var Type = attributes[attr].type;
                        var elementSize = attributes[attr].size;
                        var size = newVertexBufferSize*elementSize;
                        this.addAttribute(attr, new THREE.BufferAttribute(new Type(size), elementSize));
                    }
                }
            }

            //TODO: limit update range (right now its not working in THREEjs for non dynamic buffers? )
            for (var attr in attributes) {
                if (attributes.hasOwnProperty(attr)) {
                    this.attributes[attr].needsUpdate = true;
                }
            }

            this.verticesInUse = vertices;
            this.drawRange.count = this.verticesInUse;
        }

        this.increaseSize = function(vertices) {
            if(this.getVerticesSpace() < vertices + this.verticesInUse){      // vertex buffers too small
                var newVertexBufferSize = vertices + this.verticesInUse + verticesMargine;
                this.dispose();//release resources
                for (var attr in attributes) {
                    if (attributes.hasOwnProperty(attr)) {
                        this.addAttribute(attr, VIEW3D.utils.increaseAttribute(
                            this.attributes[attr],
                            newVertexBufferSize,
                            attributes[attr].type,
                            attributes[attr].size));
                    }
                }
            }

            //TODO: limit update range (right now its not working in THREEjs for non dynamic buffers? )
            for (var attr in attributes) {
                if (attributes.hasOwnProperty(attr)) {
                    this.attributes[attr].needsUpdate = true;
                }
            }

            this.verticesInUse += vertices;
            this.drawRange.count = this.verticesInUse;
        }

        this.splice = function(startVertex, vertices) {
            for (var attr in attributes) {
                if (attributes.hasOwnProperty(attr)) {
                    this.addAttribute(attr, VIEW3D.utils.bufferLeftShiftRange(
                        this.attributes[attr],
                        startVertex,
                        vertices,
                        attributes[attr].type,
                        attributes[attr].size,
                        undefined,
                        this.verticesInUse));
                }
            }

            //TODO: limit update range (right now its not working in THREEjs for non dynamic buffers? )
            for (var attr in attributes) {
                if (attributes.hasOwnProperty(attr)) {
                    this.attributes[attr].needsUpdate = true;
                }
            }

            this.verticesInUse -= vertices;
            this.drawRange.count = this.verticesInUse;
        }

    }
    VIEW3D.utils.ResizableNonIndexedGeometry.prototype = Object.create(THREE.BufferGeometry.prototype);
});

