﻿"use strict";
var VIEW3D = VIEW3D || {};
VIEW3D.utils = VIEW3D.utils || {};
VIEW3D.utils.optimize = function(geometry) {
    var tempPos = geometry.attributes.position.array;
    var tempNor = geometry.attributes.normal === undefined ? null : geometry.attributes.normal.array;
    var tempUv = geometry.attributes.uv === undefined ? null : geometry.attributes.uv.array;
    var count = tempPos.length / 3;
    var indices = [];
    var positions = [];
    var normals = [];
    var uv = [];
    var comparePos = function(x1, x2, y1, y2, z1, z2) {
        return x1 === x2 && y1 === y2 && z1 === z2;
    };
    var compareNor = function(x1, x2, y1, y2, z1, z2) {
        return x1 * x2 + y1 * y2 + z1 * z2 > 0.8;
    };
    var compareUv = function(x1, x2, y1, y2) {
        var dx = x1 - x2;
        var dy = y1 - y2;
        return dx * dx + dy * dy < 0.0001;
    };
    var compare = function(i, j) {
        if (!comparePos(tempPos[i * 3 + 0],
            positions[j * 3 + 0],
            tempPos[i * 3 + 1],
            positions[j * 3 + 1],
            tempPos[i * 3 + 2],
            positions[j * 3 + 2])) {
            return false;
        }
        if (!isEmpty(tempNor) &&
            !compareNor(tempNor[i * 3 + 0],
                normals[j * 3 + 0],
                tempNor[i * 3 + 1],
                normals[j * 3 + 1],
                tempNor[i * 3 + 2],
                normals[j * 3 + 2])) {
            return false;
        }
        if (!isEmpty(tempUv) && !compareUv(tempUv[i * 2 + 0], uv[j * 2 + 0], tempUv[i * 2 + 1], uv[j * 2 + 1])) {
            return false;
        }
        return true;
    };
    for (var i = 0; i < count; i++) {
        var found = false;
        var readyCount = positions.length / 3;
        for (var j = 0; j < readyCount; j++) {
            if (compare(i, j)) {
                indices.push(j);
                found = true;
                break;
            }
        }
        if (!found) {
            indices.push(readyCount);
            positions.push(tempPos[i * 3 + 0]);
            positions.push(tempPos[i * 3 + 1]);
            positions.push(tempPos[i * 3 + 2]);
            if (tempNor != null) {
                normals.push(tempNor[i * 3 + 0]);
                normals.push(tempNor[i * 3 + 1]);
                normals.push(tempNor[i * 3 + 2]);
            }
            if (tempUv != null) {
                uv.push(tempUv[i * 2 + 0]);
                uv.push(tempUv[i * 2 + 1]);
            }
        }
    }
    geometry.setIndex(new THREE.BufferAttribute(new Uint32Array(indices), 1));
    geometry.addAttribute("position", new THREE.BufferAttribute(new Float32Array(positions), 3));
    if (tempNor != null) {
        geometry.addAttribute("normal", new THREE.BufferAttribute(new Float32Array(normals), 3));
    }
    if (tempUv != null) {
        geometry.addAttribute("uv", new THREE.BufferAttribute(new Float32Array(uv), 2));
    }
};
VIEW3D.utils.getBaseFromDir = function(dir) {
    var vectX = dir.clone();
    var vectY = new THREE.Vector3(0, 0, 0);
    var vectZ = new THREE.Vector3(0, 0, 0);
    var len = vectX.length();
    var lxy = Math.sqrt(vectX.x * vectX.x + vectX.y * vectX.y);
    var ca = dir.z < 0 ? -1 : 1;
    var sa = 0;
    var cb = lxy / len;
    var sb = -vectX.z / len;
    var cg = 1;
    var sg = 0;
    if (lxy !== 0) {
        ca = vectX.x / lxy;
        sa = vectX.y / lxy;
    }
    vectX.normalize();
    vectY.x = ca * sb * sg - sa * cg;
    vectY.y = sa * sb * sg + ca * cg;
    vectY.z = cb * sg;
    vectZ.x = ca * sb * cg + sa * sg;
    vectZ.y = sa * sb * cg - ca * sg;
    vectZ.z = cb * cg;
    return {
        baseX: vectX,
        baseY: vectY,
        baseZ: vectZ
    };
};
VIEW3D.utils.Triangulation = function() {
    this.epsilon = 0.0000000001;
    this.process = function(contour, result) {
        // initialize list of vertices in polygon
        var n = contour.length;
        if (n < 3) {
            return false;
        }
        var polygonV = [];
        // counter-clockwise polygon in V
        var i;
        if (0.0 < this.area(contour)) {
            for (i = 0; i < n; i++)
                polygonV[i] = i;
        } else {
            for (i = 0; i < n; i++)
                polygonV[i] = (n - 1) - i;
        }
        var nv = n;
        // remove nv-2 Vertices, creating 1 triangle every time
        var count = 2 * nv; // error detection
        for (var m = 0, v = nv - 1; nv > 2;) {
            // if we loop, it is probably a non-simple polygon
            if (0 >= (count--)) {
                // Triangulate: ERROR - probable bad polygon!
                return false;
            }
            // three consecutive vertices in current polygon, <u,v,w>
            var u = v;
            // previous
            if (nv <= u) {
                u = 0;
            }
            v = u + 1;
            // new v
            if (nv <= v) {
                v = 0;
            }
            var w = v + 1;
            // next
            if (nv <= w) {
                w = 0;
            }
            if (this.snip(contour, u, v, w, nv, polygonV)) {
                var s, t;
                // true names of the vertices
                var a = polygonV[u];
                var b = polygonV[v];
                var c = polygonV[w];
                // output Triangle
                result.push(contour[a]);
                result.push(contour[b]);
                result.push(contour[c]);
                m++;
                // remove v from remaining polygon
                for (s = v, t = v + 1; t < nv; s++, t++) {
                    polygonV[s] = polygonV[t];
                }
                nv--;
                // resest error detection counter
                count = 2 * nv;
            }
        }
        return true;
    };
    this.snip = function(contour, u, v, w, n, polygonV) {
        var px, py;
        var ax = contour[polygonV[u]].vec2.x;
        var ay = contour[polygonV[u]].vec2.y;
        var bx = contour[polygonV[v]].vec2.x;
        var by = contour[polygonV[v]].vec2.y;
        var cx = contour[polygonV[w]].vec2.x;
        var cy = contour[polygonV[w]].vec2.y;
        if (this.epsilon > (((bx - ax) * (cy - ay)) - ((by - ay) * (cx - ax)))) {
            return false;
        }
        for (var p = 0; p < n; p++) {
            if ((p === u) || (p === v) || (p === w)) {
                continue;
            }
            px = contour[polygonV[p]].vec2.x;
            py = contour[polygonV[p]].vec2.y;
            if (this.insideTriangle(ax, ay, bx, by, cx, cy, px, py)) {
                return false;
            }
        }
        return true;
    };
    this.area = function(contour) {
        var n = contour.length;
        var a = 0.0;
        for (var p = n - 1, q = 0; q < n; p = q++) {
            a += contour[p].vec2.x * contour[q].vec2.y - contour[q].vec2.x * contour[p].vec2.y;
        }
        return a * 0.5;
    };
    this.insideTriangle = function(ax, ay, bx, by, cx, cy, px, py) {
        var newAx = cx - bx;
        var newAy = cy - by;
        var newBx = ax - cx;
        var newBy = ay - cy;
        var newCx = bx - ax;
        var newCy = by - ay;
        var apx = px - ax;
        var apy = py - ay;
        var bpx = px - bx;
        var bpy = py - by;
        var cpx = px - cx;
        var cpy = py - cy;
        var aCrosSbp = newAx * bpy - newAy * bpx;
        var cCrosSap = newCx * apy - newCy * apx;
        var bCrosScp = newBx * cpy - newBy * cpx;
        return ((aCrosSbp >= 0.0) && (bCrosScp >= 0.0) && (cCrosSap >= 0.0));
    };
};
VIEW3D.utils.increaseAttribute = function(oldBuff, newElementsCount, Type, size, bufferConstracor) {
    // it can also be set to InstancedBufferAttribute
    var bufferConstracor = bufferConstracor || THREE.BufferAttribute;
    var oldExists = !isEmpty(oldBuff) && !isEmpty(oldBuff.array);
    var oldSize = !oldExists ? 0 : oldBuff.array.length;
    var newSize = oldSize + newElementsCount * size;
    var newBuffer = new Type(newSize);
    if (oldExists) {
        newBuffer.set(oldBuff.array, 0);
    }
    return new bufferConstracor(newBuffer, size);
};
VIEW3D.utils.reduceBuffer = function(oldBuff, startPos, toRemoveCount, Type, size, verticesOffset) {
    startPos *= size;
    toRemoveCount *= size;
    var oldSize = oldBuff.array.length;
    var newSize = oldSize - toRemoveCount;
    if (newSize === 0) {
        return new THREE.BufferAttribute([], size);
    }
    var newBuffer = new Type(newSize);
    if (startPos > 0) {
        newBuffer.set(oldBuff.array.subarray(0, startPos));
    }
    if (startPos + toRemoveCount < oldSize) {
        if (verticesOffset !== undefined) {
            // remove indices from index buffer and shift indices in "older" meshes
            for (var i = 0; i < newSize - startPos; i++) {
                newBuffer[startPos + i] = oldBuff.array[startPos + toRemoveCount + i] - verticesOffset;
            }
        } else {
            newBuffer.set(oldBuff.array.subarray(startPos + toRemoveCount, oldSize), startPos);
        }
    }
    return new THREE.BufferAttribute(newBuffer, size);
};
VIEW3D.utils.bufferLeftShiftRange = function(buff, startPos, toShift, Type, size, verticesOffset, oldSize) {
    startPos *= size;
    toShift *= size;
    oldSize *= size;
    var newSize = oldSize - toShift;
    if (startPos + toShift < oldSize) {
        if (verticesOffset !== undefined) {
            // remove indices from index buffer and shift indices in "older" meshes
            for (var i = 0; i < newSize - startPos; i++) {
                buff.array[startPos + i] = buff.array[startPos + toShift + i] - verticesOffset;
            }
        } else {
            buff.array.set(buff.array.subarray(startPos + toShift, oldSize), startPos);
        }
    }
    return buff;
};
VIEW3D.utils.templates = {};
VIEW3D.utils.getColorableTemplate = function(modelId, callback) {
    var template = VIEW3D.utils.templates[modelId];
    if (!isEmpty(template)) {
        callback(template);
        return;
    }
    MODEL.repository.get3DModelAsync(modelId,
        function(model) {
            try{
                var templateGeometry = MODEL.utils.Proto.getBufferGeometryFromBase64(model.data, modelId);
                var verticesCount = templateGeometry.attributes.position.array.length / 3;
                templateGeometry.addAttribute("color", new THREE.BufferAttribute(new Float32Array(verticesCount * 3), 3));
                templateGeometry.computeBoundingBox();
                template = new VIEW3D.ColorableMeshTemplate();
                template.setGeometry(templateGeometry);
                template.boundingBox = templateGeometry.boundingBox;
                template.config = model.config;
                VIEW3D.utils.templates[modelId] = template;
            } catch (error) {
                errorCallback(error);
                return;
            }
            callback(template);
        });
};
VIEW3D.utils.getTemplateGeometry = function(modelId) {
    var template = VIEW3D.utils.templates[modelId];
    if (isEmpty(template)) {
        return null;
    }
    return template.getGeometry();
};
VIEW3D.utils.getTemplate = function(modelId, callback, errorCallback) {
    var template = VIEW3D.utils.templates[modelId];
    if (!isEmpty(template)) {
        callback(template);
        return;
    }
    MODEL.repository.get3DModelAsync(modelId,
        function(model) {
            try {
                var templateGeometry = MODEL.utils.Proto.getBufferGeometryFromBase64(model.data, modelId);
                var verticesCount = templateGeometry.attributes.position.array.length / 3;
                templateGeometry.addAttribute("color",
                    new THREE.BufferAttribute(new Float32Array(verticesCount * 3), 3));
                templateGeometry.computeBoundingBox();
                template = new VIEW3D.MeshTemplate();
                template.setGeometry(templateGeometry);
                template.boundingBox = templateGeometry.boundingBox;
                template.config = model.config;
                VIEW3D.utils.templates[modelId] = template;
            } catch (error) {
                errorCallback(error);
                return;
            }
            callback(template);
        });
};
VIEW3D.utils.getMultiMaterialTemplate = function(modelId, callback, errorCallback) {
    var template = VIEW3D.utils.templates[modelId];
    if (!isEmpty(template)) {
        callback(template);
        return;
    }
    MODEL.repository.get3DModelAsync(modelId,
        function(model) {
            try {
                var obj = MODEL.utils.Proto.getObject3dFromBase64(model.data, modelId);
                template = new VIEW3D.MultiMaterialTemplate(obj, model.config);
                VIEW3D.utils.templates[modelId] = template;
            } catch (error) {
                errorCallback(error);
                return;
            }
            callback(template);
        });
};

VIEW3D.utils.getMatrix4FromBigBase = function(base) {
    return new THREE.Matrix4().set(
        base.BaseX.X,
        base.BaseY.X,
        base.BaseZ.X,
        base.Origin.X,
        base.BaseX.Y,
        base.BaseY.Y,
        base.BaseZ.Y,
        base.Origin.Y,
        base.BaseX.Z,
        base.BaseY.Z,
        base.BaseZ.Z,
        base.Origin.Z,
        0,
        0,
        0,
        1);
};

VIEW3D.utils.getMatrix4FromBase = function(base) {
    return new THREE.Matrix4().set(
        base.baseX.x,
        base.baseY.x,
        base.baseZ.x,
        base.origin.x,
        base.baseX.y,
        base.baseY.y,
        base.baseZ.y,
        base.origin.y,
        base.baseX.z,
        base.baseY.z,
        base.baseZ.z,
        base.origin.z,
        0,
        0,
        0,
        1);
};
VIEW3D.utils.getRotationMatrix4FromBase = function(base) {
    return new THREE.Matrix4().set(
        base.baseX.x,
        base.baseY.x,
        base.baseZ.x,
        0,
        base.baseX.y,
        base.baseY.y,
        base.baseZ.y,
        0,
        base.baseX.z,
        base.baseY.z,
        base.baseZ.z,
        0,
        0,
        0,
        0,
        1);
};
VIEW3D.utils.getTranslationMatrix4FromBase = function(base) {
    return new THREE.Matrix4().set(
        1,
        0,
        0,
        base.origin.x,
        0,
        1,
        0,
        base.origin.y,
        0,
        0,
        1,
        base.origin.z,
        0,
        0,
        0,
        1);
};
VIEW3D.utils.getOriginFromBase = function(base) {
    return new THREE.Vector3(
        base.origin.x,
        base.origin.y,
        base.origin.z);
};
VIEW3D.utils.areMatrixesEqual = function(matrix1, matrix2) {
    var isMatrix1Empty = isEmpty(matrix1);
    var isMatrix2Empty = isEmpty(matrix2);
    if (isMatrix1Empty && isMatrix2Empty) {
        return true;
    } else if ((isMatrix1Empty && !isMatrix2Empty) || (!isMatrix1Empty && isMatrix2Empty)) {
        return false;
    }
    return Number(matrix1[0].toFixed(4)) === Number(matrix2[0].toFixed(4)) &&
        Number(matrix1[5].toFixed(4)) === Number(matrix2[5].toFixed(4)) &&
        Number(matrix1[12].toFixed(4)) === Number(matrix2[12].toFixed(4)) &&
        Number(matrix1[13].toFixed(4)) === Number(matrix2[13].toFixed(4)) &&
        Number(matrix1[14].toFixed(4)) === Number(matrix2[14].toFixed(4));
}
VIEW3D.utils.transtomAndMatrixAreEqual = function(t, m) {
    var te = m.elements;
    return t.baseX.x === Number(te[0].toFixed(4)) &&
        t.baseY.y === Number(te[5].toFixed(4)) &&
        t.origin.x === Number(te[12].toFixed(4)) &&
        t.origin.y === Number(te[13].toFixed(4)) &&
        t.origin.z === Number(te[14].toFixed(4));
};
VIEW3D.utils.getTransformationFromSegment = function(pt1, pt2) {
    var vectX = new THREE.Vector3(0, 0, 0);
    var vectY = new THREE.Vector3(0, 0, 0);
    var vectZ = new THREE.Vector3(0, 0, 0);
    vectX.subVectors(pt2, pt1);
    var len = vectX.length();
    var lxy = Math.sqrt(vectX.x * vectX.x + vectX.y * vectX.y);
    var ca = 1;
    var sa = 0;
    var cb = lxy / len;
    var sb = -vectX.z / len;
    var cg = 1;
    var sg = 0;
    if (lxy !== 0) {
        ca = vectX.x / lxy;
        sa = vectX.y / lxy;
    }
    vectX.normalize();
    vectY.x = ca * sb * sg - sa * cg;
    vectY.y = sa * sb * sg + ca * cg;
    vectY.z = cb * sg;
    vectZ.x = ca * sb * cg + sa * sg;
    vectZ.y = sa * sb * cg - ca * sg;
    vectZ.z = cb * cg;
    return new THREE.Matrix4().set(
        vectX.x,
        vectY.x,
        vectZ.x,
        0,
        vectX.y,
        vectY.y,
        vectZ.y,
        0,
        vectX.z,
        vectY.z,
        vectZ.z,
        0,
        0,
        0,
        0,
        1);
};
VIEW3D.utils.getRotationSpace = function(rotAixa) {
    if (rotAixa) {
        var vectX = new THREE.Vector3(0, 0, 0);
        var vectY = new THREE.Vector3(0, 0, 0);
        var vectZ = new THREE.Vector3(0, 0, 0);
        vectY.crossVectors(vectZ, new THREE.Vector3(1, 0, 0));
        if (vectY.lengthSq() < 0.01) {
            vectY.set(0, 0, 1);
        } else {
            vectY.normalize();
        }
        vectX.crossVectors(vectY, vectZ);
        return new THREE.Matrix4().set(
            vectX.x,
            vectY.x,
            vectZ.x,
            0,
            vectX.y,
            vectY.y,
            vectZ.y,
            0,
            vectX.z,
            vectY.z,
            vectZ.z,
            0,
            0,
            0,
            0,
            1);
    }
    return null;
};
VIEW3D.utils.segmentsIntersection = function(v0, v1, w0, w1) {
    var w = w1.clone().sub(w0);
    var direction = w.clone().normalize();
    var segCenter = new THREE.Vector3();
    var segDir = new THREE.Vector3();
    var diff = new THREE.Vector3();
    segCenter.copy(v0).add(v1).multiplyScalar(0.5);
    segDir.copy(v1).sub(v0).normalize();
    diff.copy(w0).sub(segCenter);
    var segExtent = v0.distanceTo(v1) * 0.5;
    var a01 = - direction.dot(segDir);
    var b0 = diff.dot(direction);
    var b1 = - diff.dot(segDir);
    var c = diff.lengthSq();
    var det = Math.abs(1 - a01 * a01);
    var s0, s1, sqrDist, extDet;
    if (det > 0) {
        // The ray and segment are not parallel.
        s0 = a01 * b1 - b0;
        s1 = a01 * b0 - b1;
        extDet = segExtent * det;
        if (s0 >= 0) {
            if (s1 >= - extDet) {
                if (s1 <= extDet) {
                    // region 0
                    // Minimum at interior points of ray and segment.
                    var invDet = 1 / det;
                    s0 *= invDet;
                    s1 *= invDet;
                    sqrDist = s0 * (s0 + a01 * s1 + 2 * b0) + s1 * (a01 * s0 + s1 + 2 * b1) + c;
                } else {
                    // region 1
                    s1 = segExtent;
                    s0 = Math.max(0, - (a01 * s1 + b0));
                    sqrDist = - s0 * s0 + s1 * (s1 + 2 * b1) + c;
                }
            } else {
                // region 5
                s1 = - segExtent;
                s0 = Math.max(0, - (a01 * s1 + b0));
                sqrDist = - s0 * s0 + s1 * (s1 + 2 * b1) + c;
            }
        } else {
            if (s1 <= - extDet) {
                // region 4
                s0 = Math.max(0, - (- a01 * segExtent + b0));
                s1 = (s0 > 0) ? - segExtent : Math.min(Math.max(- segExtent, - b1), segExtent);
                sqrDist = - s0 * s0 + s1 * (s1 + 2 * b1) + c;
            } else if (s1 <= extDet) {
                // region 3
                s0 = 0;
                s1 = Math.min(Math.max(- segExtent, - b1), segExtent);
                sqrDist = s1 * (s1 + 2 * b1) + c;
            } else {
                // region 2
                s0 = Math.max(0, - (a01 * segExtent + b0));
                s1 = (s0 > 0) ? segExtent : Math.min(Math.max(- segExtent, - b1), segExtent);
                sqrDist = - s0 * s0 + s1 * (s1 + 2 * b1) + c;
            }
        }
    } else {
        return null;
    }
    if (sqrDist < 0.001 && s0 <= w.length() + 0.001) {
        return direction.multiplyScalar(s0).add(w0);
    }
    return null;
};
VIEW3D.utils.transformDir = function(v, m) {
    var x = v.x, y = v.y, z = v.z;
    var e = m.elements;
    v.x = e[0] * x + e[4] * y + e[8] * z;
    v.y = e[1] * x + e[5] * y + e[9] * z;
    v.z = e[2] * x + e[6] * y + e[10] * z;
    return v;
};
VIEW3D.utils.normalizeMatrix = function(mat) {
    var el = mat.elements;
    var dirX = new THREE.Vector3(el[0], el[1], el[2]).normalize();
    var dirY = new THREE.Vector3(el[4], el[5], el[6]).normalize();
    var dirZ = new THREE.Vector3().crossVectors(dirX, dirY);
    dirY.crossVectors(dirZ, dirX);
    el[0] = dirX.x;
    el[1] = dirX.y;
    el[2] = dirX.z;
    el[4] = dirY.x;
    el[5] = dirY.y;
    el[6] = dirY.z;
    el[8] = dirZ.x;
    el[9] = dirZ.y;
    el[10] = dirZ.z;
};
VIEW3D.utils.get2dPosition = function(vector) {
    var position3D = new THREE.Vector3(vector.x, vector.y, vector.z);
    position3D.applyMatrix4(camera.matrixWorldInverse);
    position3D.applyMatrix4(camera.projectionMatrix);
    var x = canvas.clientWidth * (position3D.x + 1) / 2;
    var y = canvas.clientHeight * (-position3D.y + 1) / 2;
    return new THREE.Vector2(x, y);
};