﻿VIEWUI.contextMenu = {
    viewModel: {
        objectSelected: ko.observable(false),
        position3D: ko.observable(null),
        helperAvailable: ko.observable(false),
        btnSelect: function() {
            VIEWUI.toolbar.switchMode(MODE.SELECT);
            VIEWUI.viewModel.popupMode(null);
        },
        btnEdit: function () {
            VIEWUI.toolbar.switchMode(MODE.EDIT);
            if (VIEWUI.viewModel.popupMode() === POPUP_MODE.DEFINITION) {
                VIEWUI.viewModel.popupMode(null);
            }
        },
        btnRemove: function() {
            CORE.model.removeObjects(CORE.model.selectedObjects);
        },
        btnZoomArea: function() {
            CORE.mode(MODE.ZOOM_AREA);
            CORE.view3D.moveControl.updateTextBox(null);
        },
        btnZoomAll: function() {
            CORE.view3D.cameraControl.zoomAll();
            CORE.view3D.moveControl.updateTextBox(null);
        },
        btnDrawBar: function() {
            var node1 = new MODEL.Node(new MODEL.types.Vector(0, 0, 0));
            var node2 = new MODEL.Node(new MODEL.types.Vector(0, 0, 0));
            var newBar = new MODEL.Bar(node1, node2);
            CORE.model.definingObject(newBar);
            VIEWUI.toolbar.switchMode(MODE.DRAW_BAR);
        },
        btnDrawNode: function() {
            var newNode = new MODEL.Node(new MODEL.types.Vector(0, 0, 0));
            CORE.model.definingObject(newNode);
            VIEWUI.toolbar.switchMode(MODE.DRAW_NODE);
        },
        btnDrawBarForce: function() {
            var newLoad = new MODEL.load.bar.BarForce(null, null, null, null);
            CORE.model.definingObject(newLoad);
            VIEWUI.toolbar.switchMode(MODE.EDIT_BAR_FORCE);
        },
        btnDrawBarUniform: function() {
            var newLoad = new MODEL.load.bar.BarUniform(null, null);
            CORE.model.definingObject(newLoad);
            VIEWUI.toolbar.switchMode(MODE.EDIT_BAR_UNIFORM);
        },
        btnDrawBarTrapezoidal: function() {
            var newLoad = new MODEL.load.bar.BarTrapezoidal(null, null, null);
            CORE.model.definingObject(newLoad);
            VIEWUI.toolbar.switchMode(MODE.EDIT_BAR_TRAPEZOIDAL);
        },
        btnAddFill: function() {
            var newFill = new MODEL.Fill();
            CORE.model.definingObject(newFill);
            VIEWUI.toolbar.switchMode(MODE.EDIT_FILL);
        },
        btnHelpGuides:function () {
            var orthoHelper = CORE.view3D.getControls().orthogonalHelper.control;
            if(orthoHelper.enabled && !isEmpty(this.position3D)) {
                orthoHelper.setPosition(this.position3D());
                orthoHelper.show();
            }
        }
    },
    init: function () {
        var oldPos = {
            x: 0,
            y: 0
        };        
        ko.applyBindings(this.viewModel, VIEWUI.divs.contextMenusContainer.element[0]);
        $(this.clickableElements.mainCanvasContainer.id).mousedown(function (event) {
            if (event.button === 2) {
                oldPos.x = event.pageX;
                oldPos.y = event.pageY;
            }
        }).mouseup(function (event) {                   
            if (event.button === 2 && oldPos.x === event.pageX && oldPos.y === event.pageY && event.target.className !== "View3dInputWithFocus") {
                //VIEWUI.contextMenu.setAndShowContextMenu(VIEWUI.contextMenu.contextMenuTemplate.mainTemplate.id, event);
            }
        }).bind("contextmenu", function(event) {
            event.preventDefault();
        });
        // default browser context menu appear after viewer context menu
        $(this.contextMenuTemplate.mainTemplate.id).bind("contextmenu", function (event) {
            event.preventDefault();
        });       
        $(this.clickableElements.viewerContainer.id).bind("click", function() {
            VIEWUI.contextMenu.hideContextMenu(VIEWUI.contextMenu.contextMenuTemplate.mainTemplate.id);
        });
    },
    clickableElements: {
        viewerContainer: { id: "#VIEWUI_viewerContainer" },
        mainCanvasContainer: { id: "#VIEWUI_canvasesContainer" }
    },
    contextMenuTemplate: {
        mainTemplate: { id: "#VIEWUI_contextMenuMain" },
        treeTemplate: { id: "#VIEWUI_contextMenuTree" }
    },
    activeContexMenu: null,
    setAndShowContextMenu: function(contextMenuTemplate, event) {
        var template = $(contextMenuTemplate);
        var containerOffsetLeft = $(this.clickableElements.mainCanvasContainer.id).offset().left;
        var containerOffsetTop = $(this.clickableElements.mainCanvasContainer.id).offset().top;
        var eventOffsetRight = event.target.width - (event.pageX - containerOffsetLeft);
        var eventOffsetBottom = event.target.height - (event.pageY - containerOffsetTop);

        var checkIfCanBeDeleted = function() {
            for (var i = 0; i < CORE.model.selectedObjects().length; i++) {
                var element = CORE.model.selectedObjects()[i];
                if(element.type === MODEL.OBJECT_TYPE.Bar) return false;
                if(element.type === MODEL.OBJECT_TYPE.Node) return false;
            }
            return true;
        }

        if (CORE.model.selectedObjects().length) {
            this.viewModel.objectSelected(checkIfCanBeDeleted());
        } else {
            this.viewModel.objectSelected(false);
        }
        var cursor3d = CORE.view3D.getControls().cursor3D.control;
        this.viewModel.position3D(cursor3d.position ? cursor3d.position.clone() : null);
        this.viewModel.helperAvailable( !isEmpty(CORE.view3D) && CORE.view3D.getControls().orthogonalHelper.control.enabled );

        if (template.width() > eventOffsetRight) { // Dirty if statement keeps contextmenu in container bounds
            if (template.height() > eventOffsetBottom) {
                template.css({
                    "top": "",
                    "right": 0,
                    "bottom": 0,
                    "left": ""
                }).show();
            } else {
                template.css({
                    "top": event.pageY - containerOffsetTop,
                    "right": 0,
                    "bottom": "",
                    "left": ""
                }).show();
            }
        } else {
            if (template.height() > eventOffsetBottom) {
                template.css({
                    "top": "",
                    "right": "",
                    "bottom": 0,
                    "left": event.pageX - containerOffsetLeft
                }).show();
            } else {
                template.css({
                    "top": event.pageY - containerOffsetTop,
                    "right": "",
                    "bottom": "",
                    "left": event.pageX - containerOffsetLeft
                }).show();
            }
        }
        this.activeContexMenu = contextMenuTemplate;
    },
    hideContextMenu: function(contextMenuTemplate) {
        $(contextMenuTemplate).hide();
        this.activeContexMenu = null;
    },
    onKeyUp:function (event) {
        switch(event.keyCode){
            case 27://esc
                if(this.activeContexMenu) {
                    this.hideContextMenu(this.activeContexMenu);
                    return true;
                }
        }
        return false;
    }
}