﻿EXCEPTION_TYPE = { Null:0, Critical: 1, Warning: 2, Info: 3 };

ELEMENT_TYPE = { Null: 6 }
ELEMENT_TYPE = [MODEL.OBJECT_TYPE.Bar, MODEL.OBJECT_TYPE.Node, MODEL.OBJECT_TYPE.Support, ELEMENT_TYPE.Null];
ELEMENT_TYPE.Null = 6;

VIEWUI.exceptionWindow = {
    init: function() {
        ko.applyBindings(VIEWUI.exceptionWindow.viewModel, $("#VIEWUI_exceptionWindow")[0]);
        ko.applyBindings(null, $("#VIEWUI_disclaimerWindow")[0]);
        HELPERS.subscribePropertyChange(VIEWUI.exceptionWindow, CORE.model.constraints.data, "constrainsData", VIEWUI.exceptionWindow.constraintsChanged);
    },

    constraintsChanged: function (constraintsData) {
        var exceptions = constraintsData.exceptionConstraints;

        VIEWUI.exceptionWindow.viewModel.constraintsExceptions([]);

        if (!exceptions) return;

        var array = VIEWUI.exceptionWindow.viewModel.constraintsExceptions;
        VIEWUI.exceptionWindow.loadExceptions(exceptions, array);
    },

    setCalculationExceptions: function (calcExceptions) {

        VIEWUI.exceptionWindow.viewModel.calculationExceptions([]);

        var array = VIEWUI.exceptionWindow.viewModel.calculationExceptions;
        VIEWUI.exceptionWindow.loadExceptions(calcExceptions, array);
    },

    removeAllExceptions: function (){
        VIEWUI.exceptionWindow.viewModel.calculationExceptions([]);
    },

    removeInvalidExceptions: function () {       // remove exception for nonexisting objects

        var exceptions = VIEWUI.exceptionWindow.viewModel.exceptions();

        for (var i = 0; i < exceptions.length; i++) {
            var exception = exceptions[i];
            if (exception.objectType && exception.objectId)
            {
                var modelObject = VIEWUI.exceptionWindow.getExceptionModelObject(exception.objectType, exception.objectId);

                if (!modelObject)
                    VIEWUI.exceptionWindow.removeExceptionsByObjectId(exception.objectId);
            }
        }
    },

    loadExceptions: function(exceptions, array) {

        for (var i = 0; i < exceptions.length; i++) {

            var message;
            var title = translation[exceptions[i].title];

            if (exceptions[i].elementType !== ELEMENT_TYPE.Null) {

                var description = translation[exceptions[i].description];

                message = title + exceptions[i].id + description;
            }
            else {
                message = title;
            }

            if (!title) {
                if (exceptions[i].title)
                    message = "{Cannot find translation for code = " + exceptions[i].title + "}";
                else
                    message = "{Exception title empty}";
            }

            var exception = {
                description: message,
                type: exceptions[i].type,
                objectType: ELEMENT_TYPE[exceptions[i].elementType],
                objectId: exceptions[i].id
            };

            array.push(exception);
        }
    },

    getExceptionModelObject: function(objectType, objectId) {

        var object = null;

        switch (objectType) {
            case MODEL.OBJECT_TYPE.Bar:
                object = CORE.model.getBarById(objectId);
                break;
            case MODEL.OBJECT_TYPE.Node:
            case MODEL.OBJECT_TYPE.Support:             // by design
                object = CORE.model.getNodeById(objectId);
                break;
            default: break; // when objectType === null
        }

        return object;
    },

    removeExceptionsByObjectId: function (objectId) {

        VIEWUI.exceptionWindow.viewModel.calculationExceptions.remove(function (exception) {
            return exception.objectId == objectId;
        });

        VIEWUI.exceptionWindow.viewModel.constraintsExceptions.remove(function (exception) {
            return exception.objectId == objectId;
        });
    },

    viewModel: {
        constraintsExceptions: ko.observableArray(), // [{description, status}, ...]
        calculationExceptions: ko.observableArray(), // [{description, status}, ...]

        exceptions: function () {
            var self = VIEWUI.exceptionWindow.viewModel;
            //return self.constraintsExceptions().concat(self.calculationExceptions());
            return self.calculationExceptions();

            return results;
        },

        filteredExceptions: function () {
            var self = VIEWUI.exceptionWindow.viewModel;

            var filteredExceptions = self.exceptions().filter(function (element) {
                switch (element.type) {
                    case EXCEPTION_TYPE.Critical: return self.showErrors(); break;
                    case EXCEPTION_TYPE.Warning: return self.showWarnings(); break;
                    case EXCEPTION_TYPE.Info: return self.showInfo(); break;
                }
            });

            return filteredExceptions;
        },

        showErrors: ko.observable(true),
        showWarnings: ko.observable(true),
        showInfo: ko.observable(true),

        onExceptionClick: function (objectType, objectId) {
            if (!objectType || !objectId) return;

            CORE.model.deselectAll();

            var modelObject = VIEWUI.exceptionWindow.getExceptionModelObject(objectType, objectId);

            if(modelObject)
                modelObject.selected(true);
        },

        onExceptionMouseDown: function (vm, event) {
            $(event.target).closest('li').addClass('VIEWUI_exceptionHighlighted')

            setTimeout(function () {
                $(event.target).closest('li').removeClass('VIEWUI_exceptionHighlighted');
            }, 500);

            return true;
        },

        onExceptionMouseUp: function (vm, event) {
            $(event.target).closest('li').removeClass('VIEWUI_exceptionHighlighted');
        }
    },
    
}