﻿var VIEWUI = VIEWUI || {};

define([
        'js/viewui/VIEWUI.viewTree', 'js/viewui/VIEWUI.popups', 'js/viewui/VIEWUI.toolbar',
        'js/viewui/VIEWUI.definitionPopup',
        'js/viewui/VIEWUI.statusBar',
        'js/viewui/VIEWUI.contextMenu', 'js/viewui/VIEWUI.measurementEdit', 'js/viewui/VIEWUI.moveEdit',
        'js/viewui/VIEWUI.properties', 'js/viewui/VIEWUI.barEdit', 'js/viewui/VIEWUI.exceptionWindow'
    ],
    function() {
        POPUP_MODE = { DEFINITION: "Definition", PROPERTIES: "Properties" };
        VIEWUI.viewerContainerOuter = null;
        VIEWUI.divs = {
            viewerContainer: { element: null, id: "VIEWUI_viewerContainer" },
            propertiesContainer: { element: null, id: "VIEWUI_propertiesContainer" },
            popupsContainer: { element: null, id: "VIEWUI_popupsContainer" },
            viewTreeContainer: { element: null, id: "VIEWUI_viewTreeContainer" },
            toolbarContainer: { element: null, id: "VIEWUI_toolbar" },
            viewOptionsContainer: { element: null, id: "VIEWUI_viewOptions" },
            statusBarContainer: { element: null, id: "VIEWUI_statusBar" },
            contextMenusContainer: { element: null, id: "VIEWUI_contextMenus" },
            editMeasurementContainer: { element: null, id: "VIEWUI_measuremetEdit" },
            editMoveContainer: { element: null, id: "VIEWUI_moveEdit" },
            pdfGeneratorContainer: { element: null, id: "pdfGenerator" },
            editBarContainer: { element: null, id: "VIEWUI_barEdit" },
            editBarAngles1Container: { element: null, id: "VIEWUI_barEditAngles1" },
            editBarAngles2Container: { element: null, id: "VIEWUI_barEditAngles2" },
            modeMessageContainer: { element: null, id: "VIEWUI_modeMessageContainer" }
        };
        VIEWUI.templates = (function() {
            var templates = {};
            templates[MODEL.OBJECT_TYPE.Workplane] = "workplane-template";
            templates[MODEL.OBJECT_TYPE.Node] = "node-template";
            templates[MODEL.OBJECT_TYPE.Bar] = "bar-template";
            templates[MODEL.OBJECT_TYPE.NodeForce] = "nodeForce-template";
            templates[MODEL.OBJECT_TYPE.NodeMoment] = "nodeMoment-template";
            templates[MODEL.OBJECT_TYPE.NodeSupport] = "nodeSupport-template";
            templates[MODEL.OBJECT_TYPE.BarForce] = "barForce-template";
            templates[MODEL.OBJECT_TYPE.BarUniform] = "barUniform-template";
            templates[MODEL.OBJECT_TYPE.BarTrapezoidal] = "barTrapezoidal-template";
            templates[MODEL.OBJECT_TYPE.Fill] = "fill-template";
            templates[MODEL.OBJECT_TYPE.BoxDefinition] = "box-template";
            templates[MODEL.OBJECT_TYPE.Box] = "box-template";
            templates['SnapingLinesGrid'] = "snap-grid-template";
            return templates;
        })();
        VIEWUI.viewModel = function() {
            this.popupExpanded = ko.observable(); // default values set below
            //this.browserExpanded = ko.observable(true);
            this.popupMode = ko.observable(); //"properties/definition"
            this.popupCreateVisible = ko.observable(); // for workplane
            this.togglePopupExpanded = function() {
                if (!this.dragging()) {
                    this.popupExpanded(!this.popupExpanded());
                }
                if (!this.popupExpanded()) {
                    $("#VIEWUI_propertiesContainer").css("height", "");
                }
                VIEWUI.properties.checkContainerSize();
            };
            this.title = ko.computed(function() {
                    if(CORE.mode() === MODE.EDIT || CORE.mode() === MODE.MOVE_CONSTRUCTION) {
                        return translation.SNAP_GRID_PROPERTIES;
                    }

                    if (this.popupMode() === POPUP_MODE.PROPERTIES) {
                        if(typeof CORE.model.selectedObjects()[0] === "undefined") return translation.PROPERTIES;
                        if (CORE.model.selectedObjects()[0].type == "Bar") {
                            return translation.MEMBER_PROPERTIES;
                        } else if (CORE.model.selectedObjects()[0].type == "Node") {
                            return translation.NODE_PROPERTIES;
                        } else if (CORE.model.selectedObjects()[0].type == "BarForce") {
                            return translation.LOAD_PROPERTIES;
                        } else if (CORE.model.selectedObjects()[0].type == "BarTrapezoidal") {
                            return translation.DISTRIBUTED_PROPERTIES;
                        }
                        return translation.PROPERTIES;
                    };
                    if (this.popupMode() === POPUP_MODE.DEFINITION) return translation.DEFINITION;
                    else return "no-name";
                },
                this);
            this.dragging = ko.observable(false);
            this.calculationsFinished = function() {
                return !isEmpty(CORE
                        .model) &&
                    !isEmpty(CORE.model.barsResults()) &&
                    !isEmpty(CORE.model.nodesResults());
            };
            this.calculationError = ko.observable(false);
            this.maxGlobalUtilization = ko.observable(0);
            var self = this;
            this.modeMessageMapping = [];
            this.modeMessageMapping[MODE.EDIT] = translation.MODE_MESSAGE_EDIT;
            this.modeMessageMapping[MODE.CUT] = translation.MODE_MESSAGE_TRIM;
            this.modeMessageMapping[MODE.SNAP] = translation.MODE_MESSAGE_EXTEND;
            this.modeMessageMapping[MODE.MOVE_CONSTRUCTION] = translation.MODE_MESSAGE_MOVE_CONSTRUCTION;
            var definitionModes = [
                MODE.DRAW_NODE, MODE.DRAW_BAR, MODE.DRAW_WORKPLANE, MODE.EDIT_BAR_FORCE, MODE.EDIT_BAR_TRAPEZOIDAL,
                MODE.EDIT_BAR_UNIFORM,
                MODE.EDIT_FILL, MODE.EDIT_NODE_FORCE, MODE.EDIT_NODE_MOMENT,
                MODE.EDIT_SUPPORT, MODE.EDIT_BOX
            ];
            definitionModes
                .forEach(function(mode) { self.modeMessageMapping[mode] = translation.MODE_MESSAGE_DEFINE; });
            this.modeMessageVisible = ko.computed(function() {
                    return !isEmpty(this.modeMessageMapping[CORE.mode()]);
                },
                this);
            this.modeMessageText = ko.computed(function() {
                    return this.modeMessageMapping[CORE.mode()];
                },
                this);
            this.getMaxGlobalUtilizationText = function() {
                return this.maxGlobalUtilization().toFixed(VIEWUI.viewOptions.unitUtilizationPrecision()) +
                    VIEWUI.viewOptions.unitUtilization();
            }
        };
        VIEWUI.init = function(viewerContainerOuter, callback) {
            //if (typeof VIEWUI.viewModel === 'object') { callback.call(); return };  // VIEWUI already initialized
            VIEWUI.initKnockoutOptions();

            this.viewerContainerOuter = viewerContainerOuter;
            var model = CORE.model;
            /// begin
            //if (typeof VIEWUI.viewModel !== 'function') {
            //    VIEWUI.viewModel = function () {
            //        this.popupExpanded = ko.observable(true);
            //        //this.browserExpanded = ko.observable(true);
            //        this.popupMode = ko.observable(null); //"properties/definition"
            //        this.popupCreateVisible = ko.observable(false); // for workplane
            //        this.togglePopupExpanded = function () {
            //            if (!this.dragging())
            //                this.popupExpanded(!this.popupExpanded());
            //        };
            //        //this.toggleBrowserExpanded = function () {
            //        //    if (!this.dragging())
            //        //        this.browserExpanded(!this.browserExpanded());
            //        //};
            //        this.title = ko.computed(function () {
            //            if (this.popupMode() === POPUP_MODE.PROPERTIES) return "Properties";
            //            if (this.popupMode() === POPUP_MODE.DEFINITION) return "Definition";
            //            else return "no-name";
            //        },
            //            this);
            //        this.dragging = ko.observable(false);
            //        this.calculationsFinished = function () {
            //            return !isEmpty(CORE.model) &&
            //                !isEmpty(CORE.model.barsResults()) &&
            //                !isEmpty(CORE.model.nodesResults());
            //        };
            //    };
            //}
            /// end

            if (typeof VIEWUI.viewModel === 'function') // initialize if not initialized yet
                this.viewModel = new VIEWUI.viewModel();

            this.viewModel.popupExpanded(true); // reset default values (for CORE.options.viewerReloading functionality)
            this.viewModel.popupMode(null);
            this.viewModel.popupCreateVisible(false);

            $(this.viewerContainerOuter)
                .load(CORE.rootPath + "html/VIEWUI.html",
                    function(response, status, xhr) {
                        if (status === 'error') {
                            throw 'VIEWUI: failed to load resource html/VIEWUI.html, reason: ' + xhr.statusText;
                        }
                        VIEWUI.initDivs();

                        try {
                            VIEWUI.bindViewModel();
                            VIEWUI.bindOptionsModel();
                            VIEWUI.statusBar.init();
                            VIEWUI.exceptionWindow.init();
                            if (CORE.options.showObjectList) {
                                VIEWUI.viewTree.init(VIEWUI.viewerContainerOuter);
                            }
                        } catch (exception) {
                            // prevent throwing uncaught exception in CRM 
                            // (e.g. when opening calculation edit page before viewer has fully loaded)
                            console.error(exception); // because crm
                        }

                        $('#VIEWUI_propertiesTemplates')
                            .load(CORE.rootPath + "html/VIEWUI.propertiesTemplates.html",
                                function(response, status) {
                                    if (status === 'error') {
                                        console.error('VIEWUI: failed to load resource html/VIEWUI.propertiesTempates.html');
                                    }
                                    VIEWUI.properties.init(model.selectedObjects, model.activeWorkplane);
                                    $('#VIEWUI_popupsTemplates')
                                        .load(CORE.rootPath + "html/VIEWUI.popupsTemplates.html",
                                            function(response, status) {
                                                if (status === 'error') {
                                                    console.error('VIEWUI: failed to load resource html/VIEWUI.popups.html');
                                                }

                                                try {
                                                    VIEWUI.popups.init();
                                                    callback.call();
                                                    VIEWUI.toolbar.init();
                                                    VIEWUI.definitionPopup.init();
                                                    VIEWUI.contextMenu.init();
                                                    VIEWUI.measurementEdit.init();
                                                    VIEWUI.moveEdit.init();
                                                    // VIEWUI.resolveBootstrapConflicts();
                                                    VIEWUI.initCustomRadio($('.VIEWUI_customRadio'));
                                                    VIEWUI.barEdit.init();
                                                } catch (exception) {
                                                    console.error(exception); // because crm
                                                }

                                                setTimeout(function() {
                                                    VIEWUI.initAllScrollbars();
                                                }, 2000);

                                                $(window).resize(function() {
                                                    VIEWUI.initAllScrollbars();
                                                });

                                                CORE.model.modelChange.subscribe(function() {
                                                    VIEWUI.initCustomRadio($('.VIEWUI_customRadio'));
                                                });
                                            });
                                });
                    });
        };
        VIEWUI.initDivs = function() {
            var divs = VIEWUI.divs;
            for (var property in divs) {
                if (divs.hasOwnProperty(property)) {
                    divs[property].element = $('#' + divs[property].id);
                    if (divs[property].element.length === 0) {
                        console.error("VIEWUI.initDivs: div " + divs[property].id + " not found in DOM");
                    }
                }
            }
        };
        VIEWUI.initDraggable = function() {
            var containment = $('#VIEWUI_draggableContainer');
            var draggableDivs = [
                { element: this.divs.propertiesContainer.element, handle: $('#VIEWUI_propertiesTopBar') },
                { element: this.divs.viewTreeContainer.element, handle: $('#VIEWUI_viewTreeTopBar') },
                { element: this.divs.toolbarContainer.element, handle: $('#VIEWUI_toolbarHandleContainer') },
                { element: this.divs.popupsContainer.element, handle: $('#VIEWUI_popupTopBar') }
            ];
            for (var i = 0, ii = draggableDivs.length; i < ii; i++) {
                draggableDivs[i].element.draggable({
                    handle: draggableDivs[i].handle,
                    containment: containment,
                    snap: true,
                    snapTolerance: 5
                });
            }
        };
        VIEWUI.spinners = {
            init: function(container) {
                if (isEmpty(container)) { console.error("You need to specify spinners container!"); return; }

                val = 1 / Math.pow(10, VIEWUI.viewOptions.unitDistancePrecision());
                if (val === 0) {
                    $(container).find("input[name='dynamicStepChangeDistance']").attr("step", 1);
                } else {
                    $(container).find("input[name='dynamicStepChangeDistance']").attr("step", val);
                }
                val = 1 / Math.pow(10, VIEWUI.viewOptions.unitDisplacementPrecision());
                if (val === 0) {
                    $(container).find("input[name='dynamicStepChangeDisplacement']").attr("step", 1);
                } else {
                    $(container).find("input[name='dynamicStepChangeDisplacement']").attr("step", val);
                }
                val = 1 / Math.pow(10, VIEWUI.viewOptions.unitForcePrecision());
                if (val === 0) {
                    $(container).find("input[name='dynamicStepChangeForce']").attr("step", 1);
                } else {
                    $(container).find("input[name='dynamicStepChangeForce']").attr("step", val);
                }
                val = 1 / Math.pow(10, VIEWUI.viewOptions.unitAnglePrecision());
                if (val === 0) {
                    $(container).find("input[name='dynamicStepChangeAngle']").attr("step", 1);
                } else {
                    $(container).find("input[name='dynamicStepChangeAngle']").attr("step", val);
                }
                //val = 1 / Math.pow(10, VIEWUI.viewOptions.unitStressPrecision());
                //if (val === 0) {
                //    $(container).find("input[name='dynamicStepChangeStress']").attr("step", 1);
                //} else {
                //    $(container).find("input[name='dynamicStepChangeStress']").attr("step", val);
                //}
                val = 1 / Math.pow(10, VIEWUI.viewOptions.unitMomentPrecision());
                if (val === 0) {
                    $(container).find("input[name='dynamicStepChangeMoment']").attr("step", 1);
                } else {
                    $(container).find("input[name='dynamicStepChangeMoment']").attr("step", val);
                }
                val = 1 / Math.pow(10, VIEWUI.viewOptions.unitPressurePrecision());
                if (val === 0) {
                    $(container).find("input[name='dynamicStepChangePressure']").attr("step", 1);
                } else {
                    $(container).find("input[name='dynamicStepChangePressure']").attr("step", val);
                }
                var elementType = 'input[type=number]:not([readonly],[disabled])';
                var numericInputs = $(container).find(elementType);
                if (numericInputs.length > 0) {
                    $(container + ' ' + elementType)
                        .spinner({
                            create: function(e) {
                                var target = $(e.target);

                                var step = target.attr("step"); // CORE.units is setting min/max attributes
                                //var min = target.attr("min");
                                //var max = target.attr("max");

                                //var unitType = target.attr("unitType");
                                //debugger;
                                //if (unitType) {
                                //    var multi = CORE.units.getMultiByType(unitType);

                                //    if (min)
                                //        target.attr("min", min * multi);

                                //    if (max)
                                //        target.attr("max", max * multi);
                                //}

                                // display decimal places after initialization
                                if (!isEmpty(step)) {
                                    var decimalPlaces = $(e.target).attr("step").split('.')[1];
                                    if (!isEmpty(decimalPlaces)) {
                                        $(e
                                                .target)
                                            .val(parseFloat($(e.target).val()).toFixed(decimalPlaces.length));
                                    }
                                }
                            },
                            start: function(event) {
                                if (!$(event.target).is(":focus") && event.originalEvent.type === "mousewheel")
                                    event.preventDefault();
                            },
                            stop: function(e) {
                                if (e.keyCode === 38 || e.keyCode === 40 || e.which === 1) {
                                    $(this).trigger("change");
                                }
                            }
                        });
                }

            },
            destroy: function(container) {
                var elementType = "input[type=number]:not([readonly])";
                var numericInputs = $(container).find(elementType);
                if (numericInputs.length > 0) {
                    $(container + " " + elementType).spinner("destroy");
                }
            },
            refresh: function(container, timeout) {
                setTimeout(function() {
                        VIEWUI.spinners.destroy(container);
                        VIEWUI.spinners.init(container);
                    },
                    timeout);
            }
        };
        VIEWUI.initCustomRadio = function(jQueryElement) {
                for (var i = 0; i < jQueryElement.length; i++) {
                    var element = jQueryElement.eq(i);
                    if (element.data('buttonset')) {
                        element.buttonset('refresh');
                    } else {
                        element.buttonset();
                    }
                }
            }
            // VIEWUI.resolveBootstrapConflicts = function () {
            //     //resolving bootstrap vs jQueryUI - button() conflict
            //     if (typeof $.fn.button.noConflict === 'function') {
            //         $.fn.bootstrapBtn = $.fn.button.noConflict();
            //     }
            // };
        VIEWUI.initScrollbar = function(jQueryElement) {

            /*if (jQueryElement) {
                if (!jQueryElement.data("mCS")) { //check if scrollbars initialized
                    jQueryElement.mCustomScrollbar({
                        scrollInertia: 1,
                        mouseWheel: { scrollAmount: 100, preventDefault: true },
                        callbacks: {
                            onScroll: function () {
                                //$(this).trigger("scroll");    // restore default event firing
                            },
                            onInit: function () {
                                jQueryElement
                                    .data("mCS")
                                    .opt.mouseWheel.disableOverClasses
                                    .push("VIEWUI_comboboxOptions"); // do not scroll if mouse over combobox list
                            }
                        }
                    });
                } else {
                    jQueryElement.mCustomScrollbar("update");
                }
            }*/

            $('#VIEWUI_container, .VIEWUI_comboboxOptions') // cannot be #VIEWUI_container because .VIEWUI_comboboxOptions are being appended to body
                .bind('mousewheel DOMMouseScroll',
                    function(e, d) { // prevent scrolling parent page (delete after all scrolls will be made using custom js library)
                        var element = $(e.target);
                        var elementScrollableParent = $(e.target)
                            .closest(".VIEWUI_comboboxOptions, .VIEWUI_versionListContainer, .VIEWUI_searchResultsContainer, .VIEWUI_exceptionWindow ul"); //TODO: refactor, need to detect scrollable elements automatically
                        if (elementScrollableParent.length !== 0) {
                            element = elementScrollableParent;
                        }
                        var height = element.height();
                        var scrollHeight = element[0].scrollHeight;
                        var scrollTop = element[0].scrollTop;
                        if (((scrollTop === (scrollHeight - height) && d < 0) || (scrollTop === 0 && d > 0)) &&
                            e.target.tagName !== "CANVAS") {
                            e.preventDefault();
                        }
                    });
        }
        VIEWUI.initAllScrollbars = function() {
            var divsWithScroll = [
                "#VIEWUI_properties .VIEWUI_allFieldsContainer",
                "#VIEWUI_treeScrollList"
            ];
            for (var i = 0; i < divsWithScroll.length; i++) {
                var element = $(divsWithScroll[i]);
                VIEWUI.initScrollbar(element);
            }
        };
        VIEWUI.initCheckboxes = function(element) { // convert checkboxes into bootstrap switches (e.g. normal-fast connector)
            function getCheckedDataBindProperty(input) {
                var dataBindAttr = $(input).data().bind;
                var dataBindArr = dataBindAttr.split(',');
                for (var i = 0; i < dataBindArr.length; i++) {
                    if (dataBindArr[i].indexOf("checked") !== -1) {
                        var temp = dataBindArr[i].split(":");
                        return temp[1].trim();
                    }
                };
            };

            var leftArrow = '<i class="glyphicon glyphicon-arrow-left"></i>';
            var rightArrow = '<i class="glyphicon glyphicon-arrow-right"></i>';
            var checkboxes = $(element).find("input[type=checkbox]");
            checkboxes.each(function(index, item) {
                var checkedProperty = getCheckedDataBindProperty(this);
                var vm = ko.dataFor(this);
                var isChecked = vm[checkedProperty]();
                var labelText = isChecked ? rightArrow : leftArrow;
                var switchWidth = $(item).data('switch-width');
                var onText = $(item).data('switch-on-text');
                $(item)
                    .bootstrapSwitch({
                        size: 'mini',
                        labelText: labelText,
                        onText: onText ? onText : translation.CHECKBOX_ON,
                        offText: translation.CHECKBOX_OFF,
                        labelWidth: 35,
                        handleWidth: switchWidth ? switchWidth : 55,
                        disabled: item.disabled
                    });
                $(item).bootstrapSwitch('disabled', item.disabled);
            });
            checkboxes.on('switchChange.bootstrapSwitch',
                function(event, state) { // bootstrap switch doesn't change observable state, this is a workaround
                    var checkedProperty = getCheckedDataBindProperty(this);
                    var vm = ko.dataFor(this);
                    vm[checkedProperty](state);
                });
            checkboxes
                .each(function(index, item) {
                    // bootstrap switch doesn't react to observable state, have to observe manually
                    var checkedProperty = getCheckedDataBindProperty(item);
                    var vm = ko.dataFor(this);

                    vm[checkedProperty].subscribe(function(newValue) {
                        $(item).bootstrapSwitch('state', newValue);

                        $(item).bootstrapSwitch('labelText', newValue ? rightArrow : leftArrow);
                    });
                });
        };
        VIEWUI.bindViewModel = function() {
            ko.bindingHandlers.stopBinding = {
                init: function(element, valueAccessor) {
                    return {
                        controlsDescendantBindings: valueAccessor()
                    };
                }
            };
            ko.virtualElements.allowedBindings.stopBinding = true;
            ko.applyBindings(VIEWUI.viewModel, VIEWUI.divs.propertiesContainer.element[0]);
            ko.applyBindings(VIEWUI.viewModel, VIEWUI.divs.popupsContainer.element[0]);
            ko.applyBindings(VIEWUI.viewModel, VIEWUI.divs.viewTreeContainer.element[0]); //moved to VIEWUI.viewTree
            if (!isEmpty(VIEWUI.divs.pdfGeneratorContainer.element[0])) {
                ko.applyBindings(null, VIEWUI.divs.pdfGeneratorContainer.element[0]);
            }
            ko.applyBindings(VIEWUI.viewModel, VIEWUI.divs.modeMessageContainer.element[0]);
            ko.cleanNode($('#VIEWUI_popup')[0]);
        };
        VIEWUI.getImageUrl = function(imgUrl) {
            return CORE.rootPath + imgUrl;
        };
        VIEWUI.blockUI = function() {
            $("#VIEWUI_block").addClass("blocked");
        };
        VIEWUI.unblockUI = function() {
            $("#VIEWUI_block").removeClass("blocked");
        };
        VIEWUI.viewOptions = {
            showRealElements: ko.observable(true),
            supportsVisible: ko.observable(true),
            loadsVisible: ko.observable(true),
            measurementsVisible: ko.observable(true),
            pushButtonsVisible: ko.observable(true),
            boxesVisible: ko.observable(true),
            showCalcDisplacement: ko.observable(true),
            visibleCalcForce: ko.observable("none"), // fx, fy...
            showCalcForceFx: ko.observable(false),
            showCalcForceFy: ko.observable(false),
            showCalcForceFz: ko.observable(false),
            showCalcForceMx: ko.observable(false),
            showCalcForceMy: ko.observable(false),
            showCalcForceMz: ko.observable(false),
            showCalcForceUx: ko.observable(false),
            showCalcForceUy: ko.observable(false),
            showCalcForceUz: ko.observable(false),
            unitPreset: ko.observable("SI"),
            unitDistance: ko.observable("m"),
            unitDistanceResults: ko.observable("m"),
            unitDistancePrecision: ko.observable(2),
            unitDistancePrecisionResults: ko.observable(2),
            unitForce: ko.observable("N"),
            unitForceResults: ko.observable("N"),
            unitForcePrecision: ko.observable(2),
            unitForcePrecisionResults: ko.observable(2),
            unitStress: ko.observable("Pa"),
            unitStressPrecision: ko.observable(2),
            unitMomentForce: ko.observable("N"),
            unitMomentDistance: ko.observable("m"),
            unitMomentPrecision: ko.observable(2),
            unitMomentForceResults: ko.observable("N"),
            unitMomentDistanceResults: ko.observable("m"),
            unitMomentPrecisionResults: ko.observable(2),
            unitPressureForce: ko.observable("N"),
            unitPressureSurface: ko.observable("m2"),
            unitPressurePrecision: ko.observable(2),
            unitAngle: ko.observable("°"),
            unitAnglePrecision: ko.observable(0),
            unitUtilization: ko.observable("%"),
            unitUtilizationPrecision: ko.observable(2),
            unitDisplacement: ko.observable("mm"),
            unitDisplacementPrecision: ko.observable(1)
        };
        VIEWUI.bindOptionsModel = function() {
            ko.applyBindings(VIEWUI.viewOptions, VIEWUI.divs.viewOptionsContainer.element[0]);

            function hideCalcForces() {
                VIEWUI.viewOptions.showCalcForceFx(false);
                VIEWUI.viewOptions.showCalcForceFy(false);
                VIEWUI.viewOptions.showCalcForceFz(false);
                VIEWUI.viewOptions.showCalcForceMx(false);
                VIEWUI.viewOptions.showCalcForceMy(false);
                VIEWUI.viewOptions.showCalcForceMz(false);
            }

            this.oldView = VIEWUI.viewOptions.measurementsVisible();
            HELPERS.subscribePropertyChange(this,
                VIEWUI.viewOptions.showCalcDisplacement,
                "VIEWUI.viewOptions.showCalcDisplacement",
                function(showCalcDisplacement) {
                    if (!isEmpty(showCalcDisplacement)) {
                        if (VIEWUI.viewOptions.showCalcDisplacement() === true) {
                            hideCalcForces();
                            VIEWUI.viewOptions.visibleCalcForce("diff");
                        }
                    }
                });
            HELPERS.subscribePropertyChange(this,
                VIEWUI.viewOptions.visibleCalcForce,
                "VIEWUI.viewOptions.visibleCalcForce",
                function(visibleCalcForce) {
                    if (!isEmpty(visibleCalcForce)) {
                        switch (visibleCalcForce) {
                            case "none":
                                hideCalcForces();
                                if (VIEWUI.viewOptions.showCalcDisplacement() === true) {
                                    VIEWUI.viewOptions.showCalcDisplacement(false);
                                }
                                break;
                            case "diff":
                                break;
                            default:
                                if (VIEWUI.viewOptions.showCalcDisplacement() === true) {
                                    VIEWUI.viewOptions.showCalcDisplacement(false);
                                }
                                //hideCalcForces();
                                VIEWUI.viewOptions["showCalcForce" + visibleCalcForce](true);
                        }
                    }
                });
            HELPERS.subscribePropertyChange(this,
                VIEWUI.viewOptions.unitDistancePrecision,
                "VIEWUI.viewOptions.unitDistancePrecision",
                function() {
                    VIEWUI.spinners.refresh("#VIEWUI_propertiesContainer", 1);
                });
            HELPERS.subscribePropertyChange(this,
                VIEWUI.viewOptions.unitForcePrecisionResults,
                "VIEWUI.viewOptions.unitForcePrecisionResults",
                function() {
                    VIEWUI.spinners.refresh("#VIEWUI_propertiesContainer", 1);
                });
            HELPERS.subscribePropertyChange(this,
                VIEWUI.viewOptions.unitMomentPrecision,
                "VIEWUI.viewOptions.unitMomentPrecision",
                function() {
                    VIEWUI.spinners.refresh("#VIEWUI_propertiesContainer", 1);
                });
            HELPERS.subscribePropertyChange(this,
                VIEWUI.viewOptions.unitPressurePrecision,
                "VIEWUI.viewOptions.unitPressurePrecision",
                function() {
                    VIEWUI.spinners.refresh("#VIEWUI_propertiesContainer", 1);
                });
            HELPERS.subscribePropertyChange(this,
                VIEWUI.viewOptions.unitAnglePrecision,
                "VIEWUI.viewOptions.unitAnglePrecision",
                function() {
                    VIEWUI.spinners.refresh("#VIEWUI_propertiesContainer", 1);
                });
            HELPERS.subscribePropertyChange(this,
                VIEWUI.viewOptions.unitUtilizationPrecision,
                "VIEWUI.viewOptions.unitUtilizationPrecision",
                function() {
                    VIEWUI.spinners.refresh("#VIEWUI_propertiesContainer", 1);
                });
            HELPERS.subscribePropertyChange(this,
                VIEWUI.viewOptions.unitDisplacementPrecision,
                "VIEWUI.viewOptions.unitDisplacementPrecision",
                function() {
                    VIEWUI.spinners.refresh("#VIEWUI_propertiesContainer", 1);
                });
            HELPERS.subscribePropertyChange(this,
                CORE.mode,
                "CORE.mode",
                function(mode) {
                    if (mode === MODE.SELECT) {
                        //VIEWUI.viewOptions.measurementsVisible(this.oldView);
                        $('#viewOptions_measurements').attr('disabled', false);
                        $('.VIEWUI_customRadio').buttonset('refresh');
                    } else {
                        //this.oldView = VIEWUI.viewOptions.measurementsVisible();
                        //VIEWUI.viewOptions.measurementsVisible(false);
                        $('#viewOptions_measurements').attr('disabled', true);
                        $('.VIEWUI_customRadio').buttonset('refresh');
                    }
                });
        };
        VIEWUI.lastValue = 1;
        VIEWUI.validateNumber = function(object) {
            var value = Number(object.value);
            if (value < 1 || value > 100) {
                object.value = VIEWUI.lastValue.toString();
            } else {
                object.value = value.toFixed(0);
                VIEWUI.lastValue = value;
            }
        }

        VIEWUI.initDemoModeForElement = function(el, enableOffset) {

            if (CORE.options.demoMode) {

                setTimeout(function() {

                    var $el = $(el);

                    $el.find('.VIEWUI_demoModeBlockProperties').remove();

                    var demoModeBlock = $('<div class="VIEWUI_demoModeBlock VIEWUI_demoModeBlockProperties"></div>');

                    //if (offsetMode) {
                    //    demoModeBlock.offset($el.offset());
                    //    demoModeBlock.width($el.width());
                    //    demoModeBlock.height($el.height());
                    //}

                    demoModeBlock.css('top', 0);
                    demoModeBlock.css('left', 0);
                    demoModeBlock.css('bottom', 0);
                    demoModeBlock.css('right', enableOffset ? '10px' : 0); // properties - scrollbar

                    demoModeBlock.click(function() {
                        alert(translation.FUNCTION_NOT_AVAILABLE_IN_DEMO);
                        return false;
                    });

                    //demoModeBlock.scroll(function () {
                    //    //scroll scrollbar and element manually
                    //    $('.mCSB_container').eq(0).css('top', 0)
                    //    $('.mCSB_dragger').eq(0).css('top', 0)
                    //});

                    $el.append(demoModeBlock);

                }, 100);

            }
        }

        VIEWUI.initKnockoutOptions = function() {
            VIEWUI.initClickIfNotDemoBinding();
        }

        VIEWUI.initClickIfNotDemoBinding = function() {

            var originalInit = ko.bindingHandlers.click.init,
                originalUpdate = ko.bindingHandlers.click.update;

            ko.bindingHandlers.clickIfNotDemo = {
                init: function(element, valueAccessor, allBindingsAccessor, viewModel, context) {
                    var wrappedValueAccessor = function() {
                        return function(data, event) {
                            //run some pre code
                            var executeAction = ko.bindingHandlers.clickIfNotDemo.preOnClick.call(viewModel, data, event);

                            if (executeAction === false) return;

                            valueAccessor().call(viewModel, data, event);

                            //run some post code
                            ko.bindingHandlers.clickIfNotDemo.postOnClick.call(viewModel, data, event);
                        };

                    };

                    originalInit(element, wrappedValueAccessor, allBindingsAccessor, viewModel, context);
                },
                update: originalUpdate,
                preOnClick: function(data, event) {
                    if (CORE.options.demoMode) {
                        alert(translation.FUNCTION_NOT_AVAILABLE_IN_DEMO);
                        return false;
                    }
                },
                postOnClick: function(data, event) {}
            };

        }

        VIEWUI.adjustPropertiesHeight = function() {
             // setting viewer tables position after deleting one of loads
             if (document.getElementById("Loads_container") !== null && document.getElementById("VIEWUI_propertiesContainer") !== null) {
                var heightOfLoadsTable = document.getElementById("Loads_container").offsetHeight;
                var minimumStyleTopValue = 6;
                CORE.heightOfLoadsTableViewer = heightOfLoadsTable + minimumStyleTopValue;

                if (CORE.showLoads()) {
                    const minimumStyleTopValue = 10;
                    document.getElementById("VIEWUI_propertiesContainer").style.top = minimumStyleTopValue + CORE.heightOfLoadsTableViewer + "px";
                    let variable = 160 + CORE.heightOfLoadsTableViewer;
                    let maxHeight = 'calc(100% - ' + variable + "px)";
                    document.getElementById("VIEWUI_propertiesContainer").style.maxHeight = maxHeight;
                }
            }
        }

    });