﻿define(['js/view3d/VIEW3D.BarDxf'], function () {

    VIEWUI.popups = {
        divs: {
            popupsContainer: { element: null, id: "VIEWUI_popupsContainer" },
            popup: { element: null, id: "VIEWUI_popup" },
            popupContents: { element: null, id: "VIEWUI_popup_contents" },
            //popupProgress: { element: null, id: "VIEWUI_popup_progress"},
            popupSettings: { element: null, id: "VIEWUI_popup_settings" }
            //addWorkplane: { element: null, id: "VIEWUI_properties_workplane" },
        },
        templates: {
            importFromDxf: "importFromDxf-template",
            calcProgress: "calculationProgress-template",
            settings: "VIEWUI_popup_settings",
            versionList: "versionList-template",
            fixModel: "fixModel-template",
            reportSettings: "reportSettings-template",
        },
        init: function () {
            var divs = VIEWUI.popups.divs;
            for (var property in divs) {
                if (divs.hasOwnProperty(property)) {
                    divs[property].element = $('#' + divs[property].id);
                    if (divs[property].element.length === 0) {
                        console.error("VIEWUI.popups.init: div " + divs[property].id + " not found in DOM");
                    }
                }
            }
            if (this.propertiesContainer === null || this.popups === null) {
                throw 'VIEWUI.popups.init: Element does not exist!';
            }

            $(window).resize(function () {
                VIEWUI.popups.autoCenterPopup();
            })

        },
        popupViewModelBase: function () { // base class for all popup viewModels
            this.label = "default";
            this.name = "default";
            this.onOkClicked = function () { };
            this.onCloseClicked = function () { VIEWUI.popups.hide(); };
            this.editingName = ko.observable(false);
            this.template = "default";
            this.type = 'create';
            this.viewModel = 'default';
            this.viewModelInstance = 'default';
            this.onChanged = "default";
            var self = this;
            this.onKeyDown = function (event) {
                if (event.keyCode === KEYCODES.ENTER) {
                    self.onOkClicked(self);
                    document.removeEventListener("keydown", self.onKeyDown);
                }
                if (event.keyCode === KEYCODES.ESC) {
                    self.onCloseClicked();
                    document.removeEventListener("keydown", self.onKeyDown);
                }
            }
            document.addEventListener("keydown", self.onKeyDown);
        },
        addWorkplane: {
            ViewModel: function () {
                VIEWUI.popups.popupViewModelBase.call(this);
                this.label = "Create new workplane";
                this.name = ko.observable(translation.WORKPLANE + ' ' + (workplanesCount + 1));
                this.position = ko.observable(new MODEL.types.ThreeVectors(
                    new MODEL.types.Vector(0, 0, 0),
                    new MODEL.types.Vector(0, 0, 0),
                    new MODEL.types.Vector(0, 0, 0)).toViewModel());
                this.onOkClicked = VIEWUI.popups.addWorkplane.onOkClicked;
                this.template = VIEWUI.templates[MODEL.OBJECT_TYPE.Workplane];
            },
            show: function () {
                var viewModel = new this.ViewModel();
                VIEWUI.popups.setAndShowPopup(viewModel, VIEWUI.popups.divs.popupContents.element);
            },
            onOkClicked: function (viewModel) {
                var newWorkplane = new MODEL.Workplane(viewModel.position().toModelObject());
                newWorkplane.name = viewModel.name;
                CORE.model.addWorkplane(newWorkplane);
                VIEWUI.popups.hide();
            }
        },
        importFromDxf: {
            ViewModel: function () {
                VIEWUI.popups.popupViewModelBase.call(this);
                this.label = "Import from DXF file";
                this.layers = ko.observableArray();
                this.selectedLayers = ko.observableArray();
                this.onOkClicked = function (viewModel) {
                    if (this.selectedLayers().length === 0) {
                        alert("Please select at least one layer to import");
                        return;
                    }
                    var fileReader = new FileReader();
                    fileReader.viewModel = viewModel;
                    fileReader.onload = function (event) {
                        var content = event.target.result.split("base64,")[1];
                        var dxt = atob(content);

                        CORE.webApiManager.importDxfToDb(dxt, CORE.options.appId, CORE.options.projId, CORE.options.jobId, this.viewModel.selectedLayers());
                    }
                    fileReader.readAsDataURL(this.file);
                    VIEWUI.popups.hide();
                }
                this.onFileChnge = function (file) {
                    this.file = file;
                    var self = this;
                    var fileReader = new FileReader();
                    fileReader.onload = function (event) {
                        var content = event.target.result.split("base64,")[1];
                        var dxt = atob(content);
                        CORE.webApiManager.getLayersFromDxf(dxt, function (layerList) {
                            self.layers(layerList);
                        });
                    }
                    fileReader.readAsDataURL(this.file);
                }
                this.template = VIEWUI.popups.templates.importFromDxf;
            },
            show: function () {
                var viewModel = new this.ViewModel();
                VIEWUI.popups.setAndShowPopup(viewModel, VIEWUI.popups.divs.popupContents.element);
                document.getElementById('dxf').addEventListener("change", function (e) {
                    viewModel.onFilechange(e);
                });
            }
        },
        calculationsProgress: {
            ViewModel: function () {
                this.label = translation.CALCULATION_IN_PROGRESS;
                this.template = VIEWUI.popups.templates.calcProgress;
                this.statusCode = ko.observable();
                this.progress = ko.pureComputed(function () {
                    return CORE.calculations.actualProgress() + "%";
                });
                this.status = ko.pureComputed(function () {
                    this.statusCode(CORE.calculations.actualStatus());
                    if (!this.statusCode()) {
                        //return translation.PLEASE_WAIT;
                    } else if (this.statusCode() === "NOTOK") {
                        return translation.CALCULATION_FAILED + '!';
                    } else if (this.statusCode() === "OK") {
                        return translation.CALCULATION_SUCCESS + '!';
                    }
                }, this);
                /*this.onOkClicked = function () {
                    if (isEmpty(CORE.calculations.actualStatus())) {
                        CORE.calculations.cancelCalculations();
                        console.info("Calculations canceled by user");
                    }
                    VIEWUI.popups.hide();
                    VIEWUI.unblockUI();
                }*/
                this.onCancelClicked = function () {
                    VIEWUI.popups.calculationsProgress.hide();
                    CORE.webApiManager.cancelCalculationToken = true;
                }
            },
            show: function () {
                CORE.calculations.actualStatus(null);
                CORE.calculations.actualProgress(0);
                VIEWUI.blockUI();
                var viewModel = new this.ViewModel();
                VIEWUI.popups.setAndShowPopup(viewModel, VIEWUI.popups.divs.popupContents.element);
            },
            hide: function () {
                if (isEmpty(CORE.calculations.actualStatus())) {
                    CORE.calculations.cancelCalculations();
                    console.info("Calculations canceled by user");
                }
                VIEWUI.unblockUI();
                VIEWUI.popups.hide();
            }
        },
        facadeWizard: {
            ViewModel: function (x, z, numberX, numberZ) {
                this.label = "Facade Creation Wizard";
                this.template = VIEWUI.popups.templates.facadeWizard;
                this.cellWidth = typeof (x) === 'undefined' ? 2000 : x;
                this.cellHeight = typeof (z) === 'undefined' ? 2000 : z;
                this.cellNumberWidth = typeof (numberX) === 'undefined' ? 5 : numberX;
                this.cellNumberHeight = typeof (numberZ) === 'undefined' ? 3 : numberZ;
                this.nodeContainer = [];
                this.ctr = 3;
                this.isComplex = false;
                this.createConstruction = function (complex) {
                    this.isComplex = complex;
                    // Create first cell
                    this.createFirstCell();
                    for (var k = 0; k < this.cellNumberHeight; k++) {
                        // Add cell to right side
                        for (var i = 1; i < this.cellNumberWidth; i++) {
                            this.addRightCell(new MODEL.types.Vector((i * this.cellWidth / 1000), 0, (k * this.cellHeight / 1000)), k);
                        }
                        // Create first cell of the row
                        if ((k !== (this.cellNumberHeight - 1))) {
                            this.addTopCell(new MODEL.types.Vector(0, 0, ((k + 1) * this.cellHeight / 1000)), k);
                        }
                    }
                    VIEWUI.popups.facadeWizard.hide();
                };
                this.createFirstCell = function () {
                    // Node 1
                    var vec1 = new MODEL.types.Vector(0, 0, 0);
                    var node1 = new MODEL.Node(vec1);
                    CORE.model.supports()[0].assignTo(node1);
                    CORE.model.addNode(node1);
                    // Node 2
                    var vec2 = new MODEL.types.Vector(0, 0, (this.cellHeight / 1000));
                    var node2 = new MODEL.Node(vec2);
                    if (!this.isComplex) {
                        CORE.model.supports()[1].assignTo(node2);
                    }
                    CORE.model.addNode(node2);
                    // Node 3
                    var vec3 = new MODEL.types.Vector((this.cellWidth / 1000), 0, 0);
                    var node3 = new MODEL.Node(vec3);
                    CORE.model.supports()[0].assignTo(node3);
                    CORE.model.addNode(node3);
                    // Node 4
                    var vec4 = new MODEL.types.Vector((this.cellWidth / 1000), 0, (this.cellHeight / 1000));
                    var node4 = new MODEL.Node(vec4);
                    if (!this.isComplex) {
                        CORE.model.supports()[1].assignTo(node4);
                    }
                    CORE.model.addNode(node4);
                    // Bar 1+2
                    var bar1 = new MODEL.Bar(node1, node2, 600 /*add profile ID*/, 0, 0 /*add gamma*/);
                    CORE.model.addBar(bar1);
                    // Bar 1+3
                    var bar2 = new MODEL.Bar(node1, node3, 600 /*add profile ID*/, 0, 0 /*add gamma*/);
                    CORE.model.addBar(bar2);
                    // Bar 2+4
                    var bar3 = new MODEL.Bar(node2, node4, 600 /*add profile ID*/, 0, 0 /*add gamma*/);
                    CORE.model.addBar(bar3);
                    // Bar 3+4
                    var bar4 = new MODEL.Bar(node3, node4, 600 /*add profile ID*/, 0, 0 /*add gamma*/);
                    CORE.model.addBar(bar4);
                    // Add nodes to container
                    this.nodeContainer[0] = node1;
                    this.nodeContainer[1] = node2;
                    this.nodeContainer[2] = node3;
                    this.nodeContainer[3] = node4;
                };
                this.addRightCell = function (vec, mod) {
                    // Node 2
                    var vec2 = new MODEL.types.Vector((vec.x + (this.cellWidth / 1000)), vec.y, (vec.z + (this.cellHeight / 1000)));
                    var node2 = new MODEL.Node(vec2);
                    if (!this.isComplex) {
                        CORE.model.supports()[1].assignTo(node2);
                    }
                    CORE.model.addNode(node2);
                    // Bar 3+4
                    var bar4 = new MODEL.Bar(this.nodeContainer[(this.ctr)], node2, 600 /*add profile ID*/, 0, 0 /*add gamma*/);
                    CORE.model.addBar(bar4);
                    if (mod < 1) {
                        // Node 1
                        var vec1 = new MODEL.types.Vector((vec.x + (this.cellWidth / 1000)), vec.y, vec.z);
                        var node1 = new MODEL.Node(vec1);
                        CORE.model.supports()[0].assignTo(node1);
                        CORE.model.addNode(node1);
                        // Bar 3+1
                        var bar1 = new MODEL.Bar(this.nodeContainer[(this.ctr - 1)], node1, 600 /*add profile ID*/, 0, 0 /*add gamma*/);
                        CORE.model.addBar(bar1);
                        // Bar 1+2
                        var bar3 = new MODEL.Bar(node1, node2, 600 /*add profile ID*/, 0, 0 /*add gamma*/);
                        CORE.model.addBar(bar3);
                        this.nodeContainer[++this.ctr] = node1;
                    } else {
                        // Bar 1+2
                        var bar3P = new MODEL.Bar(this.nodeContainer[this.ctr - (this.cellNumberWidth * 2)], node2, 600 /*add profile ID*/, 0, 0 /*add gamma*/);
                        CORE.model.addBar(bar3P);
                        this.nodeContainer[++this.ctr] = this.nodeContainer[this.ctr - (this.cellNumberWidth * 2)];
                    }
                    // Add nodes to container               
                    this.nodeContainer[++this.ctr] = node2;
                };
                this.addTopCell = function (vec, mod) {
                    // Node 2
                    var vec2 = new MODEL.types.Vector(vec.x, vec.y, (vec.z + (this.cellHeight / 1000)));
                    var node2 = new MODEL.Node(vec2);
                    if (!this.isComplex) {
                        CORE.model.supports()[1].assignTo(node2);
                    }
                    CORE.model.addNode(node2);
                    // Node 4
                    var vec4 = new MODEL.types.Vector((vec.x + (this.cellWidth / 1000)), vec.y, (vec.z + (this.cellHeight / 1000)));
                    var node4 = new MODEL.Node(vec4);
                    if (!this.isComplex) {
                        CORE.model.supports()[1].assignTo(node4);
                    }
                    CORE.model.addNode(node4);
                    // Bar 1+2
                    var bar1 = new MODEL.Bar(this.nodeContainer[(1 + (((this.cellNumberWidth * 2) + 2) * mod))], node2, 600 /*add profile ID*/, 0, 0 /*add gamma*/);
                    CORE.model.addBar(bar1);
                    // Bar 2+4
                    var bar3 = new MODEL.Bar(node2, node4, 600 /*add profile ID*/, 0, 0 /*add gamma*/);
                    CORE.model.addBar(bar3);
                    // Bar 3+4
                    var bar4 = new MODEL.Bar(this.nodeContainer[(3 + (((this.cellNumberWidth * 2) + 2) * mod))], node4, 600 /*add profile ID*/, 0, 0 /*add gamma*/);
                    CORE.model.addBar(bar4);
                    // Add nodes to container
                    this.nodeContainer[++this.ctr] = this.nodeContainer[1 + (((this.cellNumberWidth * 2) + 2) * mod)];
                    this.nodeContainer[++this.ctr] = node2;
                    this.nodeContainer[++this.ctr] = this.nodeContainer[3 + (((this.cellNumberWidth * 2) + 2) * mod)];
                    this.nodeContainer[++this.ctr] = node4;
                };
            },
            show: function () {
                VIEWUI.blockUI();
                var viewModel = new this.ViewModel();
                VIEWUI.popups.setAndShowPopup(viewModel, VIEWUI.popups.divs.popupContents.element);
            },
            hide: function () {
                VIEWUI.unblockUI();
                VIEWUI.popups.hide();
            }
        },

        settings: {
            ViewModel: function () {
                this.activeTab = ko.observable(1);
                this.label = translation.SYSTEM_PREFERENCES;
                this.template = VIEWUI.popups.templates.settings;
                this.viewOptions = VIEWUI.viewOptions;
                this.unitSystems = [
                    {
                        name: translation.METRIC,
                        changePreset: function () {
                            VIEWUI.viewOptions.unitDistance("m");
                            VIEWUI.viewOptions.unitForce("kN");
                            VIEWUI.viewOptions.unitStress("MPa");
                            VIEWUI.viewOptions.unitMomentForce("kN");
                            VIEWUI.viewOptions.unitMomentDistance("m");
                            VIEWUI.viewOptions.unitPressureForce("kN");
                            VIEWUI.viewOptions.unitPressureSurface("m2");
                            VIEWUI.viewOptions.unitAngle("°");
                        }
                    },
                    {
                        name: translation.IMPERIAL,
                        changePreset: function () {
                            VIEWUI.viewOptions.unitDistance("ft");
                            VIEWUI.viewOptions.unitForce("kip");
                            VIEWUI.viewOptions.unitStress("MPa");
                            VIEWUI.viewOptions.unitMomentForce("kip");
                            VIEWUI.viewOptions.unitMomentDistance("ft");
                            VIEWUI.viewOptions.unitPressureForce("kip");
                            VIEWUI.viewOptions.unitPressureSurface("ft2");
                            VIEWUI.viewOptions.unitAngle("°");
                        }
                    },
                    {
                        name: translation.SI,
                        changePreset: function () {
                            VIEWUI.viewOptions.unitDistance("m");
                            VIEWUI.viewOptions.unitForce("N");
                            VIEWUI.viewOptions.unitStress("Pa");
                            VIEWUI.viewOptions.unitMomentForce("N");
                            VIEWUI.viewOptions.unitMomentDistance("m");
                            VIEWUI.viewOptions.unitPressureForce("N");
                            VIEWUI.viewOptions.unitPressureSurface("m2");
                            VIEWUI.viewOptions.unitAngle("Rad");
                        }
                    }
                ];
                this.onOkClicked = function () {
                    VIEWUI.popups.hide();

                    var proto = CORE.model.save();
                    CORE.model.sendProto(proto, { mode: SAVE_MODE.NoConstraints });
                }
                this.onCancelClicked = function () {
                    VIEWUI.popups.settings.hide();

                    var proto = CORE.model.save();
                    CORE.model.sendProto(proto, { mode: SAVE_MODE.NoConstraints });
                }
                this.setActiveTab = function (id) {
                    this.activeTab(id);
                }
            },
            show: function () {
                var viewModel = new this.ViewModel();
                VIEWUI.popups.setAndShowPopup(viewModel, VIEWUI.popups.divs.popupContents.element);
            },
            hide: function () {
                VIEWUI.popups.hide();
            }
        },
        versionList: {
            ViewModel: function () {
                this.versionList = ko.observableArray();
                this.currentVersionId = CORE.currentJobVersionId; // observable
                this.selectedVersionId = ko.observable(null);
                this.saveVersionPopupVisible = ko.observable(false);
                this.loadVersionPopupVisible = ko.observable(false);
                this.noVersionSelectedPopupVisible = ko.observable(false);
                this.deleteVersionPopupVisible = ko.observable(false);
                this.saveVersionName = ko.observable();
                this.versionOperationStatus = ko.observable();
                this.saveVersionNameValMessage = ko.observable("");
                this.label = translation.VERSION_LIST;
                this.template = VIEWUI.popups.templates.versionList;
                this.versionListStatusMessage = ko.observable("");
                this.versionCount = 0;
                this.versionToDelete = ko.observable(null);
                this.searchQuery = ko.observable("");
                this.saveVersion = function () {
                    var self = this;
                    var callback = function (response) {
                        if (response.saveVersionSucceeded) {
                            console.log("Version successfully saved");
                            self.versionOperationStatus(translation.VERSION_SAVED);
                            var callback = function () { self.currentVersionId(self.versionCount); }
                            self.init(callback);
                        }
                        else {
                            errorCallback();
                        }
                    }
                    var errorCallback = function () {
                        console.error("Failed to save version");
                        self.versionOperationStatus(translation.VERSION_FAILED);
                        self.init();
                    }
                    var versionName = this.saveVersionName();
                    if (isEmpty(versionName) || versionName === "" || versionName.length > 255) {
                        this.saveVersionNameValMessage(translation.VERSION_ENTER);
                        return;
                    }
                    self.saveVersionPopupVisible(false);
                    self.versionOperationStatus(translation.VERSION_SAVING + '...');
                    var proto = CORE.model.save();
                    CORE.model.sendProto(proto, undefined, callback, errorCallback, versionName);
                    this.saveVersionNameValMessage("");
                }
                this.loadVersionValidate = function () {
                    if (isEmpty(this.selectedVersionId())) {
                        this.noVersionSelectedPopupVisible(true);
                    } else {
                        this.loadVersionPopupVisible(true);
                    }
                }
                this.loadVersion = function () {
                    var self = this;
                    if (isEmpty(this.selectedVersionId())) {
                        alert("No version selected");
                        return;
                    }
                    self.loadVersionPopupVisible(false);
                    self.versionOperationStatus(translation.VERSION_LOADING + '...');
                    var selectedVersionId = this.selectedVersionId();
                    if (selectedVersionId === 0) {
                        selectedVersionId = null;
                    }
                    var callback = function () {
                        console.log("VERSION SUCCESSFULLY LOADED");
                        CORE.view3D.cameraControl.zoomAll();
                        self.currentVersionId(selectedVersionId !== null ? selectedVersionId : 0);
                        self.versionOperationStatus(translation.VERSION_LOAD_SUCCESS);
                        CORE.jobVersionLoaded = true; // for setting current version to 0 when model changes
                        VIEWUI.popups.versionList.hide();
                    }
                    var errorCallback = function () {
                        console.error("FAILED TO LOAD VERSION");
                        self.versionOperationStatus(translation.VERSION_LOAD_FAIL);
                    }
                    CORE.model.load(CORE.options.appId, CORE.options.projId, CORE.options.jobId, selectedVersionId, callback, errorCallback);
                }
                this.deleteVersion = function () {
                    var self = this;
                    var versionToDelete = this.versionToDelete();
                    if (isEmpty(versionToDelete)) {
                        alert("error: empty id");
                        return;
                    }
                    var callback = function (data) {
                        if (data === true) {
                            console.log("Version deleted");
                            self.versionOperationStatus(translation.VERSION_DELETED);
                        }
                        if (data === false) {
                            console.log("Error: Version doesn't exist");
                            self.versionOperationStatus(translation.VERSION_NOT_EXISTED);
                        }
                        self.init();
                        self.versionToDelete(null);
                    }
                    var errorCallback = function () {
                        console.error("Failed to delete version");
                        self.versionOperationStatus(translation.VERSION_FAIL);
                        self.init();
                    }
                    self.versionOperationStatus(translation.VERSION_DELETING + '...');
                    self.deleteVersionPopupVisible(false);
                    CORE.webApiManager.deleteVersion(CORE.options.appId, CORE.options.projId, CORE.options.jobId, versionToDelete, callback, errorCallback);
                }
                this.Version = function (id, index, name, userName, date) {
                    this.id = id;
                    this.index = index;
                    this.name = name;
                    this.userName = userName;
                    this.date = date;
                }
                this.init = function (outerCallback) {
                    var self = this;
                    var callback = function (data) {
                        try {
                            self.versionList.removeAll();
                            for (var i = 0, ii = data.length; i < ii; i++) {
                                var version = new self.Version(data[i].id, i + 1, data[i].name, data[i].userName, data[i].date);
                                self.versionList.push(version);
                            }
                            self.versionCount = data.length;
                            //self.versionList.push(new self.Version(0, "latest", CORE.options.userName, ""));   // 0 version id = latest
                            if (self.versionCount === 0) {
                                self.versionListStatusMessage(translation.VERSION_NOT_SAVED);
                            } else {
                                self.versionListStatusMessage("");
                            }
                            $('.VIEWUI_versionListContainer').scrollTop(1E10);
                            if (!isEmpty(outerCallback)) {
                                outerCallback();
                            }
                            console.log("Version list loaded successfully");
                        } catch (err) {
                            self.versionListStatusMessage(translation.VERSION_NOT_LOADED_PROGRESS);
                            self.versionOperationStatus("");
                            console.error("Failed to load version list (processing error)");
                            console.error(err.stack);
                        }
                    }
                    var errorCallback = function () {
                        self.versionList.removeAll();
                        self.versionListStatusMessage(translation.VERSION_NOT_LOADED_REQUEST);
                        self.versionOperationStatus("");
                        console.error("Failed to load version list (request failed)");
                    }
                    CORE.webApiManager.importVersionList(CORE.options.appId, CORE.options.projId, CORE.options.jobId, callback, errorCallback);
                    //$('.VIEWUI_versionListContainer').bind('mousewheel DOMMouseScroll', function(e) {
                    //    var e0 = e.originalEvent,
                    //        delta = e0.wheelDelta || -e0.detail;

                    //    this.scrollTop += (delta < 0 ? 1 : -1) * 50;
                    //    e.preventDefault();
                    //});
                };
                this.onOkClicked = function () {
                    VIEWUI.popups.hide();
                };
                this.onCancelClicked = function () {
                    VIEWUI.popups.versionList.hide();
                };
            },
            show: function () {
                var viewModel = new this.ViewModel();
                VIEWUI.popups.setAndShowPopup(viewModel, VIEWUI.popups.divs.popupContents.element);
                viewModel.versionListStatusMessage(translation.LOADING);
                viewModel.init();
                this.bindEvents(viewModel);
            },
            hide: function () {
                VIEWUI.popups.hide();
            },
            bindEvents: function (vm) { // should be moved to subobjects in viewModel?
                var onKeyDown = function (event) {
                    if (event.keyCode === KEYCODES.ENTER) {
                        vm.saveVersion();
                    }
                    if (event.keyCode === KEYCODES.ESC) {
                        vm.saveVersionPopupVisible(false);
                        vm.saveVersionName("");
                    }
                }
                vm.saveVersionPopupVisible.subscribe(function (newValue) {
                    if (newValue) {
                        $('#VIEWUI_viewerContainer').bind('keydown', onKeyDown);
                        $('.VIEWUI_versionListPopup input').eq(1).focus();
                        vm.saveVersionName("");
                        vm.saveVersionNameValMessage("");
                    } else {
                        $('#VIEWUI_viewerContainer').unbind('keydown', onKeyDown);
                    }
                });
            }
        },

        fixModel: {
            callback: null,
            ViewModel: function () {
                this.label = translation.INCOMPATIBLE_MODEL;
                this.template = VIEWUI.popups.templates.fixModel;
                this.onManualClicked = function () {
                    CORE.autoSaveDisabled = true;
                    VIEWUI.popups.fixModel.callback(true);
                    VIEWUI.popups.hide();
                }
                this.onAutoClicked = function () {
                    VIEWUI.popups.fixModel.callback(false);
                    VIEWUI.popups.fixModel.hide();
                }
            },
            show: function (construction, call) {
                this.callback = call;
                var viewModel = new this.ViewModel();
                VIEWUI.popups.setAndShowPopup(viewModel, VIEWUI.popups.divs.popupContents.element);
            },
            hide: function () {
                VIEWUI.popups.hide();
            }
        },

        reportSettings: {
            ViewModel: function () {
                this.label = translation.REPORT_SETTINGS;
                this.template = VIEWUI.popups.templates.reportSettings;
                this.multiplier = ko.observable(1);
                this.reportType = null;
                this.mode = null;
                this.getConnectorsCount = function () {
                    return CORE.model.nodes().filter(function (node) { return node.connectorId(); }).length;
                }

                this.onGenerateReportClicked = function () {

                    var multiplier = this.multiplier(); //Number(this.multiplier());

                    var settings = {
                        barMultiplier: multiplier,
                        nodeMultiplier: multiplier
                    };
                    switch (this.mode) {
                        case 'PDF':
                            generateReport(this.reportType, settings);
                            break;
                        case 'XLS':
                            generateXls(multiplier);
                            break;
                    }

                    VIEWUI.popups.hide();
                };

                this.onCancelClicked = function () {
                    VIEWUI.popups.hide();
                };
            },
            show: function (reportType, mode) {
                var viewModel = new this.ViewModel();
                viewModel.mode = mode;
                viewModel.reportType = reportType;
                VIEWUI.popups.setAndShowPopup(viewModel, VIEWUI.popups.divs.popupContents.element);
            }
        },

        hide: function () {
            //VIEWUI.popups.divs.popupsContainer.element.hide();
            VIEWUI.viewModel.popupCreateVisible(false);

            setTimeout(function () {                                        // for faster popup closing
                ko.cleanNode(VIEWUI.popups.divs.popup.element[0]);
            }, 0);

            //VIEWUI.popups.divs.popup.element.empty();
        },
        addBaseProperties: function (viewModel) { // pass a viewModel to enable auto align (also requires afterRender event hookup in html)
            viewModel.autoCenter = VIEWUI.popups.autoCenterPopup;
        },

        autoCenterPopup: function () {
            var popupWidth = VIEWUI.divs.popupsContainer.element.width();
            var popupHeight = VIEWUI.divs.popupsContainer.element.height();
            var viewerWidth = $('#VIEWUI_viewerContainer').width();
            var viewerHeight = $('#VIEWUI_viewerContainer').height();
            VIEWUI.divs.popupsContainer.element.css('left', (viewerWidth - popupWidth) / 2);
            VIEWUI.divs.popupsContainer.element.css('top', (viewerHeight - popupHeight - $("#VIEWUI_topBar").height()) / 2);
            //VIEWUI.divs.popupsContainer.element.css('width', popupWidth);
            //VIEWUI.divs.popupsContainer.element.css('height', popupHeight);

            //setTimeout(function () {
            //    var popup = $("#VIEWUI_popup");
            //    var popupWidth = popup.width();
            //    if (popupWidth > 100) {
            //        popup.css('width', popup.width() + 2);  // 'affix' popup window so it won't get smaller when resizing window
            //    }
            //}, 0);
        },

        setAndShowPopup: function (viewModel, _propertiesDiv) {
            this.addBaseProperties(viewModel);
            //propertiesDiv = _propertiesDiv.clone(); //create a new copy from template
            //VIEWUI.popups.divs.popup.element.css('visibility', 'hidden'); // hide before aligning
            //properties show
            //VIEWUI.popups.divs.popupContents.element.html(propertiesDiv);
            //VIEWUI.popups.divs.popup.element.css('visibility', 'visible'); // show after aligning
            ko.cleanNode(VIEWUI.popups.divs.popup.element[0]);
            var popup = VIEWUI.popups.divs.popup.element;
            popup.css('width', '');
            
            ko.applyBindings(viewModel, VIEWUI.popups.divs.popup.element[0] /*propertiesDiv[0]*/);
            VIEWUI.viewModel.popupCreateVisible(true); // afterRender event ko
            //propertiesDiv = null;
            VIEWUI.spinners.init("#VIEWUI_popup");
            VIEWUI.properties.initComboboxListBehaviour(VIEWUI.popups.divs.popup.element);
            VIEWUI.initScrollbar();
        }
    }

});