﻿VIEWUI.properties = VIEWUI.properties || {};

VIEWUI.properties.Bar = function(viewModel) {
    var self = this;

    // var angle0 = $("input[id='angle0']")[0];
    // var angle90 = $("input[id='angle90']")[0];
    // var angle180 = $("input[id='angle180']")[0];
    // var angle270 = $("input[id='angle270']")[0];

    var angle0 = false;
    var angle90 = false;
    var angle180 = false;
    var angle270 = false;

    for (var i = 0; i < viewModel.alternativeRotations().length; i++) {

        var epsilon = 0.001;
        var angle = viewModel.alternativeRotations()[i]*180/Math.PI;

        if (Math.abs(angle)<epsilon) {
            angle0 = true;
        } else if (Math.abs(angle-90)<epsilon) {
            angle90 = true;
        } else if (Math.abs(angle-180)<epsilon) {
            angle180 = true;
        } else if (Math.abs(angle-270)<epsilon) {
            angle270 = true;
        };
    };

    this.getAngle0 = function() {
        return angle0;
    };

    this.getAngle90 = function() {
        return angle90;
    };
    this.getAngle180 = function() {
        return angle180;
    };
    this.getAngle270 = function() {
        return angle270;
    };
    
    this.sel = ko.observable(isEmpty(viewModel.modelObject.gamma()) ? "Error in bar's rotation" : viewModel.modelObject.gamma());
    this.sel.subscribe(
        function(value) {
            value = Number(value);
            if (value === 0 || value === 90 || value === 180 || value === 270) {
                viewModel.modelObject.gamma(value);
            } else {
                console.error("Provided bar's rotation is invalid");
            }
        });
    this.length = ko.pureComputed({
        read: function() {
            if (isEmpty(this.node1()) || isEmpty(this.node2())) { // ko wants to calculate this computed after the last bar has been deleted
                return 0;
            }
            var MemberConstraints = CORE.model.constraints.getMemberConstraints(this.id());
            if (!MemberConstraints) {
                return 0;
            }
            return MemberConstraints.length;
        },
        write: function() {}
    }, viewModel);

    this.onFinishClicked = function(viewModel) {
        CORE.model.deselectAll();
        VIEWUI.toolbar.switchMode(MODE.SELECT);
        VIEWUI.viewModel.popupMode(null);
    };

    this.onProfileListItemClick = function(modelObject, profile, isCircle) {
        modelObject.profileId(profile);
        modelObject.isProfileCircle(isCircle);
    };
    this.onProfileCurrentItemClick = function(object, event) {
        this.profileSearchQuery('');
        this.profileSearchResults(CORE.model.constraints.getProfilesForBar(viewModel.modelObject.id));
    };
    setTimeout(function() {
        $('.VIEWUI_allFieldsContainer').scroll(function() {
            //self.showProfileList(false);
        }.bind(this));
    }, 0);
    $('body').click(function(event) {
        if (!$(event.target).closest('.VIEWUI_combobox').length) {
            //this.showProfileList(false);
        }
    }.bind(this));
    this.profileSearchQuery = ko.observable();
    this.onProfileSearchQueryChanged = function() {
        var query = this.profileSearchQuery();
        var profilesRepository = CORE.model.constraints.getProfilesForBar(viewModel.modelObject.id);
        var results = [];
        if (!query) {
            this.profileSearchResults(profilesRepository);
        } else {
            for (var i = 0; i < profilesRepository.length; i++) {
                profileItem = profilesRepository[i];
                if (profileItem.ElementName.toLowerCase().indexOf(query.toLowerCase()) > -1) {
                    results.push(profileItem);
                }
            }
            this.profileSearchResults(results);
        }
    };
    this.profileSearchResults = ko.observableArray();
    this.checkGamma = ko.observable(viewModel.isGammaAuto() ? -1 : Number(viewModel.gamma().toFixed(8)));
    HELPERS.subscribePropertyChange(this, this.checkGamma, "", function(checkGamma) {
        if (checkGamma === -1) {
            viewModel.isGammaAuto(true);
        } else {
            viewModel.isGammaAuto(false);
            viewModel.gamma(checkGamma);
        }
    });

    this.isCircle = function() {
        var profileConstraints = CORE.model.constraints.getBarProfile(this.id(), this.profileId());
        return profileConstraints.IsCircle;
    }

    // ========= results ============
    this.resultsTypes = ["DISPLACEMENT", "INTERNAL_FORCES"],
        this.resultsTypeSelection = VIEWUI.properties.resultsTypeSelection;

    this.visibleResultsForce = VIEWUI.properties.visibleResultsForce; // fx, fy...
    this.visibleResultsDisp = VIEWUI.properties.visibleResultsDisp; // ux,uy and uz
    this.resultsCoordinate = ko.observable(null);
    this.resultsValueInCoordinate = ko.observable(0);
    this.resultsModel = null;
    this.resultsViewOptions = VIEWUI.properties.resultsViewOptions; // viewOptions object (compatible with VIEW3D.BarsResults)
    this.extremumValue = ko.observable(null);
    this.afterCopy = function() { // custom "event" (common interface)
        if (!VIEWUI.viewModel.calculationsFinished()) {
            return 0;
        }
        var node1Vm = viewModel.modelObject.node1();
        var node2Vm = viewModel.modelObject.node2();
        var node1Pos = HELPERS.toVector3(node1Vm.position());
        var node2Pos = HELPERS.toVector3(node2Vm.position());
        var barLength = node1Pos.distanceTo(node2Pos);
        var node1 = new MODEL.Node(new MODEL.types.Vector(-barLength / 2, 0, 0));
        var node2 = new MODEL.Node(new MODEL.types.Vector(barLength / 2, 0, 0));
        var bar = new MODEL.Bar(node1, node2);
        //resultsWindowCamera.zoom = 1;                 // / barLength * (4 - 0.2);   // adjust camera so the bar fills whole window
        resultsWindowCamera.left = (-barLength / 2); // - (barLength * 0.03);
        resultsWindowCamera.right = (barLength / 2); // + (barLength * 0.03);           
        resultsWindowCamera.updateProjectionMatrix();
        CORE.resultsWindowModel.addNode(node1);
        CORE.resultsWindowModel.addNode(node2);
        CORE.resultsWindowModel.addBar(bar);

        this.resultsModel = this.getResultsModel(); // gets results model for current bar
        this.resultsModel[0].bar = bar; // only one bar in visualization
        this.updateResultsModel();

        var copyRendererContainer = function() {
            var resultsWebGlContainer = $('#resultsWebGlContainer');

            if (resultsWebGlContainer.length === 0) {
                console.warn("#resultsWebGlContainer is lost, need to reinitialize");
                CORE.initResultsRenderer();
                resultsWebGlContainer = $('#resultsWebGlContainer');
            }

            var target = $('#VIEWUI_properties #VIEWUI_barResults2dView').eq(0);
            if (target.length === 0) return false;

            resultsWebGlContainer.appendTo(target); // move webGlContainer from VIEWUI.html to properties div when properties are shown
            VIEWUI.properties.checkContainerSize();

            return true;
        }

        var successfullyCopied = copyRendererContainer();

        setTimeout(function() {
            if(CORE.isResultsScreen){
                if (!successfullyCopied) // try to copy again if failed before (firefox)
                    successfullyCopied = copyRendererContainer();
                if (!successfullyCopied) console.error("Failed to copy resultsWebGlContainer");
            }
        }, 50);

        this.render();
    }

    this.existsResults = function() {
        if (VIEWUI.viewModel.calculationsFinished() && CORE.isResultsScreen) {
            return true;
        }
        return false;
    }   

    this.getResultsModel = function() {
            var barResultsChosen = [];
            var barResultsArrayAll = CORE.model.barsResults();
            if (isEmpty(barResultsArrayAll) || !barResultsArrayAll.length) {
                throw new Error("VIEWUI.properties.Bar.getResultsModel: requested resultsWindow but barResultsArrayAll is empty!");
            }
            if (!viewModel.modelObject) {
                console.error("VIEWUI.properties.Bar.getResultsModel: viewModel.modelObject is empty!");
                return null;
            }
            for (var i = 0, ii = barResultsArrayAll.length; i < ii; i++) { //search for corresponding result 3d object
                if (barResultsArrayAll[i].bar === viewModel.modelObject) {
                    barResultsChosen.push(jQuery.extend(true, {}, barResultsArrayAll[i]));
                    return barResultsChosen;
                }
            }
            console.error("VIEWUI.properties.Bar.getResultsModel: Corresponding bar not found!");
            return null;
        },
        this.updateResultsModel = function() {
            CORE.resultsView3D.barsResults.viewOptions = this.resultsViewOptions; // set view options in VIEW3D
            CORE.resultsView3D.barsResults.subscribeViewOptions();
            CORE.resultsWindowModel.barsResults(this.resultsModel);
            this.updateAxisDescription();
            this.updateResultsValueAtCoordinate();
        }
    this.updateAxisDescription = function() {
        this.resultsModel = this.getResultsModel(); // gets results model for current bar

        var extremumName = this.resultsTypeSelection() === "DISPLACEMENT" ? this.visibleResultsDisp().toLowerCase() : this.visibleResultsForce().toLowerCase();
        var extremum = this.resultsModel[0].data.extrema[extremumName][0]; // find extremum for selected force type
        this.extremumValue(Math.max(Math.abs(extremum.minVal), Math.abs(extremum.maxVal)));
        //this.extremumValue(this.formatExtremumValue(extremumRawValue));
    }

    function hideCalcForces(self) {
        self.resultsViewOptions.showCalcForceFx(false);
        self.resultsViewOptions.showCalcForceFy(false);
        self.resultsViewOptions.showCalcForceFz(false);
        self.resultsViewOptions.showCalcForceMx(false);
        self.resultsViewOptions.showCalcForceMy(false);
        self.resultsViewOptions.showCalcForceMz(false);
    }

    function hideDisp(self) {
        self.resultsViewOptions.showCalcForceUx(false);
        self.resultsViewOptions.showCalcForceUy(false);
        self.resultsViewOptions.showCalcForceUz(false);
    }

    this.updateResultsValueAtCoordinate = function() {
        var newResultsCoordinate = this.resultsCoordinate();
        CORE.resultsWindowModel.lastXcoordinate(newResultsCoordinate);
        if (newResultsCoordinate > this.length()) { // validation
            newResultsCoordinate = this.length();
            this.resultsCoordinate(newResultsCoordinate);
        }
        if (newResultsCoordinate < 0) {
            newResultsCoordinate = 0;
            this.resultsCoordinate(newResultsCoordinate);
        }
        if (isEmpty(this.resultsModel)) {
            this.resultsModel = this.getResultsModel(); // gets results model for current bar
        }
        var splines = this.resultsModel[0].data.splines;
        var spline = null;
        for (var i = 0; i < splines.length; i++) { // find spline for selected x coordinate
            if (splines[i].end > newResultsCoordinate) {
                spline = splines[i];
                break;
            }
        }
        if (spline === null) { // if spline not found, use the last one (sometimes last bar's end is smaller than bar's length)
            spline = splines[splines.length - 1];
        }
        var barT = newResultsCoordinate;
        var polynomialValue = function(factors, x) {
            var pow = 0;
            var val = 0;
            for (var i = 0; i < factors.length; i++) {
                val += factors[i] * Math.pow(x, pow);
                pow++;
            }
            return val;
        }
        var l = null; // results value at coordinate
        if (this.resultsTypeSelection() === 'DISPLACEMENT') {
            switch (this.visibleResultsDisp()) {
                case 'Ux':
                    l = polynomialValue(spline.UX, barT);
                    break;
                case 'Uy':
                    l = polynomialValue(spline.UYRZ, barT);
                    break;
                case 'Uz':
                    l = -polynomialValue(spline.UZRY, barT);
                    break;
                default:
                    throw new Error("Unknown displacement selected");
            }
        }
        if (this.resultsTypeSelection() === 'INTERNAL_FORCES') {
            switch (this.visibleResultsForce()) {
                case 'Fx':
                    l = polynomialValue(spline.FX, barT);
                    break;
                case 'Fy':
                    var fymZderiv = spline.FYMZ.slice(1, spline.FYMZ.length);
                    fymZderiv[1] *= 2;
                    fymZderiv[2] *= 3;
                    l = polynomialValue(fymZderiv, barT);
                    break;
                case 'Fz':
                    var fzmYderiv = spline.FZMY.slice(1, spline.FZMY.length);
                    fzmYderiv[1] *= 2;
                    fzmYderiv[2] *= 3;
                    l = polynomialValue(fzmYderiv, barT);
                    break;
                case 'Mx':
                    l = polynomialValue(spline.MX, barT);
                    break;
                case 'My':
                    l = polynomialValue(spline.FZMY, barT);
                    break;
                case 'Mz':
                    l = -polynomialValue(spline.FYMZ, barT);
                    break;
                default:
                    throw new Error("Unknown force selected");
            }
        }
        if (l === null) throw new Error("Unknown resultsTypeSelection()");
        console.log(l);
        if (Math.abs(l) < 0.000001) l = 0; // epsilon
        this.resultsValueInCoordinate(l);
        this.render();
    }

    this.render = function() {
        resultsWindowRenderer.clear();
        resultsWindowRenderer.render(resultsWindowGridScene, resultsWindowGridCamera);
        resultsWindowRenderer.clearDepth();
        resultsWindowRenderer.render(resultsWindowScene, resultsWindowCamera);
    }
    var subscription1 = HELPERS.subscribePropertyChange(this, this.resultsTypeSelection, "this.resultsTypeSelection", function(resultsTypeSelection) {
        if (resultsTypeSelection === "DISPLACEMENT") {
            //this.resultsViewOptions.showCalcDisplacement(false);
            hideCalcForces(this);
            this.resultsViewOptions["showCalcForce" + this.visibleResultsDisp()](true);
        } else {
            if (resultsTypeSelection === "INTERNAL_FORCES") {
                //this.resultsViewOptions.showCalcDisplacement(false);
                hideDisp(this);
                this.resultsViewOptions["showCalcForce" + this.visibleResultsForce()](true);
            }
        }
        this.updateAxisDescription();
        this.updateResultsValueAtCoordinate();
    });
    var subscription2 = HELPERS.subscribePropertyChange(this, this.visibleResultsForce, "this.visibleResultsForce", function(visibleResultsForce) {
        if (!isEmpty(visibleResultsForce)) {
            hideCalcForces(this);
            hideDisp(this);
            //this.resultsViewOptions.showCalcDisplacement(false);
            this.resultsViewOptions["showCalcForce" + visibleResultsForce](true);
            this.updateAxisDescription();
            this.updateResultsValueAtCoordinate();
        }
    });

    var subscriptionDisp = HELPERS.subscribePropertyChange(this, this.visibleResultsDisp, "this.visibleResultsDisp", function(visibleResultsDisp) {
        if (!isEmpty(visibleResultsDisp)) {
            hideCalcForces(this);
            hideDisp(this);
            //this.resultsViewOptions.showCalcDisplacement(false);
            this.resultsViewOptions["showCalcForce" + visibleResultsDisp](true);
            this.updateAxisDescription();
            this.updateResultsValueAtCoordinate();
        }
    });
    var subscription3 = HELPERS.subscribePropertyChange(this, CORE.model.barsResults, "CORE.model.barsResults", function(newModel) {
        this.afterCopy();
    });
    var subscription6 = HELPERS.subscribePropertyChange(this, CORE.model.nodesResults, "CORE.model.nodesResults", function(newModel) {
        // this is observed because of calculationsFinished check
        // without it, the call of the afterCopy method is ignored when barsResults are ready, but nodesResults are not
        this.afterCopy();
    });

    var subscription4 = HELPERS.subscribePropertyChange(this, this.resultsCoordinate, "this.resultsCoordinate", function(newResultsCoordinate) {
        CORE.resultsWindowModel.lastXcoordinate(newResultsCoordinate);
        this.updateResultsValueAtCoordinate();
    });
    var subscription5 = HELPERS.subscribePropertyChange(this, CORE.resultsWindowModel.lastXcoordinate, "CORE.resultsWindowModel.lastXcoordinate", function(lastXcoordinate) {
        this.resultsCoordinate(lastXcoordinate); // lastXcoordinate = [0, barLength]
    });
    VIEWUI.properties.subscriptions.push(subscription1, subscription2, subscription3, subscription4, subscription5, subscription6, subscriptionDisp); //save subsciptions for easy cleanup
    this.barResultsValues = function() {
            this.resultsModel = this.getResultsModel();
            if (this.resultsModel)
                return this.resultsModel[0].data.values;
            else return function() { return {} }; // prevent throwing in ko (need to reproduce it)
        }
        // ======== /results =============
}