﻿VIEWUI.properties = VIEWUI.properties || {};

VIEWUI.properties.Node = function(viewModel) {
    // NODE RESULTS
    this.viewModel = viewModel;
    this.getResultsModel = function() {
        var nodeResultsChosen = [];
        var nodeResultsArrayAll = CORE.model.nodesResults();
        if (isEmpty(nodeResultsArrayAll) || !nodeResultsArrayAll.length) {
            throw new Error("VIEWUI.properties.Node.getResultsModel: requested resultsWindow but nodeResultsArrayAll is empty!");
        }
        for (var i = 0, ii = nodeResultsArrayAll.length; i < ii; i++) //search for corresponding result 3d object
        {
            if (nodeResultsArrayAll[i].node === viewModel.modelObject) {
                nodeResultsChosen.push(jQuery.extend(true, {}, nodeResultsArrayAll[i]));
                if (nodeResultsChosen[0].node.isSupport()) {
                    this.showReactions = true;
                }
                return nodeResultsChosen;
            }
        }
        return null;
        //throw new Error("VIEWUI.properties.Node.getResultsModel: Corresponding node not found!");
    };
    this.nodeResults = function() {
        if (this.resultsModel === null)
            this.resultsModel = this.getResultsModel();

        return this.resultsModel[0].data;
    };
    this.existsResults = function() {
        if (VIEWUI.viewModel.calculationsFinished() && CORE.isResultsScreen) {
            this.resultsModel = this.getResultsModel();

            if (this.resultsModel !== null)
                return this.viewModel.connectorId() !== 0;
            else
                return false;
        }
        return false;
    }
    this.showPressureResults = function() {
        if (Math.abs(this.resultsModel[0].data.pressure) > 0.00001)
            return true;
        else
            return false;
    };
    this.resultsTypes = ["Displacement", "Internal Forces"];
    this.resultsTypeSelection = VIEWUI.properties.resultsTypeSelection;
    this.visibleResultsForce = VIEWUI.properties.visibleResultsForce; // fx, fy...
    this.resultsCoordinate = ko.observable(null);
    this.resultsModel = null;
    this.showReactions = false;
    this.resultsViewOptions = VIEWUI.properties.resultsViewOptions;
    // NODE CONNECTOR COMBOBOX
    this.showConnectorList = ko.observable(false);

    this.onFinishClicked = function (viewModel) {
        CORE.model.deselectAll();
        VIEWUI.toolbar.switchMode(MODE.SELECT);
        VIEWUI.viewModel.popupMode(null);
    };

    this.onConnectorListItemClick = function(modelObjectProperty, itemId) {
        modelObjectProperty(itemId);
        this.showConnectorList(false);
    };
    this.onConnectorCurrentItemClick = function() {
        if (this.showConnectorList()) {
            this.showConnectorList(false);
        } else {
            this.connectorSearchQuery('');
            var connectorList = [];
            var constrainsConnectorList = CORE.model.constraints.getConnectorsForNode(viewModel.modelObject.id);
            if (constrainsConnectorList.length !== 0) {
                for (var i = 0; i < constrainsConnectorList.length; i++) {
                    connectorList.push(constrainsConnectorList[i]);
                }
            }
            this.connectorSearchResults(connectorList);
            this.showConnectorList(true);
            this.showSupportTypeList(false);
        }
    };
    this.connectorSearchQuery = ko.observable();
    this.onConnectorSearchQueryChanged = function() {
        var query = this.connectorSearchQuery();
        var connectorsRepository = CORE.model.constraints.getConnectorsForNode(viewModel.modelObject.id);
        var results = [];
        var i;
        if (connectorsRepository.length !== 0) {
            if (!query) {
                for (i = 0; i < connectorsRepository.length; i++) {
                    results.push(connectorsRepository[i]);
                }
                this.connectorSearchResults(results);
            } else {
                for (i = 0; i < connectorsRepository.length; i++) {
                    if (connectorsRepository[i].ElementName.toLowerCase().indexOf(query.toLowerCase()) > -1) {
                        results.push(connectorsRepository[i]);
                    }
                }
            }
        }
        this.connectorSearchResults(results);
    };
    this.connectorSearchResults = ko.observableArray();
    this.compatibleConnectorsFound = function() {
        return CORE.model.constraints.getConnectorsForNode(viewModel.modelObject.id).length > 0;
    };
    this.showConnectorInfo = function() {
        if ((!isEmpty(viewModel.modelObject.supportId()) && viewModel.modelObject.supportId() !== 0) ||
            viewModel.createsBars().length >= 2) {
            return true;
        }
        if (viewModel.modelObject.createsBars().length === 0) return false;
        var i, j;
        for (i = 0; i < CORE.model.bars().length; i++) {
            for (j = 0; j < CORE.model.bars()[i].attachedNodes().length; j++) {
                if (CORE.model.bars()[i].attachedNodes()[j].id === viewModel.modelObject.id) {
                    return true;
                }
            }
        }
        return false;
    };

    // NODE SUPPORT COMBOBOX
    this.showSupportTypeList = ko.observable(false);
    this.onSupportTypeListItemClick = function(id) {
        for (var i = 0; i < CORE.model.nodes().length; i++) {
            var node = CORE.model.nodes()[i];
            if (node.id === this.id()) {
                var supp = CORE.model.getSupportById(id);
                if (id === 0) {
                    if (this.supportId() !== 0) {
                        CORE.model.getSupportById(this.supportId()).removeAssignment(node);
                    }
                } else {
                    supp.assignTo(node);
                }
                break;
            }
        }
        this.showSupportTypeList(false);
    };
    this.onSupportTypeCurrentItemClick = function() {
        if (this.showSupportTypeList()) {
            this.showSupportTypeList(false);
        } else {
            this.supportTypeSearchQuery('');
            this.supportTypeSearchResults(CORE.model.constraints.getNodeConstraints(this.id()).supports);
            this.showSupportTypeList(true);
            this.showConnectorList(false);
        }
    };
    this.supportTypeSearchQuery = ko.observable();
    this.onSupportTypeSearchQueryChanged = function() {
        var query = this.supportTypeSearchQuery();
        var repository = CORE.model.constraints.getNodeConstraints(this.id()).supports;
        var results = [];
        if (!query) {
            this.supportTypeSearchResults(repository);
        } else {
            for (var i = 0; i < repository.length; i++) {
                var connectorItem = repository[i];
                if (connectorItem.ElementName.toLowerCase().indexOf(query.toLowerCase()) > -1) {
                    results.push(connectorItem);
                }
            }
            this.supportTypeSearchResults(results);
        }
    };
    this.supportTypeSearchResults = ko.observableArray();

    this.sel = ko.observable(isEmpty(viewModel.modelObject.supportId()) ? 0 : viewModel.modelObject.supportId());
    this.sel.subscribe(
        function(value) {
            var node = viewModel.modelObject;
            var support = null;
            var i;
            for (i = 0; i < CORE.model.supports().length; i++) {
                if (CORE.model.supports()[i].id === value) {
                    support = CORE.model.supports()[i];
                    break;
                }
            }
            if (support !== null && typeof support !== "undefined") {
                support.assignTo(node);
            }
        });
    this.assignSupport = function(supportId) {
        viewModel.modelObject.supportId(supportId);
    }
    var self = this;
    setTimeout(function() {
            $('.VIEWUI_allFieldsContainer')
                .scroll(function() {
                    self.showConnectorList(false);
                    self.showSupportTypeList(false);

                }.bind(this));
        },
        0);
    $('body')
        .click(function(event) {
            if (!$(event.target).closest('.VIEWUI_combobox').length) {
                self.showConnectorList(false);
                self.showSupportTypeList(false);
            }
        }.bind(this));
    this.showSupports = function() {
        var id = viewModel.id();
        if (!isEmpty(CORE.model.constraints.getNodeConstraints(id)) && 
            CORE.model.constraints.getNodeConstraints(id).supports !== null &&
            CORE.model.constraints.getNodeConstraints(id).supports.length > 1) {
            return true;
        }
        return false;
    }
    this.showOrientation = function() {
        var id = viewModel.id();
        if (!isEmpty(CORE.model.constraints.getNodeConstraints(id)) &&
            CORE.model.constraints.getNodeConstraints(id).isRotatedPossible) {
            return true;
        }
        return false;
    }

    this.showB = function() {
        var id = viewModel.id();
        if (!isEmpty(CORE.model.constraints.getNodeConstraints(id)) &&
            CORE.model.constraints.getNodeConstraints(id).isSteel) {
            return true;
        }
        return false;
    }

    this.onRotatedClicked = function() {
        viewModel.isRotated(true);
        if (viewModel.isRotatedChanged() === false)
            viewModel.isRotatedChanged(true);
        else
            viewModel.isRotatedChanged(false);
    }

    HELPERS.subscribePropertyChange(this, CORE.model.constraints.data, "CORE.model.constraints.data", this.showSupports);
}