﻿VIEWUI.properties = VIEWUI.properties || {};
// the getViewModel() function converts model object to viewModel object automatically, according to the rules:
// - normal properties are converted directly (using ko.mapping.fromJS() function)
// - immutable properties are first converted to viewModel-friendly properties using toViewModel() function
// - toViewModel() function is a common interface of all immutable types
VIEWUI.properties.vmGenerator = {
    currentSubscription: [], // keep subscriptions in an array to dispose them easily
    getViewModel: function(modelObject) {
        var i;
        for (i = 0; i < this.currentSubscription.length; i++) { // dispose previous subscriptions
            this.currentSubscription[i].dispose();
        }
        this.currentSubscription = [];
        var viewModel = ko.mapping.fromJS(modelObject); // generate initial viewModel using ko function
        for (var propertyName in modelObject) { // for each property of the model object
            if (modelObject.hasOwnProperty(propertyName)) {
                var modelProperty = ko.unwrap(modelObject[propertyName]); // modelProperty = unwrapped model property
                if (isEmpty(modelProperty)) { // if modelProperty is empty, ignore it
                    continue;
                }
                if (typeof modelProperty.toViewModel === 'function') { // if modelProperty contains toViewModel function (if it is a Vector object)				
                    viewModel[propertyName] = ko.observable(modelProperty.toViewModel()); // substitute existing viewModel property with modelProperty converted to ViewModel
                    VIEWUI.properties.vmGenerator.subscribeModelToViewModelChanges(viewModel, propertyName); // change model when viewModel changes                 
                    this.currentSubscription.push(HELPERS.subscribePropertyChange(viewModel, modelObject[propertyName], "modelProperty", this.onModelPropertyChanged, propertyName)); // change viewModel when model changes
                }
                var specialArrayProcessed = false; // specialArray (e.g. load.points)
                if (modelProperty.constructor === Array && modelProperty.length > 0 && typeof modelProperty[0] === 'function' && typeof modelProperty[0]().toViewModel === 'function') { // if property is an array containing vectors with toViewModel interface (e.g. load.points)
                    viewModel[propertyName] = ko.observableArray();
                    for (i = 0; i < modelProperty.length; i++) { // for each Vector, do the same as above
                        viewModel[propertyName].push(ko.observable(modelProperty[i]().toViewModel()));
                        VIEWUI.properties.vmGenerator.subscribeModelToViewModelChanges(viewModel[propertyName](), i, modelObject[propertyName]()); // change model when viewModel changes
                        //modelObject[propertyName]() is used as customThis because later root modelObject context is inaccessible
                        this.currentSubscription.push(HELPERS.subscribePropertyChange(viewModel[propertyName], modelObject[propertyName]()[i], "arrayModelProperty", this.onModelPropertyChanged, i, modelObject[propertyName]())); // change viewModel when model changes
                    }
                    specialArrayProcessed = true;
                }
                if (ko.isObservable(modelObject[propertyName]) && typeof modelObject[propertyName]['push'] === 'function' && !specialArrayProcessed) { // observableArrays bug with mapping.fromJS?
                    viewModel[propertyName] = modelObject[propertyName]; // need to assign manually if modelProperty is observableArray
                }
            }
        }
        // assign additional properties
        if (isEmpty(modelObject.type)) {
            console.error("VIEWUI.vmGenerator: modelObject.type is empty");
        }
        if (typeof VIEWUI.templates[modelObject.type] === 'undefined') {
            throw new Error("VIEWUI.vmGenerator: missing template in VIEWUI.templates for " + modelObject.type);
        }
        viewModel.template = VIEWUI.templates[modelObject.type];
        if (!isEmpty(viewModel.application) && !isEmpty(viewModel.application())
            && !isEmpty(VIEWUI.applicationTemplates[viewModel.application()])) { // if application is set, overwrite application properties template
            viewModel.template = VIEWUI.applicationTemplates[viewModel.application()];
        }
        this.calculationsFinished = ko.computed(function() {
            if (!isEmpty(CORE.model.barsResults()) && !isEmpty(CORE.model.nodesResults())) {
                return true;
            } else {
                return false;
            }
        });
        viewModel.modelObject = modelObject;
        viewModel.togglePopupExpanded = function(vm, event) {
            var target;
            if (event.target) {
                target = event.target;
            } else if (event.srcElement) {
                target = event.srcElement;
            }
            var innerPropertiesContainer = $(target).closest('.VIEWUI_innerPropertiesContainer');
            innerPropertiesContainer.find('.VIEWUI_group').toggle();                                                                // hide/show group
            innerPropertiesContainer.find('.VIEWUI_innerPropertiesTopBarIcon').each(function (index, item) { $(item).toggle(); });   // toggle arrows

            VIEWUI.properties.checkContainerSize();
        };
        viewModel.popupExpanded = ko.observable(true);
        viewModel.nameFieldDisabled = function () { return false };

        viewModel.afterRender = function (element) {
            VIEWUI.properties.checkContainerSize();
            setTimeout(function () {
                VIEWUI.spinners.init("#VIEWUI_propertiesContainer");
            }, 0);
        }

        /*viewModel.onDeleteClicked = function (viewModel) {
            CORE.model.removeSelectedObjects();
        }*/
        /*viewModel.isAssignableObject = ko.computed(function() {
            return typeof viewModel.modelObject.assignedTo === 'function';
        }, viewModel);*/
        return viewModel;
    },
    subscribeModelToViewModelChanges: function(viewModel, propertyName, modelObject) { //modelObject is only used as customThis when viewModel != root viewModel
        var vmProperty = ko.unwrap(viewModel[propertyName]);
        if (typeof vmProperty.getNestedProperties === 'function') { // if property has getNestedProperties() interface (e.g. threeVector)
            var nestedProperties = vmProperty.getNestedProperties(); // get nestedProperties
            for (var i = 0, ii = nestedProperties.length; i < ii; i++) { // for each of the nested properties, subscribe them separately
                if (typeof nestedProperties[i].ModelObjectConstructor === 'function') { // if is a vector (e.g. in ExtendedVector.vector)
                    HELPERS.subscribeVectorPropertyChange(viewModel, nestedProperties[i], "nestedPropertyVector",
                        VIEWUI.properties.vmGenerator.onVMPropertyChanged, propertyName, modelObject); // change model when viewModel changes
                } else {
                    HELPERS.subscribePropertyChange(viewModel, nestedProperties[i], "nestedProperty", // if is not a vector (e.g. in ExtendedVector.distance)
                        VIEWUI.properties.vmGenerator.onVMPropertyChanged, propertyName, modelObject);
                }
            }
        } else { // otherwise simply subscribe the property       
            HELPERS.subscribeVectorPropertyChange(viewModel, viewModel[propertyName], "immutableProperty",
                VIEWUI.properties.vmGenerator.onVMPropertyChanged, propertyName); // change model when viewModel changes
        }
    },
    propagateEvent: true,
    onVMPropertyChanged: function(propertyName, oldValue, customThis) { // change model when viewModel changes    
        if (!VIEWUI.properties.vmGenerator.propagateEvent) { // propagateEvent variable is used to prevent subsequent on...changed function calls (they are symmetrical)
            VIEWUI.properties.vmGenerator.propagateEvent = true;
            return;
        }
        var viewModel = this;
        var modelObject = viewModel.modelObject;
        if (typeof modelObject === 'undefined') { // when specialArrayProperty (i.e. load.points) is processed, modelObject context is changed from root to array
            modelObject = customThis; // set it from customThis (customThis = e.g. load.points array)
        }
        var vmProperty = ko.unwrap(viewModel[propertyName]); // unwrap viewModel property
        VIEWUI.properties.vmGenerator.propagateEvent = false;
        modelObject[propertyName](vmProperty.toModelObject()); // assign new value using viewModel property
        VIEWUI.properties.vmGenerator.propagateEvent = true;
    },
    onModelPropertyChanged: function(propertyName, oldValue, customThis) // change viewModel when model changes
    {
        //console.log(propertyName + " changed!");
        if (!VIEWUI.properties.vmGenerator.propagateEvent) {
            VIEWUI.properties.vmGenerator.propagateEvent = true;
            return;
        }
        var viewModel = this;
        var modelObject = viewModel.modelObject;
        if (typeof modelObject === 'undefined') { // when specialArrayProperty (i.e. load.points) is processed, modelObject context is changed from root to array
            modelObject = customThis; // set it from customThis (customThis = e.g. load.points array)
            //viewModel = customThis;
            viewModel = viewModel(); // viewModel is an observableArray
        }
        var newVmProperty = modelObject[propertyName]().toViewModel(); //create viewModel property from model property
        VIEWUI.properties.vmGenerator.propagateEvent = false;
        /*if (typeof viewModel[propertyName] === 'undefined')
        {
            return;     // ignore the call when subsequent onModelPropertyChanged is called (during specialArrayProperty (i.e. load.points) processing)
        }*/
        viewModel[propertyName](newVmProperty); // assign new value of viewModel property
        VIEWUI.properties.vmGenerator.propagateEvent = true;
        VIEWUI.properties.vmGenerator.subscribeModelToViewModelChanges(viewModel, propertyName, customThis); // refresh subscription
    }
}