﻿STATUS = { SUCCESS: "SUCCESS", ERROR: "ERROR" };
LOADING_MODE = { LOADING: "LOADING", SAVE: "SAVE" };

VIEWUI.statusBar = {
    init: function() {
        ko.applyBindings(this.viewModel, VIEWUI.divs.statusBarContainer.element[0]);

        var sub = MODEL.repository.objectsToLoad.subscribe(function () {    // subscribe at the end of processing
            CORE.model.modelChange.subscribe(function () {
                VIEWUI.statusBar.clear();
            });
            sub.dispose();
        });
    },
    initCursorPos: function(mouse3d, cursor3d) {
        HELPERS.subscribePropertyChange(this, mouse3d.position, "Mouse3D.position", function (position) {
            if(isEmpty(position)) {
                this.viewModel.cursorPos(null);
            } else {
                var pos = cursor3d.isEnabled() ? cursor3d.position : position;
                this.viewModel.cursorPos("( " + convertUnits(pos.x, "Distance") + "; "+ convertUnits(pos.y, "Distance") + "; "+ convertUnits(pos.z, "Distance") + " )");
            }
        })
    },
    viewModel: {
        status: ko.observable(null), // SUCCESS or ERROR
        //exceptions: ko.observable(null), // [{description, status}, ...]
        message: ko.observable(null),
        loading: ko.observable(false), // loader animation ON/OFF 
        loadingCounter: 0,
        loadingLabel: ko.observable(''),
        cursorPos: ko.observable(null)
    },
    setSuccess: function(message) {
        this.viewModel.status(STATUS.SUCCESS);
        this.viewModel.message(message);
    },
    setError: function(message/*, exceptions*/) {
        this.viewModel.status(STATUS.ERROR);
        this.viewModel.message(message);
        //this.viewModel.exceptions(exceptions);
        this.resetLoading();
    },
    resetLoading: function() {
        this.viewModel.loadingCounter = 0;
        this.viewModel.loading(false);
        this.viewModel.loadingLabel('');
    },
    clear: function() {
        this.viewModel.status(null);
        this.viewModel.message(null);
        //this.viewModel.exceptions(null);
    },
    startLoading: function (loadingMode) {
        CORE.viewerInProgress = true;
        this.viewModel.loadingCounter++;
        this.viewModel.loading(true);

        if (translation) {
            if (loadingMode === LOADING_MODE.SAVE)
                this.viewModel.loadingLabel(translation.SAVING);
            else
                this.viewModel.loadingLabel(translation.LOADING);
        }
    },
    stopLoading: function () {
        CORE.viewerInProgress = false;
        this.viewModel.loadingCounter--;
        if (this.viewModel.loadingCounter === 0) {
            this.viewModel.loading(false);
            this.viewModel.loadingLabel('');
        }
    }
}