﻿(function () {
    'use strict';

    describe('MODEL.Constraints', function () {
        
        describe('getConnectorsForNode', function () {
            it("returns connectors from received data", function () {
                //having
                var model = new MODEL.ConstructionViewModel();
                var constraints = new MODEL.Constraints(model);

                var constraintsData = {
                    nodeConstraints: {
                        '2': { connectors: [{ ElementId: 7 }] },
                        '5': { connectors: [{ ElementId: 3 }, { ElementId: 7 }] }
                    }
                }
                constraints.data(constraintsData);

                //when
                var connectors = constraints.getConnectorsForNode(5);

                //then
                expect(connectors.length).toBe(2);
                expect(connectors[0].ElementId).toBe(3);
                expect(connectors[1].ElementId).toBe(7);
            });
        });

        describe('getConnectorsForNode', function () {
            it("returns profiles from received data", function () {
                //having
                var model = new MODEL.ConstructionViewModel();
                var constraints = new MODEL.Constraints(model);

                var constraintsData = {
                    barConstraints: {
                        '1': { profiles: [{ ElementId: 234 }] },
                        '2': { profiles: [{ ElementId: 123 }, { ElementId: 345 }] }
                    }
                }
                constraints.data(constraintsData);

                //when
                var profiles = constraints.getProfilesForBar(1);

                //then
                expect(profiles.length).toBe(1);
                expect(profiles[0].ElementId).toBe(234);
            });
        });

        describe('applyConstraintsToModel', function () {
            it("will set default connector", function () {
                //having
                var model = new MODEL.ConstructionViewModel();
                var node = model.addNode(new MODEL.Node(new MODEL.types.Vector(0, 0, 0)));
                var constraints = new MODEL.Constraints(model);

                var constraintsData = { nodeConstraints: {}, barConstraints: {} }
                constraintsData.nodeConstraints[node.id] = { connectors: [{ ElementId: 3 }, { ElementId: 7 }] };

                //when
                constraints.data(constraintsData);

                //then
                expect(node.connectorId()).toBe(3);
            });
        });
        
        describe('applyConstraintsToModel', function () {
            it("will set default profile", function () {
                //having
                var model = new MODEL.ConstructionViewModel();
                var node1 = model.addNode(new MODEL.Node(new MODEL.types.Vector(0, 0, 0)));
                var node2 = model.addNode(new MODEL.Node(new MODEL.types.Vector(1, 0, 0)));
                var bar = model.addBar(new MODEL.Bar(node1, node2));
                var constraints = new MODEL.Constraints(model);

                var constraintsData = { nodeConstraints: {}, barConstraints: {} }
                constraintsData.barConstraints[bar.id] = { profiles: [{ ElementId: 234 }, { ElementId: 345 }] };

                //when
                constraints.data(constraintsData);

                //then
                expect(bar.profileId()).toBe(234);
            });
        });

        describe('applyConstraintsToModel', function () {
            it("will not chenge connector if still fits", function () {
                //having
                var model = new MODEL.ConstructionViewModel();
                var node = model.addNode(new MODEL.Node(new MODEL.types.Vector(0, 0, 0)));
                node.connectorId(7);

                var constraints = new MODEL.Constraints(model);
                var constraintsData = { nodeConstraints: {}, barConstraints: {} }
                constraintsData.nodeConstraints[node.id] = { connectors: [{ ElementId: 3 }, { ElementId: 7 }] };

                //when
                constraints.data(constraintsData);

                //then
                expect(node.connectorId()).toBe(7);
            });
        });


    });
})();
