﻿(function () {
    'use strict';
    var helpers = {
        addNode(nodes, node, id) {
            var n = new MODEL.Node(new MODEL.types.Vector(node.x, node.y, node.z));
            n.id = id;
            nodes.push(n);
            return n;
        },
        // creates 2 noes and bar between them
        addBar(nodes, bars, start, end) {
            var b0 = this.addNode(nodes, start, 10001);
            var b1 = this.addNode(nodes, end, 10002);
            var bar = new MODEL.Bar(b0, b1);
            bar.id = 10001;
            bars.push(bar);
            return bar;
        }
    }

    describe('MODEL.NodeAttachController', function () {
        describe('onAddNode', function () {
            it('will attach to bar if it lies on it', function () {
                var bars = ko.observableArray();
                var nodes = ko.observableArray();
                var controller = new MODEL.NodeAttachController(nodes, bars);

                var bar = helpers.addBar(nodes, bars, { x: 0, y: 0, z: 0 }, { x: 1, y: 1, z: 1 })

                //node on bar
                var node = new MODEL.Node(new MODEL.types.Vector(0.5, 0.5, 0.5))
                nodes.push(node);

                expect(bar.attachedNodes().length).toBe(1);
            });
            it('will not add to bar if it not lies on it', function () {
                var bars = ko.observableArray();
                var nodes = ko.observableArray();
                var controller = new MODEL.NodeAttachController(nodes, bars);

                var bar = helpers.addBar(nodes, bars, { x: 0, y: 0, z: 0 }, { x: 1, y: 1, z: 1 })

                //node out of bar
                var node = new MODEL.Node(new MODEL.types.Vector(0.5, 0.55, 0.5))
                nodes.push(node);

                expect(bar.attachedNodes().length).toBe(0);
            });
        });

        describe('onAddBar', function () {
            it('will attach to bar if it lies on it', function () {
                var bars = ko.observableArray();
                var nodes = ko.observableArray();
                var controller = new MODEL.NodeAttachController(nodes, bars);

                var b1 = helpers.addNode(nodes, { x: 0, y: 0, z: 0 }, 1);
                var b2 = helpers.addNode(nodes, { x: 1, y: 1, z: 1 }, 2);
                //node on bar
                var node = new MODEL.Node(new MODEL.types.Vector(0.5, 0.5, 0.5))
                nodes.push(node);

                var bar = new MODEL.Bar(b1, b2);
                bars.push(bar);

                expect(bar.attachedNodes().length).toBe(1);
            });
        });

        describe('onRemoveNode', function () {
            it('will detach single node from bar', function () {
                var bars = ko.observableArray();
                var nodes = ko.observableArray();
                var bar = helpers.addBar(nodes, bars, { x: 0, y: 0, z: 0 }, { x: 1, y: 1, z: 1 })

                //node on bar
                var node = new MODEL.Node(new MODEL.types.Vector(0.5, 0.5, 0.5));
                node.id = 123;
                nodes.push(node);

                var controller = new MODEL.NodeAttachController(nodes, bars);

                nodes.pop(node)

                expect(bar.attachedNodes().length).toBe(0);
            });
            it('will detach the right node from bar', function () { //bug fix - was reported byb Marcin
                var bars = ko.observableArray();
                var nodes = ko.observableArray();
                var bar = helpers.addBar(nodes, bars, { x: 0, y: 0, z: 0 }, { x: 1, y: 1, z: 1 })

                //3 nodes: all on bar
                var node1 = helpers.addNode(nodes, { x: 0.1, y: 0.1, z: 0.1 }, 101);
                var node2 = helpers.addNode(nodes, { x: 0.2, y: 0.2, z: 0.2 }, 102); //<--to be removed
                var node3 = helpers.addNode(nodes, { x: 0.3, y: 0.3, z: 0.3 }, 103);

                var controller = new MODEL.NodeAttachController(nodes, bars);

                nodes.splice(nodes().indexOf(node2), 1);//remove second one

                expect(bar.attachedNodes().length).toBe(2);
                expect(bar.attachedNodes()[0]).toBe(node1);
                expect(bar.attachedNodes()[1]).toBe(node3);
            });
        });

        describe('onNodePositionChage', function () {
            it('will detach node if it moves out of bar', function () {
                var bars = ko.observableArray();
                var nodes = ko.observableArray();
                var controller = new MODEL.NodeAttachController(nodes, bars);

                var bar = helpers.addBar(nodes, bars, { x: 0, y: 0, z: 0 }, { x: 1, y: 1, z: 1 })

                //node one bar
                var node = new MODEL.Node(new MODEL.types.Vector(0.5, 0.5, 0.5))
                nodes.push(node);
                node.position(new MODEL.types.Vector(0.5, 0.55, 0.5));// not on bar

                expect(bar.attachedNodes().length).toBe(0);
            });
            it('will leave it on bar if it moves but says on bar', function () {
                var bars = ko.observableArray();
                var nodes = ko.observableArray();
                var controller = new MODEL.NodeAttachController(nodes, bars);

                var bar = helpers.addBar(nodes, bars, { x: 0, y: 0, z: 0 }, { x: 1, y: 1, z: 1 })

                //node one bar
                var node = new MODEL.Node(new MODEL.types.Vector(0.5, 0.5, 0.5))
                nodes.push(node);
                node.position(new MODEL.types.Vector(0.6, 0.6, 0.6));// not on bar

                expect(bar.attachedNodes().length).toBe(1);
            });
            it('will attach it to bar if the new position is on bar', function () {
                var bars = ko.observableArray();
                var nodes = ko.observableArray();
                var controller = new MODEL.NodeAttachController(nodes, bars);

                var bar = helpers.addBar(nodes, bars, { x: 0, y: 0, z: 0 }, { x: 1, y: 1, z: 1 })

                //node out of bar
                var node = new MODEL.Node(new MODEL.types.Vector(0.5, 0.7, 0.5))
                nodes.push(node);
                node.position(new MODEL.types.Vector(0.6, 0.6, 0.6));// not on bar

                expect(bar.attachedNodes().length).toBe(1);
            });
        });

        describe('onBarPositionChange', function () {
            it('will move attached node', function () {
                var bars = ko.observableArray();
                var nodes = ko.observableArray();
                var controller = new MODEL.NodeAttachController(nodes, bars);

                var bar = helpers.addBar(nodes, bars, { x: 0, y: 0, z: 0 }, { x: 1, y: 1, z: 1 })

                //node on bar
                var node = new MODEL.Node(new MODEL.types.Vector(0.5, 0.5, 0.5))
                nodes.push(node);

                //mock -> in production context this is set by ConstructionViewModel
                bar.node1().createsBars = function () { return [bar] };

                bar.node1().position(new MODEL.types.Vector(2, 0, 0));

                expect(bar.attachedNodes()[0].position().x).toBe(1.5);
            });
            it('will detach node in detachMode', function () {
                var bars = ko.observableArray();
                var nodes = ko.observableArray();
                var controller = new MODEL.NodeAttachController(nodes, bars);

                var bar = helpers.addBar(nodes, bars, { x: 0, y: 0, z: 0 }, { x: 1, y: 1, z: 1 })

                //node on bar
                var node = new MODEL.Node(new MODEL.types.Vector(0.5, 0.5, 0.5))
                nodes.push(node);

                //mock -> in production context this is set by ConstructionViewModel
                bar.node1().createsBars = function () { return [bar] };


                controller.setDetachMode(true);
                bar.node1().position(new MODEL.types.Vector(2, 0, 0));

                expect(bar.attachedNodes().length).toBe(0);
            });
            it('will attach node in new position', function () {
                var bars = ko.observableArray();
                var nodes = ko.observableArray();
                var controller = new MODEL.NodeAttachController(nodes, bars);

                var bar = helpers.addBar(nodes, bars, { x: 0, y: 0, z: 0 }, { x: 1, y: 1, z: 1 })

                //node not on bar
                var node = new MODEL.Node(new MODEL.types.Vector(1.5, 0.5, 0.5))
                nodes.push(node);

                //mock -> in production context this is set by ConstructionViewModel
                bar.node1().createsBars = function () { return [bar] };

                bar.node1().position(new MODEL.types.Vector(2, 0, 0));

                expect(bar.attachedNodes().length).toBe(1);
            });
        });

        describe('onNodeChange', function () {
            it('will move attached node', function () {
                var bars = ko.observableArray();
                var nodes = ko.observableArray();
                var bar = helpers.addBar(nodes, bars, { x: 0, y: 0, z: 0 }, { x: 1, y: 1, z: 1 })

                //node on bar
                var node = new MODEL.Node(new MODEL.types.Vector(0.5, 0.5, 0.5));
                node.id = 9;
                nodes.push(node);

                var newNode1 = new MODEL.Node(new MODEL.types.Vector(2, 0, 0));
                newNode1.id = 10;
                nodes.push(newNode1);

                var controller = new MODEL.NodeAttachController(nodes, bars);

                bar.node1(newNode1);

                expect(bar.attachedNodes()[0].position().x).toBe(1.5);
            });
        });

    });
})();
